/*
 *  O_XV_PGI.h:	Declaration of Xv video extension Parallax access object.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     19 Apr 95 - Andy Goodrich
 *       - Initial coding
 *
 * $Log:   /plxhp/dave/pvcsrc/O_XV/archives/O_XV_PGI.h_v  $
 * 
 *    Rev 1.1   09 Sep 1996 09:52:00   dave
 * Initial revision
 * Revision 1.1  1995/05/16 10:10:13  dist
 * Initial revision
 *
 *
 *  Copyright(c) 1995, Parallax Graphics
 */
#ifndef  O_XV_PGI_INCLUDED
#define O_XV_PGI_INCLUDED

#include "O_XV.h"		/* Pick up base object for this derived object type. */


/*  Menu callback calling sequences for input_menu and output_menu methods:
 *
 *    void cb_input( O_XV_PGI* p_obj, Widget w, XvplxSignalRec* p_signal)
 *
 *    void cb_output( O_XV_PGI* p_obj, Widget w, XvplxSignalRec* p_signal)
 *
 */

typedef struct Tv_XV_PGI {
	void	(*dump)();			/* Dump contents of signal object. */
	int		(*input_menu)();	/* Create a cascading menu for inputs. */
	int		(*inputs)();		/* Return list of input signals. */
	int		(*output_menu)();	/* Create a cascading menu for outputs. */
	int		(*outputs)();		/* Return list of output signals. */
} Tv_XV_PGI;

typedef struct O_XV_PGI {
	O_XV			base;			/* Parent object for this derived type. */
	Tv_XV_PGI		*tv;			/* Methods transfer vector. */
	Atom            a_signals;   	/* Atom for signals attribute. */
} O_XV_PGI;

O_XV_PGI* O_XV_PGI_O_XV_PGI();

#define O_XV_PGI_Dump(obj, fd) \
	(* ((O_XV_PGI *)(obj))->tv->dump)(obj, fd)

#define O_XV_PGI_Input_Menu(obj, widget, callback) \
	(* ((O_XV_PGI *)(obj))->tv->input_menu)(obj, widget, callback)

#define O_XV_PGI_Inputs(obj, p_inputs) \
	(* ((O_XV_PGI *)(obj))->tv->inputs)(obj, p_inputs)

#define O_XV_PGI_Output_Menu(obj, widget, callback) \
	(* ((O_XV_PGI *)(obj))->tv->output_menu)(obj, widget, callback)

#define O_XV_PGI_Outputs(obj, p_outputs) \
	(* ((O_XV_PGI *)(obj))->tv->outputs)(obj, p_outputs)


#endif /* O_XV_PGI_INCLUDED */
