/*
 *  geometry.c: Event-driven generation of graphics overlays.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  Copyright(c) 1994, Parallax Graphics
 *
 *  REVISION HISTORY:
 *
 * $Log:   /plxhp/dave/pvcsrc/O_XV/archives/geometry.c_v  $
 * 
 *    Rev 1.1   09 Sep 1996 09:52:58   dave
 * Initial revision
 * Revision 1.1  1995/05/16 10:10:19  dist
 * Initial revision
 *
 */
#include <sys/types.h>			/* Standard types for this C environment. */
#include <stdio.h>				/* Stream input/output declarations. */
#include <X11/Xlib.h>

#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>

#include <Xvlib.h>
#include <O_XV.h>
#include <math.h>

/*
 *  Forward references:
 */

static int line_x = 0;
static int line_y = 0;
static GC  line_gc;

/*----------------------------------------------------------------------------*
 |"fill_area"
 | 
 | 
 *----------------------------------------------------------------------------*/
void fill_area( p_xv, gc, x, y, delta )
	O_XV*	p_xv;	/* -> Xv access object instance. */
	GC		gc;		/* =  graphics context containing color to use. */
	int		x;		/* =  x coordinate of center of fill area. */
	int		y;		/* =  y coordinate of center of fill area. */
	int		delta;	/* =  delta in x and y for fill area. */	
{
	int		i_y;	/* Scan line now filling. */

	for ( i_y = y - delta; i_y < y + delta; i_y++ )
	{
		XDrawLine(p_xv->display, p_xv->w_main, gc, x-delta, i_y, x+delta, i_y);
	}
}
	
/*----------------------------------------------------------------------------*
 |"polygon"
 | 
 | 
 *----------------------------------------------------------------------------*/
void polygon( p_xv, gc, x, y, n_vertices )
	O_XV*	p_xv;		/* -> Xv access object instance. */
	GC		gc;			/* =  graphics context containing color to use. */
	int		x;			/* =  x coordinate of center of fill area. */
	int		y;			/* =  y coordinate of center of fill area. */
	int		n_vertices;	/* =  # of vertices polygon has. */
{
	double	direction;	/* Angle of path vector. */
	double	f_nv2pi;	/* Floating point version of 2 pi / n_vertices. */
	int		i_v;		/* Vertex now processing. */
	double	round_off;	/* Fudge factor for rounding off double to ints. */
	double	velocity;	/* Magnitude of path vector. */

	round_off = (double).4999;

	f_nv2pi = (double)2.0 * (double)3.141592 / (double)(n_vertices);

	if ( n_vertices != 4 )
	{
		for ( i_v = 0; i_v < n_vertices-1; i_v++ )
		{
			XDrawLine(p_xv->display, p_xv->w_main, gc, 
				x + (int)( round_off + 20.0 * cos((double)(i_v)   * f_nv2pi) ),
				y + (int)( round_off + 20.0 * sin((double)(i_v)   * f_nv2pi) ),
				x + (int)( round_off + 20.0 * cos((double)(i_v+1) * f_nv2pi) ),
				y + (int)( round_off + 20.0 * sin((double)(i_v+1) * f_nv2pi) ));
		}
		XDrawLine(p_xv->display, p_xv->w_main, gc, 
			x + (int)( round_off + 20.0 * cos((double)(n_vertices-1)*f_nv2pi) ),
			y + (int)( round_off + 20.0 * sin((double)(n_vertices-1)*f_nv2pi) ),
			x + (int)( round_off + 20.0 * cos(0) ),
			y + (int)( round_off + 20.0 * sin(0) ));
	}
	else
	{
		XDrawLine(p_xv->display, p_xv->w_main, gc, x-20, y,    x,    y+20);
		XDrawLine(p_xv->display, p_xv->w_main, gc, x,    y+20, x+20, y);
		XDrawLine(p_xv->display, p_xv->w_main, gc, x+20, y,    x,    y-20);
		XDrawLine(p_xv->display, p_xv->w_main, gc, x,    y-20, x-20, y);
	}

	/* DRAW PATH VECTOR: */

	velocity = 25.0 + 5.0 * (double)( (rand()>>3) % 6);
	direction = 2.0 * 3.14159 * (double)( (rand()>>3) % 8) / 8.0;
	XDrawLine(p_xv->display, p_xv->w_main, gc, 
		x , y,
		x + (int)( velocity * cos(direction) ),
		y + (int)( velocity * sin(direction) ) );
}
	
/*----------------------------------------------------------------------------*
 |"process_geometry"
 | 
 | 
 *----------------------------------------------------------------------------*/
void process_geometry(p_xv, width, height, p_image)
	O_XV*	p_xv;	/* -> Xv access object instance. */
	int		width;
	int		height;
	XImage*	p_image;
{
	Window  	child;		/* The ID of the child of window the pointer is in.*/
	XComposeStatus composestatus; /* Specified the XCompose structure.*/
	Colormap	cmap;		/* Color map go get colors from. */
	XColor		ecolor;
	XEvent		event;		/* Event we intercepted. */
	GC			gc;			/* Graphics context with color for polygon. */
	XGCValues	gc_attrs;	/* GC attributes passed in for GC creation. */
	GC			gc_black;	/* Graphics context with color black. */
	GC			gc_green;	/* Graphics context with color green. */
	GC			gc_magenta;	/* Graphics context with color magenta. */
	GC			gc_red;		/* Graphics context with color red. */
	GC			gc_yellow;	/* Graphics context with color yellow. */
	GC			gc_white;	/* Graphics context with color white. */
	KeySym 		keysym;		/* It specifies the keysym ID computed from event.*/
	int			mask;		/* Button mask. */
	Window  	root;		/* The root window ID the pointer is currently on.*/
	int			rx;			/* Return the x and y cordinates of the */ 
    int 		ry;			/* pointer relative to the root's origin.*/
	XColor		scolor;
	char    	string[31]; /* Return the resulting string. */
	int 		wx;			/* Return the x and y coordinates of the */
	int 		wy;			/* pointer relative to the origin of window. */

	/*  ALLOCATE A SERIES OF GRAPHICS CONTEXTS WITH THE COLORS WE WILL USE. */

	cmap = XDefaultColormap(p_xv->display, p_xv->screen);

	XAllocNamedColor(p_xv->display, cmap, "black", &scolor, &ecolor);
	gc_attrs.foreground = scolor.pixel;
    gc_black = XCreateGC(p_xv->display, p_xv->w_main, GCForeground, &gc_attrs);

	XAllocNamedColor(p_xv->display, cmap, "green", &scolor, &ecolor);
	gc_attrs.foreground = scolor.pixel;
    gc_green = XCreateGC(p_xv->display, p_xv->w_main, GCForeground, &gc_attrs);

	XAllocNamedColor(p_xv->display, cmap, "magenta", &scolor, &ecolor);
	gc_attrs.foreground = scolor.pixel;
    gc_magenta = XCreateGC(p_xv->display,p_xv->w_main, GCForeground, &gc_attrs);

	XAllocNamedColor(p_xv->display, cmap, "red", &scolor, &ecolor);
	gc_attrs.foreground = scolor.pixel;
    gc_red = XCreateGC(p_xv->display, p_xv->w_main, GCForeground, &gc_attrs);

	XAllocNamedColor(p_xv->display, cmap, "yellow", &scolor, &ecolor);
	gc_attrs.foreground = scolor.pixel;
    gc_yellow = XCreateGC(p_xv->display, p_xv->w_main, GCForeground, &gc_attrs);

	XAllocNamedColor(p_xv->display, cmap, "white", &scolor, &ecolor);
	gc_attrs.foreground = scolor.pixel;
    gc_white = XCreateGC(p_xv->display, p_xv->w_main, GCForeground, &gc_attrs);

	line_gc = gc_green;

	/*  LOOP FOREVER LOOKING FOR THE EVENTS WE PROCESS. */

    for(;;)
    {
        XWindowEvent(p_xv->display, p_xv->w_main, 
			(KeyPressMask|VisibilityChangeMask), &event);
        switch ( event.type )
		{
		  case Expose:
			printf("Expose event\n");
 		  case VisibilityNotify: 
			printf("Visibility event\n");
			if ( p_image )
			{
				XPutImage(p_xv->display, p_xv->w_main, p_xv->gc, p_image,
					0, 0, 0, 0, width, height);
			}
			else
			{
				O_XV_Video_On( p_xv );
			}
			break;
		
 		  case KeyPress: 
			  XQueryPointer(p_xv->display, p_xv->w_main, &root, &child,
					  &rx, &ry, &wx, &wy, &mask);
			XLookupString(event.xkey,
				string,
				30,
				&keysym,
				&composestatus);

			switch( (rand() >> 3) % 5 )
			{
				default:
				case 0: gc = gc_green;	    break;
				case 1:	gc = gc_magenta;	break;
				case 2: gc = gc_red;	    break;
				case 3:	gc = gc_yellow;	    break;
				case 4:	gc = gc_white;	    break;
			}

			switch(keysym) 
			{
				default:  fill_area( p_xv, gc_black, wx, wy, 20 );  break;
				case '0': fill_area( p_xv, gc_black, wx, wy, 640);  break;
				case '1': line_x = wx; line_y = wy; line_gc = gc;   break;
				case '2': XDrawLine(p_xv->display, p_xv->w_main, 
						    line_gc, line_x, line_y, wx, wy);         
				          line_x = wx; line_y = wy;                 break;
				case '3': polygon( p_xv, gc, wx, wy, 3 );    		break;
				case '4': polygon( p_xv, gc, wx, wy, 4 );    		break;
				case '5': polygon( p_xv, gc, wx, wy, 5 );    		break;
				case '6': polygon( p_xv, gc, wx, wy, 6 );    		break;
				case '7': polygon( p_xv, gc, wx, wy, 7 );    		break;
				case '8': polygon( p_xv, gc, wx, wy, 8 );    		break;
				case '9': polygon( p_xv, gc, wx, wy, 20);    		break;
			}
			break;
		}
	}
}
