/*
 *  parse_value.c: Value parsing function. 
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     06 Aug 94 - Andy Goodrich
 *       - Initial coding
 *
 * $Log:   /plxhp/dave/pvcsrc/O_XV/archives/parse_value.c_v  $
 * 
 *    Rev 1.1   09 Sep 1996 09:53:08   dave
 * Initial revision
 * Revision 1.1  1995/08/27 14:04:09  dist
 * Initial revision
 *
 *
 *  Copyright(c) 1994, Parallax Graphics
 *
 *  Copyright(c) 1993, Andrew C. Goodrich
 *  Used with permission by Parallax Graphics, Inc.
 */
#include <sys/types.h>			/* Standard types for this C environment. */
#include <stdio.h>				/* Stream input/output declarations. */
#include "parse_value.h"

static short dec_table[256] =	/* Table for recognizing decimal #'s. */
{
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -3,  -1,  -3,  -2,  -1, 
  0,   1,   2,   3,   4,   5,   6,   7, 
  8,   9,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1
 };

static short	hex_table[256] =	/* Table for recognizing hexadecimal #'s. */
{
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
  0,   1,   2,   3,   4,   5,   6,   7, 
  8,   9,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  10,  11,  12,  13,  14,  15,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  10,  11,  12,  13,  14,  15,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1
};

/*---------------------------------------------------------------------------*
 |"p_value"
 |
 |  Parse a value from a string of characters. The value parsed must use up
 |  all the characters of the string. For keyword and quoted string values
 |  storage will be allocated to hold the result.
 |
 |		= type of value parsed, or error specification
 *---------------------------------------------------------------------------*/
p_value( p_str, p_result ) 
	char		*p_str;		/* -> string to be parsed for value. */
	PV_Result	*p_result;	/* -> where to place parsed value. */
{
	int		digit;	    /* Digit value for decimal number. */
	int		integer;	/* Value accummulating for decimal or hex number. */
	int		i_str;		/* Character within p_str accessing. */
	int		its_float;	/* = 1 if parsing floating point number. */
	int		n_str;	    /* Length of argument. */
	int		nibble;	    /* Nibble value for hexadecimal number. */
	int		sign;		/* Sign of integer or floating point number. */

	n_str = strlen(p_str);

	/*
	 *  HEXADECIMAL INTEGER VALUE
	 */
	if ( n_str > 2 && p_str[0] == '0' && (p_str[1] == 'x' || p_str[1] == 'X') )
	{
		for ( integer = 0, i_str = 2; i_str < n_str; i_str++ )
		{
			if ( (nibble = hex_table[p_str[i_str]]) == -1 )
			{
				return PV_Bad_Value;
			}
			integer = integer << 4 | nibble;
		}
		p_result->i = integer;
		return PV_Integer;
	}

	/*
	 *  DECIMAL INTEGER OR FLOATING POINT VALUE
	 */
	else if ( dec_table[p_str[0]] != -1 )
	{
		if ( (p_str[0] == '-') )
		{
			if (n_str == 1) return PV_Bad_Value;
			i_str = 1;
			sign = -1;
		}
		else if ( p_str[0] == '+' )
		{
			if (n_str == 1) return PV_Bad_Value;
			i_str = 1;
			sign = 1;
		}
		else
		{
			i_str = 0;
			sign = 1;
		}
			
		for ( integer = 0, its_float = 0; i_str < n_str; i_str++ )
		{
			digit = dec_table[p_str[i_str]];
			if( digit < 0 )
			{
				switch(digit)
				{
				  case -1:
				  case -3:
					return PV_Bad_Value;
				  case -2:
					if ( its_float ) return PV_Bad_Value;
					its_float = 1;
					break;
				}
			}
			else
			{
				its_float = its_float * 10;
				integer = integer * 10 + digit;
			}
		}
		if ( its_float )
		{	
			p_result->d = (double)(sign*integer) / (double)(its_float);
			return PV_Float;
		}
		else
		{
			p_result->i = sign * integer;
			return PV_Integer;
		}
	}

	/*
	 *  QUOTED STRING
	 */
	else if ( n_str > 2 && (p_str[0] == '"') || (p_str[0] == '\'' ) )
	{
		if ( p_str[n_str-1] != p_str[0] )
		{
			return PV_Bad_Value;
		}
		else if ( !(p_result->p = (char *)malloc(n_str)) )
		{
			return PV_Bad_Alloc;
		}
			
		strcpy(p_result->p, &p_str[1]);
		p_result->p[n_str-2] = '\0';
		return PV_Quoted_String;
	}

	/*
	 *  KEYWORD
	 */
	else
	{
		if ( !(p_result->p = (char *)malloc(n_str+1)) )
		{
			return PV_Bad_Alloc;
		}
			
		strcpy(p_result->p, p_str);
		return PV_Keyword;
	}
}
