/*
 *  xmcapture.c:	Frame capture program utilizing Motif Windowing system.
 *
 *
 *  REVISION HISTORY
 *
 * $Log:   /plxhp/dave/source/O_XV/archives/xmcapture.c_v  $
 * 
 *    Rev 1.0   12 Sep 1996 11:54:52   dave
 * init
 *
 *  Copyright(c) 1995, Parallax Graphics, Inc.
 */
#include <stdio.h>
#include <sys/types.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <Xm/DrawingA.h>
#include <Xm/DialogS.h>
#include <Xm/Form.h>
#include <Xm/PanedW.h>
#include "Xvlib.h"
#include "Xvddx_PGI.h"
#include "O_XV_PGI.h"
#include "support.h"
#include "xmcapture_help.h"

enum {
	MAX_ARGS = 50,
	WINDOW_HEIGHT = 480,
	WINDOW_WIDTH = 640,
};


/*  WINDOWING SYSTEM CONTEXT FOR THIS PROGRAM */

typedef struct Select {	/* Selection box parameters: */
	Pixel  				color_bg;		/* Background color. */
	Pixel  				color_select;	/* Selection highlight color. */
	Widget 			    w_dialog;		/* Whole dialog box widget. */
	Widget 				w_filter;		/* File filter text widget. */
	Widget 				w_select;		/* File selection text widget. */
} Select;

typedef struct _Capture {
	Atom			a_jpeg;			/* Atom for XvGetImage for JPEG. */
	XtAppContext	appContext;
	Atom			a_ximage;		/* Atom for XvGetImage for XImages. */
	FILE*			fd;				/* File descriptor to write to. */
	Select			file;			/* File selection information. */
	char*			file_name;		/* Name of file to write to. */
	int				format;			/* Format to capture in. */
	int				capture_state;	/* State of encoding. */
	XImage*			image;			/* Image of current window. */
	int				image_height;	/* Height of image to grab. */
	int				image_width;	/* Width of image to grab. */
	O_XV_PGI*	 	p_xv_in;       	/* Xv access object for port. */
	O_XV_PGI*	 	p_xv_out;      	/* Xv access object for port. */
	Widget          w_bar;			/* Menu bar widget. */
	Widget          w_form;
	Widget 			w_toplevel;
	Widget 			w_video_record;	/* Video input window widget. */
	Widget 			w_video_play;	/* Video output window widget. */
} Capture;

Capture         me;         /* Context for callbacks. */

enum {		/* Capture structure capture_state field values. */
	CaptureS_unallocated,
	CaptureS_allocated,
	CaptureS_encoding,
};


/*  FORWARD REFERENCES: */

void cb_about_capture();
void cb_expose();
void cb_input();
void cb_output();
void cb_exit();
void cb_format();
void cb_help();
void cb_operator();
void cb_resize();
void cb_save_to();
void cb_save_to_ok();
void cb_select_color();
void h261_capture();
void h261_setup();
void h261_teardown();
void jpeg_capture();
void jpeg_setup();
void jpeg_teardown();
void resize();
void sun_capture();
void sun_setup();
void sun_teardown();


/*  MENU DATA STRUCTURE: */

enum {				/* Capture formats. */
	Format_H261_CIF,
	Format_H261_NTSC,
	Format_H261_QCIF,
	Format_JPEG,
	Format_Sun
};
Menu_Linked_E*	p_inputs_list;  /* Inputs list menu(must be before include) */
Menu_Linked_E*	p_outputs_list; /* Outputs list menu(must be before include) */

#include "xmcapture_menu.h"


/*  H.261 GLOBAL STORAGE (USED TO COMMUNICATE WITH H.261 ENCODER) */

static int Argc = 7;
static char *Argv[100] = {
	"initp64Main",
	"-CIF",
	"-s", "images_h261",
	"-b", "100",
	"nofiles",
	NULL
};
unsigned char*  chromaBlueP;
unsigned char*  chromaRedP;
unsigned char*  lumaP;
char* 			rgbData;


/*---------------------------------------------------------------------------*
 |"main"
 |
 |  Entry point for this application.
 *---------------------------------------------------------------------------*/
main(argc, argv)
	int  argc; 
	char* argv[];
{
	XColor	 	 	exact_color; /* Exact RGB for color. */
	XFontStruct* 	font_list;   /* Font using for menus. */
	XFontStruct* 	font1;       /* System 17 font. */
	XFontStruct* 	font2;       /* User 7x17 font. */
	int			 	n_signals;	 /* # of signals for this adaptor. */
	Display* 	 	p_display;	 /* Display being used. */
	XvplxSignalRec*	p_signals;	 /* # of signals for this adaptor. */
	XColor	 	 	scrn_color;  /* Closest screen RGB to color. */	
	Colormap 	 	video_cmap;  /* Color map for video window. */


	me.fd = 0;
	me.file_name = 0;
	me.capture_state = CaptureS_unallocated;
	me.image = 0;
	me.image_height = 0;
	me.image_width = 0;

	me.w_toplevel = XtAppInitialize(&me.appContext,
		"winXt",
		NULL,
		(Cardinal) NULL,
		&argc,
		argv,
		NULL,
		NULL,
	NULL);

	p_display = XtDisplay(me.w_toplevel);

    /*
     *  DETERMINE MENU FONT.
     */
    font1 = XLoadQueryFont(p_display, "system17");
    font2 = XLoadQueryFont(p_display, "user7x17");
    if (font1)
    {
        font_list = XmFontListCreate(font1,"charset1");
        if (font2)
            font_list = XmFontListAdd(font_list,font2,"charset2");
    }
    else
    {
        if (font2)
            font_list = XmFontListCreate(font2,"charset2");
    }


    /*
     *  CREATE THE FORM WHICH WILL HOLD MENU BAR AND VIDEO AREA:
     *  THEN CREATE THE MENU AND VIDEO AREAS WITHIN THAT FORM.
     */
    me.w_form = XtVaCreateManagedWidget("main_window",
        xmPanedWindowWidgetClass,   me.w_toplevel,
        XmNwidth,                   640,
        XmNheight,                  600,
        XmNseparatorOn,             False,
        XmNspacing,                 0,
        XmNmarginHeight,            0,
        XmNmarginWidth,             0,
    NULL);
    me.w_bar = build_menu_bar( me.w_form );

	video_cmap = XDefaultColormap( p_display, XDefaultScreen(p_display));
	XAllocNamedColor(p_display, video_cmap, "black", &scrn_color, &exact_color);
    me.w_video_record = XtVaCreateManagedWidget("drawing_a",
        xmDrawingAreaWidgetClass,   me.w_form,
        XmNwidth,               	WINDOW_WIDTH,
        XmNheight,              	WINDOW_HEIGHT,
		XmNbackground,				scrn_color.pixel,
        XmNcolormap,                video_cmap,
    NULL) ;

	XtAddCallback(me.w_video_record, XmNresizeCallback, cb_resize, &me);
	XtAddCallback(me.w_video_record, XmNexposeCallback, cb_expose, &me);
	XtAddEventHandler(me.w_video_record, ButtonPressMask | KeyPressMask,
		False, cb_operator, &me);

    if ( !(me.p_xv_in = O_XV_PGI_O_XV_PGI( 0, XtDisplay(me.w_toplevel), 0, 0 )) )
    {
        fprintf(stderr, "\nCould not open display\n");
        return 0;
    }
	me.a_ximage = XInternAtom(p_display,"XV_IMAGE_XIMAGE",False);
	me.a_jpeg = XInternAtom(p_display,"XV_IMAGE_JPEG",False);

	printf("#### n_encodings = %d\n", me.p_xv_in->base.n_encodings);
	p_inputs_list = build_inputs_list("Inputs", 0, cb_input, 
			me.p_xv_in->base.n_encodings, me.p_xv_in->base.p_encodings);
	p_inputs_list = p_inputs_list->sub_menu;
	p_outputs_list = build_outputs_list("Outputs", 0, cb_output, 
			me.p_xv_in->base.n_encodings, me.p_xv_in->base.p_encodings);
	if ( p_outputs_list ) p_outputs_list = p_outputs_list->sub_menu;
	build_static_menu_buttons(NULL, font_list, me.w_bar,
		MenuBar, XtNumber(MenuBar));
		

	/*  INFORM XV WHICH WINDOW IT WILL USE AND FIRE OFF THIS APPLICATION. */
     
	XtRealizeWidget(me.w_toplevel);
    O_XV_Use_Window( me.p_xv_in, XtWindow(me.w_video_record) );
	O_XV_Video_On(me.p_xv_in);
	if ( p_outputs_list ) 
	{
		p_outputs_list = p_outputs_list->sub_menu;
		if ( !(me.p_xv_out = 
			O_XV_PGI_O_XV_PGI( 0, XtDisplay(me.w_toplevel), 0, 0 )) )
		{
			fprintf(stderr, "\nCould not open display for output\n");
			return 0;
		}
		O_XV_Use_Window( me.p_xv_out, XtWindow(me.w_video_record) );
		O_XV_Video_Off(me.p_xv_out);
	}

	sash_traversal_off(me.w_form);
	XtAppMainLoop(me.appContext);

}
/*----------------------------------------------------------------------------*
 |"beep"
 *----------------------------------------------------------------------------*/
beep(volume)
    int volume ;
{
    int count_i ;
    for (count_i = 0 ; count_i < volume ; count_i++)
    {
        putchar('\07') ;
        fflush(stdout) ;
        if ( volume ) my_usleep(250000);
    }
}


/*---------------------------------------------------------------------------*
 |"buildfile_select"
 *---------------------------------------------------------------------------*/
void buildfile_select(w, p_me, title)
    Widget    w;             /* = widget causing the callback. */
    Capture*  p_me;          /* Our context. */
{
	Colormap	cmap;	/* Color map in use with this widget. */
	String		text;	/* Text for selection and file filters. */

#ifdef MP
    p_me->file.w_dialog = mpCreateFileSelectionDialog(w,
        title, NULL,
    0);
#else
    p_me->file.w_dialog = XmCreateFileSelectionDialog(w,
        title, NULL,
    0);
#endif
    XtVaSetValues(p_me->file.w_dialog,
		XmNwidth, 400,
	NULL);

	/* 
     *  SET UP COLOR SCHEME AND ACQUIRE POINTERS TO WIDGETS WE WILL USE.
     */
    XtVaGetValues(p_me->file.w_dialog,
        XmNbackground, &(p_me->file.color_bg),
        XmNcolormap,   &cmap,
    NULL);
    XmGetColors(XtScreen(p_me->file.w_dialog), cmap, p_me->file.color_bg,
        0, 0, 0, &(p_me->file.color_select));
    p_me->file.w_select = XmFileSelectionBoxGetChild(
		p_me->file.w_dialog,XmDIALOG_TEXT);
    p_me->file.w_filter = XmFileSelectionBoxGetChild(
		p_me->file.w_dialog,XmDIALOG_FILTER_TEXT);

	/*
     *  SET UP CALL BACKS FOR THOSE COMPONENTS WE WISH TO INTERACT WITH.
     */
    XtAddCallback(p_me->file.w_select,
		XmNfocusCallback, cb_select_color, p_me);
    XtAddCallback(p_me->file.w_filter,
		XmNfocusCallback, cb_select_color, p_me);
    XtAddCallback(p_me->file.w_dialog,
		XmNcancelCallback, XtDestroyWidget, NULL);
    XtAddCallback(p_me->file.w_dialog,
		XmNhelpCallback, cb_help, "Save_To");

	/* 
     *  SET DEFAULT FILTER AND FILE NAMES.
     */
	text = XmStringCreateLtoR("/tmp/", XmSTRING_DEFAULT_CHARSET);
	XtVaSetValues(p_me->file.w_dialog, XmNdirectory, text, NULL);
	XmStringFree(text);
}


/*---------------------------------------------------------------------------*
 |"cb_about_capture"
 *---------------------------------------------------------------------------*/
void cb_about_capture(w, p_data, event)
    Widget  w;             /* = widget causing the callback. */
    char*   p_data;          /* Our instance specific data. */
    XEvent* event;         /* Event causing keyboard or mouse callback. */
{
    create_dialog(w, "Capture", "Capture 9.1 - (c) Parallax - 20 July 1995", 0);
}

 
/*---------------------------------------------------------------------------*
 |"cb_exit"
 *---------------------------------------------------------------------------*/
void cb_exit(w, p_me, event)
    Widget   w;           /* = widget causing the callback. */
    Capture* p_me;        /* Our instance specific data. */
    XEvent*  event;       /* Event causing keyboard or mouse callback. */
{
	switch( p_me->format )
	{
	  case Format_H261_CIF:
	  case Format_H261_NTSC:
	  case Format_H261_QCIF:
		h261_teardown(p_me);
		break;
	  case Format_JPEG:
		break;
	  case Format_Sun:
		sun_teardown(p_me);
		break;
	  default:
		break;	
	}
    exit(0);
}


/*---------------------------------------------------------------------------*
 |"cb_expose"
 *---------------------------------------------------------------------------*/
void cb_expose(w, p_me, event)
    Widget      w;      /* =  widget causing the callback. */
    Capture*    p_me;   /* -> our instance specific data. */
    XEvent*		event;  /* =  event causing resize callback. */
{
    O_XV_Video_On( p_me->p_xv_in );
}


/*---------------------------------------------------------------------------*
 |"cb_format"
 *---------------------------------------------------------------------------*/
void cb_format(w, format, event)
    Widget  w;          /* = widget causing the callback. */
    int   	format;     /* = format number to set. */
    XEvent* event;      /* Event causing keyboard or mouse callback. */
{
	switch( format )
	{
	  case Format_H261_CIF:
		Argv[1] = "-CIF";
		me.image_width = 352; 
		me.image_height = 288; 
		break;
	  case Format_H261_NTSC:
		Argv[1] = "-NTSC";
		me.image_width = 352; 
		me.image_height = 240; 
		break;
	  case Format_H261_QCIF:
		Argv[1] = "-QCIF";
		me.image_width = 176; 
		me.image_height = 144; 
		break;
	  case Format_JPEG:
	  case Format_Sun:
		me.image_width = me.p_xv_in->base.win_width;
		me.image_height = me.p_xv_in->base.win_height;
		break;
	  default:
		fprintf(stderr, "Unknown format %d\n", format);
		return;
	}
	me.format = format;
	XFlush(me.p_xv_in->base.display);
	resize(me, me.image_width, me.image_height);
}


/*---------------------------------------------------------------------------*
 |"cb_help"
 *---------------------------------------------------------------------------*/
void cb_help(w, item_name, event)
    Widget  w;             /* = widget causing the callback. */
    char*   item_name;     /* -> item help is desired for. */
    XEvent* event;         /* Event causing keyboard or mouse callback. */
{
	Arg 	args[20];		/* XtSet argument list. */
	Pixel   background;		/* Background color for help panel. */
	char    buf[80*25];		/* Buffer to hold text. */
	int		i_args;			/* Element in args accessing. */
	int		i_item;			/* Help item now testing against item_name. */
	int     i_line;			/* Line of help text now processing. */
	int		n_help;			/* Number of lines in data pointed to by p_help. */
	char*   p_buf;			/* Next location in buf to set. */
	char**	p_help;			/* Help information to use. */
	Widget  w_form;			/* Form widget containing 'w_text'. */
	Widget	w_help;			/* Popup shell containing the other widgets. */
	Widget 	w_pane;			/* Pane widget used to manage presentation. */
	Widget  w_text;			/* Text scrolling widget. */

	XtVaGetValues(XtParent(w),XmNbackground,&background,NULL);


	/*  CREATE A POPUP SHELL CONTAINING A PANED WINDOW TO PRESENT THE
	 *  HELP INFORMATION
	 */
	w_help = XtVaCreatePopupShell("Capture - Help",
		xmDialogShellWidgetClass,	XtParent(w),
		XmNdeleteResponse,			XmDESTROY,
		XmNbackground,				background,
	NULL);

	w_pane = XtVaCreateWidget("pane",
		xmPanedWindowWidgetClass,	w_help,
		XmNbackground,				background,
	NULL);

	w_form = XtVaCreateWidget("form",
		xmFormWidgetClass,			w_pane,
		XmNbackground,				background,
	NULL);

	/*
	 *  BUILD BUFFER WITH TEXT FOR SCROLL AREA BASED ON WHAT HELP IS DESIRED.
	 */
 	for ( p_help = 0, i_item = 0; i_item < n_help_items; i_item++ )
	{
		if ( !strcmp(item_name, help_items[i_item].name) )
		{
			p_help = help_items[i_item].p_p_text;
			n_help = help_items[i_item].n_lines;
		}
	}
	if ( p_help == 0 )
	{
		return;
	}
	for ( p_buf =  buf, i_line = 0; i_line < n_help; i_line++ )
	{
		p_buf += strlen(strcpy(p_buf, p_help[i_line]));
		*p_buf++ = '\n';
		*p_buf = 0;
	}
	*(--p_buf) = 0;			/* Remove final newline. */

	i_args = 0;
	XtSetArg(args[i_args],	XmNbackground,	      background);        i_args++;
	XtSetArg(args[i_args],  XmNcolumns,			  80);                i_args++;
	XtSetArg(args[i_args],	XmNcursorPositionVisible,False);          i_args++;
	XtSetArg(args[i_args],	XmNeditMode,		  XmMULTI_LINE_EDIT); i_args++;
	XtSetArg(args[i_args],	XmNeditable,		  False);             i_args++;
	XtSetArg(args[i_args],	XmNrows,			  25);                i_args++;
	XtSetArg(args[i_args],	XmNscrollVertical,	  True);              i_args++;
	XtSetArg(args[i_args],	XmNscrollHorizontal,  False);             i_args++;
	XtSetArg(args[i_args],	XmNvalue,			  buf);               i_args++;
	XtSetArg(args[i_args],	XmNwordWrap,		  True);              i_args++;

	w_text = XmCreateScrolledText(w_form, "help_text", args, i_args);

	XtManageChild(w_text);
	XtManageChild(w_form);

	XtManageChild(w_pane);
	XtPopup(w_help,XtGrabNone);
}


/*----------------------------------------------------------------------------*
 |"cb_input"
 |
 |  Callback routine for setting the video encoding to use.
 *----------------------------------------------------------------------------*/
void cb_input( w, p_encoding, event )
    Widget          w;          /* = widget causing the callback. */
    XvEncodingInfo* p_encoding; /* -> encoding selected by user. */
    XEvent*         event;      /* Event causing callback. */
{
	int		id;		/* Encoding selected by user. */

	id = p_encoding->encoding_id;
	O_XV_Source(me.p_xv_in, (id & XV_PLX_DMSD) >> 4, id & XV_SOURCE_TIMING,
		id & XV_SOURCE_TYPE, id & XV_PLX_INPUT);
	cb_resize(w, &me, 0);
}


/*---------------------------------------------------------------------------*
 |"cb_operator"
 |
 | Callback routine for processing user key strokes and mouse events.
 *---------------------------------------------------------------------------*/
void cb_operator(w, p_me, event)
    Widget    w;             /* = widget causing the callback. */
    Capture*  p_me;          /* Our instance specific data. */
    XEvent*   event;         /* Event causing keyboard or mouse callback. */

{
    Window 			child;          /* The ID of child of window pointer is in*/
	int				loop;			/* Loop control. */
	unsigned int    mask;     		/* The button mask. */
	O_XV*			p_xv;			/* Fast access to O_XV: p_me->p_xv_in. */
    Window 			root;           /* The root window ID pointer is on.*/
    int 			rx;    			/* Return the x and y cordinates of the */
    int 			ry;             /* Pointer relative to the root's origin.*/
    int 			wx;             /* Return the x and y coordinates of the */
    int 			wy;             /* Pointer relative to origin of window. */

	p_xv = p_me->p_xv_in;

	switch( event->type )
	{
	  case ButtonPress:
        XQueryPointer(p_xv->display, p_xv->w_main, &root, &child,
            &rx, &ry, &wx, &wy, &mask);
		switch( mask & (Button1Mask|Button2Mask|Button3Mask) ) 
		{
		  case Button1Mask: 
			beep(1);
			switch ( p_me->format )
			{
			  case Format_H261_CIF:
			  case Format_H261_NTSC:
			  case Format_H261_QCIF:
				h261_capture(p_me); 
				break;
			  case Format_JPEG:
				break;
			  case Format_Sun: 
				sun_capture(p_me);
				break;
			  default:
				fprintf(stderr, "*** Internal error %d is not a valid format\n", 
					p_me->format);
				return;
			}
			beep(2);
			break;
		  case Button2Mask: 
			for (;;)
			{
				XQueryPointer(p_xv->display, p_xv->w_main, &root, &child,
					&rx, &ry, &wx, &wy, &mask);
				if ( (mask & Button3Mask) == Button3Mask ) 
				{
					switch ( p_me->format )
					{
					  case Format_H261_CIF:
					  case Format_H261_NTSC:
					  case Format_H261_QCIF:
						break;
					  case Format_JPEG:
						break;
					  case Format_Sun: 
						break;
					}
					beep(2);
					break;
				}
				beep(1);
				switch ( p_me->format )
				{
				  case Format_H261_CIF:
				  case Format_H261_NTSC:
				  case Format_H261_QCIF:
					h261_capture(p_me); 
					break;
				  case Format_JPEG:
					printf( "write_jpeg( p_me->fd, p_me->image ); \n");
					break;
				  case Format_Sun: 
					sun_capture(p_me);
					break;
				  default:
					fprintf(stderr, 
						"*** Internal error %d is not a valid format\n", 
						p_me->format);
					return;
				}
			}
			return;
		  default: 
			break;
		}
    }
}


/*----------------------------------------------------------------------------*
 |"cb_input"
 |
 |  Callback routine for setting the video encoding to use.
 *----------------------------------------------------------------------------*/
void cb_output( w, id, event )
	Widget 	w;      /* = widget causing the callback. */
	int		id;		/* -> encoding selected by user. */
	XEvent*	event;  /* Event causing callback. */
{
    short  video_w;    /* Current width of video window. */
    short  video_h;    /* Current height of video window. */

    printf("Setting output to %08x\n", id);
    O_XV_Source(me.p_xv_out, 1, id & XV_SOURCE_TIMING,
        id & XV_SOURCE_TYPE, id & XV_PLX_INPUT);
    XtVaGetValues(me.w_video_record,
        XmNwidth, &video_w,
        XmNheight, &video_h,
    NULL);
    XvGetVideo(me.p_xv_out->base.display, me.p_xv_out->base.port_id,
                me.p_xv_out->base.w_main, me.p_xv_out->base.gc,
                0, 0, me.p_xv_out->base.sig_width, me.p_xv_out->base.sig_height,
                0, 0, video_w, video_h
    );
}


/*----------------------------------------------------------------------------*
 |"cb_resize"
 |
 |  Callback routine for resizing of video window. Note of the incoming
 |  parameters we only look at the type field of the event parameter.
 *----------------------------------------------------------------------------*/
static void cb_resize( w, p_me, event )
    Widget      w;      /* =  widget causing the callback. */
    Capture*    p_me;   /* -> our instance specific data. */
    XEvent*		event;  /* =  event causing resize callback. */
{
	short int	video_h;	/* Height coordinate of resized video window. */
	short int	video_w;	/* Width coordinate of resized video window. */

	switch( p_me->format )
	{
	  case Format_H261_CIF:
	  case Format_H261_NTSC:
	  case Format_H261_QCIF:
		resize( p_me, p_me->image_width, p_me->image_height );
		break;
	  case Format_JPEG:
	  case Format_Sun:
		XtVaGetValues(p_me->w_video_record,
			XmNwidth, &video_w,
			XmNheight, &video_h,
		NULL);
		resize( p_me, video_w, video_h );
	  default:
		break;	
	}

}

/*---------------------------------------------------------------------------*
 |"cb_save_to"
 *---------------------------------------------------------------------------*/
void cb_save_to(w, p_me, event)
    Widget    w;             /* = widget causing the callback. */
    Capture*  p_me;          /* Our context. */
    XEvent*   event;         /* Event causing keyboard or mouse callback. */
{
    buildfile_select(w, p_me, "Save Settings");
    XtAddCallback(p_me->file.w_dialog,
        XmNokCallback, cb_save_to_ok, p_me);
    XtManageChild(p_me->file.w_dialog);
}


/*---------------------------------------------------------------------------*
 |"cb_save_to_ok"
 *---------------------------------------------------------------------------*/
void cb_save_to_ok(w, p_me, event)
    Widget    w;             /* = widget causing the callback. */
    Capture*  p_me;          /* Our context. */
    XEvent*   event;         /* Event causing keyboard or mouse callback. */
{
    XmString*   p_string;       /* Raw file name specification. */

    XtVaGetValues(p_me->file.w_dialog, XmNdirSpec, &p_string, NULL);
    if(!XmStringGetLtoR(p_string, XmSTRING_DEFAULT_CHARSET, &p_me->file_name))
        return;
	XtDestroyWidget(w);
    XmStringFree(p_string);


	/*  IF WE HAVE BEEN RECORDING MAKE SURE AND TEAR DOWN FIRST. */

	switch( p_me->format )
	{
	  case Format_H261_CIF:
	  case Format_H261_NTSC:
	  case Format_H261_QCIF:
		h261_teardown(p_me);
		break;
	  case Format_JPEG:
		break;
	  case Format_Sun:
		sun_teardown(p_me);
		break;
	  default:
		break;	
	}
}


/*-------------------------------------------------------------------------*
 |"cb_select_color"
 *-------------------------------------------------------------------------*/
void cb_select_color(w, p_me)
	Widget   w;
	Capture* p_me;
{
    XtVaSetValues(w,
        XmNselectColor, p_me->file.color_select,
        XmNbackground,p_me->file.color_select,
    NULL);
    if (w == p_me->file.w_select) 
    {
		XtVaSetValues(p_me->file.w_filter,
			XmNselectColor, p_me->file.color_select,
			XmNbackground,p_me->file.color_bg,
		NULL);
    } 
	else 
	{
		XtVaSetValues(p_me->file.w_select,
        	XmNselectColor, p_me->file.color_select,
        	XmNbackground,p_me->file.color_bg,
    	NULL);
    }
 
}


/*-------------------------------------------------------------------------*
 |"h261_capture"
 |
 |  Encode an H.261 frame.
 *-------------------------------------------------------------------------*/
void h261_capture( p_me )
	Capture*	p_me;
{
	int		n_image;	/* Where to store size of buffer being returned. */
	XImage*	p_image;	/* Where to store pointer to buffer being returned. */

	switch( p_me->capture_state )
	{
	  case CaptureS_unallocated:
		h261_setup(p_me);
	  case CaptureS_allocated:
		init_p64EncodeSequence();
		p_me->capture_state = CaptureS_encoding;
		break;
	  case CaptureS_encoding:
		break;
	  default:
		fprintf(stderr, 
			"h261_capture: internal error %d is not a legal state\n",
			p_me->capture_state);
		exit(1);
	}

#ifdef NOTDEF
	XGetSubImage(p_me->p_xv_in->base.display, p_me->p_xv_in->base.w_main, 
		0, 0, p_me->image_width, p_me->image_height,
		-1, ZPixmap, p_me->image, 0, 0);
	rgbData = p_me->image->data;
#else
	O_XV_Video_Off(p_me->p_xv_in);
	XvGetImage(p_me->p_xv_in->base.display, p_me->p_xv_in->base.port_id,
		p_me->p_xv_in->base.w_main, p_me->p_xv_in->base.gc,
		p_me->a_ximage, &n_image, &p_image,
		0, 0, p_me->image_width, p_me->image_height,
		0, 0, p_me->image_width, p_me->image_height);
	O_XV_Video_On(p_me->p_xv_in);
#endif
	rgbData = p_image;
    RgbToYCrCbProc(&rgbData[3], &rgbData[2], &rgbData[1],
	    lumaP, chromaRedP, chromaBlueP, p_me->image_width, p_me->image_height);
	p64EncodeFrame();
	XDestroyImage(p_image); /* #### */
}


/*-------------------------------------------------------------------------*
 |"h261_setup"
 |
 |  If we have not allocated storage and initialized the p64 code do so
 |  here.
 *-------------------------------------------------------------------------*/
void h261_setup( p_me )
	Capture*	p_me;
{

  	/* alloc YCrCb memory */

  	lumaP = malloc((p_me->image_width*p_me->image_height*3) >> 1);
  	if( lumaP == NULL ) 
	{
    	fprintf(stderr,"malloc failure\n");
    	exit(-1);
  	}
  	chromaRedP = lumaP + p_me->image_width*p_me->image_height;
  	chromaBlueP = chromaRedP + ((p_me->image_width*p_me->image_height) >> 2);
	
	if ( !p_me->file_name )
	{
		/* #### */
	}
	Argv[3] = p_me->file_name;
  	if (initp64Main(Argc,Argv) != 0 ) 
	{
    	fprintf(stderr,"initp64Main failure\n");
    	exit(1);
  	}

#ifdef NOTDEF
	if ( !p_me->image )
	{
		p_me->image = XGetImage(p_me->p_xv_in->base.display, 
			p_me->p_xv_in->base.w_main, 0, 0, p_me->p_xv_in->base.win_width, 
			p_me->p_xv_in->base.win_height, -1, ZPixmap);
	}
#endif
}


/*-------------------------------------------------------------------------*
 |"h261_teardown"
 |
 |  Deallocate storage etc. for p64 processing.
 *-------------------------------------------------------------------------*/
void h261_teardown( p_me )
	Capture*	p_me;
{

	switch( p_me->capture_state )
	{
	  case CaptureS_unallocated:
		return;
	  case CaptureS_allocated:
	  case CaptureS_encoding:
		finish_p64EncodeSequence();
		break;
	  default:
		fprintf(stderr, 
			"h261_teardown: internal error %d is not a legal state\n",
			p_me->capture_state);
		exit(1);
	}

  	free(lumaP);
	p_me->capture_state = CaptureS_unallocated;
}

/*-------------------------------------------------------------------------*
 |"jpeg_capture"
 |
 |  If we have not allocated storage and initialized the jpeg code do so
 |  here.
 *-------------------------------------------------------------------------*/
void jpeg_capture( p_me )
	Capture*	p_me;
{
	int		n_image;	/* Where to store size of buffer being returned. */
	XImage*	p_image;	/* Where to store pointer to buffer being returned. */
    short   video_w;    /* Current width of video window. */
    short   video_h;    /* Current height of video window. */


	switch( p_me->capture_state )
	{
	  case CaptureS_unallocated:
		jpeg_setup(p_me);
	  case CaptureS_allocated:
		p_me->capture_state = CaptureS_encoding;
		break;
	  case CaptureS_encoding:
		break;
	  default:
		fprintf(stderr, 
			"jpeg_capture: internal error %d is not a legal state\n",
			p_me->capture_state);
		exit(1);
	}

    XtVaGetValues(me.w_video_record,
        XmNwidth, &video_w,
        XmNheight, &video_h,
    NULL);
	O_XV_Video_Off(p_me->p_xv_in);
	XvGetImage(p_me->p_xv_in->base.display, p_me->p_xv_in->base.port_id,
		p_me->p_xv_in->base.w_main, p_me->p_xv_in->base.gc,
		p_me->a_jpeg, &n_image, &p_image,
		0, 0, video_w, video_h, 0, 0, video_w, video_h);
	O_XV_Video_On(p_me->p_xv_in);
	free(p_image);
}



/*-------------------------------------------------------------------------*
 |"jpeg_setup"
 |
 |  If we have not allocated storage and initialized the jpeg code do so
 |  here.
 *-------------------------------------------------------------------------*/
void jpeg_setup( p_me )
	Capture*	p_me;
{
}


/*-------------------------------------------------------------------------*
 |"jpeg_teardown"
 |
 |  Deallocate storage etc. for jpeg processing.
 *-------------------------------------------------------------------------*/
void jpeg_teardown( p_me )
	Capture*	p_me;
{

	switch( p_me->capture_state )
	{
	  case CaptureS_unallocated:
		return;
	  case CaptureS_allocated:
	  case CaptureS_encoding:
		break;
	  default:
		fprintf(stderr, 
			"jpeg_teardown: internal error %d is not a legal state\n",
			p_me->capture_state);
		exit(1);
	}

	p_me->capture_state = CaptureS_unallocated;
}


/*----------------------------------------------------------------------------*
 |"resize"
 |
 |  Callback routine for resizing of video window. Note of the incoming
 |  parameters we only look at the type field of the event parameter.
 *----------------------------------------------------------------------------*/
static void resize( p_me, video_w, video_h )
    Capture*    p_me;   	/* -> our instance specific data. */
	int			video_w;	/* =  width coordinate of resized video window. */
	int		    video_h;	/* =  height coordinate of resized video window. */
{
	short int	menu_h; 	/* Height of resized menu window. */

	XtVaGetValues(p_me->w_bar,
		XmNheight, &menu_h,
	NULL);

	if ( video_w > p_me->p_xv_in->base.sig_width )  
		video_w=p_me->p_xv_in->base.sig_width;
	if ( video_h > p_me->p_xv_in->base.sig_height ) 
		video_h=p_me->p_xv_in->base.sig_height;

	XtVaSetValues(p_me->w_toplevel,
		XmNwidth, video_w,
		XmNheight, video_h + menu_h,
	NULL);

	XtVaSetValues(p_me->w_form,
		XmNwidth, video_w,
		XmNheight, video_h + menu_h,
	NULL);

	XtVaSetValues(p_me->w_video_record,
		XmNwidth, video_w,
		XmNheight, video_h,
	NULL);

	p_me->p_xv_in->base.win_width = video_w;
	p_me->p_xv_in->base.win_height = video_h;
	O_XV_Video_On(p_me->p_xv_in);
}


/*-------------------------------------------------------------------------*
 |"sun_capture"
 |
 |  If we have not allocated storage and initialized the sun code do so
 |  here.
 *-------------------------------------------------------------------------*/
void sun_capture( p_me )
	Capture*	p_me;
{
	int		n_image;	/* Where to store size of buffer being returned. */
	XImage*	p_image;	/* Where to store pointer to buffer being returned. */
    short   video_w;    /* Current width of video window. */
    short   video_h;    /* Current height of video window. */


	switch( p_me->capture_state )
	{
	  case CaptureS_unallocated:
		sun_setup(p_me);
	  case CaptureS_allocated:
		p_me->capture_state = CaptureS_encoding;
		break;
	  case CaptureS_encoding:
		break;
	  default:
		fprintf(stderr, 
			"sun_capture: internal error %d is not a legal state\n",
			p_me->capture_state);
		exit(1);
	}

#ifdef USE_XGetImage
	XGetSubImage(p_me->p_xv_in->base.display, p_me->p_xv_in->base.w_main, 
		0, 0, p_me->image_width, p_me->image_height,
		-1, ZPixmap, p_me->image, 0, 0);
	write_sun_raster( p_me->fd, p_me->image );
#else
    XtVaGetValues(me.w_video_record,
        XmNwidth, &video_w,
        XmNheight, &video_h,
    NULL);
	O_XV_Video_Off(p_me->p_xv_in);
	XvGetImage(p_me->p_xv_in->base.display, p_me->p_xv_in->base.port_id,
		p_me->p_xv_in->base.w_main, p_me->p_xv_in->base.gc,
		p_me->a_ximage, &n_image, &p_image,
		0, 0, video_w, video_h, 0, 0, video_w, video_h);
	O_XV_Video_On(p_me->p_xv_in);
	write_sun_raster( p_me->fd, p_image );
	XDestroyImage( p_image );
#endif
}


/*-------------------------------------------------------------------------*
 |"sun_setup"
 |
 |  If we have not allocated storage and initialized the sun code do so
 |  here.
 *-------------------------------------------------------------------------*/
void sun_setup( p_me )
	Capture*	p_me;
{
	char        err_msg[256];   /* Area to construct error message. */

	if ( !p_me->file_name )
	{
		/* #### */
	}
    else if ( !(p_me->fd = fopen(p_me->file_name, "w")) )
    {
        sprintf(err_msg, "Unable to open %s for write", p_me->file_name);
        create_dialog(p_me->w_toplevel, "Capture", err_msg, 0);
        return;
    }
#ifdef USE_XGetImage
	if ( !p_me->image )
	{
		p_me->image = XGetImage(p_me->p_xv_in->base.display, 
			p_me->p_xv_in->base.w_main, 0, 0, p_me->p_xv_in->base.win_width, 
			p_me->p_xv_in->base.win_height, -1, ZPixmap);
	}
#endif
}


/*-------------------------------------------------------------------------*
 |"sun_teardown"
 |
 *-------------------------------------------------------------------------*/
void sun_teardown( p_me )
	Capture*	p_me;
{
	if ( p_me->fd ) fclose(p_me->fd);
	p_me->capture_state = CaptureS_unallocated;
}	

