/*
 *  xvinfo.c:	Program to print out information about Xv extension.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 * 
 *  Copyright(c) 1995, Parallax Graphics, Inc.
 *
 *  REVISION HISTORY:
 *
 * $Log:   /plxhp/dave/pvcsrc/O_XV/archives/xvinfo.c_v  $
 * 
 *    Rev 1.1   09 Sep 1996 09:55:34   dave
 * Initial revision
 * Revision 1.1  1995/08/29 15:35:18  dist
 * Initial revision
 *
 */
#include <sys/types.h>			/* Standard types for this C environment. */
#include <stdio.h>				/* Stream input/output declarations. */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <Xvlib.h>
#include "Xvddx.h"
#include "Xvddx_PGI.h"
#include "O_XV.h"
#include "arg_parse.h"


/*----------------------------------------------------------------------------*
 |"main"
 |
 |  Entry point for system for this application.
 |
 *----------------------------------------------------------------------------*/
main( argc, argv )
	int		argc;		/* = # of command arguments parsed by shell. */
	char	*argv[];	/* = vector of pointers to command arguments parsed. */
{
	Display*		display;	/* Display accessing. */
	int				i_argv;		/* # of element in 'argv' accessing. */
	int				i_type;		/* Instance of adaptor type 'type' now using. */
	int				n_type;		/* # of adaptors of type 'type'. */
	O_XV_Select*	p_select;	/* Xv selection object. */	
	O_XV*			p_oxv;		/* Xv adaptor we allocated. */
	char*			type;		/* Type of adaptor to look for. */
	

	/*
	 *  PROCESS COMMAND LINE ARGUMENTS:
	 *
	 *    xvinfo 
	 */
	for ( i_argv = 1; i_argv < argc; i_argv++ )
	{
		type = argv[i_argv];
	}


	/*  OPEN THE DISPLAY, CREATE OUR XV SELECTION OBJECT AND DUMP ITS CONTENTS
	 */
	if ( !(display = XOpenDisplay(0)) ) 
	{
		fprintf(stderr, "\nCould not open display\n");
		return 0;
	}

	if ( !(p_select = O_XV_Select_O_XV_Select( 0, display)) )
	{
		fprintf(stderr, "\nCould not open display\n");
		return 0;
	}

	O_XV_Select_Dump(p_select, stdout);

	if ( type )
	{
		printf("\n\n");
		n_type = O_XV_Select_Query_Type(p_select, type);
		switch ( n_type )
		{
		  case 0:
			printf(" There are no adaptors of type %s\n", type);
			break;
		  case 1:
			printf(" There is 1 adaptor of type %s\n", type);
			break;
		  default:
			printf(" There are %d adaptors of type %s\n", n_type, type);
			break;
		}
		for ( i_type = 0; i_type < n_type; i_type++ )
		{
			p_oxv = O_XV_Select_Port(p_select, type, i_type, 0);
			printf(" O_XV object for %s adaptor %d port 0 is at %08x\n",
				type, i_type, p_oxv);
			if ( p_oxv != O_XV_Select_Port(p_select, type, i_type, 0) )
			{
				printf(" O_XV object for %s adaptor %d port 0 is now at %08x\n",
					type, i_type, 
					O_XV_Select_Port(p_select, "PGItv2", i_type, 0));
				printf(" And this is an error!!!\n");
			}
		}
	}

	O_XV_Select_Terminate(p_select);
}
