/*
 *  xvtv.c:	Test program for Xv video extension access object.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     26 May 93 - Andy Goodrich
 *       - Initial coding
 *
 * $Log:   /plxhp/dave/pvcsrc/O_XV/archives/xvtv.c_v  $
 * 
 *    Rev 1.2   09 Sep 1996 09:56:42   dave
 * Andy Goodrich: Added O_XV_Video_On() because the startup semantics
 *                of an O_XV object have changed, video no longer 
 *                defaults on.
 * Revision 1.2  1995/08/31 12:58:49  dist
 * Andy Goodrich: Added O_XV_Video_On() because the startup semantics
 *                of an O_XV object have changed, video no longer
 *                defaults on.
 *
 * Revision 1.1  95/05/16  10:10:46  10:10:46  dist ()
 * Initial revision
 * 
 *  Copyright(c) 1993, Andrew C. Goodrich
 */
#include <sys/types.h>			/* Standard types for this C environment. */
#include <stdio.h>				/* Stream input/output declarations. */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <Xvlib.h>

#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>
 

#include "Xvddx_PGI.h"
#include "Xvddx.h"
#include "O_XV.h"


/*----------------------------------------------------------------------------*
 |"main"
 |
 |  Entry point for system for this application.
 |
 *----------------------------------------------------------------------------*/
main( argc, argv )
	int		argc;		/* = # of command arguments parsed by shell. */
	char	*argv[];	/* = vector of pointers to command arguments parsed. */
{
	int			adaptor;		/* Adaptor to use. */
	Display*	display;		/* Display accessing. */
	int			dummy;			/* For returning values we don't want. */
	XEvent		event;			/* Window event we are processing. */
	int			gov;			/* = 1 if should provide graphics over video. */
	int			i_argv;			/* # of element in 'argv' accessing. */
	int			new_h;			/* New height of video window on screen. */
	int			new_w;			/* New width of video window on screen. */
	int			port;			/* Port # to use. */
	int			prompt;			/* = 1 if should prompt for termination. */
	O_XV*		p_xv;			/* Xv access object for port. */	
	int			source_dmsd;	/* DMSD to use. */
	int			source_number;	/* Instance number for video source. */
	int			source_format;	/* Format of source: composite, yc, etc. */
	int			source_type;	/* Type of video source: NTSC, PAL, etc. */
	
	if ( !(display = XOpenDisplay(0)) ) 
	{
		fprintf(stderr, "\nCould not open display\n");
		return 0;
	}


	/*  PROCESS COMMAND LINE ARGUMENTS:
	 */
	adaptor = 0;
	gov = 0;
	port = 0;
	prompt = 1;
	source_dmsd = -1;
	source_type = -1;
	source_format = -1;
	for ( i_argv = 1; i_argv < argc; i_argv++ )
	{
		if ( !strcmp(argv[i_argv], "adaptor") )
		{
			i_argv++;
			if ( (i_argv >= argc) || 
				 (sscanf(argv[i_argv], "%d", &adaptor) != 1) )
			{
				fprintf(stderr, "*** Missing number for adaptor keyword\n");
				return 0;
			}
		}
	    else if ( !strcmp(argv[i_argv], "composite") )
		{
			source_format = XV_COMPOSITE;
		}
		else if ( !strcmp(argv[i_argv], "dmsd") )
		{
			i_argv++;
			if ( (i_argv >= argc) || 
				 (sscanf(argv[i_argv], "%d", &source_dmsd) != 1) )
			{
				fprintf(stderr, "*** Missing number for dmsd keyword\n");
				return 0;
			}
		}
		else if ( !strcmp(argv[i_argv], "gov") )
		{
			gov = 1;
		}
		else if ( !strcmp(argv[i_argv], "help") )
		{
			fprintf(stderr, "Usage: xvtv [loop|gov] [adaptor no] [port no] ");
			fprintf(stderr, "[dmsd no] [ntsc|pal] [composite|yc][no]\n");
			return 0;
		}
		else if ( !strcmp(argv[i_argv], "loop") )
		{
			prompt = 0;
		}
		else if ( !strcmp(argv[i_argv], "ntsc") )
		{
			source_type = XV_NTSC;
		}
		else if ( !strcmp(argv[i_argv], "pal") )
		{
			source_type = XV_PAL;
		}
		else if ( !strcmp(argv[i_argv], "port") )
		{
			i_argv++;
			if ( (i_argv >= argc) || 
				 (sscanf(argv[i_argv], "%d", &port) != 1) )
			{
				fprintf(stderr, "*** Missing number for port keyword\n");
				return 0;
			}
		}
		else if ( !strcmp(argv[i_argv], "yc") )
		{
			source_format = XV_YC;
		}
		else if ( sscanf(argv[i_argv], "%d", &source_number)!=1 )
		{
			fprintf(stderr, "%s is an unrecognized parameter\n",
			argv[i_argv]);
			return 0;
		}
	}


	/*  CREATE OUR XV ACCESS OBJECT AND CREATE OUR VIDEO WINDOW:
	 *
	 *  (1) Set the source type and create the video window.
	 *  (2) If graphics over video is requested call the gov support,
	 *      otherwise just prompt the operator for a return key hit.
	 */
	if ( !(p_xv = O_XV_O_XV( 0, display, adaptor, port )) )
	{
		fprintf(stderr, "\nCould not open display\n");
		return 0;
	}
	O_XV_Set_Attribute(p_xv, 
		XInternAtom(p_xv->display, "XV_PLX_CHANNEL", False), port ? 2 : 1);
	if ( source_dmsd == -1 ) 
		source_dmsd = (p_xv->p_encodings->encoding_id & XV_PLX_DMSD) >> 4;
	if ( source_type == -1 ) 
		source_type = p_xv->p_encodings->encoding_id & XV_SOURCE_TIMING;
	if ( source_format == -1 ) 
		source_format = p_xv->p_encodings->encoding_id & XV_SOURCE_TYPE;
	source_dmsd = ( source_format == XV_YC ) ? 1 : source_dmsd;
	source_dmsd = ( source_format == XV_COMPOSITE && source_number == 3 ) ?
		0 : source_dmsd;
	O_XV_Source(p_xv, source_dmsd, source_type, source_format, source_number);

	O_XV_Window( p_xv, -1, -1 );
	O_XV_Video_On( p_xv );

	if ( gov )
	{
		process_geometry(p_xv, 0, 0, 0);
	}
	else if ( prompt )
	{
		printf("hit return to continue:");
		fgetc(stdin);
	}
	else
	{
		for(;;)
		{
			XNextEvent(p_xv->display, &event);
#ifdef NOTDEF
			XWindowEvent(p_xv->display, p_xv->w_main,
				(KeyPressMask|VisibilityChangeMask|ResizeRedirectMask),
				&event);
#endif
			switch ( event.type )
			{
			  case ConfigureNotify:
			  case Expose:
			  case VisibilityNotify:
				XGetGeometry(p_xv->display, p_xv->w_main, &dummy, &dummy, 
					&dummy, &new_w, &new_h, &dummy, &dummy);
				if ( new_w==p_xv->win_width && new_h==p_xv->win_height )break;
				if ( new_w > p_xv->sig_width ) new_w = p_xv->sig_width;
				if ( new_h > p_xv->sig_height ) new_h = p_xv->sig_height;
				if ( (new_w * p_xv->sig_height) / p_xv->sig_width  > new_h )
				{
					new_w = (new_h * p_xv->sig_width ) / p_xv->sig_height;
				}
				else
				{
					new_h = (new_w * p_xv->sig_height ) / p_xv->sig_width;
				}
				p_xv->win_width = new_w;
				p_xv->win_height = new_h;
				XResizeWindow(p_xv->display, p_xv->w_main, new_w, new_h);
				O_XV_Video_On( p_xv );
				break;
			}
		}
	}
}
