/************************************/
#define DEBUG
#ifdef DEBUG
#define dprintf(a) printf(a)
#else
#define dprintf(a)
#endif
/************************************/

/***************XV's CCube Command register values************/
#ifdef mips
#define C3_INDIAN 0
#else
#define C3_INDIAN 0x4000000
#endif
/* for reading c3 			   */
#define XVC3_RESET_TO_PIO 0
#define XVC3_START_DMA_TO_HOST_1   0x1010119 | C3_INDIAN
#define XVC3_START_DMA_TO_HOST_2   0x0050545 | C3_INDIAN
#define XVC3_PAUSE_DMA_TO_HOST     0x1050545 | C3_INDIAN
#define XVC3_PURGE_BIC_GO_PIO	   0x1040465
#define XVC3_CONT_DMA_TO_HOST_1	   0x1050511 | C3_INDIAN
#define XVC3_CONT_DMA_TO_HOST_2	   XVC3_START_DMA_TO_HOST_2
#define XVC3_PURGE_C3_CONT_DMA	   0x2050545 | C3_INDIAN

/* for writing c3			   */
#define XVC3_START_DMA_FROM_HOST_1 0x102021a | C3_INDIAN
#define XVC3_START_DMA_FROM_HOST_2 0x0060646 | C3_INDIAN
#define XVC3_PAUSE_DMA_FROM_HOST   0x0060642 | C3_INDIAN
#define XVC3_WRITE_GO_PIO	   0x0040402 /* Bob change */
#define XVC3_CONT_DMA_FROM_HOST_1  0x0060602 | C3_INDIAN
#define XVC3_CONT_DMA_FROM_HOST_2  XVC3_START_DMA_FROM_HOST_2

/***************************************/
/* stall conditions		       */
#define STALL_ALWAYS	0
#define STALL_NH_O_FE	1
#define STALL_NH_O_NFE	2
#define STALL_NH	3
#define STALL_H_O_FE	4
#define STALL_FE	5
#define STALL_H_XO_NFE	6
#define STALL_NH_FE	7
#define STALL_H_O_NFE	8
#define STALL_H_XO_FE	9
#define STALL_NFE	10
#define STALL_NH_NFE	11
#define STALL_H		12
#define STALL_H_FE	13
#define STALL_H_NFE	14
#define STALL_NEVER	15
/***************************************/
#define	STATE0	0		/* Done or idle state			  */
#define STATEW1	0x81		/* Write started, waiting for FIFO 3QFull */
				/* or end of buffer.			  */
#define	STATEW2	0x82		/* Write started, waiting for FIFO empty  */
#define STATEW3	0x83

#define	STATER1	1		/* Read started, waiting for FIFO QFull	  */
				/* or video pipeline inactive.		  */
#define	STATER2	2		/* Read started, waiting for FIFO empty	  */
#define STATEW 	0x80		/* Bit indicating a write state		  */
#define STATE_ERR -1
/***********************EOF***************/

#define TRUE 1
#define FALSE 0

#define C3HBLANK        192             /* 32 */
#define C3HSYNC         16              /* 16 */
#define C3VBLANK        128             /* 8 */
#define C3VSYNC         16              /* 8 */


#ifdef NOTDEF
extern C3_WREGS *c3_wreg;
extern C3_RREGS *c3_rreg;
#endif 

#ifdef DELAY
#undef DELAY
#endif

extern int loopsperusec;                        /* loops per usec */

#define DELAY1 1
#define DELAY5 5
#define DELAY10 10
#define DELAY100 100
#define DELAY200 200
#define DELAY500 500

#define DELAY(usecs)\
	{ \
		volatile int i = loopsperusec * usecs; \
		while(--i > 0); \
	}

