
/* file pxv.c parallax xvideo base driver for hp-ux */

/* Included header files */

#ifndef lint
static char *rcsid = "@(#)$Id: plxv.c,v 1.11 94/06/09 01:42:51 rocky Exp $ Parallax Graphics Inc";
#endif

typedef char *dm_message; /* bug in include files requires this */

#include "h/types.h"
#include "h/param.h"
#ifndef HPUX_10
#include "h/user.h"
#endif
#include "h/errno.h"
#include "h/ioctl.h"
#include "h/buf.h"
#include "machine/eisa.h" 
#include "machine/eeprom.h"
#include "h/io.h"
#include "h/framebuf.h"
#include "h/uio.h"

#include "plxv.h"
#include "xvideo.h"
#include "c3.h"
#include "c3dma.h"
#include "plxv_softc.h"

#ifdef HPUX_10
#include <sys/iomap.h>
#include "machine/inline.h"
#include "h/conf.h"
#include <sys/mman.h>
#include <sys/wsio.h>
#endif	/* HPUX_10 */

#define LEAVE_INTS_ON /* */

#if 0
enum {
    DELAY10  = 10*10,
    DELAY20  = 20*10,
    DELAY50  = 50*10,
        DELAY100 = 100*10,
        DELAY200 = 200*10,
        DELAY500 = 500*10,
};
#endif

/* #define PLXV_DIAG 0x81  /* */
#include "plxv_debug.h"

#define EISA_BASE_PHYSADDR  0xFC000000
#define PLXV_PHYSADDR_OFFSET 0x1000000

#define ROUNDUP4K(a) (((a) + 4095) & ~4095)

/* #define PLXV_MEM_SIZE (1024 * 1024 * 8) /* all of it */
#define PLXV_MEM_SIZE ROUNDUP4K(sizeof(XV_mem_map))

#define PLXV_ID 0x41980000
#define PLXV_ID_MASK 0xffff0000
#define PLXV_REV_MASK 0xf
#define PLXV_OPTION_MASK 0xf0

#define PLXV_CCUBE_IRQ 5
#define PLXV_IMAGE_IRQ 15
#define PLXV_CCUBE_CHANNEL 0
#define PLXV_IMAGE_CHANNEL 5
#define PLXV_DMA_TIMING ?
#define PLXV_DMA_MODE 0
#define PLXV_DMA_SIZE  DMA_32BYTE

#if 1
#define PLXV_INITBITS 0x02222402
#else
#define PLXV_INITBITS 0x02222000
#endif
#define PLXV_DISABLE_BITS 0x02200402

static int    dummy_strategy();
extern int    (*eisa_attach)();
static int    image_isr();
extern struct isc_table_type *get_new_isc();
extern void*  map_isa_address();

#ifdef HPUX_10
static int plxv_open();
static int plxv_close();
static int plxv_read();
static int plxv_write();
static int plxv_ioctl();
static int plxv_install();
#endif /* HPUX_10 */

static int plxv_attach();
static int plxv_init();
static int plxv_isr();
static int (*plxv_saved_attach)();
static int plxv_strategy();

#ifdef NOTDEF
static int BIntStat,IntStat;
static unsigned int ladd;
#endif

#define PLXV_NFBS 4
#define PLXV_JPEG_WRITE  0x13000000 /* dma enb. + read */
#define PLXV_JPEG_READ   0x11000000 /* dma enb. + !read */
#define PLXV_IMAGE_WRITE 0x1c000000 /* dma enb. + read */
#define PLXV_IMAGE_READ  0x14000000 /* dma enb. + !read */

#ifdef NOTDEF

typedef struct
{
    unsigned long start_address;
    unsigned long counters;
    unsigned long control;
} dma_ctl;


static struct plxv_softc {
    int             w,
                    h;                  /* resolution */
	int             BIntStat;           /* last value read from BIntStat reg. */
	int             IntStat;		    /* last value read from IntStat reg. */
    int             bit24size;          /* total size of 24 bit frame buffer */
    int             bit8size;           /* total size of 8 bit frame buffer */
    int             bit1size;           /* total size of 1 bit frame buffer */
    struct proc     *owner;             /* owner of the frame buffer */
    u_long          plxv_physical;      /* board physical base address */
    int             int_pri;            /* interrupt priority             */
    int             attached;           /* we were attached              */
    XV_struct       *xv;                /* structure returned by hlib */
    int             blank;              /* blank or not */
    int             dmastate;           /* dma / interupt state */
    struct buf      dmabuf;             /* buffer struct for dma */
    int             shmid;              /* shared memory id of address space */
    int             *regaddr;           /* register block virtual address */
    int             *memaddr;           /* board virtual address */
    struct isc_table_type *isc;         /* boards isc */
    dma_ctl         *dmactl;            /* pointer to dma control regs */
    unsigned int    dmactlbits;         /* bits for above */
    int             c3_present;         /* = 1 if c3_present. */
    unsigned int    ladd;				/* last buffer address. */
} plxv_softc[PLXV_NFBS * 2];
#endif
static SOFTC plxv_softc[PLXV_NFBS * 2];

static int n_softcs = 0;	/* # of softc structs allocated so far. */
static struct plxv_softc*  slot_to_softc[10]; /* Softc ptrs indexed by slot # */

int loopsperusec;	/* loops per micro second */

/* hpux 10 structs */

#ifdef HPUX_10

static  drv_ops_t plxv_ops = {
	plxv_open,
	plxv_close,
	plxv_strategy,
	NULL,
	NULL,
	NULL,
	plxv_read,
	plxv_write,
	plxv_ioctl,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	C_ALLCLOSES|C_MAP_BUFFER_TO_KERNEL,
};

static drv_info_t plxv_info  = {
	"plxv", 
	"char", 
	DRV_CHAR, 
	-1,
	-1,
	NULL,
	NULL,
	NULL,
	};

static wsio_drv_data_t plxv_data = {
	"Xvideo Overlay",
	T_INTERFACE,
	DRV_CONVERGED,
	NULL,
	NULL,
	};

static wsio_drv_info_t plxv_wsio_info = {
	&plxv_info,
	&plxv_ops,
	&plxv_data,
};

#endif /* HPUX_10 */
/*
 *  STATISTICS GATHERING STORAGE:
 */
static stat_disp = 0;
static stat_int = 0;
static stat_no_bits = 0;

/* timeout routine when waiting on get image */
static void wffield_timeout(unit)
int unit;
{

    DEBUGF(0x40, ("wffield_timeout\n"));
    wakeup(&plxv_softc[unit].viwake);   /* wake up strategy */
}

/* routine to cal loops per micro second */

static void init_loopsperusec(xvm)
XV_mem_map *xvm;
{
	volatile unsigned int i;
	volatile unsigned int *tmrCnt;
	volatile unsigned int *tmrSpd;
	unsigned int pri, j, k, l, m;
	unsigned long usecs, usecspertick;

	tmrCnt = &xvm->RW_regs.Read_regs.TmrCnt.l;
	tmrSpd = &xvm->RW_regs.Write_regs.TmrSpd.l;

	*tmrSpd =  0x171;	    /* Initialize the timer. */

	while(((int)(m = *tmrCnt)) < 0); /* wait 'till timer starts */

	pri = spl6();		/* we don't want ints in here */

	usecspertick = 1000000000 / 90000;
	k = 10000;
	do
	{
		k *= 10;
		i = k;
		m = *tmrCnt;
		while(--i > 0);
		l = *tmrCnt;
		j = l - m;
	} while (j < 1000);		/* must see at least 1000 ticks */

	splx(pri);		/* ok, let ints happen */

	usecs = (j * usecspertick) / 1000;

	loopsperusec = k / usecs;
}

/******************************************
        c3 utility routines
******************************************/

static int plxv_is_c3_present(p_regs)
        volatile XV_read_regs*  p_regs;
{
        int interrupts;

	 	/*  MAKE SURE CCUBE INTERRUPT IS LEVEL TRIGGERED AND THEN
		 *  CHECK TO SEE IF WE CAN CLEAR IT. IF NOT THERE IS NO CCUBE.
		 */
        ((XV_write_regs*)p_regs)->DispKludge.l = -1; 
        interrupts = p_regs->IntStatC.l;
        interrupts = p_regs->IntStatC.l;
        return ( interrupts & (1 << 1) ) ? 0 : 1; 
}

static void plxv_initc3(xv)
XV_struct *xv;
{
        int interrupts;

        PV_WREG(xv, C3Stall.l, 0x140);
        C3_WREG(xv, S_reset, ~0);
}

SOFTC* dev_to_softc( dev )
   dev_t dev;
{
   return &plxv_softc[minor(dev)];
}

SOFTC* unit_to_softc( unit )
   int  unit;
{
   return &plxv_softc[unit];
}


/********************************************
        function to init pointers
*********************************************/
XV_struct *plxv_context_init(reg,m1,m8,m24,c3,eprom)
int *reg,*m1,*m8,*m24,*c3,*eprom;
{
    int Xres,Yres,i,*ip,j;
    XV_struct      *cp;
    XV_shdw_regs   *sp;
    XV_addr_maps   *ap;

	DEBUGF(0x80, ("plxv_context_init: entering\n"));

    if ((cp = (XV_struct *)sys_memall(sizeof(XV_struct))) == NULL)
    { 
        printf("malloc failed on plxv_init_struct:main struct\n");
        exit(-1);
    }
    if ((sp = (XV_shdw_regs *)sys_memall(sizeof(XV_shdw_regs))) == NULL)
    {
        printf("malloc failed on plxv_init_struct:xv_shdw\n");
        exit(-1);
    }
    if ((ap = (XV_addr_maps *)sys_memall(sizeof(XV_addr_maps))) == NULL)
    {
        printf("malloc failed on plxv_init_struct:addr_maps\n");
        exit(-1);
    }


    /* fill in the structure */

    cp->c3_rreg = (C3_RREGS *)c3;
    cp->c3_wreg = (C3_WREGS *)c3;
    cp->eprombase = eprom;
    cp->regbase = reg;
    cp->c3base = c3;
    cp->m24base = m24;
    cp->m8base = m8;
    cp->m1base = m1;
    cp->xv_shdw = (XV_shdw_regs*)sp;
    cp->xv_wreg = (XV_write_regs*)reg;
    cp->xv_rreg = (XV_read_regs*)reg;
    cp->maps = ap;
        
    /* init the REGISTER FILE shadow ram structure */

    j = sizeof(XV_shdw_regs) >> 2;
    for (ip = (int *)sp, i = 0; i++; i<j) *ip++ = 0;


	/* INTIALIZE THE VALUES IN NVRAM WE USE. */

	cp->main.xsize = 1280;
	cp->main.ysize = 1024;
	

	DEBUGF(0x80, ("plxv_context_init: exiting\n"));
    return cp;
}

/********************************************
        function to re-init pointers
*********************************************/

static void plxv_reinit_context(cp, reg, m1, m8, m24, c3, eprom)
XV_struct *cp;
{
    cp->eprombase = eprom;
    cp->regbase = reg;
    cp->c3base = c3;
    cp->m24base = m24;
    cp->m8base = m8;
    cp->m1base = m1;
}

/********************************************
        function to map the boards address space
*********************************************/
#ifdef HPUX_10

static void *plxv_map_mem(ssp, user)
struct plxv_softc *ssp;
int user;
{
    int board_value;
    struct isc_table_type *isc;
    u_int count=PLXV_MEM_SIZE / 4096;
    int slot_addr;
    void *memaddr;
    isc= ssp->isc;
    slot_addr = ((struct eisa_if_info *)(isc->if_info))->slot;
    slot_addr *= 0x1000;
    slot_addr += EISA_BASE_PHYSADDR;
    ssp->regaddr= slot_addr;

    if(ssp->regaddr == NULL)
    {
	DEBUGF(0x80, ("plxv_map_mem: can't map eisa registers\n"));
	return NULL;
    }
    board_value = *(int*)slot_addr;
    ssp->plxv_physical= 
	((board_value&0xff000000)>>1) | ((board_value&0x00010000)<<15) +
	EISA_BASE_PHYSADDR;
    /* 
     * Map in the board memory to the driver 
     * with User Read Write Protection.
     */
    if(user)
    {
      memaddr= user_iomap(NULL,ssp->plxv_physical,count);
      if(memaddr== NULL)
      {
	printf("MAP_MEM: can't map memory to user\n");
	DEBUGF(0x80, ("plxv_map_mem: can't map memory\n"));
	return NULL;
      }
    }
    else
    {
      memaddr= kernel_iomap(NULL,ssp->plxv_physical,count,PROT_URW);
      if(memaddr== NULL)
      {
	printf("MAP_MEM: can't map memory to kernel\n");
	DEBUGF(0x80, ("plxv_map_mem: can't map memory\n"));
	return NULL;
      }
    }
    return memaddr;
}

#else /* !HPUX_10 */

static int plxv_map_mem(ssp)
struct plxv_softc *ssp;
{
	int                   board_value;	/* Value from eisa reg on board. */
    struct isc_table_type *isc;
    int                   slot_addr;    /* Address of eisa slot registers. */

    isc = ssp->isc;
    slot_addr = ((struct eisa_if_info *)(isc->if_info))->slot;
    slot_addr *= 0x1000;
    slot_addr += EISA_BASE_PHYSADDR;
    ssp->regaddr = slot_addr;

    if(ssp->regaddr == NULL)
    {
        DEBUGF(1, ("plxv_map_mem: can't map eisa registers\n"));
        return -1;
    }

#ifdef NOTDEF
    ssp->plxv_physical = PLXV_PHYSADDR_OFFSET; 
#endif
    board_value = *(int*)slot_addr;
	ssp->plxv_physical = 
		((board_value&0xff000000)>>1) | ((board_value&0x00010000)<<15);
    ssp->memaddr = map_mem_to_host(isc, ssp->plxv_physical,
                                          PLXV_MEM_SIZE);
    if(ssp->memaddr == NULL)
    {
        DEBUGF(1, ("plxv_map_mem: can't map memory\n"));
        return -1;
    }
    return 0;
}

#endif /* HPUX_10 */

#ifdef HPUX_10

/******************************************
   user virtual address handling routines and storage
******************************************/

typedef struct _addrSave
{
    int pid, unit;
    void *virtaddr;
    struct _addrSave *next;
} addrSave;

static addrSave *usrAddrs = NULL;

static void *getAddr(unit)	/* return user virtual address */
int unit;
{
	struct plxv_softc *ssp;
	int thispid;		/* our pid */
	addrSave *t, *p;		/* pointers to struct */
	void *va;			/* mapped virtual address */

	unit &= ~1;
	ssp = &plxv_softc[unit];
	thispid = getpid();		/* get our pid */

	for(p = usrAddrs; p; p = p->next)
	{
		if((p->pid == thispid) && (unit == p->unit)) /* on the list ? */
			return(p->virtaddr);    		/* yep, return the address */
	}

    /* nope, map it in */

	if((va = plxv_map_mem(ssp, 1)) == NULL)
		return(NULL);		/* big problems here */

    /* alloc the struct */
	if ((t = (addrSave *)sys_memall(sizeof(addrSave))) == NULL)
	{
		user_iounmap(va, PLXV_MEM_SIZE / 4096);
		return(NULL);
	}

	/* fill it in */
	t->pid = thispid;
	t->unit = unit;
	t->virtaddr = va;
	t->next = usrAddrs;
	usrAddrs = t;		/* we go to head of the list */
	return va;
}

static void delAddr(unit)
int unit;
{
	int thispid;
	addrSave *last;
	addrSave *p;

    unit &= ~1;
    thispid = getpid();	/* get our pid */
    /* point at head of list */

    for(p = usrAddrs; p; p = p->next)
    {
		if((p->pid == thispid) && (unit == p->unit))	/* find pid */
			break;
		last = p;
    }

    if(p == NULL)			/* not there, split */
		return;

    if(user_iounmap(p->virtaddr, PLXV_MEM_SIZE / 4096)) /* unmap it */
		printf("delAddr: user_iounmap failed\n");

    if(p == usrAddrs)			/* fix up the list */
		usrAddrs = p->next;
    else
		last->next = p->next;

    sys_memfree(p); 			/* free up storage */
}
#endif /* HPUX_10 */

/*********************************************
        function to link in our attach routine
*********************************************/

#ifdef HPUX_10
plxv_install()
#else
plxv_link()
#endif
{
    plxv_saved_attach = eisa_attach;
    eisa_attach = plxv_attach;
#ifdef HPUX_10
    return(wsio_install_driver(&plxv_wsio_info));
#endif
}

/*********************************************
        Attach routine
*********************************************/

static int plxv_attach(id, isc)
	int 					id;
	struct isc_table_type*  isc;
{
    unsigned short*    c3;
    unsigned char*     ep;
    unsigned char*     m1;
    unsigned char*     m8;
    unsigned long*     m24;
    rw_regs* 			 reg;
    struct plxv_softc* softc;
    XV_mem_map*        xvm;
    struct eeprom_slot_data esd;
    struct eeprom_function_data efd;
	int slot, i;

#ifdef HPUX_10
    register int sm;
    static char *swrev = "10.1";
#else /* !HPUX_10 */
    static char *swrev = "9.3";
#endif /* HPUX_10 */



	/* IT'S NOT US, IT'S AN INTERESTING APPROACH, BUT IT'S NOT US. */

    if ( (id & PLXV_ID_MASK) != PLXV_ID )
        return((*plxv_saved_attach)(id, isc));


	/* THE ID OF THIS CARD MATCHES OURS, SO SET UP A SOFTC STRUCTURE. */

    softc = &plxv_softc[n_softcs];
    softc->unit = n_softcs;
    n_softcs+=2;

    slot = ((struct eisa_if_info *)(isc->if_info))->slot;
    slot_to_softc[slot] = softc;

	softc->isc = isc;

    read_eisa_slot_data(slot, &esd);
    softc->intlvl = 0;
    for(i = 0; i < esd.number_of_functions; ++i)
    {
        read_eisa_function_data(slot, i, &efd); 
		if(efd.intr_cfg[0][0] != 0)
	    {
			softc->intlvl = efd.intr_cfg[0][0] & 0xf;
			break;
		}
    }

	if(softc->intlvl == 0)
	{
		printf("interrupt level == 0!!!!!!!!!\n");
        ((struct eisa_if_info *)(isc->if_info))->flags = INIT_ERROR;
        return((*plxv_saved_attach)(id, isc));
	}


    DEBUGF(0x80, ("plxv_attach: attempting to map\n"));

#ifdef HPUX_10
    if((softc->memaddr = plxv_map_mem(softc, 0)) == NULL)
#else
    if(plxv_map_mem(softc))
#endif
    {
        ((struct eisa_if_info *)(isc->if_info))->flags = INIT_ERROR;
        return((*plxv_saved_attach)(id, isc));
    }

#ifdef HPUX_10
    isrlink(isc, plxv_isr, softc->intlvl,isc,0);
    isc_claim(isc,&plxv_wsio_info);
#if 0
    softc->major_num= plxv_info.c_major;
#endif
#else
    eisa_isrlink(isc, plxv_isr, softc->intlvl, softc->intlvl);
#ifndef NOTDEF
    eisa_isrlink(isc, image_isr, PLXV_IMAGE_IRQ, PLXV_IMAGE_IRQ);
#endif
#endif

    isc->gfsw->init = plxv_init;

    ((struct eisa_if_info *)(isc->if_info))->flags = INITIALIZED;

    softc->attached = 1;
	softc->wffield = -1;
	softc->vfield = -1;
    softc->visleep = 0;

    /*  IF THERE IS NO CCUBE ITS INTERRUPT WILL NEVER GO LOW.  */

    xvm = (XV_mem_map *) softc->memaddr;

#ifdef HPUX_10
    _RSM(PSW_P, sm);                    /* allow us access to board */
#endif

    init_loopsperusec(xvm);		/* init the DELAY loop usec mult */

    if ( softc->c3_present = plxv_is_c3_present(&xvm->RW_regs) )
	{
#ifdef HPUX_10
		printf("\n");
#endif
   printf("Parallax Power Video 700 s/w rev %s at address %x, int %d, lpu %d\n",
			   swrev, xvm, softc->intlvl, loopsperusec);
	}
    else
	{
#ifdef HPUX_10
		printf("\n");
#endif
printf("Parallax Multi Video 700 s/w rev %s at address %x, int %d, lpu %d\n",
			   swrev, xvm, softc->intlvl, loopsperusec);
	}

#ifdef HPUX_10
    if(sm & PSW_P)                  /* disallow access */
        _SSM(PSW_P,0);
#endif

    return((*plxv_saved_attach)(id, isc));
}

/*********************************************
        Init routine
*********************************************/

static int plxv_init(isc)
struct isc_table_type *isc;
{
    XV_mem_map *xvm;
    struct plxv_softc *softc;
    struct eeprom_slot_data esd;
    struct eeprom_function_data efd;
    int i, slot;
	int		eisa_reg_value;

#ifdef HPUX_10
    register int sm;
    _RSM(PSW_P, sm);                    /* allow us access to board */
#endif

    softc = slot_to_softc[((struct eisa_if_info *)(isc->if_info))->slot];
    xvm = (XV_mem_map *) softc->memaddr;
    softc->dmactl = (dma_ctl *) &xvm->Prom1;

    slot = ((struct eisa_if_info *)(isc->if_info))->slot;

#if 0
    read_eisa_slot_data(slot, &esd);
    for(i = 0; i < esd.number_of_functions; ++i)
    {
		int j, k;
        read_eisa_function_data(slot, i, &efd); 
	    if(efd.function_info & HAS_TYPE_ENTRY)
			printf("function_type = %s\n", efd.type);
		for(j = 0; j < 7; ++j)
	    {
			printf("intr_cfg %d,0 = %x\n", j, efd.intr_cfg[j][0] & 0xff);
			printf("intr_cfg %d,1 = %x\n", j, efd.intr_cfg[j][1] & 0xff);
	    }
    }
#endif

    DEBUGF(0x80, ("plxv_init: initializing hardware library\n"));

#ifdef NOTDEF
    *(softc->regaddr) = PLXV_INITBITS;
#else
	eisa_reg_value = *(softc->regaddr);
	*(softc->regaddr) = 0;
	*(softc->regaddr) = eisa_reg_value;
#endif

    softc->xv = plxv_context_init(&xvm->RW_regs, &xvm->M1, &xvm->M8,
                          &xvm->M24, &xvm->Ccube, &xvm->Prom0);


	/*  INITIALIZE ENOUGH OF THE HARDWARE TO PUT THE BOARD IN A QUIESCENT STATE:
     *
     *  This is an overlay (mixer) card, so the only functions handled by the
     *  driver have to do with DMA. So we initialize the minimum amount of 
     *  hardware necessary to place the board in a quiescent state.
     */
	PV_WREG(softc->xv, DispKludge.l, 0x1000); 	/* Turn on blanking. */
	PV_WREG(softc->xv, TmrSpd.l, 0x171);	    /* Initialize the timer. */

    PV_WREG(softc->xv,Bt.red.reg0,0x01);	/* Bt495: 5-1 mux, 8-bit mode. */
    PV_WREG(softc->xv,Bt.red.reg1,0x02);
    PV_WREG(softc->xv,Bt.red.reg2,0xc0);

    PV_WREG(softc->xv,Bt.red.reg0,0x03);	/* Bt495: sync and pedestal off. */
    PV_WREG(softc->xv,Bt.red.reg1,0x02);
    PV_WREG(softc->xv,Bt.red.reg2,0x00);

#ifdef HPUX_10
    if(sm & PSW_P)                  /* disallow access */
        _SSM(PSW_P,0);
#endif

    return 0;
}

/*********************************************
        image dma interrupt service routine
*********************************************/

static int image_isr(isc, arg)
struct isc_table_type *isc;
int arg;
{
    struct buf *bp;
    struct plxv_softc *softc;
    DEBUGF(0x40, ("image_isr: arg = %d\n", arg));
    softc = slot_to_softc[((struct eisa_if_info *)(isc->if_info))->slot];
    softc++; /* Get to image version */
    bp = &softc->dmabuf;
    wakeup(bp);
}

/*********************************************
        interrupt service routine
*********************************************/

static int plxv_isr(isc, arg)
struct isc_table_type *isc;
int arg;
{

    struct plxv_softc *softc;
    struct buf *bp;
    
    extern void service_ccube(),service_timer(),service_disp();
    extern void service_ccube(),service_ext(),service_video();
    extern void service_video(),service_video(),service_video();
    extern void service_blit();
    extern void service_dmabuf();

    static void (*service_routine[10])() = {
        service_dmabuf,service_timer,service_disp,service_ccube,service_disp/*ext*/,
        service_video,service_video,service_video,service_video,service_blit};

    unsigned int c3_rcount;
    int pri, bits,check_bit, savebits;
    int i,j,k,l,n;
    int rM,rBM;

#ifdef HPUX_10
    register int sm;
    _RSM(PSW_P, sm);                        /* allow us access to board */
#endif

  savebits = 0;

#if 1
    /* get the buffer stuff from softC */
    softc = slot_to_softc[((struct eisa_if_info *)(isc->if_info))->slot];
#else
  for(n = 0; n < n_softcs; n += 2) /* look through the boards one at a time */
#endif
  {
#if 0
    softc = &plxv_softc[n];
#endif
    bp = &softc->dmabuf;

    PV_RREG(softc->xv, softc->BIntStat,BIntStatC.l);
    PV_RREG(softc->xv, softc->IntStat,IntStatC.l);

    /* read the Interrupt Status registers & Clear */
    PV_RSDW(softc->xv, rBM,BIntMask.l);
    PV_RSDW(softc->xv, rM,IntMask.l);

    bits = ((softc->IntStat & rM & 0xff) <<2 ) | ( softc->BIntStat & rBM & 0x3);
    savebits |= bits;

    /* sometimes the board doesn't respond during dma, so shut it down */
    if((bits & 0x9) && (softc->dmastate > 0) && (softc->c3_present))
    {
        if(softc->dmastate & STATEW) {
                PV_WREG(softc->xv, CCube.l, XVC3_PAUSE_DMA_FROM_HOST);
                DELAY(DELAY100);
                PV_WREG(softc->xv, CCube.l, XVC3_WRITE_GO_PIO);
                PV_WREG(softc->xv, CCube.l, XVC3_WRITE_GO_PIO);
        } else {
                PV_WREG(softc->xv, CCube.l, XVC3_PAUSE_DMA_TO_HOST);
                DELAY(DELAY500);
                PV_WREG(softc->xv, CCube.l, XVC3_PURGE_BIC_GO_PIO);
                PV_WREG(softc->xv, CCube.l, XVC3_PURGE_BIC_GO_PIO);
        }
        DELAY(DELAY100);
    }

    check_bit = 1;

    stat_int++;
    if ( !bits ) stat_no_bits++;

    DEBUGF(0x40, ("plxv_isr: slot = %d softc = %x bits = %x\n", 
		((struct eisa_if_info *)(isc->if_info))->slot, softc, bits));

    /* look thru the bits one at a time */
    for (i=0;i<10;i++)
    {   
        if ( bits == 0 ) break; /* no more bits */
        if ( bits & check_bit)
        {
                (*service_routine[i])(softc, &bits);
        }
        check_bit <<= 1;
    }


    /* let the dma continue if not restarted by ccube routines */

#if 0
    if((softc->dmastate > 0) && ((savebits & 0x9) == 0))
    {
        if(softc->dmastate & STATEW)
        {
                PV_WREG(softc->xv, CCube.l, XVC3_CONT_DMA_FROM_HOST_1);
                PV_WREG(softc->xv, CCube.l, XVC3_CONT_DMA_FROM_HOST_2);
        }
        else
        {
                PV_WREG(softc->xv, CCube.l, XVC3_CONT_DMA_TO_HOST_1);
                PV_WREG(softc->xv, CCube.l, XVC3_CONT_DMA_TO_HOST_2);
        }
    }
#endif

#ifdef HPUX_10
    if(sm & PSW_P)
        _SSM(PSW_P,0);  /* disallow access */
#endif

  }

    return(savebits ? 1: 0);
}


/*********************************************
    timeout routine for image dma
*********************************************/

void service_image_timeout(arg)
int arg;
{
    struct plxv_softc *softc;
    struct buf *bp;
    DEBUGF(0x40, ("service_image_timeout: arg = %d\n", arg));
    softc = &plxv_softc[arg];
    bp = &softc->dmabuf;
    wakeup(bp);
}


/*********************************************
        routine to wake up strategy below if
        dma resources not available
*********************************************/

plxv_strat_wakeup(parms)
struct dma_parms *parms;
{
    wakeup(parms);
}

#define WHY
#ifdef WHY
disable_eisa(softc)
struct plxv_softc *softc;
{
    softc->dmactl->control = 0;/* disable eisa dma */
}

#endif

/*********************************************
        Strategy routine
*********************************************/

static int plxv_strategy(bp)
struct buf *bp;
{
    int temp, i;
    struct dma_parms dmap;
    struct plxv_softc *softc, *osoftc;
    int pri, op, ret, resid;
    unsigned int imask, tmask, ttmask;
    unsigned long dma_bits, other_dma_bits;
    u_char unit;
    int chan;

#ifdef HPUX_10
    register int sm;
#endif

    
    unit = minor(bp->b_dev);
    softc = &plxv_softc[unit];

    switch(softc->intlvl)
	{
		case 5:
			chan = 0;
			break;
		case 9:
			chan = 5;
			break;
		case 10:
			chan = 6;
			break;
		case 15:
			chan = 7;
			break;
	}
    other_dma_bits = plxv_softc[unit ^ 1].dmactlbits;

    if(bp->b_bcount == 0)
    {
        iodone(bp);
        return 0;
    }

#ifdef HPUX_10
    _RSM(PSW_P, sm);                        /* allow us access to board */
#endif

#if 1
	if (!(unit & 1) && bp->b_flags & B_READ)
	{
		int thisfield, myfield, t;

		/* do we need to wait for a field? */

		if(softc->wffield != -1)
		{
			if(softc->vfield != -1)	/* vfield takes presidence */
				myfield = softc->vfield << 8;
			else
				myfield = (softc->wffield & 1) << 8;

#if 0
			PV_RREG(softc->xv, thisfield, IntStat.l); /* what field's this? */
			thisfield &= 0x100;

			if(thisfield == myfield); /* poss. wait for an int */
			{
#else
			do
			{
#endif
				timeout(wffield_timeout, unit, (HZ/30) + 1, NULL);
				softc->visleep = 1;
				PV_RSDW(softc->xv, t, IntMask.l);
				if((t & 0x8) == 0)
				{
					t |= 0x8; /* set int on */
					PV_WREG(softc->xv, IntMask.l, t);
					PV_RREG(softc->xv, t, IntStatC.l); /* clear any ints */
				}
#ifdef HPUX_10
				if(sm & PSW_P)
					_SSM(PSW_P, 0); /* disallow access during sleep */
#endif
				splx(pri);
				sleep(&softc->viwake);	/* wait for interrupt */
				pri = spl6();
				untimeout(wffield_timeout, unit);
				softc->visleep = 0;
#ifdef HPUX_10
				_RSM(PSW_P, sm);                 /* allow us access to board */
#endif

#if 0
	    	}
#else
				PV_RREG(softc->xv, thisfield, IntStat.l);
				thisfield &= 0x100;
			} while(myfield == thisfield);
#endif
			if((softc->wffield & 2) == 0)
				softc->wffield = -1;
		}
    }
#endif

#ifndef LEAVE_INTS_ON

	for(i = 0; i < n_softcs; i += 2)	/* shut off ints on other boards */
	{
		int j;
		osoftc = &plxv_softc[i];
		if(osoftc != softc)
		{
			PV_WREG(osoftc->xv, IntMask.l, 0);
			PV_WREG(osoftc->xv, BIntMask.l, 0);
			PV_RREG(osoftc->xv, j, IntStatC.l); /* clear any ints */
			PV_RREG(osoftc->xv, j, BIntStatC.l);
			disable_eisa(osoftc);		/* no dma */
		}
	}

#endif

    /* #### do_coef(softc->xv); */
    if(bp->b_flags & B_READ)
    {
    	DEBUGF(0x80, ("plxv_strategy: READ \n"));
        op = DMA_READ;
        if(unit & 1)
            dma_bits = PLXV_IMAGE_READ;
        else
            dma_bits = PLXV_JPEG_READ;
    }
    else
    {
    	DEBUGF(0x80, ("plxv_strategy: WRITE \n"));
        op = DMA_WRITE;
        if(unit & 1)
            dma_bits = PLXV_IMAGE_WRITE;
        else
            dma_bits = PLXV_JPEG_WRITE;
    }

    softc->dmactlbits = dma_bits;

	DEBUGF(0x80, ("plxv_strategy: Before setting DMA bits"));
    if(((unit & 1) == 0) && bp->b_flags & B_READ)
    {
        softc->dmactl->control = other_dma_bits;/* disable eisa dma */
        softc->dmactl->control = dma_bits | other_dma_bits;/* enable dma */
        purge_fifo(softc->xv);
        softc->dmactl->control = other_dma_bits;/* disable eisa dma */
        DELAY(DELAY200);
        softc->dmactl->control = dma_bits | other_dma_bits;/* enable dma */
        DELAY(DELAY200);
        softc->dmactl->control = other_dma_bits;    /* disable eisa dma */
        DELAY(DELAY200);
    }
    bp->b_s2 = &dmap;

	DEBUGF(0x80, ("plxv_strategy: Before setting DMA channel"));
    
    if(unit & 1)
    {
        dmap.channel = PLXV_IMAGE_CHANNEL;
        dmap.dma_options = DMA_BURST | DMA_32BYTE | DMA_DEMAND | op;
    }
    else
    {
        dmap.channel = chan;
        dmap.dma_options = DMA_TYPEB | DMA_DEMAND | DMA_32BYTE | op;
    }
    dmap.flags = 0;
    dmap.key = 0;
    dmap.num_entries = 0;
    dmap.buflet_key = NULL;
    dmap.chain_ptr = NULL;
    dmap.chain_count = 0;
    dmap.chain_index = 0;
    dmap.drv_routine = plxv_strat_wakeup;
    dmap.drv_arg = (int) &dmap;
    dmap.transfer_size = 0;
    dmap.addr = bp->b_un.b_addr;
    dmap.spaddr = bp->b_spaddr;
    dmap.count = bp->b_bcount;

	DEBUGF(0x80, ("plxv_strategy: Before eisa_dma_setup\n"));
    while(ret = eisa_dma_setup(softc->isc, &dmap))
    {
        if(ret == RESOURCE_UNAVAILABLE)
        {
            DEBUGF(1, ("plxv_strategy: dma setup returns unavailable, sleeping\n"));
            sleep(&dmap, PZERO+2);
        }
        else if(ret < 0)
        {
            bp->b_error = EINVAL;
            bp->b_flags = B_ERROR;
            iodone(bp);

#ifdef HPUX_10
	    if(sm & PSW_P)
		_SSM(PSW_P, 0); /* disallow access */
#endif

            DEBUGF(1, ("plxv_strategy: dma setup returned error %d\n", ret));
            return(-1);
        }
        else
        {
            DEBUGF(1, ("plxv_strategy: dma setup returned %d\n", ret));
            break;
        }
    }

	DEBUGF(0x80, ("plxv_strategy: After eisa_dma_setup\n"));
    softc->dmactl->control = other_dma_bits;/* disable eisa dma */
    softc->dmactl->control = other_dma_bits | dma_bits; /* enable eisa dma */

    pri = spl6();

#if 0
	if (!(unit & 1) && bp->b_flags & B_READ)
	{
		int thisfield, myfield, t;

		/* do we need to wait for a field? */

		if(softc->wffield != -1)
		{
			if(softc->vfield != -1)	/* vfield takes presidence */
				myfield = softc->vfield << 8;
			else
				myfield = (softc->wffield & 1) << 8;

#if 0
			PV_RREG(softc->xv, thisfield, IntStat.l); /* what field's this? */
			thisfield &= 0x100;

			if(thisfield == myfield); /* poss. wait for an int */
			{
#else
			do
			{
#endif
				timeout(wffield_timeout, unit, (HZ/30) + 1, NULL);
				softc->visleep = 1;
				PV_RSDW(softc->xv, t, IntMask.l);
				if((t & 0x8) == 0)
				{
					t |= 0x8; /* set int on */
					PV_WREG(softc->xv, IntMask.l, t);
					PV_RREG(softc->xv, t, IntStatC.l); /* clear any ints */
				}
#ifdef HPUX_10
				if(sm & PSW_P)
					_SSM(PSW_P, 0); /* disallow access during sleep */
#endif
				splx(pri);
				sleep(&softc->viwake);	/* wait for interrupt */
				pri = spl6();
				untimeout(wffield_timeout, unit);
				softc->visleep = 0;
#ifdef HPUX_10
				_RSM(PSW_P, sm);                 /* allow us access to board */
#endif

#if 0
	    	}
#else
				PV_RREG(softc->xv, thisfield, IntStat.l);
				thisfield &= 0x100;
			} while(myfield == thisfield);
#endif
			if((softc->wffield & 2) == 0)
				softc->wffield = -1;
		}
    }
#endif

	/* save int mask, and starting field  state */

	PV_RSDW(softc->xv, imask, IntMask.l);
    PV_RREG(softc->xv, softc->startfield, IntStat.l)
	softc->startfield = (softc->startfield & 0x100) >> 8;

#if 0
    softc->ladd = bp->b_un.b_addr;
#else
    softc->ladd = 0;
#endif

    if(unit & 1)
    {
        timeout(service_image_timeout, unit, 500, NULL);
    }
    else
    {
      if (bp->b_flags & B_READ)
             softc->dmastate = c3_start_read(softc->ladd, bp->b_bcount, unit);
      else
             softc->dmastate = c3_start_write(softc->ladd, bp->b_bcount, unit);
    }

	DEBUGF(0x80, ("plxv_strategy: Before snoozing\n"));

#ifdef HPUX_10
    if(sm & PSW_P)
        _SSM(PSW_P, 0); /* disallow access during sleep */
#endif

    if(unit & 1)
        snooze(bp->b_bcount/4);
    else
        sleep(bp, PRIBIO);

#ifdef HPUX_10
    _RSM(PSW_P, sm);                    /* allow us access to board */
#endif

    if(unit & 1)
    {
        untimeout(service_image_timeout, unit);
    }
    else
    {
      DEBUGF(0x80, ("plxv_strategy: after sleep, dmastate = %d\n", softc->dmastate));
    }

    softc->dmactl->control = other_dma_bits;    /* disable eisa dma */

    softc->dmactlbits = 0;

    resid = eisa_dma_cleanup(softc->isc, &dmap);

    DEBUGF(0x80, ("plxv_strategy: after dma cleanup, resid = %d\n", resid));

    if(unit & 1)
    {
       bp->b_resid = resid;
    }
    else
    {
      if(resid != bp->b_resid && bp->b_flags & B_WRITE)
        DEBUGF(1, ("Warning: cleanup resid = %d, bp.b_resid = %d\n",
               resid, bp->b_resid));
    }

    iodone(bp);

    DEBUGF(0x80, ("plxv_strategy: after iodone\n"));

    toggle_eisa_enable(unit);

	/* turn ints back on */

    PV_RSDW(softc->xv, tmask, IntMask.l);
    tmask |= (imask & 9);

    PV_WREG(softc->xv, IntMask.l, tmask);
    PV_RREG(softc->xv, ttmask, IntStatC.l);
    if(softc->vfield != -1)
    {
/*      PV_RREG(softc->xv, t, IntStat.l); /* what field's this? */
        ttmask = (ttmask & 0x100) >> 8;
        if((ttmask != softc->startfield) && (ttmask != softc->vfield))
            PV_WREG4(softc->xv, VCtl.l, 2);
    }


#ifdef HPUX_10
    if(sm & PSW_P)
        _SSM(PSW_P, 0);
#endif

    splx(pri);

    return 0;
}

toggle_eisa_enable(unit)
int unit;
{
    struct plxv_softc *softc;
    unsigned long dma_bits, other_dma_bits;

    softc = &plxv_softc[unit];
    other_dma_bits = softc->dmactlbits;
    dma_bits = PLXV_JPEG_READ;
    softc->dmactlbits = dma_bits;

    softc->dmactl->control = other_dma_bits;/* disable eisa dma */
    softc->dmactl->control = other_dma_bits | dma_bits; /* enable eisa dma */

    softc->dmactl->control = other_dma_bits;    /* disable eisa dma */
    softc->dmactlbits = 0;
}


/*********************************************
        Open
*********************************************/

int plxv_open(dev, flags)
dev_t dev;
int flags;
{
    int pri;
    u_char unit;
    XV_mem_map *xvm;
    struct plxv_softc *softc;

#ifdef HPUX_10
    register int sm;
#endif

    DEBUGF(0x80, ("plxv_open: called\n"));

    unit = minor(dev);
    softc = &plxv_softc[unit & ~1];
    if ( softc->attached != 1 )         /* Attach failed, open should too. */
    {
        return(ENODEV);
    }

    if(softc->memaddr == NULL)
    {
      softc->dmactlbits = 0x10000000;
#ifdef HPUX_10
      if((softc->memaddr = plxv_map_mem(softc, 0)) == NULL)
#else
      if(plxv_map_mem(softc))
#endif
      {
        DEBUGF(1, ("plxv_open: can't map memory\n"));
        return(ENOMEM);
      }
      xvm = (XV_mem_map *) softc->memaddr;
      plxv_reinit_context(softc->xv, &xvm->RW_regs, &xvm->M1, &xvm->M8,
                          &xvm->M24, &xvm->Ccube, &xvm->Prom0);

      bcopy(softc, &plxv_softc[unit | 1], sizeof(struct plxv_softc));
    }
    softc = &plxv_softc[unit];

#ifdef HPUX_10
    _RSM(PSW_P, sm);                    /* allow us access to board */
#endif

    if((unit & 1) == 0)
	{
			if (softc->c3_present)
			{
				plxv_initc3(softc->xv);
				softc->xv->main.c3_rev = 2;
			}
			else
			{
				softc->xv->main.c3_rev = 0;
			}
	}

#ifdef HPUX_10
    if(sm & PSW_P)
        _SSM(PSW_P, 0);
#endif

    return 0;
}

/*********************************************
        Close
*********************************************/

int plxv_close(dev)
dev_t dev;
{
    int i, nu;

#ifdef HPUX_10
   delAddr(minor(dev));   /* uniomap user space */
#endif

    DEBUGF(0x80, ("plxv_close: called\n"));
    return 0;
}

/*********************************************
        Read
*********************************************/

int plxv_read(dev, uio)
dev_t dev;
struct uio *uio;
{
    unsigned char unit_no;
    unsigned short retval = 0;
    struct plxv_softc *softc;
    struct buf *bp;

    DEBUGF(0x80, ("\nplxv_read: called\n"));
    unit_no = minor(dev);
    softc = &plxv_softc[unit_no];
    bp = &softc->dmabuf;        
    if((unit_no & 1) == 0)
    {
      if(softc->xv->main.c3_rev < 1)
      {
        DEBUGF(1, ("plxv_read: no ccube on board\n"));
        return(EIO);
      }
    }

    retval = physio(plxv_strategy, &softc->dmabuf, dev, B_READ, minphys, uio);

#if 0
/* if read and no error, stuff pointer to double word at end of buffer */

    if(bp->b_bcount != bp->b_resid)
    {
	int x = copyout(&softc->state_r1_count, bp->b_un.b_addr, sizeof(int));
	if(x)
	{
	    printf("copyout returned error %d, addr %x, count %d, resid %d\n",
		   x, bp->b_un.b_addr, bp->b_bcount, bp->b_resid);
	    printf("uio base, len, resid = %x, %d, %d\n",
		   uio->uio_iov->iov_base, uio->uio_iov->iov_len,
		   uio->uio_resid);
	}
	softc->state_r1_count = -1;
    }
    else if((bp->b_flags & B_ERROR) == 0)
    {
	DEBUGF(1, ("read: no error with count of 0, loc of double = %d\n",
		softc->state_r1_count));
    }
#endif
    return(retval);
}

/*********************************************
        Write
*********************************************/

int plxv_write(dev, uio)
dev_t dev;
struct uio *uio;
{
    unsigned char unit_no;
    unsigned short retval = 0;
    struct plxv_softc *softc;

    DEBUGF(0x80, ("\nplxv_write: called\n"));
    unit_no = minor(dev);
    softc = &plxv_softc[unit_no];

    if((unit_no & 1) == 0)
    {
      if(softc->xv->main.c3_rev < 1)
      {
        DEBUGF(1, ("plxv_write: no ccube on board\n"));
        return(ENODEV);
      }

    }

    retval = physio(plxv_strategy, &softc->dmabuf, dev, B_WRITE, minphys, uio);
    return(retval);
}

/*********************************************
        Ioctl
*********************************************/

int plxv_ioctl(dev, function, arg, flag)
dev_t dev;
int function, *arg, flag;
{
	extern int timeout_interval;
	int ret, i, unit, thisfield, myfield;
	crt_frame_buffer_t *cfb;
	struct plxv_softc *softc;

#ifdef HPUX_10
    register int sm;
#endif

    unit = minor(dev);
    softc = &plxv_softc[unit];

    switch(function)
    {
		case GCCOPROC:


#ifdef HPUX_10
			_RSM(PSW_P, sm);
#endif

#if 0
		/* flash which field: 0 = even, 1 = odd, else off */
			softc->vfield = *(int *)arg;
			if((softc->vfield < 0) || (softc->vfield > 1))
				softc->vfield = -1;

			PV_RSDW(softc->xv, i, IntMask.l);

			if(softc->vfield == -1)
			    i &= ~0x8;
			else
			    i |= 0x8;

			PV_WREG(softc->xv, IntMask.l, i);
#else
			if(((myfield = *(int *)arg) < 0) || (myfield > 1))
				return EINVAL;

			myfield <<= 8;
			do
			{
				int t, pri;
				timeout(wffield_timeout, unit, (HZ/30) + 1, NULL);
				softc->visleep = 1;
				PV_RSDW(softc->xv, t, IntMask.l);
				if((t & 0x8) == 0)
				{
					t |= 0x8; /* set int on */
					PV_WREG(softc->xv, IntMask.l, t);
					PV_RREG(softc->xv, t, IntStatC.l); /* clear any ints */
				}
#ifdef HPUX_10
				if(sm & PSW_P)
					_SSM(PSW_P, 0); /* disallow access during sleep */
#endif
				splx(pri);
				sleep(&softc->viwake);	/* wait for interrupt */
				pri = spl6();
				untimeout(wffield_timeout, unit);
				softc->visleep = 0;
#ifdef HPUX_10
				_RSM(PSW_P, sm);                 /* allow us access to board */
#endif

				PV_RREG(softc->xv, thisfield, IntStat.l);
				thisfield &= 0x100;
			} while(myfield == thisfield);

#ifdef HPUX_10
			if(sm & PSW_P)
				_SSM(PSW_P, 0);
#endif
#endif
		    break;
		case GCTERM:		/* syncronize next getcimage with field */

			softc->wffield = *(int *)arg;	/* -1 = off, 0 = even one shot,
												1 = odd one shot,
												2 = even cont., 3 = odd cont,
												else error and off */

			if((softc->wffield < -1) || (softc->wffield > 3))
			{
				softc->wffield = -1;
				return EINVAL;
			}
		    break;
        case GCSLOT:
			*arg = softc->state_r1_count;
			break;
        case GCDESCRIBE:
            cfb = (crt_frame_buffer_t *) arg;
            cfb->crt_id = PLXV_ID;
            cfb->crt_map_size = PLXV_MEM_SIZE;
            cfb->crt_x = softc->xv->main.xsize;
            cfb->crt_y = softc->xv->main.ysize;
            cfb->crt_total_x = cfb->crt_x;
            cfb->crt_total_y = cfb->crt_y;
            cfb->crt_x_pitch = cfb->crt_x * 4;
            cfb->crt_bits_per_pixel = 32;
            cfb->crt_bits_used = 24;
            cfb->crt_planes = 24;
            cfb->crt_plane_size = cfb->crt_x * cfb->crt_y * 4;
            bcopy("Parallax Xvideo\0", &cfb->crt_name, 16);
            cfb->crt_attributes = 0;
            cfb->crt_frame_base = 0;
            cfb->crt_control_base = 0;
            cfb->crt_region[0] = 0;

#ifdef HPUX_10  /* */
            cfb->crt_frame_base = getAddr(unit & ~1);
#else
            cfb->crt_frame_base = softc->memaddr;
#endif
            cfb->crt_control_base =
                    &((XV_mem_map *)cfb->crt_frame_base)->RW_regs;
            break;
        case GCMAP:
            break;
        case GCUNMAP:
            break;
        case GCTEST:
#ifdef HPUX_10
                _RSM(PSW_P, sm);
#endif
			DEBUGF(0x42, ("ioctl softc = %x arg = %d\n", softc, *arg));
                switch ( *arg )
                {
                  default:
				  case 1:
#if 1
                        stat_no_bits = 0;
                        stat_int = 0;
                        stat_disp = 0;
                        PV_WREG(softc->xv, TmrSpd.l, 0x171);
                        PV_RSDW(softc->xv, i,IntMask.l);
                        i |= 1;
                        PV_WREG(softc->xv, IntMask.l, i);
#endif
                        PV_RREG(softc->xv, softc->IntStat,IntStatC.l); /* Clear register. */
                        break;
                  case 0:
                        PV_RSDW(softc->xv, i,IntMask.l);
                        i &= ~1;
                        PV_WREG(softc->xv, IntMask.l, i);
                        break;
                  case 2:
                        PV_RREG(softc->xv, i, IntStat.l);
                        printf("IntStat = %x\n", i);
                        PV_RSDW(softc->xv, i, IntMask.l);
                        printf("IntMask = %x\n", i);
                        printf("Int: %d Disp: %d Nobits: %d\n", 
                                stat_int, stat_disp, stat_no_bits);
                        break;
                  case 3:
                        PV_RREG(softc->xv, i, IntStatC.l);
                        break;
                  case 4:
                        PV_RREG(softc->xv, i, IntStatC.l);
                        i = 4;
                        service_disp(softc, &i);
                        break;
                  case 5:
                        print_q();
                        break;
		  case 768:
			softc->xv->main.xsize = 1024;
			softc->xv->main.ysize = 768;
			break;
                  case 1024:
			softc->xv->main.xsize = 1280;
			softc->xv->main.ysize = 1024;
			break;
                }
#ifdef HPUX_10
                if(sm & PSW_P)
                    _SSM(PSW_P, 0);
#endif
                break;
        default:
                timeout_interval = *arg;
            return 0;
            break;
    }
	return 0;
}

int plxv_select(dev)
dev_t dev;
{
}

void service_disp(softc, b) /* display vertical interrupt */
    struct plxv_softc* softc;
    int*               b;
{
    int tmp;
    *b &= ~0x4;

        stat_disp++;

#ifdef hpux
        plxv_pll_alignment(softc);
#else /* !hpux */
    PV_RSDW(softc->xv, tmp,IntMask.l);
    tmp &= ~1;
    PV_WREG(softc->xv, IntMask.l, tmp); /* stop this int */
#endif /* hpux */

    DEBUGF(0x40, ("display vertical interrupt\n"));

}

void service_dmabuf(softc, b) 
    struct plxv_softc *softc;
    int *b;
{
    struct buf *bp;
    unsigned int c3_rcount;
    int pri;
        int i;

    *b &= ~0x1;

    /* get the buffer stuff from softC */

    bp = &softc->dmabuf;

    PV_RREG(softc->xv, i,Bt.blu.reg0);
    DEBUGF(0x80, ("service Dmabuf interrupt\n"));


    if ((!softc) || (!bp)) return;

    if((softc->dmastate == 0) || (softc->dmastate == -1))
        return;

    if(softc->dmastate == STATER2)	/* sometimes we get buffer done */
    {					/* with fifo done. if so ignore */
					/* the buffer done int */
	int t;
	C3_RREG(softc->xv, t, Flags);
	if(t & _Int_FIFO_Empty)
	{
	    service_ccube(softc, b);
	    return;
	}
    }

    pri = spl6();
    softc->dmastate = xvbuf_int_svc(softc->dmastate,&c3_rcount, softc->unit);
    splx(pri);

    if (softc->dmastate == -1) {
        bp->b_resid = bp->b_bcount;     /* its an error */
        bp->b_flags |= B_ERROR;
        bp->b_error = EIO;
        wakeup(bp);
    }

    else if (softc->dmastate == 0)
    {
        PV_RREG(softc->xv, c3_rcount, DMARB.l);
        bp->b_resid = bp->b_bcount - (c3_rcount - softc->ladd);
        wakeup(bp);
    }
    else if (*b & 0x8) service_ccube(softc, b);
}

void service_ccube(softc, b)
    struct plxv_softc *softc;
int *b;
{/* CCbus interrupt */
    struct buf *bp;
    unsigned int c3_rcount;
    int pri;

/*
     DEBUGF(1, ("C-cube: bits = %x, state = %x\n",*b,softc->dmabuf));
 */

    *b &= ~0x8;

    DEBUGF(0x80, ("service CCube interrupt\n"));

    /* get the buffer stuff from softC, make sure there is a ccube */

    if ( !softc->c3_present ) return;
    bp = &softc->dmabuf;

    if ((!softc) || (!bp)) return;

    pri = spl6();
    softc->dmastate = c3_int_svc(softc->dmastate,&c3_rcount, softc->unit);
    splx(pri);

    if (softc->dmastate == -1) {
        bp->b_resid = bp->b_bcount;     /* its an error */
        bp->b_flags |= B_ERROR;
        bp->b_error = EIO;
        wakeup(bp);
    }

    if (softc->dmastate == 0)
    {
#if 0
        PV_RREG(softc->xv, c3_rcount, DMARB.l);
        bp->b_resid = bp->b_bcount - (c3_rcount - softc->ladd);
#else
        bp->b_resid = bp->b_bcount - c3_rcount;
#endif

        wakeup(bp);
    }
}

void service_timeout(arg)
int arg;
{
    struct plxv_softc *softc;
    struct buf *bp;

#ifdef DBG_TIMEOUT
    DEBUGF(1, ("service_timeout: arg = %d\n", arg));
#endif

    /* get the buffer stuff from softC */
    softc = &plxv_softc[arg];
    bp = &softc->dmabuf;

    bp->b_resid = bp->b_bcount; /* its an error */
    bp->b_flags |= B_ERROR;
    bp->b_error = EIO;
    DEBUGF(0x80, ("service_timeout: doing wakeup\n"));
    wakeup(bp);
}

void service_ext(softc, b)
struct plxv_softc* softc;
int *b;
{/* external interrupt */
    *b &= ~0x10;

    DEBUGF(0x40, ("external interrupt\n"));
}

void service_video(softc, b)
struct plxv_softc* softc;
int *b;
{/* Video interrupt */
	int n, im;

	if(*b & 0x20)
	{
#if 0
		if(softc->vfield == -1)     /* shut int off */
		{
			PV_RSDW(softc->xv, im, IntMask.l);
			im &= ~8;
			PV_WREG(softc->xv, IntMask.l, im);
		}
		else
#else
		if(softc->vfield != -1)
#endif
		{
			PV_RREG(softc->xv, n, IntStat.l);
			n = (n & 0x100) >> 8;
			if(softc->vfield != n)
			{
				PV_WREG4(softc->xv, VCtl.l, 2);
			}
		}
		if(softc->visleep)
			wakeup(&softc->viwake); /* wake up strategy */
	}

	*b &= ~0x20;
	*b &= ~0x40;
	*b &= ~0x80;
	*b &= ~0x100;

	DEBUGF(0x40, ("Video interrupt\n"));
}

void service_blit(softc, b)
struct plxv_softc* softc;
int *b;
{/* Blit done  interrupt */
        *b &= ~0x200;
        DEBUGF(0x40, ("Blit done  interrupt\n"));
}

void service_timer(softc, b)                    /* Timer  interrupt */
struct plxv_softc* softc;
int *b;
{
        *b &= ~0x2;
        DEBUGF(0x40, ("Timer interrupt\n"));
/*
        plxv_set_hourint();
        plxv_inc_hours();
*/
}

/*---------------------------------------------------------------------------*
 |"plxv_pll_alignment"
 |
 |  Check that the Power Video 700 frame buffer is properly synchronized
 |
 |    = 1 if pll in alignment, 0 if adjusting alignment
 *---------------------------------------------------------------------------*/
int plxv_pll_alignment( softc )
	SOFTC*	softc;		/* Context for this device instance. */
{
	int         plx_v;      /* Video synchronization information. */
    XV_struct*  p_xv;		/* Fast access to softc->xv. */
	int         vlines;     /* Vertical lines on screen. */

	p_xv = softc->xv;
	plx_v = p_xv->xv_wreg->Bt.blu.reg0; /* plx_v = READB(0x0);      */
	vlines = p_xv->main.ysize - 1;

	/*
	 *  OUR FRAME BUFFER IS WAY OUT OF SYNCHRONIZATION WITH HOST
	 */
	if (0==(plx_v&0x80)) 
	{
			PV_WREG(p_xv,VDisp.l, vlines-64);
			softc->pll_minor_adjust=0;
			softc->pll_good_count=0;
	} 

	/*
	 *  OUR FRAME BUFFER IS MORE THAN 32 LINES OUT OF SYNCHRONIZATION WITH HOST
	 */
	else if (0==(plx_v&0x40)) 
	{
			PV_WREG(p_xv,VDisp.l, vlines-8);
			softc->pll_good_count=0;
			softc->pll_minor_adjust=0;
	} 

	/*
	 *  OUR FRAME BUFFER IS LESS THAN 32 LINES OUT OF SYNCHRONIZATION WITH HOST
	 */
	else if (((plx_v)&0xf)>=2) 
	{
			if (softc->pll_minor_adjust&1)     
			{
					PV_WREG(p_xv,VDisp.l, vlines-1);
			}
			else 
			{
					PV_WREG(p_xv,VDisp.l, vlines);
			}
			softc->pll_minor_adjust++;

	} 

	/*
	 *  OUR FRAME BUFFER IS IN SYNCHRONIZATION WITH HOST FRAME BUFFER
	 */
	else /* if ( softc->pll_good_count < 10 ) #### */
	{
			PV_WREG(p_xv,VDisp.l, vlines);
			softc->pll_minor_adjust=0;
			softc->pll_good_count++;
	}

	return (softc->pll_good_count >= 7) ? 1 : 0;
}
