/*
 *  bt261_lut.c: Support for Brooktree 261 Lookup Table Object.
 *
 *   The Bt261 is a sync lock generator with PLL pump output.
 *
 * (c) Copyright 1995 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log:   /plxhp/dave/pvcsrc/hwlib/archives/bt261_lut.c_v  $
 * 
 *    Rev 1.1   09 Sep 1996 10:08:18   dave
 * Initial revision
 * Revision 1.1  1995/12/11 20:42:01  dist
 * Initial revision
 *
 */
#include <stdio.h>			/* Pickup stdout and friends. */
#include <sys/types.h>		/* Pick up u_int and friends. */
#include "objects_sun.h"	/* Pick up object definitions. */
#include "xvdebug.h"		/* Pick up XV debugging macros. */

/*  INTERNAL OBJECT DEFINITION FOR BROOKTREE 261 OBJECT: */

typedef struct _O_Lut_Bt261 {
	TV_Lut*		atv;		/* Methods: first entry in all derived objects. */
	XV_struct*	p_xv;		/* Parallax hardware library context structure. */
} O_Lut_Bt261;



/* FORWARD FUNCTION REFERENCES: */

static void i_Lut_Bt261_load(/* p_me, addr,data */);
static void i_Lut_Bt261_load_short(/* p_me, addr,data */);
static void i_Lut_Bt261_set_addr(/* p_me, addr */);
static void i_Lut_Bt261_set_option_rsaddr(/* p_me, device_code,rsaddr */);
static int  m_Lut_Bt261_control(/* p_me, op, arg */);
static void m_Lut_Bt261_dump(/* p_me, fd */);
static int  m_Lut_Bt261_get_entries(/* p_me, start, count, p_value */);
static int  m_Lut_Bt261_init_hw(/* p_me */);
static int  m_Lut_Bt261_set_entries(/* p_me, start, count, p_value */);
static void m_Lut_Bt261_terminate(/* p_me */);


static void i_Lut_Bt261_load(p_me, addr,data)
	O_Lut_Bt261* p_me;	/* -> context for this object. */
	int 		 addr;	/* =  address to be loaded. */
	int			 data;	/* =  data value to be loaded. */
{
    i_Lut_Bt261_set_option_rsaddr(p_me, 1, 0); /* Set bt261 address reg. */
    PV_WREG(p_me->p_xv, Ext.reg1, addr);
    i_Lut_Bt261_set_option_rsaddr(p_me, 1, 1); /* Select bt261 data. */
    PV_WREG(p_me->p_xv, Ext.reg1, data);
}


static void i_Lut_Bt261_load_short(p_me, addr,data)
	O_Lut_Bt261* p_me;	/* -> context for this object. */
	int 		 addr;	/* =  address to be loaded. */
	int			 data;	/* =  data value to be loaded. */
{
	i_Lut_Bt261_load(p_me, addr, data & 0xff);
	i_Lut_Bt261_load(p_me, addr, (data >> 8) & 0xff);
}


static void i_Lut_Bt261_set_option_rsaddr(p_me, device_code,rsaddr)
	O_Lut_Bt261* p_me;			/* -> context for this object. */
	int			 device_code;	/* =  device code for option. */
	int			 rsaddr;		/* = */
{
    int i,j,k;
 
    i = (device_code & 3) << 3;
    j = rsaddr & 7;
    k = i | j;
    PV_WREG(p_me->p_xv, Ext.reg0,k);
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt261_control"
 |
 |  Perform a device-dependent control operation.
 |
 |    = 1 if success, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int m_Lut_Bt261_control( p_me, op, arg )
	O_Lut_Bt261* p_me;	/* -> context for this object. */
	char*		 op;	/* -> string specifying operation to perform. */
	int			 arg;	/* =  optional argument for operation. */
{
	{
		ERRORF((fd, "O_Lut_Bt261_control: unknown option: %s\n", op));
		return 0;
	}
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt261_dump"
 |
 |  Dump the contents of this object to a file descriptor.
 *---------------------------------------------------------------------------*/
static void m_Lut_Bt261_dump( p_me, fd )
	O_Lut_Bt261* p_me;	/* -> context for this object. */
	FILE*		 fd;	/* -> file descriptor to write object contents to. */
{
	fprintf(fd, "\nBrooktree 261 object @ 0x%08x: \n");
	fprintf(fd, "   Hardware library context at 0x%08x\n", p_me->p_xv);
	if ( p_me->atv->control != m_Lut_Bt261_control ) 
	{
		fprintf(fd, 
			"  *** control method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->control, m_Lut_Bt261_control);
	}
	else if ( p_me->atv->dump != m_Lut_Bt261_dump ) 
	{
		fprintf(fd, 
			"  *** dump method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->dump, m_Lut_Bt261_dump);
	}
	if ( p_me->atv->get_entries != m_Lut_Bt261_get_entries ) 
	{
		fprintf(fd, 
			"  *** get_entries method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->get_entries, m_Lut_Bt261_get_entries);
	}
	if ( p_me->atv->init_hw != m_Lut_Bt261_init_hw ) 
	{
		fprintf(fd, 
			"  *** init_hw method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->init_hw, m_Lut_Bt261_init_hw);
	}
	if ( p_me->atv->set_entries != m_Lut_Bt261_set_entries ) 
	{
		fprintf(fd, 
			"  *** set_entries method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->set_entries, m_Lut_Bt261_set_entries);
	}
	if ( p_me->atv->terminate != m_Lut_Bt261_terminate ) 
	{
		fprintf(fd, 
			"  *** terminate method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->terminate, m_Lut_Bt261_terminate);
	}
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt261_get_entries"
 |
 |  Return the values of a range of lookup table entries. There are three 
 |  8-in 8-out lookup tables, one for each color component. The values 
 |  pointed to by p_values are integers of the form XBGR.
 |
 |    = 1 on success, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int m_Lut_Bt261_get_entries( p_me, type, start, count, p_values )
	O_Lut_Bt261*	p_me;		/* -> context for this object. */
	int				type;		/* =  type of entries (see objects.h). */
	int				start;		/* =  first location in table to return. */
	int				count;		/* =  # of entries to return. */
	int*			p_values;	/* -> where to place entries being returned. */
{
	int		blue;		/* Blue component of value. */
	int		green;		/* Green component of value. */
	int		i_entry;	/* # of entry in lut now getting. */
	int*	p_entry;	/* Data for entry in lut now getting. */
	int		red;		/* Red component of value. */

	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt261_init_hw"
 |
 |  Initialize hardware to its default state.
 |
 |    = 1 on success, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int m_Lut_Bt261_init_hw( p_me )
	O_Lut_Bt261*	p_me;		/* -> context for this object. */
{
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt261_set_entries"
 |
 |  Set the values of a range of lookup table entries. There are three 8-in
 |  8-out lookup tables, one for each color component. The values pointed to
 |  by p_values are integers of the form XBGR.
 |
 |    = 1 on success, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int m_Lut_Bt261_set_entries( p_me, type, start, count, p_values )
	O_Lut_Bt261*	p_me;		/* -> context for this object. */
	int				type;		/* =  type of entries (see objects.h). */
	int				start;		/* =  first location in table to set. */
	int				count;		/* =  # of entries to set. */
	int*			p_values;	/* -> where to get values for entries. */
{
	int		i_entry;	/* # of entry in lut now setting. */
	int*	p_entry;	/* Data for entry in lut now setting. */

	switch ( type & O_Lut_Entry_Type_Mask )
	{
	  case O_Lut_Entry_Lut:
		p_entry = p_values;
		for ( i_entry = start; i_entry < start+count; i_entry++, p_entry++ )
		{
		}
		break;
	  default:
		return 0;
	}
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt261_terminate"
 |
 |  Terminate use of this object.
 *---------------------------------------------------------------------------*/
static void m_Lut_Bt261_terminate( p_me )
	O_Lut_Bt261*	p_me;		/* -> context for this object. */
{
	free((char*)p_me);
}


static TV_Lut methods_Lut_Bt261 = {
	m_Lut_Bt261_control,
	m_Lut_Bt261_dump,
	m_Lut_Bt261_get_entries,
	m_Lut_Bt261_init_hw,
	m_Lut_Bt261_set_entries,
	m_Lut_Bt261_terminate
};


/*---------------------------------------------------------------------------*
 |"O_Lut_Bt261_O_Lut_Bt261"
 |
 |  Create an instance of a Brooktree 261 lookup table object.
 |
 |    -> object's context on success, or 0 if error occurred.
 *---------------------------------------------------------------------------*/
O_Lut* O_Lut_Bt261_O_Lut_Bt261( p_me, p_xv )
	O_Lut_Bt261* p_me;		/* -> context to fill in, or 0 for dynamic alloc. */
	XV_struct*	 p_xv;		/* -> context for hardware library. */
{
	if ( !p_me )
	{
		if ( !(p_me = (O_Lut_Bt261*)malloc(sizeof(O_Lut_Bt261))) )
		{
			ERRORF((fd, 
				"O_Lut_Bt261_O_Lut_Bt261: Unable to allocate storage\n"));
		}
	}
	p_me->atv = &methods_Lut_Bt261;
	p_me->p_xv = p_xv;
	
	return (O_Lut*)p_me;
}
