/*
 *  symtab_generic.c:	Implementation of simple symbol table object.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     16 May 93 - Andy Goodrich
 *       - Initial coding
 *
 *  Copyright(c) 1993, Andrew C. Goodrich
 * 
 *              NO WARRANTY 
 * 
 *  BECAUSE THIS CODE IS PROVIDED FREE OF CHARGE, THERE IS NO WARRANTY
 *  FOR THIS CODE, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN 
 *  OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES 
 *  PROVIDE THIS CODE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED 
 *  OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS 
 *  TO THE QUALITY AND PERFORMANCE OF THIS CODE IS WITH YOU.  SHOULD THE 
 *  PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, 
 *  REPAIR OR CORRECTION. 
 * 
 *  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING 
 *  WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR 
 *  REDISTRIBUTE THIS CODE AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, 
 *  INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING  *  OUT OF THE USE OR INABILITY TO USE THIS CODE (INCLUDING BUT NOT LIMITED 
 *  TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY 
 *  YOU OR THIRD PARTIES OR A FAILURE OF THIS CODE TO OPERATE WITH ANY OTHER 
 *  PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGES. 
 * 
 *  Used with permission by Parallax Graphics, Inc.
 *  Modifications Copyright(c) 1994, Parallax Graphics, Inc.
 *
 * $Log:   /plxhp/dave/pvcsrc/hwlib/archives/generic_symtab.c_v  $
 * 
 *    Rev 1.1   09 Sep 1996 10:12:04   dave
 * Initial revision
 * Revision 1.1  1995/12/08 10:28:26  dist
 * Initial revision
 *
 * Revision 1.4  95/09/13  09:51:21  09:51:21  dist ()
 * Andy Goodrich: Removed "bear trap" used to get around HP X server
 *                clobbering problem.
 * 
 * Revision 1.3  95/08/24  14:01:54  14:01:54  dist ()
 * Andy Goodrich: Updated Parallax version with my code for Constant String
 *                values.
 * 
 * Revision 1.2  95/05/25  17:08:52  17:08:52  dist ()
 * Andy Goodrich - Combined HP/AIX version for HP 9.1 code freeze.
 * 
 */
#include <sys/types.h>			/* Standard types for this C environment. */
#include <stdio.h>				/* Stream input/output declarations. */
#include "parse_value.h"		/* Pick up definitions for p_value routine. */
#include "objects.h"            /* Pick up public object definitions. */

#define XV_DIAG 1 
#include "xvdebug.h"

typedef struct _I_SST_Cookie {	/* Symbol table presentation cookie. */
    int         dump_bucket;    /* # of bucket now dumping. */
    SST_Entry   *dump_entry;    /* Entry in bucket dumping next. */
} I_SST_Cookie;

typedef struct I_SST {
    O_SST       base;			/* Public symbols for object. */
    int         hash_size;      /* Size of hash table. */
    SST_Entry** hash_table;     /* Symbol table entries. */
    int         n_symbols;      /* # of symbols in symbol table. */
} I_SST;

int*	p_dump_entry = 0;
void m_Symtab_debug(label)
	char*  label;
{
	int* p_entry;
	int  i_entry;

	if ( !p_dump_entry ) return;
	DEBUGF(1, (fd, "\n"));
	for ( p_entry = p_dump_entry-5, i_entry = 0; i_entry < 10; i_entry++ )
	{
		DEBUGF(1, (fd, "%s: entry %08x is %08x\n", label, p_entry, *p_entry));
		p_entry++;
	}
}
void m_Symtab_bear_trap()
{
	if ( *p_dump_entry != 0 ) 
	{
		m_Symtab_debug("bear trap with bad value");
		/* *p_dump_entry = 0; */
	}
}

/*---------------------------------------------------------------------------*
 |"m_SST_create_cookie"
 |
 |  Create a cookie for presentations and return it.
 |
 |    -> cookie for presentations
 |    =  0 if failure
 *---------------------------------------------------------------------------*/
static void* m_SST_create_cookie(p_me)
    I_SST* p_me;   /* -> symbol table to empty. */
{
	return	(void*)malloc(sizeof(I_SST_Cookie));
}

/*---------------------------------------------------------------------------*
 |"m_SST_dump"
 |
 |  Dump a symbol table to a file descriptor.
 *---------------------------------------------------------------------------*/
static void m_SST_dump(p_me, fd)
    I_SST* p_me;   /* -> symbol table to empty. */
	FILE*  fd;	   /* =  file descriptor to dump to. */
{
    int          i_bucket;  /* Bucket now accessing. */
    SST_Entry*   p_entry;   /* Entry now accessing. */
    SST_Entry*   p_next;    /* Next entry to access. */
 
    for ( i_bucket = 0; i_bucket < p_me->hash_size; i_bucket++ )
    {
        for ( p_entry=p_me->hash_table[i_bucket]; p_entry; p_entry=p_next )
        {
            p_next = p_entry->p_next;
			fprintf(fd, "%08x: name: %s value: %08x\n", 
				p_entry, p_entry->name, p_entry->value);
        }
    }
}
 
/*---------------------------------------------------------------------------*
 |"m_SST_empty"
 |
 |  Empty a symbol table.
 *---------------------------------------------------------------------------*/
static void m_SST_empty(p_me)
    I_SST *p_me;   /* -> symbol table to empty. */
{
    int         i_bucket;  /* Bucket now accessing. */
    SST_Entry*  p_entry;   /* Entry now accessing. */
    SST_Entry*  p_next;    /* Next entry to access. */
 
    for ( i_bucket = 0; i_bucket < p_me->hash_size; i_bucket++ )
    {
        for ( p_entry=p_me->hash_table[i_bucket]; p_entry; p_entry=p_next )
        {
            p_next = p_entry->p_next;
            if ( p_me->base.delete_notify )
			{
                (*p_me->base.delete_notify)(p_entry);
			}
            free((char*)p_entry);
        }
    }
    memset(p_me->hash_table, 0, p_me->hash_size * sizeof(SST_Entry*));
    p_me->n_symbols = 0;
}
 
/*---------------------------------------------------------------------------*
 |"m_SST_find_symbol"
 |
 |  Internal routine: Find a symbol given its name.
 |
 |  -> symbol on success
 |  =  0 on failure
 *---------------------------------------------------------------------------*/
static SST_Entry* m_SST_find_symbol(p_me, name, previous)
    I_SST*      p_me;     /* -> symbol table to look up symbol in. */
    char*       name;     /* -> name of symbol to find. */
    SST_Entry** previous; /* -> where to store symbol prior found or 0. */
{
    int         hash_index; /* Index into hash table of bucket for this name. */    SST_Entry*  p_entry;    /* Symbol entry examining. */
    SST_Entry*  p_prior;    /* Symbol before one accessing. */
    int         str_len;    /* Length of name to search for. */
 
    /*  CALCULATE HASH INDEX AND LOOK UP SYMBOL.  */

    str_len = strlen(name);
    hash_index = (name[0]+name[str_len-1]) * 37 & (p_me->hash_size-1);
 
    p_prior = (SST_Entry *)(&p_me->hash_table[hash_index]);
    for (p_entry=p_prior->p_next; p_entry;
        p_prior=p_entry, p_entry=p_prior->p_next)
    {
        if ( !strcmp(p_entry->name, name) ) break;
    }
    
    if ( previous ) *previous = p_prior;

    return p_entry;
}
 
/*---------------------------------------------------------------------------*
 |"m_SST_free_cookie"
 |
 |  Empty a symbol table.
 *---------------------------------------------------------------------------*/
static void m_SST_free_cookie(p_me, p_cookie)
    I_SST* 			p_me;   	/* -> symbol table to empty. */
	I_SST_Cookie*	p_cookie;	/* -> cookie to be freed. */
{
	free(p_cookie);
}

/*---------------------------------------------------------------------------*
 |"m_SST_get"
 |
 |  Set up to dump the contents of a symbol table.
 |
 |    -> value of symbol, or 0 if the symbol does not exist.
 *---------------------------------------------------------------------------*/
static SST_Entry* m_SST_get( p_me, name )
    I_SST*      p_me;        /* -> symbol table to get symbol from. */
    char*       name;        /* -> name of symbol to get value of. */
{
    SST_Entry*    p_entry;     /* Entry for this symbol. */
 
    if ( !(p_entry = m_SST_find_symbol(p_me, name, 0)) ) return 0;
 
    return p_entry;
}

/*--------------------------------------------------------------------------*
 |"m_SST_load"
 |
 |  Load a symbol table from a file. Note this function calls itself 
 |  recursively handled nested symbol tables.
 |
 |    = # of symbols loaded on success, 0 on failure
 *--------------------------------------------------------------------------*/
int m_SST_load( p_st, fd )
	I_SST*		p_st;	/* -> symbol table to be loaded.  */
	FILE*		fd;		/* -> file descriptor to read from.  */
{
	int			c;				/* Character adding to quoted string. */
	ST_String	descr;			/* String descriptor to be filled in. */
	int			i_byte;			/* # of byte in string / context now parsing. */
	int			i_symbol;		/* # of symbols loaded. */
	int			i_value;		/* # of character in 'value' now accessing. */
    int         loop;           /* Loop escape control variable.  */
	int			n_value;		/* # of bytes in 'value'. */
	PV_Result	result;			/* Value parsed by p_value. */
	char		size[128];		/* Size of string array or context. */
    char        symbol[128];    /* Name of symbol to be set.  */
    char        value[128];     /* Unparsed value for 'symbol'.  */

    for ( i_symbol = 0;;)
    {
        if ( fscanf(fd, "%s", symbol) != 1 ) break;

		/*
		 *  COMMENT: SKIP OVER TEXT UNTIL NEW LINE.
		 */
        if ( symbol[0] == '#' )
        {
            for (loop = 1; loop; )
            {
                switch( fgetc(fd) )
                {
                  case EOF:
                  case '\n':
                    loop = 0;
                }
            }
        }

		
		/*  END OF SYMBOL TABLE: RETURN FROM THIS NESTING LEVEL. */
		 
        else if ( symbol[0] == '}' )
        {
			return 1;
		}

		
		/*  SYMBOL NAME PARSED: PARSE = AND VALUE AND PUT SYMBOL INTO TABLE */
		 
		else
		{
			if ( (fscanf(fd, "%s", value) != 1) || (value[0] != '=') )
			{
				fprintf(stderr, "No = present for %s", symbol);
				return 0;
            }

            if ( fscanf(fd, "%s", value) != 1 )
            {
                fprintf(stderr, "No value present for %s", symbol);
                return 0;
            }
			

			/*  ARRAY VALUE: PARSE SIZE, ALLOCATE STORAGE & READ IN ITS VALUES*/

			else if ( value[0] == '[' )
			{
				if ( fscanf(fd, "%s", size) != 1 )
				{
					fprintf(stderr, "No size present for array %s", symbol);
					return 0;
				}
				if( p_value(size, &result) != PV_Integer )
				{
					fprintf(stderr, "%s is not a legal array length", size);
					return 0;
				}

				if ( (fscanf(fd, "%s", value) != 1) || (value[0] != ':') )
				{
					fprintf(stderr, "Missing : in specification of array %s", 
						symbol);
					return 0;
				}
					
				descr.n_string = result.i;
				if ( !(descr.p_string = (void*)malloc(descr.n_string)) )
				{
					fprintf(stderr, "*** unable to get %d bytes for %s\n",
						descr.n_string, symbol);
					return 0;
				}
				for ( i_byte = 0; i_byte < descr.n_string; i_byte++ )
				{
					if ( fscanf(fd, "%s", value) != 1 )
					{
						fprintf(stderr, "No value present for %s[%d]", 
							symbol, i_byte);
						return 0;
					}
					if ( p_value(value, &result) != PV_Integer )
					{
						fprintf(stderr, "%s is not a legal value for %s[%d]\n",
							value, symbol, i_byte);
						return 0;
					}
					descr.p_string[i_byte] = result.i;
				}
				if ( (fscanf(fd, "%s", value) != 1) || (value[0] != ']') )
				{
					fprintf(stderr, "Missing ] in specification of array %s", 
						symbol);
					return 0;
				}
					
				O_SST_set((O_SST*)p_st, symbol, STE_String, &descr);
				i_symbol++;
			}

			
			/*  QUOTED STRING VALUE: PARSE UNTIL WE HIT ENDING QUOTE. */

			else if ( value[0] == '"' || value[0] == '\'' )
			{
				n_value = strlen(value);
				if ( value[0] != value[n_value-1] )
				{
					for ( i_value = n_value; ; )
					{
						c = fgetc(fd);
						if ( c == EOF )
						{
							fprintf(stderr, "Missing \" on end of %s's value\n",
								symbol);
							return 0;
						}
						value[i_value++] = c;
						if ( c == value[0] )
						{
							value[i_value] = '\0';
							break;
						}
					}
				}
				p_value(value, &result);
				O_SST_set((O_SST*)p_st, symbol, STE_Character, result.p);
				i_symbol++;
			}
			

			/*  NORMAL VALUE: PARSE IT AND PLACE IT IN SYMBOL TABLE. */
			 
			else 
			{
				switch( p_value(value, &result) )
				{
				  default:
					fprintf(stderr, "%s is not a legal value for %s", value, 
						symbol);
					return 0;
				  case PV_Keyword:     
					/* #### */
					O_SST_set((O_SST*)p_st, symbol, STE_Character, result.p);
					i_symbol++;
					break;
				  case PV_Float:       
					O_SST_set((O_SST*)p_st, symbol, STE_Double, result.d);
					i_symbol++;
					break;
				  case PV_Integer:      
					O_SST_set((O_SST*)p_st, symbol, STE_Integer, result.i);
					i_symbol++;
					break;
				}
            }
		}
	}
	return i_symbol;
}


/*---------------------------------------------------------------------------*
 |"m_SST_next"
 |
 |  Return the next symbol in a symbol table dump. If the name parameter is
 |  null don't return the name.
 |
 |    -> entry of symbol, 0 if no more symbols left to be dumped.
 *---------------------------------------------------------------------------*/
static SST_Entry* m_SST_next( p_me, p_cookie )
    I_SST*          p_me;       /* -> symbol table to get symbol from. */
	I_SST_Cookie*	p_cookie;	/* -> traversal cookie. */
{
    SST_Entry*   p_entry;   /* Entry to return this time. */
 
    if ( p_cookie->dump_entry )
    {
        p_entry = p_cookie->dump_entry;
        p_cookie->dump_entry = p_entry->p_next;
        return p_entry;
    }
    else
    {
        for ( p_cookie->dump_bucket = p_cookie->dump_bucket+1;
              p_cookie->dump_bucket < p_me->hash_size; p_cookie->dump_bucket++ )
        {
            if (p_cookie->dump_entry = p_me->hash_table[p_cookie->dump_bucket])
            {
                p_entry = p_cookie->dump_entry;
                p_cookie->dump_entry = p_entry->p_next;
                return p_entry;
            }
        }
    }
 
    return 0;
}

/*---------------------------------------------------------------------------*
 |"m_SST_present"
 |
 |  Set up to dump the contents of a symbol table.
 |
 |    = number of symbols in table.
 *---------------------------------------------------------------------------*/
static int m_SST_present( p_me, p_cookie ) 
    I_SST* 			p_me;  		/* -> symbol table to dump. */
	I_SST_Cookie*	p_cookie;	/* -> Presentation cookie to fill in. */
{
	p_cookie->dump_bucket = 0;
	p_cookie->dump_entry = p_me->hash_table[p_cookie->dump_bucket];

    return p_me->n_symbols;
}

/*---------------------------------------------------------------------------*
 |"m_SST_remove"
 |
 |  Remove a symbol given its name.
 *---------------------------------------------------------------------------*/
static int m_SST_remove(p_me, name ) 
    I_SST*      p_me;  	/* -> symbol table to remove symbol from. */
    char*       name;   /* -> name of symbol to remove. */
{
    SST_Entry*    p_entry;   /* Entry for this symbol. */
    SST_Entry*    prior;     /* Symbol prior to our entry. */
 
    p_entry = m_SST_find_symbol(p_me, name, &prior);
    if ( !p_entry ) return 0;
 
    if ( p_me->base.delete_notify )
        (*p_me->base.delete_notify)(p_entry);
    prior->p_next = p_entry->p_next;
    free((char *)p_entry);
    p_me->n_symbols--;
 
    return 1;
}

/*---------------------------------------------------------------------------*
 |"m_SST_set"
 |
 |  Set the value of a symbol.
 |
 |    -> symbol table entry on success, 0 on failure
 *---------------------------------------------------------------------------*/
static int m_SST_set(p_me, name, type, value ) 
    I_SST*      p_me;  	/* -> symbol table to set symbol in. */
    char*       name;  	/* -> name of symbol to remove. */
	int			type;	/* = type of value. */
    int   	    value;  /* = value to set. */
{
    SST_Entry*    p_entry;   /* Entry for this symbol. */
    SST_Entry*    p_prior;   /* Symbol prior to our entry. */

    /*
     *  LOOK UP SYMBOL, IF IT DOES NOT EXIST ALLOCATE STORAGE FOR IT
     *  AND CHAIN IT INTO THE DATA STRUCTURE. IF IT DOES EXIST AND
     *  THERE IS A DESTROY ROUTINE DELETE THE CURRENT VALUE.
     */
    p_entry = m_SST_find_symbol(p_me, name, &p_prior);
    if ( !p_entry )
    {
        if ( !(p_entry = (SST_Entry *)malloc(sizeof(SST_Entry)+strlen(name))) )
        {
            return 0;
        }
        strcpy(p_entry->name, name);
        p_entry->p_next = p_prior->p_next;
        p_prior->p_next = p_entry;
        p_me->n_symbols++;
    }
    else
    {   
        if ( (int)p_me->base.delete_notify )
            (*p_me->base.delete_notify)(p_entry);
    }

    /*
     *  SET VALUE AND TYPE OF VALUE INTO SYMBOL.
     */
	switch ( p_entry->type = type )
	{
	  case STE_Character:	p_entry->value.c = (char*)value;			break;
	  case STE_Integer:     p_entry->value.i = value;					break;
	  case STE_String_Constant:
	  case STE_String:		
		p_entry->value.s = *((ST_String*)value);	
		break;
	}

    return (int)p_entry;
}

/*--------------------------------------------------------------------------*
 |"m_SST_store"
 |
 |  Store the contents of a symbol table in a file. 
 |
 |    = 1 on success, 0 on failure
 *--------------------------------------------------------------------------*/
int m_SST_store( p_st, fd )
	I_SST*		p_st;	/* -> symbol table to be loaded.  */
	FILE*		fd;		/* -> file descriptor to read from.  */
{
	int			i_char;		/* # of character in p_string accessing. */
	int			i_move;		/* # of entry now moving in sort. */
	int			i_sort;		/* # of entry examining in sort. */
	int			i_ste;		/* Entry in p_stes now accessing. */
    void* 		iterator;   /* Iterator to use to search table. */
	int			n_symbols;	/* # of symbols to be dumped. */
    SST_Entry*  p_ste;      /* Symbol table entry printing. */
	SST_Entry**	p_stes;		/* List of symbol tables to dump. */


	/* ACQUIRE AND ALPHABETIZE SYMBOLS. */

    iterator = O_SST_create_cookie((O_SST*)p_st);
    n_symbols = O_SST_present((O_SST*)p_st, iterator);
	if ( !(p_stes = (SST_Entry**)malloc(n_symbols*sizeof(SST_Entry*))) )
	{
		fprintf(stderr, "*** m_SST_store: Unable to allocate storage\n");
		return 0;
	}
	for ( i_ste = 0, p_ste = O_SST_next((O_SST*)p_st, iterator) ; p_ste ; 
		 p_ste = O_SST_next((O_SST*)p_st, iterator) )
	{
		for ( i_sort = 0; i_sort < i_ste; i_sort++ )
		{
			if ( (strcmp(p_ste->name, p_stes[i_sort]->name) < 0) ) break;
		}
		for ( i_move = i_ste-1; i_move >= i_sort; --i_move )
		{
			p_stes[i_move+1] = p_stes[i_move];
		}
		p_stes[i_sort] = p_ste;
		i_ste++;
	}

	/*  DUMP THE SYMBOLS FROM THE ALPHABETIZED LIST. */

    for ( i_ste = 0; i_ste < n_symbols; i_ste++ )
    {
		p_ste = p_stes[i_ste];
	    fprintf(fd, "%s ", p_ste->name); 
		fprintf(fd, "= ");
        switch( p_ste->type )
        {
          case STE_Character:
            fprintf(fd, "'%s'\n", p_ste->value.c);
            break;
          case STE_Double:
            fprintf(fd, "%f\n", p_ste->value.d);
            break;
          case STE_Integer:
            fprintf(fd, "%d\t\t## Ox%08x\n", p_ste->value.i, p_ste->value.i);
            break;
          case STE_Generic:
            fprintf(fd, "%08x\n", p_ste->value.p);
            break;
		  case STE_String:
		  case STE_String_Constant:
			fprintf(fd, "\n");
			fprintf(fd, "[ %d : ", p_ste->value.s.n_string);
			for ( i_char = 0; i_char < p_ste->value.s.n_string; i_char++ )
			{
				if ( i_char % 8 == 0 ) 
				{
					fprintf(fd, "\n    ");
				}
				fprintf(fd, "0x%02x ", p_ste->value.s.p_string[i_char]);
			}
			fprintf(fd, "\n");
			fprintf(fd, "]\n");
        }
    }
    O_SST_free_cookie((O_SST*)p_st, iterator);
	free(p_stes);
	return 1;
}
/*---------------------------------------------------------------------------*
 |"m_SST_terminate"
 |
 |  Terminate use of a symbol table object, freeing any necessary storage.
 |
 *---------------------------------------------------------------------------*/
static void m_SST_terminate(p_me) 
    I_SST      *p_me;  /* -> symbol table to free up. */
{
	m_SST_empty(p_me);
	free(p_me);
}

static Tv_SST st_methods =
{
	m_SST_create_cookie,
	m_SST_dump,
	m_SST_empty,
	m_SST_free_cookie,
	m_SST_get,
	m_SST_load,
	m_SST_next,
	m_SST_present,
	m_SST_remove,
	m_SST_set,
	m_SST_store,
	m_SST_terminate
};


/*---------------------------------------------------------------------------*
 |"O_SST_Generic_Delete"
 |
 |  Generic delete routine for symbol table entries. We free storage for
 |  strings which have been dynamically allocated.
 *---------------------------------------------------------------------------*/
void O_SST_Generic_Delete( p_entry )
    SST_Entry*  p_entry;   /* -> entry being deleted. */
{
	switch ( p_entry->type )
    {
	  case STE_Character:   break;
	  case STE_Integer:     break;
	  case STE_String:      free(p_entry->value.s.p_string); break;
	  case STE_String_Constant: break;
	}
}


/*---------------------------------------------------------------------------*
 |"O_SST_O_SST"
 |
 |  Create and return a symbol table object.
 |
 |    -> symbol table object on success, 0 on failure
 *---------------------------------------------------------------------------*/
O_SST *O_SST_O_SST(average_symbols, delete_rtn)
	int		average_symbols;		/* = average # of symbols expected. */
	void	(*delete_rtn);			/* -> delete rtn to call or zero. */
{
    int     	hash_size;  /* Hash symbol table size. */
    I_SST*      p_me;  		/* Symbol table creating. */

    /*
     *  ALLOCATE SPACE FOR SYMBOL TABLE OBJECT.
     */
    for ( hash_size = 1; ; hash_size = hash_size << 1 )
    {
        if ( hash_size >= average_symbols ) break;
    }

	p_me = (I_SST *)malloc(sizeof(I_SST)+sizeof(SST_Entry *)*hash_size);

    /*
     *  INITIALIZE DATA STRUCTURE:
     */
	p_me->base.atv = &st_methods;
    p_me->base.delete_notify = (void*)delete_rtn;

    p_me->n_symbols = 0;
    p_me->hash_size = hash_size;
    p_me->hash_table = (SST_Entry **)(p_me+1);
    memset(p_me->hash_table, 0, hash_size*sizeof(SST_Entry *));
#ifndef NOTDEF
	p_dump_entry = 0 /* 0x40046000 #### (int*)&p_me->hash_table[345]*/ ;
#endif

	return (O_SST *)p_me;
}


static short dec_table[256] =	/* Table for recognizing decimal #'s. */
{
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -3,  -1,  -3,  -2,  -1, 
  0,   1,   2,   3,   4,   5,   6,   7, 
  8,   9,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1
 };

static short	hex_table[256] =	/* Table for recognizing hexadecimal #'s. */
{
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
  0,   1,   2,   3,   4,   5,   6,   7, 
  8,   9,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  10,  11,  12,  13,  14,  15,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  10,  11,  12,  13,  14,  15,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1
};

/*---------------------------------------------------------------------------*
 |"p_value"
 |
 |  Parse a value from a string of characters. The value parsed must use up
 |  all the characters of the string. For keyword and quoted string values
 |  storage will be allocated to hold the result.
 |
 |		= type of value parsed, or error specification
 *---------------------------------------------------------------------------*/
p_value( p_str, p_result ) 
	char		*p_str;		/* -> string to be parsed for value. */
	PV_Result	*p_result;	/* -> where to place parsed value. */
{
	int		digit;	    /* Digit value for decimal number. */
	int		integer;	/* Value accummulating for decimal or hex number. */
	int		i_str;		/* Character within p_str accessing. */
	int		its_float;	/* = 1 if parsing floating point number. */
	int		n_str;	    /* Length of argument. */
	int		nibble;	    /* Nibble value for hexadecimal number. */
	int		sign;		/* Sign of integer or floating point number. */

	n_str = strlen(p_str);

	/*
	 *  HEXADECIMAL INTEGER VALUE
	 */
	if ( n_str > 2 && p_str[0] == '0' && (p_str[1] == 'x' || p_str[1] == 'X') )
	{
		for ( integer = 0, i_str = 2; i_str < n_str; i_str++ )
		{
			if ( (nibble = hex_table[p_str[i_str]]) == -1 )
			{
				return PV_Bad_Value;
			}
			integer = integer << 4 | nibble;
		}
		p_result->i = integer;
		return PV_Integer;
	}

	/*
	 *  DECIMAL INTEGER OR FLOATING POINT VALUE
	 */
	else if ( dec_table[p_str[0]] != -1 )
	{
		if ( (p_str[0] == '-') )
		{
			if (n_str == 1) return PV_Bad_Value;
			i_str = 1;
			sign = -1;
		}
		else if ( p_str[0] == '+' )
		{
			if (n_str == 1) return PV_Bad_Value;
			i_str = 1;
			sign = 1;
		}
		else
		{
			i_str = 0;
			sign = 1;
		}
			
		for ( integer = 0, its_float = 0; i_str < n_str; i_str++ )
		{
			digit = dec_table[p_str[i_str]];
			if( digit < 0 )
			{
				switch(digit)
				{
				  case -1:
				  case -3:
					return PV_Bad_Value;
				  case -2:
					if ( its_float ) return PV_Bad_Value;
					its_float = 1;
					break;
				}
			}
			else
			{
				its_float = its_float * 10;
				integer = integer * 10 + digit;
			}
		}
		if ( its_float )
		{	
			p_result->d = (float)(sign*integer) / (float)(its_float);
			return PV_Float;
		}
		else
		{
			p_result->i = sign * integer;
			return PV_Integer;
		}
	}

	/*
	 *  QUOTED STRING
	 */
	else if ( n_str > 2 && (p_str[0] == '"') || (p_str[0] == '\'' ) )
	{
		if ( p_str[n_str-1] != p_str[0] )
		{
			return PV_Bad_Value;
		}
		else if ( !(p_result->p = (char *)malloc(n_str)) )
		{
			return PV_Bad_Alloc;
		}
			
		strcpy(p_result->p, &p_str[1]);
		p_result->p[n_str-2] = '\0';
		return PV_Quoted_String;
	}

	/*
	 *  KEYWORD
	 */
	else
	{
		if ( !(p_result->p = (char *)malloc(n_str+1)) )
		{
			return PV_Bad_Alloc;
		}
			
		strcpy(p_result->p, p_str);
		return PV_Keyword;
	}
}
