/*
 *  pvpci_rsrc.c:	Implementation of pvpci specific resource object.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     12 Nov 93 - Andy Goodrich
 *       - Initial coding
 *
 *  Copyright(c) 1994, Parallax Graphics, Inc.
 */
#include <sys/types.h>			/* Standard types for this C environment. */
#include <stdio.h>				/* Stream input/output declarations. */
#include "parse_value.h"		/* Pick up definitions for p_value routine. */
#include "hw_context.h"			/* Pick up hardware library context. */
#include "plxv_xpoint.h"        /* Pick up PORT_A and friends. */
#define XV_DIAG 1
#include "xvdebug.h"
 
typedef struct I_Rsrc {
    O_Rsrc      base;			/* Public symbols for object. */
	O_SST*		p_st;			/* Symbol table to use. */
} I_Rsrc;

#define DEF_RAS_MODE ~0x10

static XV_video_format input_format_dmsd[2] = {
        {640,482,43,10,20},
        {768,574,51,7,23}};

static XV_video_format output_format[2] = {
        {640,482,43,7,21},
        {768,574,51,7,34}};

static char dmsd_all[] = {
/*   00     01     02     03     04     05     06*    07    */
/*   --     --     --     --     --     --     --     --    */
     0x4c,  0x3c,  0x0d,  0xef,  0xbd,  0xe2,  0x00,  0x00,

/*   08     09     0a     0b     0c*    0d*#   0e*    0f*   */
/*   --     --     --     --     --     --     --     --    */
     0xf8,  0xf8,  0x60,  0x60,  0x0,  0x86,  0x18,  0x90,  

/*   10     11     12*    13*    14*    15*    16*    17    */
/*   --     --     --     --     --     --     --     --    */
     0x00,  0x2c,  0x40,  0x46,  0x42,  0x1a,  0xff,  0xda,

/*   18     19                                              */
/*   --     --                                              */
     0xe2, 0x8b
};

static char dmse_ntsc[] = {
/*   00     01     02     03     04     05     06     07    */
/*   --     --     --     --     --     --     --     --    */
    0x5d,  0x00,  0x00,  0x00,  0xf0,  0x2d,  0x52,  0x3a,

/*   08     09     0a     0b     0c*    0d*    0e     0f    */
/*   --     --     --     --     --     --     --     --    */
    0x60,  0x00,  0x00,  0x00,  0x56,  0x00,  0x0c,  0x00
};

static char dmse_pal[] = {
/*   00     01     02     03     04     05     06     07    */
/*   --     --     --     --     --     --     --     --    */
    0x5d,  0x00,  0x00,  0x00,  0xf0,  0x2d,  0x52,  0x3a,

/*   08     09     0a     0b     0c     0d     0e     0f    */
/*   --     --     --     --     --     --     --     --    */
	0x60,  0x00,  0x00,  0x00,  0x25,  0x00,  0x02,  0x00
};


/*---------------------------------------------------------------------------*
 |"m_Rsrc_defaults"
 |
 |  Set default resource values in this resource object.
 *---------------------------------------------------------------------------*/
static void m_Rsrc_defaults(p_me)
    I_Rsrc *p_me;   /* -> resource object to default resources of. */
{
	O_SST*	    p_st;		/* Fast access to symbol table object. */
	int		    (*set)();	/* Fast access so symbol table set routine. */
	ST_String	string;		/* Temp to build string values in. */

	p_st = p_me->p_st;
	set = p_st->atv->set;


	/* GENERAL BOARD AND DISPLAY RESOURCES: */

    (*set)(p_st, "Attr",         STE_Integer,  0x80c0);
    (*set)(p_st, "board_code",   STE_Integer,  EISA_BUS_1_135);
    (*set)(p_st, "clt_type",     STE_Integer,  BT495);
    (*set)(p_st, "c3_rev",       STE_Integer,  C3REV_1);
    (*set)(p_st, "delay",        STE_Integer,  0x0e);
    (*set)(p_st, "DispFlgs",     STE_Integer,  0x1f4a);
    (*set)(p_st, "DispKludge",   STE_Integer,  0xc41f);
    (*set)(p_st, "ga_rev",       STE_Integer,  GAREV_2);
	(*set)(p_st, "h.bp",         STE_Integer,  0x24); /* */
	(*set)(p_st, "h.disp",       STE_Integer,  255);
	(*set)(p_st, "h.fp",         STE_Integer,  1); /* */
	(*set)(p_st, "h.hsync_nser", STE_Integer,  32768);
	(*set)(p_st, "h.hsync_ser",  STE_Integer,  164);
	(*set)(p_st, "h.sy",         STE_Integer,  0x1f); /* */
    (*set)(p_st, "hlib_rev",     STE_Integer,  HLIB_REV);
    (*set)(p_st, "linebytes",    STE_Integer,  1280);
    (*set)(p_st, "misc_flags",   STE_Integer,  0);
    (*set)(p_st, "misc_flags2",  STE_Integer,  0);
    (*set)(p_st, "OLDI2C",       STE_Integer,  0);
    (*set)(p_st, "pixel_clock",  STE_Integer,  135);
    (*set)(p_st, "power_hours",  STE_Integer,  0);
    (*set)(p_st, "power_cycles", STE_Integer,  1);
    (*set)(p_st, "RasMode",      STE_Integer,  DEF_RAS_MODE);
    (*set)(p_st, "skip",         STE_Integer,  0x06);
    (*set)(p_st, "StartUp",      STE_Integer,  0x604 | 0x8000000);
    (*set)(p_st, "TmrInit",      STE_Integer,  0x171);
	(*set)(p_st, "v.bp",         STE_Integer,  0x24); /* */
	(*set)(p_st, "v.disp",       STE_Integer,  1023);
	(*set)(p_st, "v.fp",         STE_Integer,  0); /* */
	(*set)(p_st, "v.sy",         STE_Integer,  2);
    (*set)(p_st, "xsize",        STE_Integer,  1280);
    (*set)(p_st, "ysize",        STE_Integer,  1024);
	(*set)(p_st, "zoom",         STE_Integer,  1);

	/* CHANNEL A RESOURCES: */

    (*set)(p_st, "a.flags",      STE_Integer,  IO_COMP_OUT);
	string.n_string = sizeof(dmse_ntsc);
	string.p_string = (void*)dmse_ntsc;
    (*set)(p_st, "a.dmse.ntsc",  STE_String_Constant,   &string);
	string.n_string = sizeof(dmse_pal);
	string.p_string = (void*)dmse_pal;
    (*set)(p_st, "a.dmse.pal",   STE_String_Constant,   &string);
    (*set)(p_st, "a.dmse.unit",  STE_Integer,  0);
	string.n_string = sizeof(input_format_dmsd[0]);
	string.p_string = (void*)&input_format_dmsd[0];
    (*set)(p_st, "a.format[0]",  STE_String_Constant,  &string);
	string.n_string = sizeof(input_format_dmsd[1]);
	string.p_string = (void*)&input_format_dmsd[1];
    (*set)(p_st, "a.format[1]",  STE_String_Constant,  &string);


	/* CHANNEL B RESOURCES: */

    (*set)(p_st, "b.flags",      STE_Integer,  IO_DMSD);
	string.n_string = sizeof(dmsd_all);
	string.p_string = (void*)dmsd_all;
    (*set)(p_st, "b.dmsd.ntsc",  STE_String_Constant,   &string);
	string.n_string = sizeof(dmsd_all);
	string.p_string = (void*)dmsd_all;
    (*set)(p_st, "b.dmsd.pal",   STE_String_Constant,   &string);
	string.n_string = sizeof(dmsd_all);
	string.p_string = (void*)dmsd_all;
    (*set)(p_st, "b.dmsd.secam", STE_String_Constant,   &string);
    (*set)(p_st, "b.dmsd.unit",  STE_Integer,  1);
	string.n_string = sizeof(input_format_dmsd[0]);
	string.p_string = (void*)&input_format_dmsd[0];
    (*set)(p_st, "b.format[0]",  STE_String_Constant,  &string);
	string.n_string = sizeof(input_format_dmsd[1]);
	string.p_string = (void*)&input_format_dmsd[1];
    (*set)(p_st, "b.format[1]",  STE_String_Constant,  &string);


	/* CHANNEL D RESOURCES: */

    (*set)(p_st, "d.flags",      STE_Integer,  IO_DMSD);
	string.n_string = sizeof(dmsd_all);
	string.p_string = (void*)dmsd_all;
    (*set)(p_st, "d.dmsd.ntsc",  STE_String_Constant,   &string);
	string.n_string = sizeof(dmsd_all);
	string.p_string = (void*)dmsd_all;
    (*set)(p_st, "d.dmsd.pal",   STE_String_Constant,   &string);
	string.n_string = sizeof(dmsd_all);
	string.p_string = (void*)dmsd_all;
    (*set)(p_st, "d.dmsd.secam", STE_String_Constant,   &string);
    (*set)(p_st, "d.dmsd.unit",  STE_Integer,  0);
	string.n_string = sizeof(input_format_dmsd[0]);
	string.p_string = (void*)&input_format_dmsd[0];
    (*set)(p_st, "d.format[0]",  STE_String_Constant,  &string);
	string.n_string = sizeof(input_format_dmsd[1]);
	string.p_string = (void*)&input_format_dmsd[1];
    (*set)(p_st, "d.format[1]",  STE_String_Constant,  &string);
 

    /* ZOOM RESOURCES: */

    (*set)(p_st, "zoom[0].delay",        STE_Integer,  0x0e);
    (*set)(p_st, "zoom[0].DispFlgs",     STE_Integer,  0x0a48);
    (*set)(p_st, "zoom[0].factor",       STE_Integer,  2);
    (*set)(p_st, "zoom[0].h.bp",         STE_Integer,  48/2 - 1);
    (*set)(p_st, "zoom[0].h.disp",       STE_Integer,  256/2 - 1);
    (*set)(p_st, "zoom[0].h.fp",         STE_Integer,  6/2 -1);
    (*set)(p_st, "zoom[0].h.hsync_nser", STE_Integer,  32768);
    (*set)(p_st, "zoom[0].h.hsync_ser",  STE_Integer,  0x91);
    (*set)(p_st, "zoom[0].h.sy",         STE_Integer,  42/2 -1);
    (*set)(p_st, "zoom[0].skip",         STE_Integer,  0);
}


/*---------------------------------------------------------------------------*
 |"m_Rsrc_load_from_context"
 |
 |  Set resource values from  hardware context. 
 *---------------------------------------------------------------------------*/
static void m_Rsrc_load_from_context(p_me, p_xv)
    I_Rsrc*     p_me;   /* -> resource object to default resources of. */
    XV_struct*  p_xv;   /* -> context to get values from. */
{
	int		    (*set)();		/* Fast access so symbol table set routine. */
	int		    (*set_long)();	/* Fast access so symbol table set routine. */
	ST_String	string;			/* Temp to build string values in. */

	set = p_me->base.atv->set;
	set_long = p_me->base.atv->set_long;

	/* GENERAL BOARD AND DISPLAY RESOURCES: */

	(*set)(p_me, "Attr",           p_xv->main.Attr);
	(*set)(p_me, "board_code",     p_xv->main.board_code);
	(*set)(p_me, "clt_type",       p_xv->main.clt_type);
	(*set)(p_me, "c3_rev",         p_xv->main.c3_rev);
	(*set)(p_me, "delay",          p_xv->main.delay);
	(*set)(p_me, "DispFlgs",       p_xv->main.DispFlgs);
	(*set)(p_me, "DispKludge",     p_xv->main.DispKludge);
	(*set)(p_me, "ga_rev",         p_xv->main.ga_rev);
	(*set)(p_me, "h.bp",           p_xv->main.h.bp);
	(*set)(p_me, "h.disp",         p_xv->main.h.disp);
	(*set)(p_me, "h.fp",           p_xv->main.h.fp);
	(*set)(p_me, "h.hsync_nser",   p_xv->main.h.hsync_nser);
	(*set)(p_me, "h.hsync_ser",    p_xv->main.h.hsync_ser);
	(*set)(p_me, "h.sy",           p_xv->main.h.sy);
	(*set)(p_me, "hlib_rev",       p_xv->main.hlib_rev);
	(*set)(p_me, "linebytes",      p_xv->main.linebytes);
	(*set)(p_me, "misc_flags",     p_xv->main.misc_flags);
	(*set)(p_me, "misc_flags2",    p_xv->main.misc_flags2);
	(*set)(p_me, "OLDI2C",         p_xv->main.OLDI2C);
	(*set)(p_me, "opt.board_code", p_xv->opt.board_code);
	(*set)(p_me, "pixel_clock",    p_xv->main.pixel_clock);
	(*set)(p_me, "power_hours",    p_xv->main.power_hours);
	(*set)(p_me, "power_cycles",   p_xv->main.power_cycles);
	(*set)(p_me, "RasMode",        p_xv->main.RasMode);
	(*set)(p_me, "skip",           p_xv->main.skip);
	(*set)(p_me, "StartUp",        p_xv->main.StartUp);
	(*set)(p_me, "TmrInit",        p_xv->main.TmrInit);
	(*set)(p_me, "v.bp",           p_xv->main.v.bp);
	(*set)(p_me, "v.disp",         p_xv->main.v.disp);
	(*set)(p_me, "v.fp",           p_xv->main.v.fp);
	(*set)(p_me, "v.sy",           p_xv->main.v.sy);
	(*set)(p_me, "xsize",          p_xv->main.xsize);
	(*set)(p_me, "ysize",          p_xv->main.ysize);


	/* CHANNEL A RESOURCES: */

	(*set)(p_me, "a.flags", p_xv->opt.a.flags);
#ifdef NOTDEF
	(*set_long)(p_me, "a.dmse.ntsc", sizeof(p_xv->opt.a.config.dmse.val), /* #### */
		p_xv->opt.a.config.dmse.val);
#endif
	(*set)(p_me, "a.dmse.unit", p_xv->opt.a.config.dmse.unit);
	(*set_long)(p_me, "a.format[0]", sizeof(p_xv->opt.a.format[0]),
		&p_xv->opt.a.format[0]);
	(*set_long)(p_me, "a.format[1]", sizeof(p_xv->opt.a.format[1]),
		&p_xv->opt.a.format[1]);


	/* CHANNEL B RESOURCES: */

	(*set)(p_me, "b.flags", p_xv->opt.b.flags);
#ifdef NOTDEF
	(*set_long)(p_me, "b.dmsd.ntsc", sizeof(p_xv->opt.b.config.dmsd.val),
		p_xv->opt.b.config.dmsd.val);
#endif
	(*set)(p_me, "b.dmsd.unit", p_xv->opt.b.config.dmsd.unit);
	(*set_long)(p_me, "b.format[0]", sizeof(p_xv->opt.b.format[0]), 
		&p_xv->opt.b.format[0]);
	(*set_long)(p_me, "b.format[1]", sizeof(p_xv->opt.b.format[1]),
		&p_xv->opt.b.format[1]);


	/* CHANNEL D RESOURCES: */

	(*set)(p_me, "d.flags", p_xv->main.d.flags);
#ifdef NOTDEF
	(*set_long)(p_me, "d.dmsd.ntsc", sizeof(p_xv->main.d.config.dmsd.val),
		p_xv->main.d.config.dmsd.val);
#endif
	(*set)(p_me, "d.dmsd.unit", p_xv->main.d.config.dmsd.unit);
	(*set_long)(p_me, "d.format[0]", sizeof(p_xv->main.d.format[0]),
		&p_xv->main.d.format[0]);
	(*set_long)(p_me, "d.format[1]", sizeof(p_xv->main.d.format[1]),
		&p_xv->main.d.format[1]);

	
   /* ZOOM RESOURCES: */

    (*set)(p_me, "zoom[0].delay",        p_xv->zoom[0].delay);
    (*set)(p_me, "zoom[0].DispFlgs",     p_xv->zoom[0].DispFlgs);
    (*set)(p_me, "zoom[0].factor",       p_xv->zoom[0].factor);
    (*set)(p_me, "zoom[0].h.bp",         p_xv->zoom[0].h.bp);
    (*set)(p_me, "zoom[0].h.disp",       p_xv->zoom[0].h.disp);
    (*set)(p_me, "zoom[0].h.fp",         p_xv->zoom[0].h.fp);
    (*set)(p_me, "zoom[0].h.hsync_nser", p_xv->zoom[0].h.hsync_nser);
    (*set)(p_me, "zoom[0].h.hsync_ser",  p_xv->zoom[0].h.hsync_ser);
    (*set)(p_me, "zoom[0].h.sy",         p_xv->zoom[0].h.sy);
    (*set)(p_me, "zoom[0].skip",         p_xv->zoom[0].skip);
}


/*--------------------------------------------------------------------------*
 |"m_Rsrc_load_from_file"
 |
 |  Load the contents of a resource object from a file.
 |
 |    = 1 on success, 0 on failure
 *--------------------------------------------------------------------------*/
static int m_Rsrc_load_from_file( p_me, fd )
    I_Rsrc*     p_me;   /* -> resource object to be loaded.  */
    FILE*       fd;     /* -> file descriptor to read from.  */
{
    int     rc; /* Return code from symbol table load. */

    if ( fd )
    {
        return O_SST_load(p_me->p_st, fd);
    }
    else if ( !(fd = fopen("/usr/lpp/parallax/etc/plxv0", "r")) )
    {
		ERRORF((fd," could not open /usr/lpp/parallax/etc/plxv0\n"));
        return 0;
    }
    else
    {   
        rc = O_SST_load(p_me->p_st, fd);
        fclose(fd);
        return rc;
    }
}

 
/*---------------------------------------------------------------------------*
 |"m_Rsrc_store_to_context"
 |
 |  Fill in hardware context with values from this resource object.
 *---------------------------------------------------------------------------*/
static void m_Rsrc_store_to_context(p_me, p_xv)
    I_Rsrc* 	p_me;   /* -> resource object to default resources of. */
	XV_struct*	p_xv;	/* -> context to be filled in. */
{
	int		    (*get)();		/* Fast access so symbol table get routine. */
	int		    (*get_long)();	/* Fast access so symbol table get routine. */
	ST_String	string;			/* Temp to build string values in. */

	get = p_me->base.atv->get;
	get_long = p_me->base.atv->get_long;


	/* GENERAL BOARD AND DISPLAY RESOURCES: */

    p_xv->main.Attr         = (*get)(p_me, "Attr");
    p_xv->main.board_code   = (*get)(p_me, "board_code");
    p_xv->main.clt_type     = (*get)(p_me, "clt_type");
    p_xv->main.c3_rev       = (*get)(p_me, "c3_rev");
    p_xv->main.delay        = (*get)(p_me, "delay");
    p_xv->main.DispFlgs     = (*get)(p_me, "DispFlgs");
    p_xv->main.DispKludge   = (*get)(p_me, "DispKludge");
    p_xv->main.ga_rev       = (*get)(p_me, "ga_rev");
	p_xv->main.h.bp         = (*get)(p_me, "h.bp");
	p_xv->main.h.disp       = (*get)(p_me, "h.disp");
	p_xv->main.h.fp         = (*get)(p_me, "h.fp");
	p_xv->main.h.hsync_nser = (*get)(p_me, "h.hsync_nser");
	p_xv->main.h.hsync_ser  = (*get)(p_me, "h.hsync_ser");
	p_xv->main.h.sy         = (*get)(p_me, "h.sy");
    p_xv->main.hlib_rev     = (*get)(p_me, "hlib_rev");
    p_xv->main.linebytes    = (*get)(p_me, "linebytes");
    p_xv->main.misc_flags   = (*get)(p_me, "misc_flags");
    p_xv->main.misc_flags2  = (*get)(p_me, "misc_flags2");
    p_xv->main.OLDI2C       = (*get)(p_me, "OLDI2C");
    p_xv->main.pixel_clock  = (*get)(p_me, "pixel_clock");
    p_xv->main.power_hours  = (*get)(p_me, "power_hours");
    p_xv->main.power_cycles = (*get)(p_me, "power_cycles");
    p_xv->main.RasMode      = (*get)(p_me, "RasMode");
    p_xv->main.skip         = (*get)(p_me, "skip");
    p_xv->main.StartUp      = (*get)(p_me, "StartUp");
    p_xv->main.TmrInit      = (*get)(p_me, "TmrInit");
	p_xv->main.v.bp         = (*get)(p_me, "v.bp");
	p_xv->main.v.disp       = (*get)(p_me, "v.disp");
	p_xv->main.v.fp         = (*get)(p_me, "v.fp");
	p_xv->main.v.sy         = (*get)(p_me, "v.sy");
    p_xv->main.xsize        = (*get)(p_me, "xsize");
    p_xv->main.ysize        = (*get)(p_me, "ysize");
	p_xv->main.zoom         = (*get)(p_me, "zoom");

    p_xv->opt.board_code = 0;
    p_xv->main.misc_flags = 0;


	/* CHANNEL A RESOURCES: */

	p_xv->opt.a.flags = (*get)(p_me, "a.flags");
	(*get_long)(p_me, "a.dmse.ntsc", p_xv->opt.a.config.dmse.val);
	p_xv->opt.a.config.dmse.unit = (*get)(p_me, "a.dmse.unit");
	(*get_long)(p_me, "a.format[0]", &p_xv->opt.a.format[0]);
	(*get_long)(p_me, "a.format[1]", &p_xv->opt.a.format[1]);


	/* CHANNEL B RESOURCES: */

	p_xv->opt.b.flags = (*get)(p_me, "b.flags");
	(*get_long)(p_me, "b.dmsd.ntsc", p_xv->opt.b.config.dmsd.val);
	p_xv->opt.b.config.dmsd.unit = (*get)(p_me, "b.dmsd.unit");
	(*get_long)(p_me, "b.format[0]", &p_xv->opt.b.format[0]);
	(*get_long)(p_me, "b.format[1]", &p_xv->opt.b.format[1]);


	/* CHANNEL D RESOURCES: */

	p_xv->main.d.flags = (*get)(p_me, "b.flags");
	(*get_long)(p_me, "d.dmsd.ntsc", p_xv->main.d.config.dmsd.val);
	p_xv->main.d.config.dmsd.unit = (*get)(p_me, "d.dmsd.unit");
	(*get_long)(p_me, "d.format[0]", &p_xv->main.d.format[0]);
	(*get_long)(p_me, "d.format[1]", &p_xv->main.d.format[1]);


    /* ZOOM RESOURCES: */

    p_xv->zoom[0].delay        = (*get)(p_me, "zoom[0].delay");
    p_xv->zoom[0].DispFlgs     = (*get)(p_me, "zoom[0].DispFlgs");
    p_xv->zoom[0].factor       = (*get)(p_me, "zoom[0].factor");
    p_xv->zoom[0].h.bp         = (*get)(p_me, "zoom[0].h.bp");
    p_xv->zoom[0].h.disp       = (*get)(p_me, "zoom[0].h.disp");
    p_xv->zoom[0].h.fp         = (*get)(p_me, "zoom[0].h.fp");
    p_xv->zoom[0].h.hsync_nser = (*get)(p_me, "zoom[0].h.hsync_nser");
    p_xv->zoom[0].h.hsync_ser  = (*get)(p_me, "zoom[0].h.hsync_ser");
    p_xv->zoom[0].h.sy         = (*get)(p_me, "zoom[0].h.sy");
    p_xv->zoom[0].skip         = (*get)(p_me, "zoom[0].skip");
}


/*--------------------------------------------------------------------------*
 |"m_Rsrc_store_to_file"
 |
 |  Store the contents of a resource object in a file.
 |
 |    = 1 on success, 0 on failure
 *--------------------------------------------------------------------------*/
static int m_Rsrc_store_to_file( p_me, fd )
    I_Rsrc*     p_me;   /* -> resource object to be stored.  */
    FILE*       fd;     /* -> file descriptor to read from.  */
{
    int     rc; /* Return code from symbol table store. */

    if ( fd )
    {
        return O_SST_store(p_me->p_st, fd);
    }
    else if ( !(fd = fopen("/usr/lpp/parallax/etc/plxv0", "w")) )
    {
		ERRORF((fd, 
			"O_Rsrc_store: Could not open file /usr/lpp/parallax/etc/plxv0\n"));
        return 0;
    }
    else
    {   
        rc = O_SST_store(p_me->p_st, fd);
        fclose(fd);
        return rc;
    }
}

 
/*---------------------------------------------------------------------------*
 |"O_Rsrc_O_Rsrc"
 |
 |  Create and return a resource object.
 |
 |    -> resource object on success, 0 on failure
 *---------------------------------------------------------------------------*/
O_Rsrc *O_Rsrc_O_Rsrc()
{
    I_Rsrc*     p_me;  		/* Symbol table creating. */

	p_me = (I_Rsrc *)O_Rsrc_Generic(256);

	p_me->base.atv->defaults          = m_Rsrc_defaults;
	p_me->base.atv->load_from_context = m_Rsrc_load_from_context;
	p_me->base.atv->load_from_file    = m_Rsrc_load_from_file;
	p_me->base.atv->store_to_context  = m_Rsrc_store_to_context;
	p_me->base.atv->store_to_file     = m_Rsrc_store_to_file;


	m_Rsrc_defaults(p_me);

	return (O_Rsrc *)p_me;
}

