/*
 *  objects_pv700.h: Object definitions for PowerVideo 700.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     25 Sep 94 - Andy Goodrich
 *       - Initial coding
 *
 *  Copyright(c) 1994, Parallax Graphics, Inc.
 */
#ifndef  objects_pv700_h_INCLUDED
#define  objects_pv700_h_INCLUDED

#include "objects.h"		/* Generic hardware library object definitions. */
#include "video_stream.h"	/* Hardware definitions. */
#include "hw_context.h"		/* Hardware operation object. */

/*
 *  INPUT OBJECT DEFINITION:
 */
typedef struct _O_Input_pv700 { 
	TV_Input*	atv;			/* Methods: must be the first entry!! */

	int			dcsc_address;		/* Address of dcsc for this input. */
	int			dmsd_address;		/* Address of dmsd for this input. */
	int			inverse_video;		/* = 1 if we have inverse video. */
	int			port;				/* Unit number of this input. */
	XV_struct*	p_xv;				/* Hardware information. */
	char*		values_now;			/* Current dmsd values in use. */
	char 		values_ntsc[0x19];	/* Values for ntsc video. */
	char 		values_pal[0x19];	/* Values for pal video. */
	char 		values_secam[0x19];	/* Values for secam video. */
} O_Input_pv700;

O_Input* O_Input_pv700_Create(	/* Create a PowerVideo 700 input object:
	int			unit,			// =  unit # of this input.
	Xv_Struct*	p_xv			// -> xvideo object. */
);


/*
 *  MAIN ACCESS TRANSFER VECTOR DEFINITION:
 */
#define O_PV700_ccube_present( p_me ) \
	(*((TV_PV*)((XV_struct*)p_me)->atv)->ccube_present)(p_me)

#define O_PV700_dump( p_me, fd ) \
	(*((TV_PV*)((XV_struct*)p_me)->atv)->dump)(p_me, fd)

#define O_PV700_get_zoom( p_me ) \
	(*((TV_PV*)((XV_struct*)p_me)->atv)->get_zoom)(p_me)

#define O_PV700_init_context( p_me ) \
	(*((TV_PV*)((XV_struct*)p_me)->atv)->init_context)(p_me)

#define O_PV700_init_disp( p_me ) \
	(*((TV_PV*)((XV_struct*)p_me)->atv)->init_display)(p_me)

#define O_PV700_init_pll( p_me, frequency ) \
	(*((TV_PV*)((XV_struct*)p_me)->atv)->init_pll)(p_me, frequency)

#define O_PV700_init_hw( p_me, screen_erase ) \
	(*((TV_PV*)((XV_struct*)p_me)->atv)->init_hw)(p_me, screen_erase)

#define O_PV700_set_delay( p_me, delay ) \
	(*((TV_PV*)((XV_struct*)p_me)->atv)->set_delay)(p_me, delay)

#define O_PV700_set_zoom( p_me, zoom ) \
	(*((TV_PV*)((XV_struct*)p_me)->atv)->set_zoom)(p_me, zoom)

#define O_PV700_terminate( p_me ) \
	(*((TV_PV*)((XV_struct*)p_me)->atv)->terminate)(p_me)

#define O_PV700_vertical( p_me, on_off ) \
	(*((TV_PV*)((XV_struct*)p_me)->atv)->vertical)(p_me, on_off)

#define O_PV700_visibility( p_me, on_off ) \
	(*((TV_PV*)((XV_struct*)p_me)->atv)->visibility)(p_me, on_off)

XV_struct* pv_700_open();


/*
 *  OUTPUT OBJECT DEFINITION:
 */
typedef struct _O_Output_pv700 { 
	TV_Output*	atv;			/* Methods: must be the first entry!! */

	int			dmse_address;		/* Address of dmse for this output. */
	int			port;				/* Unit number of this output. */
	XV_struct*	p_xv;				/* Hardware information. */
	char*		values_now;			/* Current dmse values in use. */
	char 		values_ntsc[0x10];	/* Values for ntsc video. */
	char 		values_pal[0x10];	/* Values for pal video. */
	int			xpoint_value;		/* Value for crosspoint switch. */
} O_Output_pv700;

O_Output* O_Output_pv700_Create(	/* Create a PowerVideo 700 output object:
	int		    unit,				// =  unit # of this output.
	Xv_Struct*	p_xv				// -> xvideo object. */
);

#endif  /*  objects_pv700_h_INCLUDED */
