/*
 *  objects_sun.h: Object definitions for Parallax products on Sun platform.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     29 Dec 94 - Andy Goodrich
 *       - Initial coding
 *
 *  Copyright(c) 1994, Parallax Graphics, Inc.
 */
#ifndef  objects_sun_h_INCLUDED
#define  objects_sun_h_INCLUDED

#include "objects.h"		/* Generic hardware library object definitions. */
#include "video_stream.h"	/* Hardware definitions. */
#include "hw_context.h"		/* Hardware operation object. */

/*
 *  LOOKUP TABLE OBJECT DEFINITIONS:
 */
typedef struct _O_Lut_Bt253 { 
	TV_Lut*		atv;			/* Methods: must be the first entry!! */
	XV_struct*	p_xv;			/* Hardware information. */
} O_Lut_Bt253;

O_Lut* O_Lut_Bt253_O_Lut_Bt253(	/* Create a Brooktree 253 lookup table object:
	int			unit,			   =  unit # of this input.
	Xv_Struct*	p_xv			   -> xvideo object. */
);


#ifdef REMOVE
typedef struct _O_Lut_Bt281 { 
	TV_Lut*		atv;			/* Methods: must be the first entry!! */
	XV_struct*	p_xv;			/* Hardware information. */
} O_Lut_Bt281;
#endif REMOVE

O_Lut* O_Lut_Bt281_O_Lut_Bt281(	/* Create a Brooktree 281 lookup table object:
	int			unit,			   =  unit # of this input.
	Xv_Struct*	p_xv			   -> xvideo object. */
);


typedef struct _O_Lut_Bt463 { 
	TV_Lut*		atv;			/* Methods: must be the first entry!! */
	XV_struct*	p_xv;			/* Hardware information. */
} O_Lut_Bt463;

O_Lut* O_Lut_Bt463_O_Lut_Bt463(	/* Brooktree 463 lookup table object:
	int			unit,			   =  unit # of this input.
	Xv_Struct*	p_xv		       -> xvideo object. */
);


typedef struct _O_Lut_Bt473 { 
	TV_Lut*		atv;			/* Methods: must be the first entry!! */
	XV_struct*	p_xv;			/* Hardware information. */
} O_Lut_Bt473;

O_Lut* O_Lut_Bt473_O_Lut_Bt473(	/* Brooktree 473 lookup table object:
	int			unit,			   =  unit # of this input.
	Xv_Struct*	p_xv		       -> xvideo object. */
);


typedef struct _O_Lut_Bt495 { 
	TV_Lut*		atv;			/* Methods: must be the first entry!! */
	XV_struct*	p_xv;			/* Hardware information. */
} O_Lut_Bt495;

O_Lut* O_Lut_Bt495_O_Lut_Bt495(	/* Brooktree 495 lookup table object:
	int			unit,			   =  unit # of this input.
	Xv_Struct*	p_xv		       -> xvideo object. */
);


/*
 *  INPUT OBJECT DEFINITION:
 */
typedef struct _O_Input_pvsun { 
	TV_Input*	atv;			/* Methods: must be the first entry!! */

	int			dcsc_address;		/* Address of dcsc for this input. */
	int			dmsd_address;		/* Address of dmsd for this input. */
	int			inverse_video;		/* = 1 if we have inverse video. */
	int			port;				/* Unit number of this input. */
	XV_struct*	p_xv;				/* Hardware information. */
	char*		values_now;			/* Current dmsd values in use. */
	char 		values_ntsc[0x19];	/* Values for ntsc video. */
	char 		values_pal[0x19];	/* Values for pal video. */
	char 		values_secam[0x19];	/* Values for secam video. */
} O_Input_pvsun;

O_Input* O_Input_pvsun_Create(	/* Create a PowerVideo 700 input object:
	int			unit,			/* =  unit # of this input.
	Xv_Struct*	p_xv			/* -> xvideo object. */
);


/*
 *  MAIN ACCESS TRANSFER VECTOR DEFINITION:
 */
#define O_PVsun_ccube_present( p_me ) \
	(*((TV_PV*)((XV_struct*)p_me)->atv)->ccube_present)(p_me)

#define O_PVsun_dump( p_me, fd ) \
	(*((TV_PV*)((XV_struct*)p_me)->atv)->dump)(p_me, fd)

#define O_PVsun_get_zoom( p_me ) \
	(*((TV_PV*)((XV_struct*)p_me)->atv)->get_zoom)(p_me)

#define O_PVsun_init_context( p_me ) \
	(*((TV_PV*)((XV_struct*)p_me)->atv)->init_context)(p_me)

#define O_PVsun_init_pll( p_me, frequency ) \
	(*((TV_PV*)((XV_struct*)p_me)->atv)->init_pll)(p_me, frequency)

#define O_PVsun_init_hw( p_me ) \
	(*((TV_PV*)((XV_struct*)p_me)->atv)->init_hw)(p_me)

#define O_PVsun_set_delay( p_me, delay ) \
	(*((TV_PV*)((XV_struct*)p_me)->atv)->set_delay)(p_me, delay)

#define O_PVsun_set_zoom( p_me, zoom ) \
	(*((TV_PV*)((XV_struct*)p_me)->atv)->set_zoom)(p_me, zoom)

#define O_PVsun_vertical( p_me, on_off ) \
	(*((TV_PV*)((XV_struct*)p_me)->atv)->vertical)(p_me, on_off)

#define O_PVsun_visibility( p_me, on_off ) \
	(*((TV_PV*)((XV_struct*)p_me)->atv)->visibility)(p_me, on_off)

#ifdef NOTDEF
typedef struct _TV_PVsun {
	int		(*ccube_present)();		/* = 1 if ccube present, 0 if not. */
	void	(*dump)();				/* Dump contents of this object. */
	int		(*get_zoom)();			/* Get current zoom factor. */
	void	(*init_context)();		/* Initialize context from resource object*/
	void	(*init_hw)();			/* Initialize hardware. */
	void	(*init_pll)();			/* Initialize pll hardware. */
	void	(*set_delay)();			/* Set delay value. */
	int		(*set_zoom)();			/* Set current zoom factor. */
	void	(*terminate)();			/* Terminate use of this object. */
	int		(*vertical)();			/* Enable/disable driver vert. interrupt. */
} TV_PVsun;
#endif

XV_struct* pv_sun_open();


/*
 *  OUTPUT OBJECT DEFINITION:
 */
typedef struct _O_Output_pvsun { 
	TV_Output*	atv;			/* Methods: must be the first entry!! */

	int			dmse_address;		/* Address of dmse for this output. */
	int			port;				/* Unit number of this output. */
	XV_struct*	p_xv;				/* Hardware information. */
	char*		values_now;			/* Current dmse values in use. */
	char 		values_ntsc[0x10];	/* Values for ntsc video. */
	char 		values_pal[0x10];	/* Values for pal video. */
	int			xpoint_value;		/* Value for crosspoint switch. */
} O_Output_pvsun;

O_Output* O_Output_pvsun_Create(	/* Create a PowerVideo 700 output object:
	int		    unit,				/* =  unit # of this output.
	Xv_Struct*	p_xv				/* -> xvideo object. */
);

XV_struct*	O_HWL_O_HWL();

#endif  /*  objects_sun_h_INCLUDED */
