/*
** File:
**
**   xvdebug.h --- Macros to produce debugging and error messages for xv.
**                 Messages are written to the file /usr/local/xv/xv_log.
**                 Debugging may be turned off at compile time by failing
**                 to define XV_DIAG. Note that changing the value of 
**                 XV_DIAG may be used to provide up to 32 different
**                 debugging message enables.
**
** Author:
**
**   Andy Goodrich (RasterOps)
**
** Revisions:
**
**   08.08.92 Andy Goodrich
**     - Initial Definition.
** $Log:   /plxhp/dave/pvcsrc/include/archives/xvdebug.h_v  $
 * 
 *    Rev 1.5   09 Sep 1996 10:36:24   dave
 * Andy Goodrich: Added TRACEF macro options to look at XV_DIAG in
 *                addition to xv_trace_flags.
** Revision 1.5  1995/12/08 13:14:48  dist
** Andy Goodrich: Added TRACEF macro options to look at XV_DIAG in
**                addition to xv_trace_flags.
**
 * Revision 1.4  95/09/14  12:51:54  12:51:54  dist ()
 * Andy Goodrich: Added TRACEF run-time diagnostics macro and flag to
 *                log RCS changes.
 * 
**
*/

#ifndef xvdebug_INCLUDED
#define xvdebug_INCLUDED
/*
 *  DEBUGGING AND ERROR MACROS
 */
#ifdef XV_DIAG
static int	debug_temp;		/* Temporary for stuff. */
#   define DEBUGF( level, args ) \
    {\
        if ( level & XV_DIAG ) \
        { \
            FILE *fd;  \
            fd = stderr; \
            fprintf args; \
            fd = fopen("/tmp/xv_log", "a+");\
            if ( fd ) fprintf args ;\
            fclose(fd); \
        } \
    }
#   define ERRORF( args ) DEBUGF( 0xFFFFFFFF, args )
#   define TRACEF( level, args ) \
    {\
	    extern int	xv_trace_flags; \
	    if ( (level & xv_trace_flags) || (level & XV_DIAG) ) \
	    { \
		    FILE *fd;  \
		    fd = stderr; \
		    fprintf args; \
		    fd = fopen("/tmp/xv_log", "a+");\
		    if ( fd ) fprintf args ;\
		    fclose(fd); \
	    } \
    }
#else  /* !XV_DIAG */
#   define DEBUGF( level, args )
#   define ERRORF( args ) \
    { \
		FILE *fd;  \
		fd = stderr; \
		fprintf args; \
		fd = fopen("/tmp/xv_log", "a+");\
		if ( fd ) fprintf args ;\
		fclose(fd); \
    }
#    define TRACEF( level, args ) \
    {\
	    extern int	xv_trace_flags; \
	    if ( level & xv_trace_flags ) \
	    { \
		    FILE *fd;  \
		    fd = stderr; \
		    fprintf args; \
		    fd = fopen("/tmp/xv_log", "a+");\
		    if ( fd ) fprintf args ;\
		    fclose(fd); \
	    } \
    }
#endif /* XV_DIAG */

#define DEFINE_TRACEF( value ) \
	int xv_trace_flags = value;

#define SET_TRACEF( value ) \
{ \
	extern int	xv_trace_flags; \
	xv_trace_flags = value; \
}


#endif  /* xvdebug_INCLUDED */
