/*
 * Copyright (c) 1993 Parallax Graphics Inc.
 */

#ifndef _TVTWO_H_
#define _TVTWO_H_

#pragma ident "@(#)ccube.h	1.6 Copyright (c) 1993 Parallax Graphics Inc"

#include <sys/t_lock.h>
#include <sys/types.h>
#include <sys/kmem.h>
#include <sys/stat.h>           /* S_IFCHR */
#include <sys/conf.h>
#include <sys/ddi.h>
#include <sys/sunddi.h>
#include <sys/dditypes.h>
#include <sys/stream.h>
#include <sys/stropts.h>
#include <sys/modctl.h>
#include <sys/cmn_err.h>

#include "xvregs.h"
#include "nvram.h"
#include "c3regs.h"

/* intr values for bmask  */
#define	_Int_Dma	0x0001
#define	_Int_Timer	0x0002

/* intr values for imask */
#define	_Int_Display	0x0001
#define	_Int_CCube	0x0002
#define	_Int_Ext	0x0004
#define	_Int_Vid	0x0008
#define	_Int_Wid	0x0010
#define	_Int_None1	0x0020
#define	_Int_None2	0x0040
#define	_Int_Blit	0x0080

#define TVTWO_PROMID "PGI,tvtwo"

#ifdef DEBUG
static tvtwo_debug = 1;
#define	_STMT(a) a
#define	DEBUGF(args) _STMT(if (tvtwo_debug) cmn_err args)
#else
#define	DEBUGF(args)
#endif

extern void *state_p;

/*
 * Macros to turn devi/dev arguments into softc pointers and
 * derive instance numbers from minor.
 *
 *	Minor		Function
 *	---------	---------
 *	0 (000)		tvtwo
 *	1 (001)		nvram
 *	2 (010)		option card nvram
 *	3 (011)		tuner (?)
 *	4 (100)		ccube
 *	5 (101)		cgthree
 */
#define TVTWO_MINOR_SHIFT	(3)
#define TVTWO_MINOR_MASK	((1<<TVTWO_MINOR_SHIFT)-1)
#define TVTWO_TV2_MINOR		(0)
#define TVTWO_NVRAM_MINOR	(1)
#define TVTWO_OPT_MINOR		(2)
#define TVTWO_TUNER_MINOR	(3)
#define TVTWO_C3_MINOR		(4)
#define TVTWO_CG3_MINOR		(5)

#define DST_IDLE 	20
#define DST_READY 	21
#define DST_START 	22
#define DST_BUSY 	23
#define DST_IOLATE 	24
#define DST_IOLATE_LATE 25
#define DST_FIELD_DONE  26

#define TVTWO_INST2DEV(inst)	((inst)<<TVTWO_MINOR_SHIFT)
#define	TVTWO_DEV2INST(m)	(getminor(m) >> TVTWO_MINOR_SHIFT)
#define TVTWO_DEV2MINOR(m)	(getminor(m) & TVTWO_MINOR_MASK)

#define TVTWO_INST2MINOR_TV2(inst)	(TVTWO_INST2DEV(inst)+TVTWO_TV2_MINOR)
#define TVTWO_INST2MINOR_C3(inst)	(TVTWO_INST2DEV(inst)+TVTWO_C3_MINOR)
#define TVTWO_INST2MINOR_NVRAM(inst)	(TVTWO_INST2DEV(inst)+TVTWO_NVRAM_MINOR)
#define TVTWO_INST2MINOR_OPT(inst)	(TVTWO_INST2DEV(inst)+TVTWO_OPT_MINOR)
#define TVTWO_INST2MINOR_TUNER(inst)	(TVTWO_INST2DEV(inst)+TVTWO_TUNER_MINOR)
#define TVTWO_INST2MINOR_CG3(inst)	(TVTWO_INST2DEV(inst)+TVTWO_CG3_MINOR)

#define	TVTWO_DEVI2SOFTC(devi) \
	((ccube_softc *)ddi_get_soft_state(state_p, ddi_get_instance(devi)))
#define	TVTWO_DEV2SOFTC(dev) \
	((ccube_softc *)ddi_get_soft_state(state_p, TVTWO_DEV2INST((dev))))

/*
 * Following is mainly for ddi_map_regs(9F)
 */

#define TVTWO_UNMAP_REGS(softc, reg, ptr) \
	ddi_unmap_regs((softc)->devi, \
		(reg).regnum, \
		(caddr_t *)&(ptr), \
		(reg).offset, \
		(reg).size \
		)
#define TVTWO_MAP_REGS(softc, reg, ptr) \
	ddi_map_regs((softc)->devi, \
		(reg).regnum, \
		(caddr_t *)&(ptr), \
		(reg).offset, \
		(reg).size \
		)

typedef struct {
	u_int regnum;
	u_int base;	/* for mmap - base offset from device address 0 */
	off_t offset;
	off_t size;
	off_t voff;	/* for mmap - request address from userland */
} XV_regmap;

/*
 * This structure contains all our "virtual" register windows for
 * the XVideo board.
 */
typedef struct {
	XV_regmap xv;
	XV_regmap c3;
	XV_regmap m1;
	XV_regmap m8;
	XV_regmap m24;
	XV_regmap prom;
} XV_regmaps;

/*
 * TVTWO address map
 *
 * The address space presented to the user via mmap.
 *
 *	0x000000	1 bit framebuffer
 *	0x040000	8 bit framebuffer
 *	0x140000	true color framebuffer
 *	0x801000	prom
 *	0x800000	XVideo registers
 *	0x811000	CCube registers
 */

#define M1BASE		((int)(((XV_mem_map *)0)->M1))
#define M1SIZE		(0x28000)
#define M1VOFF		(0x0)

#define XVBASE		((int)(&((XV_mem_map *)0)->RW_regs))
#define XVSIZE		(sizeof(XV_hw_regs))
#define XVVOFF		(0x800000)

#define C3BASE		((int)(&((XV_mem_map *)0)->Ccube))
#define C3SIZE		(sizeof(C3_regs))
#define C3VOFF		(0x811000)

#define M8BASE		((int)(((XV_mem_map *)0)->M8))
#define M8SIZE		(0x100000)
#define M8VOFF		(0x40000)
#define CG2M8VOFF	(0x100000) /* for cgtwo emulation */

#define M24BASE		((int)(((XV_mem_map *)0)->M24))
#define M24SIZE		(0x400000)
#define M24VOFF		(0x140000)

#define PROMBASE	((int)(((XV_mem_map *)0)->Prom1))
#define PROMSIZE	(0x10000)
#define PROMVOFF	(0x801000)

/*
 * Old prom (w/override from tvtwo.conf) defines.
 */
#define XVREGNUM 0
#define XVREGOFF 0
#define XVREGSIZE XVSIZE
#define C3REGNUM 0
#define C3REGOFF (C3BASE-XVBASE)
#define C3REGSIZE C3SIZE
#define M1REGNUM 1
#define M1REGOFF 0
#define M1REGSIZE M1SIZE
#define M8REGNUM 0
#define M8REGOFF (M8BASE-XVBASE)
#define M8REGSIZE M8SIZE
#define M24REGNUM 0
#define M24REGOFF (M24BASE-XVBASE)
#define M24REGSIZE M24SIZE
#define PROMREGNUM 0
#define PROMREGOFF (PROMBASE-XVBASE)
#define PROMREGSIZE PROMSIZE
#define HI_COMP 127


typedef struct ccube_info {
	int		unit;		/* unit number */
	int 		state;		/* CCUBE_STATE_??? */
	int		timer_count;	/* to deal with intr's lost */
	int		timer_active;	/* to deal with intr's lost */
	int		timer_required;	/* flag for timer usage */
	int		mode;		/* CCUBE_MODE_{READ|WRITE} */
	u_long 		imask;		/* intr' mask before and after call */
	u_long		bmask;		/* intr' mask before and after call */
	u_long  	istat;		/* intr' status before call */
	u_long		bstat;		/* intr' status before call */
	caddr_t		start_addr;	/* start of xfer address */
	caddr_t		end_addr;	/* end of xfer address */
	unsigned long	count;		/* requested count */
	unsigned long	rcount;		/* actual count */
}CCUBE_INFO;


/*
 * XVideo board registers
 *
 * Old xvideo prom registers:
 *
 *	R0	40000 - 40800
 *	R1	80000 - 17d200
 *
 * New xvideo prom registers
 *
 *	R0	0 - 7fffff
 *
 * Old registers overridden by tvtwo.conf to become:
 *
 *	R0	40000 - 7fffff
 *	R1	10000 -  37fff
 *
 * In order for the OS to read the register override the first
 * register offset must match the reg offset in the prom (see sbus(4)).
 *
 *	registers=-1,0x40000,0x7c0000, -1,0x10000,0x28000;  
 */

typedef struct {
	u_char		Prom0[PROMSIZE];		/*      0 -   ffff */
	u_char		M1[M1SIZE];			/*  10000 -  37fff */
	u_char		unused0[0x8000];		/*  38000 -  3ffff */
	XV_hw_regs	RW_regs;			/*  40000 -  404d0 */
	u_char		unused1[0x10000 - XVSIZE];	/*  404d4 -  4ffff */
	C3_regs		Ccube;				/*  50000 -  5ffff */
	u_char		unused2[0x30000 - C3SIZE];	/*  60000 -  7ffff */
	u_char		M8[M8SIZE];			/*  80000 - 17ffff */
	u_char		unused3[0x80000];		/* 180000 - 1fffff */
	u_long		M24[M24SIZE/sizeof(u_long)];	/* 200000 - 6fffff */
	u_char		unused4[0xf0000];		/* 700000 - 7effff */
	u_char		Prom1[PROMSIZE];		/* 7f0000 - 7fffff */
} XV_mem_map;

/*
 * Per-instance data
 */
typedef struct {

	/*
	 * Our mapped registers
	 */
	off_t reg_offset;		/* offset to get to registers */
	off_t reg_size;			/* sizeof registers */

	/*
	 * Mutex and condition variables
	 */
	kmutex_t regs_mutex;		/* mutex for ... */
	XV_regs *regs;

	kmutex_t softc_mutex;		/* mutex for this structure */

	/*
	 * Cookies.
	 */
	ddi_iblock_cookie_t iblkc;	/* for blocking intrs */
	ddi_idevice_cookie_t idevc;	/* for interrupt vector information */
	ddi_dma_handle_t ccube_dma;	/* for our dma handle */

	/*
	 * Information pointers
	 */
	XV_regmaps *regmap;		/* how to map our registers */
	dev_info_t *devi;		/* for dev-to-devi conversion */
	u_int maxphys;			/* for tvtwo_minphys */

	struct buf *bp;			/* buf reference for tvtwo_intr */
	caddr_t addr;			/* phys dma transfer address */

	kmutex_t dma_lock;		/* mutex for ... */
	kcondvar_t dmadone;		/* dma complete/error */

	/*
	 * DMA memory management pointers 
	 * and counters.
	 */
        caddr_t dma_base; 		/* Hardware dma base address */
	caddr_t dma_addr; 		/* Current  dma transfer address */
	/* 
	 * Base address and current pointer for Virtual JPEG  data buffer.
	 */
	caddr_t bufr_base;		
	caddr_t next_bufr;
	int	bufr_size;
	int     request_size;
	int     actual_size;
	int     field_offset;
	u_int   dma_size;
	kmutex_t ccube_lock;
	u_long bstat, istat;		/* saved interrupt status bits */
	u_long bmask, imask;		/* saved interrupt mask bits */

	/*
	 * Timout handles
	 */
	int tid;			/* timeout identifier */
	int timeout;			/* flag that tid is active */

	int start_video;		/* flag set by I_START message */
	int v_chan_state;		/* Vid Channel state */
	int w_chan_state;		/* Wid Channel state */
	int ccube_event;		/* Current event */
	int ccube_state;
	int ccube_error;
	int dma_state;
	int rw_flag;
	int Vidcount;
	int record_size;
	int field_size;

	/*
	 * Configuration information.
	 */
	XV_nvram nvram;
	/*
	 * Queue pointers for ccube, network, rtvmux.
	 */
	 queue_t *rtvmux_queue;
	 queue_t *ccube_queue;
	 queue_t *net_queue;
	 CCUBE_INFO *ci;
} ccube_softc;

extern int
ccube_init(ccube_softc *softc);

extern void
ccube_uninit(ccube_softc *softc);

extern int
ccube_reinit(ccube_softc *softc);

extern int
ccube_nvram_init(XV_regs *regs, XV_nvram *nvram);

extern int
ccube_nv_write(XV_regs *regs, dev_t dev, paddr_t prt, int nv_addr, int count);

extern int
ccube_nv_read(XV_regs *regs, dev_t dev, paddr_t ptr, int nv_addr, int count);

extern void
c3_int_svc(ccube_softc *softc);

extern void
c3_load_dct(ccube_softc *softc);

extern void
ccube_service(ccube_softc *softc, void (*[])(), u_int bits);


extern void
c3_init(ccube_softc *softc);

#endif
