/*
 * Copyright (c) 1993 Parallax Graphics Inc.
 */

#ifndef _NVRAM_H_
#define _NVRAM_H_

#pragma ident "@(#)nvram.h	1.1 Copyright (c) 1993 Parallax Graphics Inc"

/*
 * Driver structures for on-board devices of the XVideo board
 * and it's option cards.  This is mainly to define the NVRAM
 * configuration.
 */

#define SBUS_1 4	/* 66hz */
#define SBUS_2 6	/* 76hz */
#define SBUS_3 7	/* 60hz */
#define SBUS_4 8	/* SQUASH */
#define SBUS_5 9	/* PERVIS */
#define OPT_1 10	/* VIO */
#define OPT_2 11	/* RGB */
#define OPT_3 12	/* SONYRGB */
#define OPT_4 13	/* SONYRGB2 */

#define DEFAULT_BD SBUS1
#define DEFAULT_OPT OPT1

#define C3REV_1 1

#define HLIB_REV 0x8008

/* defines for the various clt's being used on XVideo */
#define BT463 1
#define BT462 2
#define BT473 3

typedef enum {SBUS1=0, SBUS2=1, SBUS3=2} nvram_bdtype;

typedef enum {OPT1=1, OPT2=1, OPT3=2, OPT4=3} nvram_opttype;

/*
 * Brooktree configurations for 261 and 253.
 */
typedef struct {
	u_char	command[4];
	u_short	fgate_start;
	u_short	fgate_stop;
	u_char	ngate_start;
	u_char	ngate_stop;
	u_char	hsync_start;
	u_char	clamp_start;
	u_char	clamp_stop;
	u_long	:24;
} XV_bt261_config;
	
typedef struct {
	u_short	hcount;
	u_short	hactive;
	u_char	zero_stop;
	u_long	:24;
} XV_bt261_init;

typedef struct {
	u_char	val[6];
	u_short	:16;
} XV_bt253_config;

/*
 * data structure to define the format of the RGB_IN interface
 * this includes the Xpoint config_bits needed to set up the hardware.
 */
typedef struct {
	u_long	xpoint_bits;		/* Xpoint config value */
	XV_bt261_config	timing;
	XV_bt261_init	format[2];
	XV_bt253_config	cal[2];
} XV_rgbin_config;

#define GAREV_1 1       /* BIC: rev 1  CTC: rev 1 PPC: rev 1 */
#define GAREV_2 2       /* BIC: rev 2  CTC: rev 2 PPC: rev 1 */

typedef struct {
	u_char	rev;		/* rev of the dmsd chipset	*/
	u_char	unit;		/* which "unit" numbe		*/
	u_char	val[26];	/* location 0:22 are 7191	*/
} XV_dmsd_config;		/* locations 23:25 unused	*/

/*
 * data structure to define the format of a general interface
 * this includes the Xpoint config_bits needed to set up the hardware
 */
typedef struct {
	u_long	xpoint_bits;		/* Xpoint config value */
	u_char	val[28];
} XV_gen_config;

typedef struct {
	short	active_pixels;	/* per line */
	short	active_lines;	/* per frame */
	short	blank_lines;	/* per frame */
	short	reset_line;	/* in a field */
	short	start_line;	/* in a field */
	short	:16;
} XV_video_format;

typedef struct {
	u_char	bright;
	u_char	contrast;
	u_char	saturation;
	u_char	hue;
} XV_knob_rec;

/*
 * defines for the "flags" field of the io_port sub-structure
 */
#define IO_CHAN_TYPE 0x3f
#define IO_DMSD      01
#define IO_RGB_IN    02
#define IO_GEN_IN    03
#define IO_SONYRGB_IN 04
#define IO_OUTPUT    0x20
#define IO_COMP_OUT  0x21
#define IO_RGB_OUT   0x22
#define IO_GEN_OUT   0x23

/*
 * structure to define general interface for A and B ports
 * port can be either DMSD, or general purpose		  
 */
typedef struct {
	u_long		flags;			/* see IO_XXXX constants     */
	XV_knob_rec	factory,u1,u2;		/* 3 sets of users adjusts */
	XV_video_format	format[2];
	union  {
		    XV_gen_config	gen;
		    XV_dmsd_config	dmsd;
		    XV_rgbin_config	rgbin;
	} config;
} XV_io_port;

typedef struct {
	u_short	disp;
	u_char	fp;
	u_char	sy;
	u_char	bp;
	u_char	hsync_ser;		/* disp param for serration  */
	u_short	hsync_nser;		/* disp param for serration  */
} XV_disp_format;

/*
 * the full data structure for the CONTROLLER NVRAM
 */

typedef struct {
	u_char	board_code;		/* the PARALLAX HARDWARE NUM */
	u_char	ga_rev;			/* a code for which chip set */
	u_char	c3_rev;			/* a code for CL550 revision */
	u_char	clt_type;		/* what type of output stage */
	u_long	serial_number;
	u_short	hlib_rev;
	u_short	:16;
	u_long	power_hours;		/* hours of operation	     */
	u_short	power_cycles;		/* number of power ups	     */
	u_short	xsize;			/* Pixels in a line	     */
	u_short	ysize;			/* lines on display	     */
	u_short	Attr;			/* another startup value     */
	u_short	DispKludge;		/* a startup value	     */
	u_short	DispFlgs;		/* Stuff value at power up   */
	u_long	StartUp;		/* Stuff value at power up   */
	u_short	TmrInit;		/* Stuff value at power up   */
	u_short	RasMode;		/* Stuff value at power up   */
	u_char	OLDI2C;			/* Stuff value at power up   */
	u_long	:24;
	XV_disp_format	h,v;			/* general display timing     */
	XV_io_port	d;			/* DMSD video adjustments    */
	u_char	unused_2[10];
} XV_controller_nvram;

/*
 * the full data structure for the OPTION NVRAM
 */

typedef struct {
	u_char	board_code;		/* the PARALLAX HARDWARE NUM */
	u_char	board_rev;		/* board revision code	     */
	u_char	clt_type;		/* what type of output stage */
	u_char	:8;
	u_long	serial_number;
	u_short	hlib_rev;
	u_short	:16;
	u_long	power_hours;		/* hours of operation	     */
	u_short	power_cycles;		/* number of power ups	     */
	u_short	:16;
	XV_io_port	a,b;
	u_char	unused[10];
} XV_option_nvram;

/*
 * Currently we only maintain two configurations.  The
 * main XVideo board nvram configuration, and the option
 * board configuration.
 */
typedef struct {
	XV_controller_nvram main;
	XV_option_nvram opt;
} XV_nvram;

#endif
