/*
 * Copyright (c) 1993 Parallax Graphics Inc.
 */

#ifndef _XVREGS_H_
#define _XVREGS_H_

#pragma ident "@(#)xvregs.h	1.3 Copyright (c) 1993 Parallax Graphics Inc"

/******************************************************************************
 * 
 * The following are the XVideo Macro definitions.  These macros should be used
 * to perform all accesses to the XVideo hardware.  They automatically handle
 * updating a shadow RAM and ensure that the device registers are always 
 * accessed as longwords.
 *
 *****************************************************************************/
/* for sun machines ,BITI is true, for DEC(little indian) its false */

#ifndef CL_550
#include "c3regs.h"
#include "c3shdw.h"
#endif

#ifdef sun
#define BITI 
#endif

/*****************************************************************************
 *
 * StartUp	STARTUP REGISTER.
 * SU		Load startup information here.  Determines addressing mode,
 * su		bit/byte/addr indians, whether display is 1152 pixels wide
 * 		or is externally generated, resets video and memory timing.
 * 0000		StartUp 
 ****************************************************************************/
	/*
	 * <1:0>	amode	ADDRESSING MODE. 
	 *			0: Only address space 0-3f0000 is avail. on bus
	 *			1: SEL#2 selects the low bank (SEL#1: hi bank)
	 *			2: SEL#1 selects the low bank (SEL#2: hi bank)
	 *			3: Use the address line on bus to select bank.
	 *
	 * <2>		il5	INTERLEAVE. 
	 *			0: Interleave of 4.
	 *			1: Interleave of 5.
	 *
	 * <3>		d1152	DISPLAY 1152 PIXELS HORIZONTALLY.
	 * 			0: 1280 or 1024 pixels horiz.
	 *			1: 1152 pixels horiz.
	 *
	 * <4>		mreset	RESET MEMORY TIMING.	
	 *			0: Normal Operation.
	 *			1: Reset memory timing. Must be held for 2 us.
	 * <5:6:7)	mkludge1
	 *			bit 5 oe stretch
	 *			bit 6 cas stretch
	 *			bit 7 extra cycles on Pmode reads
	 *
 	 * <8>		ASIC test bit: leave zero
	 *
	 * <9>		biti	BIT INDIAN.
	 *			0: Bit '0' on left. (big indian)
	 *			1: Bit '0' on right.
	 * 
	 * <10>		byti	BYTE INDIAN.
	 *			0: Low order byte on left.
	 *			1: High order byte on left.
	 * 
 	 * <11>		addi	ADDRESS INDIAN.
	 *			0: Lowest address refers to low order byte
	 * 			1: Highest address refers to low order byte.
	 * <12:15>	mkludge2
	 *			12 - direct;ly delay busy on map reads
	 *			13 - shorter read map to busy delay
	 *			14 - AND off drawctl <3:1> on drawctl.0
	 *			15 - stretch mem_sample
	 * <16:17>	opti	OPTION INDIAN
	 *			byte order of option byte order of lwd
	 *			0: LSB(0)...MSB(3)
	 *			3: MSB(3)...LSB(0)
	 *			1: 1,0,3,2
	 *			2: 2,3,0,1
	 * <18>		dma_k1	for dma losses, crashes.
	 *			
	 * <27:28:29>	mkludge3
	 *			27 - don't fast repeat byte reads
	 *			28 - DMA data catch, minor
	 *			29 - DMA data catch, major
	 * <31>		slow
	 *			Nubus implementation
	 */


#ifdef BITI

	typedef struct {
		unsigned slow:		1;
		unsigned :		1;
		unsigned mkludge3:	3;
		unsigned :		8;
		unsigned dma_k1:	1;
		unsigned opti:		2;
		unsigned mkludge2:	4;
		unsigned addi:		1;
		unsigned byti:		1;
		unsigned biti:		1;	
		unsigned :		1;	
		unsigned mkludge1:	3;
		unsigned mreset:	1;	
		unsigned d1152:		1;	
		unsigned il5:		1;
		unsigned amode:		2;	
	} StartUp_bits;
#else
	typedef struct {
		unsigned amode:		2;	
		unsigned il5:		1;
		unsigned d1152:		1;	
		unsigned mreset:	1;	
		unsigned mkludge1:	3;
		unsigned :		1;	
		unsigned biti:		1;	
		unsigned byti:		1;
		unsigned addi:		1;
		unsigned mkludge2:	4;
		unsigned opti:		2;
		unsigned dma_k1:	1;
		unsigned :		8;
		unsigned mkludge3:	3;
		unsigned :		1;
		unsigned slow:		1;
	} StartUp_bits;

#endif
	typedef union {
		unsigned amode;	
		unsigned il5;
		unsigned d1152;
		unsigned mreset;
		unsigned mkludge1;
		unsigned biti;
		unsigned byti;
		unsigned addi;
		unsigned mkludge2;
		unsigned mkludge3;
		unsigned slow;
	} StartUp_lwd;


/****************************************************************************
*
* Attr		ATTRIBUTE PIN ASSIGNMENT
* 		Configuration for the Attr pins - what they do.
* 00cc		Attr
****************************************************************************/

	/*  <1:0>	attr0 pin usage
	 *			0 - Video Output Attribute 0
	 *			1 - Alternate Ras
	 *			2 - Extra video address pin
	 *			3 - ASIC PTO (no use)
	 *  <3:2>	attr1 pin useage
	 *			0 - Video Output Attribute 1
	 *			1 - Alternate Ras
	 *			2 - Extra video address pin
	 *  <5:4>	attr2 pin useage (called CC_STALL)
	 *			0 - CCube Stall output
	 *			1 - Alternate Ras
	 *			2 - EXtra video address pin
	 * 			3 - Input for VID_Stensil,I2c clock,dvid_odd
	 * <7:6>	attr3 pin (called CC_FRMEND)
	 *			0 - CCube Frame end input
	 *			1 - Alternate Ras
	 *			2 - EXtra video address pin
	 * 			3 - Input for VID_Stensil,I2c clock,dvid_odd
	 * <9:8>	vidv (video v shadow stencil input select)
	 *			0 - none, all pixel draw
	 *			1 - ATTR1 pin
	 *			2 - CC_FRMEND pin
	 *			3 - ATTR5 pin
	 * <11:10>	vidw (video w shadow stencil input select)
	 *			0 - none, all pixel draw
	 *			1 - ATTR1 pin
	 *			2 - CC_FRMEND pin
	 *			3 - ATTR5 pin
	 * <12>		i2cr (i2c readback the clock pin)
	 *			0 - ATTR5 pin
	 *			1 - CC_STALL pin
	 * <15:14>	d_odd (where does dvid_odd come from)
	 *			0 - DVID Hysnc
	 *			1 - ATTR5
	 *			2 - CC_FRMEND
	 *			3 - CC_STALL
	 ****************************************************************/


#ifdef BITI
	typedef struct {
		unsigned :		16;
		unsigned d_odd:		2;
		unsigned :		1;
		unsigned i2cr:		1;
		unsigned vidw:		2;
		unsigned vidv:		2;
		unsigned attr3:		2;
		unsigned attr2:		2;
		unsigned attr1:		2;
		unsigned attr0:		2;
	} Attr_bits;
#else
	typedef struct {
		unsigned attr0:		2;
		unsigned attr1:		2;
		unsigned attr2:		2;
		unsigned attr3:		2;
		unsigned vidv:		2;
		unsigned vidw:		2;
		unsigned i2cr:		1;
		unsigned :		1;
		unsigned d_odd:		2;
		unsigned :		16;
	} Attr_bits;
#endif
	typedef union {
		unsigned attr0;
		unsigned attr1;
		unsigned attr2;
		unsigned attr3;
		unsigned vidv;
		unsigned vidw;
		unsigned i2cr;
		unsigned d_odd;
	} Attr_lwd;


/****************************************************************************
* RasMode	Hook register for extra memories 			    *
* 00d0	 	RasMode							    *
****************************************************************************/

	/*	<0>	Vram Disp & Tag Disp (must be 1)
	 *	<1>	Vram Video V
	 *	<2>	Vram Drawing (including maps)
	 *	<3> 	Vram Video W
	 *	<4>	Vram refresh and tag (must be 1)
	 * 	<5>	Vram source blit
	 *	<6>	Dram Dispaly (should always be 0)
	 *	<7>	Dram Video V
	 *	<8>	Dram Drawing
	 *	<9>	Dram Video W
	 *	<10>	Dram refresh (must be 1 ?? if dram installed)
	 *	<11>	Dram Blit source
	 *	<12>	Tag V
	 *	<13>	Tag Draw
	 *	<14>	Tag W
	 *	<15>	Tag SRC
	 */


/****************************************************************************
* MBank		Which offscreen memory bank to use
* 		(only meaningful if Dram is 1mx4's creating 4 banks)
* 00a4		MBank
****************************************************************************/

	/*	<1:0>	disp.... which of 4
	 *	<3:2>	vvid
	 *	<5:4>	draw
	 *	<7:6>	wvid
	 *	<9:8> 	src for blits
	 */



/****************************************************************************
 * 
 * HFP		HORIZONTAL FRONT PORCH (-1).
 * hfp		Load with the desired length of the horizontal front porch -1.
 * 		Units are DCLKS (dotclock/interleave).
 * 0080		HFP
 ***************************************************************************/

/****************************************************************************
 * 
 * HSync	HORIZONTAL SYNC TIME (-1).
 * HS		Load with the desired length of horizontal sync -1.
 * hs		Units are DCLKS (dotclock/interleave).
 * 
 * 0084		HSync 
 ****************************************************************************/

/****************************************************************************
 *
 * HBP		HORIZONTAL BACK PORCH (-1).
 * hbp		Load with the desired length of the horizontal back porch -1.
 * 		Units are DCLKS (dotclock/interleave).
 * 0088		HBP
 ***************************************************************************/

/****************************************************************************
 *
 * HDisp	HORIZONTAL DISPLAY TIME (-1).
 * HD		Load with the desired length of the horizontal active line (-1).
 * hd		Units are DCLKS (dotclock/interleave). Must be 0x11f for Sun
 * 		(1152x900) modes.
 * 008c		HDisp
 ***************************************************************************/


/****************************************************************************
 *
 * VFP		VERTICAL FRONT PORCH (-1).
 * vfp		Load with the desired length of the vertical front porch -1.
 * 		Units are lines.
 * 0090		VFP
 ***************************************************************************/

/****************************************************************************
 *
 * VSync	VERTICAL SYNC TIME (-1).
 * VS		Load with desired length of vertical sync -1. Units are lines.
 * vs
 * 0094		VSync
 ***************************************************************************/

/****************************************************************************
 *
 * VBP		VERTICAL BACK PORCH (-1).
 * vbp		Load with the desired length of the vertical back porch -1.
 * 		Units are lines.
 * 0098		VBP
 ***************************************************************************/

/****************************************************************************
 *
 * VDisp	VERTICAL DISPLAY TIME (-1).
 * VD		Load with the desired number of active display lines (-1).
 * vd		
 * 009C		VDisp
 ***************************************************************************/

/****************************************************************************
 *
 * HSyncS	HORIZONTAL SYNC TIME DURING SERRATED LINES (-1).
 * HSS		Load with the desired length of horizontal sync during 
 * hss		serrated lines.  Units are DCLKS (dotclock/interleave).
 * 00B0		HSyncS
 ***************************************************************************/

/****************************************************************************
 *
 * HNSyncS	HORIZONTAL NON-SYNC TIME DURING SERRATED LINES (-1).
 * HNSS		Load with desired length of non-sync time during serrated
 * hnss		lines.  Units are DCLKS (dotclock/interleave).
 * 		*** set bit 15 to 1 to disable serations (att FP & BP to time)
 * 00b4		HNSyncS
 ***************************************************************************/


/*****************************************************************************
 *
 * DispFlgs	DISPLAY FLAGS.
 * DISF		Sets up various values needed by the display generator, such
 * disf		as warning needed for advanced display cycles, whether the
 * 		display is 1152 pixels across, etc.
 * 00a0		DispFlgs
 ****************************************************************************/

	/*
	 * <4:0>	warn	WARN TIME.
 	 *			Advanced request of display cycle.  Units are
	 *			DCLKS.  Value should be equal to about 13 
	 *			MCLKS.  If too large, will limit video B/W; if
	 *			too small, will create incorrect display.
	 *
	 * <5>		d1152	DISPLAY 1152 PIXELS HORIZONTALLY.
	 *			Set to 1 if display is 1152 pixels across.
	 *
	 * <6>		disall	RELOAD EVERY LINE
	 *			Display cycle every line.  Must be set to 1
	 *			whenever the number of pixels displayed
	 *			horizontally is not 1024,1152 or 1280.  
	 *			Leaves gap in memory map between lines.
	 *
	 * <12:8>	must	MUST TIME.
	 *			Advanced warning to stop a draw operation in
	 * 			memory, to allow FIFOs to drain before a just
	 * 			in time display cycle.  Should be as short as
	 *			possible.  If too short, display bandwidth will
	 * 			be limited severely; if too long, drawing speed
	 * 			will be impacted.
	 *
	 */


#ifdef BITI
	typedef struct {
		unsigned :		19;
		unsigned must:		5;
		unsigned :		1;
		unsigned disall:	1;
		unsigned d1152:		1;
		unsigned warn:		5;
	} DispFlgs_bits;
#else
	typedef struct {
		unsigned warn:		5;
		unsigned d1152:		1;
		unsigned disall:	1;
		unsigned :		1;
		unsigned must:		5;
		unsigned :		0;
	} DispFlgs_bits;
#endif

	typedef union {
		unsigned warn;
		unsigned d1152;
		unsigned disall;
		unsigned must;
	} DispFlgs_lwd;


/****************************************************************************
* DispKludge		Display additions and Kludges & other things
* 00b8			DispKludge
****************************************************************************/

	/*	<3:0>		vert_xor	xor_value for just in time
	 *					 cycle positioning
	 *	<5:4>		oe_delay	delay ticks for oe ( =1)
	 *	<7:6>		blnk_delay	delay ticks for blank ( =0)
	 *	<8>		extern_gen	use external disp generator
	 *	<9>		extern_jit	external gen is a just in time
	 *	<11:10>		blnk2_delay	delay ticks for blank2 ( =1)
	 *	<12>		disp_blank	blank entire display
	 *	<13> ??		disp_sync	<is this blnk the sync pin>
	 *	<14>		c3_int		0 - edge, 1 - level
	 *	<15>		ext_int		0 - edge, 1 - level
	 */
#ifdef BITI
	typedef struct {
		unsigned :		16;
		unsigned ext_int:	1;
		unsigned c3_int:	1;
		unsigned :		1;
		unsigned disp_blank:	1;
		unsigned blnk2_delay:	2;
		unsigned extern_jit:	1;
		unsigned extern_gen:	1;
		unsigned blnk_delay:	2;
		unsigned oe_delay:	2;
		unsigned vert_xor:	4;
	} DispKludge_bits;
#else
	typedef struct {
		unsigned vert_xor:	4;
		unsigned oe_delay:	2;
		unsigned blnk_delay:	2;
		unsigned extern_gen:	1;
		unsigned extern_jit:	1;
		unsigned blnk2_delay:	2;
		unsigned disp_blank:	1;
		unsigned :		1;
		unsigned c3_int:	1;
		unsigned ext_int:	1;
	} DispKludge_bits;
#endif
	typedef union {
		unsigned vert_xor;
		unsigned oe_delay;
		unsigned blnk_delay;
		unsigned extern_gen;
		unsigned extern_jit;
		unsigned blnk2_delay;
		unsigned disp_blank;
		unsigned c3_int;
		unsigned ext_int;
	} DispKludge_lwd;

/****************************************************************************
 *
 * RopCtl	DRAW ROP ENABLE AND TAG CONTROL.
 * RC		Enables drawing raster ops and the tag bits.  Sets the mask
 * rc		and compare bits for the tag.  Determines whether the last 
 * 		pixel should be drawn by vectors (for multiple vects, esp. XOR).
 * 002c		RopCtl
 ****************************************************************************/

	/* <0>		rmw	do read modify write cycles
	 * <2>		atag	do advanced tag reads
	 * <3>		em28	use <27:24> as <31:28>, ??set 27:24 to zero
	 * <4>		nfirst	don't draw first point of ANY vector
	 * <5>		nhorz	Don't use wacks for horozontal vectors
	 */

#ifdef BITI
	typedef struct {
		unsigned :		26;
		unsigned nhorz:		1;
		unsigned nfirst:	1;
		unsigned em28:		1;
		unsigned atag:		1;
		unsigned :		1;
		unsigned rmw:		1;
	} RopCtl_bits;
#else
	typedef struct {
		unsigned rmw:		1;
		unsigned :		1;
		unsigned atag:		1;
		unsigned em28:		1;
		unsigned nfirst:	1;
		unsigned nhorz:		1;
		unsigned :		26;
	} RopCtl_bits;
#endif

	typedef union {
		unsigned rmw;
		unsigned atag;
		unsigned em28;
		unsigned nfirst;
		unsigned nhorz;
	} RopCtl_lwd;


/*****************************************************************************
*
* TagClip	Tag values to use for DRAW clipping
* 00fc		TagClip
*
*****************************************************************************/

	/*	<7:0>		comp	tag compare value
	 *	<15:8>		mask	tag mask value if (readtag & mask) ==
	 *			 	comp, then draw the pixel.
	 */

#ifdef BITI
	typedef struct {
		unsigned :		16;
		unsigned mask:		8;
		unsigned comp:		8;
	} TagClip_bits;
#else
	typedef struct {
		unsigned comp:		8;
		unsigned mask:		8;
		unsigned :		16;
	} TagClip_bits;
#endif

	typedef union {
		unsigned comp;
		unsigned mask;
	} TagClip_lwd;




/*****************************************************************************
*
* TagCtl	Tag Override Mask and Value
* 0028		TagCtl
*
*****************************************************************************/

	/*	<7:0>		ovr_mux	Each bit set here allows a bit of
	 *				OvrVal to be writting to tag planes
	 *	<15:8>		ovr_val	tag mask value if (readtag & mask) ==
	 *			 	comp, then draw the pixel.
	 */

#ifdef BITI
	typedef struct {
		unsigned :		16;
		unsigned ovr_val:	8;
		unsigned ovr_mux:	8;
	} TagCtl_bits;
#else
	typedef struct {
		unsigned ovr_mux:	8;
		unsigned ovr_val:	8;
		unsigned :		16;
	} TagCtl_bits;
#endif

	typedef union {
		unsigned ovr_mux;
		unsigned ovr_val;
	} TagCtl_lwd;


/****************************************************************************
*
* KeyColMask	Video "key" color mask, changes a map color to video
* 		Lets graphics only applications use a video backdrop
* 022c		KeyColMask
*****************************************************************************/

/****************************************************************************
*
* KeyColComp	Video "key" color mask, changes a map color to video
* 		Lets graphics only applications use a video backdrop
* 0230		KeyColComp
*****************************************************************************/

/****************************************************************************
*
* KeyColCtl	Video "key" color mask, changes a map color to video
* 		Lets graphics only applications use a video backdrop
* 0234		KeyColCtl
*****************************************************************************/

	/*	<7:0>		eqtag		if (KeyColMask & draw_pix) ==
	 *					KeyColComp, then draw with
	 *					eqtag value for tags, else
	 *					use neqtag for tags
	 *	<15:8>		neqtag		
	 *	<16>		enab		if set use eq/neq tag, else
	 *					use ibus value (from draw)
	 *					OVERRIDES override this function
	 */

#ifdef BITI
	typedef struct {
		unsigned :		15;
		unsigned enab:		1;
		unsigned neqtag:	8;
		unsigned eqtab:		8;
		} KeyColCtl_bits;
#else
	typedef struct {
		unsigned eqtab:		8;
		unsigned neqtag:	8;
		unsigned enab:		1;
		unsigned :		15;
		} KeyColCtl_bits;
#endif
	typedef union {
		unsigned eqtab;
		unsigned neqtag;
		unsigned enab;
		} KeyColCtl_lwd;


/****************************************************************************
 * 
 * Sema0 - Sema15
 * Semaphor registers 
 * these are 16 1-bit. When read they read the current value and set to one.
 * When written they are cleared.  If any of these registers are written
 * they post and interrupt if the interrupt is enabled at the time of the 
 * write.
 * 0240 - 027c
 ****************************************************************************/


/****************************************************************************
 * 
 * DrawMask	DRAW PLANE MASK.
 * DM		Determines which bitplanes to write when drawing.  Video
 * dm		digitizing has a separate mask. 1=draw corresponding bitplane;
 * 		0=don't draw corresponding bitplane.
 * 0408		DrawMask
 ****************************************************************************/

	/*
	 * <7:0>	red	RED CHANNEL DRAW MASK.
	 * 
	 * <15:8>	grn	GREEN CHANNEL DRAW MASK.
	 * 
	 * <23:16>	blu	BLUE CHANNEL DRAW MASK.
	 * 
	 * <27:24>	tag	TAG BITS DRAW MASK.
	 *
	 */

#ifdef BITI
	typedef	struct {
		unsigned tag:	8;
		unsigned blu:	8;
		unsigned grn:	8;
		unsigned red:	8;
	} DrawMask_bits;
#else
	typedef	struct {
		unsigned red:	8;
		unsigned grn:	8;
		unsigned blu:	8;
		unsigned tag:	8;
	} DrawMask_bits;
#endif

	typedef union {
		unsigned red;
		unsigned grn;
		unsigned blu;
		unsigned tag;
	} DrawMask_lwd;



/*****************************************************************************
 *
 * DrawRop	DRAW RASTER OP.
 * DR		Determines ROP to be used in drawing operations.  The <enbl>
 * dr		bit must be set in RopCtl for the ROP to take effect.  If 
 * 		psuedo-color, RGB ROPS must all be the same.
 * 040c		DrawRop
 ****************************************************************************/

	/* <3:0>	red	RED CHANNEL RASTER OP
	 * 
	 * <11:8>	grn	GREEN CHANNEL RASTER OP
	 * 
	 * <19:16>	blu	BLUE CHANNEL RASTER OP
	 * 
	 * <27:24>	tag	TAG BITS RASTER OP
	 *
	 * The following are the ROP definitions:
	 *
	 * 	0	clear
	 *	1	S & P 		(AND)
	 * 	2	S & ~P
	 *	3	S 		(NOOP)
	 *	4	~S & P
	 *	5	P 		(DEFAULT)
	 *	6	S ^ P 		(XOR)
	 *	7	S | P		(OR)
	 *	8	~S & ~P		(NOR)
	 * 	9	~S ^ P		(XNOR)
	 *	a	~P		(INVERT)
	 *	b	S | ~P		
	 *	c	~S		(INVERT SOURCE)
	 *	d	~S | P
	 * 	e	~S | ~P		(NAND)
	 *	f	set
	 */	 

#ifdef BITI
	typedef struct {
		unsigned :		4;
		unsigned tag:		4;
		unsigned :		4;
		unsigned blu:		4;
		unsigned :		4;
		unsigned grn:		4;
		unsigned :		4;
		unsigned red:		4;
	} DrawRop_bits;
#else
	typedef struct {
		unsigned red:		4;
		unsigned :		4;
		unsigned grn:		4;
		unsigned :		4;
		unsigned blu:		4;
		unsigned :		4;
		unsigned tag:		4;
		unsigned :		0;
	} DrawRop_bits;
#endif

	typedef union {
		unsigned red;
		unsigned grn;
		unsigned blu;
		unsigned tag;
	} DrawRop_lwd;

/*****************************************************************************
 *
 * M1ForCol	BITPLANE FOREGROUND COLOR.
 *  		Foreground color to be used to draw into 1 bit memory map when
 *  		pixel value = 1. 
 * 0058		M1ForCol
 *
 ****************************************************************************/

/*****************************************************************************
 * 
 * M1BakCol	BITPLANE BACKGROUND COLOR.
 *  		Background color to be used to draw into 1 bit memory map when
 *  		pixel value = 0.
 * 005C		M1BakCol
 * 
 ****************************************************************************/

/*****************************************************************************
 * 
 * M1RdMask	BITPLANE READ MASK.
 *  		Mask to be used in reading back 1 bit memory map.  If display
 *  		memory pixel & M1RdMask == M1ForCol & M1RdMask, then 
 * 		read back a 1, else read back a 0.
 * 0064		M1RdMask
 ****************************************************************************/ 


/*****************************************************************************
 *
 * MMode	MAP MODE REGISTER
 *		control pseudo color readback byte as well as bitplane
 *		transparent mode
 * 0228		MMode
 ****************************************************************************/

	/*
	 * <1:0>	m8rb
	 * 
	 * <3>		trans	TRANSPARENT MODE
	 *
	 */

#ifdef BITI
	typedef struct {
		unsigned :		28;
		unsigned trans:		1;
		unsigned :		1;
		unsigned m8rb:		2;
	} MMode_bits;
#else
	typedef struct {
		unsigned m8rb:		2;
		unsigned :		1;
		unsigned trans:		1;
	} MMode_bits;
#endif

	typedef union {
		unsigned m8rb;
		unsigned trans;
	} MMode_lwd;


/*****************************************************************************
 *
 * MStart	START PIXEL OF MAP WRITE
 *		MStart used on 1-bit byte, short and longword writes
 *		and also pseudo color writes.
 *		must be set to 0 for true color writes
 * 0220		MStart
 ****************************************************************************/

/*****************************************************************************
 *
 * MEnd		END PIXEL OF MAP WRITE
 *		MEnd used on 1-bit byte, short and longword writes
 *		and also pseudo color writes.
 *		must be set to 0x1f for true color writes
 * 0060		MEnd
 ****************************************************************************/

/*****************************************************************************
 *
 * MOffset	Offset for map writes
 *		MOffset used on 1-bit byte, short and longword writes
 *		and also pseudo color writes.
 *		must be set to 0 for true color writes
 * 0004		MOffset
 ****************************************************************************/


/*****************************************************************************
 *
 * SolidCol	SOLID COLOR USED BY BLIT ENGINE.
 * SC		Solid color used by BLIT and VECTOR.  If this register is 
 * sc		loaded prior to initiaing a BLIT, then a solid fill is 
 * 		perfromed with the specified color.
 * 0420		SolidCol
 ****************************************************************************/

/*****************************************************************************
 *
 * WindowX	WINDOW TRANSLATE VALUE: X
 *		the value is this register is added to the x start coord.	
 *		give the actual x start position of the vector.
 * 0200		WindowX
 *
 ****************************************************************************/

/*****************************************************************************
 *
 * WindowY	WINDOW TRANSLATE VALUE: y
 *		the value is this register is added to the y start coord.	
 *		give the actual y start position of the vector.
 * 0204		WindowY
 ****************************************************************************/

/*****************************************************************************
 *
 * VectX0	VECTOR START X COORDINATE
 *		vector x start position that when added to the WindowX
 * 		register gives the actual vectors x start position
 * 0208		VectX0
 ****************************************************************************/

/*****************************************************************************
 *
 * VectY0	VECTOR START Y COORDINATE
 *		vector y start position that when added to the WindowY
 * 		register gives the actual vectors y start position
 * 020c		VectY0
 ****************************************************************************/


/*****************************************************************************
 *
 * VectDX	VECTOR DRAWTO RELATIVE X
 *		x coordinate width to be drawn from VectX0		
 * 	
 * 0130		VectDX
 ****************************************************************************/

/*****************************************************************************
 *
 * VectDY	VECTOR DRAWTO RELATIVE Y
 *		y coordinate width to be drawn from VectY0		
 *		loading this register initiates vector. 	
 * 0140		VectDY
 ****************************************************************************/

/*****************************************************************************
 *
 * VectX1	VECTOR DRAWTO ABSOLUTE X
 *		x coordinate to be draw to from VectX0
 * 	
 * 0330		VectX1
 ****************************************************************************/

/*****************************************************************************
 *
 * VectY1	VECTOR DRAWTO ABSOLUTE Y
 *		y coordinate to be draw to from VectY0
 * 		loading this register initiated vector.	
 * 0340		VectY1
 ****************************************************************************/


/*****************************************************************************
 *
 * VecI1	PARAMETER VECTOR l1 (12 bits)
 *		
 * 	
 * 0040		VecI1
 *
 ****************************************************************************/

/*****************************************************************************
 *
 * VecI2	PARAMETER VECTOR l2 (12 bits)
 *		
 * 	
 * 004c		VecI2
 *
 ****************************************************************************/

/*****************************************************************************
 *
 * VecAcc	PARAMETER VECTOR ACCUMULATOR (e)
 *		
 * 	
 * 0048		VecAcc
 *
 ****************************************************************************/


/*****************************************************************************
 *
 * VecMsc	PARAMETER VECTOR LENGTH AND FLAGS
 *		
 * 	
 * 0050		VecMsc
 *
 ****************************************************************************/

	/*
	 * <11:0>	dmajor	number of pixels to draw minus 1
	 *
	 * <12>		nofirst	do not draw first pixel
	 *
	 * <13>		left	draw vector to left
	 *
	 * <14>		up	draw vector upward
	 *
	 * <15>		xmajor	X is major direstion
	 */

#ifdef BITI
	typedef struct {
		unsigned :		16;
		unsigned xmajor:	1;
		unsigned up:		1;
		unsigned left:		1;
		unsigned nofirst:	1;
		unsigned dmajor:	12;
	} VecMsc_bits;
#else
	typedef struct {
		unsigned dmajor:	12;
		unsigned nofirst:	1;
		unsigned left:		1;
		unsigned up:		1;
		unsigned xmajor:	1;
		unsigned :		16;
	} VecMsc_bits;
#endif

	typedef union {
		unsigned dmajor;
		unsigned nofirst;
		unsigned left;
		unsigned up;
		unsigned xmajor;
	} VecMsc_lwd;


/*****************************************************************************
 *
 * SrcStartAdd	SOURCE START ADDRESS FOR BLIT.
 * SRCSA	Determines BLIT source (x,y) according to:
 * srcsa	SrcStartAdd = x + DW * y, where DW = 1024, 1152, or 1280, as
 * 		appropriate.
 * 0418		SrcStartAdd
 ****************************************************************************/

/****************************************************************************
 *
 * DstStartAdd	DESTINATION START ADDRESS FOR BLIT.
 * DSTSA	Determines BLIT start (x,y) according to: 
 * dstsa	DstStartAdd = x + DW * y, where DW = 1024, 1152, or 1280, as
 * 		appropriate. 
 * 041c		DstStartAdd
 ****************************************************************************/

/*****************************************************************************
 *
 * SolidCol	SOLID COLOR USED BY BLIT ENGINE.
 * SC		Solid color used by BLIT and VECTOR.  If this register is 
 * sc		loaded prior to initiaing a BLIT, then a solid fill is 
 * 		perfromed with the specified color. note: shared with vector
 * 0420		SolidCol
 ****************************************************************************/

/****************************************************************************
 *
 * DstW		DESTINATION WIDTH FOR BLIT. 
 * DSTW		Sets destination end x minus destination start x.  If negative,	 * dstw		BLIT is left going.
 * 		note: this register is shared with vectors
 * 0130		DstW
 ****************************************************************************/

/****************************************************************************
 *
 * DstH		DESTINATION HEIGHT FOR BLIT.
 * DSTH		Sets destination end y minus destination start y.  If negative, 
 * dsth		BLIT is up going.  LOADING THIS REGISTER INITIATES THE BLIT.
 * 0134		DstH
 ****************************************************************************/


/*****************************************************************************
 * 
 * IntStatC	INTERRUPT STATUS REGISTER (READ_ONLY).
 * ISC		read and clear interrupt registers.
 * isc
 * 040c		IntStatC
 *****************************************************************************/

	/*
	 * <0>		disp	vertical display interrupt.
	 *
	 * <1>		ccube	CCube chip requesting interrupt.
	 *
	 * <2>		ext	option connector requesting interrupt.
	 *
	 * <3>		vsync	video port v new field, or line count interrupt.
	 *
	 * <4>		wsync	video port w new field, or line count interrupt.
	 *
	 * <5>		verr	video port v error (loss of sync, or regaining)
	 *
	 * <6>		werr	video port w error change interrupt.	
	 *
	 * <7>		blitdn	blit operation complete.
	 *
	 * <8>		vodd	video port v is currently in odd field (not an
 	 *			interrupt).
	 *
	 * <9>		wodd	video port w is currently in odd field (not an
	 *			interrupt).
	 *
	 * <10>		vnsync	video port v currently has no sync (not an
	 *			interrupt).
	 *
	 * <11>		wnsync	video port w currently has no sync (not an 
	 *			interrupt).
	 */
#ifdef BITI
	typedef struct {
		unsigned :		20;
		unsigned wnsync:	1;
		unsigned vnsync:	1;
		unsigned wodd:		1;
		unsigned vodd:		1;
		unsigned blitdn:	1; 
		unsigned werr:		1; 
		unsigned verr:		1; 
		unsigned wsync:		1; 
		unsigned vsync:		1; 
		unsigned ext:		1;
		unsigned ccube:		1;
		unsigned disp:		1;
	} IntStatC_bits;
#else
	typedef struct {
		unsigned disp:		1;
		unsigned ccube:		1;
		unsigned ext:		1;
		unsigned vsync:		1;
		unsigned wsync:		1;
		unsigned verr:		1;
		unsigned werr:		1;
		unsigned blitdn:	1;
		unsigned vodd:		1;
		unsigned wodd:		1;
		unsigned vnsync:	1;
		unsigned wnsync:	1;
		unsigned :		20;
	} IntStatC_bits;
#endif

	typedef union {
		unsigned disp;
		unsigned ccube;
		unsigned ext;
		unsigned vsync;
		unsigned wsync;
		unsigned verr;
		unsigned werr;
		unsigned blitdn; 
		unsigned vodd;
		unsigned wodd;
		unsigned vnsync;
		unsigned wnsync;
	} IntStatC_lwd;


/*****************************************************************************
 * 
 * IntStat	INTERRUPT STATUS REGISTER (READ_ONLY).
 * 		read and but don't clear interrupt registers.
 * 
 * 0408		IntStat
 *****************************************************************************/

	/*
	 * <0>		disp	vertical display interrupt.
	 *
	 * <1>		ccube	CCube chip requesting interrupt.
	 *
	 * <2>		ext	option connector requesting interrupt.
	 *
	 * <3>		vsync	video port v new field, or line count interrupt.
	 *
	 * <4>		wsync	video port w new field, or line count interrupt.
	 *
	 * <5>		verr	video port v error (loss of sync, or regaining)
	 *
	 * <6>		werr	video port w error change interrupt.	
	 *
	 * <7>		blitdn	blit operation complete.
	 *
	 * <8>		vodd	video port v is currently in odd field (not an
 	 *			interrupt).
	 *
	 * <9>		wodd	video port w is currently in odd field (not an
	 *			interrupt).
	 *
	 * <10>		vnsync	video port v currently has no sync (not an
	 *			interrupt).
	 *
	 * <11>		wnsync	video port w currently has no sync (not an 
	 *			interrupt).
	 */
#ifdef BITI
	typedef struct {
		unsigned :		20;
		unsigned wnsync:	1;
		unsigned vnsync:	1;
		unsigned wodd:		1;
		unsigned vodd:		1;
		unsigned blitdn:	1; 
		unsigned werr:		1; 
		unsigned verr:		1; 
		unsigned wsync:		1; 
		unsigned vsync:		1; 
		unsigned ext:		1;
		unsigned ccube:		1;
		unsigned disp:		1;
	} IntStat_bits;
#else
	typedef struct {
		unsigned disp:		1;
		unsigned ccube:		1;
		unsigned ext:		1;
		unsigned vsync:		1;
		unsigned wsync:		1;
		unsigned verr:		1;
		unsigned werr:		1;
		unsigned blitdn:	1;
		unsigned vodd:		1;
		unsigned wodd:		1;
		unsigned vnsync:	1;
		unsigned wnsync:	1;
		unsigned :		20;
	} IntStat_bits;
#endif

	typedef union {
		unsigned disp;
		unsigned ccube;
		unsigned ext;
		unsigned vsync;
		unsigned wsync;
		unsigned verr;
		unsigned werr;
		unsigned blitdn;
		unsigned vodd;
		unsigned wodd;
		unsigned vnsync;
		unsigned wnsync;
	} IntStat_lwd;


/****************************************************************************
 * 
 * IntMask	INTERRUPT MASK.
 * IM		Determines whether individual interrupts are to be generated
 * im		or not.  Interrupts on a particular condition are generated only
 * 		when the corresponding bit in this register is set to 1.
 * 00a8		IntMask
 ****************************************************************************/

	/* <0>		disp	allow display interrupt.
	 * 
	 * <1>		ccube	allow ccube interrupt.
	 * 
	 * <2>		ext	allow connector interrupt.
	 * 
	 * <3>		vsync	allow video port v new field, or line count
	 *			interrupt.
	 * 
	 * <4>		wsync	allow video port w new field, or line count
	 *			interrupt.
	 * 
	 * <5>		verr	allow video port v error interrupt.
	 * 
	 * <6>		werr	allow video prot w error change interrupt.
	 * 
	 * <7>		blitdn	allow blit done interrupt.
	 *
	 * <15>		smode	use open collector output (sun).
	 */

#ifdef BITI
	typedef struct {
		unsigned :		16;
		unsigned smode:		1;
		unsigned :		7;
		unsigned blitdn:	1;
		unsigned werr:		1;
		unsigned verr:		1;
		unsigned wsync:		1;
		unsigned vsync:		1;
		unsigned ext:		1;
		unsigned ccube:		1;
		unsigned disp:		1;
	} IntMask_bits;
#else
	typedef struct {
		unsigned disp:		1;
		unsigned ccube:		1;
		unsigned ext:		1;
		unsigned vsync:		1;
		unsigned wsync:		1;
		unsigned verr:		1;
		unsigned werr:		1;
		unsigned blitdn:	1;
		unsigned :		7;
		unsigned smode:		1;
		unsigned:		16;
	} IntMask_bits;
#endif
	typedef union {
		unsigned disp;
		unsigned ccube;
		unsigned ext;
		unsigned vsync;
		unsigned wsync;
		unsigned verr;
		unsigned werr;
		unsigned blitdn;
		unsigned smode;
	} IntMask_lwd;


/***************************************************************************
 * 
 * MiscStat	MISC. STATUS BITS (read only)
 * 
 * 0400		MiscStat
 ***************************************************************************/

	/* <0>		fifone	drawing fifo not empty.	(same as dbusy)
	 * 
	 * <1>		fifof	drawing fifo full.
	 * 
	 * <2>		i2crb	i2c bus readback.
	 * 
	 * <3>		vpfull	vector pipe full (same as vbusy)
	 * 
	 * <4>		mode4	read back of cc_stall or cc_half
	 *
	 * <5>		mode5	video port c odd value being used.
	 */

#ifdef BITI
	typedef struct {
		unsigned :		26;
		unsigned mode5:		1;
		unsigned mode4:		1;
		unsigned vpfull:	1;
		unsigned i2crb:		1;
		unsigned fifof:		1;
		unsigned fifone:	1;
	} MiscStat_bits;
#else
	typedef struct {
		unsigned fifone:	1;
		unsigned fifof:		1;
		unsigned i2crb:		1;
		unsigned vpfull:	1;
		unsigned mode4:		1;
		unsigned mode5:		1;
		unsigned:		26;
	} MiscStat_bits;
#endif
	typedef union {
		unsigned fifone;
		unsigned fifof;
		unsigned i2crb;
		unsigned vpfull;
		unsigned mode4;
		unsigned mode5;
	} MiscStat_lwd;


/***************************************************************************
 * 
 * DBusy	DRAW BUSY INTERRUPT REGISTER (read only)
 * 		one bit register for wait_for_done
 * 0410		DBusy
 ***************************************************************************/

/***************************************************************************
 * 
 * VBusy	VECTOR BUSY REGISTER (read only)
 * 		one bit register for fast vector loops
 * 0418		VBusy
 ***************************************************************************/

/***************************************************************************
 * 
 * TmrSpd	TIMER SPEED REGISTER
 * 		90 khz timer speed setting
 *  		9:0 -- mclk speed divided by 90khz - 1. for 33.300 mhz
 *		use 0x171. for 39.96 mhz use 0x1bb.
 *		note: loading this reseted timer to 0.
 * 0214		TmrSpd
 ***************************************************************************/

/***************************************************************************
 * 
 * TmrInt	TIMER INTERRUPT VALUE.
 *		31:0 -- interrupt when wimer reaches this value
 *
 * 0210		TmrInt
 ***************************************************************************/


/*****************************************************************************
 *
 * DMARB	DMA ADDRESS READBACK. (READ_ONLY)
 * 		This is the current DMA address.
 *		This register is formatted exactly as it is presented
 *		to the host bus data lines.
 * 0000		DMARB
 ****************************************************************************/
	/*	Sbus	31:2	DMA address 31:2
			1:0	always zero
		Tbus	31:5	DMA address 28:2
			4:0	DMA address 33:29
	*/

/***************************************************************************
 * 
 * TmrCnt	TIMER COUNT VALUE. (READ ONLY)
 *		current count of the 90 khz timer.
 * 0004 	TmrCnt
 ***************************************************************************/

/***************************************************************************
 * 
 * TmrIntRB	TIMER INTERRUPT READBACK (READ ONLY).
 *		last value written to TmrInt register 
 *		(for diagnostic purposes)
 * 0008 	TmrIntRB
 ***************************************************************************/


/***************************************************************************
 * 
 * BIntStat	BIC INTERRUPT STATUS (READ ONLY)
 *
 * 0010 	BIntStat
 ***************************************************************************/

	/*
 	 * <0>		dma	DMA buffer change interrupt	
	 *
	 * <1>		timer	90 khz timer interrupt.
	 * 
	 * <2>		bufnm	which DMA buffer is being used.
	 */

#ifdef BITI
	typedef struct {
		unsigned :		29;
		unsigned bufnm:		1;
		unsigned timer:		1;
		unsigned dma:		1;
	} BIntStat_bits;
#else
	typedef struct {
		unsigned dma:		1;
		unsigned timer:		1;
		unsigned bufnm:		1;
		unsigned :		29;
	} BIntStat_bits;
#endif
	typedef union {
		unsigned dma;
		unsigned timer;
		unsigned bufnm;
	} BIntStat_lwd;	


/***************************************************************************
 * 
 * BIntStatC	BIC INTERRUPT STATUS READ AND CLEAR (READ ONLY)
 * 0030		BIntStatC
 ***************************************************************************/

 	/* <0>		dma	DMA buffer change interrupt	
	 *
	 * <1>		timer	90 khz timer interrupt.
	 * 
	 * <2>		bufnm	which DMA buffer is being used.
	 */

#ifdef BITI
	typedef struct {
		unsigned :		29;
		unsigned bufnm:		1;
		unsigned timer:		1;
		unsigned dma:		1;
	} BIntStatC_bits;
#else
	typedef struct {
		unsigned dma:		1;
		unsigned timer:		1;
		unsigned bufnm:		1;
		unsigned :		29;
	} BIntStatC_bits;
#endif
	typedef union {
		unsigned dma;
		unsigned timer;
		unsigned bufnm;
	} BIntStatC_lwd;	


/***************************************************************************
 * 
 * BIntMask	INTERRUPT MASK FOR BIC INTERRUPT.
 * 0218		BIntMask
 ***************************************************************************/

 	/* <0>		dma	allow DMA buffer change interrupt	
	 *
	 * <1>		timer	allow 90 khz timer interrupt.
	 */

#ifdef BITI
	typedef struct {
		unsigned :		30;
		unsigned timer:		1;
		unsigned dma:		1;
	} BIntMask_bits;
#else
	typedef struct {
		unsigned dma:		1;
		unsigned timer:		1;
		unsigned :		30;
	} BIntMask_bits;
#endif
	typedef union {
		unsigned dma;
		unsigned timer;
	} BIntMask_lwd;	


/*****************************************************************************
 * VCurAdd	VIDEO PORT V CURRENT ADDRESS. (READ_ONLY)
 * 0014		VCurAdd
 *****************************************************************************/

/*****************************************************************************
 * WCurAdd	VIDEO PORT W CURRENT ADDRESS. (READ_ONLY)
 * 0018		WCurAdd
 *****************************************************************************/

/*****************************************************************************
 * DMASA1	DMA START ADDRESS FOR BUFFER 1.
 * dmasa1	Determines the start address of DMA operations.  Current DMA
 * 		address can be read back in DMARB.  DMA is only performed by
 * 		the C-Cube compression chip.
 * 0068		DMASA1
 ****************************************************************************/

/*****************************************************************************
 * 
 * DMASA2	DMA START ADDRESS FOR BUFFER 2.
 * dmasa2	Determines the start address of DMA operations.  Current DMA
 * 		address can be read back in DMARB.  DMA is only performed by
 * 		the C-Cube compression chip.
 * 006c		DMASA2
 ****************************************************************************/

/*****************************************************************************
 * DMAEA1	DMA END ADDRESS FOR BUFFER 1.
 * dmaea1	Determines the end address of DMA operations.  Current DMA
 * 		address can be read back in DMARB.  DMA is only performed by
 * 		the C-Cube compression chip.
 * 0070		DMAEA1
 ****************************************************************************/

/*****************************************************************************
 * DMAEA2	DMA END ADDRESS FOR BUFFER 2.
 * dmaea2	Determines the end address of DMA operations.  Current DMA
 * 		address can be read back in DMARB.  DMA is only performed by
 * 		the C-Cube compression chip.
 * 0074		DMAEA2
 ****************************************************************************/

/*****************************************************************************
 * CCube	CCUBE CONTROL REGISTER
 *		this register contains many bit fields but
 *		may not be use loaded 1 bit at time because of 
 *		edge conditions.
 * 043c		CCube
 *****************************************************************************/
/***************************************************************************
 * 
 * C3Stall	C3 Stall control register
 * 00dc		as well as config bits
 ***************************************************************************/

 	/* <0:3>	Stall if .... 
	 *
	 * <4>		Stall on pixphase (0 -> if pixphase = 1)
	 *
	 * <5:7>	CE delay from stall condition
	 *
	 * <8:9>	Stall delay from stall condition
	 *
	 * <10>		Xor CE (0= same as STALL, so s.b. 1)
	 */

#ifdef BITI
	typedef struct {
		unsigned :		21;
		unsigned xorce:		1;
		unsigned sdelay:	2;
		unsigned cedelay:	3;
		unsigned phase:		1;
		unsigned cond:		4;
	} C3Stall_bits;
#else
	typedef struct {
		unsigned cond:		4;
		unsigned phase:		1;
		unsigned cedelay:	3;
		unsigned sdelay:	2;
		unsigned xorce:		1;
		unsigned :		21;
	} C3Stall_bits;
#endif
	typedef union {
		unsigned cond;
		unsigned phase;
		unsigned cedelay;
		unsigned sdelay;
		unsigned xorce;
	} C3Stall_lwd;	



/****************************************************************************
 * 
 * I2C		PHILIPS I2C BUS.
 * i2c		Register is used for controlling reads/writes to the I2C bus.
 * 		Data is written here and read back in the IntStat register.
 * 0224		I2C
 ***************************************************************************/

	/*
	 * <0>		clk	TRUE HIGH CLOCK OUTPUT.
	 *			This is the clock line for the I2C bus.
	 *
	 * <1>		enbl	DATA ENABLE.
	 *			0: Reading from I2C.
	 * 			1: Writing to I2C.
	 *
	 * <2>		data	WRITE DATA.
	 *			This is the data line for the I2C bus (used 
	 * 			for writes only; reads are done from the 
 	 *			I2CStat register).
	 * <3>		cen	CLOCK ENABLE.
 	 *			1 == reading clock as wait state
 	 *			0 == driving clock to i2c.
	 */

#ifdef BITI
	typedef struct {
		unsigned :		28;
		unsigned cen:		1;
		unsigned data:		1;
		unsigned enbl:		1;
		unsigned clk:		1;
	} I2C_bits;
#else
	typedef struct {
		unsigned clk:		1;
		unsigned enbl:		1;
		unsigned data:		1;
		unsigned cen:		1;
		unsigned :		28;
	} I2C_bits;
#endif

	typedef union { 
		unsigned clk;
		unsigned enbl;
		unsigned data;
		unsigned cen;
	} I2C_lwd;


/****************************************************************************
 * I2CStat	I2C STATUS (READ ONLY)
 * 001c		I2CStat
 ***************************************************************************/

	/*
	 * <0>		data	current state of data line
	 *
	 * <1>		clk	current state of clk line for wait states
	 */

#ifdef BITI
	typedef struct {
		unsigned :		30;
		unsigned clk:		1;
		unsigned data:		1;
	} I2CStat_bits;
#else
	typedef struct {
		unsigned data:		1;
		unsigned clk:		1;
		unsigned :		30;
	} I2CStat_bits;
#endif
	typedef union { 
		unsigned data;
		unsigned clk;
	} I2CStat_lwd;


/****************************************************************************
 * OLDI2C	OLD I2C BITS (ON PPC)
 * 		these are no longer used as i2c bits
 * 		these can be used as misc mode bits
 * 0438		OLDI2C
 ***************************************************************************/

	/*
	 * <0>		clk	TRUE HIGH CLOCK OUTPUT.
	 *			This is the clock line for the I2C bus.
	 *
	 * <1>		enbl	DATA ENABLE.
	 *			0: Reading from I2C.
	 * 			1: Writing to I2C.
	 *
	 * <2>		data	WRITE DATA.
	 *			This is the data line for the I2C bus (used 
	 * 			for writes only; reads are done from the 
 	 *			I2CStat register).
	 */

#ifdef BITI
	typedef struct {
		unsigned :		29;
		unsigned data:		1;
		unsigned enbl:		1;
		unsigned clk:		1;
	} OLDI2C_bits;
#else
	typedef struct {
		unsigned clk:		1;
		unsigned enbl:		1;
		unsigned data:		1;
		unsigned :		29;
	} OLDI2C_bits;
#endif

	typedef union { 
		unsigned clk;
		unsigned enbl;
		unsigned data;
	} OLDI2C_lwd;


/*****************************************************************************
 * MiscMode	MISCELLANEOUS MODE BITS.
 * MM		Determines the state of miscellaneous mode bits on the PPCs.
 * 0458		MiscMode
 *****************************************************************************/
	/*
	 * <2:0>	red	MISC BITS ON RED PPC.
	 *
	 * <10:8>	grn	MISC BITS ON GREEN PPC.
	 * 
	 * <18:16>	blu	MISC BITS ON BLUE PPC.
	 *
	 */ 
#ifdef BITI
	typedef struct {
		unsigned :	13;
		unsigned blu:	3;
		unsigned : 	5;
		unsigned grn:	3;
		unsigned :	5;
		unsigned red:	3;

	} MiscMode_bits;
#else
	typedef struct {
		unsigned red:	3;
		unsigned :	5;
		unsigned grn:	3;
		unsigned : 	5;
		unsigned blu:	3;
		unsigned :	0;
	} MiscMode_bits;
#endif

	typedef union {
		unsigned red;
		unsigned grn;
		unsigned blu;
	} MiscMode_lwd;


/*****************************************************************************
 * Bt		BROOKTREE 462 (3) or 463 RAMDAC (READ_WRITE).
 * 		This register space maps to the brooktree RAMDACs.  An eight
 *		bit data path is provided to communicate with the Brooktree.
 *		Lower eight bits of read/writes are vaild; upper 24 bits are
 *		garbage.
 * 		Accesses have the form: Bt.red.reg0 = xx; etc.
 * 00480-004bc	Bt
 *****************************************************************************/

typedef struct {
	unsigned int reg0;
	unsigned int reg1;
	unsigned int reg2;
	unsigned int reg3;
} bt_regs;

typedef struct {
	bt_regs red;
	bt_regs grn;
	bt_regs blu;
	bt_regs all;
} Brooktree_regs;

/*****************************************************************************
 *
 * Ext		EXTERNAL CONNECTOR COMMUNICATION REGISTERS.
 * 		These four registers generate accesses to the option connector.
 *		The purpose of these registers is dependent on each option card.
 * 		Accesses have the form: Ext.reg0 = xx; etc.
 * 04c0-04cc	Ext
 *****************************************************************************/

typedef struct {
	unsigned int reg0;
	unsigned int reg1;
	unsigned int reg2;
	unsigned int reg3;
} Ext_regs;


/****************************************************************************
 * 
 * VCtl		VIDEO PORT V CONTROL
 * 		Determines whether the video channel is an input or output, 
 * 		whether the flash is a single or continuous field/frame, and 
 * 		whether the flash should be done immediately or on the next
 * 		field.  Also can be used to toggle the even/odd field detect.
 *		Low byte of this register must be repeated into all bytes.
 *		Whether to flash a field or frame is set by VSqzY:<inter>.
 * 0428		VCtl
 ****************************************************************************/
	/* <0>		dir	PORT DIRECTION.	(MUST BE 0.)
	 *			0: Input.
	 *			1: Output.
 	 *
	 * <1>		single	SINGLE FRAME (OR FIELD).
	 *			0: No action. 
	 *			1: Flash a single field/frame.
	 *
	 * <2>		cont	CONTINUOUS.
	 *			0: No action.
	 *			1: Flash fields/frames continuously.
	 *
	 * <3>		now	DO NOW.
	 *			0: Wait until next field to start digitization.
	 *			1: Start digitization immediately.
	 *
	 * <5>		togodd	TOGGLE EVEN/ODD DETECT.
	 *			0: No action.
	 *			1: Compliment odd/even detect.
	 *
 	 * NOTE: bits <2:1> must go through state <0:0> when changing from
	 * <1:0> to <0:1> or vice versa; state <1:1> is illegal.
  	 * (repeat bits <7:0> into <31:24> <23:16> <15:8> )
	 */

#ifdef BITI
	typedef struct {
		unsigned :		26;
		unsigned togodd:	1;
		unsigned :		1;
		unsigned now:		1;
		unsigned cont:		1;
		unsigned single:	1;
		unsigned dir:		1;	
	} VCtl_bits;
#else
	typedef struct {
		unsigned dir:		1;	
		unsigned single:	1;
		unsigned cont:		1;
		unsigned now:		1;
		unsigned :		1;
		unsigned togodd:	1;
		unsigned :		0;
	} VCtl_bits;
#endif
	typedef union {
		unsigned dir;
		unsigned single;
		unsigned cont;
		unsigned now;
		unsigned togodd;
	} VCtl_lwd;

/****************************************************************************
 * 
 * WCtl		VIDEO PORT W CONTROL
 * 		Determines whether the video channel is an input or output, 
 * 		whether the flash is a single or continuous field/frame, and 
 * 		whether the flash should be done immediately or on the next
 * 		field.  Also can be used to toggle the even/odd field detect.
 *		Low byte of this register must be repeated into all bytes.
 *		Whether to flash a field or frame is set by WSqzY:<inter>.
 * 0424		WCtl
 ****************************************************************************/

	/* <0>		dir	PORT DIRECTION.	
	 *			0: Input.
	 *			1: Output.
 	 *
	 * <1>		single	SINGLE FRAME (OR FIELD).
	 *			0: No action. 
	 *			1: Flash a single field/frame.
	 *
	 * <2>		cont	CONTINUOUS.
	 *			0: No action.
	 *			1: Flash fields/frames continuously.
	 *
	 * <3>		now	DO NOW.
	 *			0: Wait until next field to start digitization.
	 *			1: Start digitization immediately.
	 *
	 * <5>		togodd	TOGGLE EVEN/ODD DETECT.
	 *			0: No action.
	 *			1: Compliment odd/even detect.
	 *
 	 * NOTE: bits <2:1> must go through state <0:0> when changing from
	 * <1:0> to <0:1> or vice versa; state <1:1> is illegal.
  	 * (repeat bits <7:0> into <31:24> <23:16> <15:8> )
	 */

#ifdef BITI
	typedef struct {
		unsigned :		26;
		unsigned togodd:	1;
		unsigned :		1;
		unsigned now:		1;
		unsigned cont:		1;
		unsigned single:	1;
		unsigned dir:		1;	

	} WCtl_bits;
#else
	typedef struct {
		unsigned dir:		1;	
		unsigned single:	1;
		unsigned cont:		1;
		unsigned now:		1;
		unsigned :		1;
		unsigned togodd:	1;
		unsigned :		0;
	} WCtl_bits;
#endif

	typedef union {
		unsigned dir;
		unsigned single;
		unsigned cont;
		unsigned now;
		unsigned togodd;
	} WCtl_lwd;


/****************************************************************************
 * VStart	VIDEO PORT V START LINE.
 *		start line for video port v
 * 00e0		VStart
 ***************************************************************************/

	/* <11:0>	line	digitizing and outputting will not
	 *			start until this many hsyncs are seen
	 *			after vsysnc
 	 *
	 * <15>		intr	if this bit is set the video interrupt
	 *			will occur after the first enabled line,	
	 *			rather than vsync	
	 */

#ifdef BITI
	typedef struct {
		unsigned :		16;
		unsigned intr:		1;
		unsigned :		3;
		unsigned line:		12;
	} VStart_bits;
#else
	typedef struct {
		unsigned line:		12;	
		unsigned :		3;	
		unsigned intr:		1;
		unsigned :		16;
	} VStart_bits;
#endif

	typedef union {
		unsigned line;
		unsigned intr;
	} VStart_lwd;


/****************************************************************************
 * WStart	VIDEO PORT W START LINE.
 *		start line for video port w
 * 00e8		WStart
 ***************************************************************************/

	/* <11:0>	line	digitizing and outputting will not
	 *			start until this many hsyncs are seen
	 *			after vsysnc
 	 *
	 * <15>		intr	if this bit is set the video interrupt
	 *			will occur after the first enabled line,	
	 *			rather than vsync	
	 */

#ifdef BITI
	typedef struct {
		unsigned :		16;
		unsigned intr:		1;
		unsigned :		3;
		unsigned line:		12;
	} WStart_bits;
#else
	typedef struct {
		unsigned line:		12;	
		unsigned :		3;	
		unsigned intr:		1;
		unsigned :		16;
	} WStart_bits;
#endif

	typedef union {
		unsigned line;
		unsigned intr;
	} WStart_lwd;


/****************************************************************************
 * VStop	VIDEO PORT V STOP LINE.
 *		stop line for video port v
 * 00e4		VStop
 ***************************************************************************/

	/* <11:0>	line	digitizing and outputting will stop
	 *			after this many lines within the field.
 	 *
	 * <15>		intr	if this bit is set the video interrupt
	 *			will occur after the last enabled line,	
	 *			rather than vsync	
	 */

#ifdef BITI
	typedef struct {
		unsigned :		16;
		unsigned intr:		1;
		unsigned :		3;
		unsigned line:		12;
	} VStop_bits;
#else
	typedef struct {
		unsigned line:		12;	
		unsigned :		3;	
		unsigned intr:		1;
		unsigned :		16;
	} VStop_bits;
#endif
	typedef union {
		unsigned line;
		unsigned intr;
	} VStop_lwd;


/****************************************************************************
 * WStop	VIDEO PORT W STOP LINE.
 *		stop line for video port w
 * 00ec		WStop
 ***************************************************************************/

	/* <11:0>	line	digitizing and outputting will stop
	 *			after this many lines within the field.
 	 *
	 * <15>		intr	if this bit is set the video interrupt
	 *			will occur after the last enabled line,	
	 *			rather than vsync	
	 */

#ifdef BITI
	typedef struct {
		unsigned :		16;
		unsigned intr:		1;
		unsigned :		3;
		unsigned line:		12;
	} WStop_bits;
#else
	typedef struct {
		unsigned line:		12;	
		unsigned :		3;	
		unsigned intr:		1;
		unsigned :		16;
	} WStop_bits;
#endif
	typedef union {
		unsigned line;
		unsigned intr;
	} WStop_lwd;


/****************************************************************************
 * VStartAdd	VIDEO PORT V START ADDRESS
 * 		Load with (x-DX) + (y-DY) * DW, where DX is the number of 
 * 		blanked lines between Vsync and the first good line, DY is the
 * 		number of pixels between Hsync and the first good pixel, and
 *		DX is 1024, 1152, or 1280, as appropriate.
 * 0410		VStartAdd
 ***************************************************************************/

/****************************************************************************
 * WStartAdd	VIDEO PORT W START ADDRESS
 * WSA		Load with (x-DX) + (y-DY) * DW, where DX is the number of 
 * wsa		blanked lines between Vsync and the first good line, DY is the
 * 		number of pixels between Hsync and the first good pixel, and
 *		DX is 1024, 1152, or 1280, as appropriate.
 * 0414		WStartAdd
 ***************************************************************************/


/*****************************************************************************
 * VMask	VIDEO PORT V PLANE MASK. 
 * VM		Write mask to be used when digitizing from video channel V.
 * vm		A bitplane is drawn to if the corrsponding bit in this register
 * 		is set to 1.  Tagplanes cannot be digitized.
 * 0400		VMask
 ****************************************************************************/
	/*
	 * <7:0>	red	RED CHANNEL DRAW MASK.
	 * 
	 * <15:8>	grn	GREEN CHANNEL DRAW MASK.
	 * 
	 * <23:16>	blu	BLUE CHANNEL DRAW MASK.
	 * 
	 */

#ifdef BITI
	typedef	struct {
		unsigned :	8;
		unsigned blu:	8;
		unsigned grn:	8;
		unsigned red:	8;

	} VMask_bits;
#else
	typedef	struct {
		unsigned red:	8;
		unsigned grn:	8;
		unsigned blu:	8;
		unsigned :	0;
	} VMask_bits;
#endif

	typedef union {
		unsigned red;
		unsigned grn;
		unsigned blu;
	} VMask_lwd;


/*****************************************************************************
 * WMask	VIDEO PORT W PLANE MASK.
 * WM		Write mask to be used when digitizing from video channel W.
 * wm		A bitplane is drawn to if the corrsponding bit in this register
 * 		is set to 1.  Tagplanes cannot be digitized.
 * 0404		WMask
 ****************************************************************************/

	/*
	 * <7:0>	red	RED CHANNEL DRAW MASK.
	 * 
	 * <15:8>	grn	GREEN CHANNEL DRAW MASK.
	 * 
	 * <23:16>	blu	BLUE CHANNEL DRAW MASK.
	 * 
	 */

#ifdef BITI
	typedef	struct {
		unsigned :	8;
		unsigned blu:	8;
		unsigned grn:	8;
		unsigned red:	8;

	} WMask_bits;
#else
	typedef	struct {
		unsigned red:	8;
		unsigned grn:	8;
		unsigned blu:	8;
		unsigned :	0;
	} WMask_bits;
#endif
	typedef union {
		unsigned red;
		unsigned grn;
		unsigned blu;
	} WMask_lwd;


/****************************************************************************
 * VTag		VIDEO PORT V TAG MASK/COMPARE VALUES.
 * VT		Defines the mask and tag compare values to be used for video
 * vt		digitization from channel V.  Video is drawn if:
 * 		Tagplanes & <mask> = <val>.  RopCtl:<enbl> must be 1.
 * 0018		VTag 
 ****************************************************************************/

	/*
	 * <7:0>	val	COMPARE VALUE.
	 *			Value to be compared against Tagplanes & mask.
	 * 
	 * <15:8>	mask	MASK VALUE.
	 *			Mask to be used in testing Tagplanes.
	 * 
 	 */

#ifdef BITI
	typedef struct {
		unsigned :		16;
		unsigned mask:		8;
		unsigned val:		8;
	} VTag_bits;
#else
	typedef struct {
		unsigned val:		8;
		unsigned mask:		8;
	} VTag_bits;
#endif

	typedef union {
		unsigned val;
		unsigned mask;
	} VTag_lwd;


/****************************************************************************
 * WTag		VIDEO PORT W TAG MASK/COMPARE VALUES.
 * WT		Defines the mask and tag compare values to be used for video
 * wt		digitization from channel W.  Video is drawn if:
 * 		Tagplanes & <mask> = <val>.  RopCtl:<enbl> must be 1.
 * 001c		WTag 
 ****************************************************************************/

	/*
	 * <7:0>	val	COMPARE VALUE.
	 *			Value to be compared against Tagplanes & mask.
 	 * 
	 * <15:8>	mask	MASK VALUE.
	 *			Mask to be used in testing Tagplanes.
	 * 
 	 */

#ifdef BITI
	typedef struct {
		unsigned :		16;
		unsigned mask:		8;
		unsigned val:		8;
	} WTag_bits;
#else
	typedef struct {
		unsigned val:		8;
		unsigned mask:		8;
	} WTag_bits;
#endif

	typedef union {
		unsigned val;
		unsigned mask;
	} WTag_lwd;


/****************************************************************************
 * VSqzX	VIDEO PORT V SQUEEZE X
 * VSX		Determines X squeeze factor for video input V.  Must also set
 * vsx		direction of drawing in this register (must match VSqzY).
 * 0020		VSqzX
 ***************************************************************************/

	/*
	 * <11:0>	factor	SQUEEZE FACTOR. 
	 *			X squeeze factor -1.
	 * 
 	 * <14>		ltor	LEFT TO RIGHT.
	 *			0: Draw from left to right
	 *			1: Draw from right to left (mirror).
	 *			
	 * 			(Must match VSqzY:<ltor>.)
	 *
	 */

#ifdef BITI
	typedef struct {
		unsigned :		17;
		unsigned ltor:		1;
		unsigned :		2;
		unsigned factor:	12;
	} VSqzX_bits;
#else
	typedef struct {
		unsigned factor:	12;
		unsigned :		2;
		unsigned ltor:		1;
		unsigned :		0;
	} VSqzX_bits;
#endif

	typedef union {
		unsigned factor;
		unsigned ltor;
	} VSqzX_lwd;


/****************************************************************************
 * WSqzX	VIDEO PORT W SQUEEZE X
 * WSX		Determines X squeeze factor for video input W. Must also set
 * wsx		direction of drawing in this register (must match WSqzY).
 * 0024		WSqzX
 ***************************************************************************/

	/*
	 * <11:0>	factor	SQUEEZE FACTOR.
	 *			X squeeze factor -1.
	 * 
 	 * <14>		ltor	LEFT TO RIGHT.
	 *			0: Draw from left to right (normal).
	 *			1: Draw from right to left (mirror).
	 * 			(Must match WSqzY:<ltor>.)
	 *
	 */


#ifdef BITI
	typedef struct {
		unsigned :		17;
		unsigned ltor:		1;
		unsigned :		2;
		unsigned factor:	12;
	} WSqzX_bits;
#else
	typedef struct {
		unsigned factor:	12;
		unsigned :		2;
		unsigned ltor:		1;
		unsigned :		0;
	} WSqzX_bits;
#endif

	typedef union {
		unsigned factor;
		unsigned ltor;
	} WSqzX_lwd;


/***************************************************************************
 * VSqzY	VIDEO PORT V SQUEEZE Y.
 * VSY		Determines Y squeeze factor for video port V.  Also determines
 * vsy		whether the input is treated as interlaced and what directon
 * 		to draw.
 * 0014		VSqzY
 ***************************************************************************/

	/*
 	 * <11:0>	factor	SQUEEZE FACTOR.
	 *			Y squeeze factor -1. 
	 *
	 * <13>		inter	INTERLACED INPUT.
	 *			0: Interlaced input. (Flash frames).
	 *			1: Non-interlaced input. (Flash fields).
	 * 
	 * <14>		ltor	LEFT TO RIGHT.
	 *			0: Draw from left to right (normal).
	 *			1: Draw from right to left (mirror).
	 *
	 * <15>		ttob	TOP TO BOTTOM.
	 *			0: Draw from bottom to top (mirror).
	 *			1: Draw from top to bottom (normal).
	 *
	 */

#ifdef BITI
	typedef struct {
		unsigned :		16;
		unsigned ttob:		1;
		unsigned ltor:		1;
		unsigned inter:		1;
		unsigned :		1;
		unsigned factor:	12;

	} VSqzY_bits;
#else
	typedef struct {
		unsigned factor:	12;
		unsigned :		1;
		unsigned inter:		1;
		unsigned ltor:		1;
		unsigned ttob:		1;
		unsigned :		16;
	} VSqzY_bits;
#endif

	typedef union {
		unsigned factor;
		unsigned inter;
		unsigned ltor;
		unsigned ttob;
	} VSqzY_lwd;	


/***************************************************************************
 * WSqzY	VIDEO PORT W SQUEEZE Y.
 * WSY		Determines Y squeeze factor for video port W.  Also determines
 * wsy		whether the input is treated as interlaced and what direction
 * 		to draw.
 * 000c		WSqzY
 ***************************************************************************/

	/*
 	 * <11:0>	factor	SQUEEZE FACTOR.
	 *			Y squeeze factor -1. 
	 *
	 * <13>		inter	INTERLACED INPUT.
	 *			0: Non-interlaced input. (Flash fields).
	 *			1: Interlaced input. (Flash frames).
	 * 
	 * <14>		ltor	LEFT TO RIGHT.
	 *			0: Draw from right to left (mirror).
	 *			1: Draw from left to right (normal).
	 *
	 * <15>		ttob	TOP TO BOTTOM.
	 *			0: Draw from bottom to top (mirror).
	 *			1: Draw from top to bottom (normal).
	 *
	 */

#ifdef BITI
	typedef struct {
		unsigned :		16;
		unsigned ttob:		1;
		unsigned ltor:		1;
		unsigned inter:		1;
		unsigned :		1;
		unsigned factor:	12;

	} WSqzY_bits;
#else
	typedef struct {
		unsigned factor:	12;
		unsigned :		1;
		unsigned inter:		1;
		unsigned ltor:		1;
		unsigned ttob:		1;
		unsigned :		0;
	} WSqzY_bits;
#endif

	typedef union {
		unsigned factor;
		unsigned inter;
		unsigned ltor;
		unsigned ttob;
	} WSqzY_lwd;	



/****************************************************************************
 * WAttrH	VIDEO PORT W ATTRIBUTE CONTROL - HIGH WORD
 *		this register alowng with WAttrL generates the ATTR1:0
 *		bits which go to the option connector. each of the 2 bit
 *		fields is what the value of ATTR1:0 will be for each
 *		given value of high order tag planes.
 * 00d8		WAttrH
 ****************************************************************************/

	/*
	 * <1:0>	value8
	 * 
 	 * <3:2>	value9
	 * 
 	 * <5:4>	value10
	 * 
 	 * <7:6>	value11
	 * 
 	 * <9:8>	value12
	 * 
 	 * <11:10>	value13
	 * 
 	 * <13:12>	value14
	 * 
 	 * <15:14>	value15
	 *
	 */

#ifdef BITI
	typedef struct {
		unsigned :		16;
		unsigned value15:	2;
		unsigned value14:	2;
		unsigned value13:	2;
		unsigned value12:	2;
		unsigned value11:	2;
		unsigned value10:	2;
		unsigned value9:	2;
		unsigned value8:	2;
	} WAttrH_bits;
#else
	typedef struct {
		unsigned value8:	2;
		unsigned value9:	2;
		unsigned value10:	2;
		unsigned value11:	2;
		unsigned value12:	2;
		unsigned value13:	2;
		unsigned value14:	2;
		unsigned value15:	2;
		unsigned :		16;
	} WAttrH_bits;
#endif

	typedef union {
		unsigned value8;
		unsigned value9;
		unsigned value10;
		unsigned value11;
		unsigned value12;
		unsigned value13;
		unsigned value14;
		unsigned value15;
	} WAttrH_lwd;


/****************************************************************************
 * WAttrL	VIDEO PORT W ATTRIBUTE CONTROL - LOW WORD
 *		this register alowng with WAttrH generates the ATTR1:0
 *		bits which go to the option connector. each of the 2 bit
 *		fields is what the value of ATTR1:0 will be for each
 *		given value of high order tag planes.
 * 00d4		WAttrL
 ****************************************************************************/

	/*
	 * <1:0>	value0
	 * 
 	 * <3:2>	value1
	 * 
 	 * <5:4>	value2
	 * 
 	 * <7:6>	value3
	 *
 	 * <9:8>	value4
	 * 
 	 * <11:10>	value5
	 * 
 	 * <13:12>	value6
	 * 
 	 * <15:14>	value7
	 *
	 */

#ifdef BITI
	typedef struct {
		unsigned :		16;
		unsigned value7:	2;
		unsigned value6:	2;
		unsigned value5:	2;
		unsigned value4:	2;
		unsigned value3:	2;
		unsigned value2:	2;
		unsigned value1:	2;
		unsigned value0:	2;
	} WAttrL_bits;
#else
	typedef struct {
		unsigned value0:	2;
		unsigned value1:	2;
		unsigned value2:	2;
		unsigned value3:	2;
		unsigned value4:	2;
		unsigned value5:	2;
		unsigned value6:	2;
		unsigned value7:	2;
		unsigned :		16;
	} WAttrL_bits;
#endif

	typedef union {
		unsigned value0;
		unsigned value1;
		unsigned value2;
		unsigned value3;
		unsigned value4;
		unsigned value5;
		unsigned value6;
		unsigned value7;
	} WAttrL_lwd;


/****************************************************************************
 * AFormat	VIDEO CHANNEL A MULTIPLEXOR FORMAT.
 * AF		Controls channel A of the crosspoint switch (option connector).
 * af		Same as BFormat and CFormat, except for Pixel Selector and 
 * 		C-cube bit (CFormat only).
 * 0440		AFormat
 ****************************************************************************/

	/*
	 * <1:0>	psel0	PIXEL SELECTOR (must match <9:8> & <17:16>)
	 *			Determines where pixels are routed to/from.
	 *			0: Select W output
	 *			1: Echo B input
	 *			2: Echo C input
	 *			3: Echo D input
	 * 
	 * <2>		pe0	PIXEL ENABLE. (Must match <10>, <18>.)
	 *			Determines whether A bus in an output or input.
	 *			0: Input (disable driver).
	 *			1: Output (enable driver).
	 * 
	 * <3>		se	SYNC ENABLE.
	 *			Determines whether sync information is output.
	 *			0: Disable hsync, vsync, odd drivers. 
	 *			1: Output hsync, vsync, odd (enable driver).
	 * 
	 * <4>		xorce	XOR CLOCK ENABLE
	 *			0: Clock enable is true low
	 *			1: Clock enable is true high
	 *
 	 * <5>		xorhs	XOR HORIZONTAL SYNC
	 *			0: Hsync is true low
	 *			1: Hsync is true high
	 *
	 * <6>		xoro	XOR ODD
	 *			0: ODD is true high
	 *			1: ODD is ture low
	 *
	 * <7>		ce	CLOCK ENABLE.
	 *			Determines whether pixel clock info is output.
	 *			0: Disable clock, clock enable drivers.
	 *			1: Output clock, clock enable (enable driver).
	 * 
	 * <9:8>	psel1	PIXEL SELECTOR (must match <1:0> & <17:16>)
	 *			Determines where pixels are routed to/from.
	 *			0: Select W output
	 *			1: Echo B input
	 *			2: Echo C input
	 *			3: Echo D input
	 * 

	 * <10>		pe1	PIXEL ENABLE. (Must match <2>, <18>.)
	 *			Determines whether A bus in an output or input.
	 *			0: Input (disable driver).
	 *			1: Output (enable driver).
	 * 
	 * <12:11>	cs	CLOCK SOURCE
	 *			If clock is output, determines where it comes
	 *			from.
	 *			0: RESERVED
	 *			1: Echo B clock
	 *			2: Echo D clock divided by 2 
	 *			3: Echo D clock duvided by 2
	 *
	 * <13>		xorvs	XOR VERTICAL SYNC
	 *			0: Vsync is true low
	 *			1: Vsync is true high
	 * 
 	 * <14>		shuttle	SHUTTLE MODE
	 *			0: Use/echo Odd as input
	 *			1: If stuck on Odd/Even, oscillate Odd output.
	 * 
	 * <17:16>	psel2	PIXEL SELECTOR (must match <1:0> & <9:8>)
	 *			Determines where pixels are routed to/from.
	 *			0: Select W output
	 *			1: Echo B input
	 *			2: Echo C input
	 *			3: Echo D input
	 * 
	 * <18>		pe2	PIXEL ENABLE. (Must match <2>, <10>.)
	 *			Determines whether A bus in an output or input.
	 *			0: Input (disable driver).
	 *			1: Output (enable driver).
	 * 
	 *
	 * <25:24>	csel	CLOCK/SYNC SELECT.
	 * 			0: RESERVED
	 * 			1: Echo B input
	 *			2: Echo C input
	 *			3: Echo D input
	 *
	 * <27:26>	osel	ODD/EVEN SELECT.
	 *			If input:
	 *			0: Odd setup/hold 1us to trailing edge of Vsync
	 *			1: Odd setup/hold 40ns to trailing edge of Vsync
	 *			2: Zero (subject to XOR)
	 *			3: One (subject to XOR)
	 * 			If output:
	 * 			0: Echo filtered Odd (fields forced to 
 	 *			   oscillate, setup to trail).
	 * 			1: Echo unfiltered Odd (Xor only).
	 *			2: Echo Zero.
	 * 			3; Echo One.
	 * 
	 */



#ifdef BITI
	typedef struct {
		unsigned :		4;
		unsigned osel:		2;
		unsigned csel:		2;
		unsigned :		5;
		unsigned pe2:		1;
		unsigned psel2:		2;
		unsigned :		1;
		unsigned shuttle:	1;
		unsigned xorvs:		1;
		unsigned cs:		2;
		unsigned pe1:		1;
		unsigned psel1:		2;
		unsigned ce:		1;
		unsigned xoro:		1;
		unsigned xorhs:		1;
		unsigned xorce:		1; 
		unsigned se:		1;
		unsigned pe0:		1;
		unsigned psel0:		2;

	} AFormat_bits;
#else
	typedef struct {
		unsigned psel0:		2;
		unsigned pe0:		1;
		unsigned se:		1;
		unsigned xorce:		1; 
		unsigned xorhs:		1;
		unsigned xoro:		1;
		unsigned ce:		1;
		unsigned psel1:		2;
		unsigned pe1:		1;
		unsigned cs:		2;
		unsigned xorvs:		1;
		unsigned shuttle:	1;
		unsigned :		1;
		unsigned psel2:		2;
		unsigned pe2:		1;
		unsigned :		5;
		unsigned csel:		2;
		unsigned osel:		2;
		unsigned :		0;
	} AFormat_bits;
#endif
	typedef union {
		unsigned psel0;
		unsigned pe0;
		unsigned se;
		unsigned xorce; 
		unsigned xorhs;
		unsigned xoro;
		unsigned ce;
		unsigned psel1;
		unsigned pe1;
		unsigned cs;
		unsigned xorvs;
		unsigned shuttle;
		unsigned psel2;
		unsigned pe2;
		unsigned csel;
		unsigned osel;
	} AFormat_lwd;


/****************************************************************************
 * 
 * BFormat	VIDEO CHANNEL B MULTIPLEXOR FORMAT.
 * BF		Controls channel B of the crosspoint switch (option connector).
 * bf		Same as AFormat and CFormat, except for Pixel Selector and
 * 		C-Cube bit (CFormat only).
 * 0444		BFormat
 ****************************************************************************/

	/*
	 * <1:0>	psel0	PIXEL SELECTOR (must match <9:8> & <17:16>)
	 *			Determines where pixels are routed to/from.
	 *			0: Echo A input 
	 *			1: Select W output
	 *			2: Echo C input
	 *			3: Echo D input
	 * 
	 * <2>		pe0	PIXEL ENABLE. (Must match <10>, <18>.)
	 *			Determines whether A bus in an output or input.
	 *			0: Input (disable driver).
	 *			1: Output (enable driver).
	 * 
	 * <3>		se	SYNC ENABLE.
	 *			Determines whether sync information is output.
	 *			0: Disable hsync, vsync, odd drivers. 
	 *			1: Output hsync, vsync, odd (enable driver).
	 * 
	 * <4>		xorce	XOR CLOCK ENABLE
	 *			0: Clock enable is true low
	 *			1: Clock enable is true high
	 *
 	 * <5>		xorhs	XOR HORIZONTAL SYNC
	 *			0: Hsync is true low
	 *			1: Hsync is true high
	 *
	 * <6>		xoro	XOR ODD
	 *			0: ODD is true high
	 *			1: ODD is ture low
	 *
	 * <7>		ce	CLOCK ENABLE.
	 *			Determines whether pixel clock info is output.
	 *			0: Disable clock, clock enable drivers.
	 *			1: Output clock, clock enable (enable driver).
	 * 
	 * <9:8>	psel1	PIXEL SELECTOR (must match <1:0> & <17:16>)
	 *			Determines where pixels are routed to/from.
	 *			0: Echo A input 
	 *			1: Select W output
	 *			2: Echo C input
	 *			3: Echo D input
	 * 

	 * <10>		pe1	PIXEL ENABLE. (Must match <2>, <18>.)
	 *			Determines whether A bus in an output or input.
	 *			0: Input (disable driver).
	 *			1: Output (enable driver).
	 * 
	 * <12:11>	cs	CLOCK SOURCE
	 *			If clock is output, determines where it comes
	 *			from.
	 *			0: Echo A clock
	 *			1: Echo B clock
	 *			2: Echo D clock divided by 2
	 *			3: Echo D clock divided by 2
	 *
	 * <13>		xorvs	XOR VERTICAL SYNC
	 *			0: Vsync is true low
	 *			1: Vsync is true high
	 * 
 	 * <14>		shuttle	SHUTTLE MODE
	 *			0: Use/echo Odd as input
	 *			1: If stuck on Odd/Even, oscillate Odd output.
	 * 
	 * <17:16>	psel2	PIXEL SELECTOR (must match <1:0> & <9:8>)
	 *			Determines where pixels are routed to/from.
	 *			0: Echo A input 
	 *			1: Select W output
	 *			2: Echo C input
	 *			3: Echo D input
	 * 
	 * <18>		pe2	PIXEL ENABLE. (Must match <2>, <10>.)
	 *			Determines whether A bus in an output or input.
	 *			0: Input (disable driver).
	 *			1: Output (enable driver).
	 * 
	 *
	 * <25:24>	csel	CLOCK/SYNC SELECT.
	 * 			0: Echo A input
	 * 			1: Echo B input
	 *			2: Echo C input
	 *			3: Echo D input
	 *
	 * <27:26>	osel	ODD/EVEN SELECT.
	 *			If input:
	 *			0: Odd setup/hold 1us to trailing edge of Vsync
	 *			1: Odd setup/hold 40ns to trailing edge of Vsync
	 *			2: Zero (subject to XOR)
	 *			3: One (subject to XOR)
	 * 			If output:
	 * 			0: Echo filtered Odd (fields forced to 
 	 *			   oscillate, setup to trail).
	 * 			1: Echo unfiltered Odd (Xor only).
	 *			2: Echo Zero.
	 * 			3; Echo One.
	 * 
	 */


#ifdef BITI
	typedef struct {
		unsigned :		4;
		unsigned osel:		2;
		unsigned csel:		2;
		unsigned :		5;
		unsigned pe2:		1;
		unsigned psel2:		2;
		unsigned :		1;
		unsigned shuttle:	1;
		unsigned xorvs:		1;
		unsigned cs:		2;
		unsigned pe1:		1;
		unsigned psel1:		2;
		unsigned ce:		1;
		unsigned xoro:		1;
		unsigned xorhs:		1;
		unsigned xorce:		1; 
		unsigned se:		1;
		unsigned pe0:		1;
		unsigned psel0:		2;
	} BFormat_bits;
#else
	typedef struct {
		unsigned psel0:		2;
		unsigned pe0:		1;
		unsigned se:		1;
		unsigned xorce:		1; 
		unsigned xorhs:		1;
		unsigned xoro:		1;
		unsigned ce:		1;
		unsigned psel1:		2;
		unsigned pe1:		1;
		unsigned cs:		2;
		unsigned xorvs:		1;
		unsigned shuttle:	1;
		unsigned :		1;
		unsigned psel2:		2;
		unsigned pe2:		1;
		unsigned :		5;
		unsigned csel:		2;
		unsigned osel:		2;
		unsigned :		0;
	} BFormat_bits;
#endif

	typedef union {
		unsigned psel0;
		unsigned pe0;
		unsigned se;
		unsigned xorce; 
		unsigned xorhs;
		unsigned xoro;
		unsigned ce;
		unsigned psel1;
		unsigned pe1;
		unsigned cs;
		unsigned xorvs;
		unsigned shuttle;
		unsigned psel2;
		unsigned pe2;
		unsigned csel;
		unsigned osel;
	} BFormat_lwd;



/****************************************************************************
 * 
 * CFormat	VIDEO CHANNEL C MULTIPLEXOR FORMAT.
 * CF		Controls channel C of the crosspoint switch (normally connected
 * cf		to the CL550).  Same as AFormat and BFormat, except for the 
 * 		Pixel Selector.
 * 0448		CFormat
 ****************************************************************************/

	/*
	 * <1:0>	psel0	PIXEL SELECTOR (must match <9:8> & <17:16>)
	 *			Determines where pixels are routed to/from.
	 *			0: Echo A input 
	 *			1: Echo B input
	 *			2: Select W output 
	 *			3: Echo D input
	 * 
	 * <2>		pe0	PIXEL ENABLE. (Must match <10>, <18>.)
	 *			Determines whether A bus in an output or input.
	 *			0: Input (disable driver).
	 *			1: Output (enable driver).
	 * 
	 * <3>		se	SYNC ENABLE.
	 *			Determines whether sync information is output.
	 *			0: Disable hsync, vsync, odd drivers. 
	 *			1: Output hsync, vsync, odd (enable driver).
	 * 
	 * <4>		xorce	XOR CLOCK ENABLE
	 *			0: Clock enable is true low
	 *			1: Clock enable is true high
	 *
 	 * <5>		xorhs	XOR HORIZONTAL SYNC
	 *			0: Hsync is true low
	 *			1: Hsync is true high
	 *
	 * <6>		xoro	XOR ODD
	 *			0: ODD is true high
	 *			1: ODD is ture low
	 *
	 * <7>		xorpp	XOR PIXPHASE TO CL550. (No clock driver on C.)
	 *			0: Pixphase is true low.
	 *			1: Pixphase is true high.
	 * 
	 * <9:8>	psel1	PIXEL SELECTOR (must match <1:0> & <17:16>)
	 *			Determines where pixels are routed to/from.
	 *			0: Echo A input 
	 *			1: Echo B input
	 *			2: Select W output 
	 *			3: Echo D input
	 * 

	 * <10>		pe1	PIXEL ENABLE. (Must match <2>, <18>.)
	 *			Determines whether A bus in an output or input.
	 *			0: Input (disable driver).
	 *			1: Output (enable driver).
	 * 
	 * <12:11>	cs	CLOCK SOURCE
	 *			If clock is output, determines where it comes
	 *			from.
	 *			0: Echo A clock
	 *			1: Echo B clock
	 *			2: Echo D clock divided by 2
	 *			3: Echo D clock divided by 2
	 *
	 * <13>		xorvs	XOR VERTICAL SYNC
	 *			0: Vsync is true low
	 *			1: Vsync is true high
	 * 
 	 * <14>		shuttle	SHUTTLE MODE
	 *			0: Use/echo Odd as input
	 *			1: If stuck on Odd/Even, oscillate Odd output.
	 * 
	 * <17:16>	psel2	PIXEL SELECTOR (must match <1:0> & <9:8>)
	 *			Determines where pixels are routed to/from.
	 *			0: Echo A input 
	 *			1: Echo B input
	 *			2: Select W output 
	 *			3: Echo D input
	 * 
	 * <18>		pe2	PIXEL ENABLE. (Must match <2>, <10>.)
	 *			Determines whether A bus in an output or input.
	 *			0: Input (disable driver).
	 *			1: Output (enable driver).
	 * 
	 *
	 * <25:24>	csel	CLOCK/SYNC SELECT.
	 * 			0: Echo A input
	 * 			1: Echo B input
	 *			2: Echo C input
	 *			3: Echo D input
	 *
	 * <27:26>	osel	ODD/EVEN SELECT.
	 *			If input:
	 *			0: Odd setup/hold 1us to trailing edge of Vsync
	 *			1: Odd setup/hold 40ns to trailing edge of Vsync
	 *			2: Zero (subject to XOR)
	 *			3: One (subject to XOR)
	 * 			If output:
	 * 			0: Echo filtered Odd (fields forced to 
 	 *			   oscillate, setup to trail).
	 * 			1: Echo unfiltered Odd (Xor only).
	 *			2: Echo Zero.
	 * 			3; Echo One.
	 * 
	 */


#ifdef BITI
	typedef struct {
		unsigned :		4;
		unsigned osel:		2;
		unsigned csel:		2;
		unsigned :		5;
		unsigned pe2:		1;
		unsigned psel2:		2;
		unsigned :		1;
		unsigned shuttle:	1;
		unsigned xorvs:		1;
		unsigned cs:		2;
		unsigned pe1:		1;
		unsigned psel1:		2;
		unsigned ce:		1;
		unsigned xoro:		1;
		unsigned xorhs:		1;
		unsigned xorce:		1; 
		unsigned se:		1;
		unsigned pe0:		1;
		unsigned psel0:		2;
	} CFormat_bits;
#else
	typedef struct {
		unsigned psel0:		2;
		unsigned pe0:		1;
		unsigned se:		1;
		unsigned xorce:		1; 
		unsigned xorhs:		1;
		unsigned xoro:		1;
		unsigned xorpp:		1;
		unsigned psel1:		2;
		unsigned pe1:		1;
		unsigned cs:		2;
		unsigned xorvs:		1;
		unsigned shuttle:	1;
		unsigned :		1;
		unsigned psel2:		2;
		unsigned pe2:		1;
		unsigned :		5;
		unsigned csel:		2;
		unsigned osel:		2;
		unsigned :		0;
	} CFormat_bits;
#endif

	typedef union {
		unsigned psel0;
		unsigned pe0;
		unsigned se;
		unsigned xorce; 
		unsigned xorhs;
		unsigned xoro;
		unsigned xorpp;
		unsigned psel1;
		unsigned pe1;
		unsigned cs;
		unsigned xorvs;
		unsigned shuttle;
		unsigned psel2;
		unsigned pe2;
		unsigned csel;
		unsigned osel;
	} CFormat_lwd;



/****************************************************************************
 * 
 * DFormat	VIDEO CHANNEL D MULTIPLEXOR FORMAT.
 * DF		Controls channel D of the crosspoint switch (normall connected
 * df		to the Philips DMSD chip set).  Significantly different than
 * 		the other crosspoint switch controls.
 * 044c		DFormat
 ****************************************************************************/

	/*
	 * <4>		xorce	XOR CLOCK ENABLE.
	 *			0: Clock enable is true low.
	 *			1: Clock enable is true high.
	 *
 	 * <5>		xorhs	XOR HORIZONTAL SYNC.
	 *	 		0: Hsync/Vsync are true low
	 *			1: Hsync/Vsync are true high
	 * 
	 * <6>		xoro	XOR ODD.
	 * 			0: Odd is true high
	 *  			1: Odd is true low
	 *
 	 * <7>		xorcp	XOR CLOCK PHASE.
	 *
 	 * (repeat byte <7:0> into <23:16> and <15:8> )
	 *
	 * <27:26>	osel	ODD/EVEN SELECT.
	 *			0: Odd setup/hold 1us to trailing edge of Vsync
	 *			1: Odd setup/hold 40ns to trailing edge of Vsync
	 *			2: Zero (subject to XOR)
	 *			3: One (subject to XOR)
	 * 
	 */

#ifdef BITI
	typedef struct {
		unsigned :		4;
		unsigned osel:		2;
		unsigned :		18;
		unsigned xorcp:		1;
		unsigned xoro:		1;
		unsigned xorhs:		1;
		unsigned xorce:		1; 
		unsigned :		4;

	} DFormat_bits;
#else
	typedef struct {
		unsigned :		4;
		unsigned xorce:		1; 
		unsigned xorhs:		1;
		unsigned xoro:		1;
		unsigned xorcp:		1;
		unsigned :		18;
		unsigned osel:		2;
		unsigned :		0;
	} DFormat_bits;
#endif

	typedef union {
		unsigned xorce;
		unsigned xorhs;
		unsigned xoro;
		unsigned xorcp;
		unsigned osel;
	} DFormat_lwd;


/****************************************************************************
 * VFormat	VIDEO PORT V FORMAT.
 * VF		Determines where the channel V video input (one of the actual
 * vf		PPC inputs) gets its data from.
 * 0450		VFormat
 ****************************************************************************/

	/*
	 * <1:0>	psel	PIXEL SELECT
	 * 			0: Use channel A pixel input.
	 * 			1: Use channel B pixel input.
	 * 			2: Use channel C pixel input.
	 * 			3: Use channel D pixel input.
	 * 
	 * <3:2>	ssel	SYNC SELECT.
	 * 			0: Use channel A sync/odd/clock enable inputs.
	 * 			1: Use channel B sync/odd/clock enable inputs.
	 * 			2: Use channel C sync/odd/clock enable inputs.
	 * 			3: Use channel D sync/odd/clock enable inputs.
	 *
	 * <5:4>	cs	CLOCK SOURCE.
	 * 			0: Use channel A clock source.
	 * 			1: Use channel B clock source.
	 *			2: Use D/2 clock source (from DMSD).
	 *			3: Use D/2 clock source (from DMSD).
	 * 
	 * <6>		cp	CLOCK POLARITY.
	 *			0: Clock is true ______
 	 * 			1: Clock is true ______
	 * 
	 * (repeat byte <7:0> into <23:16> and <15:8> )
	 *
	 */

#ifdef BITI
	typedef struct {
		unsigned :	25;
		unsigned cp:	1;
		unsigned cs:	2;
		unsigned ssel:	2;
		unsigned psel:	2;

	} VFormat_bits;
#else
	typedef struct {
		unsigned psel:	2;
		unsigned ssel:	2;
		unsigned cs:	2;
		unsigned cp:	1;
		unsigned :	0;
	} VFormat_bits;
#endif

	typedef union {
		unsigned psel;
		unsigned ssel;
		unsigned cs;
		unsigned cp;
	} VFormat_lwd;


/****************************************************************************
 * WFormat	VIDEO PORT W FORMAT.
 * WF		Determines where the channel W video input/output (one of the
 * wf		PPC video channels) gets its data from (or send it to).
 * 0454		WFormat
 ****************************************************************************/

	/*
	 * <1:0>	psel	PIXEL SELECT
	 * 			0: Use channel A pixel input/output.
	 * 			1: Use channel B pixel input/output.
	 * 			2: Use channel C pixel input/output.
	 * 			3: Use channel D pixel input/output.
	 * 
	 * <3:2>	ssel	SYNC SELECT.
	 * 			0: Use channel A sync/odd/clock enable inputs.
	 * 			1: Use channel B sync/odd/clock enable inputs.
	 * 			2: Use channel C sync/odd/clock enable inputs.
	 * 			3: Use channel D sync/odd/clock enable inputs.
	 *
	 * <5:4>	cs	CLOCK SOURCE.
	 * 			0: Use channel A clock source.
	 * 			1: Use channel B clock source.
	 *			2: Use D/2 clock source (from DMSD).
	 *			3: Use D/2 clock source (from DMSD).
	 * 
	 * <6>		cp	CLOCK POLARITY.
	 *			0: Clock is true ______
 	 * 			1: Clock is true ______
	 * 
	 * (repeat byte <7:0> into <23:16> and <15:8> )
	 *
	 */

#ifdef BITI
	typedef struct {
		unsigned :	25;
		unsigned cp:	1;
		unsigned cs:	2;
		unsigned ssel:	2;
		unsigned psel:	2;

	} WFormat_bits;
#else
	typedef struct {
		unsigned psel:	2;
		unsigned ssel:	2;
		unsigned cs:	2;
		unsigned cp:	1;
		unsigned :	0;
	} WFormat_bits;
#endif

	typedef union {
		unsigned psel;
		unsigned ssel;
		unsigned cs;
		unsigned cp;
	} WFormat_lwd;


/*****************************************************************************
 * 
 * XVIDEO REGISTERS	WRITE REGISTERS
 *
 *			These registers are defined such that references to a
 * 			bitfield result in a longword access to the word 
 *                      containing the bitfield.  The XV_WREG() macro should
 *                      be used to access these registers.  They are
 *                      write-only, but a copy of their contents is kept in
 *                      a shadow RAM (see below); a XV_RSDW() macro will
 *                      return the value of a register or bitfield.
 *
 *****************************************************************************/

typedef struct {
	union {
		StartUp_lwd b;
		unsigned long l;
	} StartUp;			/* 0000 */

	union {
		unsigned long l;
	} MOffset;			/* 0004 */

	unsigned long unused_1;		/* 0008 */

	union {
		WSqzY_lwd b;
		unsigned long l;
	} WSqzY;			/* 000c */

	unsigned long unused_2;		/* 0010-0012 */

	union {
		VSqzY_lwd b;
		unsigned long l;
	} VSqzY;			/* 0014 */

	union {
		VTag_lwd b;
		unsigned long l;
	} VTag;				/* 0018 */

	union {
		WTag_lwd b;
		unsigned long l;
	} WTag;				/* 001c */

	union {
		VSqzX_lwd b;
		unsigned long l;
	} VSqzX;			/* 0020 */

	union {
		WSqzX_lwd b;
		unsigned long l;
	} WSqzX;			/* 0024 */

	union {
		TagCtl_lwd b;
		unsigned long l;
	} TagCtl;			/* 0028 */

	union {
		RopCtl_lwd b;
		unsigned long l;
	} RopCtl;			/* 002c */

	unsigned long unused_3[4];	/* 0030-003c */

	union {
		unsigned long l;
	} VecI1;			/* 0040 */

	unsigned long unused_4;		/* 0044 */

	union {
		int l;
	} VecAcc;			/* 0048 */

	union {
		unsigned long l;
	} VecI2;			/* 004c */
	
	union {
		VecMsc_lwd b;
		unsigned long l;
	} VecMsc;			/* 0050 */

	unsigned long unused_5;		/* 0054 */

	union {
		unsigned long l;
	} M1ForCol;			/* 0058 */

	union {
		unsigned long l;
	} M1BakCol;			/* 005c */

	union {
		unsigned long l;
	} MEnd;				/* 0060 */

	union {
		unsigned long l;
	} M1RdMask;			/* 0064 */

	union {
		unsigned long l; 
	} DMASA1;			/* 0068 */

	union {
		unsigned long l; 
	} DMASA2;			/* 006c */

	union {
		unsigned long l; 
	} DMAEA1;			/* 0070 */

	union {
		unsigned long l; 
	} DMAEA2;			/* 0074 */

	unsigned long unused_6[2];	/* 0078-007c */

	union {
		unsigned long l; 
 	} HFP;				/* 0080 */

	union {
		unsigned long l; 
	} HSync;			/* 0084 */

	union {
		unsigned long l; 
	} HBP;				/* 0088 */

	union {
		unsigned long l; 
	} HDisp;			/* 008c */

	union {
		unsigned long l; 
	} VFP;				/* 0090 */

	union {
		unsigned long l; 
	} VSync;			/* 0094 */

	union {
		unsigned long l;
	} VBP;				/* 0098 */

	union {
		unsigned long l;
	} VDisp;			/* 009c */

	union {
		DispFlgs_lwd b;
		unsigned long l;
	} DispFlgs;			/* 00a0 */

	union {
		unsigned long l;
	} MBank; 			/* 00a4 */

	union {
		IntMask_lwd b;
		unsigned long l;
	} IntMask;			/* 00a8 */

	unsigned long unused_7;		/* 00ac */

	union {
		unsigned long l;
	} HSyncS;			/* 00b0 */

	union {
		unsigned long l;
	} HNSyncS;			/* 00b4 */

	union {
		DispKludge_lwd b;
		unsigned long l;
	} DispKludge;			/* 00b8 */

	unsigned long unused_8[4];	/* 00bc-00c8 */
					
	union {
		Attr_lwd b;
		unsigned long l;	
	} Attr;				/* 00cc */

	union {
		unsigned long l;
	} RasMode;			/* 00d0 */

	union {
		WAttrL_lwd b;
		unsigned long l;
	} WAttrL;			/* 00d4 */

	union {
		WAttrH_lwd b;
		unsigned long l;
	} WAttrH;			/* 00d8 */
	
	union {
		C3Stall_lwd b;
		unsigned long l;
	} C3Stall;			/* 00dc */
	
	union {
		VStart_lwd b;
		unsigned long l;
	} VStart;			/* 00e0 */

	union {
		VStop_lwd b;
		unsigned long l;
	} VStop;			/* 00e4 */

	union {
		WStart_lwd b;
		unsigned long l;
	} WStart;			/* 00e8 */

	union {
		WStop_lwd b;
		unsigned long l;
	} WStop;			/* 00ec */
	
	unsigned long unused_9[3];	/* 00f0-00f8 */

	union {
		TagClip_lwd b;
		unsigned long l;
	} TagClip;			/* 00fc */

	unsigned long unused_10[0xc];	/* 00ff-012c */

	union {
		unsigned long l;
	} DstW;				/* 0130 */

	union {
		unsigned long l;
	} DstH;				/* 0134 */

	unsigned long unused_11[2];	/* 0138 */
					/* 013c */

	union {
		unsigned long l;
	} VectDY;			/* 0140 */

	unsigned long unused_12[0x2f];	/* 0144-01ff */

	union {
		unsigned long l;
	} WindowX;			/* 0200 */

	union {
		unsigned long l;
	} WindowY;			/* 0204 */

	union {
		unsigned long l;
	} VectX0;			/* 0208 */

	union {
		unsigned long l;
	} VectY0;			/* 020c */

	union {
		unsigned long l;
	} TmrInt;			/* 0210 */

	union {
		unsigned long l;
	} TmrSpd;			/* 0214 */

	union {
		BIntMask_lwd b;
		unsigned long l;
	} BIntMask;			/* 0218 */

	unsigned long unused_13;	/* 021c */

	union {
		unsigned long l;
	} MStart;			/* 0220 */

	union {
		I2C_lwd b;
		unsigned long l;
	} I2C;				/* 0224 */

	union {
		MMode_lwd b;
		unsigned long l;
	} MMode;			/* 0228 */

	union {
		unsigned long l;
	} KeyColMask;			/* 022c */

	union {
		unsigned long l;
	} KeyColComp;			/* 0230 */

	union {
		unsigned long l;
	} KeyColCtl;			/* 0234 */

	unsigned long unused_15[2];	/* 023c */

	union {
		unsigned long l;
	} Sema0;			/* 0240 */

	union {
		unsigned long l;
	} Sema1;			/* 0244 */

	union {
		unsigned long l;
	} Sema2;			/* 0248 */

	union {
		unsigned long l;
	} Sema3;			/* 024c */

	union {
		unsigned long l;
	} Sema4;			/* 0250 */

	union {
		unsigned long l;
	} Sema5;			/* 0254 */

	union {
		unsigned long l;
	} Sema6;			/* 0258 */

	union {
		unsigned long l;
	} Sema7;			/* 025c */

	union {
		unsigned long l;
	} Sema8;			/* 0260 */

	union {
		unsigned long l;
	} Sema9;			/* 0264 */

	union {
		unsigned long l;
	} Sema10;			/* 0268 */

	union {
		unsigned long l;
	} Sema11;			/* 026c */

	union {
		unsigned long l;
	} Sema12;			/* 0270 */

	union {
		unsigned long l;
	} Sema13;			/* 0274 */

	union {
		unsigned long l;
	} Sema14;			/* 0278 */

	union {
		unsigned long l;
	} Sema15;			/* 027c */

	unsigned long unused_15a[((0x330 - 0x27c)/4)-1];     	/* 0280-032c */

	union {
		unsigned long l;
	} VectX1;			/* 0330 */

	unsigned long unused_16[3];	/* 0334-033c */

	union {
		unsigned long l;
	} VectY1;			/* 0340 */

	unsigned long unused_17[0x2f];	/* 0334-033c */

	union {
		VMask_lwd b;
		unsigned long l;
	} VMask;			/* 0400 */	

	union {
		WMask_lwd b;
		unsigned long l;
	} WMask;			/* 0404 */

	union {
		DrawMask_lwd b;
		unsigned long l;
	} DrawMask;			/* 0408 */

	union {
		DrawRop_lwd b;
		unsigned long l;
	} DrawRop;			/* 040c */

	union {
		unsigned long l;
	} VStartAdd;			/* 0410 */

	union {
		unsigned long l;
	} WStartAdd;			/* 0414 */

	union {
		unsigned long l;
	} SrcStartAdd;			/* 0418 */

	union {
		unsigned long l;
	}DstStartAdd;			/* 041c */

	union {
		unsigned long l;
	}SolidCol;			/* 0420 */

	union {
		WCtl_lwd b;
		unsigned long l;	
	} WCtl;				/* 0424 */

	union {
		VCtl_lwd b;
		unsigned long l;	
	} VCtl;				/* 0428 */

	unsigned long unused_18[3];

	union {
		OLDI2C_lwd b;
		unsigned long l;
	} OLDI2C;			/* 0438 */

	union {
		unsigned long l;
	} CCube;			/* 043c */

	union {
		AFormat_lwd b;
		unsigned long l;
	} AFormat;			/* 0440 */

	union {
		BFormat_lwd b;
		unsigned long l;
	} BFormat;			/* 0444 */

	union {
		CFormat_lwd b;
		unsigned long l;
	} CFormat;			/* 0448 */

	union {
		DFormat_lwd b;
		unsigned long l;
	} DFormat;			/* 044c */

	union {
		VFormat_lwd b;
		unsigned long l;
	} VFormat;			/* 0450 */

	union {
		WFormat_lwd b;
		unsigned long l;
	} WFormat;			/* 0454 */

	union {
		MiscMode_lwd b;
		unsigned long l;
	} MiscMode;			/* 0458 */

	unsigned long unused_19[9];

	Brooktree_regs Bt;		/* 0480-04bc */

	Ext_regs Ext;			/* 04c0-04cc */

} XV_write_regs;


/*****************************************************************************
 * 
 * READBACK REGISTERS	The following are readback registers for the XVideo
 * 			hardware.  Only these registers, together with the
 * 			Brooktree, C-Cube, and possibly external registers,
 *			can be read back; all other registers are write only.
 *
 *****************************************************************************/

typedef struct {
	union {
		unsigned l;
	} DMARB;			/* 0000 */

	union {
		unsigned l;
	} TmrCnt;			/* 0004 */

	union {
		unsigned l;
	} TmrIntRB;			/* 0008 */

	unsigned unused_1;		/* 000c ro */

	union {
		BIntStat_lwd b;
		unsigned l;
	} BIntStat;			/* 0010 */

	union {
		unsigned l;
	} VCurAdd;			/* 0014 */

	union {
		unsigned l;
	} WCurAdd;			/* 0018 */

	union {
		I2CStat_lwd b;
		unsigned l;
	} I2CStat;			/* 001c */

	unsigned unused_2[4];		/* 0020-002c ro */

	union {
		BIntStatC_lwd b;
		unsigned l;
	} BIntStatC;			/* 0030 */

	unsigned unused_2a[((0x40-0x30)/4)-1];

	union {
		unsigned long l;
	} SemaR0;			/* 0040 */

	union {
		unsigned long l;
	} SemaR1;			/* 0044 */

	union {
		unsigned long l;
	} SemaR2;			/* 0048 */

	union {
		unsigned long l;
	} SemaR3;			/* 004c */

	union {
		unsigned long l;
	} SemaR4;			/* 0050 */

	union {
		unsigned long l;
	} SemaR5;			/* 0054 */

	union {
		unsigned long l;
	} SemaR6;			/* 0058 */

	union {
		unsigned long l;
	} SemaR7;			/* 005c */

	union {
		unsigned long l;
	} SemaR8;			/* 0060 */

	union {
		unsigned long l;
	} SemaR9;			/* 0064 */

	union {
		unsigned long l;
	} SemaR10;			/* 0068 */

	union {
		unsigned long l;
	} SemaR11;			/* 006c */

	union {
		unsigned long l;
	} SemaR12;			/* 0070 */

	union {
		unsigned long l;
	} SemaR13;			/* 0074 */

	union {
		unsigned long l;
	} SemaR14;			/* 0078 */

	union {
		unsigned long l;
	} SemaR15;			/* 007c */

	unsigned unused_2b[((0x240-0x7c)/4)-1];

	union {
		unsigned long l;
	} Sema0;			/* 0240 */

	union {
		unsigned long l;
	} Sema1;			/* 0244 */

	union {
		unsigned long l;
	} Sema2;			/* 0248 */

	union {
		unsigned long l;
	} Sema3;			/* 024c */

	union {
		unsigned long l;
	} Sema4;			/* 0250 */

	union {
		unsigned long l;
	} Sema5;			/* 0254 */

	union {
		unsigned long l;
	} Sema6;			/* 0258 */

	union {
		unsigned long l;
	} Sema7;			/* 025c */

	union {
		unsigned long l;
	} Sema8;			/* 0260 */

	union {
		unsigned long l;
	} Sema9;			/* 0264 */

	union {
		unsigned long l;
	} Sema10;			/* 0268 */

	union {
		unsigned long l;
	} Sema11;			/* 026c */

	union {
		unsigned long l;
	} Sema12;			/* 0270 */

	union {
		unsigned long l;
	} Sema13;			/* 0274 */

	union {
		unsigned long l;
	} Sema14;			/* 0278 */

	union {
		unsigned long l;
	} Sema15;			/* 027c */

	unsigned unsued_3[((0x400-0x27c)/4)-1];		/* 0280-03fc */

	union {
		MiscStat_lwd b;
		unsigned l;
	} MiscStat;			/* 0400 */

	unsigned unused_4;		/* 0404 */

	union {
		IntStat_lwd b;
		unsigned l;
	} IntStat;			/* 0408 */

	union {
		IntStatC_lwd b;
		unsigned l;
	} IntStatC;			/* 040c */

	union {
		unsigned l;
	} DBusy;			/* 0410 */

	unsigned unused_5;		/* 0414 */

	union {
		unsigned l;
	} VBusy;			/* 0418 */

	unsigned unused_6[0x19];	/* 041c-047c */

	Brooktree_regs Bt;		/* 0480-04bc */

	Ext_regs Ext;			/* 04c0-04cc */

} XV_read_regs;



/*****************************************************************************
 *
 * SHADOW RAM	The shadow RAM is somehwat complicated so that the XV_WREG
 * 		and XV_RSDW macros work nicely.  The XV_WREG macro writes
 * 		to the shadow RAM using a bitfield (a read-modify-write 
 * 		cycle, which the XVideo device registers can't do); then
 * 		a longword copy is done from the Shadow RAM location to the
 * 		XVideo device register.	
 * 
 *****************************************************************************/

typedef struct {
	union {
		StartUp_bits b;
		unsigned long l;
	} StartUp;			/* 0000 */

	union {
		unsigned long l;
	} MOffset;			/* 0004 */

	unsigned long unused_1;		/* 0008 */

	union {
		WSqzY_bits b;
		unsigned long l;
	} WSqzY;			/* 000c */

	unsigned long unused_2;		/* 0010-0012 */

	union {
		VSqzY_bits b;
		unsigned long l;
	} VSqzY;			/* 0014 */

	union {
		VTag_bits b;
		unsigned long l;
	} VTag;				/* 0018 */

	union {
		WTag_bits b;
		unsigned long l;
	} WTag;				/* 001c */

	union {
		VSqzX_bits b;
		unsigned long l;
	} VSqzX;			/* 0020 */

	union {
		WSqzX_bits b;
		unsigned long l;
	} WSqzX;			/* 0024 */

	union {
		TagCtl_bits b;
		unsigned long l;
	} TagCtl;			/* 0028 */

	union {
		RopCtl_bits b;
		unsigned long l;
	} RopCtl;			/* 002c */

	unsigned long unused_3[4];	/* 0030-003c */

	union {
		unsigned long l;
	} VecI1;			/* 0040 */

	unsigned long unused_4;		/* 0044 */

	union {
		int l;
	} VecAcc;			/* 0048 */

	union {
		unsigned long l;
	} VecI2;			/* 004c */
	
	union {
		VecMsc_bits b;
		unsigned long l;
	} VecMsc;			/* 0050 */

	unsigned long unused_5;		/* 0054 */

	union {
		unsigned long l;
	} M1ForCol;			/* 0058 */

	union {
		unsigned long l;
	} M1BakCol;			/* 005c */

	union {
		unsigned long l;
	} MEnd;				/* 0060 */

	union {
		unsigned long l;
	} M1RdMask;			/* 0064 */

	union {
		unsigned long l; 
	} DMASA1;			/* 0068 */

	union {
		unsigned long l; 
	} DMASA2;			/* 006c */

	union {
		unsigned long l; 
	} DMAEA1;			/* 0070 */

	union {
		unsigned long l; 
	} DMAEA2;			/* 0074 */

	unsigned long unused_6[2];	/* 0078-007c */

	union {
		unsigned long l; 
 	} HFP;				/* 0080 */

	union {
		unsigned long l; 
	} HSync;			/* 0084 */

	union {
		unsigned long l; 
	} HBP;				/* 0088 */

	union {
		unsigned long l; 
	} HDisp;			/* 008c */

	union {
		unsigned long l; 
	} VFP;				/* 0090 */

	union {
		unsigned long l; 
	} VSync;			/* 0094 */

	union {
		unsigned long l;
	} VBP;				/* 0098 */

	union {
		unsigned long l;
	} VDisp;			/* 009c */

	union {
		DispFlgs_bits b;
		unsigned long l;
	} DispFlgs;			/* 00a0 */

	union {
		unsigned long l;
	} MBank; 			/* 00a4 */

	union {
		IntMask_bits b;
		unsigned long l;
	} IntMask;			/* 00a8 */

	unsigned long unused_7;		/* 00ac */

	union {
		unsigned long l;
	} HSyncS;			/* 00b0 */

	union {
		unsigned long l;
	} HNSyncS;			/* 00b4 */

	union {
		DispKludge_bits b;
		unsigned long l;
	} DispKludge;			/* 00b8 */

	unsigned long unused_8[4];	/* 00bc-00c8 */
					
	union {
		Attr_bits b;
		unsigned long l;	
	} Attr;				/* 00cc */

	union {
		unsigned long l;
	} RasMode;			/* 00d0 */

	union {
		WAttrL_bits b;
		unsigned long l;
	} WAttrL;			/* 00d4 */

	union {
		WAttrH_bits b;
		unsigned long l;
	} WAttrH;			/* 00d8 */

	
	union {
		C3Stall_bits b;
		unsigned long l;
	} C3Stall;			/* 00dc */

	union {
		VStart_bits b;
		unsigned long l;
	} VStart;			/* 00e0 */

	union {
		VStop_bits b;
		unsigned long l;
	} VStop;			/* 00e4 */

	union {
		WStart_bits b;
		unsigned long l;
	} WStart;			/* 00e8 */

	union {
		WStop_bits b;
		unsigned long l;
	} WStop;			/* 00ec */
	
	unsigned long unused_9[3];	/* 00f0-00f8 */

	union {
		TagClip_bits b;
		unsigned long l;
	} TagClip;			/* 00fc */

	unsigned long unused_10[0xc];	/* 00ff-012c */

	union {
		unsigned long l;
	} DstW;				/* 0130 */

	union {
		unsigned long l;
	} DstH;				/* 0134 */

	unsigned long unused_11[2];	/* 0138 */
					/* 013c */

	union {
		unsigned long l;
	} VectDY;			/* 0140 */

	unsigned long unused_12[0x2f];	/* 0144-01ff */

	union {
		unsigned long l;
	} WindowX;			/* 0200 */

	union {
		unsigned long l;
	} WindowY;			/* 0204 */

	union {
		unsigned long l;
	} VectX0;			/* 0208 */

	union {
		unsigned long l;
	} VectY0;			/* 020c */

	union {
		unsigned long l;
	} TmrInt;			/* 0210 */

	union {
		unsigned long l;
	} TmrSpd;			/* 0214 */

	union {
		BIntMask_bits b;
		unsigned long l;
	} BIntMask;			/* 0218 */

	unsigned long unused_13;	/* 021c */

	union {
		unsigned long l;
	} MStart;			/* 0220 */

	union {
		I2C_bits b;
		unsigned long l;
	} I2C;				/* 0224 */

	union {
		MMode_bits b;
		unsigned long l;
	} MMode;			/* 0228 */

	union {
		unsigned long l;
	} KeyColMask;			/* 022c */

	union {
		unsigned long l;
	} KeyColComp;			/* 0230 */

	union {
		unsigned long l;
	} KeyColCtl;			/* 0234 */

	unsigned long unused_15[2];	/* 023c */

	union {
		unsigned long l;
	} Sema0;			/* 0240 */

	union {
		unsigned long l;
	} Sema1;			/* 0244 */

	union {
		unsigned long l;
	} Sema2;			/* 0248 */

	union {
		unsigned long l;
	} Sema3;			/* 024c */

	union {
		unsigned long l;
	} Sema4;			/* 0250 */

	union {
		unsigned long l;
	} Sema5;			/* 0254 */

	union {
		unsigned long l;
	} Sema6;			/* 0258 */

	union {
		unsigned long l;
	} Sema7;			/* 025c */

	union {
		unsigned long l;
	} Sema8;			/* 0260 */

	union {
		unsigned long l;
	} Sema9;			/* 0264 */

	union {
		unsigned long l;
	} Sema10;			/* 0268 */

	union {
		unsigned long l;
	} Sema11;			/* 026c */

	union {
		unsigned long l;
	} Sema12;			/* 0270 */

	union {
		unsigned long l;
	} Sema13;			/* 0274 */

	union {
		unsigned long l;
	} Sema14;			/* 0278 */

	union {
		unsigned long l;
	} Sema15;			/* 027c */

	unsigned long unused_15a[((0x330 - 0x27c)/4)-1];  /* 0280-032c */

	union {
		unsigned long l;
	} VectX1;			/* 0330 */

	unsigned long unused_16[3];	/* 0334-033c */

	union {
		unsigned long l;
	} VectY1;			/* 0340 */

	unsigned long unused_17[0x2f];	/* 0334-033c */

	union {
		VMask_bits b;
		unsigned long l;
	} VMask;			/* 0400 */	

	union {
		WMask_bits b;
		unsigned long l;
	} WMask;			/* 0404 */

	union {
		DrawMask_bits b;
		unsigned long l;
	} DrawMask;			/* 0408 */

	union {
		DrawRop_bits b;
		unsigned long l;
	} DrawRop;			/* 040c */

	union {
		unsigned long l;
	} VStartAdd;			/* 0410 */

	union {
		unsigned long l;
	} WStartAdd;			/* 0414 */

	union {
		unsigned long l;
	} SrcStartAdd;			/* 0418 */

	union {
		unsigned long l;
	}DstStartAdd;			/* 041c */

	union {
		unsigned long l;
	}SolidCol;			/* 0420 */

	union {
		WCtl_bits b;
		unsigned long l;	
	} WCtl;				/* 0424 */

	union {
		VCtl_bits b;
		unsigned long l;	
	} VCtl;				/* 0428 */

	unsigned long unused_18[3];

	union {
		OLDI2C_bits b;
		unsigned long l;
	} OLDI2C;			/* 0438 */
	union {
		unsigned long l;
	} CCube;			/* 043c */
	union {
		AFormat_bits b;
		unsigned long l;
	} AFormat;			/* 0440 */
	union {
		BFormat_bits b;
		unsigned long l;
	} BFormat;			/* 0444 */
	union {
		CFormat_bits b;
		unsigned long l;
	} CFormat;			/* 0448 */
	union {
		DFormat_bits b;
		unsigned long l;
	} DFormat;			/* 044c */
	union {
		VFormat_bits b;
		unsigned long l;
	} VFormat;			/* 0450 */
	union {
		WFormat_bits b;
		unsigned long l;
	} WFormat;			/* 0454 */
	union {
		MiscMode_bits b;
		unsigned long l;
	} MiscMode;			/* 0458 */

	unsigned long unused_19[9];	/* 045c-047c */

	Brooktree_regs Bt;		/* 0480-04bc */

	Ext_regs Ext;			/* 04c0-04cc */

} XV_sdw_bit_regs;

typedef union {
	XV_sdw_bit_regs b;
	XV_write_regs l;	
} XV_shdw_regs;

typedef union {
	volatile XV_write_regs w;
	volatile XV_read_regs r;
} XV_hw_regs;

typedef struct {
	XV_hw_regs *xv;
	XV_shdw_regs sxv;
#ifndef CL_550
	c3shdw     *shdw;
#endif
	volatile C3_regs *c3;
} XV_regs;

#ifdef TRACE
#define XV_WRITE_TRACE(regs, dst, src) cmn_err(CE_CONT, "XV_WREG(dst,src)\n"),
#define XV_READ_TRACE(regs, src) cmn_err(CE_CONT, "XV_RREG(dst,src)\n"),
#else
#define XV_WRITE_TRACE(regs, dst, src)
#define XV_READ_TRACE(regs, src)
#endif

#define XV_WREG4(regs, dst, src)			\
	(XV_WRITE_TRACE(regs, dst, src)			\
	(regs)->sxv.b.dst = (src),			\
	(regs)->sxv.l.dst &= 0xff,			\
	(regs)->sxv.l.dst |= ((regs)->sxv.l.dst)<<8,	\
	(regs)->sxv.l.dst |= ((regs)->sxv.l.dst)<<16,	\
	(regs)->xv->w.dst = (regs)->sxv.l.dst		\
	)

#define XV_WREG(regs, dst, src)				\
	(XV_WRITE_TRACE(regs, dst, src)			\
	(regs)->sxv.b.dst = (src), 			\
	(regs)->xv->w.dst = (regs)->sxv.l.dst		\
	)

#define XV_RREG(regs, src) 				\
	(XV_READ_TRACE(regs, src)			\
	(regs)->xv->r.src				\
	)

#define XV_WSDW(regs, dst, src) 			\
	(XV_WRITE_TRACE(regs, dst, src)			\
	(regs)->sxv.b.dst = (src)			\
	)

#define XV_RSDW(regs, src) 				\
	(XV_READ_TRACE(regs, src)			\
	(regs)->sxv.b.src				\
	)

/*
 * Supply open collector on Sun platforms
*/
#define XV_INT_OC	(0x8000) 

/*
 * Defines for interrupt masks on the XVideo board.
 */
#define XV_INT_BLIT	(XV_INT_OC | 0x80)    
#define XV_INT_WERR	(XV_INT_OC | 0x40)    
#define XV_INT_VERR	(XV_INT_OC | 0x20)    
#define XV_INT_WSYNC	(XV_INT_OC | 0x10)    
#define XV_INT_VSYNC	(XV_INT_OC | 0x08)    
#define XV_INT_EXT	(XV_INT_OC | 0x04)    
#define XV_INT_CCUBE	(XV_INT_OC | 0x02)    
#define XV_INT_DISP	(XV_INT_OC | 0x01)    
#define XV_BINT_DMA	(0x01)
#define XV_BINT_TIMER	(0x02)
#define XV_BINT_SEMA	(0x04)
#define XV_BINT_BUSERR	(0x08)

#endif
