/*
 * Copyright (c) 1993 Parallax Graphics Inc.
 */

#ifndef _TVTWO_H_
#define _TVTWO_H_

#pragma ident "@(#)tvtwo.h	1.6 Copyright (c) 1993 Parallax Graphics Inc"

#include <sys/t_lock.h>
#include <sys/ddi.h>
#include <sys/sunddi.h>
#include <sys/errno.h>

#include "xvregs.h"
#include "nvram.h"
#include "tvtwo_fb.h"
#include "krtv.h"

#define TVTWO_PROMID "PGI,tvtwo"
#define TVTHREE_PROMID "PGI,tvthree"
#define PLX_DISPLAY (('P'<<8)|0)

#ifdef DEBUG
static tvtwo_debug = 1;
#define	_STMT(a) a
#define	DEBUGF(args) _STMT(if (tvtwo_debug) cmn_err args)
#else
#define	DEBUGF(args)
#endif

extern void *state_p;

/*
 * Macros to turn devi/dev arguments into softc pointers and
 * derive instance numbers from minor.
 *
 *	Minor		Function
 *	---------	---------
 *	0 (000)		tvtwo
 *	1 (001)		nvram
 *	2 (010)		option card nvram
 *	3 (011)		tuner (?)
 *	4 (100)		ccube
 *	5 (101)		cgthree
 */
#define TVTWO_MINOR_SHIFT	(3)
#define TVTWO_MINOR_MASK	((1<<TVTWO_MINOR_SHIFT)-1)
#define TVTWO_TV2_MINOR		(0)
#define TVTWO_NVRAM_MINOR	(1)
#define TVTWO_OPT_MINOR		(2)
#define TVTWO_TUNER_MINOR	(3)
#define TVTWO_C3_MINOR		(4)
#define TVTWO_CG3_MINOR		(5)

#define	CCUBE_STATE_ERROR	-1
#define	CCUBE_STATE_IDLE	0

#define TVTWO_INST2DEV(inst)	((inst)<<TVTWO_MINOR_SHIFT)
#define	TVTWO_DEV2INST(m)	(getminor(m) >> TVTWO_MINOR_SHIFT)
#define TVTWO_DEV2MINOR(m)	(getminor(m) & TVTWO_MINOR_MASK)

#define TVTWO_INST2MINOR_TV2(inst)	(TVTWO_INST2DEV(inst)+TVTWO_TV2_MINOR)
#define TVTWO_INST2MINOR_C3(inst)	(TVTWO_INST2DEV(inst)+TVTWO_C3_MINOR)
#define TVTWO_INST2MINOR_NVRAM(inst)	(TVTWO_INST2DEV(inst)+TVTWO_NVRAM_MINOR)
#define TVTWO_INST2MINOR_OPT(inst)	(TVTWO_INST2DEV(inst)+TVTWO_OPT_MINOR)
#define TVTWO_INST2MINOR_TUNER(inst)	(TVTWO_INST2DEV(inst)+TVTWO_TUNER_MINOR)
#define TVTWO_INST2MINOR_CG3(inst)	(TVTWO_INST2DEV(inst)+TVTWO_CG3_MINOR)

#define	TVTWO_DEVI2SOFTC(devi) \
	((tvtwo_softc *)ddi_get_soft_state(state_p, ddi_get_instance(devi)))
#define	TVTWO_DEV2SOFTC(dev) \
	((tvtwo_softc *)ddi_get_soft_state(state_p, TVTWO_DEV2INST((dev))))

/*
 * Following is mainly for ddi_map_regs(9F)
 */

#if defined(TV3) && defined(DDI25)

#define TVTWO_UNMAP_REGS(softc) \
	ddi_regs_map_free(&(softc)->acc_hand)

#define TVTWO_MAP_REGS(softc, reg, ptr, attr) \
        ddi_regs_map_setup((softc)->devi, \
			   (reg).regnum + 1, \
			   (caddr_t *)&(ptr), \
			   (reg).offset, \
			   (reg).size, \
			   &attr, \
			   &(softc)->acc_hand)
#else
#define TVTWO_UNMAP_REGS(softc, reg, ptr) \
	ddi_unmap_regs((softc)->devi, \
		(reg).regnum, \
		(caddr_t *)&(ptr), \
		(reg).offset, \
		(reg).size \
		)
#define TVTWO_MAP_REGS(softc, reg, ptr) \
	ddi_map_regs((softc)->devi, \
		(reg).regnum, \
		(caddr_t *)&(ptr), \
		(reg).offset, \
		(reg).size \
		)
#endif

typedef struct {
	u_int regnum;
	u_int base;	/* for mmap - base offset from device address 0 */
	off_t offset;
	off_t size;
	off_t voff;	/* for mmap - request address from userland */
} XV_regmap;

/*
 * This structure contains all our "virtual" register windows for
 * the XVideo board.
 */
typedef struct {
	XV_regmap xv;
	XV_regmap c3;
	XV_regmap m1;
	XV_regmap m8;
	XV_regmap m24;
	XV_regmap prom;
} XV_regmaps;

static ddi_dma_lim_t ccube_limits = {
	0,
	0xffffffff,
	0xffffffff,
	(u_int)1,
	(u_int)1,
	0,
	};
/*
 * TVTWO address map
 * Dave has modified this to accomodate 8k page sizes.
 * Prom, and CCube offsets are bumped up to 8k boundries.
 *
 * The address space presented to the user via mmap.
 *
 *	0x000000	1 bit framebuffer
 *	0x040000	8 bit framebuffer
 *	0x140000	true color framebuffer
 *	0x800000	XVideo registers
 *  old way
 *	0x801000	prom
 *	0x811000	CCube registers
 *  new way
 *	0x802000	prom
 *	0x812000	CCube registers
 */

#define M1BASE		((int)(((XV_mem_map *)0)->M1))
#define M1SIZE		(0x28000)
#define M1VOFF		(0x0)

#define XVBASE		((int)(&((XV_mem_map *)0)->RW_regs))
#define XVSIZE		(sizeof(XV_hw_regs))
#define XVVOFF		(0x800000)

#define C3BASE		((int)(&((XV_mem_map *)0)->Ccube))
#define C3SIZE		(sizeof(C3_regs))
#define C3VOFF		(0x812000)

#define M8BASE		((int)(((XV_mem_map *)0)->M8))
#define M8SIZE		(0x100000)
#define M8VOFF		(0x40000)
#define CG2M8VOFF	(0x100000) /* for cgtwo emulation */

#define M24BASE		((int)(((XV_mem_map *)0)->M24))
#ifdef TV3
#define M24SIZE		(0x500000)
#else /* !TV3 */
#define M24SIZE		(0x400000)
#endif /* TV3 */
#define M24VOFF		(0x140000)

#define PROMBASE	((int)(((XV_mem_map *)0)->Prom1))
#define PROMSIZE	(0x10000)
#define PROMVOFF	(0x802000)

/*
 * Old prom (w/override from tvtwo.conf) defines.
 */
#define XVREGNUM 0
#define XVREGOFF 0
#define XVREGSIZE XVSIZE
#define C3REGNUM 0
#define C3REGOFF (C3BASE-XVBASE)
#define C3REGSIZE C3SIZE
#define M1REGNUM 1
#define M1REGOFF 0
#define M1REGSIZE M1SIZE
#define M8REGNUM 0
#define M8REGOFF (M8BASE-XVBASE)
#define M8REGSIZE M8SIZE
#define M24REGNUM 0
#define M24REGOFF (M24BASE-XVBASE)
#define M24REGSIZE M24SIZE
#define PROMREGNUM 0
#define PROMREGOFF (PROMBASE-XVBASE)
#define PROMREGSIZE PROMSIZE


#define WCHAN_IN  1
#define WCHAN_OUT 2

#define CCUBE_R0 0
#define CCUBE_R1 1
#define CCUBE_R2 2
#define CCUBE_W0 3
#define CCUBE_W1 4
#define CCUBE_W2 5
#define CCUBE_W3 6
#define CCUBE_D1 7
#define CCUBE_D2 8
#define CCUBE_D3 9
#define VCHAN_INTERRUPT 10
#define WCHAN_INTERRUPT 11
#define CCUBE_INTERRUPT 20
#define PROCESS_EVENT   21
#define CCUBE_TRACE     30

#define	_Int_Dma	0x0001
#define	_Int_Timer	0x0002

/* intr values for imask */
#define	_Int_Display	0x0001
#define	_Int_CCube	0x0002
#define	_Int_Ext	0x0004
#define	_Int_Vid	0x0008
#define	_Int_Wid	0x0010
#define	_Int_None1	0x0020
#define	_Int_None2	0x0040
#define	_Int_Blit	0x0080

#define CHAN0		1
#define CHAN1		2
#define CHAN2		4
#define CHAN3		8
#define NMARKS 10
#define NDBGBUF 100*NMARKS
#define RTV_HOLD    10

#ifdef CRASH_DBG
#define mark( string, val1, val2 )	subr_mark(softc, __LINE__ , val1, val2 )
#else
#define mark( module,index,val1, val2 )	subr_mark(softc, module,index , val1, val2 )
#endif

#define NET 1
#define DISK 2
typedef struct dvss_debug_buf {
	int		line;
	int		time;
	int		module;
	u_int		dma_addr;
	int		VcurAdd;
	int		WcurAdd;
	u_int		value1;
	u_int		value2;
} dvss_debug_buf_t;

struct mark_buf {
	char            marker[8];
	dvss_debug_buf_t Dbg_buf[NDBGBUF];
}               Mark_buf;

static int snd_hdr[4];

#define TVTWO_PROMID "PGI,tvtwo"
/*
 * XVideo board registers
 *
 * Old xvideo prom registers:
 *
 *	R0	40000 - 40800
 *	R1	80000 - 17d200
 *
 * New xvideo prom registers
 *
 *	R0	0 - 7fffff
 *
 * Old registers overridden by tvtwo.conf to become:
 *
 *	R0	40000 - 7fffff
 *	R1	10000 -  37fff
 *
 * In order for the OS to read the register override the first
 * register offset must match the reg offset in the prom (see sbus(4)).
 *
 *	registers=-1,0x40000,0x7c0000, -1,0x10000,0x28000;  
 */

typedef struct {
	u_char		Prom0[PROMSIZE];		/*      0 -   ffff */
	u_char		M1[M1SIZE];			/*  10000 -  37fff */
	u_char		unused0[0x8000];		/*  38000 -  3ffff */
	XV_hw_regs	RW_regs;			/*  40000 -  404d0 */
	u_char		unused1[0x10000 - XVSIZE];	/*  404d4 -  4ffff */
	C3_regs		Ccube;				/*  50000 -  5ffff */
	u_char		unused2[0x30000 - C3SIZE];	/*  60000 -  7ffff */
	u_char		M8[M8SIZE];			/*  80000 - 17ffff */
	u_char		unused3[0x80000];		/* 180000 - 1fffff */
	u_long		M24[M24SIZE/sizeof(u_long)];	/* 200000 - 6fffff */
	u_char		unused4[0xf0000];		/* 700000 - 7effff */
	u_char		Prom1[PROMSIZE];		/* 7f0000 - 7fffff */
} XV_mem_map;

typedef struct ccube_info {
	int		unit;		/* unit number */
	int 		state;		/* CCUBE_STATE_??? */
	int		timer_state;  /* State to return to after timeout */
	int		timer_count;	/* to deal with intr's lost */
	int		timer_active;	/* to deal with intr's lost */
	int		timer_required;	/* flag for timer usage */
	int		mode;		/* CCUBE_MODE_{READ|WRITE} */
	u_long 		imask;		/* intr' mask before and after call */
	u_long		bmask;		/* intr' mask before and after call */
	u_long  	istat;		/* intr' status before call */
	u_long		bstat;		/* intr' status before call */
	caddr_t		start_addr;	/* start of xfer address */
	caddr_t		end_addr;	/* end of xfer address */
	unsigned long	count;		/* requested count */
	unsigned long	rcount;		/* actual count */
}CCUBE_INFO;

typedef struct {
	int prev_qindex[2];
	int prev_rate[2];
	short qfactor[2];
	}RATE_INFO;

typedef struct {
	int start_x;
	int start_y;
	int width;
	int height;
	int video_tag;
	int current_rate;
	}WTBL;
/*
 * Per-instance data
 */
typedef struct {

	/*
	 * Our mapped registers
	 */
	off_t reg_offset;		/* offset to get to registers */
	off_t reg_size;			/* sizeof registers */

	/*
	 * Mutex and condition variables
	 */
	kmutex_t regs_mutex;		/* mutex for ... */
	XV_regs *regs;

	kmutex_t softc_mutex;		/* mutex for this structure */

	/*
	 * Cookies.
	 */
	ddi_iblock_cookie_t iblkc;	/* for blocking intrs */
	ddi_idevice_cookie_t idevc;	/* for interrupt vector information */
	ddi_dma_handle_t dma;		/* for our dma handle */


	/*
	 * Information pointers
	 */
	XV_regmaps *regmap;		/* how to map our registers */
	dev_info_t *devi;		/* for dev-to-devi conversion */
	u_int maxphys;			/* for tvtwo_minphys */

	struct buf *bp;			/* buf reference for tvtwo_intr */
	caddr_t addr;			/* phys dma transfer address */

	u_int state;			/* dma state variable */
	kmutex_t dma_lock;		/* mutex for ... */
	kcondvar_t dmadone;		/* dma complete/error */

	u_int bstat, istat;		/* saved interrupt status bits */

	/*
	 * Timout handles
	 */
	int tid;			/* timeout identifier */
	int timeout;			/* flag that tid is active */

	/*
	 * Configuration information.
	 */
	XV_nvram nvram;

	/*
	 * Framebuffer specific information
	 */
	tvtwo_fbd fbd;
    /*
     * field flash, and compression synchronization flags
     */
       int ffield;             /* continuously flash this field */
       int sfield;             /* syncronize compression with this field */
       int visleep;    /* sleep and wakeup flags for sync. */
       kcondvar_t viwake;

/*
 *  Beginning of RTV additions.
 */
	kmutex_t rtv_dma_lock;		/* mutex for rtv I/O ... */
	kcondvar_t rtv_dma_done;		/* dma complete/error */

	ddi_dma_handle_t ccube_read_dma;	/* for our read dma handle */
	ddi_dma_handle_t ccube_write_dma;	/* for our write dma handle */
	/*
	 *  Ccube field processing flags and counters.
	 */
	int   request_size;
	int   actual_input_size;
	int   actual_output_size;
	int   single_field;
	int   dma_size;
	int   rtv_mode;
	int   ccube_comp_state;
	int   ccube_decomp_state;
	int   ccube_event;
	int   start_decomp;		/* flag set by I_START message */
	int   v_chan_state;		/* Vid Channel state */
	int   w_chan_state;		/* Wid Channel state */
	int   ccube_error;
	int   dma_state;
	int   rtv_dma_busy;
	int   rtv_dma_state;
	int   channels_displayed;
	int   flds_done;
	int   interlace;
	int   Vidcount;
	int   Widcount;
	int   record_size;
	int   record_window;
	int   play_window;
	int   frame_count;
	int   overflow;
	int   record_start_pixel[4];
	int   play_start_pixel[4];
	int   skip_fields;
	int   field_offset[2];
	int   field_size;
	int   field_polarity;
	short   field_qfactor;
	short   field_qindex;
	int   compress_qindex;
	int   decomp_qindex;
	int   record_action;
	int   play_action;
	int   xmit_action;
	int   recv_action;
	int   stream_id;
	int   enable_interrupts;
	u_int interrupt_bits;
	int   min_queue;
	int   min_rate;
	int   max_field_size;
	int   max_rate;
	int   que_level_in[4];
	int   que_level_out[4];
	int   frame_rate;
	int   first_qindex;
	int   stream_mask;
	u_int start_time;
	u_int end_time;
	int   field_time;
	int   intr_count;
	int   w_chan_mode;
	int   ccube_operation;
	int   media;
	WTBL  play_wintbl[4];
	WTBL  record_wintbl[4];
	RATE_INFO rate_info;

	caddr_t dma_base;
	caddr_t dma_input_addr;
	caddr_t dma_output_addr;

	caddr_t input_shared_data;
	caddr_t output_shared_data;
	caddr_t bufr_base;
	caddr_t next_bufr;
	u_short *qtab;
	u_short *htab;
	u_int imask, bmask;

	FIELD_INFO *field;

	CCUBE_INFO *ci;

       SHM_FIELD_QUE  *shm_que;      /* shared memory field que info */
#ifdef TV3
	int     pll_good_count;			 /* Number of times overlay in synch. */
	int     pll_minor_adjust;        /* # of times in minor adjustment. */
	ddi_acc_handle_t acc_hand;	/*  ddi access handle */
#endif

} tvtwo_softc;

extern int
tvtwo_init(tvtwo_softc *softc);

extern void
tvtwo_uninit(tvtwo_softc *softc);

extern int
tvtwo_reinit(tvtwo_softc *softc);

extern int
tvtwo_nvram_init(XV_regs *regs, XV_nvram *nvram);

extern int
tvtwo_nv_write(XV_regs *regs, dev_t dev, paddr_t prt, int nv_addr, int count);

extern int
tvtwo_nv_read(XV_regs *regs, dev_t dev, paddr_t ptr, int nv_addr, int count);

extern int
c3_start_dma(tvtwo_softc *softc, caddr_t addr, struct buf *bp);

extern void
c3_int_svc(tvtwo_softc *softc);

extern void
xvbuf_int_svc(tvtwo_softc *softc);

extern int
tvtwo_ioctl_fb(dev_t, int, int, int, cred_t *, int *);

extern void
tvtwo_service(tvtwo_softc *softc, void (*[])(), u_int bits);

extern void
tvtwo_continue_dma(tvtwo_softc *softc);

extern void
tvtwo_pause_dma(tvtwo_softc *softc);

extern void
tvtwo_init_fb(tvtwo_softc *softc);

#endif
