/*
 * C3 state machine. all access to the reading and writing of the C3
 * chip is done here.  It controls it's own state and expects that calls
 * are made as described below.
 */

#ident "@(#)$Id$"

#include	<sys/param.h>
#include	<sys/types.h>
#include	<sys/kmem.h>

#include 	"tvtwo.h"
/*
#define DELAX(x) DELAY(delay_multi*(x))
#define TRACE_STATE_TIME
*/
#define DELAX(x) DELAY(x)
int delay_multi=1;

#define	_Int_Dma	0x0001
#define	_Int_CCube	0x0002
#define _Int_Sbus	0x0008

#ifdef  TRACE_STATE_TIME
extern int trace_state();
#endif
/*
 * The following are provide as public routines.
 *
 *
 *	ccube_init_state(softc);
 *
 *
 *	ccube_status(softc, &count);
 */


#define	CCUBE_STATE_TIMEOUT	1
#define	CCUBE_STATE_R1		2
#define	CCUBE_STATE_R2		3
#define	CCUBE_STATE_W1		4
#define	CCUBE_STATE_W2		5
#define	CCUBE_STATE_W3		6

#define	CCUBE_MODE_UNKNOWN	0
#define	CCUBE_MODE_READ		1
#define	CCUBE_MODE_WRITE	2

#define	CCUBE_ACTION_NONE	0
#define	CCUBE_ACTION_STARTWRITE	1
#define	CCUBE_ACTION_STARTREAD	2
#define	CCUBE_ACTION_INTR_BUF	3
#define	CCUBE_ACTION_INTR_CCUBE	4
#define	CCUBE_ACTION_INTR_VID	5
#define	CCUBE_ACTION_INTR_WID	6
#define	CCUBE_ACTION_TIMER	7
#define	CCUBE_ACTION_ABORT	8

#define	CCUBE_MAX_DEVICES	1


static void ccube_statemachine();
static void ccube_error();

static void ccube_timer_off();
static void ccube_timer_start();
int dbg_status;
int cur_addr;
int mark_index;
int c3_flags;

/*
 * public routines
 */

int
ccube_init_state(tvtwo_softc *softc)
{
	CCUBE_INFO *ci= softc->ci;

	if(ci == NULL)
	   softc->ci=ci =(CCUBE_INFO *)kmem_zalloc(sizeof(CCUBE_INFO),KM_SLEEP);
	ci->state = CCUBE_STATE_IDLE;
	ci->timer_count = 0;
	ci->timer_active = 0;
	ci->timer_required = 0;
	ci->mode = CCUBE_MODE_UNKNOWN;
	ci->imask = 0;
	C3_WREG(softc->regs,Flags,C3_OFF);
	return (1);
}

static int
c3_wait_for_vnac(tvtwo_softc *softc)
{
	/*
	 * Wait for the video to go active and enable interrupts.
	 * Instead of hanging the machine if something goes wrong
	 * we fail the request.
	 */

	int cnt = 0;
	while((c3_flags= C3_RREG(softc->regs, Flags)) & C3_VInactive) {
		DELAY(5);
		if (cnt++ > 200) {
			cmn_err(CE_WARN,
				"ccube: VInactive never went low (0x%x)\n",
				C3_RREG(softc->regs, Flags));
#ifdef TRACE_STATE_TIME
			    trace_state(softc,15,0);
#endif
			return (DDI_FAILURE);
		}
	}

	return (DDI_SUCCESS);
}
int
ccube_read(tvtwo_softc *softc) 
{
	CCUBE_INFO *ci = softc->ci;

	switch (ci->state) {
	case CCUBE_STATE_ERROR:
	case CCUBE_STATE_TIMEOUT:
	case CCUBE_STATE_IDLE:
		break;
	default:
		printf("ccube: state error: read request: state=%d mode=%d\n",
			ci->state,
			ci->mode);
		return (-1);
	}

	switch (ci->mode) {
	case CCUBE_MODE_UNKNOWN:
	case CCUBE_MODE_WRITE:
		/* XXX change mode to read from write */
		ci->mode = CCUBE_MODE_READ;
		break;
	case CCUBE_MODE_READ:
		/* nothing to do */
		break;
	}

	/* save away the new address and count */
	ci->start_addr = softc->dma_input_addr;
	ci->count = softc->record_size;
	ci->end_addr = (caddr_t)0;
	ci->rcount = 0;

	ci->imask = softc->imask;
	ci->bmask = softc->bmask;

	ccube_statemachine(softc, CCUBE_ACTION_STARTREAD, 0);

	softc->imask = ci->imask;
	softc->bmask = ci->bmask;

	switch (ci->state) {
	case CCUBE_STATE_ERROR:
		return (-1);
	case CCUBE_STATE_IDLE:
		return (0);
	}

	return (1);
}

int
ccube_write(tvtwo_softc *softc )
{
	CCUBE_INFO *ci = softc->ci;

	switch (ci->state) {
	case CCUBE_STATE_ERROR:
	case CCUBE_STATE_TIMEOUT:
	case CCUBE_STATE_IDLE:
		break;
	default:
		printf("ccube: state error: write request: state=%d mode=%d\n",
			ci->state,
			ci->mode);
		return (-1);
	}

	switch (ci->mode) {
	case CCUBE_MODE_UNKNOWN:
	case CCUBE_MODE_READ:
		/* XXX change mode to write from read */
		ci->mode = CCUBE_MODE_WRITE;
		break;
	case CCUBE_MODE_WRITE:
		/* nothing to do */
		break;
	}

	/* save away the new address and count */
#ifdef DUAL_DMA
	ci->start_addr = softc->dma_output_addr;
#else
	ci->start_addr = softc->dma_input_addr;
#endif
	ci->count = softc->field_size;
	ci->end_addr = (caddr_t)0;
	ci->rcount = 0;

	ci->imask = softc->imask;
	ci->bmask = softc->bmask;

	ccube_statemachine(softc, CCUBE_ACTION_STARTWRITE, 0);

	softc->imask = ci->imask;
	softc->bmask = ci->bmask;

	switch (ci->state) {
	case CCUBE_STATE_ERROR:
		return (-1);
	case CCUBE_STATE_IDLE:
		return (0);
	}

	return (1);
}

int
process_c3_intr(tvtwo_softc *softc)
{
CCUBE_INFO *ci;
	
	
	ci = softc->ci;
	ci->imask = softc->imask;
	ci->bmask = softc->bmask;

#ifdef TRACE_BITS
      cmn_err(CE_WARN,"!c3_intr: bits=0x%x\n",softc->interrupt_bits);
#endif
	if (softc->interrupt_bits & 0x0008) {
		ccube_statemachine(softc, CCUBE_ACTION_INTR_CCUBE, 0);
		softc->interrupt_bits &= ~0x0008;
		softc->imask = ci->imask;
		softc->bmask = ci->bmask;
	}

	switch (ci->state) {
	case CCUBE_STATE_ERROR:
		return (-1);
	case CCUBE_STATE_IDLE:
		return (0);
	}

	if (softc->interrupt_bits & 0x0001) {
		ccube_statemachine(softc, CCUBE_ACTION_INTR_BUF, 0);
		softc->interrupt_bits &= ~0x0001;
		softc->imask = ci->imask;
		softc->bmask = ci->bmask;
	}

	switch (ci->state) {
	case CCUBE_STATE_ERROR:
		return (-1);
	case CCUBE_STATE_IDLE:
		return (0);
	}

	return (1);
}

int
ccube_status(tvtwo_softc *softc)
{
	CCUBE_INFO *ci= softc->ci;


	switch (ci->state) {
	case CCUBE_STATE_ERROR:
	case CCUBE_STATE_TIMEOUT:
		ci->state = CCUBE_STATE_IDLE;
		return (-1);
	case CCUBE_STATE_IDLE:
		return (0);
	}

	return (1);
}

/*
 * private routines
 */
static void
ccube_r0(tvtwo_softc *softc)
{
	u_int  start_address, end_address;
	unsigned int c3_status;
	int	wcount;
	CCUBE_INFO *ci;

	ci = softc->ci;
#ifdef  MARKER_TRACE_INPUT
        mark(CCUBE_R0,1,ci->start_addr,ci->count);
#endif
#ifdef  TRACE_STATE_TIME
	trace_state(softc,1,0);
#endif
	start_address = (( u_int)(ci->start_addr)) >> 2;
	end_address = (u_int)(ci->start_addr + ci->count -4 ) >> 2;

	/* Set initial state */

	XV_WREG(softc->regs,DMASA1.l, start_address);	
	XV_WREG(softc->regs,DMASA2.l, start_address);	
	XV_WREG(softc->regs,DMAEA1.l, end_address);
	XV_WREG(softc->regs,DMAEA2.l, end_address);

	/*
	 * Purge FIFO
	 */

	XV_WREG(softc->regs,CCube.l, XVC3_START_DMA_TO_HOST_1);
	DELAX(1);
	XV_WREG(softc->regs,CCube.l, XVC3_START_DMA_TO_HOST_2);
	DELAX(1);
	XV_WREG(softc->regs,CCube.l, XVC3_PURGE_BIC_GO_PIO);
	DELAX(10);
	XV_WREG(softc->regs,CCube.l, XVC3_RESET_TO_PIO);
	XV_WREG(softc->regs,C3Stall.b.cond, STALL_H);

	/* Load the DMA buffer address */

	XV_WREG(softc->regs,DMASA1.l, start_address);	
	XV_WREG(softc->regs,DMASA2.l, start_address);	
	XV_WREG(softc->regs,DMAEA1.l, end_address);
	XV_WREG(softc->regs,DMAEA2.l, end_address);

	/* start the compressor */

	C3_WREG(softc->regs,HV_Enable, C3_ON);
	C3_WREG(softc->regs,HV_Enable, C3_OFF);
	C3_WREG(softc->regs,HV_Enable, C3_ON);
	C3_WREG(softc->regs,Start, C3_ON);

	if (c3_wait_for_vnac(softc) != DDI_SUCCESS) {
			return;
	}
#ifdef  MARKER_TRACE_INPUT
        mark(CCUBE_R0,2,c3_flags,0);
#endif

	XV_WREG(softc->regs,C3Stall.b.cond, STALL_H_O_FE);

	C3_WREG(softc->regs,NMRQ_Enable, C3_VInactive);

	/* Start the DMA going */
	C3_WREG(softc->regs,DRQ_Enable, _Int_Codec_RegBusy | _Int_FIFO_QFull);

	XV_WREG(softc->regs,CCube.l, XVC3_START_DMA_TO_HOST_1);
	XV_WREG(softc->regs,CCube.l, XVC3_START_DMA_TO_HOST_2);

	ccube_timer_start(softc);

	ci->imask |= _Int_CCube;
	ci->bmask |= _Int_Dma;
	ci->state = CCUBE_STATE_R1;
#ifdef  TRACE_STATE_TIME
	trace_state(softc,2,0);
#endif
	return;
}

static int
ccube_r1_cc(tvtwo_softc *softc)
{
	unsigned long end_addr, c3_status;
	int delta;
	CCUBE_INFO *ci= softc->ci;

#ifdef  TRACE_STATE_TIME
	delta=trace_state(softc,3,0);
#endif
        XV_WREG(softc->regs,C3Stall.b.cond, STALL_ALWAYS);
	end_addr = XV_RREG(softc->regs, DMARB.l);
	end_addr += 4;
#ifdef OverFlowTest
	ci->end_addr = (caddr_t)end_addr;
	if(delta> 1500)
	    cmn_err(CE_WARN,"! delta=%d qfactor=%d size=%d\n",
		delta,softc->field_qfactor,(int)(ci->end_addr-ci->start_addr));
#endif

#ifdef  MARKER_TRACE_INPUT
	/* Marker trace code */
        mark(CCUBE_R1,1,c3_flags,0);
#endif

	XV_WREG(softc->regs,CCube.l, XVC3_PAUSE_DMA_TO_HOST);
	DELAX(50);
	XV_WREG(softc->regs,CCube.l, XVC3_PURGE_BIC_GO_PIO);
	DELAX(10);
	c3_status = C3_RREG(softc->regs, Flags);

#ifdef  MARKER_TRACE_INPUT
	mark(CCUBE_R1,2,c3_status,0);
#endif

	C3_WREG(softc->regs,NMRQ_Enable, 0);

	C3_WREG(softc->regs,DRQ_Enable, 0);

	/* 
	 * The video pipeline is now stopped.  Now tell the C3
	 * to stop and drain the FIFO.
	 */
	if ((c3_status & C3_VInactive) == 0) {
		ccube_timer_off(ci);
		ccube_error(ci, "Bad Interrupt");

		C3_WREG(softc->regs,Flags, 0);
		C3_WREG(softc->regs,Flags, ~0);
		C3_WREG(softc->regs,Flags, 0);
		C3_WREG(softc->regs,DRQ_Enable, 0);
		C3_WREG(softc->regs,NMRQ_Enable, 0);
		XV_WREG(softc->regs,CCube.l, XVC3_RESET_TO_PIO);

		ci->rcount = -1;
		ci->imask &= ~ _Int_CCube;
		ci->bmask &= ~ _Int_Dma;
		ci->state = CCUBE_STATE_ERROR;
		return;
	}
	ccube_timer_start(softc);


	C3_WREG(softc->regs,HV_Enable, ~0);
	C3_WREG(softc->regs,HV_Enable, 0);
	C3_WREG(softc->regs,HV_Enable, ~0);
	C3_WREG(softc->regs,Start, 0);

	C3_WREG(softc->regs,DRQ_Enable, _Int_Codec_RegBusy | _Int_FIFO_NotEmpty);

	C3_WREG(softc->regs,NMRQ_Enable, _Int_FIFO_Empty);

	XV_WREG(softc->regs,CCube.l, XVC3_CONT_DMA_TO_HOST_1);
	XV_WREG(softc->regs,CCube.l, XVC3_CONT_DMA_TO_HOST_2);

	/*  Marker trace code */
#ifdef  MARKER_TRACE_INPUT
        mark(CCUBE_R1,3,c3_status,0);
#endif

	ci->imask |= _Int_CCube;
	ci->bmask |= _Int_Dma;
	ci->state = CCUBE_STATE_R2;
#ifdef  TRACE_STATE_TIME
	trace_state(softc,4,0);
#endif
	return;
}

static int
ccube_r2_cc(tvtwo_softc *softc)
{
	unsigned long end_addr, c3_status;
	CCUBE_INFO *ci= softc->ci;

#ifdef  TRACE_STATE_TIME
	trace_state(softc,5,0);
#endif
        XV_WREG(softc->regs,C3Stall.b.cond, STALL_ALWAYS);
	end_addr = XV_RREG(softc->regs,DMARB.l);
	end_addr += 4;

	XV_WREG(softc->regs,CCube.l, XVC3_PAUSE_DMA_TO_HOST);
	DELAX(50);
	XV_WREG(softc->regs,CCube.l, XVC3_PURGE_BIC_GO_PIO);
	DELAX(10);
	c3_status = C3_RREG(softc->regs, Flags);
	C3_WREG(softc->regs,NMRQ_Enable, 0);
	C3_WREG(softc->regs,DRQ_Enable, 0);

	/*  Marker trace code */
#ifdef  MARKER_TRACE_INPUT
        mark(CCUBE_R2,1,c3_status,end_addr);
#endif

	/*
	 * The FIFO is now empty.  Pump the Codec one more time
	 * to get the last word.  Gather stats and exit.
	 */
	if ((c3_status & _Int_FIFO_Empty) == 0) {
		ccube_timer_off(ci);
		ccube_error(ci, "Bad Interrupt");
common_error:
		C3_WREG(softc->regs,Flags, 0);
		C3_WREG(softc->regs,Flags, ~0);
		C3_WREG(softc->regs,Flags, 0);
		C3_WREG(softc->regs,DRQ_Enable, 0);
		C3_WREG(softc->regs,NMRQ_Enable, 0);
		XV_WREG(softc->regs,CCube.l, XVC3_RESET_TO_PIO);

		ci->rcount = -1;
		ci->imask &= ~ _Int_CCube;
		ci->bmask &= ~ _Int_Dma;
		ci->state = CCUBE_STATE_ERROR;
		return;
	}

	ccube_timer_off(ci);

	/* We're done, get the last word (4 bytes) */

	XV_WREG(softc->regs,CCube.l, XVC3_CONT_DMA_TO_HOST_1);
	XV_WREG(softc->regs,CCube.l, XVC3_CONT_DMA_TO_HOST_2);
	XV_WREG(softc->regs,CCube.l, XVC3_PURGE_C3_CONT_DMA);
	DELAX(10);
	XV_WREG(softc->regs,CCube.l, XVC3_PAUSE_DMA_TO_HOST);
	DELAX(10);
	XV_WREG(softc->regs,CCube.l, XVC3_PURGE_BIC_GO_PIO);
	DELAX(10);
	XV_WREG(softc->regs,CCube.l, XVC3_RESET_TO_PIO);

	C3_WREG(softc->regs,DRQ_Enable, 0);

	end_addr = XV_RREG(softc->regs, DMARB.l);
	end_addr += 4;

	ci->end_addr = (caddr_t)end_addr;
	ci->rcount = ci->end_addr - ci->start_addr;
#ifdef  MARKER_TRACE_INPUT
        mark(CCUBE_R2,2,c3_status,ci->rcount);
#endif
	ci->imask &= ~ _Int_CCube;
	ci->bmask &= ~ _Int_Dma;
	ci->state = CCUBE_STATE_IDLE;
#ifdef  TRACE_STATE_TIME
	trace_state(softc,6,0);
#endif

	return;
}

static int
ccube_w0(tvtwo_softc *softc)
{
	unsigned long addr, count;
	CCUBE_INFO *ci= softc->ci;


#ifdef  MARKER_TRACE_OUTPUT
        mark(CCUBE_W0,1,ci->start_addr,ci->count);
#endif
	/*
	 * Wait for accelerator
	while(XV_RREG(softc->regs, DBusy.l));
	 */

	/* Set initial state */
        XV_WREG(softc->regs,CCube.l, XVC3_RESET_TO_PIO);
        XV_WREG(softc->regs,C3Stall.b.cond, STALL_ALWAYS);

	/* Load the DMA buffer address */
	addr = (unsigned long)(ci->start_addr);
	count = ci->count;

	XV_WREG(softc->regs,DMASA1.l, addr >> 2);	
	XV_WREG(softc->regs,DMASA2.l, addr >> 2);	
	XV_WREG(softc->regs,DMAEA1.l, (addr + count - 4) >> 2);
	XV_WREG(softc->regs,DMAEA2.l, (addr + count - 4) >> 2);

	/*
	 * DRQ until FIFO is 3/4 full, then interrupt.  DRQs are
	 * disabled and an interrupt is generated if a marker code is
	 * found. 
	 */

	C3_WREG(softc->regs,NMRQ_Enable, _Int_FIFO_3QFull /* | _Int_MarkerCode */);
	C3_WREG(softc->regs,DRQ_Enable, _Int_Codec_RegBusy | _Int_FIFO_Not3QFull);


        XV_WREG(softc->regs,CCube.l, XVC3_START_DMA_FROM_HOST_1);
        XV_WREG(softc->regs,CCube.l, XVC3_START_DMA_FROM_HOST_2);

	ccube_timer_start(softc);

	ci->imask |= _Int_CCube;
	ci->bmask |= _Int_Dma;
	ci->state = CCUBE_STATE_W1;
	return;
}

/*
static 
*/
void ccube_w1_buf(tvtwo_softc *softc)
{
	unsigned long end_addr, c3_status;
	CCUBE_INFO *ci= softc->ci;

	end_addr = XV_RREG(softc->regs, DMARB.l);
	end_addr += 4;
	XV_WREG(softc->regs,CCube.l, XVC3_PAUSE_DMA_FROM_HOST);
	DELAX(10);
	XV_WREG(softc->regs,CCube.l, XVC3_WRITE_GO_PIO);
	DELAX(10);
	c3_status = C3_RREG(softc->regs, Flags);
	C3_WREG(softc->regs,NMRQ_Enable, 0);
	C3_WREG(softc->regs,DRQ_Enable, 0);

	/*
	 * all the data has been read from the bus...now start the
	 * decompressor and wait for the video pipe to go inactive.
	 */
	C3_WREG(softc->regs,HV_Enable, ~0);
	C3_WREG(softc->regs,HV_Enable, 0);
	C3_WREG(softc->regs,HV_Enable, ~0);
	C3_WREG(softc->regs,Start, ~0);

	XV_WREG(softc->regs,C3Stall.b.cond, STALL_NEVER);

	if (c3_wait_for_vnac(softc) != DDI_SUCCESS) {
			return ;
	}

	C3_WREG(softc->regs,NMRQ_Enable, C3_VInactive);
	XV_WREG(softc->regs,C3Stall.b.cond, STALL_FE);

	XV_WREG(softc->regs,CCube.l, XVC3_CONT_DMA_FROM_HOST_1);
	XV_WREG(softc->regs,CCube.l, XVC3_CONT_DMA_FROM_HOST_2);

	ci->imask |= _Int_CCube;
	ci->bmask |= _Int_Dma;
	ci->state = CCUBE_STATE_W3;
	return;
}

/*
static
*/
void ccube_w1_cc(tvtwo_softc *softc)
{
	unsigned long end_addr, c3_status;
	CCUBE_INFO *ci= softc->ci;


        XV_WREG(softc->regs,C3Stall.b.cond, STALL_ALWAYS);
	end_addr = XV_RREG(softc->regs, DMARB.l);
	end_addr += 4;
	XV_WREG(softc->regs,CCube.l, XVC3_PAUSE_DMA_FROM_HOST);
	DELAX(10);
	XV_WREG(softc->regs,CCube.l, XVC3_WRITE_GO_PIO);
	DELAX(10);
	c3_status = C3_RREG(softc->regs, Flags);
	C3_WREG(softc->regs,NMRQ_Enable, 0);
	C3_WREG(softc->regs,DRQ_Enable, 0);
#ifdef  MARKER_TRACE_OUTPUT
        mark(CCUBE_W1,1,c3_status,end_addr);
#endif

	if ((c3_status & _Int_FIFO_3QFull) == 0) {
		ccube_timer_off(ci);
		ccube_error(ci, "Bad Interrupt");

		C3_WREG(softc->regs,Flags, 0);
		C3_WREG(softc->regs,Flags, ~0);
		C3_WREG(softc->regs,Flags, 0);
		C3_WREG(softc->regs,DRQ_Enable, 0);
		C3_WREG(softc->regs,NMRQ_Enable, 0);
		XV_WREG(softc->regs,CCube.l, XVC3_RESET_TO_PIO);

#ifdef  MARKER_TRACE_OUTPUT
        mark(CCUBE_W1,2,c3_status,0);
#endif
		ci->rcount = -1;
		ci->imask &= ~ _Int_CCube;
		ci->bmask &= ~ _Int_Dma;
		ci->state = CCUBE_STATE_ERROR;
		return;
	}
#ifdef  MARKER_TRACE_OUTPUT
        mark(CCUBE_W1,3,c3_status,0);
#endif

	ccube_timer_start(softc);

	/*
	 * The FIFO is now 3/4 full.  Reset the DRQ and IRQ
	 * conditions: DRQ while FIFO is not 3/4 full and 
	 * interrupt when end of buffer has been reached.
	 */
	C3_WREG(softc->regs,NMRQ_Enable, 0);
	C3_WREG(softc->regs,DRQ_Enable, _Int_Codec_RegBusy | _Int_FIFO_Not3QFull);
	/*
	 * Start the decompressor.
	 */
	C3_WREG(softc->regs,HV_Enable, ~0);
	C3_WREG(softc->regs,HV_Enable, 0);
	C3_WREG(softc->regs,HV_Enable, ~0);
	C3_WREG(softc->regs,Start, ~0);
	XV_WREG(softc->regs,C3Stall.b.cond, STALL_NH);

	do {
		c3_status = C3_RREG(softc->regs, Flags);
	} while (c3_status & C3_VInactive);


	XV_WREG(softc->regs,CCube.l, XVC3_CONT_DMA_FROM_HOST_1);
	XV_WREG(softc->regs,CCube.l, XVC3_CONT_DMA_FROM_HOST_2);

#ifdef  MARKER_TRACE_OUTPUT
        mark(CCUBE_W1,4,c3_status,end_addr);
#endif
	ci->imask |= _Int_CCube;
	ci->bmask |= _Int_Dma;
	ci->state = CCUBE_STATE_W2;
	return;
}

/*
static 
*/
void ccube_w2_buf(tvtwo_softc *softc)
{
	unsigned long end_addr, c3_status;
	unsigned long t_start, t_now;
	CCUBE_INFO *ci= softc->ci;

	end_addr = XV_RREG(softc->regs, DMARB.l);
	end_addr += 4;

	XV_WREG(softc->regs,CCube.l, XVC3_PAUSE_DMA_FROM_HOST);
	DELAX(10);
	XV_WREG(softc->regs,CCube.l, XVC3_WRITE_GO_PIO);
	DELAX(10);
	c3_status = C3_RREG(softc->regs, Flags);
	C3_WREG(softc->regs,NMRQ_Enable, 0);
	C3_WREG(softc->regs,DRQ_Enable, 0);

	ccube_timer_off(ci);

	/*
	 * all the data has been read from the bus... now wait for
	 * the video pipe to go inactive.
	 */

	XV_WREG(softc->regs,C3Stall.b.cond, STALL_NEVER);
	t_start= XV_RREG(softc->regs, TmrCnt.l);
	t_start += 60;
	do {
		c3_status = C3_RREG(softc->regs, Flags);
		t_now= XV_RREG(softc->regs, TmrCnt.l);
		if ((t_start < 60) && (t_now > 10000)) {
			t_now = 0;
		}
	} while ((t_now < t_start) && (!(c3_status & C3_VInactive)));

	C3_WREG(softc->regs,HV_Enable, ~0);
	C3_WREG(softc->regs,HV_Enable, 0);
	C3_WREG(softc->regs,HV_Enable, ~0);
	C3_WREG(softc->regs,Start, 0);
	XV_WREG(softc->regs,CCube.l, XVC3_RESET_TO_PIO);

	ci->end_addr = (caddr_t)end_addr;
	ci->rcount = ci->end_addr - ci->start_addr;
	ci->imask &= ~ _Int_CCube;
	ci->bmask &= ~ _Int_Dma;
	ci->state = CCUBE_STATE_IDLE;

	return;
}
/*

static
*/
void ccube_w2_cc(tvtwo_softc *softc)
{
	unsigned long end_addr, c3_status;
	CCUBE_INFO *ci= softc->ci;
	/*
	 * Wait for accelerator
	while(XV_RREG(softc->regs, DBusy.l));
	 */

        XV_WREG(softc->regs,C3Stall.b.cond, STALL_ALWAYS);

	end_addr = XV_RREG(softc->regs, DMARB.l);
	end_addr += 4;

	XV_WREG(softc->regs,CCube.l, XVC3_PAUSE_DMA_FROM_HOST);
	DELAX(10);
	XV_WREG(softc->regs,CCube.l, XVC3_WRITE_GO_PIO);
	DELAX(10);
	c3_status = C3_RREG(softc->regs, Flags);
	C3_WREG(softc->regs,NMRQ_Enable, 0);
	C3_WREG(softc->regs,DRQ_Enable, 0);
#ifdef  MARKER_TRACE_OUTPUT
        mark(CCUBE_W2,1,c3_status,end_addr);
#endif

	ccube_timer_off(ci);
	ccube_error(ci, "Bad Interrupt");

	C3_WREG(softc->regs,Flags, 0);
	C3_WREG(softc->regs,Flags, ~0);
	C3_WREG(softc->regs,Flags, 0);
	C3_WREG(softc->regs,DRQ_Enable, 0);
	C3_WREG(softc->regs,NMRQ_Enable, 0);
	XV_WREG(softc->regs,CCube.l, XVC3_RESET_TO_PIO);

	ci->rcount = -1;
	ci->imask &= ~ _Int_CCube;
	ci->bmask &= ~ _Int_Dma;
	ci->state = CCUBE_STATE_ERROR;

	return;
}

static void
ccube_w3_cc(tvtwo_softc *softc)
{
	unsigned long end_addr, c3_status;
	CCUBE_INFO *ci= softc->ci;

        XV_WREG(softc->regs,C3Stall.b.cond, STALL_ALWAYS);

	end_addr = XV_RREG(softc->regs, DMARB.l);
	end_addr += 4;

	XV_WREG(softc->regs,CCube.l, XVC3_PAUSE_DMA_FROM_HOST);
	DELAX(50);
	XV_WREG(softc->regs,CCube.l, XVC3_WRITE_GO_PIO);
	DELAX(10);
	c3_status = C3_RREG(softc->regs, Flags);
	C3_WREG(softc->regs,NMRQ_Enable, 0);
	C3_WREG(softc->regs,DRQ_Enable, 0);
#ifdef  MARKER_TRACE_OUTPUT
        mark(CCUBE_W3,1,c3_status,end_addr);
#endif
	/*
	 * The video pipe has gone inactive.  We need to tell the
	 * decompressor to stop so that it doesn't expect more
	 * frames.
	 */
	C3_WREG(softc->regs,HV_Enable, ~0);
	C3_WREG(softc->regs,HV_Enable, 0);
	C3_WREG(softc->regs,HV_Enable, ~0);
	C3_WREG(softc->regs,Start, 0);
	C3_WREG(softc->regs,DRQ_Enable, 0);
	C3_WREG(softc->regs,NMRQ_Enable, 0);
	XV_WREG(softc->regs,CCube.l, XVC3_RESET_TO_PIO);

	ccube_timer_off(ci);

	ci->end_addr = (caddr_t)end_addr;
	ci->rcount = ci->end_addr - ci->start_addr;
	ci->imask &= ~ _Int_CCube;
	ci->bmask &= ~ _Int_Dma;
	ci->state = CCUBE_STATE_IDLE;
#ifdef  MARKER_TRACE_OUTPUT
        mark(CCUBE_W3,2,c3_status,ci->rcount);
#endif

	return;
}

ccube_timeout(tvtwo_softc *softc)
{
	CCUBE_INFO *ci= softc->ci;

	cmn_err(CE_WARN,"!ccube_timeout:state=%d\n",ci->state);

	XV_WREG(softc->regs,CCube.l, XVC3_PURGE_BIC_GO_PIO);
	DELAX(10);
	XV_WREG(softc->regs,CCube.l, XVC3_RESET_TO_PIO);
	C3_WREG(softc->regs,DRQ_Enable, 0);
	C3_WREG(softc->regs,NMRQ_Enable, 0);

	ci->rcount = -1;
	ci->imask &= ~ _Int_CCube;
	ci->bmask &= ~ _Int_Dma;
	ci->state = CCUBE_STATE_TIMEOUT;

	if(softc->rtv_dma_state==RECORD)
	{
	    ccube_finish_compress(softc);
	}
	else
	{
	    ccube_finish_decomp(softc);
	}
	return;
}

void
ccube_statemachine(tvtwo_softc *softc, int action, int info)
{
	CCUBE_INFO *ci= softc->ci;

	switch (ci->state) {
	case CCUBE_STATE_ERROR:
		switch (action) {
		case CCUBE_ACTION_TIMER:
			printf("error/TIMER\n");
			ccube_timer_off(ci);
			break;
		default:
			goto badstate;
		}
		break;
	case CCUBE_STATE_IDLE:
		switch (action) {
		case CCUBE_ACTION_STARTWRITE:
			ccube_w0(softc);
			break;
		case CCUBE_ACTION_STARTREAD:
			ccube_r0(softc);
			break;
		case CCUBE_ACTION_TIMER:
			printf("IDLE/TIMER\n");
			ci->state = CCUBE_STATE_ERROR;
			ccube_timer_off(ci);
			break;
		default:
			goto badstate;
		}
		break;
	case CCUBE_STATE_TIMEOUT:
		switch (action) {
		case CCUBE_ACTION_TIMER:
			ci->state = CCUBE_STATE_ERROR;
			printf("TIMEOUT/TIMER\n");
			ccube_timer_off(ci);
			break;
		default:
			goto badstate;
		}
	case CCUBE_STATE_R1:
		switch (action) {
		case CCUBE_ACTION_INTR_CCUBE:
			ccube_r1_cc(softc);
			break;
		case CCUBE_ACTION_TIMER:
			ci->state = CCUBE_STATE_ERROR;
			printf("R1/TIMER\n");
			ccube_timeout(softc);
			break;
		default:
			goto badstate;
		}
		break;
	case CCUBE_STATE_R2:
		switch (action) {
		case CCUBE_ACTION_INTR_CCUBE:
			ccube_r2_cc(softc);
			break;
		case CCUBE_ACTION_TIMER:
			ci->state = CCUBE_STATE_ERROR;
			printf("R2/TIMER\n");
			ccube_timeout(softc);
			break;
		default:
			goto badstate;
		}
		break;
	case CCUBE_STATE_W1:
		switch (action) {
		case CCUBE_ACTION_INTR_BUF:
			ccube_w1_buf(softc);
			break;
		case CCUBE_ACTION_INTR_CCUBE:
			ccube_w1_cc(softc);
			break;
		case CCUBE_ACTION_TIMER:
			ci->state = CCUBE_STATE_ERROR;
			printf("W1/TIMER\n");
			ccube_timeout(softc);
			break;
		default:
			goto badstate;
		}
		break;
	case CCUBE_STATE_W2:
		switch (action) {
		case CCUBE_ACTION_INTR_BUF:
			ccube_w2_buf(softc);
			break;
		case CCUBE_ACTION_INTR_CCUBE:
			ccube_w2_cc(softc);
			break;
		case CCUBE_ACTION_TIMER:
			ci->state = CCUBE_STATE_ERROR;
			printf("W2/TIMER\n");
			ccube_timeout(softc);
			break;
		default:
			goto badstate;
		}
		break;
	case CCUBE_STATE_W3:
		switch (action) {
		case CCUBE_ACTION_INTR_CCUBE:
			ccube_w3_cc(softc);
			break;
		case CCUBE_ACTION_TIMER:
			ci->state = CCUBE_STATE_ERROR;
			printf("W3/TIMER\n");
			ccube_timeout(softc);
			break;
		default:
			goto badstate;
		}
		break;
	default:
		goto badstate;
	}
	return;

badstate:

	printf("ccube: state error: state=%d mode=%d action=%d info=%d\n",
		ci->state,
		ci->mode,
		action,
		info);

	return;
}

static void
ccube_error(ci, reason)
CCUBE_INFO *ci;
char *reason;
{
	printf("ccube: error: %s: state=%d mode=%d\n",
		reason,
		ci->state,
		ci->mode);

	return;
}


static void
ccube_timer_intr(tvtwo_softc *softc)
{
CCUBE_INFO *ci=softc->ci;

	ci->timer_active = 0;

	if (ci->timer_required == 0) {
		return;
	}

	ci->timer_count++;

	if (ci->timer_count >= 2) {
		ccube_timer_off(ci);
		ccube_timeout(softc);

		if (ci->timer_required == 0) {
			return;
		}
	}

	if (ci->timer_active == 0) {
		ci->timer_active = 
		    timeout(ccube_timer_intr, (caddr_t)softc, 100);
	}

	return;
}

static void
ccube_timer_start(tvtwo_softc *softc)
{
CCUBE_INFO *ci=softc->ci;

	ci->timer_required = 1;
	ci->timer_count = 0;

	if (ci->timer_active == 0) {
	    ci->timer_active = 
		timeout(ccube_timer_intr, (caddr_t)softc, 100);
	}

	return;
}
static void
ccube_timer_off(ci)
CCUBE_INFO *ci;
{

	ci->timer_required = 0;
	ci->timer_count = 0;
	untimeout(ci->timer_active);
	ci->timer_active=0;

	return;
}

