/*
 * Copyright (c) 1993 Parallax Graphics Inc.
 */

#pragma ident "@(#)ccube_compress.c 1.6 Copyright (c) 1993 Parallax Graphics Inc"

#include <sys/types.h>
#include <sys/param.h>
#include <sys/cred.h>
#include <sys/errno.h>
#include <sys/cmn_err.h>
#include <sys/open.h>
#include <sys/file.h>
#include <sys/kmem.h>
#include <sys/t_lock.h>
#include <sys/stat.h>
#include <sys/conf.h>
#include <sys/ddi.h>
#include <sys/sunddi.h>

#include "tvtwo.h"
#define COMPRESSION_TABLES
#include "qfact.h"
#include "huff.h"

#define CCUBE_STATE_TIMEOUT 1
/*
#define TRACE_STATE_TIME
#define TRACE_COMP_TIMER
 */

get_next_field(tvtwo_softc *softc)
{
ddi_dma_cookie_t dma_cookie;    /* for dma information */
FIELD_INFO *field;
register int prev_rate;
static short  qfactor,qindex;
static int polarity,sid;
int    start_pixel;
int    save_stat,wid;
unsigned short *qtab;
SHM_FIELD_QUE *shm_que = (SHM_FIELD_QUE *)softc->shm_que;
static  int trace=0;
    
    mutex_enter(&softc->rtv_dma_lock);
    if(softc->rtv_dma_busy)
        cv_wait(&softc->rtv_dma_done, &softc->rtv_dma_lock);
    softc->rtv_dma_busy=1;
    mutex_exit(&softc->rtv_dma_lock);
    softc->rtv_dma_state=RECORD;
    softc->ccube_comp_state=DST_BUSY;
    /*
     ***** Put dma address and size into dma cookie.
     ***** Adjust dma handle to next field_offset.
    */
     if (ddi_dma_htoc(softc->ccube_read_dma,
         softc->field_offset[0],&dma_cookie) != DDI_SUCCESS) 
     {
        cmn_err(CE_PANIC,"ccube %d: ddi_dma_htoc failed",
            ddi_get_instance(softc->devi));
     }
    /*
     ***** Load dma address and size from dma cookie.
     */
     softc->dma_input_addr = (caddr_t)dma_cookie.dmac_address;
    /*
     ***** Flush memory with respect to device.
     */
     (void)ddi_dma_sync(softc->ccube_read_dma,softc->field_offset[0],
               softc->record_size,DDI_DMA_SYNC_FORDEV);
    /*
     * Rate index is used here to look up appropriate queue table based
     * on previous rates.
     * Determine rate index and qfactor based on rate of previous fields ,
     * minimum q, and maximum rate. Also get qfactor and store in 
     * shared memory field info. Previous rates are stored in 
     * softc->rate_info by the "process_event" routine.
     */
         polarity= softc->field_polarity;
    /*
     * Computing next qfactor and loading tables was moved to 
     * follow field compression.
     */
     wid=softc->record_window;
     SetC3GetImageTiming(softc,softc->record_wintbl[wid].width,
                softc->record_wintbl[wid].height,softc->interlace);
     start_pixel=softc->record_start_pixel[wid];
     if(softc->field_polarity==0)
         XV_WREG(softc->regs,WStartAdd.l,start_pixel);
     else
         XV_WREG(softc->regs,WStartAdd.l,start_pixel+1152);
#ifdef TRACE_STATE_TIME
         trace_state(softc,0,1);
#endif
     ccube_read(softc);
}
ccube_select_comp_tables(tvtwo_softc *softc,int qindex)
{
    softc->qtab = (unsigned short *)&CL550Qtable_C[qindex][0];
    softc->field_qfactor= ccube_index_to_q(qindex);
    softc->htab= defaultCL550CompHtable;
    C3InitCompress(softc);
    LoadCL550Qtable(softc,softc->qtab);
    LoadCL550Htable(softc,softc->htab);
    c3_load_dct(softc); 
    enableC3(softc);
}
/*
 * Load qtable and initalize Ccube for compression operation.
 */
ccube_load_compression_tables(tvtwo_softc *softc)
{
register int qindex,polarity;

/*
    if(softc->field_polarity==0)
 */
    {
           polarity=softc->field_polarity;
           qindex=ccube_set_new_q(softc->rate_info.prev_qindex[polarity],
           softc->rate_info.prev_rate[polarity],softc->max_rate,softc->min_queue);
           softc->field_qfactor= ccube_index_to_q(qindex);
           softc->rate_info.prev_qindex[polarity]= qindex;
           softc->rate_info.qfactor[polarity]=softc->field_qfactor;
           softc->qtab = (unsigned short *)&CL550Qtable_C[qindex][0];
    }
    C3InitCompress(softc);
    LoadCL550Qtable(softc,softc->qtab);
    LoadCL550Htable(softc,softc->htab);
    c3_load_dct(softc); 
    enableC3(softc);
}
ccube_finish_compress(tvtwo_softc *softc)
{
QUEUE_INFO *playback_qinfo;
SHM_FIELD_QUE *shm_que = (SHM_FIELD_QUE *)softc->shm_que;
CCUBE_INFO *ci = softc->ci;
register int i,wait_time,n;
static int trace=0;
static int frmcnt=0;
u_int *bptr;
/* 
 * DMA completion - Sync dma buffer for kernel access.
 */
        (void)ddi_dma_sync(softc->ccube_read_dma,softc->field_offset[0],
         softc->record_size,DDI_DMA_SYNC_FORKERNEL);
#ifdef TRACE_STATE_TIME
           trace_state(softc,7,0);
#endif
        bptr=(u_int *)(softc->input_shared_data + softc->field_offset[0]
                +ci->rcount);
        *bptr= 0xffd90000;
        ci->rcount+=4;
        softc->rate_info.prev_rate[softc->field_polarity]= ci->rcount;
     /*
      * Wake up mutex
      */
        softc->rtv_dma_busy=0;
        cv_signal(&softc->rtv_dma_done);
        if(ci->state == CCUBE_STATE_TIMEOUT)
        {
	     cmn_err(CE_WARN,"! compress: TIMEOUT!\n");
             softc->field_polarity=0;
             softc->ccube_comp_state= DST_START;
             softc->imask |= _Int_Wid | _Int_Vid;
             XV_WREG(softc->regs,IntMask.l, softc->imask);
             XV_WREG(softc->regs,BIntMask.l, softc->bmask);
             C3InitCompress(softc);
             LoadCL550Qtable(softc,softc->qtab);
             LoadCL550Htable(softc,softc->htab);
             c3_load_dct(softc); 
             enableC3(softc);
             return;
        }
        else
        {
            if(softc->ci->rcount >= MAX_FIELD_SIZE)
            {
                cmn_err(CE_WARN,"!Overflow field size=%d qfactor=%d polarity=%d\n",
                softc->ci->rcount,softc->field_qfactor,softc->field_polarity);
                if(softc->field_polarity==0)softc->overflow=1;
                softc->field_polarity=0;
                softc->ccube_comp_state=DST_FIELD_DONE;
                ccube_load_compression_tables(softc);
                return;
           }
           process_event(softc,COMPRESSION_IODONE);
#ifdef TRACE_STATE_TIME
    trace_state(softc,8,0);
#endif
           ccube_load_compression_tables(softc);
           if(softc->ccube_comp_state==DST_IOLATE)
           {
                if(softc->record_action == RECORD)
                    get_next_field(softc);
           }
	   else if(softc->ccube_comp_state==DST_IOLATE_LATE)
	   {
		cmn_err(CE_WARN,"! compress: IOLATE_LATE\n");
		if(softc->field_polarity==0)
		    softc->overflow=2;
		else 
		    softc->overflow=1;
                softc->ccube_comp_state= DST_FIELD_DONE;
		return;
	   }
           else
                softc->ccube_comp_state= DST_FIELD_DONE;
      }
}
process_event(tvtwo_softc *softc,int event)
{
register int polarity,current_qindex;
FIELD_INFO *record_field;
QUEUE_INFO *playback_qinfo;
static int trace=0;
static int seconds=0;
CCUBE_INFO *ci = softc->ci;
SHM_FIELD_QUE *shm_que = (SHM_FIELD_QUE *)softc->shm_que;


    switch(event)
    {
    case START_COMPRESSION:
         if(softc->ccube_comp_state == DST_IDLE)
         {
             softc->ccube_comp_state= DST_START;
             softc->Vidcount=0;
             softc->field_polarity=0;
             softc->skip_fields=5;
         /*
          *  Enable v_chan interrupts.
          */
             softc->imask |= _Int_Vid ;
             XV_WREG(softc->regs,IntMask.l,softc->imask);
             XV_WREG(softc->regs,BIntMask.l,softc->bmask);
         }
         break;
    case START_DECOMP:
         if(softc->ccube_comp_state == DST_IDLE)
         {
             softc->ccube_comp_state= DST_READY;
             softc->Vidcount=0;
             softc->field_polarity=0;
             ccube_load_compression_tables(softc);
         /*
          *  Enable v_chan interrupts.
          */
             softc->imask |= _Int_Wid | _Int_Vid;
             XV_WREG(softc->regs,IntMask.l,softc->imask);
             XV_WREG(softc->regs,BIntMask.l,softc->bmask);
         }
         break;
    case COMPRESSION_IODONE:
         switch(softc->ccube_comp_state)
         {
         case DST_IOLATE_LATE:
	 /*
	      cmn_err(CE_WARN,"! process_event: DST_IOLATE_LATE!\n");
	  */
         case DST_IOLATE:
         case DST_BUSY:
         case DST_FIELD_DONE:
              softc->field_size=softc->ci->rcount;
              if(softc->record_action==RECORD)
              {
                  if(softc->skip_fields)
                  {
                        softc->skip_fields--;
                        softc->field_polarity = (softc->field_polarity) ? 0:1;
                        return;
                  }
                  record_field= (FIELD_INFO *)shm_que->record_input;
                  record_field->size = softc->field_size;
                  record_field->offset= softc->field_offset[0];
                  record_field->qfactor=softc->field_qfactor;
                  record_field->polarity= softc->field_polarity;
                  record_field->frame_no=softc->frame_count;
                  record_field->clock= shm_que->clock_time;
                  record_field->inuse=1;
                  shm_que->record_input++;
                  shm_que->record_qlevel++;
                  if(shm_que->record_input>=shm_que->record_tail)
                      shm_que->record_input=shm_que->record_head;
#ifdef NETWORK_SUPPORT
                if(softc->xmit_action==SEND && shm_que->wq)
                    que_field(softc,shm_que->wq);
#endif
                  softc->field_offset[0] += softc->field_size;
                  if(softc->field_offset[0]+MAX_FIELD_SIZE >= 
                      softc->actual_input_size)softc->field_offset[0]=0;
                  if(softc->field_polarity)
                  {    
                      softc->frame_count++ ;
                      shm_que->clock_time++ & 0x3fff;
                  }
                  softc->field_polarity = (softc->field_polarity) ? 0 : 1;

              }
              break;
         }
         break;
    }
}

#ifdef NETWORK_SUPPORT
static int que_field(tvtwo_softc *softc,queue_t *wq)
{
mblk_t *mp;
struct iocblk *iocp;
int frame_no;
static int trace=0;
SHM_FIELD_QUE *shm_que = (SHM_FIELD_QUE *)softc->shm_que;

        mp= allocb(4,BPRI_MED);
        if(mp != NULL)
        {
                mp->b_datap->db_type=M_IOCTL;
                iocp = (struct iocblk *)mp->b_rptr;
            iocp->ioc_cmd=SEND_NEXT_SHM_FIELD;
            iocp->ioc_count=1;
            putq(wq,mp);
        }
}
#endif

