/*
 * ccube q code.
 */

#include "tvtwo.h"
#include "c3.h"
/*
 * The following represent 5% increments in data rates, and their associated
 * average data rates (bytes/field).  The rates must be scaled based on the
 * source material.  Even scaled, the results are estimates, but will be 
 * accurate for reletively small q steps.
 */

#define RGB422
#define CL550HTSIZE (0x1c80/4)

#define C3HBlank  64
#define C3HSync  16
#define C3VBlank 22	
#define C3VSync  2	

int lastlastq, lastq;
int lastlastrate, lastrate;

static struct ccube_q_values {
	int qfactor;			/* q factor */
	int byte_size;			/* bytes/field */
} ccube_q_values[] = {
	{ 25, 25144 },	{ 28, 23540 },	{ 31, 22073 },	{ 34, 20878 },
	{ 37, 19655 },	{ 40, 18418 },	{ 44, 17329 },	{ 47, 16404 },
	{ 50, 15426 },	{ 56, 14520 },	{ 61, 13640 },	{ 66, 12882 },
	{ 71, 12190 }, 	{ 76, 11572 },	{ 83, 10955 },	{ 90, 10367 },
	{ 98,  9766 },	{ 108, 9224 },	{ 118, 8734 },	{ 129, 8282 },
	{ 143, 7864 },	{ 158, 7455 },	{ 175, 7066 },	{ 195, 6739 },
};

static int ccube_q_values_n = sizeof(ccube_q_values) / sizeof(ccube_q_values[0]);

/*  sets a new q factor based on the last field digitized.
 *  oldq is an index into the qsteps table,
 *  oldrate is the number of bytes in the last field
 *  maxrate is the number of bytes that should not be exceeded on next field
 *  minq is the lowest allowed q (it is an actual q, not an index)
 *
 * how it works:
 * the qrate table shows the average data rates per q factor.  This varies
 * widely by input material.  Still one can predict that varying the q will 
 * have approximately proportional effects on data rates than average.
 * (approximately only, It can be far off if: scene changes, b) large q
 * changes, or c) q<40 or q>140)
 * The normallized rate is the maximum rate normallized to the qrate table
 */

int ccube_set_new_q(oldq, oldrate, maxrate, minq)
int oldq, oldrate, maxrate, minq;
{
	int normlrate;
	int ii;
	struct ccube_q_values *p;

	normlrate = (maxrate * ccube_q_values[oldq].byte_size) / oldrate;
	
	for (ii=0,p=ccube_q_values;ii<ccube_q_values_n;ii++,p++) {
		if ((p->qfactor >= minq) && (normlrate>=p->byte_size))  {
			return(ii);
		}
	}
	return(ii-1);
}

int
ccube_q_to_index(q)
{
	int ii;
	struct ccube_q_values *p;

	for (ii=0,p=ccube_q_values;ii<ccube_q_values_n;ii++,p++) {
		if (q <= p->qfactor)
			return (ii);
	}
	return (ii - 1);
}

int
ccube_index_to_q(idx)
{
	return (ccube_q_values[idx].qfactor);
}

ccube_make_q_tables()
{
}

ccube_q_to_rate( q )
{
	return ( ccube_q_values[ccube_q_to_index(q)].byte_size );
}



/*****************************************************************************
 *
 * Load CL550 Quantization Table A.  (B is unused).  Expects an array of
 * 128 shorts to be loaded into the CL550 QuantA registers.
 *
 *****************************************************************************/

LoadCL550Qtable(tvtwo_softc *softc,unsigned short *table)
{
	register int i;
	C3_WREGS *c3_wreg = (C3_WREGS *)softc->regs->c3;

	ORIG_C3_WREG(QuantSelect,0);

	for (i=0; i < 128 ; i++)
		ORIG_C3_WREG(QuantA[i],table[i]);

	ORIG_C3_WREG(QuantSelect,1);

}
/*****************************************************************************
 *
 * Prepare the CL550 for compression.  Called by C3GetImage.
 *
 *****************************************************************************/

C3InitCompress(tvtwo_softc *softc)
{
	char s[20];
	int i1,i2;
	C3_WREGS *c3_wreg = (C3_WREGS *)softc->regs->c3;


	ORIG_C3_WREG(S_reset, ~0);

	/* Load configuration registers */

#ifdef RGB422
	ORIG_C3_WREG(Config, 0x38);
#else
	ORIG_C3_WREG(Config, 0x28); 
#endif
	ORIG_C3_WREG(DRQ_Enable,0);
	ORIG_C3_WREG(NMRQ_Enable,0); 

#ifdef RGB422
	ORIG_C3_WREG(Init_1, 0x003c);
	ORIG_C3_WREG(Init_2, 0x003d);
	ORIG_C3_WREG(Init_3, 0x007b);
	ORIG_C3_WREG(Init_4, 0x00f1);
	ORIG_C3_WREG(Init_5, 0x000a);
	ORIG_C3_WREG(Init_6, 0x0000);
	ORIG_C3_WREG(Init_7, 0x0017);
	ORIG_C3_WREG(QuantSync, 0x0400);
#else
	ORIG_C3_WREG(Init_1, 0x0042);
	ORIG_C3_WREG(Init_2, 0x0001);
	ORIG_C3_WREG(Init_3, 0x0081);
	ORIG_C3_WREG(Init_4, 0x00f7);
	ORIG_C3_WREG(Init_5, 0x0000);
	ORIG_C3_WREG(Init_6, 0x0000);
	ORIG_C3_WREG(Init_7, 0x001d);
	ORIG_C3_WREG(QuantSync, 0x0406);
#endif
	ORIG_C3_WREG(QuantYC, 0x2099);
	ORIG_C3_WREG(QuantAB, 0x0000);
	ORIG_C3_WREG(QuantSelect, 1);
#ifdef RGB422
	ORIG_C3_WREG(VideoLatency, 0x0185);
#else
	ORIG_C3_WREG(VideoLatency, 0x017f);
#endif

	/* Load coder configuration registers */

	ORIG_C3_WREG(CodecSeq, 0x00cc);
	ORIG_C3_WREG(CodecCompH, 0x0088);
	ORIG_C3_WREG(CodecCompL, 0x0044);
	ORIG_C3_WREG(CoderAttr, 0x00004);
	ORIG_C3_WREG(CodingIntH, 0x0000);
#ifdef RGB422
	ORIG_C3_WREG(CodingIntL, 0x0002);
#else
	ORIG_C3_WREG(CodingIntL, 0x001);
#endif
	ORIG_C3_WREG(DecLength, 0x0008);
	ORIG_C3_WREG(CodeOrder, 0x0001);

	/* Load RGB-YUV conversion matrix */
	ORIG_C3_WREG(R2Y,0x00000133);
	ORIG_C3_WREG(G2Y,0x00000259);
	ORIG_C3_WREG(B2Y,0x00000074);
	ORIG_C3_WREG(R2U,0x00000F54);
	ORIG_C3_WREG(G2U,0x00000EAD);
	ORIG_C3_WREG(B2U,0x000001FF);
	ORIG_C3_WREG(R2V,0x000001FF);
	ORIG_C3_WREG(G2V,0x00000E53);
	ORIG_C3_WREG(B2V,0x00000FAE);

}
LoadCL550Htable(softc,table)
tvtwo_softc *softc;
unsigned short *table;
{
	C3_WREGS *c3_wreg = (C3_WREGS *)softc->regs->c3;
	register int i;
	register int tblsiz;

	tblsiz= CL550HTSIZE;
	ORIG_C3_WREG(Huff_Enable, -1);
	for (i=0;i<tblsiz;i++) 
		ORIG_C3_WREG(HuffYAC[i],table[i]);

	ORIG_C3_WREG(Huff_Enable, 0);
}
/*****************************************************************************
 *
 * Prepare the CL550 for decompression.  This is called by C3PutImage.
 *
 *****************************************************************************/
C3InitDecompress(tvtwo_softc *softc)
{
	C3_WREGS *c3_wreg = (C3_WREGS *)softc->regs->c3;
	ORIG_C3_WREG(S_reset, ~0);

	/* Load configuration registers */

#ifdef RGB422
	ORIG_C3_WREG(Config, 0x100 + 0x38);
#else
	ORIG_C3_WREG(Config, 0x100 + 0x28); 
#endif

	ORIG_C3_WREG(DRQ_Enable,0);
	ORIG_C3_WREG(NMRQ_Enable,0);

	ORIG_C3_WREG(Init_1, 0x003e);
	ORIG_C3_WREG(Init_2, 0x0037);
	ORIG_C3_WREG(Init_3, 0x01ff);
	ORIG_C3_WREG(Init_4, 0x0049);
	ORIG_C3_WREG(Init_5, 0x0005);
	ORIG_C3_WREG(Init_7, 0x0022);
	ORIG_C3_WREG(QuantSync, 0x043e);
	ORIG_C3_WREG(QuantYC, 0x2033);
	ORIG_C3_WREG(QuantAB, 0x0000);
	ORIG_C3_WREG(QuantSelect, 1);
	ORIG_C3_WREG(VideoLatency, 0x017f);

	/* Load coder configuration registers */

	ORIG_C3_WREG(CodecSeq, 0x00cc);
	ORIG_C3_WREG(CodecCompH, 0x0088);
	ORIG_C3_WREG(CodecCompL, 0x0044);
	ORIG_C3_WREG(CoderAttr, 0x00004);
	ORIG_C3_WREG(CodingIntH, 0x0000);
#ifdef RGB422
	ORIG_C3_WREG(CodingIntL, 0x0002);
#else
	ORIG_C3_WREG(CodingIntL, 0x001);
#endif
	ORIG_C3_WREG(DecLength, 0x0008);
	ORIG_C3_WREG(CodeOrder, 0x0001);

	/* Load RGB-YUV conversion matrix */

	ORIG_C3_WREG(R2Y,0x00000400);
	ORIG_C3_WREG(G2Y,0x00000FFE);
	ORIG_C3_WREG(B2Y,0x000005C9);
	ORIG_C3_WREG(R2U,0x00000400);
	ORIG_C3_WREG(G2U,0x00000EA2);
	ORIG_C3_WREG(B2U,0x00000D23);
	ORIG_C3_WREG(R2V,0x00000400);
	ORIG_C3_WREG(G2V,0x0000071B);
	ORIG_C3_WREG(B2V,0x00000FFD);

}


SetC3GetImageTiming(tvtwo_softc *softc, int width,int height,int interlaced) 
{
	C3_WREGS *c3_wreg = (C3_WREGS *)softc->regs->c3;

	ORIG_C3_WREG(HPeriod,  width + C3HBlank  -1);
	ORIG_C3_WREG(HSync,    C3HSync);
	ORIG_C3_WREG(HDelay,   C3HBlank/2);
	ORIG_C3_WREG(HActive,  2 * (width/8) -1);
	ORIG_C3_WREG(VSync,    C3VSync);
	ORIG_C3_WREG(VDelay,   C3VBlank/2);	
	ORIG_C3_WREG(VActive,  (height/8/((interlaced) ? 2 : 1)) );
	ORIG_C3_WREG(VPeriod,  height/((interlaced) ? 2 : 1) + C3VBlank  -1);
	ORIG_C3_WREG(HControl, 0);
	ORIG_C3_WREG(VControl, 0);
}

SetC3PutImageTiming(tvtwo_softc *softc,int width, int height)
{
	C3_WREGS *c3_wreg = (C3_WREGS *)softc->regs->c3;

	ORIG_C3_WREG(HPeriod,  width + C3HBlank  -1);
	ORIG_C3_WREG(HSync,    C3HSync);
	ORIG_C3_WREG(HDelay,   C3HBlank/2);
	ORIG_C3_WREG(HActive,  2 * (width/8) -1);
	ORIG_C3_WREG(VSync,    C3VSync);
	ORIG_C3_WREG(VDelay,   C3VBlank/2 - 9); 
	ORIG_C3_WREG(VPeriod,  height + C3VBlank  -1 );
	ORIG_C3_WREG(VActive,  (height/8));
	ORIG_C3_WREG(HControl, ~0);
	ORIG_C3_WREG(VControl, ~0);
}

