/*
 * Copyright (c) 1993 Parallax Graphics Inc.
 */

#pragma ident "@(#)tvtwo_fb.abw.c	1.5 Copyright (c) 1993 Parallax Graphics Inc"

/*
 * Color frame buffer with video io (tv2) driver
 */
#define NWIN 2

#include <sys/param.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/fbio.h>
#include <sys/ddi.h>
#include <sys/sunddi.h>
#include <sys/user.h>
#include <sys/sysmacros.h>
#include <sys/visual_io.h>

#include <pixrect/pixrect.h>
#include <pixrect/pr_planegroups.h>
#include "tvtwo.h"
#include "pvioctl.h"
/* #### */
#ifdef SUN4M
typedef struct _Trace_Entry {
	unsigned char  state;
	unsigned char  unused;
	unsigned short flags;
	int	       address;
	int	       time_stamp;
} Trace_Entry;

typedef struct _State_Trace {
	int i_entry;
	Trace_Entry entries[20];
} State_Trace;

extern State_Trace state_trace;
#endif
#define TVTWO_FBSIZE(softc, depth) \
	(ctob(btoc(mpr_linebytes((softc)->fbd.w, (depth)) * (softc)->fbd.h)))

extern char freqcode;

static char vis_id_name[128] = "PGItv2";

/* 
 * default structure for FBIOGTYPE ioctl
 */

static struct fbtype tv2typedefault = {
	FBTYPE_NOTSUN1,		/* type		*/
	0,			/* height	*/
	0,			/* width	*/
	32,			/* depth	*/
	256,			/* cms		*/
	0			/* size		*/
};

/* 
 * default structure for cgthree FBIOGATTR ioctl
 */

static struct fbgattr cg3_attr = {
	FBTYPE_SUN3COLOR,		/* real type	*/
	0,				/* owner	*/
	{	
		FBTYPE_SUN3COLOR,	/* type		*/
		0,			/* height	*/
		0,			/* width	*/
		8,			/* depth	*/
		256,			/* cms		*/
		0			/* size		*/
	},
	{	
		0,			/* flags	*/
		FBTYPE_SUN4COLOR,	/* emu type	*/
		{
			0		/* device specific */
		}
	},
	{	/* emu_types */
		FBTYPE_SUN3COLOR,
		FBTYPE_SUN4COLOR,
		-1,
		-1
	}
};

/*
 * default structure for tvtwo FBIOGATTR ioctl
 */

static struct fbgattr tv2attrdefault = {
	FBTYPE_NOTSUN1,			/* real type	*/
	0,				/* owner	*/
	{	
		FBTYPE_NOTSUN1,		/* type		*/
		0,			/* height	*/
		0,			/* width	*/
		32,			/* depth	*/
		256,			/* cms		*/
		0			/* size		*/
	},
	{	
		0,			/* flags	*/
		FBTYPE_NOTSUN1,		/* emu type	*/
		{
			0		/* device specific */
		}
	},
	{	/* emu_types */
		FBTYPE_SUN4COLOR,
		-1,
		-1,
		-1
	}
};

/* frame buffer description table */
static struct cg4_fbdesc {
	short	depth;			/* depth, bits */
	short	group;			/* plane group */
	int	allplanes;		/* initial plane mask */
} cg4_fbdesc[TV2_NFBS + 1] = {
	{ 1, PIXPG_OVERLAY,		0 },
	{ 1, PIXPG_OVERLAY_ENABLE,	0 },
	{ 8, PIXPG_8BIT_COLOR,		255 }
};

#define	TV2_FBINDEX_OVERLAY	0
#define TV2_FBINDEX_ENABLE	1
#define	TV2_FBINDEX_COLOR	2

/* initial active frame buffer */
#ifndef	TV2_INITFB
#define	TV2_INITFB	TV2_FBINDEX_COLOR
#endif

/*
 * replacement for pfind() which disappeared in 5.0
 */
extern struct proc *prfind();
extern kmutex_t pidlock;
static struct proc *
pfind(pid_t pid)
{
	struct proc *proc_p;

	mutex_enter(&pidlock);
	proc_p = prfind(pid);
	mutex_exit(&pidlock);

	return (proc_p);
}

#if NWIN > 0

/* SunWindows specific stuff */


static int
cg4_putattributes(Pixrect *pr, int *planesp)
{
	register int planes;
	register struct cg4_data *cgd;
	register int group;

	if (planesp == 0)
		return (0);

	planes = *planesp;
	/*
	 * for those call putattributes with ~0.  This can be better
	 * handled if the check for PIX_DONT_SET_PLANE is inverted.
	 */
	if (planes == ~0)
		planes &= ~PIX_DONT_SET_PLANES;
	cgd = cg4_d(pr);

	/*
	 * If user is trying to set the group, look for the frame
	 * buffer which implements the desired group and make it
	 * the active frame buffer.
	 */
	group = PIX_ATTRGROUP(planes);
	if (group == PIX_GROUP_MASK)
		planes &= PIX_ALL_PLANES;
	if (group != PIXPG_CURRENT || group == PIXPG_INVALID) {
		register int active;
		register struct cg4fb *fbp;
		int	cmsize;

		for (active = 0, fbp = cgd->fb;
			active < CG4_NFBS; active++, fbp++)
			if (group == fbp->group) {
				cgd->planes = PIX_GROUP(group);
				cgd->active = active;

				if (!(planes & PIX_DONT_SET_PLANES))
					fbp->mprp.planes =
						planes & PIX_ALL_PLANES;

				cgd->mprp = fbp->mprp;
				cgd->planes |= fbp->mprp.planes;
				pr->pr_depth = fbp->depth;
				cmsize = (pr->pr_depth==8) ? 256 : 2;
				cg4_ioctl(pr, FBIOSCMSIZE, (caddr_t)&cmsize);
				return (0);
			}
	}

	/* set planes for current group */
	if (!(planes & PIX_DONT_SET_PLANES)) {
		planes &= PIX_ALL_PLANES;
		cgd->planes = (cgd->planes & ~PIX_ALL_PLANES) | planes;
		cgd->mprp.planes = planes;
		cgd->fb[cgd->active].mprp.planes = planes;
	}

	return (0);
}

static int
tv2_putcolormap(Pixrect *pr, u_int index, u_int count, u_char *red, u_char *green, u_char *blue)
{
	register u_int rindex = (u_int) index;
	register u_int rcount = (u_int) count;
	register u_int entries;

	entries = TV2_CMAP_SIZE;

	/* check arguments */
	if ((rindex >= entries) || ((rindex + rcount) > entries))  {
		return (PIX_ERR);
	}

	if (rcount == 0)
		return (0);

/*
	plxv_load_clts(softc, 0, index , count, red, green, blue);
*/

	return (0);
}
/* kernel pixrect ops vector */
struct pixrectops tv2_ops = {
	mem_rop,
	tv2_putcolormap,
	cg4_putattributes,
	0,
};
#endif NWIN > 0

/* ARGSUSED */
int
tvtwo_ioctl_fb(dev_t dev, int cmd, int arg, int mode, cred_t * cred_p, int *rval_p)
{
	register tvtwo_softc *softc = TVTWO_DEV2SOFTC(dev);
	int error;
#ifdef SUN4M
	extern State_Trace state_trace;
#endif
	
	switch (cmd) {

#ifdef TV3
	case PV_DISP_ONOFF:		/* Turn display interrupt processing on or off. */
		{
			int enable;
			int mask;
			/* #### XV_WREG(softc->regs, TmrSpd.l, 0x171); */
			error = copyin(arg, &enable, sizeof(int));
			switch( enable )
			{
			  case 1:
				mask = XV_RSDW(softc->regs, IntMask.l) | 1;
				XV_WREG(softc->regs, IntMask.l, mask);
				XV_RREG(softc->regs, IntStatC.l); /* Clear register. */
				break;
			  case 0:
				mask = XV_RSDW(softc->regs, IntMask.l) & ~1;
				XV_WREG(softc->regs, IntMask.l, mask);
				XV_RREG(softc->regs, IntStatC.l); /* Clear register. */
				break;
			}
		}
		break;
	case PV_RES_SET:		/* Set the screen resolution. */
		{		
			int	ysize;

			error = copyin(arg, &ysize, sizeof(int));
			softc->nvram.main.ysize = ysize;
			softc->nvram.main.v.disp = ysize-1;
		}
		break;
#endif /* TV3 */
#ifdef SUN4M
	case FBIOGCURSOR:
	    if (ddi_copyout((caddr_t)&state_trace, (caddr_t)arg,
				sizeof (State_Trace), mode) != 0) {
			error = EFAULT;
		}	
		state_trace.i_entry = 0;
		break;
#endif
	case FBIOGETCMAP:
	case FBIOPUTCMAP:
		{
			struct	fbcmap	cmap;
			u_int index;
			u_int count;
			union	fbunit	*map;
			u_int entries;
			u_int pgroup;
			int		i, type;
			u_char		tmp[3][TV2_CMAP_SIZE];

			error = copyin(arg, &cmap, sizeof(struct fbcmap));

			if (error)
				return (error);

			index = (u_int) cmap.index;
			count = (u_int) cmap.count;
			pgroup = PIX_ATTRGROUP (index);

			if (count == 0)
				return (0);

			/* The most significant 7 bits is the group number. */
			switch (pgroup) {
			case 0:
			case PIXPG_8BIT_COLOR:
				type	= 0;
				entries	= TV2_CMAP_SIZE;
				break;
			case PIXPG_24BIT_COLOR:
				type	= 1;
				entries	= TV2_CMAP_SIZE;
				break;
			case PIXPG_OVERLAY:
				type	= 2;
				map	= softc->fbd.omap_rgb;
				entries	= TV2_OMAP_SIZE;
				break;
			default:
				return (EINVAL);
			}

			if (((index &= PIX_ALL_PLANES) >= entries) ||
				(index + count > entries)) {
				return (EINVAL);
			}

			if (cmd == FBIOPUTCMAP) {
				if (error = copyin((caddr_t)cmap.red,
					(caddr_t)tmp[0],count))
					return (error);
				if (error = copyin((caddr_t)cmap.green,
					(caddr_t)tmp[1],count))
					return (error);
				if (error = copyin((caddr_t)cmap.blue,
					(caddr_t)tmp[2],count))
					return (error);

				if(type < 2) {
					plxv_load_clts(softc, type, index , count,
						tmp[0], tmp[1], tmp[2]);
					return (0);
				}

				for (i=0,map+=index; count; i++,count--,map++)
					map->packed =
						tmp[0][i]|
						(tmp[1][i]<<8)|
						(tmp[2][i]<<16);

				plxv_colors_1(softc,
					softc->fbd.omap_rgb[3].packed |
					(PLXV_TAG_TRUE_COLOR << 24),
					softc->fbd.omap_rgb[1].packed |
					(PLXV_TAG_TRUE_COLOR << 24),
					PLXV_WMASK_ALL);
				plxv_wmask(softc);
				break;
			} else /* FBIOGETCMAP */ {
				if (count + index > TV2_CMAP_SIZE)
					return (EINVAL);
				if(type == 2) {
					for (i=0; i<count; i++,index++) {
						tmp[0][i] =
						softc->fbd.omap_rgb[index].channel.R;
						tmp[1][i] =
						softc->fbd.omap_rgb[index].channel.G;
						tmp[2][i] =
						softc->fbd.omap_rgb[index].channel.B;
					}
				} else {
					plxv_read_clts(softc, type, index , count,
						tmp[0], tmp[1], tmp[2]);
				}
				if (error = copyout((caddr_t)tmp[0],
					(caddr_t)cmap.red,count))
					return (error);
				if (copyout((caddr_t)tmp[0],
					(caddr_t)cmap.red,count))
					return (error);
				if (error = copyout((caddr_t)tmp[1],
					(caddr_t)cmap.green,count))
					return (error);
				if (error = copyout((caddr_t)tmp[2],
					(caddr_t)cmap.blue,count))
					return (error);
				break;
			}
		}
	case FBIOSATTR:
		{
			struct fbsattr sattr;

#ifdef ONLY_OWNER_CAN_SATTR
			/* this can only happen for the owner */
			if (softc->fbd.owner != curproc)
				return (ENOTTY);
#endif ONLY_OWNER_CAN_SATTR

			error = copyin(arg, &sattr, sizeof(struct fbsattr));

			if (error) {
				return (error);
			}

			softc->fbd.gattr.sattr.flags = sattr.flags;
			if (sattr.emu_type != -1) {
				DEBUGF((CE_CONT,
					"tvtwo: ioctl: FBIOSATTR emu type=%d\n",
					sattr.emu_type));
				softc->fbd.gattr.sattr.emu_type = sattr.emu_type;
			}

			if (sattr.flags & FB_ATTR_DEVSPECIFIC) {
				bcopy ((char *)sattr.dev_specific,
					(char *)softc->fbd.gattr.sattr.dev_specific,
					sizeof sattr.dev_specific);

				if (softc->fbd.gattr.sattr.dev_specific
				    [FB_ATTR_TV2_SETOWNER_CMD] == 1) {
					struct proc *newowner = 0;

					if (softc->fbd.gattr.sattr.dev_specific
					    [FB_ATTR_TV2_SETOWNER_PID] > 0 &&
					    (newowner = pfind (
					    softc->fbd.gattr.sattr.dev_specific
					    [FB_ATTR_TV2_SETOWNER_PID]))) {
						softc->fbd.owner = newowner;
						softc->fbd.gattr.owner =
							newowner->p_pid;
					}
					softc->fbd.gattr.sattr.dev_specific
						[FB_ATTR_TV2_SETOWNER_CMD] = 0;
					softc->fbd.gattr.sattr.dev_specific
						[FB_ATTR_TV2_SETOWNER_PID] = 0;

					if (!newowner)
						return (ESRCH);
				}
				else if (softc->fbd.gattr.sattr.dev_specific
				    [FB_ATTR_TV2_SETOWNER_CMD] == 2) {
					     softc->sfield = softc->fbd.gattr.sattr.dev_specific[1];
					if((softc->sfield) < 0 || (softc->sfield > 1))
					{
						softc->sfield = -1;
						return EINVAL;
					}
				}
				else if (softc->fbd.gattr.sattr.dev_specific
				    [FB_ATTR_TV2_SETOWNER_CMD] == 3) {
					int t;
					softc->ffield = softc->fbd.gattr.sattr.dev_specific[1];
					t = XV_RSDW(softc->regs, IntMask.l);
					if((softc->ffield) < 0 || (softc->ffield > 1))
						softc->ffield = -1;
					if(softc->ffield == -1)
						t &= ~0x8;			/* shut int off */
					else
					    t |= 0x8;			/* turn int on */
					XV_WREG(softc->regs, IntMask.l, t);
				}
			}
		}
		break;

	case FBIOGATTR:
		{
			struct fbgattr gattr = tv2attrdefault;

			gattr.fbtype.fb_height = softc->fbd.h;
			gattr.fbtype.fb_width = softc->fbd.w;
			gattr.fbtype.fb_size = TVTWO_FBSIZE(softc, 8);

		        if (ddi_copyout((caddr_t)&gattr, (caddr_t)arg,
				sizeof (gattr), mode) != 0) {
			        error = EFAULT;
			        break;
		        }
		}
		break;

	case FBIOGTYPE:
		{
			struct fbtype fb;

			/* set owner if not owned or previous owner is dead */
			if (softc->fbd.owner == NULL ||
			    softc->fbd.owner->p_stat == NULL ||
			    softc->fbd.owner->p_pid != softc->fbd.gattr.owner) {

				softc->fbd.owner = curproc;
				softc->fbd.gattr.owner = curproc->p_pid;
			}

			switch (softc->fbd.gattr.sattr.emu_type) {
			case FBTYPE_SUN4COLOR:
				fb = tv2typedefault;
				fb.fb_height = softc->fbd.h;
				fb.fb_width = softc->fbd.w;
				fb.fb_size = TVTWO_FBSIZE(softc, 8);
				break;
			default:
				fb = softc->fbd.gattr.fbtype;
				break;
			}
			if (error = copyout(&fb, arg, sizeof(struct fbtype)))
				return (error);
		}
		break;

#if NWIN > 0

	case FBIOGPIXRECT:
		{
			struct fbpixrect fbpr;
			register struct cg4fb *fbp;
			register struct cg4_fbdesc *descp;
			register int i;

			mutex_enter(&softc->fbd.pr_lock);

			/* "Allocate" pixrect and private data */
			fbpr.fbpr_pixrect = &softc->fbd.pr;
			softc->fbd.pr.pr_data = (caddr_t) &softc->fbd.cgd;

			/* initialize pixrect */
			softc->fbd.pr.pr_ops = &tv2_ops;
			softc->fbd.pr.pr_size.x = softc->fbd.w;
			softc->fbd.pr.pr_size.y = softc->fbd.h;

			/* initialize private data */
			softc->fbd.cgd.flags = 0;
			softc->fbd.cgd.planes = 0;
			softc->fbd.cgd.fd = minor (dev);

			/* set up everything except for the image pointer */
			for (fbp = softc->fbd.cgd.fb, descp = cg4_fbdesc, i = 0;
			    i < TV2_NFBS; fbp++, descp++, i++) {
				fbp->group = descp->group;
				fbp->depth = descp->depth;
				fbp->mprp.mpr.md_linebytes =
				    mpr_linebytes (softc->fbd.w, descp->depth);
				fbp->mprp.mpr.md_image = (short *) softc->fbd.fb[i];
				fbp->mprp.mpr.md_offset.x = 0;
				fbp->mprp.mpr.md_offset.y = 0;
				fbp->mprp.mpr.md_primary = 0;
				fbp->mprp.mpr.md_flags = 
					(descp->allplanes != 0)
					? MP_DISPLAY | MP_PLANEMASK
					: MP_DISPLAY;
				fbp->mprp.planes = descp->allplanes;

			}

			{
				u_int		initplanes;
				/*
				Pixrect		*tmppr = &softc->pr;
				*/

				initplanes =
					PIX_GROUP(cg4_fbdesc[TV2_FBINDEX_COLOR].group) |
					cg4_fbdesc[TV2_FBINDEX_COLOR].allplanes;

/*
				(void) cg4_putattributes (tmppr, &initplanes);
*/
			}

		if (error = kcopy(&fbpr, arg, sizeof(struct fbpixrect))) {
			mutex_exit(&softc->fbd.pr_lock);
			return (error);
		}

		mutex_exit(&softc->fbd.pr_lock);
		}
		break;

#endif NWIN > 0

	case FBIOSVIDEO:
		{
			int on;

			if (error = copyin(arg, &on, sizeof(int)))
				return (error);

			softc->fbd.blank = on & FBVIDEO_ON;
			plxv_disp_blank(softc, (softc->fbd.blank) ? 0 : 1);
		}
		break;

		/* get video flags */
	case FBIOGVIDEO:
		if (error = copyout(&softc->fbd.blank, arg, sizeof(int)))
			return (error);
		break;

	case VIS_GETIDENTIFIER:
		if (error = copyout(vis_id_name, arg, sizeof(vis_id_name)))
			return (error);
		break;

	default:
		return (ENOTTY);

	} /* switch(cmd) */
	return (0);
}

static int
cg4_ioctl (Pixrect *pr, int cmd, caddr_t data)
{
    static int      cmsize;

	switch (cmd) {
	
		case FBIOGPLNGRP:
		*(int *) data = PIX_ATTRGROUP(cg4_d(pr)->planes);
		break;

		case FBIOAVAILPLNGRP:
		{
			static int      cg4groups =
			MAKEPLNGRP(PIXPG_OVERLAY) |
			MAKEPLNGRP(PIXPG_OVERLAY_ENABLE) |
			MAKEPLNGRP(PIXPG_8BIT_COLOR);

			*(int *) data = cg4groups;
			break;
		}

		case FBIOGCMSIZE:
		if (cmsize < 0)
			return (-1);

		*(int *) data = cmsize;
		break;

		case FBIOSCMSIZE:
		cmsize = *(int *) data;
		break;

		default:
		return (ENOTTY);
	}
	return (0);
}

void
tvtwo_init_fb(tvtwo_softc *softc)
{
	switch(freqcode)
	{
	    case '1':
	    case '2':
	    case '3':
	    default:
		softc->fbd.w = 1152;
		softc->fbd.h = 900;
		break;
	    case '4':
		softc->fbd.w = 1024;
		softc->fbd.h = 768;
		break;
	    case '5':
		softc->fbd.w = 640;
		softc->fbd.h = 480;
		break;
	    case 'g':
		softc->fbd.w = 1024;
		softc->fbd.h = 1024;
		break;
	}
/*
	softc->fbd.w = (int)softc->nvram.main.xsize;
	softc->fbd.h = (int)softc->nvram.main.ysize;
*/
	softc->fbd.gattr.fbtype = tv2typedefault;
	softc->fbd.gattr.fbtype.fb_height = softc->fbd.h;
	softc->fbd.gattr.fbtype.fb_width = softc->fbd.w;
	softc->fbd.gattr.fbtype.fb_size = TVTWO_FBSIZE(softc, 8);
	/*
	 * Map in the frame buffers XXX
	 */
	softc->fbd.fb[0] = softc->fbd.fb[1] = softc->fbd.m1;
	softc->fbd.fb[2] = softc->fbd.m8;
}
