/*
 * Copyright (c) 1993 Parallax Graphics Inc.
 */

#pragma ident "@(#)tvtwo_init.c	1.8 Copyright (c) 1993 Parallax Graphics Inc"

#include "tvtwo.h"
#include "xpoint.h"
#include "xvideo.h"

extern char freqcode;

int
tvtwo_init(tvtwo_softc *softc);

void
tvtwo_uninit(tvtwo_softc *softc);

int
tvtwo_reinit(tvtwo_softc *softc);

static void
tvtwo_init_disp(tvtwo_softc *softc);

static void
tvtwo_init_draw(tvtwo_softc *softc);

static void
tvtwo_init_color(tvtwo_softc *softc);

#ifdef not_obsolete
static void
tvtwo_init_bt(tvtwo_softc *softc);
#endif

static void
tvtwo_init_video(tvtwo_softc *softc);

int
tvtwo_init(tvtwo_softc *softc)
{
	/*
	 * Map the 1 and 8 bit framebuffers
	 */
	if (TVTWO_MAP_REGS(softc, softc->regmap->m1, softc->fbd.m1)
		!= DDI_SUCCESS) {
		cmn_err(CE_WARN,
			"tvtwo%d: can't map M1 registers\n",
			ddi_get_instance(softc->devi));
		return (DDI_FAILURE);
	}
	if (TVTWO_MAP_REGS(softc, softc->regmap->m8, softc->fbd.m8)
		!= DDI_SUCCESS) {
		cmn_err(CE_WARN,
			"tvtwo%d: can't map M8 registers\n",
			ddi_get_instance(softc->devi));
		TVTWO_UNMAP_REGS(softc, softc->regmap->m1, softc->fbd.m1);
		return (DDI_FAILURE);
	}

	/*
	 * Read our configuration from the nvram.
	 * We shouldn't fail here because the user
	 * could reinitialize by writting into the
	 * nvram device.
	 */
	if (tvtwo_nvram_init(softc->regs, &softc->nvram) != 0) {
		/*return (DDI_FAILURE); XXX */
	}

	/*
	 * Initialize basic framebuffer support
	 */
	tvtwo_init_fb(softc);

	/*
	 * The rest is to initialize the XVideo hardware
	 */
	tvtwo_init_disp(softc);

	/*
	 * Initialize the timer
	 */
	XV_WREG(softc->regs, TmrSpd.l, 0x171);	/* 90,000 timer ticks/second */

	tvtwo_init_video(softc);
	tvtwo_init_draw(softc);
	tvtwo_init_color(softc);

	if(softc->nvram.main.c3_rev>0)
	    c3_init(softc);

	return (DDI_SUCCESS);
}

void
tvtwo_uninit(tvtwo_softc *softc)
{
	/*
	 * Unmap the framebuffers
	 */
	TVTWO_UNMAP_REGS(softc, softc->regmap->m1, softc->fbd.m1);
	TVTWO_UNMAP_REGS(softc, softc->regmap->m8, softc->fbd.m8);
	/* XXX disable interrupts XXX */
}

enable_ints(tvtwo_softc *softc)
{
	XV_WREG(softc->regs,IntMask.l, softc->imask);	/* turn on int's */
	XV_WREG(softc->regs,BIntMask.l, softc->bmask);	/* turn on int's */
}

int
tvtwo_reinit(tvtwo_softc *softc)
{
	tvtwo_uninit(softc);
	return tvtwo_init(softc);
}
set_int_mask(tvtwo_softc *softc,int mask)
{
printf("set_int_mask: mask=0x%x\n",mask);
	XV_WREG(softc->regs, IntMask.l, mask);
}

static void
set_bt463_addr(XV_regs *regs, int addr)
{
    int a,b;

    a = (addr >> 8) & 0xff;
    b = addr & 0xff;
    XV_WREG(regs, Bt.red.reg0, b); /* load low order Bt463 address register */
    XV_WREG(regs, Bt.red.reg1, a); /* load high order Bt463 address register */
}

static void
plxv_set_bt463_wtab(XV_regs *regs, int tnum, int shft, int planes, int mode, int ovlyloc, int ovlymsk, int strtad, int bypass)
{
    int val;

    if (tnum > 0xf)
	cmn_err(CE_WARN, "error: table number out of range\n");
    val  =  (shft    & 0x1f)       | ((planes  & 0xf)  << 5);
    val |= ((mode    & 0x7) << 9)  | ((ovlyloc & 0x1)  << 12);
    val |= ((ovlymsk & 0xf) << 13) | (((strtad >> 4)  & 0x3f) << 17);
    val |= ((bypass & 0x1) << 23) ;
    set_bt463_addr(regs, 0x300 | (tnum & 0xf));
    XV_WREG(regs, Bt.red.reg2, val & 0xff);
    XV_WREG(regs, Bt.red.reg2, (val >> 8)  & 0xff);
    XV_WREG(regs, Bt.red.reg2, (val >> 16) & 0xff);
}

static void
plxv_set_normal_463(XV_regs *regs)
{
    int i,t = 0;
    plxv_set_bt463_wtab(regs, t++,0,8,0,0,0,0    ,0);    /* 0:pseudo     0 - ff */
    plxv_set_bt463_wtab(regs, t++,0,8,0,0,0,0    ,1);    /* 1:true       bypass */
    plxv_set_bt463_wtab(regs, t++,0,8,0,0,0,0    ,1);    /* 2:video 1    bypass */
    plxv_set_bt463_wtab(regs, t++,0,8,0,0,0,0    ,1);    /* 3:video 2    bypass */

    plxv_set_bt463_wtab(regs, t++,0,8,0,0,0,0    ,0);    /* 4:pseudo     0 - ff */
    plxv_set_bt463_wtab(regs, t++,0,8,0,0,0,0    ,1);    /* 5:true       bypass */
    plxv_set_bt463_wtab(regs, t++,0,8,0,0,0,0    ,1);    /* 6:video 1    bypass */
    plxv_set_bt463_wtab(regs, t++,0,8,0,0,0,0    ,1);    /* 7:video 2    bypass */

    for (i=0;i<4;i++)
	plxv_set_bt463_wtab(regs, t++,27,1,0,0,0,0x100,0);	/* cursor forground */
    for (i=0;i<4;i++)
	plxv_set_bt463_wtab(regs, t++,27,1,0,0,0,0x110,0);	/* cursor background */
}

static void
tvtwo_init_bt463(tvtwo_softc *softc)
{
    /* load the BT463 command registers 0,1,&2 BANG BANG BANG */
    switch(softc->nvram.main.board_code) {
	case SBUS_1:
	case SBUS_2:
	case SBUS_3:
	case SBUS_4:
	case SBUS_5:
	    set_bt463_addr(softc->regs, 0x201);
	    XV_WREG(softc->regs, Bt.red.reg2, 0x40);	/* 4:1 mux , blink xx */
	    XV_WREG(softc->regs, Bt.red.reg2, 0x40);	/* overlay common, 24 planes, 4 over */
					/* 16 tables, no under, no cursor */
	    XV_WREG(softc->regs, Bt.red.reg2, 0x40);	/* no sync ,pedistal */
	    break;
	default:
	    cmn_err(CE_CONT, "plxv_init_bt463: unkown board type\n");
    }
    /* load read mask register = all planes */
    set_bt463_addr(softc->regs, 0x205);
    XV_WREG(softc->regs, Bt.red.reg2, 0xff);
    XV_WREG(softc->regs, Bt.red.reg2, 0xff);
    XV_WREG(softc->regs, Bt.red.reg2, 0xff);
    XV_WREG(softc->regs, Bt.red.reg2, 0xff);

   /* no blinking mask */
    XV_WREG(softc->regs, Bt.red.reg2, 0);
    XV_WREG(softc->regs, Bt.red.reg2, 0);
    XV_WREG(softc->regs, Bt.red.reg2, 0);
    XV_WREG(softc->regs, Bt.red.reg2, 0);

    /* NOTE: the BT463 must be in BLANK to load the window table */
    /* so we will turn off the display via vreset !!!! */
    /* check GA REV HERE */
    switch(softc->nvram.main.ga_rev)
    {
	case GAREV_2:
	    XV_WREG(softc->regs, DispKludge.b.disp_blank,1);
	    break;
	default:
	    cmn_err(CE_CONT, "463init: don't know what rev GAs in use\n");
    }

    plxv_set_normal_463(softc->regs);

    /* now turn back on the display */
    switch(softc->nvram.main.ga_rev)
    {
        case GAREV_2:
	    XV_WREG(softc->regs, DispKludge.b.disp_blank,0);
            break;
        default:
	    break;
    }

}

static void
set_option_rsaddr(XV_regs *regs, int code, int addr)
{
	XV_WREG(regs, Ext.reg0, ((code&0x3)<<3)|(addr&0x7));
}

static void
tvtwo_init_bt473(tvtwo_softc *softc)
{
	/* set command register */
	set_option_rsaddr(softc->regs, 3,6);
	XV_WREG(softc->regs, Ext.reg1,0x30);

	/* set read mask */
	set_option_rsaddr(softc->regs, 3,2);
	XV_WREG(softc->regs, Ext.reg1,0xff);
}

typedef struct XV_color {
	u_char red;
	u_char green;
	u_char blue;
} XV_color;

#ifdef not_obsolete
static void
tvtwo_init_bt(tvtwo_softc *softc)
{
	if (softc->nvram.main.clt_type != BT463) {
		cmn_err(CE_WARN, "tvtwo: unknown color table type (0x%x)\n",
			softc->nvram.main.clt_type);
	} else {
		tvtwo_init_bt463(softc);
	}
	
	if (softc->nvram.opt.clt_type == BT473) {
		tvtwo_init_bt473(softc);
	}
	
	{
		XV_color init_c[0x100];
		int i,j;

		for (i=0,j=0;i<0x100;i++)
		{
			init_c[j].red    =  i;
			init_c[j].green  =  i;
			init_c[j++].blue =  i;
		}
		plxv_load_clt(softc,1,0,0x100,init_c);
		plxv_load_clt(softc,2,0,0x100,init_c);
		plxv_load_clt(softc,3,0,0x100,init_c);
	}
}

static int
plxv_load_clt(tvtwo_softc *softc, int tag, int index, int count, XV_color *cptr)
{
	int i,j;
	XV_color *ptr;

	switch(softc->nvram.main.clt_type) {
		case BT463:
			if (tag > 1)
				break;	/* do nothing for video palettes on 463 */
			set_bt463_addr(softc->regs, (0x100 * tag) + index);
			for (i=0, ptr=cptr; i<count; i++) {
				j = ptr->red;
				XV_WREG(softc->regs, Bt.red.reg3,j);
				j = ptr->green;
				XV_WREG(softc->regs, Bt.red.reg3,j);
				j = ptr->blue;
				XV_WREG(softc->regs, Bt.red.reg3,j);
				ptr++;
			}
			break;
		default:
			cmn_err(CE_WARN, "tvtwo: unknown color table type\n");
			return(-1);
	}

	/* now make the video output card's color tables match */
	switch(softc->nvram.opt.clt_type) {
		case BT473:
			if ( tag > 0)
				break;		/* only pseudo-color palette avail */
			set_option_rsaddr(softc->regs, 3,0); /* setup to load start address */
			XV_WREG(softc->regs, Ext.reg1,index);
			set_option_rsaddr(softc->regs, 3,1); /* setup to load palette data */
			for (i=0,ptr=cptr; i<count; i++) {
				j = ptr->red;
				XV_WREG(softc->regs, Ext.reg1,j);
				j = ptr->green;
				XV_WREG(softc->regs, Ext.reg1,j);
				j = ptr++->blue;
				XV_WREG(softc->regs, Ext.reg1,j);
			}
			break;
		default:
			break;
	}

	return(0);
}
#endif

static void
tvtwo_init_disp(tvtwo_softc *softc)
{
	if((softc->nvram.main.board_code != SBUS_1) &&
	   (softc->nvram.main.board_code != SBUS_2) &&
	   (softc->nvram.main.board_code != SBUS_3)) return;
/*
DEBUGF((CE_CONT, "updating disp parameters\n"));
*/
	XV_WREG(softc->regs, DispFlgs.l, softc->nvram.main.DispFlgs);
	XV_WREG(softc->regs, DispKludge.l, softc->nvram.main.DispKludge);
	XV_WREG(softc->regs, HFP.l, softc->nvram.main.h.fp);
	XV_WREG(softc->regs, HSync.l, softc->nvram.main.h.sy);
	XV_WREG(softc->regs, HBP.l, softc->nvram.main.h.bp);
	XV_WREG(softc->regs, HDisp.l, softc->nvram.main.h.disp);
	XV_WREG(softc->regs, VFP.l, softc->nvram.main.v.fp);
	XV_WREG(softc->regs, VBP.l, softc->nvram.main.v.bp);
	XV_WREG(softc->regs, VDisp.l, softc->nvram.main.v.disp);
	XV_WREG(softc->regs, VSync.l, softc->nvram.main.v.sy);
	{
		int i;
		i = softc->nvram.main.h.hsync_ser;
		if(i & 0x80)
			i += 0x80;
		XV_WREG(softc->regs, HSyncS.l, i);
	}
	XV_WREG(softc->regs, HNSyncS.l, softc->nvram.main.h.hsync_nser);
	XV_WREG(softc->regs, OLDI2C.l, softc->nvram.main.OLDI2C);
}

static void
tvtwo_init_draw(tvtwo_softc *softc)
{
	/*
	 * disable video blue hardware here for now
	 */
	XV_WREG(softc->regs, KeyColCtl.l, 0);

	/*
	 * Wait for accelerator
	 */
	while(XV_RREG(softc->regs, DBusy.l))
		;

	XV_WREG(softc->regs, MStart.l, 0);
	XV_WREG(softc->regs, MEnd.l, 0x1f);
	XV_WREG(softc->regs, MOffset.l, 0);
	XV_WREG(softc->regs, WindowX.l, 0);
	XV_WREG(softc->regs, WindowY.l, 0);
}

void
plxv_wmask(tvtwo_softc *softc)
{
	/*
	 * Wait for accelerator
	 */
	while(XV_RREG(softc->regs, DBusy.l))
		;

	/*
	 * sets the write mask for all 28 bitplanes 1 bits enable writes
	 * to that bit, 0 bits disable writes.  This overrides all tag &
	 * stensil modes.
	 *
	 * NOTE: video cycles can never write to the TAG planes (bits 24-27)             */
	XV_WREG(softc->regs, DrawMask.l, PLXV_WMASK_PIXEL_ALL);
}

static void
tvtwo_init_color(tvtwo_softc *softc)
{
	/*
	 * 28 now?
	 */
	XV_WREG(softc->regs, RopCtl.b.em28, 0);

	/*
	 * Wait for accelerator
	 */
	while(XV_RREG(softc->regs, DBusy.l))
		;

	/*
	 * CONFIGURE the TAG value override control
	 * when writing pixels the tag planes are determined by MUX & VAL
	 * if a bit in MUX is on then the corresponding bit in VAL is written
	 * instead of the source pixel value (from bus or blit)
	 * thus if MUX =0, the FIFO contents are written to the TAG.
	 */

	XV_WREG(softc->regs, TagCtl.b.ovr_mux, PLXV_TAG_PIXEL_ATTR);
	XV_WREG(softc->regs, TagCtl.b.ovr_val, PLXV_TAG_PSEUDO_COLOR);

	plxv_wmask(softc);

	/*
	 * Stencil decides on a pixel by pixel basis
	 * if a pixel is to written to the memories. 
	 * if (PIXEL'S TAG & MASK) == VALUE, then write the pixel
	 */
	XV_WREG(softc->regs, RopCtl.b.rmw, 1); /* XXX plxv_stencil? XXX */
	XV_WREG(softc->regs, TagClip.b.mask, 0);
	XV_WREG(softc->regs, TagClip.b.comp, 0);

	/*
	 * Wait for accelerator
	 */
	while(XV_RREG(softc->regs, DBusy.l))
		;

	/*
	 * Sets the ROP type for all MAP access and accell operations
	 * normally set to COPY....   See plxv.h for all ROP modes
	 */
	XV_WREG4(softc->regs, DrawRop.l, PLXV_ROP_COPY);
	XV_WREG(softc->regs, RopCtl.b.rmw, 1); /* XXX plxv_stencil? XXX */

	/*
	 * Wait for accelerator
	 */
	while(XV_RREG(softc->regs, DBusy.l))
		;

	/*
	 * disable 1 bit transparent mode .... must do this if not in 1 bit map
	 */
	XV_WREG(softc->regs, MMode.b.trans,0);

	/*
	 * Wait for accelerator
	 */
	while(XV_RREG(softc->regs, DBusy.l))
		;
	
	/*
	 * support the offsetting map hardware in all access modes
	 */
	XV_WREG(softc->regs, MStart.l, 0);
	XV_WREG(softc->regs, MEnd.l, 0x1f);
	XV_WREG(softc->regs, MOffset.l, 0);

	/*
	 * Wait for accelerator
	 */
	while(XV_RREG(softc->regs, DBusy.l))
		;
	
	/*
	 * select which byte to read from when accessing 8 bit map
	 * select=0 means readback red, 1=green, 2= blue and 3=tag
	 */
	XV_WREG(softc->regs, MMode.b.m8rb, 0);

	plxv_wmask(softc);
}
ccube_wchan_mode(tvtwo_softc *softc,int mode)
{
u_int bits;

	if(mode == PLAY) /* Decompress to w channel input */
	{
	    XV_WREG(softc->regs,CFormat.l,CE_HIGH);
	    bits= SEL_DATA_INPUT_C|SEL_CTL_INPUT_C|SEL_CLK_INPUT_C|SEL_CLK_HIGH;
	    XV_WREG(softc->regs,WFormat.l,bits);
	    /*
	    XV_WREG4(softc->regs,WCtl.l,2);
	     */
	    XV_WREG4(softc->regs,WCtl.l,4);
	}
	else /* Compress from w channel output */
	{
	    XV_WREG(softc->regs,CFormat.l,0x60616);
	    bits= SEL_CTL_INPUT_C|SEL_CLK_INPUT_C;
	    XV_WREG(softc->regs,WFormat.l,bits);
	    /*
	    XV_WREG4(softc->regs,WCtl.l,3);
	    */
	    XV_WREG4(softc->regs,WCtl.l,5);
	}
}

static void
tvtwo_init_video(tvtwo_softc *softc)
{
	/*
	 * Initialize the video input hardware and crosspoint switch.
	 * Set both V and W channels to recieve the onboard D channel.
	 * Set DMSD for NTSC mode, and turn off flashing NOW
	 */

	XV_WREG(softc->regs, VFormat.l,
		SEL_DATA_INPUT_D | SEL_CTL_INPUT_D |
		SEL_CLK_INPUT_D | SEL_CLK_HIGH);
	XV_WREG(softc->regs, WFormat.l,
		SEL_DATA_INPUT_D | SEL_CTL_INPUT_D |
		SEL_CLK_INPUT_D | SEL_CLK_HIGH);

	/*
	 * Stop flashing on both ports NOW
	 */
	XV_WREG(softc->regs, VCtl.b.now, 1);
	XV_WREG(softc->regs, WCtl.b.now, 1);
	XV_WREG(softc->regs, VCtl.b.single, 0);
	XV_WREG(softc->regs, WCtl.b.single, 0);
	XV_WREG(softc->regs, VCtl.b.cont, 0);
	XV_WREG(softc->regs, WCtl.b.cont, 0);

	/*
	 * V flash off
	 */
	XV_WREG(softc->regs, VCtl.l, 0);

	/*
	 * W flash off (don't clear the direction bit)
	 */
	{
		int dir = XV_RSDW(softc->regs, WCtl.l);
		XV_WREG(softc->regs, WCtl.l, dir & ~0x60606);
	}

	XV_WREG(softc->regs, VStart.l, 4);
	XV_WREG(softc->regs, VStop.l, 260);
	XV_WREG(softc->regs, WStart.l, 4);
	XV_WREG(softc->regs, WStop.l, 260);
}

int
plxv_load_clts(tvtwo_softc *softc, int tag, int index, int count, char *rptr0, char *gptr0, char *bptr0)
{
	int i,j;
	char *rptr, *gptr, *bptr;

	rptr = rptr0;  gptr = gptr0;  bptr = bptr0;  
	switch(softc->nvram.main.clt_type) {
	case BT463:
		if (tag > 1)
			return(0); /* do nothing for video palettes on 463 */
		set_bt463_addr(softc->regs, (0x100 * tag) + index);
		for (i=0; i<count; i++) {
			j = *rptr++;
			XV_WREG(softc->regs, Bt.red.reg3,j);
			j = *gptr++;
			XV_WREG(softc->regs, Bt.red.reg3,j);
			j = *bptr++;
			XV_WREG(softc->regs, Bt.red.reg3,j);
		}
		break;
	default:
		cmn_err(CE_WARN, "tvtwo: unknown color table type\n");
		return(-1);
	}

	/* now make the video output card's color tables match */
	rptr = rptr0;  gptr = gptr0;  bptr = bptr0;  
	switch(softc->nvram.opt.clt_type) {
		case BT473:
			if ( tag > 0)
				break;	/* only pseudo-color palette avail */
			set_option_rsaddr(softc->regs, 3,0); /* setup to load start address */
			XV_WREG(softc->regs, Ext.reg1,index);
			set_option_rsaddr(softc->regs, 3,1);	/* setup to load palette data */
			for (i=0; i<count; i++) {
				j = *rptr++;
				XV_WREG(softc->regs, Ext.reg1,j);
				j = *gptr++;
				XV_WREG(softc->regs, Ext.reg1,j);
				j = *bptr++;
				XV_WREG(softc->regs, Ext.reg1,j);
			}
			break;
		default:
			break;
	}

	return(0);
}

int
plxv_read_clts(tvtwo_softc *softc, int tag, int index, int count, char *rptr, char *gptr, char *bptr)
{
	int i;

	if (index + count > TV2_CMAP_SIZE) {
		DEBUGF((CE_CONT, "read_clts: 0x%x/0x%x\n", index, count));
		return(-1);
	}
	switch(softc->nvram.main.clt_type) {
		case BT463:
			if (tag > 1)
				return(0); /* do nothing for video palettes on 463 */
			set_bt463_addr(softc->regs, (0x100 * tag) + index);
			for (i=0; i<count; i++) {
				*rptr++ = XV_RREG(softc->regs, Bt.red.reg3) & 0xff;
				*gptr++ = XV_RREG(softc->regs, Bt.red.reg3) & 0xff;
				*bptr++ = XV_RREG(softc->regs, Bt.red.reg3) & 0xff;
			}
			break;
		default:
			cmn_err(CE_WARN, "tvtwo: unknown color table type\n");
			return(-1);
	}
	return (0);
}

void
plxv_colors_1(tvtwo_softc *softc, int fgcolor, int bgcolor, int readback_mask)
{
	/*
	 * Wait for accelerator
	 */
	while(XV_RREG(softc->regs, DBusy.l))
		;
	XV_WREG(softc->regs, M1ForCol.l,fgcolor);
	XV_WREG(softc->regs, M1BakCol.l,bgcolor);
	XV_WREG(softc->regs, M1RdMask.l,readback_mask);
}

void
plxv_disp_blank(tvtwo_softc *softc, int sel)
{
/*
	switch (softc->nvram.main.ga_rev) {
		case GAREV_2:
			XV_WREG(softc->regs, DispKludge.b.disp_blank, sel);
	}
*/
}
