/*
 * Copyright (c) 1993 Parallax Graphics Inc.
 */

#pragma ident "@(#)tvtwo_svc.c	1.6 Copyright (c) 1993 Parallax Graphics Inc"

/*
 * TVTWO device driver template
 */

#include <sys/types.h>
#include <sys/param.h>
#include <sys/cred.h>
#include <sys/errno.h>
#include <sys/cmn_err.h>
#include <sys/open.h>
#include <sys/file.h>
#include <sys/kmem.h>
#include <sys/t_lock.h>
#include <sys/stat.h>
#include <sys/conf.h>
#include <sys/ddi.h>
#include <sys/sunddi.h>

#include "tvtwo.h"

extern void ccube_svc_dma(), ccube_svc_c3(),ccube_svc_v_chan(),ccube_svc_w_chan();
void
tvtwo_service(tvtwo_softc *softc, void (*[])(), u_int bits);

void
tvtwo_continue_dma(tvtwo_softc *softc);

void
tvtwo_pause_dma(tvtwo_softc *softc);

static void
tvtwo_svc_dma(tvtwo_softc *softc);

static void
tvtwo_svc_timer(tvtwo_softc *softc);

static void
tvtwo_svc_disp(tvtwo_softc *softc);

static void
tvtwo_svc_c3(tvtwo_softc *softc);

static void
tvtwo_svc_ext(tvtwo_softc *softc);

static void
tvtwo_svc_video(tvtwo_softc *softc);

static void
tvtwo_svc_w_chan(tvtwo_softc *softc);

static void
tvtwo_svc_blit(tvtwo_softc *softc);

static void
tvtwo_null(tvtwo_softc *softc);

static void
tvtwo_svc_sbus(tvtwo_softc *softc);

void (*bint_services[])() = {
	tvtwo_svc_dma,
	tvtwo_svc_timer,
	tvtwo_null,
	tvtwo_svc_sbus
};

void (*int_services[])() = {
	tvtwo_svc_disp,		/* disp */
	tvtwo_svc_c3,		/* c3 */
	tvtwo_svc_ext,		/* ext */
	tvtwo_svc_blit,		/* blit */
    tvtwo_svc_video,    /* vsync */
    tvtwo_svc_video,    /* wsync */
    tvtwo_svc_video,    /* verr */
    tvtwo_svc_video,    /* werr */
	tvtwo_null,		/* vodd */
	tvtwo_null,		/* wodd */
	tvtwo_null,		/* vnsync */
	tvtwo_null		/* wnsync */
};
void (*all_services[])() = {
	ccube_svc_dma,		/* dmabuf 0x0001 */
	tvtwo_svc_timer,	/* timer  0x0002 */
	tvtwo_svc_disp,		/* disp   0x0004 */
	ccube_svc_c3,		/* c3     0x0008 */
	tvtwo_svc_ext,		/* ext    0x0010 */
	ccube_svc_v_chan,	/* vsync  0x0020 */
	ccube_svc_w_chan,	/* wsync  0x0040 */
	tvtwo_null,	/* verr */
	tvtwo_null,	/* werr */
	tvtwo_svc_blit,		/* blit */
};

void
tvtwo_service(tvtwo_softc *softc, void (*services[])(), u_int bits)
{
	while(bits) {
		if (bits & 0x1) {
			(*services)(softc);
		}
		bits = bits >> 1;
		services++;
	}
}

void
tvtwo_continue_dma(tvtwo_softc *softc)
{
	if(softc->state > 0) {
		if(softc->state & STATEW) {
			XV_WREG(softc->regs, CCube.l,
				XVC3_CONT_DMA_FROM_HOST_1);
			XV_WREG(softc->regs, CCube.l,
				XVC3_CONT_DMA_FROM_HOST_2);
		} else {
			XV_WREG(softc->regs, CCube.l,
				XVC3_CONT_DMA_TO_HOST_1);
			XV_WREG(softc->regs, CCube.l,
				XVC3_CONT_DMA_TO_HOST_2);
		}
	}
}


void
tvtwo_pause_dma(tvtwo_softc *softc)
{
	if (softc->state != 0) {
		if (softc->state & STATEW) {
			XV_WREG(softc->regs, CCube.l, XVC3_PAUSE_DMA_FROM_HOST);
			DELAY(10);
			XV_WREG(softc->regs, CCube.l, XVC3_WRITE_GO_PIO);
		} else {
			XV_WREG(softc->regs, CCube.l, XVC3_PAUSE_DMA_TO_HOST);
			DELAY(50);
			XV_WREG(softc->regs, CCube.l, XVC3_PURGE_BIC_GO_PIO);
		}
	}
}

static void
tvtwo_svc_dma(tvtwo_softc *softc)
{
	if ((softc->state == STATE0) || (softc->state == STATE_ERR))
		return;

	xvbuf_int_svc(softc);

	if ((softc->state == STATE0) || (softc->state == STATE_ERR)) {
		cv_broadcast(&softc->dmadone);
	}
}
static void
tvtwo_svc_sbus(tvtwo_softc *softc)
{
	DEBUGF((CE_WARN, "tvtwo_svc_sbus%d\n",
		ddi_get_instance(softc->devi)));

}

static void
tvtwo_svc_timer(tvtwo_softc *softc)
{
	DEBUGF((CE_CONT, "tvtwo_svc_timer%d\n",
		ddi_get_instance(softc->devi)));

	/*plxv_set_hourint();*/
	/*plxv_inc_hours();*/
}

static void
tvtwo_svc_disp(tvtwo_softc *softc)
{
/*
	DEBUGF((CE_CONT, "tvtwo_svc_disp%d\n",
		ddi_get_instance(softc->devi)));
 */
#ifdef TV3
	tvtwo_pll_alignment( softc );
#else /* ! TV3 */
	softc->end_time = XV_RREG(softc->regs,TmrCnt.l);
	printf("D%d\n",softc->end_time-softc->start_time);
	softc->start_time=softc->end_time;
#endif /* TV3 */
}

static void
tvtwo_svc_c3(tvtwo_softc *softc)
{
	c3_int_svc(softc);

	if ((softc->state == STATE0) || (softc->state == STATE_ERR)) {
		cv_broadcast(&softc->dmadone);
	}
}

static void
tvtwo_svc_ext(tvtwo_softc *softc)
{
	DEBUGF((CE_CONT, "tvtwo_svc_ext%d\n",
		ddi_get_instance(softc->devi)));
}
/* 
enable_intr(tvtwo_softc *softc)
{
	softc->save_imask &= ~(_Int_Wid | _Int_Vid);
	XV_WREG(IntMask.l, softc->save_imask);
	XV_WREG(BIntMask.l, softc->save_bmask);
}
*/

static void
tvtwo_svc_video(tvtwo_softc *softc)
{
	int n;
	DEBUGF((CE_CONT, "tvtwo_svc_video%d\n",
		ddi_get_instance(softc->devi)));

	n = XV_RREG(softc->regs, IntStat.l);
	
	if(softc->ffield != -1)
	{
		n = (n & 0x100) >> 8;
		if(softc->ffield != n)
		{
			XV_WREG4(softc->regs, VCtl.l, 2);
		}
	}
	if(softc->visleep)
		cv_broadcast(&softc->viwake); /* wake up strategy */

}
static void
tvtwo_svc_w_chan(tvtwo_softc *softc)
{
printf("tvtwo_svc_w_chan: clearing interupts\n");
	XV_WREG(softc->regs, IntMask.l, 0x8000);
}

static void
tvtwo_svc_blit(tvtwo_softc *softc)
{
	DEBUGF((CE_CONT, "tvtwo_svc_blit%d\n",
		ddi_get_instance(softc->devi)));
}

static void
tvtwo_null(tvtwo_softc *softc)
{
	DEBUGF((CE_CONT, "tvtwo_null%d\n",
		ddi_get_instance(softc->devi)));
}

/*---------------------------------------------------------------------------*
 |"tvtwo_pll_alignment"
 |
 |  Check that the Power Video 700 frame buffer is properly synchronized
 |
 |    = 1 if pll in alignment, 0 if adjusting alignment
 *---------------------------------------------------------------------------*/
int tvtwo_pll_alignment( softc )
	tvtwo_softc*	softc;		/* Context for this device instance. */
{
	int         plx_v;      /* Video synchronization information. */
	int         vlines;     /* Vertical lines on screen. */

	plx_v = XV_RREG(softc->regs, Bt.blu.reg0);
	vlines = softc->nvram.main.ysize - 1;

	/*  FRAME BUFFER IS WAY OUT OF SYNCHRONIZATION WITH HOST */
	 
	if (0==(plx_v&0x80)) 
	{
		XV_WREG(softc->regs,VDisp.l, vlines-64);
		softc->pll_minor_adjust=0;
		softc->pll_good_count=0;
	} 


	/*  FRAME BUFFER IS MORE THAN 32 LINES OUT OF SYNCHRONIZATION WITH HOST */
	 
	else if (0==(plx_v&0x40)) 
	{
		XV_WREG(softc->regs,VDisp.l, vlines-8);
		softc->pll_good_count=0;
		softc->pll_minor_adjust=0;
	} 


	/*  FRAME BUFFER IS LESS THAN 32 LINES OUT OF SYNCHRONIZATION WITH HOST */

	else if (((plx_v)&0xf)>=2) 
	{
		if (softc->pll_minor_adjust&1)     
		{
			XV_WREG(softc->regs,VDisp.l, vlines-1);
		}
		else 
		{
			XV_WREG(softc->regs,VDisp.l, vlines);
		}
		softc->pll_minor_adjust++;

	} 


	/*  FRAME BUFFER IS IN SYNCHRONIZATION WITH HOST FRAME BUFFER */
	 
	else /* if ( softc->pll_good_count < 10 ) #### */
	{
		XV_WREG(softc->regs,VDisp.l, vlines);
		softc->pll_minor_adjust=0;
		softc->pll_good_count++;
	}

	return (softc->pll_good_count >= 7) ? 1 : 0;
}
