/***********************************************************************
 *  file: share.h
 *
 *
 *  ******************************************************************
 *  *  (c) Copyright Hewlett-Packard Company, 1990, 1991.	     *
 *  *  All rights are reserved.  Copying or other reproduction of    *
 *  *  this program except for archival purposes is prohibited       *
 *  *  without prior  written consent of Hewlett-Packard Company.    *
 *  ******************************************************************
 *
 *  Program purpose: include file common to the server and library for 
 *		     SharedX extensions
 *
 *		Hewlett Packard Ft. Collins Workstation Lab
 *		Project -- SharedX Extensions
 *		Dan Garfinkel -- MTS
 *
 */

#include <X11/Xmd.h>

#define SHARENAME 	"ShareExtension"

#define SX_SERVER_NAME		"SX_SERVER_NAME"
#define SX_WINDOW_ID		"SX_WINDOW_ID"
#define SX_DONT_SHARE		"SX_DONT_SHARE"
#define SX_SHARE_ALONE		"SX_SHARE_ALONE"
#define SX_DAISY_CHAIN_ALLOWED	"SX_DAISY_CHAIN_ALLOWED"
#define SX_ACCEPT_INPUT_FROM_ALL "SX_ACCEPT_INPUT_FROM_ALL"

#define ALL_WINDOWS	0

/*
 * defines for floor control
 */

#define sxDynamic	0
#define sxLocal		1
#define sxRemote	2

/*
 * these are the list of actions for the share command
 */

#define SHARE_CLIENT	0x00
#define SHARE_WINDOW	0x01
#define UNSHARE_WINDOW  0x02
#define UNSHARE_HOST	0x03
#define SET_FLOOR	0x04
#define SET_DELAY	0x05
#define SX_NOTIFY	0x06
#define ALLOW_INPUT	0x07
#define DISALLOW_INPUT	0x08
#define SX_VERSION	0x09
#define SX_CHECK_AUTH	0x0A

#define SX_SEND_FOCUS	0x0B

#define SX_DHA_CLIENT	0x20


/*
 * SharedX request
 */

typedef struct _Share {
    CARD8  reqType;         /* always ShareReqCode */
    CARD8  action;          /* the actual action to perform (see above) */
    CARD16 length;	    
    CARD32 server_id;	    /* Atom of the server_id */
    CARD32 window;	    /* window to share */
    CARD32 parent;	    /* parent at destination (0 == root) */
} xShareReq;

#define sz_xShareReq 16

/*
 * SharedX wire Event
 */

typedef struct {
    BYTE type;
    BYTE detail;
    CARD16 sequenceNumber B16;
    Window window B32;		/* the window effected */
    Atom host B32;		/* the host effected */
    Time time B32;		/* the time it happened */
    CARD32 tag B32;		/* window sharing tag */
    CARD32 extra B32;		/* extra information */

    CARD32 pad3 B32;		/* padding bytes 25-28 */
    CARD32 pad4 B32;		/* padding bytes 29-32 */
} sxEvent;

/*
 * SharedX Xlib Event
 */

typedef struct {
    int type;
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;    /* true if this came from a SendEvent request */
    Display *display;	/* Display the event was read from */
    Window window;	/* window on which event was requested in event mask */
    Atom host;		/* the host which this message pertains to */
    int message;	/* the message sent */
    int tag;		/* window instance tag */
    int extra;		/* extra information */
} SXEvent;

/*
 * the list of message number 
 */

#define CONNECTION_MADE		0
#define CONNECTION_REFUSED	1
#define CONNECTION_CLOSED	2
#define WINDOW_SHARED		3
#define WINDOW_UNSHARED		4
#define WINDOW_UNSHARABLE	5
#define BAD_FONT		6
#define INPUT_ALLOWED		7
#define INPUT_DISALLOWED	8
#define WINDOW_PROP_CHANGE	9
#define SERVER_TIMEOUT		10
#define BAD_COLOR		11
#define SX_IO_ERROR		12
#define BAD_GC_FUNCTION		13
#define SX_DHX_CLIENT		14
#define SX_COLORMAP_TOO_SMALL	15
#define SX_NO_COLORMAP		16

#define CONNECTION_REFUSED_LOCAL	100
#define SX_OUT_OF_MEM			101
#define SX_CANT_SHARE			102
#define SX_FOCUS_NOTIFY			103

/*
 * these are for the CRS client
 */

#define SX_DHA_WINDOW_SHARED		110
#define SX_DHA_WINDOW_UNSHARED		111

/*
 * add reasons for connection refused.  Will be returned in the "extra" field
 * of the sharedX event.
 */

#define SX_NO_REASON	 0	/* for backward compatability */
#define SX_NO_IP	 1	/* cannot convert name to IP address */
#define SX_NO_CONNECT	 2	/* cannot access the named machine */
#define SX_NO_XSERVER	 3	/* no X server running the machine */
#define SX_NO_PERMISSION 4	/* no host access to the X server */

/* 
 * NOTE: SX_OUT_OF_MEM is a vaild reason for connection refused.  
 * It's number is still 101. (from above)
 */
