/******************************************************************************
 *
 *			Hpddx Screen-Specific Include File
 *
 * (c) Copyright Hewlett-Packard Company, 1991-1994.
 * All rights are reserved.  Copying or other
 * reproduction of this program except for archival
 * purposes is prohibited without the prior 
 * written consent of Hewlett-Packard Company. 
 *
 * RESTRICTED RIGHTS LEGEND
 *
 * Use, duplication, or disclosure by the U.S. Government
 * is subject to restrictions as set forth in 
 * subdivision (c) (1) (ii) of the Rights in Technical
 * Data and Computer Software clause at 
 * 252.227-7013.
 *
 * HEWLETT-PACKARD COMPANY
 * Fort Collins, Colorado
 *
 * $Header:   /export/home/dist1/source/xv/X11R6/hp/archives/hpddxScreen.h_v   1.0   17 Sep 1996 06:57:12   dave  $
 *
 *	This file contains various screen-specific #include's, typedefs,
 *	macros and extern declarations for Hpddx (Hewlett-Packard's X server).
 *	All DDX drivers plugging into Xhp must #include this file.
 *
 *****************************************************************************/

#ifndef HPDDX_SCREEN_H
#define HPDDX_SCREEN_H

/*
 ******************************************************************************
 **
 ** Files that must be #include'd or else this file will not compile
 **
 ******************************************************************************
 */

#include <scrnintstr.h>
#include <miscstruct.h>
#include <windowstr.h>
#include <X11/extensions/hpext.h>

/*
 ******************************************************************************
 **
 ** Miscellaneous defines
 **
 ******************************************************************************
 */

#define	MAX_PATH_NAME	            256 /* device pathname - multiple of 4 */

#if OSMAJORVERSION >= 10
#define SCREENS_DIR   "/etc/X11/"
#else
#define SCREENS_DIR   "/usr/lib/X11/"
#endif

/*
 ******************************************************************************
 **
 ** Basic Storage Types
 **
 ******************************************************************************
 */

#ifdef USE_TYPEDEFS

typedef unsigned char  Card8;
typedef unsigned short Card16;
typedef unsigned int   Card32;
typedef char           Int8;

/* Used for bit fields in structs and unions: */
typedef unsigned short HpddxBitField;

#else /* USE_TYPEDEFS */

#ifndef Int8
#define Int8           char
#endif

#ifndef Card8
#define Card8          unsigned char
#endif

#ifndef Int16
#define Int16          short
#endif

#ifndef Card16
#define Card16         unsigned short
#endif

#ifndef Int32
#define Int32          int
#endif

#ifndef Card32
#define Card32         unsigned int
#endif

/* Used for bit fields in structs and unions: */
#ifndef HpddxBitField
#define HpddxBitField  unsigned short
#endif

#endif /* USE_TYPEDEFS */

/* The pHpddxNilRegion variable is used throughout some of our smart
 * clipping code segments.  It is not necessary for third parties to
 * use this variable.  It is provided for convience.  We initialize
 * the pHpddxNilRegion variable for all drivers during the
 * FirstTimeInitialization procedure.
 */
extern RegionPtr	pHpddxNilRegion;

/*
 ******************************************************************************
 **
 ** These are constants representing the legal tokens that can be expected in
 ** an X*screens file.  These constants are returned by getToken() if a
 ** reserved word is found.
 **
 ******************************************************************************
 */

#define SERVER_OPTIONS               1
#define GRAPHICS_SHMEM_SIZE          2
#define PRIMITIVE_THRESHOLD          3
#define MONITOR_POWER_SAVE_LEVEL     4
#define SCREEN                       5
#define DEFAULT_VISUAL               6
#define CLASS                        7
#define CLASS_STATICGRAY             8
#define CLASS_GRAYSCALE              9
#define CLASS_STATICCOLOR           10
#define CLASS_PSEUDOCOLOR           11
#define CLASS_TRUECOLOR             12
#define CLASS_DIRECTCOLOR           13
#define DEPTH                       14
#define LAYER                       15
#define LAYER_OVERLAY               16
#define LAYER_IMAGE                 17
#define TRANSPARENT_VISUAL          18
#define MONITOR_SIZE                19
#define INCHES                      20
#define MM                          21
#define SCREEN_OPTIONS              22
#define SINGLE_LOGICAL_SCREEN       23 /* reserved for future use */

/*
 ******************************************************************************
 **
 ** Hpddx Screen Private Index (used to index into pScreen->devPrivates)
 **
 ******************************************************************************
 */

extern Int32 hpddxScreenPrivIndex; /* globals.c */

/* This structure allows us to attach version numbers to our
 * internal hpddx structures.
 */

/* This structure definition is changing from R5 to R6
 * that is 9.* to 10.* repectively.
 */
#ifdef X11R6
typedef struct _HpddxVersionRec
{
    Card16  majorNumber;
    Card16  minorNumber;
} HpddxVersionRec, *HpddxVersionPtr;

#else

typedef struct _HpddxVersionRec
{
    Card8  majorNumber;
    Card8  minorNumber;
    Card16  patchNumber;
} HpddxVersionRec, *HpddxVersionPtr;
#endif

/* This structure is used to build token lists when parsing an X*screens file.
 */

typedef struct _TokenListItemRec
{
    char                     *name;
    Int16                    id;
    Int16                    lineNumber;
    Int16                    reserved;
    struct _TokenListItemRec *previous;
    struct _TokenListItemRec *next;
} TokenListItemRec, *TokenListItemPtr;

/*
 ******************************************************************************
 **
 ** Hpddx Screen Private Structure
 **
 ** The hpddxScreenPrivRec structure is a generic screen private structure
 ** which deals with HP-specific interfaces between DDX and all other layers
 ** (that is, OS, Input, DIX, Extensions, GRM).  It will be necessary for third
 ** parties writing drivers for HP platforms to properly initialize this
 ** structure.
 **
 ******************************************************************************
 */

typedef struct _HpddxScreenPrivRec
{
    HpddxVersionRec version;

    ScreenPtr pScreen;

    /**** Hpddx DDX DHA Interface ****/

    Int32 (*RegisterWindow)(
#ifdef _PROTOTYPES
       DrawablePtr            pDrawable,
       xHPRegisterWindowReply *rep,
       ClientPtr              client
#endif
    );

    Int32 (*UnRegisterWindow)(
#ifdef _PROTOTYPES
        DrawablePtr            pDrawable,
        xHPRegisterWindowReply *rep,
        ClientPtr              client
#endif
    );

    Bool (*IsWindowDHA)(
#ifdef _PROTOTYPES
        WindowPtr pWin
#endif
    );

    /* Allow driver specific clip list extension processing */

    Int32 (*HPGetClipLists)(
#ifdef _PROTOTYPES
        DrawablePtr pDrawable,
        GCPtr       pGC,
        Int32       nClipLists,
        Int32       *pFormats,
        Int32       *x,
        Int32       *y,
        Int32       *width,
        Int32       *height,
        Int32       *pNReplyBytes,
        pointer     *pReply
#endif
    );

    Int32(*HPFreeClipLists)(
#ifdef _PROTOTYPES
        pointer *pClipList
#endif
    );

    /* SbToDdxMsg allows DDX & CORE to send device/driver specific messages. */

    Int32 (*SbToDdxMsg)(
#ifdef _PROTOTYPES
        WindowPtr pWindow,
        Int32     nMsgBytes,
        Int32     *pMsg,
        Int32     needReply,
        Int32     *nReplyBytes,
        pointer   *pReply
#endif
    );

    /**** Hpddx DDX Grab Server Interface ****/

    /* The following two grab procedures are called from within
     * hpddx[UnG G]rabServer which in turn are called from
     * Proc[UnG G]rabServer.  The GrabScreen procedure will typically
     * acquire the lock for the current screen and set the
     * isScreenGrabbed boolean to true.  The UnGrabScreen call will
     * release the lock for the screen and set isScreenGrabbed to
     * false.
     */

    void (*GrabScreen)(
#ifdef _PROTOTYPES
        ScreenPtr pScreen
#endif
    );

    void (*UnGrabScreen)(
#ifdef _PROTOTYPES
        ScreenPtr pScreen
#endif
    );

    /* The isScreenGrabbed flag (assigned within the previous GrabScreen
     * and UnGrabScreen procedures) is used by the DDX drivers to identify
     * which screen is currently locked (aka grabbed).  If a grab is in
     * progress then we need to ensure that we don't prematurely break the
     * grab by unlocking - this flag is used to accomplish this.
     */

    Bool isScreenGrabbed;

    /* The ServerMode field (used by SProcHPGetServerMode and
     * ProcHPGetServerMode) is for identifying the display mode of the
     * Server for clients on machines with a different byte ordering.
     * Most drivers assign COMBINED_MODE (defined within XHPproto.h as
     * 0x4) to this field.  Those not supporting combined mode assign 
     * IMAGE_MODE (0x2).  Other values include OVERLAY_MODE (0x1) and
     * the obsolete STACKED_MODE (0x3).
     */

    Int32 ServerMode;

    /**** Hpddx DDX Cursor/Sprite Interface ****/

    void (*SetCursorID)(
#ifdef _PROTOTYPES
        WindowPtr pWin,
        Cursor    cursorID
#endif
    );

    Cursor (*GetCursorID)(
#ifdef _PROTOTYPES
        WindowPtr pWin
#endif
    );

    void (*ChangeScreen)(
#ifdef _PROTOTYPES
        ScreenPtr pScreen
#endif
    );

    void (*CursorOff)(
#ifdef _PROTOTYPES
        ScreenPtr pScreen
#endif
    );

    void (*RemoveWinFromSaveUnder)(
#ifdef _PROTOTYPES
        WindowPtr pWin
#endif
    );

    void (*TurnOffWinSaveUnder)(
#ifdef _PROTOTYPES
        WindowPtr pWin
#endif
    );


    /* usingFriendlyBoot is a Boolean used to identify if the server should be
     * started in Friendly Boot mode.  Friendly Boot is only true if the server
     * is started by VUE (the vuelogin process).  What is the server doing for
     * the Friendly Boot process:
     *
     * - The root weave will not be painted.  This is good.  When VUE starts up
     *   the server there is a short time (<2sec) that the root weave shows up
     *   (as a flash) before the vuelogin process kicks off the server.  We
     *   will get rid of the flash.
     *
     * - The first root window clear from VUE will not be processed, for the
     *   same reasons in part 1.
     *
     * - The first implicit installation of the default colormap will only
     *   install the first 16 colormap entries.  This will keep the Friendly
     *   Boot slate (that looks just like the VUE login slate) from going
     *   technicolor.
     */

    Bool usingFriendlyBoot;

    /* This field is used to determine what Monitor Power Save level the user
     * is requesting.  The field is set based on a MonitorPowerSaveLevel entry
     * in the X*screens file.  The meaning of the field is as follows:
     *
     *    0 - Regular Blank or Unblank (no Power Save)
     *    1 - Stand-by State           (remove horizontal sync)
     *    2 - Suspend State            (remove vertical sync)
     *    3 - Off State	               (remove both syncs)
     */

     Card32 monitorPowerSaveLevel;

    /* The following three fields (ReadColorMap, HPGetClipList &
     * HPFreeClipList) are being obsoleted.  They are still required
     * for backwards compatibility.  New drivers and 3rd parties should
     * assign a noop to these procedure pointers.
     */
    
    /* This pointer is to allow XHPSyncronizeColorRange to read the hardware
     * colormap.  3rd parties should assign a noop to this procedure pointer.
     * This field is being obsoleted and needs to be removed (when 9.* 
     * applications are obsolete and no longer supported).  This function
     * pointer is called from ProcHPSynchronizeColorRange(client) within
     * extensions/server/hp/hpsyncolor.c
     */

    void (*ReadColorMap)(
#ifdef _PROTOTYPES
        pointer pDisplayState,
        Int32   start,
        Int32   ncolors,
        Int16   *values
#endif
    );

    /* Allow driver specific clip list extension processing */

    Int32 (*HPGetClipList)();
    Int32 (*HPFreeClipList)();

    Int32 pad[64];

    /**** Hpddx DDX PolyPrimitive Extension Interface ****/

    /*### NOT IMPLEMENTED YET */ 

} HpddxScreenPrivRec, *HpddxScreenPrivPtr;

/*
 ******************************************************************************
 **
 ** Initialization structure for hpxInit.c.  This structure lives during
 ** InitOutput().  We parse all the tokens into a doubly linked list then let
 ** the *ScreenInit procedures process them.
 **
 ******************************************************************************
 */

typedef struct _HpddxScreenInitRec
{
    /* Version flag for structure.  Currently this field is unused, we're
     * reserving space for the future.
     */

    HpddxVersionRec version;

    /* Full pathname for device, e.g., /dev/crt.  This field is required by the
     * *ScreenInit procedures as well as InitOutput and the hpx[Bidder Loader]
     * procedures.  For consistency sake, this name will be duplicated in the
     * tokenList.
     */

    char deviceName[MAX_PATH_NAME];

    /* Screen entry tokens parsed from X*screens file */
    TokenListItemPtr screenEntryTokenList;

    /* Screen options tokens parsed from X*screens file */
    TokenListItemPtr screenOptionsTokenList;

    /* Note: If type of monitorDiagonalSize changes, also need to change
     * MAX_MONITOR_DIAGONAL_SIZE.
     */

    Card16 monitorDiagonalSize;
    Card8  monitorDiagonalUnits;
    Card8  monitorPowerSaveLevel;

    /* Note: If type of defaultVisualDepth changes, also need to change
     * MAX_DEFAULT_VISUAL_DEPTH.
     */
    Card8 defaultVisualDepth;
    Card8 defaultVisualClass;
    Card8 defaultVisualLayer;
    Card8 defaultVisualTransparent;

    Int32 defaultDeviceID;

} HpddxScreenInitRec, *HpddxScreenInitPtr;

/*
 ******************************************************************************
 **
 ** Obsolete structures required for backwards compatibility.
 ** These structures are not to be used by new drivers.
 **
 ******************************************************************************
 */

/*### There are serveral DDX coding standard violations below.
 *### They were necessary to keep from editing DIX code.
 *### Perhaps we can address modifying DIX code later.
 *### "DisplayState" should be "displayState".
 *### "hpddxColormapRec" should be "HpddxColormapRec".
 *### "hpddxColormapPtr" should be "HpddxColormapPtr".
 */

typedef struct _HpddxColormapRec
{
    pointer DisplayState;
    Int32   lastIndex;
} hpddxColormapRec, *hpddxColormapPtr;

/*
 ******************************************************************************
 **
 ** Macros to Access the Hpddx Screen Private Structure
 **
 ******************************************************************************
 */

#define HPDDX_SCREEN_PRIV(pScreen) \
    ((HpddxScreenPrivPtr)((pScreen)->devPrivates[hpddxScreenPrivIndex].ptr))

#define HPDDX_SCREEN_PRIV_FROM_DRAWABLE(pDrawable)\
    HPDDX_SCREEN_PRIV((pDrawable)->pScreen)

#define HPDDX_SCREEN_PRIV_FROM_WINDOW(pWindow)\
    HPDDX_SCREEN_PRIV((pWindow)->drawable.pScreen)

#define HPDDX_SCREEN_PRIV_FROM_PIXMAP(pPixmap)\
    HPDDX_SCREEN_PRIV((pPixmap)->drawable.pScreen)

#define HPDDX_SCREEN_PRIV_FROM_GC(pGC)\
    HPDDX_SCREEN_PRIV((pGC)->pScreen)

#endif /* HPDDX_SCREEN_H */

/*
 ******************************************************************************
 **
 ** Function Prototypes
 **
 ******************************************************************************
 */

char *getTokenValueStr(
#ifdef _PROTOTYPES
   TokenListItemPtr pTokenList,
   char             *key,
   Bool             pair
#endif
);


/*
 ******************************************************************************
 **
 ** Macro to support 3rd party GLX
 **
 ******************************************************************************
 */

#define GLX_3RD_PARTY_VENDOR	"OpenGL-3P"
