/* @(#)57   1.3  src/UMS/usr/lpp/XV/include/Xvddx.h, Xv, ums212 3/9/95 12:01:40
*/
/*
 * COMPONENT_NAME: xv
 *
 * FUNCTIONS: include file
 *
 * ORIGINS: 16, 145, 27
 *
 * (C) COPYRIGHT International Business Machines Corp. 1995
 * All Rights Reserved
 * Licensed Materials - Property of IBM
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 */
#ifndef Xvddx_h_INCLUDED
#define Xvddx_h_INCLUDED
/*
** File:
**
**   Xvddx.h --- Client accessable structures and definitions associated
**               with Xv video extension device-dependent layer.
**
** Author:
** 
**   Andy Goodrich (Parallax)
**
** Revisions:
**
**   01 Jan 94 Goodrich
**     - original coding
**
** $Log:   /plxhp/dave/pvcsrc/xv/include/archives/Xvddx.h_v  $
 * 
 *    Rev 1.2   09 Sep 1996 10:37:42   dave
 * Andy Goodrich: Combined HP/UX and AIX platform versions.
** Revision 1.2  1995/08/22 13:31:01  dist
** Andy Goodrich: Combined HP/UX and AIX platform versions.
**
** Revision 1.1  95/05/02  11:55:15  11:55:15  dist ()
** Initial revision under source control
** 
*/

/*
 *  STANDARD DEVICE-DEPENDENT ATTRIBUTES:
 *
 *  Long attributes:
 *     1) "XV_ATTRIBUTES" - returns list of atoms of supported attributes.
 *                          THIS ATTRIBUTE IS REQUIRED FOR ALL IMPLEMENTATIONS.
 *     2) "XV_CONTROLS" - returns list of atoms of device dependent controls.
 * 
 *
 *  Integer attributes:
 *     1) "XV_BRIGHTNESS" - get or set brightness level (-1000..1000).
 *     2) "XV_CONTRAST" - get or set contrast level (-1000..1000).
 *     3) "XV_HUE" - get or set hue level (-1000..1000).
 *     4) "XV_SATURATION" - get or set saturation level (-1000..1000).
 *     5) "XV_STATUS" - get status of this port (see enum below).
 *
 *  Control operations:
 *     None
 */

/*  XV PORT ENCODING SPECIFICATIONS:
 *
 *  For the comments below assume a byte ordering where byte 0 is
 *  the most significant byte and byte 3 is the least significant.
 *
 */
enum { /*                vv           Miscellaneous information (byte 0) : */
    XV_GRAY_SCALE =    0x01000000,
};

enum { /*                  vv         Source timing (byte 1):  */
    XV_SOURCE_TIMING = 0x00FF0000,
    XV_NTSC =          0x00010000,
    XV_PAL  =          0x00020000,
    XV_SECAM =         0x00030000,
	XV_SOFTWARE =      0x00040000,
};

enum { /*                    vv       Source type (byte 2):  */
    XV_SOURCE_TYPE =   0x0000FF00,
    XV_COMPOSITE =     0x00000100,
    XV_RGB =           0x00000200,
    XV_YC =            0x00000300,
    XV_YUV =           0x00000400,
	XV_SYSTEM =        0x00000500,
};

enum { /*                      vv     Source number by type (0..n) (byte 3): */
    XV_SOURCE_NUMBER = 0x000000FF,
    XV_SOURCE_CONNECTION = (XV_SOURCE_TYPE | XV_SOURCE_NUMBER),
};


/* VALUES FOR XV_STATUS PORT ATTRIBUTE:
 *
 *  Low order 16 bits are reserved for generic XV information. 
 *  High-order 16 bits are for vendor device-specific information.
 */
enum {              
    XVS_Attached = 0x00000001,   /* Port is attached to a drawable. */
    XVS_Sync =     0x00000002,   /* Port's input has an active sync. */
    XVS_Input =    0x00000004,   /* Port is capable of PutVideo. */
    XVS_Output =   0x00000008,   /* Port is capable of GetVideo. */ 
    XVS_Display =  0x00000010,   /* Port's PutVideo can be visible. */
    XVS_Vendor =   0x00010000	 /* First vendor device-specific bit. */
};

#endif /* Xvddx_h_INCLUDED */
