/*
 *
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 */
/*
** File: 
**
**  Xvddx_PGI.h: Client accessable definitions for Xv video extension
**               DD level associated with Parallax adaptors.
**
** Author: 
**
**   Andy Goodrich (Parallax)
**
** Revisions:
**
**   20.01.94 Goodrich
**     - Initial coding.
** $Log:   /plxhp/dave/pvcsrc/xv/include/archives/Xvddx_PGI.h_v  $
 * 
 *    Rev 1.1   09 Sep 1996 10:37:52   dave
 * Initial revision
** Revision 1.1  1995/05/02 11:55:24  dist
** Initial revision
**
*/
#ifndef Xvddx_PGI_h_INCLUDED
#define Xvddx_PGI_h_INCLUDED
#include <sys/types.h>
#include "Xvddx.h"		/* Pick up generic DD level definitions. */

/*
 *  DEVICE-DEPENDENT ATTRIBUTES SUPPORTED:
 *
 *  Long attributes:
 *     1) "XV_ATTRIBUTES"" - return list of attribute atoms supported.
 *     2) "XV_CONTROLS"" - return list of control atoms supported.
 *     3) "XV_PLX_NVRAM_VALUES" - get or set nvram values.
 *     4) "XV_PLX_QUERY_SIGNALS" - return parallax signal info on each encoding.
 *     5) "XV_PLX_TABLE_COMPRESSION" - access compression q table information.
 *     6) "XV_PLX_TABLE_DECOMPRESSION" - access decompress. q table information.
 *
 *  Integer attributes:
 *     1) "XV_BRIGHTNESS" - get or set brightness level (-1000..1000).
 *     2) "XV_CONTRAST" - get or set contrast level (-1000..1000).
 *     3) "XV_ENCODING" - get or set current signal encoding (see Xvddx.h).
 *     4) "XV_HUE" - get or set hue level (-1000..1000).
 *     5) "XV_PLX_CCUBE" - return number of ccube chips on device.
 *     6) "XV_PLX_CHANNEL" - set digitizer channel affinity of a port (0, 1, 2).
 *     7) "XV_PLX_CROP" - set cropping or scaling of video (1 = crop 0 = scale).
 *     8) "XV_PLX_NVRAM_CONTROL" - perform an nvram operation (see enum below.)
 *     9) "XV_PLX_NVRAM_VALUES" - get or set nvram values.
 *    10) "XV_PLX_RASTEROPS" - recognize old RasterOps signal encodings(1 or 0).
 *    11) "XV_PLX_TEST" - test atom(semantics change depending on what you test)
 *    12) "XV_SATURATION" - get or set saturation level (-1000..1000).
 *    13) "XV_STATUS" - get status of this port (see Xvddx.h for status values).
 *    14) "XV_ZOOM" - get or set screen zoom factor (1, 2, or 4).
 *
 *  Control operations:
 *     1) "XV_PLX_GETCIMAGE" - get a compressed image
 *     2) "XV_PLX_FIRST_PUTCIMAGE" - put first of a set of compressed images
 *     3) "XV_PLX_PUTCIMAGE" - put subsequent compressed image
 */

/*
 *  SIGNAL INFORMATION RECORD:
 */
typedef struct _XvplxSignalRec {
    unsigned int encoding_id;	/* Encoding id for this configuration. */
    unsigned int channel;   	/* Input or output channel no. */
    unsigned int type;      	/* = 0: input channel, = 1: output channel. */
    unsigned int standard;		/* Timing standard (e.g., PLX_NTSC). */
    unsigned int format;		/* Format (e.g., PLX_COMP). */
    unsigned int width;
    unsigned int height;
    unsigned int blank;
    unsigned int rate;
    unsigned int instance;
    unsigned int sync_ok;
} XvplxSignalRec;


/*
 *  ADDITIONAL ENCODING TYPES
 */
enum { /*            vv        */
    XV_ZOOM_4  = 0x00800000,
    XV_ZOOM_2  = 0x00400000,
	XV_NTSC_2X = XV_NTSC | XV_ZOOM_2,
	XV_NTSC_4X = XV_NTSC | XV_ZOOM_4,
	XV_PAL_2X = XV_PAL | XV_ZOOM_2,
	XV_PAL_4X = XV_PAL | XV_ZOOM_4,
	XV_SECAM_2X = XV_SECAM | XV_ZOOM_2,
	XV_SECAM_4X = XV_SECAM | XV_ZOOM_4,
};

/*
 *  SUBFIELDS IN ENCODING PORT ATTRIBUTE:
 */
enum {
	XV_PLX_DMSD =  0x000000f0, /* DMSD # portion of XV_SOURCE_NUMBER field. */
	XV_PLX_INPUT = 0x0000000f  /* Input # portion of XV_SOURCE_NUMBER field. */
};

/* ADDITIONAL VALUES FOR XV_STATUS PORT ATTRIBUTE:
 */
enum {
	XVS_Plx_Ccube = 0x10000 /* There is a ccube in this system. */
};

/* NVRAM CONTROL OPERATIONS (ON XV_PLX_NVRAM_CONTROL PORT ATTRIBUTE):
 */
enum {
	XV_PLX_NVRAM_READ      = 0,	/* Read in contents of nvram. */
	XV_PLX_NVRAM_READ_INIT = 1,	/* Read in nvram contents and init from them. */
	XV_PLX_NVRAM_INIT_DISP = 2, /* Initialize horizontal from nvram values. */
	XV_PLX_NVRAM_INIT_FROM = 3, /* Initialize hardware from nvram values. */
	XV_PLX_NVRAM_WRITE     = 9	/* Write out to nvram. */
};

#endif /* Xvddx_PGI_h_INCLUDED */

