/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *
 *			 All Rights Reserved
 *
 *  Permission to use, copy, modify, and distribute this software and its 
 *  documentation for any purpose and without fee is hereby granted, 
 *  provided that the above copyright notice appear in all copies and that
 *  both that copyright notice and this permission notice appear in 
 *  supporting documentation, and that the names of Parallax or MIT not be
 *  used in advertising or publicity pertaining to distribution of the
 *  software without specific, written prior permission.  
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 */
/*
** File:
**
**  xvddx_dummy.c: Hardware specific DD layer routines for the dummy
**                  adaptor to test the rest of XV without hardware.
**
** Author:
**
**   Andy Goodrich (Parallax)
**
** Revisions:
**
**   12 Jan 95 Goodrich
**     - initial coding.
*/
#include <sys/types.h>  /* Pick up u_int and friends. */
#include <stdio.h>              /* For diagnostics and error messages. */
 
/*  XV_DIAG bit definitions:
 *
 *    0x01 - normal one-shot diagnostics
 *    0x08 - messages from this adaptor about routine execution
 *    0x40 - request tracing
 *    0x80 - server initialization tracing
 */
#define MSG 0x08
#define XV_DIAG (MSG)
#include "xvdebug.h" 
  
#include <sys/fcntl.h>      /* Pick up O_RDWR and friends. */
#include "X.h"
#include "Xv.h"
#include "Xproto.h"
#include "scrnintstr.h"
#include "validate.h"
#include "windowstr.h"
#include "pixmapstr.h"
#include "gcstruct.h"
#include "dixstruct.h"
#include "xvdix.h"      	 /* Pick up device-independent layer. */
#include "xvddx.h"      	 /* Pick up device-dependent layer: XvddConfigPtr.*/
#include "Xvddx.h" 			 /* Pick up XV_xxx encoding declarations. */
#include "xvhw_Dummy.h"		 /* Pick up our hardware object definition. */

/*
 *  ENCODING AND SIGNAL INFORMATION:
 */
static XvEncodingRec DummyBaseEncodings[] =
{
    {XV_COMPOSITE | XV_NTSC, (ScreenPtr)NULL,
       "ntsc-composite", -10, 650, -10, 490, 640, 480, 5994, 100},
    {XV_COMPOSITE | XV_PAL, (ScreenPtr)NULL,
       "pal-composite", -10, 778, -10, 582, 768, 576, 50, 1},
    {XV_COMPOSITE | XV_SECAM, (ScreenPtr)NULL,
       "secam-composite", -10, 778, -10, 582, 768, 576, 50, 1},
    {XV_COMPOSITE | XV_NTSC | 1, (ScreenPtr)NULL,
       "ntsc-composite", -10, 650, -10, 490, 640, 480, 5994, 100},
    {XV_COMPOSITE | XV_PAL | 1, (ScreenPtr)NULL,
       "pal-composite", -10, 778, -10, 582, 768, 576, 50, 1},
    {XV_COMPOSITE | XV_SECAM | 1, (ScreenPtr)NULL,
       "secam-composite", -10, 778, -10, 582, 768, 576, 50, 1},
};
#define nDummyBaseEncodings \
        (sizeof(DummyBaseEncodings)/sizeof(DummyBaseEncodings[0]))


static struct _Attributes {

	/* GENERIC PORT CONTROLS: */

	Atom DummyAttributes;
	Atom DummyBrightness;
	Atom DummyChannel;
	Atom DummyContrast;
	Atom DummyControls;
	Atom DummyEncoding;
	Atom DummyHue;
	Atom DummySaturation;
	Atom DummyStatus;
 
	/* DEVICE-DEPENDENT PORT CONTROLS: */
   
	Atom XvropsTest;
} attr_atoms;

static int l_attr_atoms = sizeof(attr_atoms);

/* 
 *  FORWARD REFERENCES:
 */
static int		DummyControl();
static int		DummyGetLongPortAttribute();
static int		DummyGetPortAttribute();
static int		DummySetLongPortAttribute();
static int		DummySetPortAttribute();


/*----------------------------------------------------------------------------*
 |"DummyAdaptorInit"
 |
 |  Routine to allocate and initialize the Xv adaptor record associated with
 |  a Parallax PowerVideo 700 card.
 |
 |    -> initialized adaptor record on success
 |    =  0 on failure
 *----------------------------------------------------------------------------*/
XvAdaptorPtr DummyAdaptorInit(pScreen, device)
    ScreenPtr pScreen;			/* -> screen this adaptor is connected to. */
	char*	  device;			/* =  device name of this adaptor. */
{
    int            base_id;     /* Base port id for Adaptor. */
    int            id;          /* Client id for calling AddResource. */
    int            iEncode;     /* # of encodings processed so far. */
    int            iPort;       /* # of port records processed so far. */
	pointer		   mapping;		/* Mapping for this adaptor. */
    int            nPorts;      /* # of ports supported by this adaptor. */
    XvAdaptorPtr   pAdaptor;    /* -> adapator record initializing. */
    XvddPortPtr    pddPort;     /* -> private port rec(pPort->devPriv.ptr). */
    XvEncodingPtr  pEncode;     /* -> encoding record accessing. */
	HwDummy*       phw;     	/* Hardware object for this port. */
    XvPortPtr      pPort;      	/* -> xv port record accessing (pPorts[i]). */
    XvPortPtr      pPorts;     	/* -> storage allocated for xv port records. */
    Xvhw_Signal    signal;      /* Signal timing information. */
	extern GCFuncs XvddGCFuncs;	/* XV's GC wrapper functions. */

    DEBUGF(0x80, (fd, "DummyAdaptorInit: entry\n"));


    /*  MAP IN THE DEVICE WE WERE PASSED. IF WE CAN'T RETURN FAILURE. */
     
    if ( !(mapping = (pointer)xv_dummy_map(device)) )
    {
        return 0;
    }
	DEBUGF(0x80,(fd, "DummyAdaptorInit: after mapping board\n"));
 

	/*  DEVICE MAPPED, SO ALLOCATE AN ADAPTOR RECORD AND FILL IN DD STUFF. */

	DEBUGF(0x80, (fd, "DummyAdaptorInit: before generic initialization\n"));
	nPorts = 3;
	pAdaptor = (XvAdaptorPtr)XvddAdaptorInit(pScreen, nPorts, 1);
	if ( !pAdaptor )
	{
		ERRORF((fd, "Unable to allocate adaptor record\n"));
		xfree((char*)mapping);
		return 0;
	}
	DEBUGF(0x80, (fd, "DummyAdaptorInit: after generic initialization\n"));
	 
	pAdaptor->name = "Dummy";
    pAdaptor->ddControl              = DummyControl;
    pAdaptor->ddGetLongPortAttribute = DummyGetLongPortAttribute;
    pAdaptor->ddGetPortAttribute     = DummyGetPortAttribute;
    pAdaptor->ddSetLongPortAttribute = DummySetLongPortAttribute;
    pAdaptor->ddSetPortAttribute     = DummySetPortAttribute;


	/*  INTIALIZE PORT RECORDS ASSOCIATED WITH THIS ADAPTOR. */

	pPort = pAdaptor->pPorts;
	for ( iPort = 0; iPort < nPorts; iPort++, pPort++ )
	{
		DEBUGF(0x80,(fd, "DummyAdaptorInit: intializing port %d\n", iPort));


		/*  ALLOCATE HARWARE LEVEL STRUCTURES FOR THIS PORT. */
 
		if ( !(phw = (HwDummy*)xv_dummy_open(iPort, mapping)) )
		{
			xfree(mapping);
			return 0;
		}
		pddPort = (XvddPortPtr)pPort->devPriv.ptr;
		pddPort->phw = (pointer)phw;
		DEBUGF(0x80, (fd, "DummyAdaptorInit: after call to dummy_open\n"));
 
 
		/*  SET SIGNAL INFORMATION AND CALL HARDWARE INITIALIZATION ROUTINE. */
	 
		Xvhw_Get_Signal_Type(phw, &signal);
		signal.type = XV_COMPOSITE;
		signal.timing = XV_NTSC;
		Xvhw_Initialize(phw, &signal);
 
 
		/*  PERFORM SPECIAL PROCESSING BASED ON PORT NUMBER. */
 
		DEBUGF(0x80, (fd, "DummyAdaptorInit: before special processing\n"));
		switch ( iPort )
		{
		  case 0:
			XvImageInit();
			Xvhw_Fill_Video_Enable(phw,
				0, 0, Xvhw_Max_Width(phw), Xvhw_Max_Height(phw), 0);
			break;
		}


		/*  SET SIGNAL ENCODING INFORMATION. */
 
		DEBUGF(0x80, (fd, "DummyAdaptorInit: before setting encodings\n"));
		pPort->nEncodings = nDummyBaseEncodings;
		pPort->pEncodings = DummyBaseEncodings;
		pPort->type = XvInputMask;
		pPort->pEncoding = pPort->pEncodings;
		pEncode = pPort->pEncodings;
		for (iEncode = 0; iEncode < pPort->nEncodings; iEncode++)
		{
			pEncode->pScreen = pScreen;
			pEncode++;
		}
 
		DEBUGF(0x80, (fd, "DummyAdaptorInit: after encoding screens\n"));

	}




    /*  ALLOCATE THIS ADAPTOR'S ATOMS FOR PORT ATTRIBUTES --- MORE LATER */
     
    attr_atoms.DummyAttributes      = MakeAtom("XV_ATTRIBUTES", 
                                    strlen("XV_ATTRIBUTES"), 
                                    xTrue);
    attr_atoms.DummyBrightness      = MakeAtom("XV_BRIGHTNESS", 
                                    strlen("XV_BRIGHTNESS"), 
                                    xTrue);
    attr_atoms.DummyContrast        = MakeAtom("XV_CONTRAST", 
                                    strlen("XV_CONTRAST"), 
                                    xTrue);
    attr_atoms.DummyControls        = MakeAtom("XV_CONTROLS", 
                                    strlen("XV_CONTROLS"), 
                                    xTrue);
    attr_atoms.DummyEncoding        = MakeAtom("XV_ENCODING", 
                                    strlen("XV_ENCODING"), 
                                    xTrue);
    attr_atoms.DummyHue             = MakeAtom("XV_HUE", 
                                    strlen("XV_HUE"), 
                                    xTrue);
    attr_atoms.DummySaturation      = MakeAtom("XV_SATURATION", 
                                    strlen("XV_SATURATION"), 
                                    xTrue);
    attr_atoms.DummyStatus          = MakeAtom("XV_STATUS", 
                                    strlen("XV_STATUS"), 
                                    xTrue);
	xfree(mapping);
    DEBUGF(0x80, (fd, "DummyAdaptorInit: exit\n"));
    return pAdaptor;
}


/*---------------------------------------------------------------------------*
 |"DummyControl"
 |
 |  Transfer vector routine to perform an arbitrary device dependent
 |  operation. In addition to the normal request header there may be
 |  ancilliary data to qualify the request. This is supplied by the 
 |  pQualify pointer. This routine is responsible for generating the 
 |  X Reply for this request on success.
 |
 |    = 0 on success.
 |    = error on failure.
 *---------------------------------------------------------------------------*/
static int DummyControl(client, pPort, operation, length, pQualify)
    ClientPtr client;		/* -> client request data. */
    XvPortPtr pPort;		/* -> port request is on. */
    Atom      operation;	/* =  operation to be performed. */
	int		  length;		/* =  length of qualifying data. */
    char      *pQualify;	/* -> additional data for qualifying request. */
{
	int			i_value;	/* Entry in 'p_value' accessing. */
	char*		p_value;	/* 'value' aliased as bytes. */

    DEBUGF(MSG, 
		(fd,"DummyControl: port: %08x operation %08x length %d:",
		pPort, operation, length));
	for ( p_value = (char*)pQualify, i_value = 0; i_value < length; i_value++ )
	{
		if ( i_value % 4 == 0 ) DEBUGF(MSG, (fd, " "));
		if ( i_value % 32 == 0 ) DEBUGF(MSG, (fd, "\n    "));
		DEBUGF(MSG, (fd, "%02x", p_value[i_value]));
	}
	DEBUGF(MSG, (fd, "\n"));

	return Success;
}


/*---------------------------------------------------------------------------*
 |"DummyGetLongPortAttribute"
 |
 |  Transfer vector routine to get a long attribute's value.
 |
 |    = 0 on success.
 |    = error on failure.
 *---------------------------------------------------------------------------*/
static int DummyGetLongPortAttribute(client, pPort, attribute)
    ClientPtr client;		/* -> client request data. */
    XvPortPtr pPort;		/* -> port request is on. */
    Atom      attribute;	/* =  attribute requested. */
{
	int			board_code;	/* Board code. */
	int			n_nvram;	/* Size of nvram area. */
	XvddPortPtr pddPort;	/* Device-dependent storage for pPort. */
	int*		p_nvram;	/* Pointer to nvram area. */

	DEBUGF(MSG, (fd, "DummyGetLongPortAttribute: port = %d atom = %08x\n", 
		pPort, attribute));

	if (attribute == attr_atoms.DummyAttributes)
	{
		SendXvGetLongPortAttributeReply(client, l_attr_atoms, &attr_atoms);
	}
	else
	{
		SendXvGetLongPortAttributeReply(client, 0, &attr_atoms);
	}

	return Success;
}

/*----------------------------------------------------------------------------*
 |"DummyGetPortAttribute"
 |
 |  Transfer vector routine to get an attribute's value.
 *----------------------------------------------------------------------------*/
static int DummyGetPortAttribute(client, pPort, attribute, p_value)
    ClientPtr client;		/* -> client request data. */
    XvPortPtr pPort;		/* -> port request is on. */
    Atom      attribute;	/* =  attribute requested. */
    INT32     *p_value;	    /* -> where to store attribute value. */
{
	XvddPortPtr pddPort;	/* Device-dependent storage for pPort. */


	pddPort = (XvddPortPtr)pPort->devPriv.ptr;

	if (attribute == attr_atoms.DummyEncoding)
	{
		*p_value = pPort->pEncoding->id;
	}
	else if (attribute == attr_atoms.DummyBrightness)
	{
		*p_value = pddPort->brightness;
	}
	else if (attribute == attr_atoms.DummyContrast)
	{
		*p_value = pddPort->contrast;
	}
	else if (attribute == attr_atoms.DummyHue)
	{
		*p_value = pddPort->hue;
	}
	else if (attribute == attr_atoms.DummySaturation)
	{
		*p_value = pddPort->saturation;
	}
	else 
	{
		*p_value = 0;
	}

    DEBUGF(MSG, 
		(fd,"DummyGetPortAttribute: port: %08x attribute %08x = 0x%08x (%d)\n",
		pPort, attribute, *p_value, *p_value));
	return Success;
}


/*----------------------------------------------------------------------------*
 |"DummySetLongPortAttribute"
 |
 |  Transfer vector routine to set an attribute for a port.
 *----------------------------------------------------------------------------*/
static int DummySetLongPortAttribute(client, pPort, attribute, length, value)
	ClientPtr client;
	XvPortPtr pPort;
	Atom      attribute;
	INT32     length;
	INT32     *value;
{
	int			i_value;	/* Entry in 'p_value' accessing. */
	char*		p_value;	/* 'value' aliased as bytes. */

    DEBUGF(MSG, 
		(fd,"DummySetLongPortAttribute: port: %08x attribute %08x length %d:",
		pPort, attribute, length));
	for ( p_value = (char*)value, i_value = 0; i_value < length; i_value++ )
	{
		if ( i_value % 4 == 0 ) DEBUGF(MSG, (fd, " "));
		if ( i_value % 32 == 0 ) DEBUGF(MSG, (fd, "\n    "));
		DEBUGF(MSG, (fd, "%02x", p_value[i_value]));
	}
	DEBUGF(MSG, (fd, "\n"));

	XvdiSendPortNotify(pPort, attribute, ((int)value & 3) ? 0xdeadbeef: *value);

	return Success;
}

/*----------------------------------------------------------------------------*
 |"DummySetPortAttribute"
 |
 |  Transfer vector routine to set an attribute for a port.
 *----------------------------------------------------------------------------*/
static int DummySetPortAttribute(client, pPort, attribute, value)
	ClientPtr client;
	XvPortPtr pPort;
	Atom      attribute;
	INT32     value;
{
	Xvhw_Drw_Area dst_area;   /* PIP destination area. */
	int           iEncoding;
	Xvhw_Levels   levels;     /* PIP color levels. */
	unsigned char lut[256];
	int           on_off;
	XvddPortPtr   pddPort;
	XvEncodingPtr pEncode;
	Xvhw_Signal   signal;     /* Signal type and timing. */

	pddPort = (XvddPortPtr)pPort->devPriv.ptr;


	/* FIND THE ATTRIBUTE AND SET IT; XXX --- MORE LATER */

	if (attribute == attr_atoms.DummyEncoding)
	{
		pEncode = pPort->pEncodings;
		for (iEncoding = 0; iEncoding < pPort->nEncodings; iEncoding++)
		{
			if (pEncode->id == (XvEncodingID)value)
			{

				/* SAVE NEW ENCODING ID */

				pPort->pEncoding = pEncode;
				break;
			}
			pEncode++;
		}
		if (iEncoding>=pPort->nEncodings) 
		{
			return 0;
		}
	}
	else if (attribute == attr_atoms.DummyBrightness)
	{
		if (value < -1000)      { pddPort->brightness = -1000; }
		else if (value > 1000)  { pddPort->brightness = 1000;  }
		else                    { pddPort->brightness = value; }
		DEBUGF(MSG, (fd, "DummySetPortAttribute: port %08x: brightness = %d\n", 
			pPort, pddPort->brightness));
	}
	else if (attribute == attr_atoms.DummyContrast)
	{
		if (value < -1000)      { pddPort->contrast = -1000; }
		else if (value > 1000)  { pddPort->contrast = 1000;  }
		else                    { pddPort->contrast = value; }
		DEBUGF(MSG, (fd, "DummySetPortAttribute: port %08x: contrast = %d\n", 
			pPort, pddPort->contrast));
	}

	else if (attribute == attr_atoms.DummyHue)
	{
		if (value < -1000)      { pddPort->hue = -1000; }
		else if (value > 1000)  { pddPort->hue = 1000;  }
		else                    { pddPort->hue = value; }
		DEBUGF(MSG, (fd, "DummySetPortAttribute: port %08x: hue = %d\n", 
			pPort, pddPort->hue));
	}
	else if (attribute == attr_atoms.DummySaturation)
	{
		if (value < -1000)      { pddPort->saturation = -1000; }
		else if (value > 1000)  { pddPort->saturation = 1000;  }
		else                    { pddPort->saturation = value; }
		DEBUGF(MSG, (fd, "DummySetPortAttribute: port %08x: hue = %d\n", 
			pPort, pddPort->hue));
	}
	else
	{
		DEBUGF(MSG, (fd, 
			"DummySetPortAttribute: port %08x atom = %08x value = %08x (%d)\n",
			pPort, attribute, value, value));
	}

	XvdiSendPortNotify(pPort, attribute, value);

	return Success;
}
