/*
 *  Calibrate.c:	Program to align Parallax overlay card.
 *
 *   Notes:
 *    (1) Derived from "testmenus" with permission.
 *
 *	Copyright(c) 1994, Parallax Graphics, Inc.
 */
#include <stdio.h>
#include <sys/types.h>
#include <X11/Xlib.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <Xm/DialogS.h>
#include <Xm/Form.h>
#include <Xm/PanedW.h>
#include <Xm/PushBG.h>
#include <Xm/PushB.h>
#include <Xm/DrawingA.h>
#include <Xm/SashP.h>
#include <Xm/MessageB.h>
#include "Xvlib.h"

#include "Xvddx_PGI.h"
#include "O_XV.h"
#include "support.h"
#include "hw_context.h"
#include "help_text.h"

/*
 *  FORWARD REFERENCES:
 */
void    cb_adaptor();
void    cb_help();
void    cb_default_ok();
void    cb_nvram_load();
void    cb_nvram_store();
void    cb_save_as_ok();
void    cb_select_color();
void    cb_zoom();
void    draw_host();
XImage* draw_xv();
void    set_title();
void    send_to_server();

enum {
	HOST_GRID = 80,
	WIN_HEIGHT = 480,
	WIN_WIDTH = 1280,
	XV_GRID = 80,
	XV_WIDTH = 1280, 
	XV_HEIGHT = 480,
};

typedef struct Select {	/* Selection box parameters: */
	Pixel  				color_bg;		/* Background color. */
	Pixel  				color_select;	/* Selection highlight color. */
	Widget 			    w_dialog;		/* Whole dialog box widget. */
	Widget 				w_filter;		/* File filter text widget. */
	Widget 				w_select;		/* File selection text widget. */
} Select;

typedef struct Calib {				/* Context for callbacks: */
	Atom		 		 a_nvram_c;	/* Atom to control nvram operations with */
	Atom		 		 a_nvram_m;	/* Atom to access main nvram values with */
	Atom		 		 a_nvram_z;	/* Atom to access zoom nvram values with */
	Atom		 		 a_ximage;	/* Atom to for Ximages via XvPutimage. */
	Atom		 		 a_zoom;	/* Atom for XV_PLX_ZOOM attribute. */
	Select				 file;		/* File dialog parameters. */
	char				 fname[99];	/* Device characteristics file name. */
	GC					 gc_hp;		/* Graphics context for lines in hp fb. */
	GC					 gc_black;	/* Graphics context for fill in hp fb. */
	int					 ysize;     /* Y size of display. */
	int					 n_main;	/* =  size of are pointed to by 'p_main' */
	int					 n_xvs;		/* =  # of xv adaptors of our type. */
	int					 n_zoom;	/* =  size of are pointed to by 'p_zoom' */
	XImage*				 p_image;	/* -> image to write to xv memory. */
	XV_controller_nvram* p_main;	/* -> current main nvram settings. */
	O_Rsrc*				 p_rsrc;	/* -> resource object to use. */
	O_XV_Select*		 p_select;	/* -> xv selection object. */
	O_XV*				 p_xv;		/* -> xv access object now accessing. */
	XV_zoom_nvram*       p_zoom;	/* -> current zoom nvram settings. */
	XV_struct			 xv_struct;	/* Hardware structure. */
	int					 zoom;		/* =  current zoom factor. */
} Calib;

XtAppContext	appContext;
int				host_grid = HOST_GRID;
Calib 			me;					/* Context for callbacks. */
Menu_Linked_E*  p_adaptors_list; 	/* Head of adaptor list. */
Widget			w_form;	
int				win_height = WIN_HEIGHT;
int				win_width = WIN_WIDTH;
Widget 			w_toplevel;
Widget 			w_video;
int				xv_grid = XV_GRID;
int				xv_height = XV_HEIGHT;
int				xv_width = XV_WIDTH;
int				hvkeys = 0;
int			previous_xsize;
int			previous_ysize;

static char* adaptor_labels[] = {
#ifdef _AIX
    "/dev/pv0", 
    "/dev/pv2", 
    "/dev/pv4", 
    "/dev/pv6", 
    "/dev/pv8", 
    "/dev/pv10" 
};
#define PREFIX "/usr/lpp/parallax/etc"
#else /* !_AIX */
    "/dev/plxv0", 
    "/dev/plxv2",
    "/dev/plxv4", 
    "/dev/plxv6", 
    "/dev/plxv8", 
    "/dev/plxv10"
};
#define PREFIX "/opt/parallax/etc"
#endif /* _AIX */

/*---------------------------------------------------------------------------*
 |"build_adaptors_list"
 *---------------------------------------------------------------------------*/
static Menu_Linked_E* build_adaptors_list(n_adaptors)
	int		n_adaptors;		/* = # of adaptors in list. */
{
	int				i_adaptor;	/* # of adaptor now building entry for. */
	Menu_Linked_E*	p_entry;	/* Entry now building. */
	Menu_Linked_E*	p_head;		/* Menu list to return. */

	for ( p_head = 0, i_adaptor = n_adaptors-1; i_adaptor >= 0; --i_adaptor )
	{
        p_entry = (Menu_Linked_E*)malloc(sizeof(Menu_Linked_E));
		p_entry->name = adaptor_labels[i_adaptor];
        p_entry->p_next = p_head;
		p_head = p_entry;
        p_entry->mnemonic = 0;
        p_entry->misc = 0;
        p_entry->cb_data = (void*)i_adaptor;
        p_entry->cb_func = cb_adaptor;
        p_entry->sub_menu = 0;
        p_entry->sub_menu_title = 0;
		p_entry->toggle_flag = 0;
        p_entry->widget = 0;
	}
	return p_head;
}

/*---------------------------------------------------------------------------*
 |"build_file_select"
 *---------------------------------------------------------------------------*/
void build_file_select(w, p_me, title)
    Widget  w;             /* = widget causing the callback. */
    Calib*  p_me;          /* Our context. */
{
	Colormap	cmap;	/* Color map in use with this widget. */
	String		text;	/* Text for selection and file filters. */

#ifdef MP
    p_me->file.w_dialog = mpCreateFileSelectionDialog(w,
        title, NULL,
    0);
#else
    p_me->file.w_dialog = (Widget)XmCreateFileSelectionDialog(w,
        title, NULL,
    0);
#endif
    XtVaSetValues(p_me->file.w_dialog,
		XmNwidth, 400,
	NULL);

	/* 
     *  SET UP COLOR SCHEME AND ACQUIRE POINTERS TO WIDGETS WE WILL USE.
     */
    XtVaGetValues(p_me->file.w_dialog,
        XmNbackground, &(p_me->file.color_bg),
        XmNcolormap,   &cmap,
    NULL);
    XmGetColors(XtScreen(p_me->file.w_dialog), cmap, p_me->file.color_bg,
        0, 0, 0, &(p_me->file.color_select));
    p_me->file.w_select = (Widget)XmFileSelectionBoxGetChild(
		p_me->file.w_dialog,XmDIALOG_TEXT);
    p_me->file.w_filter = (Widget)XmFileSelectionBoxGetChild(
		p_me->file.w_dialog,XmDIALOG_FILTER_TEXT);

	/*
     *  SET UP CALL BACKS FOR THOSE COMPONENTS WE WISH TO INTERACT WITH.
     */
    XtAddCallback(p_me->file.w_select,
		XmNfocusCallback, cb_select_color, p_me);
    XtAddCallback(p_me->file.w_filter,
		XmNfocusCallback, cb_select_color, p_me);
    XtAddCallback(p_me->file.w_dialog,
		XmNcancelCallback, XtDestroyWidget, NULL);
    XtAddCallback(p_me->file.w_dialog,
		XmNhelpCallback, cb_help, "Save_As");

	/* 
     *  SET DEFAULT FILTER AND FILE NAMES.
     */
#ifdef _AIX
	text = XmStringCreateLtoR("/usr/lpp/parallax/etc/calibrate/", 
		XmSTRING_DEFAULT_CHARSET);
#endif
#if defined(__hpux) || defined(sun)
	text = (char*)XmStringCreateLtoR("/opt/parallax/etc/calibrate/", 
		XmSTRING_DEFAULT_CHARSET);
#endif /* defined(__hpux) || defined(sun) */
	XtVaSetValues(p_me->file.w_dialog, XmNdirectory, text, NULL);
	XmStringFree(text);
}

/*---------------------------------------------------------------------------*
 |"cb_about_calibrate"
 *---------------------------------------------------------------------------*/
void cb_about_calibrate(w, p_data, event)
    Widget  w;             /* = widget causing the callback. */
    char*   p_data;          /* Our instance specific data. */
    XEvent* event;         /* Event causing keyboard or mouse callback. */
{
	create_dialog(w, "Calibrate", 
		"Calibrate 2.0 - (c) Parallax - 05 May 1996", 0);
}

/*---------------------------------------------------------------------------*
 |"cb_adaptor"
 *---------------------------------------------------------------------------*/
void cb_adaptor(w, i_adaptor, event)
    Widget  w;             /* =  widget causing the callback. */
    Calib*  i_adaptor;     /* =  number of adaptor chosen. */
    XEvent* event;         /* Event causing keyboard or mouse callback. */
{
	set_xv_adaptor(&me, i_adaptor);
	O_XV_Use_Window( me.p_xv, XtWindow(w_video) );
}

/*---------------------------------------------------------------------------*
 |"cb_default"
 *---------------------------------------------------------------------------*/
void cb_default(w, p_me, event)
    Widget  w;             /* = widget causing the callback. */
    Calib*  p_me;          /* Our context. */
    XEvent* event;         /* Event causing keyboard or mouse callback. */
{
	build_file_select(w, p_me, "Default Settings");
    XtAddCallback(p_me->file.w_dialog, XmNokCallback, cb_default_ok, p_me);
    XtManageChild(p_me->file.w_dialog);
}

/*---------------------------------------------------------------------------*
 |"cb_default_ok"
 *---------------------------------------------------------------------------*/
void cb_default_ok(w, p_me, event)
    Widget  w;             /* = widget causing the callback. */
    Calib*  p_me;          /* Our context. */
    XEvent* event;         /* Event causing keyboard or mouse callback. */
{
	char	    error_msg[256];	/* Error message. */
	FILE*		fd;				/* File to load from. */
	char*		p_file_name;	/* Temporary for converted file name. */
	XmString*	p_string;		/* Raw file name specification. */

	p_me->ysize = p_me->p_main->ysize;
	XtVaGetValues(p_me->file.w_dialog, XmNdirSpec, &p_string, NULL);
	if(!XmStringGetLtoR(p_string, XmSTRING_DEFAULT_CHARSET, &p_file_name))
		return;
#ifdef OLDWAY
	if ( load_settings(w, p_me, p_file_name) )
	{
		p_me->p_main->zoom = p_me->zoom;
		send_to_server(p_me);
		XtDestroyWidget(w);
	}
	XmStringFree(p_string);
#else /* ! OLDWAY */
 	O_Rsrc_load_from_context(p_me->p_rsrc, &p_me->xv_struct);
	if ( fd = fopen(p_file_name, "r") )
	{
		O_Rsrc_load_from_file(p_me->p_rsrc, fd);
		fclose(fd);
		O_Rsrc_store_to_context(p_me->p_rsrc, &p_me->xv_struct);
		p_me->p_main->zoom = p_me->zoom;
		send_to_server(p_me);
		set_title(p_me);
	}
	else
	{
		sprintf(error_msg, "Unable to open %s for read", p_me->fname);
		create_dialog(w, "Calibrate", error_msg, 0);
	}
#endif /* OLDWAY */
}

/*---------------------------------------------------------------------------*
 |"quit"
 *---------------------------------------------------------------------------*/
void quit(w, client_data, call_data)
    Widget      w;             /* = widget causing the callback. */
    XtPointer   client_data;   /* Not used. */
    XtPointer   call_data;     /* Not used. */
{
       exit(0);
}

/*---------------------------------------------------------------------------*
 |"cb_exit"
 *---------------------------------------------------------------------------*/
void cb_exit(w, p_data, event)
    Widget  w;             /* = widget causing the callback. */
    char*   p_data;          /* Our instance specific data. */
    XEvent* event;         /* Event causing keyboard or mouse callback. */
{
        if(previous_xsize == me.p_main->xsize &&
	   previous_ysize == me.p_main->ysize)
	{
	  /*
	   * The resolution has not changed, so just exit.
	   */
	  exit(0);
	}
	else
	{
	  /*
	   * The resolution has changed.  The server must be
	   * restarted.  Inform the user with a dialog box,
	   * then exit.
	   */

	  Widget messageWidget;
	  Widget messageDialog;
	  XmString message;
	
	  messageDialog = XtVaCreateWidget("Warning", xmDialogShellWidgetClass,
					   w_toplevel,
					   NULL);
					 
	  message = XmStringCreateLocalized("Restart the window server for resolution changes to take affect.");
	  messageWidget = XtVaCreateManagedWidget("Warning Message", xmMessageBoxWidgetClass,
						  messageDialog,
						  XmNmessageString, message,
						  NULL);
	  XtUnmanageChild(XmMessageBoxGetChild(messageWidget,
					       XmDIALOG_HELP_BUTTON));
	  XtUnmanageChild(XmMessageBoxGetChild(messageWidget,
					       XmDIALOG_CANCEL_BUTTON));
	  XtAddCallback(XmMessageBoxGetChild(messageWidget,
					     XmDIALOG_OK_BUTTON),
			XmNactivateCallback,
			quit,
			NULL);
	  XtPopup(messageDialog, XtGrabExclusive);
	}
}

/*---------------------------------------------------------------------------*
 |"cb_expose"
 *---------------------------------------------------------------------------*/
void cb_expose(w, p_me, event)
    Widget  w;             /* = widget causing the callback. */
    Calib*  p_me;          /* Our context. */
    XEvent* event;         /* Event causing keyboard or mouse callback. */
{
	int n = p_me->p_image->bytes_per_line * xv_height;
	draw_host(p_me,host_grid/2, host_grid/2, win_width-host_grid, 
		win_height-host_grid);
#ifdef _AIX
	XPutImage(p_me->p_xv->display, p_me->p_xv->w_main, p_me->gc_hp, 
		p_me->p_image, 0, 0, 0, 0, xv_width, xv_height);
#else
	XvPutImage(p_me->p_xv->display, p_me->p_xv->port_id, p_me->p_xv->w_main, 
		p_me->p_xv->gc, p_me->a_ximage, n, p_me->p_image, 
		0, 0, xv_width, xv_height, 0, 0, xv_width, xv_height);

#endif

}


/*---------------------------------------------------------------------------*
 |"cb_help"
 *---------------------------------------------------------------------------*/
void cb_help(w, item_name, event)
    Widget  w;             /* = widget causing the callback. */
    char*   item_name;     /* -> item help is desired for. */
    XEvent* event;         /* Event causing keyboard or mouse callback. */
{
	Arg 	args[20];		/* XtSet argument list. */
	Pixel   background;		/* Background color for help panel. */
	char    buf[80*25];		/* Buffer to hold text. */
	int		i_args;			/* Element in args accessing. */
	int		i_item;			/* Help item now testing against item_name. */
	int     i_line;			/* Line of help text now processing. */
	int		n_help;			/* Number of lines in data pointed to by p_help. */
	char*   p_buf;			/* Next location in buf to set. */
	char**	p_help;			/* Help information to use. */
	Widget  w_form;			/* Form widget containing 'w_text'. */
	Widget	w_help;			/* Popup shell containing the other widgets. */
	Widget 	w_pane;			/* Pane widget used to manage presentation. */
	Widget  w_text;			/* Text scrolling widget. */

	XtVaGetValues(XtParent(w),XmNbackground,&background,NULL);

	/*
	 *  CREATE A POPUP SHELL CONTAINING A PANED WINDOW TO PRESENT THE
	 *  HELP INFORMATION
	 */
	w_help = XtVaCreatePopupShell("Calibrate - Help",
		xmDialogShellWidgetClass,	XtParent(w),
		XmNdeleteResponse,			XmDESTROY,
		XmNbackground,				background,
	NULL);

	w_pane = XtVaCreateWidget("pane",
		xmPanedWindowWidgetClass,	w_help,
		XmNbackground,				background,
	NULL);

	w_form = XtVaCreateWidget("form",
		xmFormWidgetClass,			w_pane,
		XmNbackground,				background,
	NULL);

	/*
	 *  BUILD BUFFER WITH TEXT FOR SCROLL AREA BASED ON WHAT HELP IS DESIRED.
	 */
 	for ( p_help = 0, i_item = 0; i_item < n_help_items; i_item++ )
	{
		if ( !strcmp(item_name, help_items[i_item].name) )
		{
			p_help = help_items[i_item].p_p_text;
			n_help = help_items[i_item].n_lines;
		}
	}
	if ( p_help == 0 )
	{
		return;
	}
	for ( p_buf =  buf, i_line = 0; i_line < n_help; i_line++ )
	{
		p_buf += strlen(strcpy(p_buf, p_help[i_line]));
		*p_buf++ = '\n';
		*p_buf = 0;
	}
	*(--p_buf) = 0;			/* Remove final newline. */

	i_args = 0;
	XtSetArg(args[i_args],	XmNbackground,	      background);        i_args++;
	XtSetArg(args[i_args],  XmNcolumns,			  80);                i_args++;
	XtSetArg(args[i_args],	XmNcursorPositionVisible,False);          i_args++;
	XtSetArg(args[i_args],	XmNeditMode,		  XmMULTI_LINE_EDIT); i_args++;
	XtSetArg(args[i_args],	XmNeditable,		  False);             i_args++;
	XtSetArg(args[i_args],	XmNrows,			  25);                i_args++;
	XtSetArg(args[i_args],	XmNscrollVertical,	  True);              i_args++;
	XtSetArg(args[i_args],	XmNscrollHorizontal,  False);             i_args++;
	XtSetArg(args[i_args],	XmNvalue,			  buf);               i_args++;
	XtSetArg(args[i_args],	XmNwordWrap,		  True);              i_args++;

	w_text = (Widget)XmCreateScrolledText(w_form, "help_text", args, i_args);

	XtManageChild(w_text);
	XtManageChild(w_form);

	XtManageChild(w_pane);
	XtPopup(w_help,XtGrabNone);
}

/*---------------------------------------------------------------------------*
 |"cb_nvram_load"
 *---------------------------------------------------------------------------*/
void cb_nvram_load(w, p_me, event)
    Widget  w;             /* = widget causing the callback. */
    Calib*  p_me;          /* Our context. */
    XEvent* event;         /* Event causing keyboard or mouse callback. */
{
	char	error_msg[256];	/* Error message. */
	FILE*	fd;				/* File descriptor for p_me->fname. */

 	O_Rsrc_load_from_context(p_me->p_rsrc, &p_me->xv_struct);
	if ( fd = fopen(p_me->fname, "r") )
	{
		O_Rsrc_load_from_file(p_me->p_rsrc, fd);
		fclose(fd);
		O_Rsrc_store_to_context(p_me->p_rsrc, &p_me->xv_struct);
		p_me->p_main->zoom = p_me->zoom;
		send_to_server(p_me);
	}
	else
	{
		sprintf(error_msg, "Unable to open %s for read", p_me->fname);
		create_dialog(w, "Calibrate", error_msg, 0);
	}
}


/*---------------------------------------------------------------------------*
 |"cb_nvram_store"
 *---------------------------------------------------------------------------*/
void cb_nvram_store(w, p_me, event)
    Widget  w;             /* = widget causing the callback. */
    Calib*  p_me;          /* Our context. */
    XEvent* event;         /* Event causing keyboard or mouse callback. */
{
	char	error_msg[256];	/* Error message. */
	FILE*	fd;				/* File descriptor for p_me->fname. */

 	O_Rsrc_load_from_context(p_me->p_rsrc, &p_me->xv_struct);
	if ( fd = fopen(p_me->fname, "w") )
	{
		O_Rsrc_store_to_file(p_me->p_rsrc, fd);
		fclose(fd);
		chmod(p_me->fname, 0666);
	}
	else
	{
		sprintf(error_msg, "Unable to open %s for write", p_me->fname);
		create_dialog(w, "Calibrate", error_msg, 0);
	}
}

/*---------------------------------------------------------------------------*
 |"cb_operator"
 |
 | Callback routine for processing user key strokes and mouse events.
 *---------------------------------------------------------------------------*/
void cb_operator(w, p_me, event)
    Widget  w;             /* = widget causing the callback. */
    Calib*  p_me;          /* Our instance specific data. */
    XEvent* event;         /* Event causing keyboard or mouse callback. */

{
    Window 			child;          /* The ID of child of window pointer is in*/
    XComposeStatus  composestatus;  /* Specified the XCompose structure.*/
    KeySym 		    keysym;         /* KeySym ID computed from event.*/
	int				loop;			/* Loop control. */
	unsigned int    mask;     		/* The button mask. */
	O_XV*			p_xv;			/* Fast access to O_XV: p_me->p_xv. */
    Window 			root;           /* The root window ID pointer is on.*/
    int 			rx;    			/* Return the x and y cordinates of the */
    int 			ry;             /* Pointer relative to the root's origin.*/
    char 			string[31];     /* Storage to hold converted string. */
    int 			wx;             /* Return the x and y coordinates of the */
    int 			wy;             /* Pointer relative to origin of window. */

	p_xv = p_me->p_xv;

	switch( event->type )
	{
	  case KeyPress:
        XLookupString(event, string, 30, &keysym, &composestatus);


        /*  USER PRESSED SHIFT + LEFT OR RIGHT: FINE TUNE USING THE DELAY VALUE
		 *  USER PRESSED SHIFT + S: ADJUST THE SKIP FACTOR UP.
         */
        if (event->xkey.state & ShiftMask) 
		{
            switch(keysym) 
			{

			  case XK_H:
			    if(hvkeys == 0)
				  return;
				switch(p_me->zoom)
				{
				    case 2:  p_me->p_zoom->h.fp++; break;
				    default: p_me->p_main->h.fp++; break;
				}
				break;

#ifdef sun
			  case XK_F30:
#endif
			  case XK_KP_4:
			  case XK_Shift_L :
			  case XK_Left:
				switch(p_me->zoom)
				{
				  case 2:
					if ( p_me->p_zoom->delay > 0 ) p_me->p_zoom->delay -= 2;
					break;
				  default:
					if ( p_me->p_main->delay > 0 ) p_me->p_main->delay -= 2;
					break;
				}
				break;


#ifdef sun
			  case XK_F32:
#endif
			  case XK_Shift_R :
			  case XK_KP_6:
			  case XK_Right:
				switch(p_me->zoom)
				{
				  case 2:
					if ( p_me->p_zoom->delay < 0x80 ) p_me->p_zoom->delay += 2;
					break;
				  default:
					if ( p_me->p_main->delay < 0x80 ) p_me->p_main->delay += 2;
					break;
				}
				break;

			  case XK_S:
				switch(p_me->zoom)
				{
				  case 2:
					if ( p_me->p_zoom->skip < 14 ) p_me->p_zoom->skip += 2;
					break;
				  default:
					if ( p_me->p_main->skip < 14 ) p_me->p_main->skip += 2;
					break;
				}
				break;
		
			  case XK_V:
			    if(hvkeys == 0)
				  return;
				switch(p_me->zoom)
				{
					case 2:  p_me->p_zoom->v.fp++; break;
					default: p_me->p_main->v.fp++; break;
				}
				break;

			  default :
				/* fprintf(stderr, "%02x!!!\n", keysym); */
				return;
            }
			send_to_server(p_me);
        }


        /*
         *  USER PRESSED LEFT, OR RIGHT: ADJUST USING FRONT AND BACK PORCH 
		 *  VALUES.
         */
        else
        {
            switch(keysym) 
			{
			  case XK_h:
			    if(hvkeys == 0)
				  return;
				switch(p_me->zoom)
				{
				  case 2:
					if ( p_me->p_zoom->h.fp > 0 ) --p_me->p_zoom->h.fp;
				  	else if ( p_me->p_zoom->h.bp > 0 ) --p_me->p_zoom->h.bp;
					break;
				  default:
					if ( p_me->p_main->h.fp > 0 ) --p_me->p_main->h.fp;
					else if ( p_me->p_main->h.bp > 0 ) --p_me->p_main->h.bp;
					break;
				}
				break;

			  case XK_Left:
				switch(p_me->zoom)
				{
				  case 2:
					if ( p_me->p_zoom->h.bp > 0 )
					{
						p_me->p_zoom->h.bp -= 1;
						p_me->p_zoom->h.fp += 1;
					}
					break;
				  default:
					if ( p_me->p_main->h.bp > 0 )
					{
						p_me->p_main->h.bp -= 1;
						p_me->p_main->h.fp += 1;
					}
					break;
				}
                break;

              case XK_Up :
				switch(p_me->zoom)
				{
				  case 2:
					if ( p_me->p_zoom->v.bp > 0 )
					{
						p_me->p_zoom->v.bp -= 1;
						p_me->p_zoom->v.fp += 1;
					}
					break;
				  default:
					if ( p_me->p_main->v.bp > 0 )
					{
						p_me->p_main->v.bp -= 1;
						p_me->p_main->v.fp += 1;
					}
					break;
				}
                break;

              case XK_Right :
				switch(p_me->zoom)
				{
				  case 2:
					if ( p_me->p_zoom->h.fp > 0 )
					{
						p_me->p_zoom->h.fp -= 1;
						p_me->p_zoom->h.bp += 1;
					}
					break;
				  default:
					if ( p_me->p_main->h.fp > 0 )
					{
						p_me->p_main->h.fp -= 1;
						p_me->p_main->h.bp += 1;
					}
					break;
				}
                break;

              case XK_Down:
				switch(p_me->zoom)
				{
				  case 2:
					if ( p_me->p_zoom->v.fp > 0 )
					{
						p_me->p_zoom->v.fp -= 1;
						p_me->p_zoom->v.bp += 1;
					}
					break;
				  default:
					if ( p_me->p_main->v.fp > 0 )
					{
						p_me->p_main->v.fp -= 1;
						p_me->p_main->v.bp += 1;
					}
					break;
				}
                break;

			  case XK_s:
				switch(p_me->zoom)
				{
				  case 2:
					if ( p_me->p_zoom->skip > 0 ) p_me->p_zoom->skip -= 2;
					break;
				  default:
					if ( p_me->p_main->skip > 0 ) p_me->p_main->skip -= 2;
					break;
				}
				break;

			  case XK_v:
			    if(hvkeys == 0)
				  return;
				switch(p_me->zoom)
				{
				  case 2:
					if ( p_me->p_zoom->v.fp > 0 ) --p_me->p_zoom->v.fp;
					else if ( p_me->p_zoom->v.bp > 0 ) --p_me->p_zoom->v.bp;
					break;
				  default:
					if ( p_me->p_main->v.fp > 0 ) --p_me->p_main->v.fp;
					else if ( p_me->p_main->v.bp > 0 ) --p_me->p_main->v.bp;
					break;
				}
				break;
			  default:
				return;
            }
			send_to_server(p_me);
        }
		break;


#ifdef DIAG
	  case ButtonPress:
        XQueryPointer(p_xv->display, p_xv->w_main, &root, &child,
            &rx, &ry, &wx, &wy, &mask);

        /*
         * USER PRESSED SHIFT KEY AND MOVED MOUSE:
         */
        if(mask & ShiftMask)
        {
            for( loop = 1; loop; )
            {
                XQueryPointer(p_xv->display,p_xv->w_main, &root, &child,
                    &rx, &ry, &wx, &wy, &mask);
				switch( mask & (Button1Mask|Button2Mask|Button3Mask) ) 
				{
				  case Button1Mask: 
					printf("Shift mouse button 1\n"); break;
				  case Button1Mask|Button2Mask: 
					printf("Shift mouse buttons 1 and 2\n"); break;
				  case Button1Mask|Button2Mask|Button3Mask: 
					printf("Shift mouse buttons 1, 2 and 3\n"); break;
				  case Button1Mask|Button3Mask: 
					printf("Shift mouse buttons 1 and 3\n"); break;
				  case Button2Mask: 
					printf("Shift mouse button 2\n"); break;
				  case Button2Mask|Button3Mask: 
					printf("Shift mouse buttons 2 and 3\n"); break;
				  case Button3Mask: 
					printf("Shift mouse button 3\n"); break;
				  default: 
					loop = 0; break;
				}
            }
        }

        /*
         * USER MOVED MOUSE:
         */
		for( loop = 1; loop; )
		{
			XQueryPointer(p_xv->display,p_xv->w_main,
				&root, &child, &rx, &ry, &wx, &wy, &mask);
			switch( mask & (Button1Mask|Button2Mask|Button3Mask) ) 
			{
			  case Button1Mask: 
				printf("Mouse button 1\n"); break;
			  case Button1Mask|Button2Mask: 
				printf("Mouse buttons 1 and 2\n"); break;
			  case Button1Mask|Button2Mask|Button3Mask: 
				printf("Mouse buttons 1, 2 and 3\n"); break;
			  case Button1Mask|Button3Mask: 
				printf("Mouse buttons 1 and 3\n"); break;
			  case Button2Mask: 
				printf("Mouse button 2\n"); break;
			  case Button2Mask|Button3Mask: 
				printf("Mouse buttons 2 and 3\n"); break;
			  case Button3Mask: 
				printf("Mouse button 3\n"); break;
			  default: 
				loop = 0; break;
			}
        }
#endif /* DIAG */
    }

	set_title(p_me);
 
}

/*---------------------------------------------------------------------------*
 |"cb_save"
 *---------------------------------------------------------------------------*/
void cb_save(w, p_data, event)
    Widget  w;             /* = widget causing the callback. */
    char*   p_data;          /* Our instance specific data. */
    XEvent* event;         /* Event causing keyboard or mouse callback. */
{
}

/*---------------------------------------------------------------------------*
 |"cb_save_as"
 *---------------------------------------------------------------------------*/
void cb_save_as(w, p_me, event)
    Widget  w;             /* = widget causing the callback. */
    Calib*  p_me;          /* Our context. */
    XEvent* event;         /* Event causing keyboard or mouse callback. */
{
	build_file_select(w, p_me, "Save Settings");
    XtAddCallback(p_me->file.w_dialog,
		XmNokCallback, cb_save_as_ok, p_me);
    XtManageChild(p_me->file.w_dialog);
}

/*---------------------------------------------------------------------------*
 |"cb_save_as_ok"
 *---------------------------------------------------------------------------*/
void cb_save_as_ok(w, p_me, event)
    Widget  w;             /* = widget causing the callback. */
    Calib*  p_me;          /* Our context. */
    XEvent* event;         /* Event causing keyboard or mouse callback. */
{
	char*		p_file_name;	/* Temporary for converted file name. */
	XmString*	p_string;		/* Raw file name specification. */

	XtVaGetValues(p_me->file.w_dialog, XmNdirSpec, &p_string, NULL);
	if(!XmStringGetLtoR(p_string, XmSTRING_DEFAULT_CHARSET, &p_file_name))
		return;
	if ( save_settings(w, p_me, p_file_name) )
	{
		XtDestroyWidget(w);
	}
	XmStringFree(p_string);
		
}

/*-------------------------------------------------------------------------*
 |"cb_select_color"
 *-------------------------------------------------------------------------*/
void cb_select_color(w, p_me)
	Widget w;
	Calib* p_me;
{
    XtVaSetValues(w,
        XmNselectColor, p_me->file.color_select,
        XmNbackground,p_me->file.color_select,
    NULL);
    if (w == p_me->file.w_select) 
    {
		XtVaSetValues(p_me->file.w_filter,
			XmNselectColor, p_me->file.color_select,
			XmNbackground,p_me->file.color_bg,
		NULL);
    } 
	else 
	{
		XtVaSetValues(p_me->file.w_select,
        	XmNselectColor, p_me->file.color_select,
        	XmNbackground,p_me->file.color_bg,
    	NULL);
    }
 
}


/*---------------------------------------------------------------------------*
 |"cb_zoom"
 *---------------------------------------------------------------------------*/
void cb_zoom(w, factor, event)
    Widget  w;          /* = widget causing the callback. */
    int     factor;     /* = zoom factor desired. */
    XEvent* event;      /* Event causing keyboard or mouse callback. */
{

	XFillRectangle(me.p_xv->display, me.p_xv->w_main, me.gc_black, 
		0, 0, win_width, win_height);
	me.zoom = factor;
	me.p_main->zoom = factor;
	O_XV_Set_Attribute(me.p_xv, me.a_zoom, me.zoom);
	win_width = WIN_WIDTH; 
	win_height = me.p_xv->p_encodings->height;

	switch ( me.zoom )
	{
	  case 2:  host_grid = HOST_GRID*2; break;
	  default: host_grid = HOST_GRID;   break;
	}

    XtVaSetValues(w_toplevel, 
		XmNwidth,   win_width, 
		XmNheight,  win_height+120,
    NULL);
    XtVaSetValues(w_form, 
		XmNwidth,   win_width, 
		XmNheight,  win_height+120,
    NULL);
    XtVaSetValues(w_video, 
		XmNwidth,   win_width, 
		XmNheight,  win_height,
    NULL);

#ifdef _AIX
	draw_host(&me, host_grid/2, host_grid/2, win_width-host_grid, 
		win_height-host_grid);
	XFlush(me.p_xv->display);
	O_XV_Video_On(me.p_xv);
	O_XV_Video_Off(me.p_xv);
	XPutImage(me.p_xv->display, me.p_xv->w_main, me.gc_hp, 
		me.p_image, 0, 0, 0, 0, xv_width, xv_height);
#else
#if 0
	cb_expose(w_video, &me, 0);
#else
	refreshmain();
#endif
#endif /* _AIX */
	set_title(&me);
}

/*---------------------------------------------------------------------------*
 |"draw_host"
 *---------------------------------------------------------------------------*/
void draw_host(p_me, x, y, width, height)
	Calib*	p_me;			/* -> context for this program. */
	int		x;				/* =  x coordinate in window to start grid at. */
	int		y;				/* =  y coordinate in window to start grid at. */
	int		width;			/* = width of grid to draw. */
	int		height;			/* = height of grid to draw. */
{
	int		i_x;		/* Next x pixel in 'w' to place grid line at. */
	int		i_y;		/* Next y pixel in 'w' to place grid line at. */
	O_XV*	p_xv;		/* Fast access to O_XV context: p_me->p_xv. */

	p_xv = p_me->p_xv;
	width += x;
	height += y;


	/*  DRAW HORIZONTAL GRID: */

	for ( i_y = y; i_y <= height; i_y += host_grid )
	{
		XDrawLine(p_xv->display, p_xv->w_main, p_me->gc_hp, x,i_y, width-1,i_y);
	}


	/*  DRAW VERTICAL GRID: */

	for ( i_x = x; i_x <= width; i_x += host_grid )
	{
		XDrawLine(p_xv->display, p_xv->w_main,  p_me->gc_hp,i_x,y,i_x,height-1);
	}
}

/*---------------------------------------------------------------------------*
 |"draw_xv"
 |
 |  Create and return a 32-bit image which contains a grid to be placed into
 |  an Xv-selected drawable via XPutImage.
 |
 |    -> image on success
 |    =  0 if error detected
 *---------------------------------------------------------------------------*/
XImage* draw_xv(display, visual, width, height, startx, starty, endx, endy,
                        incrx, incry, back, grid)
    int     width;      /* =  width of XImage to create. */
    int     height;     /* =  height of XImage to create. */
    int     startx;     /* =  starting x within image for 1st grid line. */
    int     starty;     /* =  starting y within image for 1st grid line. */
    int     endx;       /* =  ending x within image for last grid line. */
    int     endy;       /* =  ending y within image for last grid line. */
    int     incrx;      /* =  increment between grid lines in x direction. */
    int     incry;      /* =  increment between grid lines in y direction. */
    uint    back;       /* =  background color for XImage. */
    uint    grid;       /* =  color of grid lines. */
{
    int     i_col;      /* Column now processing. */
    uint   *im_data;   /* Data for image. */
    uint   *im_pixel;  /* Pixel within im_data accessing. */
    int     i_pixel;    /* Number of pixels in im_data left to process. */
    int     i_row;      /* Row now processing. */

 
    /*
     *  ALLOCATE STORAGE FOR THE DATA AND CREATE THE GRID
     *  (1) Clear the background.
     *  (2) Generate the horizontal lines (including one at height-1)
     *  (3) Generate the vertical lines (including one at width-1)
     */
    if ( (im_data = (u_int*)malloc(width*height*4)) == 0 ) return 0;
     
    for ( i_pixel = width*height, im_pixel = im_data; i_pixel; i_pixel-- )
    {
        *im_pixel++ = back;
    }
       
    for ( i_row = starty; i_row <= endy; i_row += incry )
    {  
        for ( i_col = startx; i_col < endx; i_col++ )
        {
            im_data[i_row*width+i_col] = grid;
        }
    }
 
    for ( i_col = startx; i_col <= endx; i_col += incrx )
    {
        for ( i_row = starty; i_row < endy; i_row++ )
        {
            im_data[i_row*width+i_col] = grid;
        }
    }
 
    /*
     *  CREATE THE XIMAGE AND RETURN IT TO OUR CALLER
     */
    return XCreateImage(display, visual, 32, ZPixmap, 
		0, im_data, width, height, 32, 0);
}




/*---------------------------------------------------------------------------*
 |"load_settings"
 |
 |    = 1 on success, 0 on failure.
 *---------------------------------------------------------------------------*/
int load_settings( w, p_me, file_name )
	Widget	w;			/* =  parent widget for dialog boxes. */
	Calib*	p_me;		/* -> context for this application. */
	char*	file_name;	/* -> name of file to load from. */
{
	PV_Result	bin_value;		/* 'value' after being parsed. */
	int			delay;			/* Delay value. */
	int			disp_flags;		/* Display flags value. */
	char		err_msg[256];	/* Error message. */
	FILE*		fd;				/* File descriptor for opened 'file_name'. */
	int			h_bp;			/* Horizontal back porch value. */
	int			h_fp;			/* Horizontal front porch value. */
	int			h_sy;			/* Horizontal sync value. */
	int			h_disp;			/* Horizontal visible pixels. */
	char		item[128];		/* Name of item to be set. */
	int			loop;			/* Loop escape control variable. */
	int			pixel_clock;	/* Pixel clock value. */
	int			skip;			/* Skip value. */
	int			v_bp;			/* Vertical back porch value. */
	int			v_fp;			/* Vertical front porch value. */
	int			v_sy;			/* Vertical sync value. */
	int			v_disp;			/* Vertical visible pixels. */
	char		value[128];		/* Unparsed value for 'item'. */
	int			xsize;			/* X size of screen. */
	int			ysize;			/* Y size of screen. */
	int			zoom_delay;		/* Zoom: horizontal delay value. */
	int			zoom_h_bp;		/* Zoom: horizontal back porch value. */
	int			zoom_h_fp;		/* Zoom: horizontal front porch value. */
	int			zoom_h_sy;		/* Zoom: horizontal sync value. */
	int			zoom_h_disp;	/* Zoom: horizontal visible pixels. */
	int			zoom_skip;		/* Zoom: horizontal skip value. */

	if ( !(fd = fopen(file_name, "r")) )
	{
		sprintf(err_msg, "Unable to open %s for read", file_name);
		create_dialog(w, "Calibrate", err_msg, 0);
		return 0;
	}

	delay = disp_flags = h_bp = h_fp = h_sy = h_disp = skip = -1;
	zoom_delay = zoom_h_bp = zoom_h_fp = zoom_h_sy = zoom_h_disp = zoom_skip=-1;
	v_bp = v_fp = v_sy = v_disp = -1;
	pixel_clock = xsize = ysize = -1;
	for (;;)
	{
		if ( fscanf(fd, "%s", item) != 1 ) break;
		if ( item[0] == '#' ) 
		{
			for (loop = 1; loop; )
			{
				switch( fgetc(fd) )
				{
				  case EOF:
				  case '\n':
					loop = 0;
				}
			}
		}
		else 
		{
			if ( fscanf(fd, "%s", value) != 1 )
			{
				sprintf(err_msg, "No value present for %s", item);
				create_dialog(w, "Calibrate", err_msg, 0);
				return 0;
			}
			switch( p_value( value, &bin_value) )
			{
			  default:
				sprintf(err_msg, "%s is not a legal value for %s", value, item);
				create_dialog(w, "Calibrate", err_msg, 0);
				return 0;
			  case PV_Integer:
				break;
			}
			if      ( !strcmp(item, "delay") )	delay = bin_value.i;
			else if ( !strcmp(item, "DispFlgs") ) disp_flags = bin_value.i;
			else if ( !strcmp(item, "h.bp") )	h_bp = bin_value.i;
			else if ( !strcmp(item, "h.fp") )	h_fp = bin_value.i;
			else if ( !strcmp(item, "h.sy") )	h_sy = bin_value.i;
			else if ( !strcmp(item, "h.disp") )	h_disp = bin_value.i;
			else if ( !strcmp(item, "pixel_clock") ) pixel_clock = bin_value.i;
			else if ( !strcmp(item, "skip") )	skip = bin_value.i;
			else if ( !strcmp(item, "v.bp") )	v_bp = bin_value.i;
			else if ( !strcmp(item, "v.fp") )	v_fp = bin_value.i;
			else if ( !strcmp(item, "v.sy") )	v_sy = bin_value.i;
			else if ( !strcmp(item, "v.disp") )	v_disp = bin_value.i;
			else if ( !strcmp(item, "xsize") )	xsize = bin_value.i;
			else if ( !strcmp(item, "ysize") )	ysize = bin_value.i;
			else if ( !strcmp(item, "zoom[0].delay") )	
				zoom_delay = bin_value.i;
			else if ( !strcmp(item, "zoom[0].h.bp") )	
				zoom_h_bp = bin_value.i;
			else if ( !strcmp(item, "zoom[0].h.fp") )	
				zoom_h_fp = bin_value.i;
			else if ( !strcmp(item, "zoom[0].h.sy") )	
				zoom_h_sy = bin_value.i;
			else if ( !strcmp(item, "zoom[0].h.disp") )	
				zoom_h_disp = bin_value.i;
			else if ( !strcmp(item, "zoom[0].skip") )	
				zoom_skip = bin_value.i;
			else
			{
				sprintf(err_msg, "%s is not a legal property", item);
				create_dialog(w, "Calibrate", err_msg, 0);
				return 0;
			}
		}
	}

	if      ( delay == -1 )
	{
		create_dialog(w, "Calibrate", "No value given for delay", 0);
		return 0;
	}
	if      ( h_bp == -1 )
	{
		create_dialog(w, "Calibrate", "No value given for h.bp", 0);
		return 0;
	}
	if      ( h_fp == -1 )
	{
		create_dialog(w, "Calibrate", "No value given for h.fp", 0);
		return 0;
	}
	if      ( h_sy == -1 )
	{
		create_dialog(w, "Calibrate", "No value given for h.sy", 0);
		return 0;
	}
	if      ( h_disp == -1 )
	{
		create_dialog(w, "Calibrate", "No value given for h.disp", 0);
		return 0;
	}
	if      ( pixel_clock == -1 )
	{
		create_dialog(w, "Calibrate", "No value given for pixel_clock", 0);
		return 0;
	}
	if      ( skip == -1 )
	{
		create_dialog(w, "Calibrate", "No value given for skip", 0);
		return 0;
	}
	if      ( v_bp == -1 )
	{
		create_dialog(w, "Calibrate", "No value given for v.bp", 0);
		return 0;
	}
	if      ( v_fp == -1 )
	{
		create_dialog(w, "Calibrate", "No value given for v.fp", 0);
		return 0;
	}
	if      ( v_sy == -1 )
	{
		create_dialog(w, "Calibrate", "No value given for v.sy", 0);
		return 0;
	}
	if      ( v_disp == -1 )
	{
		create_dialog(w, "Calibrate", "No value given for v.disp", 0);
		return 0;
	}
	if      ( xsize == -1 )
	{
		create_dialog(w, "Calibrate", "No value given for xsize", 0);
		return 0;
	}
	if      ( xsize == -1 )
	{
		create_dialog(w, "Calibrate", "No value given for ysize", 0);
		return 0;
	}
	if      ( zoom_delay == -1 )
	{
		create_dialog(w, "Calibrate", "No value given for zoom[0].delay", 0);
		return 0;
	}
	if      ( zoom_h_bp == -1 )
	{
		create_dialog(w, "Calibrate", "No value given for zoom[0].h.bp", 0);
		return 0;
	}
	if      ( zoom_h_fp == -1 )
	{
		create_dialog(w, "Calibrate", "No value given for zoom[0].h.fp", 0);
		return 0;
	}
	if      ( zoom_h_sy == -1 )
	{
		create_dialog(w, "Calibrate", "No value given for zoom[0].h.sy", 0);
		return 0;
	}
	if      ( zoom_h_disp == -1 )
	{
		create_dialog(w, "Calibrate", "No value given for zoom[0].h.disp", 0);
		return 0;
	}
	if      ( zoom_skip == -1 )
	{
		create_dialog(w, "Calibrate", "No value given for zoom[0].skip", 0);
		return 0;
	}

	if ( disp_flags != -1 ) p_me->p_main->DispFlgs = disp_flags;

	p_me->p_main->delay        = delay;
	p_me->p_main->h.bp         = h_bp;
	p_me->p_main->h.fp         = h_fp;
	p_me->p_main->h.sy         = h_sy;
	p_me->p_main->h.disp       = h_disp;
	p_me->p_main->pixel_clock  = pixel_clock;
	p_me->p_main->skip         = skip;
	p_me->p_main->v.bp         = v_bp;
	p_me->p_main->v.fp         = v_fp;
	p_me->p_main->v.sy         = v_sy;
	p_me->p_main->v.disp       = v_disp;
	p_me->p_main->xsize        = xsize;
	p_me->p_main->ysize        = ysize;
	p_me->p_zoom->delay        = zoom_delay;
	p_me->p_zoom->h.bp         = zoom_h_bp;
	p_me->p_zoom->h.fp         = zoom_h_fp;
	p_me->p_zoom->h.sy         = zoom_h_sy;
	p_me->p_zoom->h.disp       = zoom_h_disp;
	p_me->p_zoom->skip         = zoom_skip;

	return 1;
}

usage()
{
    fprintf(stderr, "usage: Calibrate [-hv]\n");
    exit(1);
}

/*---------------------------------------------------------------------------*
 |"main"
 |
 *---------------------------------------------------------------------------*/
main(argc, argv)
	int  argc; 
	char* argv[];
{
	XColor	 	 	exact_color;	 /* Exact RGB for color. */
	XFontStruct* 	font_list;		 /* Font using for menus. */
	XFontStruct* 	font1;			 /* System 17 font. */
	XFontStruct* 	font2;			 /* User 7x17 font. */
	XGCValues	 	gc_attr;		 /* Attributes for creating gc. */
	int			 	i_adaptor;		 /* Adaptor now examining. */
	Display* 	 	p_display;	 	 /* Display being used. */
	O_XV*		 	p_xv;       	 /* Xv access object for port. */
	XColor	 	 	scrn_color;      /* Closest screen RGB to color. */	
	Colormap 	 	video_cmap;      /* Color map for video window. */

#include "menu.h"


	/*
     * INTIALIZE OUR Xt LIBRARY APPLICATION:
     */
	w_toplevel = XtAppInitialize(&appContext,
		"winXt",
		NULL,
		(Cardinal) NULL,
		&argc,
		argv,
		NULL,
		NULL,
	NULL);

    if(argc > 2)
		usage();

    if(argc == 2)
    {
	    if(strcmp(argv[1], "-hv"))
		    usage();
	    hvkeys = 1;
    }

	p_display = XtDisplay(w_toplevel);


	/*  CREATE THE XV VIDEO SELECTION OBJECT AND SET OUR WINDOW GEOMETRY.  */

	if ( !(me.p_select = O_XV_Select_O_XV_Select( 0, XtDisplay(w_toplevel) )) )
	{
        fprintf(stderr, "\nCould find an Xv adaptor\n");
        return 0;
    }
	if ( !(me.n_xvs = O_XV_Select_Query_Type(me.p_select, "PGItv2")) )
	{
        fprintf(stderr, "\nCould find a Parallax Xv adaptor\n");
        return 0;
    }
	set_xv_adaptor(&me, 0);
	p_adaptors_list = build_adaptors_list(me.n_xvs);
	me.p_rsrc = (O_Rsrc*)O_Rsrc_O_Rsrc();
#if 1 /* def NOTDEF  */
    O_Rsrc_defaults(me.p_rsrc, 0);
	O_Rsrc_store_to_context(me.p_rsrc, &me.xv_struct);
#else
	O_Rsrc_load_from_context(me.p_rsrc, &me.xv_struct);
#endif
	win_width = WIN_WIDTH;
	win_height = me.p_xv->p_encodings->height;
	xv_width = XV_WIDTH;
	xv_height = XV_HEIGHT;
	xv_grid = XV_GRID;
	switch ( me.zoom )
	{
	  case 2:  host_grid = HOST_GRID*2; break;
	  default: host_grid = HOST_GRID;   break;
	}
	previous_xsize = me.p_main->xsize;
	previous_ysize = me.p_main->ysize;

	/* 
     *  DETERMINE MENU FONT.
     */
	font1 = XLoadQueryFont(p_display, "system17");
	font2 = XLoadQueryFont(p_display, "user7x17");
    if (font1)
    {
        font_list = (XFontStruct*)XmFontListCreate(font1,"charset1");
        if (font2)
            font_list = (XFontStruct*)XmFontListAdd(font_list,font2,"charset2");
    }
    else if (font2)
    {
		font_list = (XFontStruct*)XmFontListCreate(font2,"charset2");
    }
    else
    {   
        font_list = 0;
    }


	/*
     *  CREATE THE FORM WHICH WILL HOLD MENU BAR AND VIDEO AREA:
     *  THEN CREATE THE MENU AND VIDEO AREAS WITHIN THAT FORM.
     */
    w_form = XtVaCreateManagedWidget("main_window",
		xmPanedWindowWidgetClass,   w_toplevel,
		XmNwidth,                   WIN_WIDTH,
		XmNheight,                  WIN_HEIGHT+120,
		XmNseparatorOn,         	False,
		XmNspacing,					0,
		XmNmarginHeight,        	0,
		XmNmarginWidth,         	0,
    NULL);

	/* Create the menu from the description.  (in menus.h) */

	build_static_menu_buttons(NULL, font_list, build_menu_bar(w_form), 
		MenuBar, XtNumber(MenuBar));

	video_cmap = XDefaultColormap( p_display, XDefaultScreen(p_display));
	XAllocNamedColor(p_display, video_cmap, "black", &scrn_color, &exact_color);
    w_video = XtVaCreateManagedWidget("drawing_a",
        xmDrawingAreaWidgetClass,   w_form,
        XmNwidth,               	win_width,
        XmNheight,              	win_height,
		XmNbackground,				scrn_color.pixel,
        XmNcolormap,                video_cmap,
    NULL);
	XtAddCallback(w_video, XmNexposeCallback, cb_expose, &me);
	XtAddEventHandler(w_video, ButtonPressMask | KeyPressMask,
		False, cb_operator, &me);
    XtAddEventHandler(w_toplevel, StructureNotifyMask, False, cb_expose, &me);



	/*
	 *  ASSOCIATE XV ACCESS OBJECT WITH THE DRAWABLE WE WILL USE.
	 */
	sash_traversal_off(w_form);
	XtRealizeWidget(w_toplevel);
    O_XV_Use_Window( me.p_xv, XtWindow(w_video) );
#ifdef _AIX
	O_XV_Video_On(me.p_xv);
	O_XV_Video_Off(me.p_xv);
#endif /* _AIX */


	/*  SET UP THE GRIDS:
	 *
	 *  (1) Create the graphics context for writing to the HP frame buffer and
	 *      draw the grid there.
	 *  (2) Create the image to write into XV memory and write it their.
	 */
	XAllocNamedColor(p_display, video_cmap, "black", &scrn_color, &exact_color);
	gc_attr.foreground = scrn_color.pixel;
	me.gc_black=XCreateGC(p_display, XtWindow(w_video), GCForeground, &gc_attr);
	XAllocNamedColor(p_display, video_cmap, "green", &scrn_color, &exact_color);
	gc_attr.foreground = scrn_color.pixel;
	me.gc_hp = XCreateGC(p_display, XtWindow(w_video), GCForeground, &gc_attr);
	draw_host(&me, host_grid/2, host_grid/2, win_width-host_grid, 
		win_height-host_grid);

	me.p_image = draw_xv(p_display, 
		XDefaultVisual(p_display, XDefaultScreen(p_display)), 
		xv_width, xv_height,
		xv_grid/2, xv_grid/2, xv_width-(xv_grid/2), xv_height-(xv_grid/2), 
		xv_grid, xv_grid, 0x00000000, 0x00ff00ff);
#ifdef _AIX
	XPutImage(me.p_xv->display, me.p_xv->w_main, me.gc_hp, 
		me.p_image, 0, 0, 0, 0, xv_width, xv_height);
#else 
#if 0
	cb_expose(0, &me, 0);
#else
	refreshmain();
#endif
#endif

	set_title( &me );

	XtAppMainLoop(appContext);
}

/*---------------------------------------------------------------------------*
 |"save_settings"
 |
 |    = 1 on success, 0 on failure.
 *---------------------------------------------------------------------------*/
int save_settings( w, p_me, file_name )
    Widget  w;          /* =  parent widget for dialog boxes. */
    Calib*  p_me;       /* -> context for this application. */
    char*   file_name;  /* -> name of file to save to. */
{
    char        err_msg[256];   /* Error message. */
    FILE*       fd;             /* File descriptor for opened 'file_name'. */
 
    if ( !(fd = fopen(file_name, "w")) )
    {
        sprintf(err_msg, "Unable to open %s for write", file_name);
        create_dialog(w, "Calibrate", err_msg, 0);
        return 0;
    }

	fprintf(fd, "# %s\n", file_name);
	fprintf(fd, "delay  %d\n", p_me->p_main->delay);
	fprintf(fd, "h.bp   %d\n", p_me->p_main->h.bp);
	fprintf(fd, "h.fp   %d\n", p_me->p_main->h.fp);
	fprintf(fd, "h.sy   %d\n", p_me->p_main->h.sy);
	fprintf(fd, "h.disp %d\n", p_me->p_main->h.disp);
	fprintf(fd, "v.bp   %d\n", p_me->p_main->v.bp);
	fprintf(fd, "v.fp   %d\n", p_me->p_main->v.fp);
	fprintf(fd, "v.sy   %d\n", p_me->p_main->v.sy);
	fprintf(fd, "v.disp %d\n", p_me->p_main->v.disp);
	fprintf(fd, "zoom[0].delay  %d\n", p_me->p_zoom->delay);
	fprintf(fd, "zoom[0].h.bp   %d\n", p_me->p_zoom->h.bp);
	fprintf(fd, "zoom[0].h.fp   %d\n", p_me->p_zoom->h.fp);
	fprintf(fd, "zoom[0].h.sy   %d\n", p_me->p_zoom->h.sy);
	fprintf(fd, "zoom[0].h.disp %d\n", p_me->p_zoom->h.disp);

	fclose(fd);
	return 1;
}


/*---------------------------------------------------------------------------*
 |"send_to_server"
 *---------------------------------------------------------------------------*/
void send_to_server( p_me )
	Calib*	p_me;		/* -> calibration object. */
{
	XvSetLongPortAttribute(p_me->p_xv->display, p_me->p_xv->port_id, 
		p_me->a_nvram_z, p_me->n_zoom, p_me->p_zoom);
	XvSetLongPortAttribute(p_me->p_xv->display, p_me->p_xv->port_id, 
		p_me->a_nvram_m, p_me->n_main, p_me->p_main);
	if ( p_me->ysize != p_me->p_main->ysize )
	{
		p_me->ysize = p_me->p_main->ysize;
		XvSetPortAttribute(p_me->p_xv->display, p_me->p_xv->port_id, 
			p_me->a_nvram_c, XV_PLX_NVRAM_INIT_FROM);
#if 0
		cb_expose(0, p_me, 0);
#else
		refreshmain();
#endif
	}
	else
	{
		XvSetPortAttribute(p_me->p_xv->display, p_me->p_xv->port_id, 
			p_me->a_nvram_c, XV_PLX_NVRAM_INIT_DISP);
#if 0
		refreshmain();
#endif
	}
}


/*---------------------------------------------------------------------------*
 |"set_title"
 |
 |   Set up title bar to reflect current values.
 *---------------------------------------------------------------------------*/
void set_title( p_me )
	Calib*	p_me;		/* -> calibration object. */
{
	char			title[256];		 /* Storage to build new title in. */

	switch(p_me->zoom)
	{
	  case 2:
	    sprintf(title, "hfp:%d  hbp:%d  hsy:%d  delay:%d  skip:%d\n",
			p_me->p_zoom->h.fp, p_me->p_zoom->h.bp, p_me->p_zoom->h.sy,
			p_me->p_zoom->delay, p_me->p_zoom->skip);
		break;
	  default:
	    sprintf(title, 
		"vfp:%d  vbp:%d  vsy:%d  hfp:%d  hbp:%d  hsy:%d  delay:%d  skip:%d\n",
			p_me->p_main->v.fp, p_me->p_main->v.bp, p_me->p_main->v.sy,
			p_me->p_main->h.fp, p_me->p_main->h.bp, p_me->p_main->h.sy,
			p_me->p_main->delay, p_me->p_main->skip);
		break;
	}
    XtVaSetValues(w_toplevel, XmNtitle, title, NULL);
    XFlush(p_me->p_xv->display);
}


/*---------------------------------------------------------------------------*
 |"set_xv_adaptor"
 |
 |   Notes:
 |     (1) Note we use the p_zoom and p_main fields of the Calib structure
 |         to get strings from the X server. We then set them to point to
 |         the appropriate place in our xv_struct storage.
 *---------------------------------------------------------------------------*/
int set_xv_adaptor(p_me, i_adaptor)
	Calib*	p_me;		/* -> calibration object. */
	int		i_adaptor;	/* =  # of adaptor to calibrate. */
{
	p_me->p_xv = O_XV_Select_Port(p_me->p_select, "PGItv2", i_adaptor, 0);
	sprintf(p_me->fname, "%s/%s", PREFIX, 
		&adaptor_labels[i_adaptor][strlen("/dev/")]);


#ifndef _AIX
	/*  CREATE IMAGE MANIPULATION ATOMS. */

	p_me->a_ximage = XInternAtom(p_me->p_xv->display, "XV_IMAGE_XIMAGE",
		False);
#endif /* !_AIX */


	/*  CREATE ZOOM FACTOR ATOM AND GET CURRENT ZOOM FACTOR. */

	p_me->a_zoom = XInternAtom(p_me->p_xv->display, "XV_PLX_ZOOM", False);
	XvGetPortAttribute(p_me->p_xv->display, p_me->p_xv->port_id, p_me->a_zoom, 
		&p_me->zoom);


	/*  CREATE THE NVRAM ACCESS ATOMS AND READ IN THE CURRENT NVRAM VALUES. */

	p_me->a_nvram_c = XInternAtom(p_me->p_xv->display, "XV_PLX_NVRAM_CONTROL", 
		False);
	p_me->a_nvram_m = XInternAtom(p_me->p_xv->display, "XV_PLX_NVRAM_MAIN", 
		False);
	p_me->a_nvram_z = XInternAtom(p_me->p_xv->display, "XV_PLX_NVRAM_ZOOM", 
		False);


	/*  GET CURRENT ZOOM VALUES FROM THE X SERVER. */

	XvGetLongPortAttribute(p_me->p_xv->display, p_me->p_xv->port_id, 
		p_me->a_nvram_z, &p_me->n_zoom, &p_me->p_zoom);
	if ( p_me->n_zoom != sizeof(XV_zoom_nvram) )
	{
		fprintf(stderr, 
			"Expected %d bytes for zoom nvram structure, got %d\n",
			sizeof(XV_zoom_nvram), p_me->n_zoom);
		exit(0);
	}
	memcpy(&p_me->xv_struct.zoom, p_me->p_zoom, p_me->n_zoom);
	free(p_me->p_zoom);
	p_me->p_zoom = (XV_zoom_nvram*)&p_me->xv_struct.zoom;


	/*  GET CURRENT NORMAL VALUES FROM THE X SERVER. */

	XvGetLongPortAttribute(p_me->p_xv->display, p_me->p_xv->port_id, 
		p_me->a_nvram_m, &p_me->n_main, &p_me->p_main);
	if ( p_me->n_main != sizeof(XV_controller_nvram) )
	{
		fprintf(stderr, 
			"Expected %d bytes for main nvram structure, got %d\n",
			sizeof(XV_controller_nvram), p_me->n_main);
		exit(0);
	}
	memcpy(&p_me->xv_struct.main, p_me->p_main, p_me->n_main);
	free(p_me->p_main);
	p_me->p_main = &p_me->xv_struct.main;

}
