/*
 *  O_XV.h:	Declaration of Xv video extension access object.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     26 May 93 - Andy Goodrich
 *       - Initial coding
 *
 *  Copyright(c) 1993, Andrew C. Goodrich
 *
 * $Log:   /plxhp/dave/pvcsrc/O_XV/archives/O_XV.h_v  $
 * 
 *    Rev 1.4   09 Sep 1996 09:51:38   dave
 * Andy Goodrich: Changed O_XV_Status_Video_In and friends to use defines
 *                since the AIX compiler wants signed enumerators!
 * Revision 1.4  1995/08/31 13:05:03  dist
 * Andy Goodrich: Changed O_XV_Status_Video_In and friends to use defines
 *                since the AIX compiler wants signed enumerators!
 *
 * Revision 1.3  95/08/31  12:50:56  12:50:56  dist ()
 * Andy Goodrich: Added output support and augmentation of status
 *                to include current state of digitizers and encoder.
 * 
 * Revision 1.2  95/08/29  15:31:25  15:31:25  dist ()
 * Andy Goodrich: Added terminate method to O_XV and created O_XV_Select
 *                object.
 * 
 * Revision 1.1  95/05/16  10:10:01  10:10:01  dist ()
 * Initial revision
 * 
 */
#ifndef  O_XV_INCLUDED
#define O_XV_INCLUDED

typedef struct Tv_XV {
	Window	(*create_window)();	/* Open an xv window for a port. */	
	void	(*dump)();			/* Dump object's contents. */
	int    	(*get_attribute)();	/* Get a port attribute. */
	int    	(*set_attribute)();	/* Set a port attribute. */
	int		(*source)();		/* Set up video source information. */
	int		(*status)();		/* Return status of a port. */
	void	(*terminate)();		/* Terminate use of an object. */
	void 	(*use_window)();	/* Specify a window to use with a port. */	
	void	(*video_off)();		/* Turn off video in the window. */
	void	(*video_on)();		/* Turn on video in the window. */
	void	(*video_output)();	/* Turn on video output from the window. */
} Tv_XV;

typedef struct O_XV {
	Tv_XV			*tv;			/* Methods transfer vector. */
	Atom			a_brightness;	/* Atom for Xv brightness attribute. */
	Atom			a_contrast;		/* Atom for Xv contrast attribute. */
	Atom			a_channel;		/* Atom for channel affinity attribute. */
	Atom			a_encoding;		/* Atom for Xv encoding attribute. */
	Atom			a_hue;			/* Atom for Xv hue attribute. */
	Atom			a_saturation;	/* Atom for Xv saturation attribute. */
	Atom			a_status;		/* Atom for Xv status attribute. */
	Atom			a_zoom;			/* Atom for Xv zoom factor attribute. */
	Colormap		color_map;		/* Color map using for our window. */
	Display			*display;		/* Display xv window is on. */
	GC				gc;				/* Graphics context for 'w_main'. */
	int				i_adaptor;		/* # of adaptor this object is for. */
	int				i_port;			/* # of port this object is for. */
	int				n_adaptors;		/* # of adaptors 'p_adaptors' points to. */
	int				n_encodings;	/* # of encodings 'p_encodings' points to */
	XvAdaptorInfo	*p_adaptor;		/* Information on adaptors we are using. */
	XvAdaptorInfo	*p_adaptors;	/* Information on adaptors for 'dpy'. */
	XvEncodingInfo	*p_encodings;	/* Encoding information for port using. */
	Visual			*p_visual;		/* Information on our window's visual. */
	int				port_id;		/* Port identifier of this object's port. */
	int				screen;			/* Default screen for display 'dpy'. */
	int				sig_height;     /* Video signal width. */
	int				sig_width;      /* Video signal height. */
	int				video_in_on;	/* = 1 if port is performing video in. */
	int				video_out_on;	/* = 1 if port is performing video out. */
	int				win_height;     /* Main window height. */
	int				win_width;      /* Main window width. */
	Window			w_main;			/* Main window for this application. */
	Window			w_root;			/* Root window for display 'dpy'. */
	u_int			xv_error_base;	/* Base for error messages. */
	u_int			xv_event_base;	/* Base for event ids. */
	u_int			xv_major_op;	/* Major opcode for this version of xv. */
	u_int 			xv_revision;	/* Revision within version of xv. */
	u_int 			xv_version;		/* Version of xv being used. */
} O_XV;

O_XV *O_XV_O_XV();

#define O_XV_Dump(obj, fd) \
	(* ((O_XV *)(obj))->tv->dump)(obj, fd)

#define O_XV_Source(obj, dmsd, timing, encoding, instance) \
	(* ((O_XV *)(obj))->tv->source)(obj, dmsd, timing, encoding, instance)

#define O_XV_Get_Attribute(obj, atom, value) \
	(* ((O_XV *)(obj))->tv->get_attribute)(obj, atom, value)

#define O_XV_Set_Attribute(obj, atom, value) \
	(* ((O_XV *)(obj))->tv->set_attribute)(obj, atom, value)

#define O_XV_Status(obj) \
	(* ((O_XV *)(obj))->tv->status)(obj)

/* Status bits added to server XV status: */

#define	O_XV_Status_Video_In  0x80000000
#define	O_XV_Status_Video_Out 0x40000000

#define O_XV_Terminate(obj) \
	(* ((O_XV *)(obj))->tv->terminate)(obj)

#define O_XV_Use_Window(obj, window) \
	(*((O_XV *)(obj))->tv->use_window)(obj, window)

#define O_XV_Video_Off(obj) \
	(* ((O_XV *)(obj))->tv->video_off)(obj)

#define O_XV_Video_On(obj) \
	(* ((O_XV *)(obj))->tv->video_on)(obj)

#define O_XV_Video_Output(obj) \
	(* ((O_XV *)(obj))->tv->video_output)(obj)

#define O_XV_Window(obj, vis_type, depth) \
	(*((O_XV *)(obj))->tv->create_window)(obj, vis_type, depth)


/*
 *   O_XV_Select Object Definitions
 */

typedef struct _O_XV_Sel_Adapt {
	O_XV**	p_ports;
} O_XV_Sel_Adapt;

typedef struct Tv_XV_Select {
	void	(*dump)();			/* Dump object's contents. */
	int		(*query_type)();	/* Get number of adaptors of a given type. */
	O_XV*	(*select_port)();	/* Select a port from an adaptor. */
	void	(*terminate)();		/* Terminate use of an object. */
} Tv_XV_Select;

typedef struct O_XV_Select {
	Tv_XV_Select* 	tv;			    /* Methods transfer vector. */
	Display*		p_display;		/* Display xv window is on. */
	int				n_adaptors;		/* # of adaptors 'p_adaptors' points to. */
	int				n_encodings;	/* # of encodings 'p_encodings' points to */
	XvAdaptorInfo*	p_adaptor;		/* Information on adaptors we are using. */
	XvAdaptorInfo*	p_adaptors;		/* Information on adaptors for 'dpy'. */
	XvEncodingInfo*	p_encodings;	/* Encoding information for port using. */
	O_XV_Sel_Adapt* p_oxv_adaptors;	/* */
	int				screen;			/* Default screen for display 'dpy'. */
	Window			w_root;			/* Root window for display 'dpy'. */
	u_int			xv_error_base;	/* Base for error messages. */
	u_int			xv_event_base;	/* Base for event ids. */
	u_int			xv_major_op;	/* Major opcode for this version of xv. */
	u_int 			xv_revision;	/* Revision within version of xv. */
	u_int 			xv_version;		/* Version of xv being used. */
} O_XV_Select;

O_XV_Select *O_XV_Select_O_XV_Select();

#define O_XV_Select_Dump(obj, fd) \
	(* ((O_XV_Select *)(obj))->tv->dump)(obj, fd)

#define O_XV_Select_Port(obj, type, i_adaptor, i_port) \
	(* ((O_XV_Select *)(obj))->tv->select_port)(obj, type, i_adaptor, i_port)

#define O_XV_Select_Query_Type(obj, type) \
	(* ((O_XV_Select *)(obj))->tv->query_type)(obj, type)

#define O_XV_Select_Terminate(obj) \
	(* ((O_XV_Select *)(obj))->tv->terminate)(obj)

#endif /* O_XV_INCLUDED */
