/*
 *  O_XV_PGI.c:	Implementation of Parallax Xv video access object.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     19 Apr 95 - Andy Goodrich
 *       - Initial coding from O_XV.c
 *
 * $Log:   /plxhp/dave/pvcsrc/O_XV/archives/O_XV_PGI.c_v  $
 * 
 *    Rev 1.1   09 Sep 1996 09:51:50   dave
 * Initial revision
 * Revision 1.1  1995/05/16 10:10:10  dist
 * Initial revision
 *
 *
 *  Copyright(c) 1995, Parallax Graphics, Inc.
 *
 *     Copyright(c) 1993, Andy Goodrich
 *     Permission to use granted
 *
 */
#include <sys/types.h>			/* Standard types for this C environment. */
#include <stdio.h>				/* Stream input/output declarations. */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <Xvlib.h>
#include <Xm/DialogS.h>
#include <Xm/PushB.h>
#include <Xm/CascadeB.h>
#include <Xm/RowColumn.h>
#include <Xm/ToggleB.h>
#include <Xm/Label.h>
#include <Xm/Form.h>
#include <Xm/PanedW.h>
#include <Xm/PushBG.h>
#include <Xm/DrawingA.h>
#include <Xm/SashP.h>
#include "Xvddx_PGI.h"
#include "O_XV_PGI.h"


/*----------------------------------------------------------------------------*
 |"method_xv_pgi_dump"
 |
 |  Dump the contents of this object to a file descriptor.
 |
 |
 *----------------------------------------------------------------------------*/
static void method_xv_pgi_dump( p_me, fd )
	O_XV_PGI*	p_me;	/* -> object instance using. */
	FILE*		fd;		/* -> file descriptor to dump to. */
{
	fprintf(fd, "\nDump of xv pgi access object at address %08x\n", p_me);
	O_XV_Dump(p_me, fd);
}


/*---------------------------------------------------------------------------*
 |"method_xv_pgi_input_menu"
 |
 |  Create a cascading input menu for the adaptors connected to this object.
 |
 |    = number of signals found ( number of menu leafs ) 
 *---------------------------------------------------------------------------*/
static int method_xv_pgi_input_menu( p_me, w, font_list, callback )
	O_XV_PGI*	p_me;			/* -> object instance using. */
	Widget		w;				/* =  widget to be parent of cascading menu. */
	XmFontList  font_list;      /* =  list of fonts to use. */
	void		(*callback)();	/* -> callback routine for menu leaf items. */
{
	int n_leaves;			/* Number of leaf nodes created. */

	
	return n_leaves;
}



/*---------------------------------------------------------------------------*
 |"method_xv_pgi_inputs"
 |
 |  Return a list of input signals available on hardware for this object.
 |
 |    = number of signals found 
 *---------------------------------------------------------------------------*/
static int method_xv_pgi_inputs( p_me, p_inputs )
	O_XV_PGI*			p_me;		/* -> object instance using. */
	XvplxSignalRec**	p_inputs;	/* -> where to store pointer to inputs. */
{
	int n_signals;			/* Number of signals returning in list. */

	XvGetLongPortAttribute(p_me->base.display, p_me->base.port_id, 
		p_me->a_signals, &n_signals, p_inputs );
	return n_signals / sizeof(XvplxSignalRec);
}

/*---------------------------------------------------------------------------*
 |"method_xv_pgi_output_menu"
 |
 |  Create a cascading output menu for the adaptors connected to this object.
 |
 |    = number of signals found ( number of menu leafs ) 
 *---------------------------------------------------------------------------*/
static int method_xv_pgi_output_menu( p_me, w, font_list, callback )
	O_XV_PGI*	p_me;			/* -> object instance using. */
	Widget		w;				/* =  widget to be parent of cascading menu. */
	XmFontList  font_list;      /* =  list of fonts to use. */
	void		(*callback)();	/* -> callback routine for menu leaf items. */
{
	int n_leaves;			/* Number of leaf nodes created. */

	return n_leaves;
}


/*---------------------------------------------------------------------------*
 |"method_xv_pgi_outputs"
 |
 |  Return a list of output signals available on hardware for this object.
 |
 |    = number of signals found 
 *---------------------------------------------------------------------------*/
static int method_xv_pgi_outputs( p_me, p_outputs )
	O_XV_PGI*			p_me;		/* -> object instance using. */
	XvplxSignalRec**	p_outputs;	/* -> where to store pointer to inputs. */
{
	int n_signals;			/* Number of signals returning in list. */

	return n_signals;
}


static Tv_XV_PGI O_XV_PGI_transfer_vector = {
	method_xv_pgi_dump,
	method_xv_pgi_input_menu,
	method_xv_pgi_inputs,
	method_xv_pgi_output_menu,
	method_xv_pgi_outputs
};

/*----------------------------------------------------------------------------*
 |"O_XV_PGI_O_XV_PGI"
 |
 |  Create an Xv video extension access object for a Parallax graphics
 |  compatible adaptor and port of a display.
 |
 |    -> object on success, 0 on failure.
 *----------------------------------------------------------------------------*/
O_XV_PGI *O_XV_PGI_O_XV_PGI( p_me, display, i_adapt, i_port )
	O_XV_PGI* 	p_me;		/* -> storage for object creating, or 0. */
	Display* 	display;	/* -> display to open Xv port on. */
	int		 	i_adapt;	/* =  # of adaptor to use (0..m). */
	int		 	i_port;		/* =  number of port on adaptor to use (0..n). */
{
	int		status;		/* Status from function call. */

	/*
     *  CREATE OUR OBJECT AND FILL IN THE PARAMETERS WHICH WERE PASSED TO US.
	 */
	if ( !p_me )
	{
		if ( !(p_me = (O_XV_PGI *)malloc(sizeof(O_XV_PGI))) )
		{
			fprintf(stderr,"\n O_XV_PGI_O_XV_PGI: couldn't allocate storage\n");
			return 0;
		}
	}
	if ( !O_XV_O_XV(p_me, display, i_adapt, i_port) )
	{
		fprintf(stderr,"\n O_XV_PGI_O_XV_PGI: couldn't initialize O_XV obj \n");
		free((char*)p_me);
		return 0;
	}
	
	p_me->tv = &O_XV_PGI_transfer_vector;
	p_me->a_signals = 
		XInternAtom(p_me->base.display, "XV_PLX_QUERY_SIGNALS", False);

	return p_me;
}

