/*
 *  support.c: Suport code for XV video extension programs.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     06 Aug 94 - Andy Goodrich
 *       - Initial coding
 *
 * $Log:   /plxhp/dave/pvcsrc/O_XV/archives/support.c_v  $
 * 
 *    Rev 1.8   09 Sep 1996 09:54:06   dave
 * Andy Goodrich: Fixed a bug in linking from static menu
 *                data structures to linked menu data structures
 *                in build_static_menu_buttons.
 * Revision 1.8  1995/08/31 12:52:36  dist
 * Andy Goodrich: Fixed a bug in linking from static menu
 *                data structures to linked menu data structures
 *                in build_static_menu_buttons.
 *
 * Revision 1.7  95/08/27  14:06:45  14:06:45  dist ()
 * Andy Goodrich: Moved p_value to its own file.
 *                This eliminated a duplicate symbol when linking with
 *                the hardware library.
 * 
 * Revision 1.6  95/07/26  15:58:18  15:58:18  dist ()
 * Andy Goodrich: Changed include for time.h to use /usr/include/sys/time.h
 * 
 * Revision 1.5  95/07/25  13:00:09  13:00:09  dist ()
 * Andy Goodrich: added sash_traversal_off subroutine.
 * 
 * Revision 1.4  95/07/24  10:51:53  10:51:53  dist ()
 * Andy Goodrich: Freeze for real HP 9.1 release.
 * 
 * Revision 1.3  95/07/12  13:50:51  13:50:51  dist ()
 * Andy Goodrich: Added ability to provide a null linked menu list to build
 *                static menu.
 * 
 * Revision 1.2  95/06/21  17:16:25  17:16:25  dist ()
 * Andy Goodrich: Changes for xmcapture and general reorganization.
 * 
 *
 *  Copyright(c) 1994, Parallax Graphics
 *
 *  Code derived from Xm support: build_menus.c, dialog.c 
 *  Copyright(c) 1993, Andrew C. Goodrich
 *  Used with permission by Parallax Graphics, Inc.
 */
#include <sys/types.h>			/* Standard types for this C environment. */
#include <stdio.h>				/* Stream input/output declarations. */
#include <sys/time.h>
#include <X11/StringDefs.h>
#include <Xm/Xm.h>
#include <Xm/Separator.h>
#include <Xm/PushB.h>
#include <Xm/CascadeB.h>
#include <Xm/RowColumn.h>
#include <Xm/ToggleB.h>
#include <Xm/Label.h>
#include <Xm/SashP.h>
#ifdef NOTDEF
#include <Xm/MessageB.h>
#include <Xm/SelectioB.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <X11/Shell.h>
#endif


#include "support.h"

#define MAX_ARGS 20
extern Widget w_shadow_image;
static Widget w_dialog;
/*  
 *  Private (internal) functions:
 */

/* 
 *  Public functions:
 */
void   build_linked_menu_buttons();
Widget build_menu_bar();
void build_static_menu_buttons();


/*---------------------------------------------------------------------------*
 |"build_linked_menu_buttons"
 |
 |	Internal function to create the buttons in a menu pane given a table 
 |  of menu items.
 *----------------------------------------------------------------------------*/
void build_linked_menu_buttons(title, font_list, menu, menu_list)
   char*           title;		/* -> title for menu or 0. */
   XmFontList      font_list;	/* =  list of fonts to use. */
   Widget          menu;		/* =  widget for menu pane. */
   Menu_Linked_E*  menu_list;	/* -> linked list of menu items. */
{
   WidgetList 	  buttons;			/* List of 'buttons' in the menu. */
   int			  i_button;			/* Entry in 'buttons' now accessing. */
   int        	  i_menu;			/* # of entry in menu_list accessing. */
   int		  	  n_menus;			/* # of menus in this panel. */
   Menu_Linked_E* p_menu;			/* Menu item in menu_list accessing. */
   int        	  separators;       /* # of menu separators (non-buttons.) */
   Widget     	  w_pulldown;       /* Pane for pulldown menu. */
   Arg        	  wargs[2];
   XmString   	  text  ;


   /* ALLOCATE A WIDGET LIST TO HOLD ALL THE MENU'S BUTTON WIDGETS.
    *
    * If a title is given, create Label and Separator widgets.
    */
   for ( n_menus = 0, p_menu = menu_list; p_menu; p_menu = p_menu->p_next )
   {
	   n_menus++;
   }
   buttons = (WidgetList) XtMalloc(n_menus * sizeof(Widget));
   if (title)
   {  
		XtCreateManagedWidget(title, xmLabelWidgetClass, menu, NULL, 0);
       	XtCreateManagedWidget("separator", 
			xmSeparatorWidgetClass, menu, NULL, 
		0);
   }


   /*  CREATE A MENU ENTRY FOR EACH ITEM IN THE MENU TABLE PASSED TO US
	*
	*  There are four cases to be processed:
	*    (1) The button name is NULL: this is a menu separator (non-button)
	*    (2) There is a callback function: build the menu item and register
	*        the callback function.
	*    (3) There is a submenu: create a pulldown menu pane and call 
	*        ourselves recursively.
	*    (4) There is no callback function and no sub_menu: the entry is a
	*        label.
    */
    p_menu = menu_list;
    for( i_menu=0, i_button = -1, separators = 0; i_menu < n_menus; i_menu++)
    {
		if (p_menu->name == NULL)
    	{ 
			XtCreateManagedWidget("separator", 
			    xmSeparatorWidgetClass, menu, NULL, 
			0);
			separators++; 
    	}
	    else if(p_menu->cb_func)
	    {
			if (p_menu->toggle_flag) 
			{
				text = XmStringCreateLtoR(p_menu->name,"charset1") ;
				buttons[++i_button]=XtVaCreateWidget(p_menu->name,
					xmToggleButtonWidgetClass, menu, 
					XmNmnemonic,p_menu->mnemonic,
					XmNmnemonicCharSet,"charset1",
					XmNlabelString,text,
					XmNfontList,font_list,
				NULL);
				if ( p_menu->widget ) 
#ifdef NOTDEF
				if (i_menu == 0) 
#endif
					p_menu->widget  = buttons[i_button];	
				
				XtAddCallback(buttons[i_button], 
					XmNvalueChangedCallback, p_menu->cb_func, p_menu->cb_data);
			}
			else 
			{
				text = XmStringCreateLtoR(p_menu->name,"charset1") ;

				buttons[++i_button] = XtVaCreateManagedWidget(
					p_menu->name,
					xmPushButtonWidgetClass, 	menu,
					XmNmnemonic,				p_menu->mnemonic,
					XmNmnemonicCharSet,			"charset1",
					XmNlabelString,				text,
					XmNfontList,				font_list,
				NULL) ;
				XtAddCallback(buttons[i_button], XmNactivateCallback, 
			  		p_menu->cb_func, p_menu->cb_data);
			}
			if ( p_menu->widget ) 
				*p_menu->widget = buttons[i_button];	
		}
		else if ( p_menu->sub_menu )
		{  
			w_pulldown = XmCreatePulldownMenu(menu, 
				p_menu->sub_menu_title, NULL, 0);
			text = XmStringCreateLtoR(p_menu->name,"charset1") ;
			buttons[++i_button]= XtVaCreateWidget(p_menu->name, 
				xmCascadeButtonWidgetClass, menu,
				XmNsubMenuId, 				w_pulldown,
				XmNmnemonic, 				(p_menu->mnemonic),
				XmNmnemonicCharSet,			"charset1",
				XmNlabelString, 			text,
				XmNfontList,				font_list,
			0);
			if ( p_menu->widget ) 
				*p_menu->widget = buttons[i_button];	
			build_linked_menu_buttons(p_menu->sub_menu_title, font_list,
				w_pulldown, p_menu->sub_menu);
		}
		else
		{
			text = XmStringCreateLtoR(p_menu->name,"charset1") ;
			buttons[++i_button] = XtVaCreateManagedWidget(
				p_menu->name, 
				xmToggleButtonWidgetClass,	menu, 
				XmNmnemonic,             	p_menu->mnemonic,
				XmNmnemonicCharSet,			"charset1",
				XmNlabelString, 			text,
				XmNfontList,				font_list,
			0);
			if ( p_menu->widget ) 
				*p_menu->widget = buttons[i_button];	
		}
	    p_menu = p_menu->p_next;
   } 

	/*
	 *   MANAGE ALL BUTTON WIDGETS, ANY MENU PANES CREATED ARE NOT MANAGED
     */
    XtManageChildren(buttons, ++i_button );
}

/*---------------------------------------------------------------------------*
 |"build_menu_bar"
 |
 |  Create a menu bar widget given a parent window.
 |
 |    = menu bar created
 *----------------------------------------------------------------------------*/
Widget build_menu_bar(parent)
   Widget         parent;       /* = widget to create menu bar for. */
{
    Arg       args[MAX_ARGS];   /* Arguments for calling XmCreateMenuBar. */
    Cardinal  i_args;           /* Index: entry in args to set next. */
    Widget    menu_bar;         /* Menu bar creating. */

    i_args = 0;
    XtSetArg(args[i_args], XmNtopAttachment,   XmATTACH_FORM);   i_args++;
    XtSetArg(args[i_args], XmNleftAttachment,  XmATTACH_FORM);   i_args++;
    XtSetArg(args[i_args], XmNresizePolicy,    XmRESIZE_ANY);    i_args++;
    XtSetArg(args[i_args], XmNrightAttachment, XmATTACH_FORM);   i_args++;
    menu_bar = XmCreateMenuBar(parent, "MenuBar", args, i_args);

    XtManageChild(menu_bar);

    return(menu_bar);
}
#ifdef NOTDEF
/*---------------------------------------------------------------------------*
 |"build_menu_bar"
 |
 |  Create a menu bar widget given a parent window.
 |
 |    = menu bar created 
 *----------------------------------------------------------------------------*/
Widget build_menu_bar(parent, font_list, menu_list, nitems)
   Widget 		  parent;		/* = widget to create menu bar for. */
   XmFontList     font_list;	/* =  list of fonts to use. */
   Menu_Entry*    menu_list;	/* -> list of menu items. */
   int            nitems;		/* =  number of menu items. */
{
    Arg       args[MAX_ARGS];	/* Arguments for calling XmCreateMenuBar. */
    Cardinal  i_args;			/* Index: entry in args to set next. */
    Widget    menu_bar;			/* Menu bar creating. */

    i_args = 0;
    XtSetArg(args[i_args], XmNtopAttachment,   XmATTACH_FORM);   i_args++;
    XtSetArg(args[i_args], XmNleftAttachment,  XmATTACH_FORM);   i_args++;
    XtSetArg(args[i_args], XmNresizePolicy,    XmRESIZE_ANY);    i_args++;
    XtSetArg(args[i_args], XmNrightAttachment, XmATTACH_FORM);   i_args++;
    menu_bar = XmCreateMenuBar(parent, "MenuBar", args, i_args);

    XtManageChild(menu_bar);

     /* Create the menu from the description.  (in menus.h) */

    build_static_menu_buttons(NULL, font_list, menu_bar, menu_list, nitems);

    return(menu_bar);
} 
#endif /* NOTDEF */


/*---------------------------------------------------------------------------*
 |"build_static_menu_buttons"
 |
 |	Internal function to create the buttons in a menu pane given a table 
 |  of menu items.
 *----------------------------------------------------------------------------*/
void build_static_menu_buttons(title, font_list, menu, menu_list, nitems)
   char*           title;		/* -> title for menu or 0. */
   XmFontList      font_list;	/* =  list of fonts to use. */
   Widget          menu;		/* =  widget for menu pane. */
   Menu_Entry      *menu_list;	/* -> list of menu items. */
   int             nitems;		/* =  number of menu items. */
{
	WidgetList 		buttons;	/* List of 'buttons' in the menu. */
    int		   		i_button;	/* Entry in 'buttons' now accessing. */
   	int        		i_menu;		/* Index: entry in menu_list accessing. */
	Menu_Linked_E*	p_link;		/* Head of linked menu entries to do. */
   	int        		separators; /* # of menu separators (non-buttons.) */
   	Widget     		w_pulldown; /* Pane for pulldown menu. */
   	Arg        		wargs[2];
   	XmString   		text  ;

   	/*
   	 * ALLOCATE A WIDGET LIST TO HOLD ALL THE MENU'S BUTTON WIDGETS.
   	 *
   	 * If a title is given, create Label and Separator widgets.
   	 */
   	buttons = (WidgetList) XtMalloc(nitems * sizeof(Widget));
   	if (title)
   	{  
		XtCreateManagedWidget(title, xmLabelWidgetClass, menu, NULL, 0);
       	XtCreateManagedWidget("separator", 
			xmSeparatorWidgetClass, menu, NULL, 
		0);
   	}

   	/*
	 *  CREATE A MENU ENTRY FOR EACH ITEM IN THE MENU TABLE PASSED TO US
	 *
	 *  There are four cases to be processed:
	 *    (1) The button name is NULL: this is a menu separator (non-button)
	 *    (2) There is a callback function: build the menu item and register
	 *        the callback function.
	 *    (3) There is a submenu: create a pulldown menu pane and call 
	 *        ourselves recursively. Note there are two forms of submenu,
	 *        static and linked. Linked menus have a count of zero.
	 *    (4) There is no callback function and no sub_menu: the entry is a
	 *        label.
     */
    for( i_menu=0, i_button = -1, separators = 0; i_menu < nitems; i_menu++)
    {
    	if (menu_list[i_menu].name == NULL)		/* Case 1 */
    	{ 
			XtCreateManagedWidget("separator", 
			    xmSeparatorWidgetClass, menu, NULL, 
			0);
			separators++; 
    	}


		else if(menu_list[i_menu].cb_func)		/* Case 2 */
		{
			if (menu_list[i_menu].toggle_flag) 
			{
				text = XmStringCreateLtoR(menu_list[i_menu].name,"charset1") ;
				buttons[++i_button] = XtVaCreateWidget(
					menu_list[i_menu].name,
					xmToggleButtonWidgetClass, menu, 
					XmNmnemonic,menu_list[i_menu].mnemonic,
					XmNmnemonicCharSet,"charset1",
					XmNlabelString,text,
					XmNfontList,font_list,
				NULL);
				if (i_menu == 0) 
					*menu_list[i_menu].widget  = buttons[i_button];	
				
				XtAddCallback(buttons[i_button], 
					XmNvalueChangedCallback, 
		  			menu_list[i_menu].cb_func, menu_list[i_menu].cb_data);
			}
			else 
			{
				text = XmStringCreateLtoR(menu_list[i_menu].name,"charset1") ;

				buttons[++i_button] = XtVaCreateManagedWidget(
					menu_list[i_menu].name,
					xmPushButtonWidgetClass, 	menu,
					XmNmnemonic,				menu_list[i_menu].mnemonic,
					XmNmnemonicCharSet,			"charset1",
					XmNlabelString,				text,
					XmNfontList,				font_list,
				NULL) ;
				XtAddCallback(buttons[i_button], XmNactivateCallback, 
			  	menu_list[i_menu].cb_func, menu_list[i_menu].cb_data);
			}
			if ( menu_list[i_menu].widget ) 
				*menu_list[i_menu].widget = buttons[i_button];	
		}


		else if ( menu_list[i_menu].sub_menu )    /* Case 3 */
		{  
			if ( menu_list[i_menu].sub_menu_count == 0 && 
				 *(int*)menu_list[i_menu].sub_menu == 0 )
			{
				continue;	/* Null list, don't create the button. */
			}
			w_pulldown = XmCreatePulldownMenu(menu, 
				menu_list[i_menu].sub_menu_title, NULL, 0);
			text = XmStringCreateLtoR(menu_list[i_menu].name,"charset1") ;
			buttons[++i_button]= XtVaCreateWidget(menu_list[i_menu].name,
				xmCascadeButtonWidgetClass, menu,
				XmNsubMenuId, 				w_pulldown,
				XmNmnemonic, 				(menu_list[i_menu].mnemonic),
				XmNmnemonicCharSet,			"charset1",
				XmNlabelString, 			text,
				XmNfontList,				font_list,
			0);
			if ( menu_list[i_menu].widget ) 
				*menu_list[i_menu].widget = buttons[i_button];	
			if (i_menu == (nitems - 1))  /* Assumes the last button is "Help" */
			{
				XtVaSetValues(menu,	
					XmNmenuHelpWidget, (XtArgVal)buttons[i_button],
				0);
			}
			if ( menu_list[i_menu].sub_menu_count )
			{
				build_static_menu_buttons(menu_list[i_menu].sub_menu_title, 
					font_list, w_pulldown, menu_list[i_menu].sub_menu,
					menu_list[i_menu].sub_menu_count);
			}
			else
			{
				p_link = (Menu_Linked_E*)menu_list[i_menu].sub_menu;
				build_linked_menu_buttons(menu_list[i_menu].sub_menu_title, 
					font_list, w_pulldown, p_link->p_next);
			}
		}


		else						/* Case 4 */
		{
			text = XmStringCreateLtoR(menu_list[i_menu].name,"charset1") ;
			buttons[++i_button] = XtVaCreateManagedWidget(
				menu_list[i_menu].name, 
				xmToggleButtonWidgetClass,	menu, 
				XmNmnemonic,             	menu_list[i_menu].mnemonic,
				XmNmnemonicCharSet,			"charset1",
				XmNlabelString, 			text,
				XmNfontList,				font_list,
			0);
			if ( menu_list[i_menu].widget ) 
				*menu_list[i_menu].widget = buttons[i_button];	
		}

   	} 

	/*
   	 *   MANAGE ALL BUTTON WIDGETS, ANY MENU PANES CREATED ARE NOT MANAGED
     */
    XtManageChildren(buttons, ++i_button );
}

void cb_dialog(w,data)
	Widget w ;
	int data ;
{
	if (data)
		exit(0) ;
	else
		XtUnmanageChild(w_dialog) ;
}

void create_dialog(w, title, msg,flag)
	Widget  w ;
	char*	title;
	char	msg[] ;        /* A string for error message */
	int     flag ;
{
	void      cb_dialog() ;
	Widget 		errorButton ;
	Widget 		errorFrame ;
	Widget 		errorLabel ;
	char        errorMessage[200] ;
	XmString	message ;
	XmString	okMessage ;
	XmString	titleMessage ;

	strcpy(errorMessage,msg) ;

	message = XmStringCreateLtoR(errorMessage,
			"charset1") ;
	okMessage = XmStringCreateLtoR((char *)"OK",
			"charset1") ;
	titleMessage = XmStringCreateSimple(title) ;
	if (!flag) {
	    w_dialog = XmCreateMessageDialog(w,"PV700 Calibrate - ",NULL, 0) ;
	    XtVaSetValues(w_dialog,XmNdialogType,XmDIALOG_MESSAGE,
			XmNmessageAlignment,XmALIGNMENT_BEGINNING,
			XmNmessageString,message,
			XmNdefaultButtonType,XmDIALOG_OK_BUTTON,
			XmNdialogTitle,	titleMessage,
		NULL) ;

	    XtAddCallback(w_dialog,XmNokCallback,cb_dialog,(void*)flag) ;
	    XtUnmanageChild(XmMessageBoxGetChild(
			w_dialog,XmDIALOG_CANCEL_BUTTON)) ;
	    XtUnmanageChild(XmMessageBoxGetChild(
			w_dialog,XmDIALOG_HELP_BUTTON)) ;
	    XtManageChild(w_dialog) ;
	} 
	XmStringFree(message) ;
	XmStringFree(okMessage) ;
	XmStringFree(titleMessage) ;
	
}

void my_usleep( usecs )
	int		usecs;	/* = # of microseconds to sleep. */
{
	struct timeval	tval;	/* Time value for select command. */

	tval.tv_sec = usecs / 1000000;
	tval.tv_usec = usecs % 1000000;

	select(0, 0, 0, 0, &tval);
}

			
/*----------------------------------------------------------------------------*
 |"sash_traversal_off"
 *----------------------------------------------------------------------------*/
sash_traversal_off(w)
	Widget w;
{
    int    n_children;	/* # of children of 'w'. */
    Widget *p_children;	/* Array of pointers to children of 'w'. */

    XtVaGetValues(w,
        XmNchildren,    &p_children,
        XmNnumChildren, &n_children,
    NULL);

    while ( n_children-- > 0)
	{
        if ( XmIsSash(p_children[n_children]) )
		{
            XtVaSetValues(p_children[n_children],
                XmNtraversalOn,			False,
                XmNmappedWhenManaged,	False,
            NULL);
		}
	}
}

