/*
 *  parse_value.h: Definitions for parse_value facility.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *     19 July 93 - Andy Goodrich
 *        - Initial coding
 *
 *  Copyright(c) 1993, Andrew C. Goodrich
 *  Used with permission by Parallax Graphics, Inc.
 */
#ifndef support_h_INCLUDED 
#define support_h_INCLUDED

typedef struct _menu_entry {
   char                  *name;           /* Name for menu item. */
   char                  mnemonic;        /* mnemonic; NULL if none */
   void                  (*cb_func)();    /* Callback function to be invoked. */
   caddr_t               cb_data;         /* Data for the callback function. */
   struct _menu_entry*   sub_menu;        /* Sub-menu list for this item. */
   int                   sub_menu_count;  /* How many items in sub_menu. */
   char*                 sub_menu_title;  /* Title of sub-menu. */
   Widget*               widget;          /* Store widget handle here if not 0*/
   int                   toggle_flag ;    /* Widget flags for menu selection. */
 } Menu_Entry;

typedef struct _menu_linked {
   struct _menu_linked*  p_next;          /* Next menu entry in this panel. */
   char*                 name;            /* Name for menu item. */
   char                  mnemonic;        /* mnemonic; NULL if none */
   void                  (*cb_func)();    /* Callback function to be invoked. */
   caddr_t               cb_data;         /* Data for the callback function. */
   struct _menu_linked*  sub_menu;        /* Sub-menu list for this item. */
   char*                 sub_menu_title;  /* Title of sub-menu. */
   int                   misc;            /* Temporary used in building list. */
   Widget*               widget;          /* Store widget handle here if not 0*/
   int                   toggle_flag ;    /* Widget flags for menu selection. */
 } Menu_Linked_E;

typedef union _PV_Result {		/* Result of p_value call: */
	int	   i;					/* ... value if integer parsed. */
	float  f;					/* ... value if floating point parsed. */
	char  *p;					/* ... value if character string parsed. */
} PV_Result;

enum {					/* Return values for p_value: */
	PV_Bad_Alloc = 0,	/* Unable to allocate storage for value. */
	PV_Bad_Value,		/* Unable to parse input string. */
	PV_Float,			/* Parsed floating point number. */
	PV_Integer,			/* Parsed integer value. */
	PV_Keyword,			/* Parsed keyword string. */
	PV_Quoted_String,	/* Parsed quoted string. */
};

#endif  /* support_h_INCLUDED */
