/*
 *  xmtv.c:	TV demonstration program utilizing Motif Windowing system.
 *
 *
 *  REVISION HISTORY
 *
 * $Log:   /plxhp/dave/pvcsrc/O_XV/archives/xmtv.c_v  $
 * 
 *    Rev 1.11   09 Sep 1996 09:54:58   dave
 * Andy Goodrich: Added better banner message support and menu
 *                items for file and help.
 * Revision 1.11  1995/08/31 12:52:00  dist
 * Andy Goodrich: Added better banner message support and menu
 *                items for file and help.
 *
 * Revision 1.10  95/08/18  12:26:22  12:26:22  dist ()
 * Andy Goodrich: Added code to allow selection of adaptor and port for 
 *                multiple device case.
 * 
 * Revision 1.9  95/07/25  13:03:36  13:03:36  dist ()
 * Andy Goodrich: Fixed typo in sash traversal invocation.
 * 
 * Revision 1.8  95/07/25  13:00:48  13:00:48  dist ()
 * Andy Goodrich: Added call to sash_traversal_off to get rid of resizable
 *                menu bar.
 * 
 * Revision 1.7  95/07/24  10:50:13  10:50:13  dist ()
 * Andy Goodrich: Miscellaneous cleanup and freeze for real HP 9.1 release.
 * 
 * Revision 1.6  95/07/19  15:53:28  15:53:28  dist ()
 * Andy Goodrich: Minor cosmetic changes to input/output banner.
 * 
 * Revision 1.5  95/07/19  15:45:55  15:45:55  dist ()
 * Andy Goodrich: Added banner with input type.
 * 
 * Revision 1.4  95/07/13  12:06:22  12:06:22  dist ()
 * Andy Goodrich: Added code to fix resizing of windows and properly handle
 *                scaling of output.
 * 
 * Revision 1.3  95/07/12  13:51:35  13:51:35  dist ()
 * Andy Goodrich: Added output menu to program.
 * 
 * Revision 1.2  95/06/07  16:06:20  16:06:20  dist ()
 * Andy Goodrich: Modified version which uses new build_linked_menu
 *                stuff.
 * 
 * Revision 1.1  95/05/16  10:10:29  10:10:29  dist ()
 * Initial revision
 * 
 *
 *  Copyright(c) 1995, Parallax Graphics, Inc.
 */
#include <stdio.h>
#include <sys/types.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <Xm/DrawingA.h>
#include <Xm/DialogS.h>
#include <Xm/Form.h>
#include <Xm/PanedW.h>
#include "Xvlib.h"
#include "Xvddx_PGI.h"
#include "O_XV_PGI.h"
#include "support.h"

enum {
	MAX_ARGS = 50,
	WINDOW_HEIGHT = 480,
	WINDOW_WIDTH = 640,
};

XtAppContext	appContext;
Menu_Linked_E*  p_inputs_list;          /* Decoders list menu. */
Menu_Linked_E*  p_outputs_list;         /* Encoders list menu. */
O_XV_PGI*	 	p_xv_in;       			/* Xv access object for port. */
O_XV_PGI*	 	p_xv_out;      			/* Xv access object for port. */
Widget          w_bar;
Widget          w_form;
Widget 			w_toplevel;
Widget 			w_video;

void cb_about_xmtv();
void cb_exit();
void cb_expose();
void cb_help();
void cb_input();
void cb_input_off();
void cb_output();
void cb_output_off();
void cb_resize();
void set_io_banner();

#include "xmtv_menu.h"	/* Must come after p_inputs_list & p_outputs_list. */
#include "xmtv_help.h"

/*---------------------------------------------------------------------------*
 |"cb_about_xmtv"
 *---------------------------------------------------------------------------*/
void cb_about_xmtv(w, p_data, event)
    Widget  w;             /* = widget causing the callback. */
    char*   p_data;          /* Our instance specific data. */
    XEvent* event;         /* Event causing keyboard or mouse callback. */
{
    create_dialog(w, "XMTV", "XMTV 9.2 - (c) Parallax - 31 August 1995", 0);
}


/*---------------------------------------------------------------------------*
 |"cb_exit"
 *---------------------------------------------------------------------------*/
void cb_exit(w, p_me, event)
    Widget   w;           /* = widget causing the callback. */
    void* p_me;        	  /* Our instance specific data. */
    XEvent*  event;       /* Event causing keyboard or mouse callback. */
{
	O_XV_Video_Off(p_xv_in);
	if ( p_outputs_list ) O_XV_Video_Off(p_xv_out);
	exit(0);
}


/*---------------------------------------------------------------------------*
 |"cb_expose"
 *---------------------------------------------------------------------------*/
void cb_expose(w, p_xv, event)
  Widget  	w;       /* = widget causing the callback. */
  O_XV_PGI* p_xv;    /* Our context. */
  XEvent*   event;   /* Event causing keyboard or mouse callback. */
{
	if ( O_XV_Status( p_xv ) & O_XV_Status_Video_In ) O_XV_Video_On( p_xv );
}


/*---------------------------------------------------------------------------*
 |"cb_help"
 *---------------------------------------------------------------------------*/
void cb_help(w, item_name, event)
    Widget  w;             /* = widget causing the callback. */
    char*   item_name;     /* -> item help is desired for. */
    XEvent* event;         /* Event causing keyboard or mouse callback. */
{
	Arg 	args[20];		/* XtSet argument list. */
	Pixel   background;		/* Background color for help panel. */
	char    buf[80*25];		/* Buffer to hold text. */
	int		i_args;			/* Element in args accessing. */
	int		i_item;			/* Help item now testing against item_name. */
	int     i_line;			/* Line of help text now processing. */
	int		n_help;			/* Number of lines in data pointed to by p_help. */
	char*   p_buf;			/* Next location in buf to set. */
	char**	p_help;			/* Help information to use. */
	Widget  w_form;			/* Form widget containing 'w_text'. */
	Widget	w_help;			/* Popup shell containing the other widgets. */
	Widget 	w_pane;			/* Pane widget used to manage presentation. */
	Widget  w_text;			/* Text scrolling widget. */

	XtVaGetValues(XtParent(w),XmNbackground,&background,NULL);


	/*  CREATE A POPUP SHELL CONTAINING A PANED WINDOW TO PRESENT THE
	 *  HELP INFORMATION
	 */
	w_help = XtVaCreatePopupShell("XMTV - Help",
		xmDialogShellWidgetClass,	XtParent(w),
		XmNdeleteResponse,			XmDESTROY,
		XmNbackground,				background,
	NULL);

	w_pane = XtVaCreateWidget("pane",
		xmPanedWindowWidgetClass,	w_help,
		XmNbackground,				background,
	NULL);

	w_form = XtVaCreateWidget("form",
		xmFormWidgetClass,			w_pane,
		XmNbackground,				background,
	NULL);

	/*
	 *  BUILD BUFFER WITH TEXT FOR SCROLL AREA BASED ON WHAT HELP IS DESIRED.
	 */
 	for ( p_help = 0, i_item = 0; i_item < n_help_items; i_item++ )
	{
		if ( !strcmp(item_name, help_items[i_item].name) )
		{
			p_help = help_items[i_item].p_p_text;
			n_help = help_items[i_item].n_lines;
		}
	}
	if ( p_help == 0 )
	{
		return;
	}
	for ( p_buf =  buf, i_line = 0; i_line < n_help; i_line++ )
	{
		p_buf += strlen(strcpy(p_buf, p_help[i_line]));
		*p_buf++ = '\n';
		*p_buf = 0;
	}
	*(--p_buf) = 0;			/* Remove final newline. */

	i_args = 0;
	XtSetArg(args[i_args],	XmNbackground,	      background);        i_args++;
	XtSetArg(args[i_args],  XmNcolumns,			  80);                i_args++;
	XtSetArg(args[i_args],	XmNcursorPositionVisible,False);          i_args++;
	XtSetArg(args[i_args],	XmNeditMode,		  XmMULTI_LINE_EDIT); i_args++;
	XtSetArg(args[i_args],	XmNeditable,		  False);             i_args++;
	XtSetArg(args[i_args],	XmNrows,			  25);                i_args++;
	XtSetArg(args[i_args],	XmNscrollVertical,	  True);              i_args++;
	XtSetArg(args[i_args],	XmNscrollHorizontal,  False);             i_args++;
	XtSetArg(args[i_args],	XmNvalue,			  buf);               i_args++;
	XtSetArg(args[i_args],	XmNwordWrap,		  True);              i_args++;

	w_text = XmCreateScrolledText(w_form, "help_text", args, i_args);

	XtManageChild(w_text);
	XtManageChild(w_form);

	XtManageChild(w_pane);
	XtPopup(w_help,XtGrabNone);
}


/*---------------------------------------------------------------------------*
 |"cb_input"
 *---------------------------------------------------------------------------*/
void cb_input( w, p_encoding, event )
	Widget  		w;          /* = widget causing the callback. */
	XvEncodingInfo*	p_encoding;	/* -> encoding selected by user. */
	XEvent* 		event;      /* Event causing callback. */
{
	int		id;			/* Fast access to the encoding id. */

	id = p_encoding->encoding_id;
	O_XV_Source(p_xv_in, (id & XV_PLX_DMSD) >> 4, id & XV_SOURCE_TIMING,
		id & XV_SOURCE_TYPE, id & XV_PLX_INPUT);
	O_XV_Video_On(p_xv_in);
	set_io_banner(w_toplevel, p_xv_in, p_xv_out);
	cb_resize(w, p_xv_in, 0);
}


/*---------------------------------------------------------------------------*
 |"cb_input_off"
 *---------------------------------------------------------------------------*/
void cb_input_off( w, p_encoding, event )
	Widget  		w;          /* = widget causing the callback. */
	XvEncodingInfo*	p_encoding;	/* -> encoding selected by user. */
	XEvent* 		event;      /* Event causing callback. */
{
	O_XV_Video_Off(p_xv_in);
	set_io_banner(w_toplevel, p_xv_in, p_xv_out);
}


/*---------------------------------------------------------------------------*
 |"cb_output"
 *---------------------------------------------------------------------------*/
void cb_output( w, id, event )
	Widget  		w;     	/* = widget causing the callback. */
	int	            id;		/* -> encoding selected by user. */
	XEvent* 		event; 	/* Event causing callback. */
{
	short	video_w;	/* Current width of video window. */
	short	video_h;	/* Current height of video window. */

	XtVaGetValues(w_video,
		XmNwidth, &video_w,
		XmNheight, &video_h,
	NULL);
	O_XV_Source(p_xv_out, 1, id & XV_SOURCE_TIMING,
		id & XV_SOURCE_TYPE, id & XV_PLX_INPUT);
	p_xv_out->base.win_width = video_w;
	p_xv_out->base.win_height = video_h;
	O_XV_Video_Output(p_xv_out);
#ifdef NOTDEF
	XvGetVideo(p_xv_out->base.display, p_xv_out->base.port_id, 
				p_xv_out->base.w_main, p_xv_out->base.gc, 
				0, 0, p_xv_out->base.sig_width, p_xv_out->base.sig_height,
				0, 0, video_w, video_h
	);
#endif
	set_io_banner(w_toplevel, p_xv_in, p_xv_out);
}


/*---------------------------------------------------------------------------*
 |"cb_output_off"
 *---------------------------------------------------------------------------*/
void cb_output_off( w, id, event )
	Widget  		w;     	/* = widget causing the callback. */
	int	            id;		/* -> encoding selected by user. */
	XEvent* 		event; 	/* Event causing callback. */
{
	O_XV_Video_Off(p_xv_out);
	set_io_banner(w_toplevel, p_xv_in, p_xv_out);
}


/*----------------------------------------------------------------------------*
 |"cb_resize"
 |
 |  Callback routine for resizing of video window. Note of the incoming
 |  parameters we only look at the type field of the event parameter.
 *----------------------------------------------------------------------------*/
static void cb_resize( w, p_xv, event )
    Widget      w;      /* = widget causing the callback. */
    O_XV_PGI*   p_xv;   /* Our instance specific data. */
    XEvent*		event;  /* Event causing resize callback. */
{
	short	menu_h; 	/* Height of resized menu window. */
	short	video_h;	/* Height coordinate of resized video window. */
	short	video_w;	/* Width coordinate of resized video window. */


	XtVaGetValues(w_bar,
		XmNheight, &menu_h,
	NULL);
	XtVaGetValues(w_video,
		XmNwidth, &video_w,
		XmNheight, &video_h,
	NULL);

	if ( event == 0 )
	{
		video_w = p_xv->base.sig_width;
		video_h = p_xv->base.sig_height;
	}
	else
	{
		if ( video_w > p_xv->base.sig_width )  video_w = p_xv->base.sig_width;
		if ( video_h > p_xv->base.sig_height ) video_h = p_xv->base.sig_height;
	}

	XtVaSetValues(w_toplevel,
		XmNwidth, video_w,
		XmNheight, video_h + menu_h,
	NULL);

	XtVaSetValues(w_form,
		XmNwidth, video_w,
		XmNheight, video_h + menu_h,
	NULL);

	XtVaSetValues(w_video,
		XmNwidth, video_w,
		XmNheight, video_h,
	NULL);

	XtVaGetValues(w_video,
		XmNwidth, &video_w,
		XmNheight, &video_h,
	NULL);
	p_xv->base.win_width = video_w;
	p_xv->base.win_height = video_h;
	if ( O_XV_Status( p_xv ) & O_XV_Status_Video_In ) O_XV_Video_On( p_xv );
}


/*----------------------------------------------------------------------------*
 |"main"
 |
 |  Entry point for this application.
 *----------------------------------------------------------------------------*/
main(argc, argv)
	int  argc; 
	char* argv[];
{
	int             adaptor;	   /* Adaptor to open. */
	XColor	 	 	exact_color;   /* Exact RGB for color. */
	XFontStruct* 	font_list;     /* Font using for menus. */
	XFontStruct* 	font1;         /* System 17 font. */
	XFontStruct* 	font2;         /* User 7x17 font. */
	int				i_argv;		   /* Element in argv now accessing. */
	int			 	n_signals;	   /* # of signals for this adaptor. */
	Display* 	 	p_display;	   /* Display being used. */
	int				port;		   /* Port on adaptor to use. */
	XvplxSignalRec*	p_signals;	   /* # of signals for this adaptor. */
	XColor	 	 	scrn_color;    /* Closest screen RGB to color. */	
	Colormap 	 	video_cmap;    /* Color map for video window. */


	adaptor = 0;
	port = 0;
    for ( i_argv = 1; i_argv < argc; i_argv++ )
    {
        if ( !strcmp(argv[i_argv], "adaptor") )
        {
            i_argv++;
            if ( (i_argv >= argc) ||
                 (sscanf(argv[i_argv], "%d", &adaptor) != 1) )
            {
                fprintf(stderr, "*** Missing number for adaptor keyword\n");
                return 0;
            }
		}
        else if ( !strcmp(argv[i_argv], "port") )
        {
            i_argv++;
            if ( (i_argv >= argc) ||
                 (sscanf(argv[i_argv], "%d", &port) != 1) )
            {
                fprintf(stderr, "*** Missing number for port keyword\n");
                return 0;
            }
		}
		else if ( !strcmp(argv[i_argv], "help") )
		{
			printf("Usage: xmtv [adaptor N]\n");
			return 0;
		}
	}

	w_toplevel = XtAppInitialize(&appContext,
		"winXt",
		NULL,
		(Cardinal) NULL,
		&argc,
		argv,
		NULL,
		NULL,
	NULL);


	p_display = XtDisplay(w_toplevel);

    /*
     *  DETERMINE MENU FONT.
     */
    font1 = XLoadQueryFont(p_display, "system17");
    font2 = XLoadQueryFont(p_display, "user7x17");
    if (font1)
    {
        font_list = XmFontListCreate(font1,"charset1");
        if (font2)
            font_list = XmFontListAdd(font_list,font2,"charset2");
    }
    else
    {
        if (font2)
            font_list = XmFontListCreate(font2,"charset2");
    }


    /*
     *  CREATE THE FORM WHICH WILL HOLD MENU BAR AND VIDEO AREA:
     *  THEN CREATE THE MENU AND VIDEO AREAS WITHIN THAT FORM.
     */
    w_form = XtVaCreateManagedWidget("main_window",
        xmPanedWindowWidgetClass,   w_toplevel,
        XmNwidth,                   640,
        XmNheight,                  600,
        XmNseparatorOn,             False,
        XmNspacing,                 0,
        XmNmarginHeight,            0,
        XmNmarginWidth,             0,
    NULL);
    w_bar = build_menu_bar( w_form );

	video_cmap = XDefaultColormap( p_display, XDefaultScreen(p_display));
	XAllocNamedColor(p_display, video_cmap, "black", &scrn_color, &exact_color);
    w_video = XtVaCreateManagedWidget("drawing_a",
        xmDrawingAreaWidgetClass,   w_form,
        XmNwidth,               	WINDOW_WIDTH,
        XmNheight,              	WINDOW_HEIGHT,
		XmNbackground,				scrn_color.pixel,
        XmNcolormap,                video_cmap,
    NULL) ;


	p_xv_in = O_XV_PGI_O_XV_PGI( 0, XtDisplay(w_toplevel), adaptor, port);
    if ( !p_xv_in )
    {
        fprintf(stderr, "\nCould not open display\n");
        return 0;
    }
	XtAddCallback(w_video,	XmNresizeCallback, cb_resize, p_xv_in);
	XtAddCallback(w_video, XmNexposeCallback, cb_expose, p_xv_in);

	p_inputs_list = build_inputs_list("Off", cb_input_off, cb_input, 
			p_xv_in->base.n_encodings, p_xv_in->base.p_encodings);

	p_outputs_list = build_outputs_list("Off", cb_output_off, cb_output, 
			p_xv_in->base.n_encodings, p_xv_in->base.p_encodings);
		
    build_static_menu_buttons(NULL, font_list, w_bar,
        MenuBar, XtNumber(MenuBar));

	XtRealizeWidget(w_toplevel);
    O_XV_Use_Window( p_xv_in, XtWindow(w_video) );
	O_XV_Video_On(p_xv_in);
	if ( p_outputs_list ) 
	{
		if ( !(p_xv_out = O_XV_PGI_O_XV_PGI( 0, XtDisplay(w_toplevel), 0, 1 )) )
		{
			fprintf(stderr, "\nCould not open display for output\n");
			return 0;
		}
		O_XV_Use_Window( p_xv_out, XtWindow(w_video) );
		O_XV_Video_Off(p_xv_out);
	}
	set_io_banner(w_toplevel, p_xv_in, p_xv_out);

	sash_traversal_off(w_form);
	XtAppMainLoop(appContext);

}


/*---------------------------------------------------------------------------*
 |"set_io_banner"
 |
 | Set up banner for window to indicate what video components are being used.
 *---------------------------------------------------------------------------*/
static void set_io_banner(w, p_xv_in, p_xv_out)
	Widget  w;		/* = widget whose banner is to be set. */
	O_XV*	p_xv_in;	/* -> input port. */
	O_XV*	p_xv_out;	/* -> output port or 0. */
{
	int		in_id;			/* Current input encoding id. */
	int		instance;		/* Video input instance. */
	int		out_id;			/* Current output encoding id. */
	char*	p_in_format;	/* Format of input encoding. */
	char*	p_in_timing;	/* Timing for input encoding. */
	char*	p_out_timing;	/* Timing for output decoding. */
	char	title[256];		/* Area to build title bar string. */
	int		video_input;	/* = 1 if video input is on. */
	int		video_output;	/* = 1 if video output is on. */

	video_input = O_XV_Status(p_xv_in) & O_XV_Status_Video_In ? 1 : 0;
	video_output = O_XV_Status(p_xv_out) & O_XV_Status_Video_Out ? 1 : 0;

	/*  SET UP INPUT TIMING IF VIDEO IS ACTIVE. */

	if ( video_input )
	{
		XvGetPortAttribute(p_xv_in->display, p_xv_in->port_id, 
			p_xv_in->a_encoding, &in_id);
		switch( in_id & XV_SOURCE_TIMING )
		{
		  case XV_NTSC:	 p_in_timing = "NTSC";		break;
		  case XV_PAL:	 p_in_timing = "PAL";		break;
		  case XV_SECAM: p_in_timing = "SECAM";		break;
		  default:       p_in_timing = "Unknown";	break;
		}
		switch( in_id & XV_SOURCE_TYPE )
		{
		  case XV_COMPOSITE: p_in_format = "Composite";	break;
		  case XV_YC: 		 p_in_format = "S-Video";	break;
		  default:			 p_in_format = "Unknown";	break;
		}
		instance = 1+ (in_id & 0xf);
	}


	/*  OUTPUT POSSIBLE, SHOW OUTPUT AND INPUT INFORMATION. */

	if ( p_xv_out )
	{
		if ( video_output )
		{
			XvGetPortAttribute(p_xv_out->display, p_xv_out->port_id, 
				p_xv_out->a_encoding, &out_id);
			switch( out_id & XV_SOURCE_TIMING )
			{
			  case XV_NTSC:	 p_out_timing = "NTSC";		break;
			  case XV_PAL:	 p_out_timing = "PAL";		break;
			  default:		 p_out_timing = "Unknown";	break;
			}
		}
		else
		{
			p_out_timing = "Off";
		}

		if ( video_input )
		{
			sprintf(title, "Decoder %d: %s %s %d   Encoder: %s", 
				1 + ((in_id & XV_PLX_DMSD) >> 4), p_in_timing, 
				p_in_format, instance, p_out_timing);

		}
		else
		{
			sprintf(title, "Decoders: Off   Encoder: %s", p_out_timing);
		}
	}


	/*  NO OUTPUT HARDWARE AVAILABLE, SHOW ONLY INPUT INFORMATION. */

	else 
	{
		if ( video_input )
		{
			sprintf(title, "Decoder %d: %s %s %d", 
				1+((in_id&XV_PLX_DMSD)>>4), p_in_timing, p_in_format,instance);
		}

		else
		{
			sprintf(title, "Decoders: Off");
		}
	}

	XtVaSetValues(w, XmNtitle, title, NULL);
}
