/*
 * xvshow.c:	Test program for Xv video extension access object.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     26 May 93 - Andy Goodrich
 *       - Initial coding
 *
 * $Log:   /plxhp/dave/pvcsrc/O_XV/archives/xvshow.c_v  $
 * 
 *    Rev 1.4   09 Sep 1996 09:56:04   dave
 * Andy Goodrich: Freeze for real HP 9.1 release.
 * Revision 1.4  1995/07/24 10:51:00  dist
 * Andy Goodrich: Freeze for real HP 9.1 release.
 *
 * Revision 1.3  95/06/21  17:17:38  17:17:38  dist ()
 * Andy Goodrich: Added support for viewing multiple images from a file.
 * 
 * Revision 1.2  95/06/07  16:05:19  16:05:19  dist ()
 * Andy Goodrich: Correct scaling problems introduced when doing some 
 *                testing for JJ.
 * 
 * Revision 1.1  95/05/16  10:10:37  10:10:37  dist ()
 * Initial revision
 * 
 *
 *  Copyright(c) 1993, Andrew C. Goodrich
 */
#include <sys/types.h>			/* Standard types for this C environment. */
#include <stdio.h>				/* Stream input/output declarations. */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <Xvlib.h>
#include "Xvddx.h"
#include "O_XV.h"
#include "rasterfile.h"

/*  FORWARD REFERENCES: */

XImage*	read_sun_raster();

/*----------------------------------------------------------------------------*
 |"main"
 |
 |  Entry point for system for this application.
 |
 *----------------------------------------------------------------------------*/
main( argc, argv )
	int		argc;	/* = # of command arguments parsed by shell. */
	char*	argv[];	/* = vector of pointers to command arguments parsed. */
{
	Atom		a_ximage;	/* Atom designating an XImage. */
	int			dump_hdr;	/* = 1 if should dump raster file header. */
	XEvent		event;		/* Information about X event we intercepted. */
	FILE*		fd;			/* File descriptor to read image from. */
	int			height;		/* Height of image to display. */
	int			i_argv;		/* # of element in 'argv' accessing. */
	int			output;		/* = 1 if should output to tv. */
	Display*	p_display;	/* Display accessing. */
	XImage*		p_image;	/* X image to display. */
	O_XV*		p_xv;		/* Xv access object. */	
	O_XV*		p_xv1;		/* Xv access object. */	
	int			width;		/* Width of image to display. */
	
	dump_hdr = 0;
	output = 0;
	switch ( argc )
	{
	  case 2:
		break;
	  case 3:
		if ( !strcmp(argv[2], "dump") )
		{
			dump_hdr = 1;
			break;
		}
		else if ( !strcmp(argv[2], "output") )
		{
			output = 1;
			break;
		}
	  default:
		fprintf(stderr, "Usage: xvshow file [dump | output]\n");
		return 0;
	}

	if ( !(p_display = XOpenDisplay(0)) ) 
	{
		fprintf(stderr, "\nCould not open display\n");
		return 0;
	}
	a_ximage = XInternAtom(p_display,"XV_IMAGE_XIMAGE",False);

	if ( !(fd = fopen(argv[1], "r")) )
	{
		fprintf(stderr, "*** Unable to open file %s\n", argv[1]);
		return 0;
	}
	p_image = read_sun_raster(fd, dump_hdr, p_display, &width, &height);
	if ( !p_image )
	{
		fprintf(stderr, "Unable to parse a sun raster file in %s\n", argv[1]);
		return 0;
	}
#ifdef sun
	p_image->depth = 24;
#endif

	if ( !(p_xv = O_XV_O_XV( 0, p_display, 0, 0 )) )
	{
		fprintf(stderr, "\nCould create O_XV object\n");
		return 0;
	}

	fprintf(stderr, "Outputing image of size %d by %d\n", width, height);
	O_XV_Window( p_xv, -1, -1 );
	p_xv->win_width = width;
	p_xv->win_height = height;
	O_XV_Video_Off( p_xv );
	XFlush(p_display);
#ifdef NOTDEF
	XPutImage(p_xv->display, p_xv->w_main, p_xv->gc, p_image, 
		0, 0, 0, 0, width, height);
#else
	XvPutImage(p_xv->display, p_xv->port_id, p_xv->w_main, p_xv->gc, a_ximage,
		0, p_image, 0, 0, width, height, 0, 0, width, height);
#endif
	if ( output )
	{
		if ( !(p_xv1 = O_XV_O_XV( 0, p_display, 0, 1 )) )
		{
			fprintf(stderr, "\nCould create O_XV object\n");
			return 0;
		}
		O_XV_Source( p_xv1, 0, XV_NTSC, XV_COMPOSITE, 0 );
		/* O_XV_Video_Off( p_xv1 ); */
		XvGetVideo(p_display, p_xv->port_id, p_xv->w_main, p_xv->gc,
			0, 0, p_xv->sig_width, p_xv->sig_height,
			0, 0, p_xv->sig_width, p_xv->sig_height);
	}

#ifndef NOTDEF
	XSetGraphicsExposures(p_xv->display, p_xv->gc, True);
	XvSelectVideoNotify(p_xv->display, p_xv->w_main, True);
    for(;;)
    {
#ifndef NOTDEF
		XWindowEvent(p_display, p_xv->w_main, 0xffffffff,  &event);
#else
		XNextEvent(p_display, &event); 
		printf("Event = %d (", event.type);
		print_event(stdout, &event);
		printf(")\n");
#endif
		switch ( event.type )
		{
		  case ButtonPress:
			break;
		  case KeyPress:
			p_image = read_sun_raster(fd, dump_hdr, p_display, &width, &height);
			if ( p_image )
			{
#ifdef NOTDEF
				XPutImage(p_xv->display, p_xv->w_main, p_xv->gc, p_image, 
					0, 0, 0, 0, width, height);
#else
	XvPutImage(p_xv->display, p_xv->port_id, p_xv->w_main, p_xv->gc, a_ximage,
		0, p_image, 0, 0, width, height, 0, 0, width, height);
#endif
			}
			else
			{
				return 0;
			}
			break;
		  case VisibilityNotify:
#ifdef NOTDEF
			XPutImage(p_xv->display, p_xv->w_main, p_xv->gc, p_image, 
				0, 0, 0, 0, width, height);
#else
			XvPutImage(p_xv->display, p_xv->port_id, p_xv->w_main, p_xv->gc, 
				a_ximage, 0, p_image, 0, 0, width, height, 
				0, 0, width, height);
#endif
			break;
		  case Expose:
#ifdef NOTDEF
			XPutImage(p_xv->display, p_xv->w_main, p_xv->gc, p_image, 
				0, 0, 0, 0, width, height);
#else
			XvPutImage(p_xv->display, p_xv->port_id, p_xv->w_main, p_xv->gc, 
				a_ximage, 0, p_image, 0, 0, width, height, 
				0, 0, width, height);
#endif
			break;
		  case NoExpose:
			printf("NoExpose notify\n");
			break;
		  case GraphicsExpose:
			printf("GraphicsExpose notify\n");
			break;
		}
    }
#else
	process_geometry(p_xv, width, height, p_image);
#endif
}

print_event(fd, event)
	FILE*	fd;
	XEvent*	event;
{
	switch( event->type )
	{
	  case KeyPress:            fprintf(fd, "KeyPress");			break;
	  case MappingNotify:       fprintf(fd, "MappingNotify");      	break;
	  case KeyRelease:          fprintf(fd, "KeyRelease");			break;
	  case ButtonPress:         fprintf(fd, "ButtonPress");			break;
	  case ButtonRelease:       fprintf(fd, "ButtonRelease");		break;
	  case MotionNotify:        fprintf(fd, "MotionNotify");		break;
	  case EnterNotify:         fprintf(fd, "EnterNotify");			break;
	  case LeaveNotify:         fprintf(fd, "LeaveNotify");			break;
	  case FocusIn:             fprintf(fd, "FocusIn");				break;
	  case FocusOut:            fprintf(fd, "FocusOut");			break;
	  case KeymapNotify:        fprintf(fd, "KeymapNotify");		break;
	  case Expose:              fprintf(fd, "Expose");				break;
	  case GraphicsExpose:      fprintf(fd, "GraphicsExpose");		break;
	  case NoExpose:            fprintf(fd, "NoExpose");			break;
	  case VisibilityNotify:    fprintf(fd, "VisibilityNotify");	break;
	  case CreateNotify:        fprintf(fd, "CreateNotify");		break;
	  case DestroyNotify:       fprintf(fd, "DestroyNotify");		break;
	  case UnmapNotify:         fprintf(fd, "UnmapNotify");			break;
	  case MapNotify:           fprintf(fd, "MapNotify");			break;
	  case MapRequest:      	fprintf(fd, "MapRequest");			break;
	  case ReparentNotify:      fprintf(fd, "ReparentNotify");		break;
	  case ConfigureNotify:     fprintf(fd, "ConfigureNotify");		break;
	  case ConfigureRequest:    fprintf(fd, "ConfigureRequest");	break;
	  case GravityNotify:       fprintf(fd, "GravityNotify");		break;
	  case ResizeRequest:       fprintf(fd, "ResizeRequest");		break;
	  case CirculateNotify:     fprintf(fd, "CirculateNotify");		break;
	  case CirculateRequest:    fprintf(fd, "CirculateRequest");	break;
	  case PropertyNotify:      fprintf(fd, "PropertyNotify");		break;
	  case SelectionClear:      fprintf(fd, "SelectionClear");		break;
	  case SelectionRequest:    fprintf(fd, "SelectionRequest");	break;
	  case SelectionNotify:     fprintf(fd, "SelectionNotify");		break;
	  case ColormapNotify:      fprintf(fd, "ColormapNotify");		break;
	  case ClientMessage:       fprintf(fd, "ClientMessage");		break;
	}
}
