/*
 * xvshow.c:	Test program for Xv video extension access object.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     26 May 93 - Andy Goodrich
 *       - Initial coding
 *
 *  Copyright(c) 1993, Andrew C. Goodrich
 */
#include <sys/types.h>			/* Standard types for this C environment. */
#include <stdio.h>				/* Stream input/output declarations. */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <Xvlib.h>
#include "Xvddx.h"
#include "O_XV.h"
#include "rasterfile.h"

/*  FORWARD REFERENCES: */

XImage*	read_sun_raster();

/*----------------------------------------------------------------------------*
 |"main"
 |
 |  Entry point for system for this application.
 |
 *----------------------------------------------------------------------------*/
main( argc, argv )
	int		argc;	/* = # of command arguments parsed by shell. */
	char*	argv[];	/* = vector of pointers to command arguments parsed. */
{
	XEvent		event;		/* Information about X event we intercepted. */
	int			height;		/* Height of image to display. */
	int			i_argv;		/* # of element in 'argv' accessing. */
	Display*	p_display;	/* Display accessing. */
	XImage*		p_image;	/* X image to display. */
	O_XV*		p_xv;		/* Xv access object. */	
	O_XV*		p_xv1;		/* Xv access object. */	
	int			width;		/* Width of image to display. */
	
	if ( argc != 2 )
	{
		fprintf(stderr, "Usage: xvshow file\n");
		return 0;
	}

	if ( !(p_display = XOpenDisplay(0)) ) 
	{
		fprintf(stderr, "\nCould not open display\n");
		return 0;
	}

	if ( !(p_image = read_sun_raster(argv[1], p_display, &width, &height)) )
	{
		fprintf(stderr, "Unable to parse a sun raster file in %s\n", argv[1]);
		return 0;
	}
#ifdef sun
	p_image->depth = 24;
#endif

	if ( !(p_xv = O_XV_O_XV( 0, p_display, 0, 0 )) )
	{
		fprintf(stderr, "\nCould create O_XV object\n");
		return 0;
	}

	O_XV_Window( p_xv, -1, -1 );
	O_XV_Video_Off( p_xv );
	XFlush(p_display);
	fprintf(stderr, "Outputing image of size %d by %d\n", width, height);
	XPutImage(p_xv->display, p_xv->w_main, p_xv->gc, p_image, 
		0, 0, 0, 0, width, height);
    O_XV_Source( p_xv1, 0, XV_NTSC, XV_COMPOSITE, 0 );
    XvGetVideo(p_display, p_xv1->port_id, p_xv->w_main, p_xv->gc,
        0, 0, p_xv->sig_width, p_xv->sig_height,
        0, 0, p_xv->sig_width, p_xv->sig_height);

#ifndef NOTDEF
	XSetGraphicsExposures(p_xv->display, p_xv->gc, True);
	XvSelectVideoNotify(p_xv->display, p_xv->w_main, True);
    for(;;)
    {
#ifndef NOTDEF
		XWindowEvent(p_display, p_xv->w_main, 0xffffffff,  &event);
#else
		XNextEvent(p_display, &event); 
#endif
		printf("Event = %d (", event.type);
		print_event(stdout, &event);
		printf(")\n");
		switch ( event.type )
		{
		  case VisibilityNotify:
			XPutImage(p_xv->display, p_xv->w_main, p_xv->gc, p_image, 
				0, 0, 0, 0, width, height);
			break;
		  case Expose:
			XPutImage(p_xv->display, p_xv->w_main, p_xv->gc, p_image, 
				0, 0, 0, 0, width, height);
			break;
		  case NoExpose:
			printf("NoExpose notify\n");
			break;
		  case GraphicsExpose:
			printf("GraphicsExpose notify\n");
			break;
		}
    }
#else
	process_geometry(p_xv, width, height, p_image);
#endif
}

/*----------------------------------------------------------------------------*
 |"read_sun_raster"
 |
 |  Read a sun raster file and convert it into an XImage.
 |
 *----------------------------------------------------------------------------*/
XImage* read_sun_raster( p_name, p_display, p_width, p_height )
	char*		p_name;		/* -> name of file to read image from. */
	Display*	p_display;	/* -> display to create XImage for. */
	int*		p_width;	/* -> where to store width of image. */
	int*		p_height;	/* -> where to store height of image. */
{
	FILE*				fd;			 /* File descriptor for file reading. */
	struct rasterfile	header;		 /* Header for Sun raster format file. */
	int				    i_image;	 /* Entry in p_im_dataN accessing. */
	int					n_read;		 /* Number of bytes read. */
	u_int*				p_cmap;		 /* Lookup table (colormap) data. */
	XImage*				p_image;	 /* Image */
	u_char*				p_im_data8;	 /* 8-bit image data. */
	u_int*				p_im_data32; /* 32-bit image data. */


	/*  OPEN THE FILE AND READ IN THE RASTER HEADER. */

	if ( (fd = fopen(p_name, "r")) == 0 ) return 0;
	if ( fread(&header, sizeof(header), 1, fd) != 1 )
	{
		fprintf(stderr, "Could not read in header\n");
		return 0;
	}
	if ( header.ras_magic != RAS_MAGIC )
	{
		fprintf(stderr, "Bad magic number %08x should be %08x\n", 
			header.ras_magic, RAS_MAGIC);
		return 0;
	}
	if ( header.ras_depth < 24 )
	{
		fprintf(stderr, "Bad depth %d\n", header.ras_depth);
		return 0;
	}

	/*  READ IN COLOR MAP IF THERE IS ONE. */

	if ( header.ras_maplength != 0 )
	{
		if ( !(p_cmap = (u_int*)malloc(header.ras_maplength)) )
		{
			fprintf(stderr, "Unable to allocate %d byte buffer for cmap\n",
				header.ras_maplength);
			return 0;
		}
		n_read = fread(p_cmap, 1, header.ras_maplength, fd); 
		if ( n_read != header.ras_maplength )
		{
			fprintf(stderr, "Short cmap length expected %d bytes got %d\n",
				header.ras_maplength, n_read);
			return 0;
		}
	}


	/*  ALLOCATE BUFFER FOR RASTER DATA AND READ IT IN. */

	if ( !(p_im_data8 = (u_int*)malloc(header.ras_length)) )
	{
		fprintf(stderr, "Unable to allocate %d byte buffer for image\n",
			header.ras_length);
		return 0;
	}
	n_read = fread(p_im_data8, 1, header.ras_length, fd); 
	if ( n_read != header.ras_length )
	{
		fprintf(stderr, "Short data length expected %d bytes got %d\n",
			header.ras_length, n_read);
		return 0;
	}


	/*  IF THE RASTER DATA IS 8-BIT PSEUDO COLOR CONVERT IT TO TRUE COLOR. */

	if ( header.ras_maplength != 0 )
	{
		if ( !(p_im_data32 = (u_int*)malloc(header.ras_length*sizeof(int))) )
		{
			fprintf(stderr, "Unable to allocate %d byte buffer for image\n",
				header.ras_length*sizeof(int));
			return 0;
		}
		for ( i_image = 0; i_image < header.ras_length; i_image++ )
		{
			p_im_data32[i_image] = p_cmap[p_im_data8[i_image]];
		}
		free((char*)p_cmap);
		free(p_im_data8);
	}


	/*  TRUE COLOR RASTER DATA, JUST USE IT. */

	else
	{
		p_im_data32 = (int*)p_im_data8;
	}


	/*  BUILD X IMAGE AND RETURN IT TO OUR CALLER. */

	*p_width = header.ras_width;
	*p_height = header.ras_height;

	return XCreateImage(p_display, 
		XDefaultVisual(p_display, XDefaultScreen(p_display)), 
		32, ZPixmap, 0, p_im_data32, *p_width, *p_height, 32, 0);
}


print_event(fd, event)
	FILE*	fd;
	XEvent*	event;
{
	switch( event->type )
	{
	  case KeyPress:            fprintf(fd, "KeyPress");			break;
	  case MappingNotify:       fprintf(fd, "MappingNotify");      	break;
	  case KeyRelease:          fprintf(fd, "KeyRelease");			break;
	  case ButtonPress:         fprintf(fd, "ButtonPress");			break;
	  case ButtonRelease:       fprintf(fd, "ButtonRelease");		break;
	  case MotionNotify:        fprintf(fd, "MotionNotify");		break;
	  case EnterNotify:         fprintf(fd, "EnterNotify");			break;
	  case LeaveNotify:         fprintf(fd, "LeaveNotify");			break;
	  case FocusIn:             fprintf(fd, "FocusIn");				break;
	  case FocusOut:            fprintf(fd, "FocusOut");			break;
	  case KeymapNotify:        fprintf(fd, "KeymapNotify");		break;
	  case Expose:              fprintf(fd, "Expose");				break;
	  case GraphicsExpose:      fprintf(fd, "GraphicsExpose");		break;
	  case NoExpose:            fprintf(fd, "NoExpose");			break;
	  case VisibilityNotify:    fprintf(fd, "VisibilityNotify");	break;
	  case CreateNotify:        fprintf(fd, "CreateNotify");		break;
	  case DestroyNotify:       fprintf(fd, "DestroyNotify");		break;
	  case UnmapNotify:         fprintf(fd, "UnmapNotify");			break;
	  case MapNotify:           fprintf(fd, "MapNotify");			break;
	  case MapRequest:      	fprintf(fd, "MapRequest");			break;
	  case ReparentNotify:      fprintf(fd, "ReparentNotify");		break;
	  case ConfigureNotify:     fprintf(fd, "ConfigureNotify");		break;
	  case ConfigureRequest:    fprintf(fd, "ConfigureRequest");	break;
	  case GravityNotify:       fprintf(fd, "GravityNotify");		break;
	  case ResizeRequest:       fprintf(fd, "ResizeRequest");		break;
	  case CirculateNotify:     fprintf(fd, "CirculateNotify");		break;
	  case CirculateRequest:    fprintf(fd, "CirculateRequest");	break;
	  case PropertyNotify:      fprintf(fd, "PropertyNotify");		break;
	  case SelectionClear:      fprintf(fd, "SelectionClear");		break;
	  case SelectionRequest:    fprintf(fd, "SelectionRequest");	break;
	  case SelectionNotify:     fprintf(fd, "SelectionNotify");		break;
	  case ColormapNotify:      fprintf(fd, "ColormapNotify");		break;
	  case ClientMessage:       fprintf(fd, "ClientMessage");		break;
	}
}
