/*
 *  xvsource.c:	Program to set input using Xv video extension access object.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     28 May 93 - Andy Goodrich
 *       - Initial coding
 *
 * $Log:   /plxhp/dave/pvcsrc/O_XV/archives/xvsource.c_v  $
 * 
 *    Rev 1.1   09 Sep 1996 09:56:28   dave
 * Initial revision
 * Revision 1.1  1995/05/16 10:10:42  dist
 * Initial revision
 *
 *
 *  Copyright(c) 1993, Andrew C. Goodrich
 *  Copyright(c) 1995, Parallax Graphics, Inc.
 */
#include <sys/types.h>			/* Standard types for this C environment. */
#include <stdio.h>				/* Stream input/output declarations. */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <Xvlib.h>
#include "Xvddx.h"
#include "Xvddx_PGI.h"
#include "O_XV.h"
#include "arg_parse.h"

/*
 *  ARGUMENT PARSING TABLES
 */
static int		adaptor;	/* Xv adaptor to create object for. */
static int		encoding;	/* Input encoding (e.g., composite). */
static int		encoding_id;	/* Xv encoding id (e.g., 0x0001011). */
static int		dmsd;		/* Input dmsd. */
static int		instance;	/* Input instance. */
static int		port;		/* Xv port to create object for. */
static int		sleep_time;	/* Amount to sleep before checking signal. */
static int		timing;		/* Input timing (e.g., ntsc). */

static Pos_Arg pos_args[] =
{
	{Pos_Integer, &instance, 0},
	{Pos_Fence, 0, 0}
};
static Opt_Arg	opt_args[] =
{
	{Opt_Integer,  "adaptor", &adaptor, 0},
	{Opt_No_Value, "composite", &encoding, XV_COMPOSITE},
	{Opt_Integer,  "dmsd", &dmsd, 0},
	{Opt_Integer,  "id", &encoding_id, 0},
	{Opt_No_Value, "ntsc", &timing, XV_NTSC},
	{Opt_No_Value, "pal", &timing, XV_PAL},
	{Opt_Integer,  "port", &port, 0},
	{Opt_No_Value, "secam", &timing, XV_SECAM},
	{Opt_Integer,  "sleep", &sleep_time, 0},
	{Opt_No_Value, "yc", &encoding, XV_YC},
	{Opt_Fence, 0, 0,  0}
};

/*----------------------------------------------------------------------------*
 |"main"
 |
 |  Entry point for system for this application.
 |
 *----------------------------------------------------------------------------*/
main( argc, argv )
	int		argc;		/* = # of command arguments parsed by shell. */
	char	*argv[];	/* = vector of pointers to command arguments parsed. */
{
	Display*		display;	/* Display accessing. */
	int				i_argv;		/* # of element in 'argv' accessing. */
	int				i_poll;		/* # of status poll performing. */
	O_XV*			p_xv;		/* Xv access object. */	
	int				status;		/* Status returned by XV for port. */
	
	/*
	 *  PROCESS COMMAND LINE ARGUMENTS:
	 *
	 *    xvsource timing encoding [instance]
	 */
	adaptor = 0;
	dmsd = 0;
	encoding = 0;
	encoding_id = 0;
	instance = 0;
	port = 0;
	sleep_time = 0;
	if ( (argc == 1) || !arg_parse( &pos_args, &opt_args, argc, argv ) )
	{
		fprintf(stderr, "Usage: xvsource [adaptor #] [port #] ntsc|pal ");
		fprintf(stderr, "composite|yc [dmsd #] [#]\n");
		fprintf(stderr, "  The # without a keyword is the input number\n");
		fprintf(stderr, "  # values default to 0\n");
		return 0;
	}

	/*
	 *  OPEN THE DISPLAY, CREATE OUR XV OBJECT AND SET THE SOURCE
	 *  DETERMINED FROM THE COMMAND LINE ARGUMENTS.
	 */
	if ( !(display = XOpenDisplay(0)) ) 
	{
		fprintf(stderr, "\nCould not open display\n");
		return 0;
	}

	if ( !(p_xv = O_XV_O_XV( 0, display, adaptor, port )) )
	{
		fprintf(stderr, "\nCould not open display\n");
		return 0;
	}

	if ( encoding_id ) 
    {
	    timing = encoding_id & XV_SOURCE_TIMING;
        encoding = encoding_id & XV_SOURCE_TYPE;
        dmsd = (encoding_id & XV_PLX_DMSD) >> 4;
		instance = encoding_id & XV_PLX_INPUT;
    }
	printf("Setting port %d (id %d) to %08x\n", port, p_xv->port_id, 
	    (timing | encoding | (dmsd << 4) | instance));

	O_XV_Source( p_xv, dmsd, timing, encoding, instance );

	if ( sleep_time ) sleep(sleep_time);
	for ( i_poll = 0; i_poll < 1; i_poll++ ) 
	{
		status = O_XV_Status( p_xv );
		printf("Pass %d status is: %08x %s\n", i_poll, status, status & 2 ?
			"SYNC" : "NO SYNC");
	}
}
