/*
 *  pvpci_mmap.c:	Implementation For PCI PowerVideo Memory Mapping Object
 *
 *
 * (c) Copyright 1994 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log:   /plxhp/dave/pvcsrc/hwlib/archives/pvpci_mmap.c_v  $
 * 
 *    Rev 1.1   09 Sep 1996 10:13:56   dave
 * Initial revision
 * Revision 1.1  1995/12/08 10:18:59  dist
 * Initial revision
 *
 * Revision 1.2  95/09/13  10:01:44  10:01:44  dist ()
 * Andy Goodrich: Changed exit calls in to returns of 0 for object creation
 *                so that the X server will operate correctly when no 
 *                Parallax card is found.
 * 
 * Revision 1.1  95/06/02  11:51:25  11:51:25  dist ()
 * Initial revision
 * 
 */
#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
/* #include <time.h> */
#include "objects.h"
#include "xvdebug.h"
#include "video_stream.h"
#include "pvioctl.h"
#define XV_DIAG 1
#include "xvdebug.h"

#define PAUSE(string) {printf(string); fgetc(stdin);}

/*---------------------------------------------------------------------------*
 |"m_Mmap_dump"
 |
 |  Dump the contents of a memory map object.
 |
 *---------------------------------------------------------------------------*/
static void m_Mmap_dump( p_me, fd )
	O_Mmap*	p_me;	/* -> mmap object instance to dump contents of. */
	FILE*	fd;		/* -> file descriptor to dump contents to. */
{
	fprintf(fd, "\nDump of memory map object at %08x:\n", p_me);
	fprintf(fd, "  Region    Virtual   Real(Sbus)  Length\n");
	fprintf(fd, "  ------    -------   ----------  ------\n");
	fprintf(fd, "  ccube     %08x  %08x    %08x\n", 
		p_me->maps[MM_Ccube].virtual, p_me->maps[MM_Ccube].sbus, 
		p_me->maps[MM_Ccube].length);
	fprintf(fd, "  eprom     %08x  %08x    %08x\n", 
		p_me->maps[MM_Eprom].virtual, p_me->maps[MM_Eprom].sbus, 
		p_me->maps[MM_Eprom].length);
	fprintf(fd, "  1-bit     %08x  %08x    %08x\n", 
		p_me->maps[MM_M1].virtual, p_me->maps[MM_M1].sbus, 
		p_me->maps[MM_M1].length);
	fprintf(fd, "  8-bit     %08x  %08x    %08x\n", 
		p_me->maps[MM_M8].virtual, p_me->maps[MM_M8].sbus, 
		p_me->maps[MM_M8].length);
	fprintf(fd, "  24-bit    %08x  %08x    %08x\n", 
		p_me->maps[MM_M24].virtual, p_me->maps[MM_M24].sbus, 
		p_me->maps[MM_M24].length);
	fprintf(fd, "  regs      %08x  %08x    %08x\n", 
		p_me->maps[MM_Regs].virtual, p_me->maps[MM_Regs].sbus, 
		p_me->maps[MM_Regs].length);
}

/*---------------------------------------------------------------------------*
 |"m_Mmap_real"
 |
 |  Return the real (sbus) address corresponding to a virtual address.
 |
 |	  =  real (sbus) address or -1 on failure.
 *---------------------------------------------------------------------------*/
static u_int m_Mmap_real( p_me, virtual )
	O_Mmap*	p_me;		/* Object initializing. */
	u_int	virtual;	/* = virtual address to get real (sbus) address for. */
{
	int		i_reg;	/* Entry in 'maps' now accessing. */

	for ( i_reg = 0; i_reg < nMM; i_reg++ )
	{
		if ( p_me->maps[i_reg].virtual <= virtual && 
			virtual < p_me->maps[i_reg].virtual + p_me->maps[i_reg].length )
		{
			return p_me->maps[i_reg].sbus + 
				(virtual - p_me->maps[i_reg].virtual);
		}
	}
	fprintf(stderr, "Unable to find map for address %08x\n", virtual);
	return -1;
}


/*---------------------------------------------------------------------------*
 |"m_Mmap_terminate"
 |
 |  Return the virtual address corresponding to a real (SBUS) address.
 |
 |    =  virtual address or -1 on failure.
 *---------------------------------------------------------------------------*/
static void m_Mmap_terminate( p_me )
    O_Mmap  *p_me;  /* Object initializing. */
{
	close(p_me->fd_ccube);
	free((char*)p_me);
}


/*---------------------------------------------------------------------------*
 |"m_Mmap_virtual"
 |
 |  Return the virtual address corresponding to a real (SBUS) address.
 |
 |	  =  virtual address or -1 on failure.
 *---------------------------------------------------------------------------*/
static u_int m_Mmap_virtual( p_me, real )
	O_Mmap*	p_me;	/* Object initializing. */
	u_int	real;	/* = real (sbus) address to get virtual address for. */
{
	int		i_reg;	/* Entry in 'maps' now accessing. */

	for ( i_reg = 0; i_reg < nMM; i_reg++ )
	{
		if ( p_me->maps[i_reg].sbus <= real && 
			 real < p_me->maps[i_reg].sbus + p_me->maps[i_reg].length )
		{
			return p_me->maps[i_reg].virtual + 
				(real - p_me->maps[i_reg].sbus);
		}
	}
	fprintf(stderr, "Unable to find map for address %08x\n", real);
	return -1;
}


static Tv_Mmap methods_tv = {
	m_Mmap_dump,
	m_Mmap_real,
	m_Mmap_terminate,
	m_Mmap_virtual
};

/*---------------------------------------------------------------------------*
 |"O_Mmap_O_Mmap_No_Driver"
 |
 |  Create and return a Mmap facility access object when there is no
 |  driver in the system. 
 |
 |	  -> object on success, 0 on failure.
 *---------------------------------------------------------------------------*/
O_Mmap *O_Mmap_O_Mmap_No_Driver()
{
#ifdef AIX
	fprintf(stderr, "O_Mmap_O_Mmap_No_Driver called\n");
	return NULL;
#else
	int			i_map;	/* Memory map entry accessing. */
	O_Mmap*		p_me;	/* Object initializing. */
	XV_mem_map*	xvm;

	/*
	 *  ALLOCATE SPACE FOR OUR OBJECT AND INITIALIZE THE METHODS
	 *  TRANSFER VECTOR. DEFAULT MAP ENTRIES SO THAT FAILING MAPS 
	 *  WILL NOT MATCH VALID ADDRESSES IN real AND virtual METHODS.
	 */
	if ( !(p_me = (O_Mmap *)malloc(sizeof(O_Mmap))) )
	{
		fprintf(stderr, "Unable to allocate O_Mmap object\n");
		return 0;
	}
	p_me->atv = &methods_tv;

	for ( i_map = 0; i_map < nMM; i_map++ )
	{
		p_me->maps[i_map].virtual = 0xffffffff;
		p_me->maps[i_map].sbus =    0xffffffff;
		p_me->maps[i_map].length =  0x00000000;
	}
		
	/*
	 *  MAP IN CONTROL REGISTER AREA.
	 */
	if ( (p_me->fd_regs= open("/dev/pv0",O_RDWR)) == -1 )
	{
	    printf("Can't open device /dev/pv0\n");
	    exit(-1);
	}
	if ( ioctl(p_me->fd_regs,PV_MMAP,&i_map)  < 0)
	{
		fprintf(stderr, "Unable to memory map registers!!!\n");
	}
	i_map += s
	if ( ioctl(p_me->fd_regs,PV_MMAP,&p_me->maps[MM_Regs].virtual) )
	p_me->maps[MM_Regs].sbus =   0x00040000;
	p_me->maps[MM_Regs].length = 0x00000800;

	/*
	 *  MAP IN EEPROM AREA.
	 */
	if ( (p_me->fd_eprom= open("/dev/plx_eprom",O_RDWR)) == -1 )
	{
	    printf("Can't open device /dev/plx_eprom\n");
	    exit(-1);
	}
	if (ioctl(p_me->fd_eprom,IOMAPMAP,&p_me->maps[MM_Eprom].virtual))
	{
		fprintf(stderr, "Unable to memory map eeprom!!!\n");
	}
	p_me->maps[MM_Eprom].sbus =   0x00000000;
	p_me->maps[MM_Eprom].length = 0x00001000;

	/*
	 *  MAP IN CCUBE AREA.
	 */
	if ( (p_me->fd_ccube = open("/dev/plx_ccube",O_RDWR)) == -1 )
	{
	    printf("Can't open device /dev/plx_ccube\n");
	    exit(-1);
	}
	if (ioctl(p_me->fd_ccube,IOMAPMAP,&p_me->maps[MM_Ccube].virtual))
	{
		fprintf(stderr, "Unable to memory map ccube!!!\n");
	}
	p_me->maps[MM_Ccube].sbus =   0x00050000;
	p_me->maps[MM_Ccube].length = 0x00001000;

	/*
	 *  MAP IN 1-BIT MEMORY AREA.
	 */
	if ( (p_me->fd_m1 = open("/dev/plx_m1",O_RDWR)) == -1 )
	{
	    printf("Can't open device /dev/plx_m1\n");
	    exit(-1);
	}
	if ( ioctl(p_me->fd_m1,IOMAPMAP,&p_me->maps[MM_M1].virtual) )
	{
		fprintf(stderr, "Unable to memory map 1-bit memory!!!\n");
	}
	p_me->maps[MM_M1].sbus =   0x00010000;
	p_me->maps[MM_M1].length = 0x00030000;


	/*
	 *  MAP IN 24-BIT MEMORY AREA.
	 */
	if ( (p_me->fd_m24 = open("/dev/plx_m24",O_RDWR)) == -1 )
	{
	    printf("Can't open device /dev/plx_m24\n");
	    exit(-1);
	}
	if ( ioctl(p_me->fd_m24,IOMAPMAP,&p_me->maps[MM_M24].virtual) )
	{
		fprintf(stderr, "Unable to memory map 24-bit memory!!!\n");
	}
	p_me->maps[MM_M24].sbus =   0x00200000;
	p_me->maps[MM_M24].length = 0x00500000;

	return p_me;
#endif /* AIX */
}

/*---------------------------------------------------------------------------*
 |"O_Mmap_O_Mmap"
 |
 |  Create and return a Mmap facility access object. 
 |
 |	  -> object on success, 0 on failure.
 *---------------------------------------------------------------------------*/
O_Mmap *O_Mmap_O_Mmap(device)
	char*			device;		/* Name of the device we are opening. */
{
	int			    fd;			/* File descriptor for device. */
	int		 		i_map;		/* Memory map entry accessing. */
	O_Mmap*	   		p_me;		/* Object initializing. */
	XV_mem_map* 	xv_map;		/* Layout of X video board. */

	if ( !device ) device = "/dev/pv0";

	/*  OPEN DRIVER AND GET MEMORY MAPPING INFORMATION */

    if((fd = open(device, O_RDWR)) < 0)
    {
		ERRORF((fd, "Could not open device driver!!!!\n"));
		return 0;
    }

    if(ioctl(fd, PV_MMAP, &xv_map) < 0)
    {
        perror("PV_MMAP");
        return 0;
    }
	

	/*
	 *  ALLOCATE SPACE FOR OUR OBJECT AND INITIALIZE THE METHODS
	 *  TRANSFER VECTOR. DEFAULT MAP ENTRIES SO THAT FAILING MAPS 
	 *  WILL NOT MATCH VALID ADDRESSES IN real AND virtual METHODS.
	 */
	if ( !(p_me = (O_Mmap *)malloc(sizeof(O_Mmap))) )
	{
		fprintf(stderr, "Unable to allocate O_Mmap object\n");
		return 0;
	}
	p_me->atv = &methods_tv;

	for ( i_map = 0; i_map < nMM; i_map++ )
	{
		p_me->maps[i_map].virtual = 0xffffffff;
		p_me->maps[i_map].sbus =    0xffffffff;
		p_me->maps[i_map].length =  0x00000000;
	}
		
	/*
     *  FILL IN THE ENTRIES IN OUR MEMORY MAP OBJECT:
     */
	p_me->maps[MM_Eprom].virtual = &xv_map->Prom0;
	p_me->maps[MM_Eprom].sbus =    0x00000000;
	p_me->maps[MM_Eprom].length =  0x00001000;

	p_me->maps[MM_Regs].virtual = &xv_map->RW_regs;
	p_me->maps[MM_Regs].sbus =    0x00040000;
	p_me->maps[MM_Regs].length =  0x00000800;

	p_me->maps[MM_Ccube].virtual = &xv_map->Ccube;
	p_me->maps[MM_Ccube].sbus =    0x00050000;
	p_me->maps[MM_Ccube].length =  0x00001000;

	p_me->maps[MM_M1].virtual = &xv_map->M1;
	p_me->maps[MM_M1].sbus =    0x00010000;
	p_me->maps[MM_M1].length =  0x00030000;

	p_me->maps[MM_M8].virtual = &xv_map->M8;
	p_me->maps[MM_M8].sbus =    0x00080000;
	p_me->maps[MM_M8].length =  0x00140000;

	p_me->maps[MM_M24].virtual = &xv_map->M24;
	p_me->maps[MM_M24].sbus =    0x00200000;
	p_me->maps[MM_M24].length =  0x00500000;

	p_me->fd_ccube = fd;

	strcpy(p_me->name, device);
	DEBUGF(1, (fd, "Device name is %s\n"));

	return p_me;
}
