#include <stdio.h>
#include <fcntl.h>
#ifndef _AIX
#include <sys/framebuf.h>
#endif /* _AIX */
#include "video_stream.h"
#include "hw_context.h"
#include "c3.h"
#ifndef DRIVER
#include <sys/time.h>
#endif /* DRIVER */

#define NVRAM_CONFIG
#define TICKS_PER_HOUR (90000 * 10)

#ifdef XV_DIAG
#include "xvdebug.h"
#endif

#ifdef DEBUG
#define dprintf(a) printf(a)
#else
#define dprintf(a) 
#endif

/******************************************/
/* ALL OF THE GLOBAL POINTERS DEFINED HERE */
int *xv_regbase,*xv_m1base,*xv_m24base,*xv_c3base,*xv_eprombase;
int plxv_addr_wrap = 0x100000;
#ifdef NOTDEF
char *xv_m8base;
XV_write_regs *xv_wreg;
XV_read_regs *xv_rreg;
C3_RREGS *c3_rreg;
C3_WREGS *c3_wreg;
XV_shdw_regs *xv_shdw;
XV_struct *xv_cp;
XV_addr_maps *xv_ap;
XV_mem_map *xvm;
#endif /* NOTDEF */
/******************************************/

XV_video_format loop_format = {640,482,43,7,21};

XV_struct *plxv_init_from_nvram();

plxv_inc_powerups(p_me)
    XV_struct* p_me;
{
    int i,j;

    i = NV_POSITION(p_me, main,power_cycles);
    p_me->main.power_cycles += 1;
    if (O_I2C_nv_write(p_me->p_i2c, 0,&(p_me->main.power_cycles),i,sizeof(short)))
    {
        printf("plxv_inc_powerups: can't write option nvram\n");
        return(-1);
    }
    if (p_me->opt.board_code)
    {   /* service the option card (its there ) */
        p_me->opt.power_cycles += 1;
        i = NV_POSITION(p_me, opt,power_cycles);
        if (O_I2C_nv_write(p_me->p_i2c, 1,&(p_me->opt.power_cycles),i,sizeof(short)))
        {
            printf("plxv_inc_powerups: can't write option nvram\n");
            return(-1);
        }
    }
    return(0);
}

#ifdef DRIVER
XV_struct *plxv_init_drvr(reg,m1,m8,m24,c3,eprom)
unsigned long *reg,*m1,*m24,*c3,*eprom;
unsigned char *m8;
{
        XV_struct* p_me;


    p_me = plxv_init_context(reg,m1,m8,m24,c3,eprom);

    plxv_init_bus(p_me);

    plxv_init_disp(p_me);


    plxv_init_draw(p_me);
    plxv_init_timer(p_me);
    plxv_init_video(p_me);
    plxv_init_bt(p_me, 0);
    plxv_mode_24(p_me);
    return(p_me);
}

plxv_set_hourint(p_me)
    XV_struct* p_me;
{
    static unsigned int next_intr_time = 0;
    unsigned int plxv_read_timer();

    if (next_intr_time == 0) next_intr_time  = plxv_read_timer(p_me);
    next_intr_time  += TICKS_PER_HOUR;
    PV_WREG(p_me,TmrInt.l,next_intr_time);
    /*
    PV_RREG(p_me,i,TmrIntRB.l);
    */
    PV_WREG(p_me,BIntMask.b.timer,1);
}
    
plxv_inc_hours(p_me)
    XV_struct* p_me;
{
#ifdef NOTDEF
    int i,j;

    i = NV_POSITION(p_me, main,power_hours);
#endif
    p_me->main.power_hours += 1;
    if (p_me->opt.board_code)
    {   /* service the option card (its there ) */
        p_me->opt.power_hours += 1;
    }
    return(0);
}

#endif

plxv_init_bus(p_me)
    XV_struct* p_me;
{
    int k,i;


        TestHwr(p_me);
        PV_WREG(p_me,StartUp.l,0x8000618);
        for(i=0;i<10000;i++);
        PV_WREG(p_me,StartUp.l,0x8000608);
        for(i=0;i<10000;i++);
        PV_WREG(p_me,DispFlgs.l,0xa28);

    PV_WREG(p_me,Attr.l,p_me->main.Attr);
    
    PV_WREG(p_me,BIntMask.l,0);
    PV_WREG(p_me,IntMask.l,0x8000);
    PV_WREG(p_me,RasMode.l,p_me->main.RasMode);
}
int
TestHwr(p_me)
        XV_struct* p_me;
{
        int i;
        if (!TestHwrWord(p_me,0)) return(0);
        if (!TestHwrWord(p_me,-1)) return(0);
        if (!TestHwrWord(p_me,0x480)) return(0);
        for (i=0; i<32; i++) {
            if (!TestHwrWord(p_me,1<<i)) return(0);
            if (!TestHwrWord(p_me,~(1<<i))) return(0);
        }
        return(1);
}       
TestHwrWord(p_me,x)
XV_struct* p_me;
int x;
{
    int i;
    PV_WREG(p_me,TmrInt.l,x);
    PV_RREG(p_me,i,TmrIntRB.l);
    if (x!=i) {
        printf("Data Bus Error: wrote %x read %x\n",x,i);
        return (0);
    }
    return (1);
}
plxv_init_timer(p_me)
    XV_struct* p_me;
{
    /* init the timer */
    PV_WREG(p_me,TmrSpd.l,p_me->main.TmrInit);
}


unsigned int plxv_read_timer(p_me, a)
    XV_struct* p_me;
int a;
{
    int i;
    PV_RREG(p_me,i,TmrIntRB.l);
    PV_RREG(p_me,i,TmrCnt.l);
    return(i);
}

plxv_init_disp(p_me)
    XV_struct* p_me;
{
    short i;
    PV_WREG(p_me,StartUp.l,p_me->main.StartUp);
    PV_WREG(p_me,DispFlgs.l,p_me->main.DispFlgs);
    PV_WREG(p_me,DispKludge.l,p_me->main.DispKludge);
    PV_WREG(p_me,HFP.l,p_me->main.h.fp);
    PV_WREG(p_me,HSync.l,p_me->main.h.sy);
    PV_WREG(p_me,HBP.l,p_me->main.h.bp);
    PV_WREG(p_me,HDisp.l,p_me->main.h.disp);  
    PV_WREG(p_me,VFP.l,p_me->main.v.fp); 
    PV_WREG(p_me,VBP.l,p_me->main.v.bp);
    PV_WREG(p_me,VDisp.l,p_me->main.v.disp);
    PV_WREG(p_me,VSync.l,p_me->main.v.sy);
    i = p_me->main.h.hsync_ser;
    if(i & 0x80) i += 0x80;
    PV_WREG(p_me,HSyncS.l,i);
    PV_WREG(p_me,HNSyncS.l,p_me->main.h.hsync_nser);
    /*
    PV_WREG(p_me,OLDI2C.l,p_me->main.OLDI2C);
    PV_WREG(p_me,DispFlgs.l,0xa28);
    PV_WREG(p_me,StartUp.l,0x8000608);
    */
}

/* function to initialize context structure and 
   SWITCH CONTEXT TO THE INDICATED BOARD          */

XV_struct *plxv_bare_context(reg,m1,m8,m24,c3,eprom)
int *reg,*m1,*m8,*m24,*c3,*eprom;
{
    int Xres,Yres,i,*ip,j;
    XV_struct      *cp;
    XV_shdw_regs   *sp;
    XV_addr_maps   *ap;

#ifdef DRIVER
    if ((cp = (XV_struct *)sys_memall(sizeof(XV_struct))) == NULL)
    {   /* malloc failed */
        printf("malloc failed on plxv_init_struct:main struct\n");
        exit(-1);
    }
    if ((sp = (XV_shdw_regs *)sys_memall(sizeof(XV_shdw_regs))) == NULL)
    {   /* malloc failed */
        printf("malloc failed on plxv_init_struct:xv_shdw\n");
        exit(-1);
    }
    if ((ap = (XV_addr_maps *)sys_memall(sizeof(XV_addr_maps))) == NULL)
    {   /* malloc failed */
        printf("malloc failed on plxv_init_struct:addr_maps\n");
        exit(-1);
    }
#else
    if ((cp = (XV_struct *)malloc(sizeof(XV_struct))) == NULL)
    {   /* malloc failed */
        printf("malloc failed on plxv_init_struct:main struct\n");
        exit(-1);
    }
    if ((sp = (XV_shdw_regs *)malloc(sizeof(XV_shdw_regs))) == NULL)
    {   /* malloc failed */
        printf("malloc failed on plxv_init_struct:xv_shdw\n");
        exit(-1);
    }
    if ((ap = (XV_addr_maps *)malloc(sizeof(XV_addr_maps))) == NULL)
    {   /* malloc failed */
        printf("malloc failed on plxv_init_struct:addr_maps\n");
        exit(-1);
    }
#endif

    /* fill in the structure */
    cp->c3_rreg = (C3_RREGS *)c3;
    cp->c3_wreg = (C3_WREGS *)c3;
    cp->eprombase = eprom;
    cp->regbase = reg;
    cp->c3base = c3;
    cp->m24base = m24;
    cp->m8base = m8;
    cp->m1base = m1;
    cp->xv_shdw = (XV_shdw_regs*)sp;
    cp->xv_wreg = (XV_write_regs*)reg;
    cp->xv_rreg = (XV_read_regs*)reg;
    cp->maps = ap;
        
    /* init the REGISTER FILE shadow ram structure */
    j = sizeof(XV_shdw_regs) >> 2;
    for (ip = (int *)sp, i = 0; i++; i<j) *ip++ = 0;

#ifdef NOTDEF
    cp->p_i2c = O_I2C_O_I2C(cp);
	cp->p_rsrc = O_Rsrc_O_Rsrc_pv700();
#endif /* NOTDEF */
	return cp;
}

#ifdef NOTDEF_ANDY
XV_struct *plxv_init_context(reg,m1,m8,m24,c3,eprom)
int *reg,*m1,*m8,*m24,*c3,*eprom;
{
    XV_struct      *cp;

	cp = plxv_bare_context(reg, m1, m8, m24, c3, eprom);
	printf("before resource initcontext\n");
	O_Rsrc_init_context(cp->p_rsrc, cp);
	printf("after resource initcontext\n");
#ifdef NOTDEF
    plxv_init_from_nvram(cp);
#endif
    cp->plxv_io_format[5] = &loop_format;
    cp ->format_v_ptr = &(cp->plxv_io_format[5]);
    cp ->format_w_ptr = &(cp->plxv_io_format[5]);
    return cp;
}

XV_struct *plxv_init_from_nvram(cp)
    XV_struct *cp;
{
    int Xres,Yres,i,j;
    XV_addr_maps   *ap;

    ap = cp->maps;

#ifndef NVRAM_CONFIG

    plxv_load_struct(cp);

#else /* NVRAM_CONFIG */

    /* load in the nvram structures from the board */
    if (O_I2C_nv_read(cp->p_i2c, 0,&(cp->main),0,sizeof(XV_controller_nvram)))
    {   /* read error on nvram */
        printf("Error: can't read controller nvram \n");
        cp->main.board_code = 0;
        plxv_load_struct(cp);
#ifdef NOTDEF
    } else if (cp->main.hlib_rev != HLIB_REV)
    {   /* oops new library has differences in NVRAM STRUCT */
        int size,i;
        unsigned char save_buf[100],*cp1;

        dprintf("moving controller NV to new hardware library revision\n");

        /* save the first 14 bytes of NVRAM */
        cp1 = (unsigned char *)(&cp->main);
        size = ((unsigned char *)(&cp->main.xsize)) - (cp1);
        for (i=0;i<size;i++) save_buf[i] = *cp1++;

        /* load the new defaults            */
        plxv_init_ram(cp,cp->main.board_code,0);

        /* put back the first 14 bytes      */
        cp1 = (unsigned char *)(&cp->main);
        for (i=0;i<size;i++) *cp1++ = save_buf[i];;

        /* poke the new HLIB revision */
        cp->main.hlib_rev = HLIB_REV;

        /* write the NVRAM */
        dprintf("Writing update NV structure to controller\n");

        if (O_I2C_nv_write(cp->p_i2c, 0,&(cp->main),0,sizeof(XV_controller_nvram)))
        {
            printf("main board nvram not responding to write\n");
            return(cp);
        }
#endif
    }
#ifdef OPTION_NVRAM_LOADED
    if (O_I2C_nv_read(cp->p_i2c, 1,&(cp->opt),0,sizeof(XV_option_nvram)))
    {   /* read error on nvram */
        dprintf("No option card present\n");
        cp->opt.board_code = 0;
        cp->opt.clt_type = 0;
#ifdef NOTDEF
    } else if (cp->opt.hlib_rev != HLIB_REV)
    {   /* oops new library has differences in NVRAM STRUCT */
        int size,i;
        unsigned char save_buf[100],*cp1;

        dprintf("moving option card NV to new hardware library revision\n");

        /* save the first 14 bytes of NVRAM */
        cp1 = (unsigned char *)(&cp->opt);
        size = ((unsigned char *)(&cp->opt.unused_2[0])) - (cp1);
        for (i=0;i<size;i++) save_buf[i] = *cp1++;

        /* load the new defaults            */
        plxv_init_ram(cp,cp->opt.board_code,1);

        /* put back the first 14 bytes      */
        cp1 = (unsigned char *)(&cp->opt);
        for (i=0;i<size;i++) *cp1++ = save_buf[i];;

        /* poke the new HLIB revision */
        cp->opt.hlib_rev = HLIB_REV;

        /* write the NVRAM */
        dprintf("Writing update NV structure to option card\n");
        if (O_I2C_nv_write(cp->p_i2c, 1,&(cp->opt),0,sizeof(XV_option_nvram)))
        {
            printf(" option board nvram not responding to write\n");
            return(cp);
        }
#endif

    } else printf("option board %x installed\n",cp->opt.board_code);

#else   /* !OPTION_NVRAM_LOADED */

    if ( cp->main.misc_flags & (PV700_DMSD2 | PV700_DMSE) )
    {
        plxv_init_pv700_option_ram( cp, OPT_1, 0 );
    }
#endif  /* OPTION_NVRAM_LOADED */

#endif  /*NVRAM_CONFIG*/

    Xres = cp->main.xsize;
    Yres = cp->main.ysize;

    if ( Xres == 1152) cp->addr_wrap = 0x100000;
    if ( Xres == 1280) cp->addr_wrap = 0x140000;

    for (i=0;i<Yres;i++) ap->reg[i] =  (Xres * i);
    for (i=0;i<Yres;i++) ap->m24[i] = (int *)( (int) cp->m24base + ((Xres*4) * i));
    for (i=0;i<Yres;i++) ap->m1[i] = (int *)( (int) cp->m1base + ((Xres/8) * i));
    return(cp);
}
#endif /* NOTDEF_ANDY */

/* routine to switch hardware library to new controller */
plxv_switch_context(cp)
XV_struct *cp;
{

        return; 
#ifdef NOTDEF
    if (xv_cp == cp) return; else
    {
        xv_cp = cp;
        xv_regbase = cp->regbase;
        xv_wreg = (XV_write_regs *)cp->regbase;
        xv_rreg = (XV_read_regs *)cp->regbase;
        xv_eprombase = cp->eprombase;
        xv_c3base = cp->c3base;
        c3_rreg = (C3_RREGS *)xv_c3base;
        c3_wreg = (C3_WREGS *)xv_c3base;
        xv_m24base = cp->m24base;
        xv_m8base = cp->m8base;
        xv_m1base = cp->m1base;
        xv_shdw = (XV_shdw_regs *)cp->xv_shdw;
        xv_ap = cp->maps;
    }
#endif /* NOTDEF */
}
#ifndef DRIVER
#if !defined(sun) && !defined(_AIX)
int  usleep( timer_micros )
        int     timer_micros;
{
        struct timeval  to;             /* Timer value for wait. */

        to.tv_sec = 0;
        to.tv_usec = timer_micros;
        select(0,0,0,0,&to);
}
#endif /* sun */
#endif /* DRIVER */

static int mungeword(in)
int in;
{
    typedef union w
    {
        int i;
        char c[4];
    };

    union w munged, *j;
    j = &in;

    munged.c[0] = j->c[3];
    munged.c[1] = j->c[2];
    munged.c[2] = j->c[1];
    munged.c[3] = j->c[0];

    return munged.i;
}

typedef struct
{
    unsigned long start_address;
    unsigned long counters;
    unsigned long control;
    unsigned long counter_control;
} dma_ctl;

plxv_dma_setup(p_me, sx,sy,pixels,lines)
    XV_struct* p_me;
int sx,sy,pixels,lines;
{
int startaddr,base_addr;
dma_ctl *dma_control;

        dma_control = (dma_ctl *) &p_me->xvm->Prom1;
        startaddr= xv_m24base + (sy*p_me->main.linebytes)+ sx;

        dma_control->counter_control = 0x2000000;
        dma_control->start_address = mungeword(startaddr);
        dma_control->counter_control = 0;
        dma_control->start_address = mungeword(startaddr);
        dma_control->counters = mungeword((lines-1 << 12) | pixels-1);
        dma_control->counter_control = 0x1000000;
}

