/*
 * (c) Copyright 1991 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log:   /plxhp/dave/pvcsrc/hwlib/archives/btlib.c_v  $
 * 
 *    Rev 1.1   09 Sep 1996 10:09:16   dave
 * Initial revision
 * Revision 1.1  1995/12/08 13:05:05  dist
 * Initial revision
 *
 * Revision 1.1  95/04/17  19:15:18  19:15:18  dist ()
 * Initial revision
 * 
 * Revision 1.11  93/06/18  16:46:05  xan
 * Rtv merge
 * 
 * Revision 1.9  91/12/20  15:57:43  dave
 * driver no longer screws up color tables
 * 
 * Revision 1.8  91/11/13  15:25:01  eric
 * steal gamma table for cursursoror
 * 
 * Revision 1.7  91/10/23  09:55:37  mears
 * new rev
 * 
 * 
 * Revision 1.6  91/10/14  13:42:54  mears
 * put option clts back into driver code
 * 
 * Revision 1.5  91/10/08  16:57:22  mears
 * new ifdef DRIVER's
 * 
 * Revision 1.4  91/10/04  12:44:07  mears
 * ifdef DRIVER
 * 
 * Revision 1.3  91/09/30  16:18:06  mears
 * many changes
 * 
 * Revision 1.2  91/07/26  19:06:42  eric
 * new from bill
 * 
 * Revision 1.1  91/06/12  16:06:27  eric
 * Initial revision
 * 
 *
 */

#ifndef lint
static char *rcsid = "@(#)$Id: btlib.c,v 1.1 1995/12/08 13:05:05 dist Exp $ Parallax Graphics Inc";
#endif

/* BROOKTREE LIBRARY INTERFACE FOR XVIDEO PRODUCT FAMILY */
/* CONTAINED DETAILED ROUTINES WHICH NEED NOT BE ACCESS BY USERS */
/* standard tag definitions:

	00	- pseudo color
	01	- true color w/tables
	10	- video #1 (no tables on 463)
	11	- video #2 (no tables on 463)

standard calls:

plxv_init_bt(p_me)
plxv_load_clt(p_me, tag,start,count,pntr)
plxv_load_clts(p_me, tag,start,count,rptr,gptr,bptr) driver version of above
plxv_load_cursor(p_me, ptr);

*/

#include <stdio.h>
#include "hw_context.h"
#define XV_DIAG 1 
#include "xvdebug.h"


plxv_init_bt(p_me, bypass)
XV_struct *p_me;
int bypass;
{
    int a,b,i,j,k,t;
    XV_color init_c[0x100];

    switch(p_me->main.clt_type)
    {
      case BT463:
	plxv_init_bt463(p_me, bypass);
	break;
      case BT495:
	plxv_init_bt495(p_me);
	break;
      default:
	printf("init_bt: unknown color table type\n");
    }

    switch(p_me->opt.clt_type)
    {
      case BT473:
	plxv_init_bt473(p_me);
	break;
      default:
	break;
    }

#ifndef DRIVER 

    /* load default pseudo color palette*/
    j = 0;
    for (i=0;i<0x8;i++)
    {
	init_c[j].red    = (i & 0x1) * 0xff;
	init_c[j].green  = ((i & 0x2)>>1) * 0xff;
	init_c[j++].blue = ((i & 0x4)>>2) * 0xff;
    }
    for (i=0;i<0x8;i++)
    {
	init_c[j].red    = (i * 0x1f) + 7;
	init_c[j].green  = (i * 0x1f) + 7;
	init_c[j++].blue = (i * 0x1f) + 7;
    }
    for (i=0;i<0x10;i++)
    {
	init_c[j].red    = 0xff - (i * 0x10);
	init_c[j].green  = 0xff - (i * 0x10);
	init_c[j++].blue = 0xff - (i * 0x10);
    }
    for (i=1;i<8;i++)
    {
	for (k = 0;k<0x20;k++)
	{
	    init_c[j].red    = ((i & 0x1) * (k * 8)) + 7;
	    init_c[j].green  = (((i & 0x2)>>1) * (k * 8)) + 7;
	    init_c[j++].blue = (((i & 0x4)>>2) * (k * 8)) + 7;
	}
    }
    plxv_load_clt(p_me,0,0,0x100,init_c);


    /* load the cusor colors */
    plxv_load_cursor(p_me,init_c);		/* colors black & red */

#endif  /* DRIVER */

    /* true color palette */
    j = 0;
    for (i=0;i<0x100;i++)
    {
	init_c[j].red    =  i;
	init_c[j].green  =  i;
	init_c[j++].blue =  i;
    }
    plxv_load_clt(p_me,1,0,0x100,init_c);
    /*load video 1 */
    plxv_load_clt(p_me,2,0,0x100,init_c);
    /*load video 2 */
    plxv_load_clt(p_me,3,0,0x100,init_c);

}
    
plxv_load_clt(p_me,tag,start,num,cptr)
XV_struct *p_me;
int tag,start,num;
XV_color *cptr;
{
    int i,j,k;
    XV_color *ptr;

    if (tag > 3)
    {
	printf("tag value too large for color space\n");
	return(-1);
    }
    if (start > 0xff)
    {
	printf("color table start address exceeded\n");
	return(-1);
    }
    if ( (start + num) > 0x100)
    {
	printf("color table load size too big\n");
	return(-1);
    }

    ptr = cptr;
    switch(p_me->main.clt_type)
    {
      case BT463:
	if (tag > 1) break;	/* do nothing for video palettes on 463 */
	set_bt463_addr(p_me, (0x100 * tag) + start);
	for (i=0; i<num; i++)
	{
	    j = ptr->red;
	    PV_WREG(p_me, Bt.red.reg3,j);
	    j = ptr->green;
	    PV_WREG(p_me,Bt.red.reg3,j);
	    j = ptr->blue;
	    PV_WREG(p_me,Bt.red.reg3,j);
	    ptr++;
	}
	break;
      case BT495:
	break;
      default:
        printf("load_clt: unknown color table type\n");
        return(-1);
    }
    ptr = cptr;

    /* now make the video output card's color tables match */
    switch(p_me->opt.clt_type)
    {
      case BT473:
	if ( tag > 0) break;		/* only pseudo-color palette avail */
        set_option_rsaddr(p_me, 3,0);		/* setup to load start address */
        PV_WREG(p_me,Ext.reg1,start);
	set_option_rsaddr(p_me, 3,1);		/* setup to load palette data */
	for (i=0; i<num; i++)
	{ /* address will auto-increment and use modulo 3 shit */
	    j = ptr->red;
	    PV_WREG(p_me,Ext.reg1,j);
	    j = ptr->green;
	    PV_WREG(p_me,Ext.reg1,j);
	    j = ptr++->blue;
	    PV_WREG(p_me,Ext.reg1,j);
	}
	break;
      default:
	break;
    }

    return(0);
}

#ifdef DRIVER
plxv_load_clts(p_me, tag, start, num, rptr0, gptr0, bptr0)
XV_struct* p_me;
int tag,start,num;
char *rptr0, *gptr0, *bptr0;
{
    int i,j,k;
    char *rptr, *gptr, *bptr;

    if (tag > 3)
    {
	printf("tag value too large for color space\n");
	return(-1);
    }
    if (start > 0xff)
    {
	printf("color table start address exceeded\n");
	return(-1);
    }
    if ( (start + num) > 0x100)
    {
	printf("color table load size too big\n");
	return(-1);
    }

    rptr = rptr0;  gptr = gptr0;  bptr = bptr0;  
    switch(p_me->main.clt_type)
    {
      case BT463:
        if (tag > 1) return(0); /* do nothing for video palettes on 463 */
        set_bt463_addr(p_me, (0x100 * tag) + start);
        for (i=0; i<num; i++)
        {
            j = *rptr++;
            PV_WREG(p_me,Bt.red.reg3,j);
            j = *gptr++;
            PV_WREG(p_me,Bt.red.reg3,j);
            j = *bptr++;
            PV_WREG(p_me,Bt.red.reg3,j);
        }
        break;
      case BT495:
	break;
      default:
        printf("load_clt: unknown color table type\n");
        return(-1);
    }

    /* now make the video output card's color tables match */
    rptr = rptr0;  gptr = gptr0;  bptr = bptr0;  
    switch(p_me->opt.clt_type)
    {
      case BT473:
	if ( tag > 0) break;		/* only pseudo-color palette avail */
        set_option_rsaddr(p_me, 3,0);		/* setup to load start address */
        PV_WREG(p_me,Ext.reg1,start);
	set_option_rsaddr(p_me, 3,1);		/* setup to load palette data */
	for (i=0; i<num; i++)
	{ /* address will auto-increment and use modulo 3 shit */
	    j = *rptr++;
	    PV_WREG(p_me,Ext.reg1,j);
	    j = *gptr++;
	    PV_WREG(p_me,Ext.reg1,j);
	    j = *bptr++;
	    PV_WREG(p_me,Ext.reg1,j);
	}
	break;
      default:
	break;
    }
    return(0);
}

/* this is used by the driver */
plxv_read_clts(p_me, tag, start, num, rptr, gptr, bptr)
XV_struct* p_me;
int tag,start,num;
char *rptr, *gptr, *bptr;
{
    int i,j,k;

    if (tag > 3)
    {
	printf("tag value too large for color space\n");
	return(-1);
    }
    if (start > 0xff)
    {
	printf("color table start address exceeded\n");
	return(-1);
    }
    if ( (start + num) > 0x100)
    {
	printf("color table load size too big\n");
	return(-1);
    }

    switch(p_me->main.clt_type)
    {
      case BT463:
        if (tag > 1) return(0); /* do nothing for video palettes on 463 */
        set_bt463_addr(p_me, (0x100 * tag) + start);
        for (i=0; i<num; i++)
        {
            PV_RREG(p_me,j, Bt.red.reg3);
            *rptr++ = j & 0xff;
            PV_RREG(p_me,j, Bt.red.reg3);
            *gptr++ = j & 0xff;
            PV_RREG(p_me,j, Bt.red.reg3);
            *bptr++ = j & 0xff;
        }
        break;
      case BT495:
	break;
      default:
        printf("load_clt: unknown color table type\n");
        return(-1);
    }
}
 
#endif /*DRIVER*/

plxv_load_cursor(p_me, ptr0)
XV_struct*  p_me;
XV_color*   ptr0;
{
    int i,j,k;
    XV_color *ptr;

    ptr = ptr0;
    switch(p_me->main.clt_type)
    {
      case BT463:
        set_bt463_addr(p_me, 0x202);        /* px:  */
	for(i=0; i<2; i++)
	{
	    j = ptr->red;
	    PV_WREG(p_me,Bt.red.reg3,j);
	    j = ptr->green;
	    PV_WREG(p_me,Bt.red.reg3,j);
	    j = ptr->blue;
	    PV_WREG(p_me,Bt.red.reg3,j);
	    ptr++;
	}
	break;
      case BT495:
	break;
      default:
	break;
    }
    ptr = ptr0;

    switch(p_me->opt.clt_type)
    {
      case BT473:
	/* background color at address 1 overlay palette */
	set_option_rsaddr(p_me, 3,4);  /* setup to load start address */
	PV_WREG(p_me,Ext.reg1,1);	 /* start address is 1	 	*/
	set_option_rsaddr(p_me, 3,5);  /* setup to load palette data */

	/* address will auto-increment and use modulo 3 stuff */
	j = ptr->red;
	PV_WREG(p_me,Ext.reg1,j);
	j = ptr->green;
	PV_WREG(p_me,Ext.reg1,j);
	j = ptr++->blue;
	PV_WREG(p_me,Ext.reg1,j);

	/* foreground color at address 3 overlay palette */
	set_option_rsaddr(p_me, 3,4);  /* setup to load start address */
	PV_WREG(p_me,Ext.reg1,2);	 /* start address is 2		*/
	set_option_rsaddr(p_me, 3,5);  /* setup to load palette data */

	/* address will auto-increment and use modulo 3 stuff */
	j = ptr->red;
	PV_WREG(p_me,Ext.reg1,j);
	j = ptr->green;
	PV_WREG(p_me,Ext.reg1,j);
	j = ptr++->blue;
	PV_WREG(p_me,Ext.reg1,j);
      default:
	break;
    }
    return(0);
}

plxv_disp_blank( p_me, sel)
XV_struct*  p_me;
int         sel;
{
    int a,b;
    switch(p_me->main.ga_rev)
    {
      case GAREV_2:
	if (sel) a = 1; else a = 0;
	PV_WREG(p_me,DispKludge.b.disp_blank,a);
	break;
      default:
	printf("DISPBLNK: don't know what Gate array rev\n");
	break;
    }
    switch(p_me->opt.clt_type)
    {
      case BT473:
	break;
      default:
	break;
    }
}

 
plxv_init_bt463(p_me, bypass)
XV_struct*  p_me;
int         bypass;
{
    int i,t;

    /* load the BT463 command registers 0,1,&2 BANG BANG BANG */
    switch(p_me->main.board_code)
    {
	case TCHAN_1:
	    set_bt463_addr(p_me, 0x201);
	    PV_WREG(p_me,Bt.red.reg2, 0xC0);	/* 2:1 mux , blink xx */
	    PV_WREG(p_me,Bt.red.reg2, 0x40);	/* overlay common, 24 planes, 4 over */
					/* 16 tables, no under, no cursor */
	    PV_WREG(p_me,Bt.red.reg2, 0xc0);	/* sync on green, + pedistal */
	    break;
	case SBUS_1:
	case SBUS_2:
	case SBUS_3:
	case SBUS_4:
	    set_bt463_addr(p_me, 0x201);
	    PV_WREG(p_me,Bt.red.reg2, 0x40);	/* 4:1 mux , blink xx */
	    PV_WREG(p_me,Bt.red.reg2, 0x40);	/* overlay common, 24 planes, 4 over */
					/* 16 tables, no under, no cursor */
	    PV_WREG(p_me,Bt.red.reg2, 0x40);	/* no sync ,pedistal */
	    break;
	default:
	    printf("plxv_init_bt463: unkown board type\n");
    }
    /* load read mask register = all planes */
    set_bt463_addr(p_me, 0x205);
    PV_WREG(p_me,Bt.red.reg2, 0xff);
    PV_WREG(p_me,Bt.red.reg2, 0xff);
    PV_WREG(p_me,Bt.red.reg2, 0xff);
    PV_WREG(p_me,Bt.red.reg2, 0xff);

   /* no blinking mask */
    PV_WREG(p_me,Bt.red.reg2, 0);
    PV_WREG(p_me,Bt.red.reg2, 0);
    PV_WREG(p_me,Bt.red.reg2, 0);
    PV_WREG(p_me,Bt.red.reg2, 0);

    /* NOTE: the BT463 must be in BLANK to load the window table */
    /* so we will turn off the display via vreset !!!! */
    /* check GA REV HERE */
    switch(p_me->main.ga_rev)
    {
	case GAREV_2:
	    PV_WREG(p_me,DispKludge.b.disp_blank,1);
	    break;
	default:
	    printf("463init: don't know what rev GAs in use\n");
    }

#ifdef DRIVER
    set_normal_463(p_me);
#else
    /* now load the window table */
    switch(bypass)
    {
	case 0:			/* normal window tag tables */
	    set_normal_463(p_me);
	    break;
	case 1:			/* bypass mode  =  24 bit true only */
    	    for (i=0;i<0x10;i++) set_bt463_wtab(p_me, i,0,8,0,0,0,0,1);
	    break;
	case 2:			/* 8 bit only from tags */
	    break;
	case 3:	 /* showtags for em28 mode */
	    for (i=0;i<0x10;i++) set_bt463_wtab(p_me, i,24,4,1,0,0,0,0);
	    break;
	case 4:		/* 8 over 24 */
	    set_8_over_24_463(p_me);
	   break;
	case 5:		/* 6 over 24 */
	    break;	
	default:
	    break;
    }
#endif /*NOT DRIVER*/
    /* now turn back on the display */
    switch(p_me->main.ga_rev)
    {
        case GAREV_2:
	    PV_WREG(p_me,DispKludge.b.disp_blank,0);
            break;
        default:
	    break;
    }

}

set_bt463_wtab(p_me, tnum,shft,planes,mode,ovlyloc,ovlymsk,strtad,bypass)
XV_struct*	p_me;
int tnum,shft,planes,mode,ovlyloc,ovlymsk,strtad,bypass;
{
    int i,val;

    if (tnum > 0xf) printf("error: table number out of range\n");
    val  =  (shft    & 0x1f)       | ((planes  & 0xf)  << 5);
    val |= ((mode    & 0x7) << 9)  | ((ovlyloc & 0x1)  << 12);
    val |= ((ovlymsk & 0xf) << 13) | (((strtad >> 4)  & 0x3f) << 17);
    val |= ((bypass & 0x1) << 23) ;
    /* printf(" %x loading %x\n",tnum,val); */
    set_bt463_addr(p_me, 0x300 | (tnum & 0xf));
    PV_WREG(p_me,Bt.red.reg2, val & 0xff);
    PV_WREG(p_me,Bt.red.reg2, (val >> 8)  & 0xff);
    PV_WREG(p_me,Bt.red.reg2, (val >> 16) & 0xff);
}

set_bt463_addr(p_me, addr)
XV_struct* p_me;
int addr;
{
    int a,b;
	
    a = (addr >> 8) & 0xff;
    b = addr & 0xff;
    PV_WREG(p_me,Bt.red.reg0, b);	/* load low order Bt463 address register */
    PV_WREG(p_me,Bt.red.reg1, a);	/* load high order Bt463 address register */
}

set_normal_463(p_me)
XV_struct* p_me;
{
    int i,t = 0;
    set_bt463_wtab(p_me, t++,0,8,0,0,0,0    ,0);    /* 0:pseudo     0 - ff */
    set_bt463_wtab(p_me, t++,0,8,0,0,0,0    ,1);    /* 1:true       bypass */
    set_bt463_wtab(p_me, t++,0,8,0,0,0,0    ,1);    /* 2:video 1    bypass */
    set_bt463_wtab(p_me, t++,0,8,0,0,0,0    ,1);    /* 3:video 2    bypass */

    set_bt463_wtab(p_me, t++,0,8,0,0,0,0    ,0);    /* 4:pseudo     0 - ff */
    set_bt463_wtab(p_me, t++,0,8,0,0,0,0    ,1);    /* 5:true       bypass */
    set_bt463_wtab(p_me, t++,0,8,0,0,0,0    ,1);    /* 6:video 1    bypass */
    set_bt463_wtab(p_me, t++,0,8,0,0,0,0    ,1);    /* 7:video 2    bypass */

    for (i=0;i<4;i++) set_bt463_wtab(p_me, t++,27,1,0,0,0,0x100,0);	/* cursor forground */
    for (i=0;i<4;i++) set_bt463_wtab(p_me, t++,27,1,0,0,0,0x110,0);	/* cursor background */
}

set_8_over_24_463(p_me)
XV_struct* p_me;
{
    int t = 0;

     /*enable 8 over 24 in the command register */
    /* load the BT463 command registers 0,1,&2 BANG BANG BANG */
    switch(p_me->main.board_code)
    {
	case TCHAN_1:
	    set_bt463_addr(p_me, 0x201);
	    PV_WREG(p_me,Bt.red.reg2, 0xC0);	/* 2:1 mux , blink xx */
	    PV_WREG(p_me,Bt.red.reg2, 0x10);	/* overlay common, 24 planes, 4 over */
					/* 16 tables, no under, no cursor */
	    PV_WREG(p_me,Bt.red.reg2, 0xc0);	/* sync on green, + pedistal */
	    break;
	case SBUS_1:
	case SBUS_2:
	case SBUS_3:
	case SBUS_4:
	    set_bt463_addr(p_me, 0x201);
	    PV_WREG(p_me,Bt.red.reg2, 0x40);	/* 4:1 mux , blink xx */
	    PV_WREG(p_me,Bt.red.reg2, 0x10);	/* overlay common, 24 planes, 4 over */
					/* 16 tables, no under, no cursor */
	    PV_WREG(p_me,Bt.red.reg2, 0x40);	/* no sync ,pedistal */
	    break;
	default:
	    printf("plxv_init_bt463: unkown board type\n");
    }
    for (t=0;t<0x10;t++)
    {   /* for 8 over 24, set all window tables to
	   no shift,8 planes, true color, all overlay enab, std overlay,
	   start address = 0x100 					*/
        set_bt463_wtab(p_me, t,0,8,0,0,0xf,0x100,0);
    }
}


/* the bt473 is used on the option card for output of pseudo colors
   to the low-res video output.
*/
plxv_init_bt473(p_me)
    XV_struct* p_me;
{
    /* set command register */
    set_option_rsaddr(p_me, 3,6);
    PV_WREG(p_me,Ext.reg1,0x30);

    /* set read mask */
    set_option_rsaddr(p_me, 3,2);
    PV_WREG(p_me,Ext.reg1,0xff);
}


set_option_rsaddr(p_me, device_code,rsaddr)
XV_struct* p_me;
int rsaddr;
{
    int i,j,k;

    i = (device_code & 3) << 3;
    j = rsaddr & 7;
    k = i | j;
    PV_WREG(p_me,Ext.reg0,k);
}

#ifndef DRIVER

/****************************  bt473 stuff ********************/
/* useful bt473 information.
s1 = 0
attr 1/0	SO  OL1  OL0		CALLED
  0  0		0    0    0		pseduocolor mode
  0  1		1    0    0		true color mode
  1  0		x    0    1		cursor background
  1  1		x    1    1		cursor forgroundo
*/



load_bt473_palette(p_me, start,num,ptr,overlay_flag)
XV_struct* p_me;
int start,num;
XV_color *ptr;
{
    /* if overlay_flag == 1 then the overlay palette
	will be addressed instead of the data palette */

    int i=0,j;

    if (overlay_flag) 
	i = 4;
    set_option_rsaddr(p_me, 3,i | 0);  /* setup to load start address */
    PV_WREG(p_me,Ext.reg1,start);
    set_option_rsaddr(p_me, 3,i | 1);  /* setup to load palette data */
    for (i=0; i<num; i++)
    {
	/* address will auto-increment and use modulo 3 shit */
	j = ptr->red;
	PV_WREG(p_me,Ext.reg1,j);
	j = ptr->green;
	PV_WREG(p_me,Ext.reg1,j);
	j = ptr++->blue;
	PV_WREG(p_me,Ext.reg1,j);
    }
}

read_bt473_palette(p_me, start,num,ptr,overlay_flag)
XV_struct* 	p_me;
int start,num;
XV_color *ptr;
{
    /* if overlay_flag == 1 then the overlay palette
	will be addressed instead of the data palette */

    int i=0,j;

    if (overlay_flag) 
	i = 4;
    set_option_rsaddr(p_me, 3,i | 3);  /* setup to load start address */
    PV_WREG(p_me,Ext.reg1,start);
    set_option_rsaddr(p_me, 3,i | 1);  /* setup to load palette data */
    for (i=0; i<num; i++)
    {
	/* address will auto-increment and use modulo 3 shit */
	PV_RREG(p_me,j,Ext.reg1);
	ptr->red = j & 0xff;
	PV_RREG(p_me,j,Ext.reg1);
	ptr->green = j & 0xff;
	PV_RREG(p_me,j,Ext.reg1);
	ptr++->blue = j & 0xff;
    }
}

/******   End of support for BT473			   *****/

/******   Brooktree support routines for the RGB input card *****/
/******   Here (in this file) because brooktree makes them !*****/

/* the bt261 is a sync lock generator whith PLL pump output */

plxv_init_bt261(p_me, port,sel_625)
XV_struct* p_me;
int port,sel_625;
{
#ifdef NOTANDY
    int i,hcount,hactive,zero_stop;
    unsigned char *cp;
    XV_io_port *iop,*plxv_io_port();
    XV_bt261_config *btp;

    if (( iop = plxv_io_port(port)) == (XV_io_port *)0)
    {
	return(-1);
    }
    btp = &(iop->config.rgbin.timing);

    cp = &(iop->config.rgbin.timing.command[0]);
    for (i=0;i<4;i++) plxv_load_bt261(p_me, i,*cp++);

    /* get the parametric data parameters */
    sel_625 &= 0x1;
    hcount = iop->config.rgbin.format[sel_625].hcount;
    hactive = iop->config.rgbin.format[sel_625].hactive;
    zero_stop = iop->config.rgbin.format[sel_625].zero_stop;

    plxv_load_bt261_short(p_me, 0x5,hcount/4);	/* osc count */
    plxv_load_bt261_short(p_me, 0x18,hcount/2 - btp->ngate_start);
    plxv_load_bt261_short(p_me, 0x1a,hcount/2 + btp->ngate_stop);	
/*
    plxv_load_bt261_short(p_me, 0x18,hcount/2 - scale_param(hcount,btp->ngate_start));
    plxv_load_bt261_short(p_me, 0x1a,hcount/2 + scale_param(hcount,btp->ngate_stop));	
*/
    plxv_load_bt261_short(p_me, 0x1c,hcount - 1);	/* hcount */
    plxv_load_bt261(p_me, 4,hcount/4);		/* vsync sample register */
    plxv_load_bt261_short(p_me, 0x8,scale_param(hcount,btp->hsync_start));
    plxv_load_bt261_short(p_me, 0xa,0); 		/* hsync stop */
    plxv_load_bt261_short(p_me, 0xc,scale_param(hcount,btp->clamp_start));
    plxv_load_bt261_short(p_me, 0xe,scale_param(hcount,btp->clamp_stop));

    plxv_load_bt261_short(p_me, 0x12,zero_stop);           /* zero stop */
    plxv_load_bt261_short(p_me, 0x10,zero_stop + hactive); /* zero start */

    plxv_load_bt261_short(p_me, 0x14,scale_param(hcount,btp->fgate_start));
    plxv_load_bt261_short(p_me, 0x16,scale_param(hcount,btp->fgate_stop));
#endif /* NOTANDY */
}

static int scale_param(hc,v)
int hc,v;
{
    return(v*hc/780);
}


plxv_load_bt261(p_me, addr,data)
XV_struct* p_me;	
int addr,data;
{
    set_option_rsaddr(p_me, 1,0);		/* select bt261 internal address reg */
    PV_WREG(p_me,Ext.reg1,addr);
    set_option_rsaddr(p_me, 1,1);		/* select bt261 data */
    PV_WREG(p_me,Ext.reg1,data);
}

plxv_read_bt261(p_me, addr,data)
XV_struct* p_me;	
int addr,*data;
{
    set_option_rsaddr(p_me, 1,0);		/* select bt261 internal address reg */
    PV_WREG(p_me,Ext.reg1,addr);
    set_option_rsaddr(p_me, 1,1);		/* select bt261 data */
    PV_RREG(p_me,*data,Ext.reg1);
}


plxv_load_bt261_short(p_me, addr,data)
XV_struct* p_me;	
int addr,data;
{
    plxv_load_bt261(p_me, addr,data & 0xff);		
    plxv_load_bt261(p_me, addr + 1,(data >> 8) & 0xff);	
}

    
/************************************************/
/* the bt253 is ADC's *c plus calibrating DAC's */
plxv_init_bt253(p_me, port,sel_yuv)
XV_struct* p_me;	
int port;
{
#ifdef NOTANDY
    unsigned char *cp;
    int i;
    XV_io_port *iop;

    if (( iop = plxv_io_port(port)) == 0)
    {
	return(-1);
    }


    /* bt253 setup for RGB separate sync */    
    plxv_load_bt253(p_me, 0,0x60);		/* command register */

    sel_yuv &= 0x1;
    cp = &(iop->config.rgbin.cal[sel_yuv].val[0]);
    for (i=0;i<6;i++) plxv_load_bt253(p_me, i+1,*cp++);
#endif /* NOTANDY */
}


plxv_load_bt253(p_me, reg_num,data)
XV_struct* p_me;	
int reg_num,data;
{
    /*
        reg_num 0 --> command register 
		1 --> red white leve 2 --> clamp level offset
		3 --> green white leve 4 --> clamp level offset
		5 --> blue white leve 6 --> blue clamp level
    */
    set_option_rsaddr(p_me, 0,reg_num);
    PV_WREG(p_me,Ext.reg1,data);
}

plxv_read_bt253(p_me, reg_num,data)
XV_struct* p_me;	
int reg_num,*data;
{
    set_option_rsaddr(p_me, 0,reg_num);
    PV_RREG(p_me,*data,Ext.reg1);
}

/*************** end of bt253 support ********************/

/* ****** suppport for the bt281  ************************/
/* the bt281 is a color space converter, as well as a set */
/* of color lookup tabes individual for R, G, & b         */
/* Color space converter performs normalized operations	  */
/* The color tables are used to set offset and gain       */
/**********************************************************/

/* routine to load the bt281 color tables (3 x 0x100) */
/* mask: 1 = red, 2 = green,4 = blue, 7 = all	      */
plxv_load_bt281_clt(p_me, clt,mask)
XV_struct* p_me;	
int *clt,mask;

{

    int *ip;
    int i,j,k;

    ip = clt;
    for (i=0;i,i<0x100;i++)
    {
	if (mask & 1) plxv_load_bt281(p_me, 0x000 + i, *ip);
	if (mask & 2) plxv_load_bt281(p_me, 0x100 + i, *ip);
	if (mask & 4) plxv_load_bt281(p_me, 0x200 + i, *ip);
	ip++;
    } 
}


/* routine to read back the CLT ram in the bt281 (for testing) */
/* bank: 0 = red, 1 = green, 2 = blue */
plxv_read_bt281_clt(p_me, bank,clt)
XV_struct* p_me;	
int *clt,bank;
{
    int i;

    for (i=0;i<0x100;i++)
    {
	    plxv_read_bt281_reg(p_me, i | (bank << 8),clt++);
    }
}

/* routine to load the Color Space Converter Matrix */
/* part of the 281				    */
/* RGB uses the Identity matry			    */
/* YUV uses a normalized matrix			    */

plxv_load_bt281_csc(p_me, matrix)
XV_struct* p_me;	
float *matrix;
{
    int i;
    DEBUGF(2, (fd, "plxv_load_bt281_csc: matrix addr = %x\n",matrix));
    for (i=0;i<9;i++) plxv_load_bt281_matrix_element(p_me, i,*matrix++);
}

plxv_load_bt281_matrix_element(p_me, reg_num,mat_val)
XV_struct* p_me;	
int reg_num;
float mat_val;
{
    int reg_val;
    int mat_addr;
    int val_low,val_high;
    float divisor;

    divisor = (3.996 - 0.0) / (0x3ff);
    /* first convert the floating point mat_val into a register pair value */
    if (mat_val > -divisor)
    reg_val = (int)(mat_val / divisor);
    else
    {
	reg_val = (int)((mat_val + 4.000) / divisor); 
	reg_val |= 0x400;	/* set the sign bit */
    }
    /* now load the appropriate register pair with reg_val */
    mat_addr = 0x300 + reg_num  * 2;
    val_low = reg_val & 0xff;
    val_high = (reg_val >> 8) & 0xff;

    plxv_load_bt281(p_me, mat_addr,0xff & reg_val);
    plxv_load_bt281(p_me, mat_addr + 1,0x7 & (reg_val >> 8));
}

plxv_read_bt281_reg(p_me, addr,data)
XV_struct* p_me;	
int addr,*data;
{
    set_bt281_addr(p_me, addr);
    set_option_rsaddr(p_me, 2,2);
    PV_RREG(p_me,*data,Ext.reg1);
}

plxv_load_bt281(p_me, addr,data)
XV_struct* p_me;	
int addr,data;
{
    set_bt281_addr(p_me, addr);
    set_option_rsaddr(p_me, 2,2);
    PV_WREG(p_me,Ext.reg1,data);
}

set_bt281_addr(p_me, addr)
XV_struct* p_me;	
int addr;
{
    int a,b;

    a = 0xff & addr;
    b = 0xff & (addr >> 8);
    set_option_rsaddr(p_me, 2,0);
    PV_WREG(p_me,Ext.reg1,a);
    set_option_rsaddr(p_me, 2,1);
    PV_WREG(p_me,Ext.reg1,b);
}

/* end bt281 support routines */

plxv_init_bt281(p_me, sel_yuv)
XV_struct* p_me;	
int sel_yuv;
{
    DEBUGF(2, (fd, "plxv_init_bt281: sel_yuv = %d\n",sel_yuv));
    if (sel_yuv)
    {
	/* setup to use csc */
    	plxv_load_bt281(p_me, 0x312,0x20);		/* command register 0 */
    	plxv_load_bt281(p_me, 0x313,0);		/* command register 1 */
    }
    else
    {
	/* setup for rgb. for bypass load 0x313 with 0xe0 */
    	plxv_load_bt281(p_me, 0x312,0);		/* command register 0 */
    	plxv_load_bt281(p_me, 0x313,0);		/* command register 1 */
    }
}
#endif /*NOT DRIVER*/

/****
 ****  Brooktree 495 Support
 ****/
int plxv_rd_bt495(p_me, ad)
XV_struct* p_me;	
int ad;
{
	int i;
    PV_WREG(p_me,Bt.red.reg0,ad&0xff);
    PV_WREG(p_me,Bt.red.reg1,(ad>>8)&0x7);
    PV_RREG(p_me,i,Bt.red.reg2);
    return(i);
}

plxv_rd_bt495_lut(p_me, ad, p_r, p_g, p_b)
XV_struct* p_me;	
int ad;
int* p_r;
int* p_g;
int* p_b;
{
    PV_WREG(p_me,Bt.red.reg0,ad&0xff);
	PV_WREG(p_me,Bt.red.reg1,0);
    PV_RREG(p_me,*p_r,Bt.red.reg3);
    PV_RREG(p_me,*p_g,Bt.red.reg3);
    PV_RREG(p_me,*p_b,Bt.red.reg3);
	*p_r &= 0xff;
	*p_g &= 0xff;
	*p_b &= 0xff;
}

plxv_wr_bt495(p_me, ad,data)
XV_struct* p_me;	
int ad,data;
{
    PV_WREG(p_me,Bt.red.reg0,ad&0xff);
    PV_WREG(p_me,Bt.red.reg1,(ad>>8)&0x7);
    PV_WREG(p_me,Bt.red.reg2,data);
}

plxv_wr_bt495_lut(p_me, ad,r,g,b)
XV_struct* p_me;	
int ad,r,g,b;
{
    PV_WREG(p_me,Bt.red.reg0,ad&0xff);
	PV_WREG(p_me,Bt.red.reg1,0);
    PV_WREG(p_me,Bt.red.reg3,r);
    PV_WREG(p_me,Bt.red.reg3,g);
    PV_WREG(p_me,Bt.red.reg3,b);
}

plxv_init_bt495(p_me)
XV_struct* p_me;	
{
    int i;
	int	id;
	int rb_blue;
	int rb_green;
	int rb_red;

    id = plxv_rd_bt495(p_me, 0x200) & 0xff;
    if ( id != 0x2d )
	{
		printf("cannot find BT495 RAMDAC expected 0x2d got 0x%x\n", id);
		return;
    }

    plxv_wr_bt495(p_me, 0x201,0xc0); /* Command Register_0 sets 5:1 mux, 8 bit mode */
    plxv_wr_bt495(p_me, 0x203,0x00); /* Command Register_2 sets sync&pedestal off */
#if NOTDEF /* dual monitor mode.  */
    plxv_wr_bt495(p_me, 0x203,0x80); /* Command Register_2 sets sync&pedestal off */
#endif /* NOTDEF */
    plxv_wr_bt495(p_me, 0x204,0x00); /* Interleave Register: display config */
    plxv_wr_bt495(p_me, 0x205,0xff); /* Red Lookup Bypass disabled */
    plxv_wr_bt495(p_me, 0x206,0xff); /* Green Lookup Bypass disabled */
    plxv_wr_bt495(p_me, 0x207,0xff); /* Blue Lookup Bypass disabled */
    plxv_wr_bt495(p_me, 0x208,0x01); /* LB Lookup Bypass disabled */
    plxv_wr_bt495(p_me, 0x209,0x00); /* Red Blink disabled */
    plxv_wr_bt495(p_me, 0x20a,0x00); /* Green Blink disabled */
    plxv_wr_bt495(p_me, 0x20b,0x00); /* Blue Blink disabled */
    plxv_wr_bt495(p_me, 0x20c,0x00); /* LB Blink disabled */
    plxv_wr_bt495(p_me, 0x210,0x00); /* Cursor disabled */

    for (i=0x400; i<=0x7ff; i++) 
		plxv_wr_bt495(p_me, i,0); /* Clear Cursor Ram */

    for (i=0x0; i<=0xff; i++) 
		plxv_wr_bt495_lut(p_me, i,0,0,0); /* black entire LUT */

    for (i=0x0; i<=0xff; i++) 
	{
		plxv_rd_bt495_lut(p_me, i, &rb_red, &rb_green, &rb_blue); 
		if ( rb_red )
			DEBUGF(1, (fd, "#### Entry %d: red = 0x%x should be 0x00\n", i, rb_red));
		if ( rb_green )
			DEBUGF(1, (fd, "#### Entry %d: green = 0x%x should be 0x00\n", i, rb_green));
		if ( rb_blue )
			DEBUGF(1, (fd, "#### Entry %d: blue = 0x%x should be 0x00\n", i, rb_blue));
	}
    
}
