/*
 *  sun_mmap.c:	Implementation For Sun TV2 & TV3 Memory Mapping Objects.
 *
 *
 * (c) Copyright 1994 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 */
#include <stdio.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <signal.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <setjmp.h>
#include "objects.h"
#include "plxioctl.h"

#include "video_stream.h"
#include "c3regs.h"

#ifdef TV3
#    define WIDTH 1280
#    define HEIGHT 1024
#else
#    define WIDTH 1152
#    define HEIGHT 900
#endif

/*  XV_DIAG FLAGS:
 *
 *    0x80 - initialization 
 *    0x40 - initialization (verbose)
 */
/* #define XV_DIAG 0xC0  */
#include "xvdebug.h"

#define PAUSE(string) {printf(string); fgetc(stdin);}

enum { 
	INTEGRIX_PAGE_SIZE  = 0x100000,
	NSLOTS  = 4, 
    PLXV_EPROM_SIZE =  0x10000,
	SEARCHLEN = 64
};

/*  FORWARD REFERENCES: */

static int test_slot();
static int test_slots();
O_Mmap*    use_driver();

/*  GLOBAL STORAGE */

static jmp_buf	jmp_env;				/* Environment save area for setjmp. */
static char*  	name_intx="INTX,sec";  	/* Name of integrix chassis. */
static char*    name_tv2="PGI,tvtwo";	/* Name of Parallax card. */
static char*    name_tv3="PGI,tvthree";	/* Name of Parallax overlay card. */

 
static char *sol1_slots[NSLOTS] = 		/* Solaris 1 all architectures. */
{
	"/dev/sbus2",
	"/dev/sbus1",
	"/dev/sbus3",
	"/dev/sbus0"
};

static char *sol2_4c_slots[NSLOTS] = 	/* Solaris 2 sun4c architecture. */
{
	"/devices/sbus@1,f8000000/sbusmem@2,0:slot2",
	"/devices/sbus@1,f8000000/sbusmem@1,0:slot1",
	"/devices/sbus@1,f8000000/sbusmem@3,0:slot3",
	"/devices/sbus@1,f8000000/sbusmem@0,0:slot0"
};

static char *sol2_4m_slots[NSLOTS] = 	/* Solaris 2 sun4m architecture. */
{
	"/devices/iommu@f,e0000000/sbus@f,e0001000/sbusmem@0,0:slot0",
    "/devices/iommu@f,e0000000/sbus@f,e0001000/sbusmem@1,0:slot1",
    "/devices/iommu@f,e0000000/sbus@f,e0001000/sbusmem@2,0:slot2",
    "/devices/iommu@f,e0000000/sbus@f,e0001000/sbusmem@3,0:slot3"
};
#define NSLOTS_4M (sizeof(sol2_4m_slots)/sizeof(char*))

static char *sol2_4u_slots[] = 			/* Solaris 2 sun4u architecture. */
{
#ifdef NOTDEF
	"/devices/sbus@1f,0/sbusmem@f,0:slotf",
	"/devices/sbus@1f,0/sbusmem@e,0:slote",
	"/devices/sbus@1f,0/sbusmem@d,0:slotd",
#endif
	"/devices/sbus@1f,0/sbusmem@3,0:slot3",
	"/devices/sbus@1f,0/sbusmem@2,0:slot2",
	"/devices/sbus@1f,0/sbusmem@1,0:slot1",
	"/devices/sbus@1f,0/sbusmem@0,0:slot0"
};
#define NSLOTS_4U (sizeof(sol2_4u_slots)/sizeof(char*))

static char *sol2_sparc5_slots[NSLOTS] = /* Solaris 2 sparc5 architecture. */
{
	"/devices/iommu@0,10000000/sbus@0,10001000/sbusmem@1,0:slot1",
	"/devices/iommu@0,10000000/sbus@0,10001000/sbusmem@2,0:slot2",
	"/devices/iommu@0,10000000/sbus@0,10001000/sbusmem@3,0:slot3",
	"/devices/iommu@0,10000000/sbus@0,10001000/sbusmem@3,0:slot0"
};
	 

/*---------------------------------------------------------------------------*
 |"buserr"
 |
 |  Bus error routine. Perform a long jump to the location given by jmp_env.
 *---------------------------------------------------------------------------*/
static void buserr( dummy )
	int  dummy;
{
/*  printf("bus error\n");fflush(stdout); */
    longjmp(jmp_env,-1);
}


#if 0
int getpagesize() { return(4096); }
#endif

/*---------------------------------------------------------------------------*
 |"mmap_area"
 |
 |  Memory map an area of Sbus memory given a file descriptor for the 
 |  Sbus slot.
 |
 |    -> mapped area on success, -1 on failure.
 *---------------------------------------------------------------------------*/
static char* mmap_area(fd, offset, length)
	int		fd;		/* =  file descriptor for Sbus slot mapping. */
	int		offset;	/* =  offset in bytes into Sbus slot of area to map. */
	int		length;	/* =  length of area in bytes to be mapped. */
{
    int     first_page;		/* Byte offset of 1st page of area to be mapped. */
	caddr_t p_area;			/* Address of region which was mmapped. */
    int     page_length;	/* #### why is this necessary? */
	int     page_size;		/* Size of page in bytes with this operating sys. */

	page_size = getpagesize();
#if 0
	first_page = (offset/page_size) * page_size;
	page_length = (offset % page_size);
#else
	first_page = (offset & ~(page_size - 1));
	page_length = offset & (page_size - 1);
#endif


	/* Map device memory to user space */

	p_area = mmap((caddr_t)0, length+page_length,
					PROT_READ|PROT_WRITE, MAP_SHARED,
					fd, first_page);

	if (p_area == (caddr_t) -1) 
    {
		perror("mmap:");
		exit(-1);
	}

    return p_area;
}


/*---------------------------------------------------------------------------*
 |"m_Mmap_dump_pvsun"
 |
 |  Dump the contents of a memory map object.
 |
 *---------------------------------------------------------------------------*/
static void m_Mmap_dump_pvsun( p_me, fd )
	O_Mmap	*p_me;	/* -> mmap object instance to dump contents of. */
	FILE	*fd;		/* -> file descriptor to dump contents to. */
{
	fprintf(fd, "\nDump of memory map object at %08x:\n", p_me);
	fprintf(fd, "  Region    Virtual   Real(Sbus)  Length\n");
	fprintf(fd, "  ------    -------   ----------  ------\n");
	fprintf(fd, "  ccube     %08x  %08x    %08x\n", 
		p_me->maps[MM_Ccube].virtual, p_me->maps[MM_Ccube].sbus, 
		p_me->maps[MM_Ccube].length);
	fprintf(fd, "  eprom     %08x  %08x    %08x\n", 
		p_me->maps[MM_Eprom].virtual, p_me->maps[MM_Eprom].sbus, 
		p_me->maps[MM_Eprom].length);
	fprintf(fd, "  1-bit     %08x  %08x    %08x\n", 
		p_me->maps[MM_M1].virtual, p_me->maps[MM_M1].sbus, 
		p_me->maps[MM_M1].length);
	fprintf(fd, "  8-bit     %08x  %08x    %08x\n", 
		p_me->maps[MM_M8].virtual, p_me->maps[MM_M8].sbus, 
		p_me->maps[MM_M8].length);
	fprintf(fd, "  24-bit    %08x  %08x    %08x\n", 
		p_me->maps[MM_M24].virtual, p_me->maps[MM_M24].sbus, 
		p_me->maps[MM_M24].length);
	fprintf(fd, "  regs      %08x  %08x    %08x\n", 
		p_me->maps[MM_Regs].virtual, p_me->maps[MM_Regs].sbus, 
		p_me->maps[MM_Regs].length);
}

/*---------------------------------------------------------------------------*
 |"m_Mmap_real_pvsun"
 |
 |  Return the real (sbus) address corresponding to a virtual address.
 |
 |	  =  real (sbus) address or -1 on failure.
 *---------------------------------------------------------------------------*/
static u_int m_Mmap_real_pvsun( p_me, virtual )
	O_Mmap	*p_me;	/* Object initializing. */
	u_int	virtual;	/* = virtual address to get real (sbus) address for. */
{
	int		i_reg;	/* Entry in 'maps' now accessing. */

	for ( i_reg = 0; i_reg < nMM; i_reg++ )
	{
		if ( p_me->maps[i_reg].virtual <= virtual && 
			virtual < p_me->maps[i_reg].virtual + p_me->maps[i_reg].length )
		{
			return p_me->maps[i_reg].sbus + 
				(virtual - p_me->maps[i_reg].virtual);
		}
	}
	ERRORF((fd, "Unable to find map for address %08x\n", virtual));
	return -1;
}


/*---------------------------------------------------------------------------*
 |"m_Mmap_terminate"
 |
 |  Terminate use of a memory map object, freeing its storage and mappings.
 |
 *---------------------------------------------------------------------------*/
static void m_Mmap_terminate( p_me )
	O_Mmap	*p_me;	/* Object initializing. */
{
	free((char*)p_me);
}


/*---------------------------------------------------------------------------*
 |"m_Mmap_virtual_pvsun"
 |
 |  Return the virtual address corresponding to a real (SBUS) address.
 |
 |	  =  virtual address or -1 on failure.
 *---------------------------------------------------------------------------*/
static u_int m_Mmap_virtual_pvsun( p_me, real )
	O_Mmap	*p_me;	/* Object initializing. */
	u_int	real;	/* = real (sbus) address to get virtual address for. */
{
	int		i_reg;	/* Entry in 'maps' now accessing. */

	for ( i_reg = 0; i_reg < nMM; i_reg++ )
	{
		if ( p_me->maps[i_reg].sbus <= real && 
			 real < p_me->maps[i_reg].sbus + p_me->maps[i_reg].length )
		{
			return p_me->maps[i_reg].virtual + 
				(real - p_me->maps[i_reg].sbus);
		}
	}
	ERRORF((fd, "Unable to find map for address %08x\n", real));
	return -1;
}

static Tv_Mmap methods_tv = {
	m_Mmap_dump_pvsun,
	m_Mmap_real_pvsun,
	m_Mmap_terminate,
	m_Mmap_virtual_pvsun
};


/*---------------------------------------------------------------------------*
 |"O_Mmap_O_Mmap"
 |
 |  Create and return a Mmap facility access object. 
 |
 |	  -> object on success, 0 on failure.
 *---------------------------------------------------------------------------*/
O_Mmap *O_Mmap_O_Mmap(device)
	char*	device;	/* -> device name, or 0 if should poll. */
{
	int		fd;		/* File descriptor for the Sbus slot containing our card. */
	int		height;	/* Height of frame buffer area in pixels. */
	int		i_map;	/* Memory map entry accessing. */
	int		offset;	/* Amount to skip into Sbus slot for our card. */
	O_Mmap*	p_me;	/* Object initializing. */
	int		width;	/* Width of frame buffer area in pixels. */


	/*  WE WERE PASSED A DEVICE NAME SO OPEN UP A DEVICE DRIVER NOT A SLOT. */
	 
    if(device == NULL)
#ifdef SUNOS4
		device = "/dev/tvtwo0";
#else
		device = "/dev/fbs/tvtwo0";
#endif
	if( device && strcmp(device, "/dev/bus-mapped") ) return use_driver(device);


	/*  DETERMINE WHICH SLOT OUR CARD IS IN AND OPEN IT. */

	switch ( test_slots(&fd, &offset) )
	{
	  case 2:
		width = 1152;
		height = 900;
		break;
	  case 3:
		width = 1280;
		height = 1024;
		break;
	  default:
		return 0;
	}

		
	/*  ALLOCATE SPACE FOR OUR OBJECT AND SET UP THE METHODS TRANSFER VECTOR. 
     *
     *  Default map entries so that failing maps will not match valid addresses.
	 */
	if ( !(p_me = (O_Mmap *)malloc(sizeof(O_Mmap))) )
	{
		ERRORF((fd, "Unable to allocate O_Mmap object\n"));
		return 0;
	}
	p_me->atv = &methods_tv;

	for ( i_map = 0; i_map < nMM; i_map++ )
	{
		p_me->maps[i_map].virtual = 0xffffffff;
		p_me->maps[i_map].sbus =    0xffffffff;
		p_me->maps[i_map].length =  0x00000000;
	}


	/*  MAP IN ACTUAL AREAS OF THE CARD. */

	p_me->maps[MM_Regs].sbus =   0x00040000 + offset;	/* REGISTERS */
	p_me->maps[MM_Regs].length = 0x00000800;
	p_me->maps[MM_Regs].virtual = (int)
		mmap_area(fd, p_me->maps[MM_Regs].sbus, p_me->maps[MM_Regs].length);

	p_me->maps[MM_Eprom].sbus =   0x00000000 + offset;	/* EPROM */
	p_me->maps[MM_Eprom].length = 0x00001000;
	p_me->maps[MM_Eprom].virtual = (int)
		mmap_area(fd, p_me->maps[MM_Eprom].sbus, p_me->maps[MM_Eprom].length);

	p_me->maps[MM_Ccube].sbus =   0x00050000 + offset;	/* CCUBE */
	p_me->maps[MM_Ccube].length = 0x00001000;
	p_me->maps[MM_Ccube].virtual = (int)
		mmap_area(fd, p_me->maps[MM_Ccube].sbus, p_me->maps[MM_Ccube].length);


	p_me->maps[MM_M1].sbus =   0x00010000 + offset;		/* 1-BIT MEMORY */
	p_me->maps[MM_M1].length = (width * height) / 8;
	p_me->maps[MM_M1].virtual = (int)
		mmap_area(fd, p_me->maps[MM_M1].sbus, p_me->maps[MM_M1].length);


	p_me->maps[MM_M8].sbus =   0x00080000 + offset;		/* 8-BIT MEMORY */
	p_me->maps[MM_M8].length = width * height;
	p_me->maps[MM_M8].virtual = (int)
		mmap_area(fd, p_me->maps[MM_M8].sbus, p_me->maps[MM_M8].length);


	p_me->maps[MM_M24].sbus =   0x00200000 + offset;	/* 24-BIT MEMORY */
	p_me->maps[MM_M24].length = width * height * 4;
	p_me->maps[MM_M24].virtual = (int)
		mmap_area(fd, p_me->maps[MM_M24].sbus, p_me->maps[MM_M24].length);
	strcpy(p_me->name, "/dev/sbusx");
	return p_me;
}


/*---------------------------------------------------------------------------*
 |"test_slot"
 |
 |  Given a name, map in an SBus slot and test to see if it contains a 
 |  Parallax Card. 
 |
 |  Notes:
 |    (1) We supply our own bus trap exception routine to trap any bus 
 |        errors we might generate. We always restore the signal handler
 |        for bus errors to the default one before exiting this routine.
 |    (2) We use a long jump to recover from any bus errors we generate.
 |    (3) Note we pass back an offset into the SBus slot for the card in
 |        case we encounter an expansion chassis.
 |
 |    =  3 on successful mapping of a Parallax TV3 device.
 |       2 on successful mapping of a Parallax TV2 device.
 |      -1 if could not open slot
 |      -2 if slot does not contain a Parallax card
 *---------------------------------------------------------------------------*/
static int test_slot(slot_name, p_fd, p_offset)
	char*	slot_name;		/* = name of slot to map and test. */
	int*	p_fd;			/* -> where to place file descriptor on success. */
	int*	p_offset;		/* -> where to place sbus offset on success. */
{
	int	  fd_slot;			/* File descriptor for slot. */
	int   i_integrix;		/* # of expansion chassis "slot" testing. */
	int   i_name;			/* Char # in prom string to start name search on. */
	int   i_slot;			/* # of slot now examining for Parallax hardware. */
	char* p_name;			/* Char in prom string to start name search on. */
	int   prom_peek;		/* Temporary to test prom existence with. */
	char* p_prom;			/* Pointer to memory mapped prom area of slot. */
	int*  plxv_mmap();
	int   sbus_offset;
	
	i_integrix = 0;
	sbus_offset = 0;

	signal(SIGBUS, buserr);

	if (setjmp(jmp_env)==0) 
	{
		/*  OPEN THE SLOT AND MAP IN THE EPROM AREA. SEARCH FOR THE NAMES
		 *  WE RECOGNIZE.
		 */
		DEBUGF(0x40, (fd, "test_slot: testing slot %s\n", slot_name));
		if ((fd_slot = open(slot_name, O_RDWR)) < 0) 
		{
			signal(SIGBUS, SIG_DFL);       /* re-instate default bus trap */
			return -1;
		}

		DEBUGF(0xC0, (fd, "test_slot: opened slot %s\n", slot_name));
		p_prom = mmap_area(fd_slot, 0x0, PLXV_EPROM_SIZE);
		prom_peek = *p_prom;  	/* trap to buserr() if you will */
		p_name = (char *) p_prom;
		DEBUGF(0xC0, (fd, "test_slot: slot %s ", slot_name));
		DEBUGF(0xC0, (fd, "has name %s\n", p_name));
	
		for (i_name=0; i_name<SEARCHLEN; i_name++,p_name ++) 
		{ 
			if ( strncmp(name_tv2, p_name , strlen(name_tv2))==0 )
			{
				munmap(p_prom, PLXV_EPROM_SIZE);
				signal(SIGBUS, SIG_DFL);   
				*p_fd = fd_slot;
				*p_offset = 0;
				return 2;
			}
			if ( strncmp(name_tv3, p_name , strlen(name_tv3))==0 ) 
			{
				munmap(p_prom, PLXV_EPROM_SIZE);
				signal(SIGBUS, SIG_DFL);   
				*p_fd = fd_slot;
				*p_offset = 0;
				return 3;
			}
			if (strncmp(name_intx, p_name , strlen(name_intx))==0) 
			{
				i_integrix=1;
				break;
			}
		}


		/*  DEVICE IN SLOT IF NOT AN EXPANSION CHASSIS OR A PARALLAX CARD */

		if ( !i_integrix )
		{
			munmap(p_prom, PLXV_EPROM_SIZE);
			signal(SIGBUS, SIG_DFL);   
			return -2;
		}


		/*  DEVICE IN SLOT IS AN EXPANSION CHASSIS, TEST EACH POTENTIAL
		 *  SUB-SLOT FOR A PARALLAX CARD.
		 */
		for (; i_integrix < 25; i_integrix++ )
		{
			if (setjmp(jmp_env)==0) 
			{
				munmap(p_prom, PLXV_EPROM_SIZE);
				sbus_offset = i_integrix*INTEGRIX_PAGE_SIZE;
				p_prom= mmap_area(sbus_offset, PLXV_EPROM_SIZE);
				prom_peek = *p_prom;  /* Test for bus err */
				p_name = (char *) p_prom;
				for (i_name=0; i_name<SEARCHLEN; i_name++,p_name++)  
				{
					if (strncmp(name_tv2, p_name, strlen(name_tv2))==0 )
					{
						munmap(p_prom, PLXV_EPROM_SIZE);
						signal(SIGBUS, SIG_DFL);   
						*p_fd = fd_slot;
						*p_offset = sbus_offset;
						return 2;
					}
					if ( strncmp(name_tv3, p_name, strlen(name_tv3))==0 ) 
					{
						munmap(p_prom, PLXV_EPROM_SIZE);
						signal(SIGBUS, SIG_DFL);   
						*p_fd = fd_slot;
						*p_offset = sbus_offset;
						return 3;
					}
				}
			}
		}


		/*  FELL OUT OF LOOP ABOVE, OUR CARD IS NOT IN THE CHASSIS. */

		munmap(p_prom, PLXV_EPROM_SIZE);
		signal(SIGBUS, SIG_DFL);       /* re-instate default bus trap */
		close(fd_slot);
		return -2;
	}


	/*  IF WE GET HERE WE FELL OUT OF THE SET JUMP ABOVE. THAT MEANS WE
	 *  CAUSED A BUS ERROR, SO RETURN AN INDICATION OF NO CARD PRESENT.
	 */
	DEBUGF(0xC0, (fd, "\ntest_slot: slot %s bus error\n", slot_name));
	munmap(p_prom, PLXV_EPROM_SIZE);
	signal(SIGBUS, SIG_DFL);       /* re-instate default bus trap */
	return -1;
}



/*---------------------------------------------------------------------------*
 |"test_slots"
 |
 |  Test all potential Sbus slots for the existence of a Parallax Card.
 |
 |    =  3 on success map of TV3
 |       2 on success map of TV2
 |      -1 on failure
 *---------------------------------------------------------------------------*/
static int test_slots(p_fd, p_offset)
	int*	p_fd;			/* -> where to place file descriptor on success. */
	int*	p_offset;		/* =  where to place sbus offset on success. */
{
	int		i_slot;		/* # of Sbus slot now testing. */
	int		rc;			/* Return code from test slot. */


	/*  TRY SOLARIS 2.x Sun 4M ARCHITECTURE: */

	for ( i_slot = 0; i_slot < NSLOTS; i_slot++ )
	{
		rc = test_slot(sol2_4m_slots[i_slot], p_fd, p_offset);
		if ( rc >=0 ) return rc;
		rc = test_slot(sol2_sparc5_slots[i_slot], p_fd, p_offset);
		if ( rc >=0 ) return rc;
	}


	/*  TRY SOLARIS 2.x Sun 4U ARCHITECTURE: */

	for ( i_slot = 0; i_slot < NSLOTS_4U; i_slot++ )
	{
		rc = test_slot(sol2_4u_slots[i_slot], p_fd, p_offset);
		if ( rc >=0 ) return rc;
	}

	/*  TRY SOLARIS 2.x Sun 4C ARCHITECTURE: */

	for ( i_slot = 0; i_slot < NSLOTS; i_slot++ )
	{
		rc = test_slot(sol2_4c_slots[i_slot], p_fd, p_offset);
		if ( rc >=0 ) return rc;
	}



	/*  TRY SOLARIS 1.x ARCHITECTURES: */

	for ( i_slot = 0; i_slot < NSLOTS; i_slot++ )
	{
		rc = test_slot(sol1_slots[i_slot], p_fd, p_offset);
		if ( rc >=0 ) return rc;
	}


	/*  IF WE FELL THROUGH TO HERE THERE IS NO CARD PRESENT. */

	return -1;
}


/*---------------------------------------------------------------------------*
 |"use_driver"
 |
 |  Create and return a Mmap facility access object using device driver. 
 |
 |	  -> object on success, 0 on failure.
 *---------------------------------------------------------------------------*/
O_Mmap *use_driver(device)
	char*	device;	/* -> device name, or 0 if should poll. */
{
	int		fd;		/* File descriptor for the Sbus slot containing our card. */
	int		i_map;	/* Memory map entry accessing. */
	int		offset;	/* Amount to skip into Sbus slot for our card. */
	O_Mmap	*p_me;	/* Object initializing. */


	DEBUGF(0xC0, (fd, "use_driver(%s) entry\n", device));
	if ((fd = open(device, O_RDWR)) < 0) 
	{
		ERRORF((fd, "use_driver(%s) exit w/open failure\n"));
		return 0;
	}
	offset = 0;

	/*  ALLOCATE SPACE FOR OUR OBJECT AND SET UP THE METHODS TRANSFER VECTOR. 
     *
     *  Default map entries so that failing maps will not match valid addresses.
	 */
	if ( !(p_me = (O_Mmap *)malloc(sizeof(O_Mmap))) )
	{
		ERRORF((fd, "Unable to allocate O_Mmap object\n"));
		ERRORF((fd, "use_driver(%s) exit w/storage allocation failure\n"));
		return 0;
	}
	p_me->atv = &methods_tv;

    p_me->fd_ccube = fd;

	for ( i_map = 0; i_map < nMM; i_map++ )
	{
		p_me->maps[i_map].virtual = 0xffffffff;
		p_me->maps[i_map].sbus =    0xffffffff;
		p_me->maps[i_map].length =  0x00000000;
	}


	/*  MAP IN ACTUAL AREAS OF THE CARD. */

	p_me->maps[MM_Regs].sbus =   0x00040000;		/* REGISTERS */
	p_me->maps[MM_Regs].length = 0x00000800;
	p_me->maps[MM_Regs].virtual = (int)
		mmap_area(fd, 0x00800000+offset, p_me->maps[MM_Regs].length);

#if 1 /* ifdef NOTDEF */
	p_me->maps[MM_Eprom].sbus =   0x00801000 + offset;		/* EPROM */
	p_me->maps[MM_Eprom].length = 0x00010000;
	p_me->maps[MM_Eprom].virtual = (int)
		mmap_area(fd, p_me->maps[MM_Eprom].sbus, p_me->maps[MM_Eprom].length);
#endif

	p_me->maps[MM_Ccube].sbus =   0x00050000;		/* CCUBE */
#if 1
	p_me->maps[MM_Ccube].length = 0x00010000;
#else
	p_me->maps[MM_Ccube].length = sizeof(C3_regs);
#endif
	p_me->maps[MM_Ccube].virtual = (int)
#ifdef SUNOS4
		mmap_area(fd, 0x00811000 + offset, p_me->maps[MM_Ccube].length);
#else
		mmap_area(fd, 0x00812000 + offset, p_me->maps[MM_Ccube].length);
#endif


	p_me->maps[MM_M1].sbus =   0x00010000;			/* 1-BIT MEMORY */
	p_me->maps[MM_M1].length = (WIDTH * HEIGHT) / 8;
	p_me->maps[MM_M1].virtual = (int)
		mmap_area(fd, 0x00000000 + offset, p_me->maps[MM_M1].length);


	p_me->maps[MM_M8].sbus =   0x00080000;			/* 8-BIT MEMORY */
	p_me->maps[MM_M8].length = WIDTH * HEIGHT;
	p_me->maps[MM_M8].virtual = (int)
		mmap_area(fd, 0x00040000 + offset, p_me->maps[MM_M8].length);


	p_me->maps[MM_M24].sbus =   0x00200000;		/* 24-BIT MEMORY */
	p_me->maps[MM_M24].length = WIDTH * HEIGHT * 4;
	p_me->maps[MM_M24].virtual = (int)
		mmap_area(fd, 0x00140000 + offset, p_me->maps[MM_M24].length);

	strcpy(p_me->name, device);

	DEBUGF(0xC0, (fd, "use_driver(%s) exit\n"));
	return p_me;
}


