/*
 *  objects_internal.h: Internal object definitions for Parallax H/W Library
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     25 Sep 94 - Andy Goodrich
 *       - Initial coding
 *
 *  Copyright(c) 1994, Parallax Graphics, Inc.
 */
#ifndef  objects_internal_h_INCLUDED
#define  objects_internal_h_INCLUDED


typedef struct _TV_PV {
    int     (*ccube_present)();     /* = 1 if ccube present, 0 if not. */
    void    (*dump)();              /* Dump contents of this object. */
    int     (*get_zoom)();          /* Get current zoom factor. */
    void    (*init_context)();      /* Initialize context from resource object*/
    void    (*init_display)();      /* Initialize display from resource object*/
    void    (*init_hw)();           /* Initialize hardware. */
    int     (*init_pll)();          /* Initialize pll hardware. */
    void    (*set_delay)();         /* Set delay value. */
    int     (*set_zoom)();          /* Set current zoom factor. */
    void    (*terminate)();         /* Terminate use of this context. */
    int     (*vertical)();          /* Enable/disable driver vert. interrupt. */
    void    (*visibility)();         /* Turn off and on visibility. */
} TV_PV;

#if defined(sun)
/* #include "objects_sun.h" */
#include "objects_pv700.h"
#endif
#if defined(__hpux)
#include "objects_pv700.h"
#endif
#if defined(_AIX)
#include "objects_pv700.h"
#endif

#endif  /*  objects_internal_h_INCLUDED */
