#ifndef RTV_OPS_INCLUDED
#define RTV_OPS_INCLUDED

typedef struct {
    int sid;
	int win_x;
	int win_y;
	int win_h;
	int win_w;
        int interlace;
	int qfactor;
	int rate;
	int fps;
	int frame_count;
	int toc_count;
	int spare[4];
	}RTV_ATTRIBUTES;

typedef struct {
        int 	loc_msgsock;
	int 	rmt_msgsock;
	int 	mode;
	int	shmid;
	int 	action;
	int     video_status;
	int  	stream_id;
	int  	request;
	int     qfactor;
	int     errno;
	int	frame_count;	
	int	next_frame;
	int	next_field;
	int	audio_mode;
	int     audio_status;
	char    *local_host;
	char    *remote_host;
	RTV_ATTRIBUTES attrs;
	}RTV_OBJECT;



#define RTV_MODE 0x40
RTV_OBJECT *RtvOpen();

/*
 **** TO DO: 
 ****
 **** FIX States to be in assending order.
 ****
 ****/
/*
 *	States of each mode.
 */
#define STOP_RTV_RECORD		1
#define START_RTV_RECORD	2
#define INIT_RTV_RECORD		3
#define START_RTV_PLAY		4
#define INIT_RTV_PLAY		5
#define STOP_RTV_PLAY		8
#define STOP_RTV		0x10
#define PAUSE_RTV_PLAY		0x20
#define RESUME_RTV_PLAY		0x40
#define PAUSE_RTV_RECORD	0x80
#define RESUME_RTV_RECORD	0x81
#define RTV_SET_PLAY_WINDOW 	0x82
#define RTV_SET_RECORD_WINDOW 	0x83
#define RTV_PAUSE_PLAY_WINDOW   0x84
#define RTV_RESUME_PLAY_WINDOW  0x85
#define RTV_SET_PLAY_ORIGIN 	0x86
#define RTV_SET_RECORD_ORIGIN 	0x87
#define IDLE_RTV_PLAY		0xff


/*
 *	Modes of Operation.
 */

#define RTV_DISK_OPEN			1
#define RTV_INIT			2
#define RTV_RECORD      		3
#define RTV_PLAY     			4
#define RTV_START			5
#define RTV_STOP			6
#define RTV_REWIND			7
#define RTV_CLOSE			8
#define RTV_SPARE_9			9
#define CLIENT_EXIT			10
#define RTV_SET_WINDOW			11
#define SET_RTV_ATTRIBUTES		12
#define SRVR_EXIT			13
#define RTV_RESERVED			14
#define RTV_VIDEO_INIT			15
#define RTV_PAUSE			16
#define RTV_RESUME			17
#define RTV_GET_FRAME_COUNT		18
#define RTV_GET_CURRENT_FRAME		20
#define RTV_PLAY_SINGLE			21
#define RTV_SHUTTLE			22
#define RTV_SELECT_FRAME		23
#define RTV_GET_STATUS			24
#define RTV_AUDIO_MODE			25
#define RTV_AUDIO_STATUS		26
#define GET_RTV_WINDOW			27
#define GET_RTV_ATTRIBUTES		28
#define RTV_SELECT_FIELD		29
#define RTV_SHM_ATTACH			30
#define RTV_PAUSE_WINDOW		31
#define RTV_RESUME_WINDOW		32
#define RTV_SET_ORIGIN			33

/*
 *	Attribute controls.
 */
#define HOSTNAME		10
#define QFACTOR			11
#define BIT_RATE		12
#define RATE			14
#define RcvrServerPort 5789
#define XmitServerPort 5790
#define RmtServerPort  5791
/*
 *  RTV Error codes;
 */

#define	RTV_ENOENT	 -2		/* No such file or directory	*/
#define RTV_BADARG	 -3     /* Bad argument supplied		*/
#define	RTV_EIO	     -5		/* I/O error					*/
#define RTV_EAGAIN	-11     /* Resource temporarily unavailable */
#define	RTV_ENODEV	-19		/* No such device				*/
#define	RTV_ENOSPC	-28		/* No space left on device		*/
#define	RTV_EPIPE	-32		/* Broken pipe					*/
#define	RTV_ECANCELED -47	/* Operation canceled			*/
#define RTV_END_FRAME -98
#define RTV_EOF_DETECT -99

typedef struct {
	char *string;
	int   opcode;
}CMDTBL;

typedef struct {
	char *string;
	int  value;
	}ARGTBL;


static CMDTBL cmdtbl[]= { 
			  {"open",RTV_DISK_OPEN},
			  {"init",RTV_INIT},
			  {"record",RTV_RECORD},
			  {"play",RTV_PLAY},
			  {"start",RTV_START},
			  {"stop",RTV_STOP},
			  {"rewind",RTV_REWIND},
			  {"close",RTV_CLOSE},
			  {"spare9",RTV_SPARE_9},
			  {"disconnect",CLIENT_EXIT},
			  {"set_window",RTV_SET_WINDOW},
			  {"set_attrs",SET_RTV_ATTRIBUTES},
			  {"quit",SRVR_EXIT},
			  {"reserved",RTV_RESERVED},
			  {"vidinit",RTV_VIDEO_INIT},
			  {"pause",RTV_PAUSE},
			  {"resume",RTV_RESUME},
			  {"framecnt",RTV_GET_FRAME_COUNT},
			  {"frameno",RTV_GET_CURRENT_FRAME},
			  {"single",RTV_PLAY_SINGLE},
			  {"shuttle",RTV_SHUTTLE},
			  {"select_frm",RTV_SELECT_FRAME},
			  {"status",RTV_GET_STATUS},
			  {"audio_mode",RTV_AUDIO_MODE},
			  {"audio_status",RTV_AUDIO_STATUS},
			  {"get_window",GET_RTV_WINDOW},
			  {"get_attrs",GET_RTV_ATTRIBUTES},
			  {"select_fld",RTV_SELECT_FIELD},
			  {"shmatt",RTV_SHM_ATTACH},
			  {"pause_win",RTV_PAUSE_WINDOW},
			  {"resume_win",RTV_RESUME_WINDOW},
			  { "set_origin",RTV_SET_ORIGIN},
			  {"\0",0},
			  };
#define CMDTBLSIZ 	        sizeof(cmdtbl)

#endif /* RTV_OPS_INCLUDED */
