/*
 *  xv_obj.h:	Declaration of Xv video extension access object.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     26 May 93 - Andy Goodrich
 *       - Initial coding
 *
 *  Copyright(c) 1993, Andrew C. Goodrich
 */
#ifndef  xv_obj_INCLUDED
#define xv_obj_INCLUDED

typedef struct Tv_XV {
	Window	(*create_window)();	/* Open an xv window for a port. */	
	void	(*dump)();			/* Dump object's contents. */
	int		(*source)();		/* Set up video source information. */
	int		(*status)();		/* Return status of a port. */
	void 	(*use_window)();	/* Specify a window to use with a port. */	
	void	(*video_off)();		/* Turn off video in the window. */
	void	(*video_on)();		/* Turn on video in the window. */
} Tv_XV;

typedef struct O_XV {
	Tv_XV			*tv;			/* Methods transfer vector. */
	Atom			a_brightness;	/* Atom for Xv brightness attribute. */
	Atom			a_contrast;		/* Atom for Xv contrast attribute. */
	Atom			a_encoding;		/* Atom for Xv encoding attribute. */
	Atom			a_hue;			/* Atom for Xv hue attribute. */
	Atom			a_saturation;	/* Atom for Xv saturation attribute. */
	Atom			a_status;		/* Atom for Xv status attribute. */
	Colormap		color_map;		/* Color map using for our window. */
	Display			*display;		/* Display xv window is on. */
	GC				gc;				/* Graphics context for 'w_main'. */
	int				i_adaptor;		/* # of adaptor this object is for. */
	int				i_port;			/* # of port this object is for. */
	int				n_adaptors;		/* # of adaptors 'p_adaptors' points to. */
	int				n_encodings;	/* # of encodings 'p_encodings' points to */
	XvAdaptorInfo	*p_adaptor;		/* Information on adaptors we are using. */
	XvAdaptorInfo	*p_adaptors;	/* Information on adaptors for 'dpy'. */
	XvEncodingInfo	*p_encodings;	/* Encoding information for port using. */
	Visual			*p_visual;		/* Information on our window's visual. */
	int				port_id;		/* Port identifier of this object's port. */
	int				screen;			/* Default screen for display 'dpy'. */
	int				sig_height;     /* Video signal width. */
	int				sig_width;      /* Video signal height. */
	Window			w_main;			/* Main window for this application. */
	Window			w_root;			/* Root window for display 'dpy'. */
	u_int			xv_error_base;	/* Base for error messages. */
	u_int			xv_event_base;	/* Base for event ids. */
	u_int			xv_major_op;	/* Major opcode for this version of xv. */
	u_int 			xv_revision;	/* Revision within version of xv. */
	u_int 			xv_version;		/* Version of xv being used. */
} O_XV;

O_XV *O_XV_Create();

#define O_XV_Dump(obj, fd) \
	(* ((O_XV *)(obj))->tv->dump)(obj, fd)

#define O_XV_Source(obj, dmsd, timing, encoding, instance) \
	(* ((O_XV *)(obj))->tv->source)(obj, dmsd, timing, encoding, instance)

#define O_XV_Status(obj) \
	(* ((O_XV *)(obj))->tv->status)(obj)

#define O_XV_Video_Off(obj) \
	(* ((O_XV *)(obj))->tv->video_off)(obj)

#define O_XV_Video_On(obj) \
	(* ((O_XV *)(obj))->tv->video_on)(obj)

#define O_XV_Use_Window(obj, window) \
	(*((O_XV *)(obj))->tv->use_window)(obj, window)

#define O_XV_Window(obj, vis_type, depth) \
	(*((O_XV *)(obj))->tv->create_window)(obj, vis_type, depth)

#endif /* xv_obj_INCLUDED */
