/* $XConsortium: windowstr.h,v 5.16 92/12/18 16:41:02 rws Exp $ */
/***********************************************************
Copyright 1987 by Digital Equipment Corporation, Maynard, Massachusetts,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/

#ifdef SMART_CLIPPING
/******************************************************************************
 *
 *			       HPX Smart Clipping Code
 *
 * (c) Copyright Hewlett-Packard Company, 1991.
 * All rights are reserved.  Copying or other
 * reproduction of this program except for archival
 * purposes is prohibited without the prior 
 * written consent of Hewlett-Packard Company. 
 *
 * RESTRICTED RIGHTS LEGEND
 *
 * Use, duplication, or disclosure by the U.S. Government
 * is subject to restrictions as set forth in 
 * subdivision (c) (1) (ii) of the Rights in Technical
 * Data and Computer Software clause at 
 * 252.227-7013.
 *
 * HEWLETT-PACKARD COMPANY
 * Fort Collins, Colorado
 *
 * $Header:   /export/home/dist1/source/xv/X11R5/server/archives/windowstr.h_v   1.0   16 Sep 1996 17:26:52   dave  $
 *
 *	The contents of this file conditionally compiled by #define'ing
 *	SMART_CLIPPING deals with the HPX Smart Clipping code.
 *
 ******************************************************************************/
#endif /* SMART_CLIPPING */
#ifdef HOODIX
/******************************************************************************
 *
 *                          HPX Fast DIX Window Code
 *
 * (c) Copyright Hewlett-Packard Company, 1992.
 * All rights are reserved.  Copying or other
 * reproduction of this program except for archival
 * purposes is prohibited without the prior
 * written consent of Hewlett-Packard Company.
 *
 * RESTRICTED RIGHTS LEGEND
 *
 * Use, duplication, or disclosure by the U.S. Government
 * is subject to restrictions as set forth in
 * subdivision (c) (1) (ii) of the Rights in Technical
 * Data and Computer Software clause at
 * 252.227-7013.
 *
 * HEWLETT-PACKARD COMPANY
 * Fort Collins, Colorado
 *
 * $Header:   /export/home/dist1/source/xv/X11R5/server/archives/windowstr.h_v   1.0   16 Sep 1996 17:26:52   dave  $
 *
 *      The contents of this file conditionally compiled by #define'ing
 *      HOODIX deals with the HPX fast DIX window code.
 *
 ******************************************************************************/
#endif /* HOODIX */

#ifndef WINDOWSTRUCT_H
#define WINDOWSTRUCT_H

#include "window.h"
#include "pixmapstr.h"
#include "regionstr.h"
#include "cursor.h"
#include "property.h"
#include "resource.h"	/* for ROOT_WINDOW_ID_BASE */
#include "dix.h"
#include "miscstruct.h"
#include "Xprotostr.h"

#define GuaranteeNothing	0
#define GuaranteeVisBack	1

#define SameBackground(as, a, bs, b)				\
    ((as) == (bs) && ((as) == None ||				\
		      (as) == ParentRelative ||			\
 		      SamePixUnion(a,b,as == BackgroundPixel)))

#define SameBorder(as, a, bs, b)				\
    EqualPixUnion(as, a, bs, b)

typedef struct _WindowOpt {
    VisualID		visual;		   /* default: same as parent */
    CursorPtr		cursor;		   /* default: window.cursorNone */
    Colormap		colormap;	   /* default: same as parent */
    Mask		dontPropagateMask; /* default: window.dontPropagate */
    Mask		otherEventMasks;   /* default: 0 */
    struct _OtherClients *otherClients;	   /* default: NULL */
    struct _GrabRec	*passiveGrabs;	   /* default: NULL */
    PropertyPtr		userProps;	   /* default: NULL */
    unsigned long	backingBitPlanes;  /* default: ~0L */
    unsigned long	backingPixel;	   /* default: 0 */
#ifdef SHAPE
    RegionPtr		boundingShape;	   /* default: NULL */
    RegionPtr		clipShape;	   /* default: NULL */
#endif
#ifdef XINPUT
    struct _OtherInputMasks *inputMasks;   /* default: NULL */
#endif
} WindowOptRec, *WindowOptPtr;

#define BackgroundPixel	    2L
#define BackgroundPixmap    3L

#ifdef SMART_CLIPPING
typedef struct _HpxSmartClipTreeRec {
    WindowPtr		parent;		/* ancestor chain */
    WindowPtr		nextSib;	/* next lower sibling */
    WindowPtr		prevSib;	/* next higher sibling */
    WindowPtr		firstChild;	/* top-most child */
    WindowPtr		lastChild;	/* bottom-most child */
} HpxSmartClipTreeRec, *HpxSmartClipTreePtr;


typedef struct _HpxSmartClipRec {
    HpxSmartClipTreeRec	stree;		/* Secondary Window Tree Pointers */
    HpxSmartClipTreeRec	utree;		/* Utility Window Tree Pointers */
    WindowPtr		pWindow;	/* Pointer back to the window that
					 * points to this structure */
    RegionRec		clipList;	/* clipping rectangle for output */
    RegionRec		borderClip;	/* NotClippedByChildren + border */
    union _Validate	*valdata;
} HpxSmartClipRec, *HpxSmartClipPtr;

#define	UTREE_REC(pWin)\
    ((HpxSmartClipTreeRec)((pWin)->pSmartClip->utree))
#define	UTREE_PTR(pWin)\
    ((HpxSmartClipTreePtr)(&(pWin)->pSmartClip->utree))
#define	STREE_REC(pWin)\
    ((HpxSmartClipTreeRec)((pWin)->pSmartClip->stree))
#define	STREE_PTR(pWin)\
    ((HpxSmartClipTreePtr)(&(pWin)->pSmartClip->stree))


/**** The window structure's "smartClipState" field is a 3-bit mask with the
 **** following boolean fields:
 ****
 ****	<Secondary Window><Utility Window><Secondary Descedants>
 ****
 **** The window structure's "smartClipState" field contains the following
 **** encodings:
 ****/

/* "Normal windows" are those that are on neither the secondary (stree) or
 * utility (utree) window trees and don't have any secondary descendant
 * windows.  Such a window's pSmartClip field is NULL.
 */
#define	SC_NORMAL_WINDOW		0

/* An stree ancestor window is one that isn't on either tree, but does have
 * one or more secondary window descendants.  Such a window's pSmartClip field
 * points directly to the HpxSmartClipRec of the highest descendant on the
 * utree (which is either an stree leaf or utree node window).
 */
#define	SC_STREE_ANCESTOR_WINDOW	1

/* An utree node window is a non-secondary window that has one or more other
 * utree node or stree descendant windows.  Such a window's pSmartClip field
 * points to its own HpxSmartClipRec structure (however, the "stree" sub-struct
 * isn't valid).
 */
#define	SC_UTREE_NODE_WINDOW		3

/* An stree leaf window is a secondary window that has no secondary descendant
 * windows.  Such a window is a leaf on both the stree and utree.  Such a
 * window's pSmartClip field points to its own HpxSmartClipRec structure, which
 * is fully filled out.
 */
#define	SC_STREE_LEAF_WINDOW		6

/* An stree node window is a secondary window that has secondary descendant
 * windows.  Such a window is a node on both the stree and utree.  Such a
 * window's pSmartClip field points to its own HpxSmartClipRec structure, which
 * is fully filled out.
 */
#define	SC_STREE_NODE_WINDOW		7


/**** The following values provide useful Short-cuts to "and" with
 **** pWin->smartClipState:
 ****/
#define	SC_UTREE_WINDOW			0x2
#define	SC_STREE_WINDOW			0x4
#endif /* SMART_CLIPPING */

typedef struct _Window {
    DrawableRec		drawable;
    WindowPtr		parent;		/* ancestor chain */
    WindowPtr		nextSib;	/* next lower sibling */
    WindowPtr		prevSib;	/* next higher sibling */
    WindowPtr		firstChild;	/* top-most child */
    WindowPtr		lastChild;	/* bottom-most child */
    RegionRec		clipList;	/* clipping rectangle for output */
    RegionRec		borderClip;	/* NotClippedByChildren + border */
    union _Validate	*valdata;
    RegionRec		winSize;
    RegionRec		borderSize;
    DDXPointRec		origin;		/* position relative to parent */
    unsigned short	borderWidth;
    unsigned short	deliverableEvents;
    Mask		eventMask;
    PixUnion		background;
    PixUnion		border;
    pointer		backStorage;	/* null when BS disabled */
    WindowOptPtr	optional;
    unsigned		backgroundState:2; /* None, Relative, Pixel, Pixmap */
    unsigned		borderIsPixel:1;
    unsigned		cursorIsNone:1;	/* else real cursor (might inherit) */
    unsigned		backingStore:2;
    unsigned		saveUnder:1;
    unsigned		DIXsaveUnder:1;
    unsigned		bitGravity:4;
    unsigned		winGravity:4;
    unsigned		overrideRedirect:1;
    unsigned		visibility:2;
    unsigned		mapped:1;
    unsigned		realized:1;	/* ancestors are all mapped */
    unsigned		viewable:1;	/* realized && InputOutput */
    unsigned		dontPropagate:3;/* index into DontPropagateMasks */
    unsigned		forcedBS:1;	/* system-supplied backingStore */
    DevUnion		*devPrivates;
#ifdef SMART_CLIPPING
    unsigned		smartClipState:3;
    unsigned		useSCRegions:1;	/* The window's normal clip regions
					 * come from pSmartClip */
    unsigned            scVisibility:2; /* all stree window's visibility */
    HpxSmartClipPtr	pSmartClip;
#endif /* SMART_CLIPPING */
    
} WindowRec;

#ifdef HOODIX
typedef struct _WindowAllocRec
{
    struct _WindowAllocRec      *next;
    struct _WindowAllocRec      *nextWithFree;

    long                        index;
    unsigned long               mask;

    /* Number of WindowRec's currently free: */
    unsigned long               nFreeWins;

    /* Bit fields: one bit for each of 2048 windows */
    unsigned long               freeWins[64];   /* 0-31, 32-63, 64-95, 96-127 */

    /* 128 of the following: 4 bytes that state which of the 128 this "one" is,
     * followed by enough memory for a WindowRec & its associated private(s):
     */
    char                        *pWindows;
} WindowAllocRec, *WindowAllocPtr;


typedef struct _WindowAllocMainRec
{
    WindowAllocPtr      pFirstWindowAllocRec;
    WindowAllocPtr      pLastWindowAllocRec;
    WindowAllocPtr      pFirstFreeWindowAllocRec;
    long                nWindowAllocRecs;
} WindowAllocMainRec, *WindowAllocMainPtr;
#endif /* HOODIX */

/*
 * Ok, a bunch of macros for accessing the optional record
 * fields (or filling the appropriate default value)
 */

extern WindowPtr    FindWindowWithOptional();
extern Mask	    DontPropagateMasks[];

#define wTrackParent(w,field)	((w)->optional ? \
				    w->optional->field \
 				 : FindWindowWithOptional(w)->optional->field)
#define wUseDefault(w,field,def)	((w)->optional ? \
				    w->optional->field \
				 : def)

#define wVisual(w)		wTrackParent(w, visual)
#define wCursor(w)		((w)->cursorIsNone ? None : wTrackParent(w, cursor))
#define wColormap(w)		((w)->drawable.class == InputOnly ? None : wTrackParent(w, colormap))
#define wDontPropagateMask(w)	wUseDefault(w, dontPropagateMask, DontPropagateMasks[(w)->dontPropagate])
#define wOtherEventMasks(w)	wUseDefault(w, otherEventMasks, 0)
#define wOtherClients(w)	wUseDefault(w, otherClients, NULL)
#ifdef XINPUT
#define wOtherInputMasks(w)	wUseDefault(w, inputMasks, NULL)
#else
#define wOtherInputMasks(w)	NULL
#endif
#define wPassiveGrabs(w)	wUseDefault(w, passiveGrabs, NULL)
#define wUserProps(w)		wUseDefault(w, userProps, NULL)
#define wBackingBitPlanes(w)	wUseDefault(w, backingBitPlanes, ~0L)
#define wBackingPixel(w)	wUseDefault(w, backingPixel, 0)
#ifdef SHAPE
#define wBoundingShape(w)	wUseDefault(w, boundingShape, NULL)
#define wClipShape(w)		wUseDefault(w, clipShape, NULL)
#endif
#define wClient(w)		(clients[CLIENT_ID((w)->drawable.id)])
#define wBorderWidth(w)		((int) (w)->borderWidth)

/* true when w needs a border drawn. */

#ifdef SHAPE
#define HasBorder(w)	((w)->borderWidth || wClipShape(w))
#else
#define HasBorder(w)	((w)->borderWidth)
#endif

extern int DeleteWindow();
extern int ChangeWindowAttributes();
extern int WalkTree();
extern Bool CreateRootWindow();
extern WindowPtr CreateWindow();
extern int DeleteWindow();
extern int DestroySubwindows();
extern int ChangeWindowAttributes();
extern int GetWindowAttributes();
extern int ConfigureWindow();
extern int ReparentWindow();
extern int MapWindow();
extern int MapSubwindow();
extern int UnmapWindow();
extern int UnmapSubwindow();
extern RegionPtr NotClippedByChildren();
extern void SendVisibilityNotify();

typedef struct _ScreenSaverStuff {
    WindowPtr pWindow;
    XID       wid;
    char      blanked;
    Bool      (*ExternalScreenSaver)();	
} ScreenSaverStuffRec, *ScreenSaverStuffPtr;

#define SCREEN_IS_BLANKED   0
#define SCREEN_ISNT_SAVED   1
#define SCREEN_IS_TILED     2
#define SCREEN_IS_BLACK	    3

#define HasSaverWindow(i)   (savedScreenInfo[i].pWindow != NullWindow)

extern int ScreenSaverBlanking, ScreenSaverAllowExposures;
extern int screenIsSaved;

extern ScreenSaverStuffRec savedScreenInfo[MAXSCREENS];

#endif /* WINDOWSTRUCT_H */
