/******************************************************************************
 *
 *                     Dynamic Loading Mechanism
 *
 * (c) Copyright Hewlett-Packard Company, 1994-1995.
 * All rights are reserved.  Copying or other
 * reproduction of this program except for archival
 * purposes is prohibited without the prior
 * written consent of Hewlett-Packard Company.
 *
 * RESTRICTED RIGHTS LEGEND
 *
 * Use, duplication, or disclosure by the U.S. Government
 * is subject to restrictions as set forth in
 * subdivision (c) (1) (ii) of the Rights in Technical
 * Data and Computer Software clause at
 * 252.227-7013.
 *
 * HEWLETT-PACKARD COMPANY
 * Fort Collins, Colorado
 *
 * $Header:   /export/home/dist1/source/xv/X11R6/hp/archives/broker.h_v   1.0   17 Sep 1996 06:57:12   dave  $
 *
 *      This file contains various typedefs, macros and extern declarations
 *      used by Dynamic Loaded Brokers.
 *
 ******************************************************************************/

#ifndef BROKER_H
#define BROKER_H

#include <sys/framebuf.h>
#include "dl.h"
#include "X.h"
#include "misc.h"
#include "hpddxScreen.h"

/*
 ******************************************************************************
 **
 ** DDX Screen Brokers Description
 **
 **
 ** DDX Brokers Description: Procedure:         ?GetScreenBrokerInterface()
 ** DDX Brokers Description: Procedure:         ?GetScreenBid()
 ** DDX Brokers Description: Procedure:         ?GetScreenModuleList()
 ** 
 ** void ?GetScreenBrokerInterface(HpddxVersionRec *pVersion)
 **
 ** void ?GetScreenBid(
 **	  	DlDdxScreenBidPtr pDeviceAndExtensionDesc)
 ** 
 ** void ?GetScreenModuleList(
 **       	DlDdxScreenDevicePtr    pScreenDevices,
 **       	Int32                   nScreens,
 **       	Int32                   whichScreen,
 ** 	  	DlModuleListItemPtr     *ppModuleList,
 **       	char                    **nameScreenInit)
 **
 **      The Xserver assumes that the base function names, listed below,
 **      will be prefixed with the base name of the broker shared library
 **      created from this module.
 **              - ?GetScreenBrokerInterface()
 **              - ?GetScreenBid()
 **              - ?GetScreenModuleList()
 **      e.g. crx.1
 **              => crxGetScreenBrokerInterface()
 **              => crxGetScreenBid()
 **              => crxGetScreenModuleList()
 **
 **      Further it is encouraged that the shared library for this module,
 **      the broker module, will have the same name as the shared library
 **      module for the actual driver. With the exception that the first
 **	 letter of the broker module be capitalized to avoid a "xdb" debugger
 **	 limitation.
 **
 **	<base_module_name>GetScreenBrokerInterface()
 **	     This entry point will be called with a pointer to an initialized
 **	     version record.  The initial version values identify the current
 **	     broker interface version. This routine needs to reset these values
 **	     to reflect the broker interfaces that the other entry points
 **	     in this module are based on.
 **
 **	     - Returning the same/matching version number values will result
 **	       in calls to the other entry points in this module.
 **
 **	     - Returning version numbers representing an earlier broker
 **	       interface version may result in continued processing.  That
 **	       will be based on the ability of the current X server to deal
 **	       with the given earlier broker interfaces.
 **
 **	     - Returning version numbers representing a newer broker
 **	       interface version will result in rejection of this broker.
 **	       The current X server will not attempt to deal with broker
 **	       interfaces newer than its current implementation.
 **
 **          Requested Information: 
 **              - Broker Interface Version:                
 **              	- Major Number:              e.g. "1"
 **              	- Minor Version:             e.g. "0"
 **
 **	<base_module_name>GetScreenBid()
 **	     Needs to determine if the current screen(s) can be supported
 **	     and bid accordingly.  Where the higher the bid the greater the
 **	     the likelihood of being selected as the DDX driver for a
 **	     given screen.
 **
 **	     The screen broker's bidder is passed all current devices at
 **	     once so that multihead configurations can be taken under
 **	     consideration.
 **
 **	     For HP-UX 10.0 HP brokers will use bids in the range of 1-100,
 **	     third party brokers should bid above this range.
 **
 **	     Also, the list of contending dynamic extension brokers
 **	     should be reviewed and votes cast as to available support
 **	     for each extension. The highest bidding DDX brokers
 **	     determine which contending extensions will be launched through
 **          this mechanism.
 **
 **
 **	<base_module_name>GetScreenModuleList()
 **          Needs to define for the X server the list of modules to load to
 **          support the DDX driver and provide the name of the DDX
 **          initialization routine. The module list must be created from a
 **          continuous block of memory allocated by this routine and
 **	     later freed by the X server. The first NULL pointer to the
 **	     "next" data structure in the module list is interpreted by
 **	     the X server as the end of the module list.
 **
 **	     The screen broker is passed the entire list of current screen
 **	     devices incase multihead configurations influences the final
 **	     module list to use.
 **
 **	     Returning NULL list indicates a failure condition.
 **
 **          Requested Information:
 **            For Module List;
 **              - DDX Module Name:           e.g. "crx.1"
 **              - DDX Module Path:           e.g. "hp"
 **                      (relative: "/usr/lib/X11/Xserver/modules/screens/")
 **              - Extension Library Bind State:     e.g. BIND_DEFERRED
 **
 **            For Routine interfaces;
 **              - Initialization Routine Name:  e.g. "crxScreenInit"
 ******************************************************************************
 */

/*
 ******************************************************************************
 **
 ** Extension Brokers Description
 **
 ** DDX Brokers Description: Procedure:         ?GetExtensionBrokerInterface()
 ** DDX Brokers Description: Procedure:         ?GetExtensionInformation()
 ** DDX Brokers Description: Procedure:         ?GetScreenModuleList()
 ** 
 ** void ?GetExtensionBrokerInterface(HpddxVersionRec *pVersion)
 ** 
 ** void ?GetExtensionInformation(
 **	  	DlExtensionInfoPtr pExtensionInfoRec)
 ** 
 ** void ?GetExtensionModuleList(
 **       	DlModuleListItemPtr *ppModuleList,
 **       	char               **nameExtnInit,
 **       	char               **nameExtnRegister)
 ** 
 **      The Xserver assumes that the base function names, listed below,
 **      will be prefixed with the base name of the broker shared module
 **      created from this module.
 **              - ?GetExtensionBrokerInterface()
 **              - ?GetExtensionInformation()
 **              - ?GetExtensionModuleList()
 **      e.g. mbx.1
 **              => mbxGetExtensionBrokerInterface()
 **              => mbxGetExtensionInformation()
 **              => mbxGetExtensionModuleList()
 **
 **      Further it is encouraged that the shared library for this module,
 **      the broker module, will have the same name as the shared library
 **      module for the actual extension. With the exception that the first
 **	 letter of the broker module be capitalized to avoid a "xdb" debugger
 **	 limitation.
 **
 **     <base_module_name>GetScreenBrokerInterface()
 **          This entry point will be called with a pointer to an initialized
 **          version record.  The initial version values identify the current
 **          broker interface version. This routine needs to reset these values
 **          to reflect the broker interfaces that the other entry points
 **          in this module are based on.
 **
 **          - Returning the same/matching version number values will result
 **            in calls to the other entry points in this module.
 **
 **          - Returning version numbers representing an earlier broker
 **            interface version may result in continued processing.  That
 **            will be based on the ability of the current X server to deal
 **            with the given earlier broker interfaces.
 **
 **          - Returning version numbers representing a newer broker
 **            interface version will result in rejection of this broker.
 **            The current X server will not attempt to deal with broker
 **            interfaces newer than its current implementation.
 **
 **          Requested Information: 
 **              - Broker Interface Version:                
 **              	- Major Number:              e.g. "1"
 **              	- Minor Version:             e.g. "0"
 **
 **      <base_module_name>GetExtensionInformation()
 **          Needs to simply fully initialize the data structure provided by
 **          the X server.  Failure to do so will result in the extension
 **          being rejected by the X server.
 **
 **          Requested Information: 
 **              - Extension Name:                   e.g. "Multi-Buffering"
 **              - Extension Vendor Name:            e.g. "Hewlett-Packard"
 **              - Extension Version:                
 **              	- Major Number:              e.g. "1"
 **              	- Minor Version:             e.g. "0"
 **              - Extension/DDX Input Dependancy:   (see list below)
 **              - Extension/DDX Screen Dependancy:  (see list below)
 **
 **              Where Extension/DDX Dependencies are defined as: (in broker.h)
 **                  DL_EXTN_DDX_DEPEND_NONE
 **                      Extension will work with no DDX involvement and
 **                      no DDX involvement is possible.
 **                  DL_EXTN_DDX_DEPEND_BENEFITICIAL
 **                      Extension will work without DDX involvement, however
 **                      performance and/or functionality will improve
 **                      with DDX support.
 **                  DL_EXTN_DDX_DEPEND_TIGHT
 **                      Extension requires some DDX involvement to function.
 **                  DL_EXTN_DDX_DEPEND_EXTREME
 **                      Extension requires significant DDX support to function.
 **
 **      <base_module_name>GetExtensionModuleList()
 **          Needs to define for the X server the list of modules to load to
 **          support the extension and provide the names of the extension
 **          initialization and screen registrations routines. The module
 **          list should be created from a continuous block of memory
 **          allocated by this routine and later freed by the X server.
 **          The first NULL pointer to the "next" data structure in the module
 **          list is interpreted by the X server as the end of the module list.
 **	     The extension registration routine name may be a NULL string, "",
 **	     if the extension does not require hardware support.
 **
 **          Requested Information:
 **            For Module List;
 **              - Extension Module Name:           e.g. "mbx.1"
 **              - Extension Module Path:           e.g. "hp"
 **                      (relative: "/usr/lib/X11/Xserver/modules/extensions/")
 **              - Extension Library Bind State:     e.g. BIND_DEFERRED
 **
 **            For Routine interfaces;
 **              - Initialization Routine Name:  e.g. "MultibufferExtensionInit"
 **              - Registration Routine Name:    e.g. "MbxRegisterFunction"
 **
 ******************************************************************************
 */




/*
 ******************************************************************************
 **
 ** Standard HP Data-type's:
 **
 ******************************************************************************
 */

#ifndef Card8
#define Card8  unsigned char
#endif

#ifndef Card16
#define Card16  unsigned short
#endif


#ifndef Card32
#define Card32  unsigned long
#endif

#ifndef Int32
#define Int32  long
#endif

/*
 ******************************************************************************
 **
 ** Dl current interface version numbers
 ** Only these Version numbers are guaranteed to be accepted.
 **
 ** Version numbers greater/later than these are invalid
 ** Version numbers prior to these may be valid. An attempt will be made
 ** to proces brokers with prior version numbers
 **
 ******************************************************************************
 */

#define BROKER_INTERFACE_MAJOR_NUMBER	1
#define BROKER_INTERFACE_MINOR_NUMBER	0

/*
 ******************************************************************************
 **
 ** Dl bind states for shared module loading
 **
 ******************************************************************************
 */

typedef Int32 DlBindState;

/* The following are the legal values of the DlBindState type: */
#define	DL_BIND_IMMEDIATE 	BIND_IMMEDIATE	/* Resolve symbol references
						 * when the module is loaded.
				 		 */
#define DL_BIND_DEFERRED	BIND_DEFERRED	/* Delay code symbol resolution
						 * until actual reference.
				 		 */
#define DL_BIND_FIRST		BIND_FIRST	/* Place the library at the
						 * head of the shared module
						 * symbol search order.
						 */
#define DL_BIND_VERBOSE		BIND_VERBOSE	/* Print verbose messages
						 * concerning possible
				 		 * unsatisfied symbols.
				 		 */

/*
 ******************************************************************************
 **
 ** Information returned by DDX or Extension Broker's
 ** <broker_basefile_name>GetScreenModuleList() routine:
 ** <broker_basefile_name>GetExtensionModuleList() routine:
 **
 ** "*next == NULL" terminates list.
 **
 ** List must be allocated thru a single memory request so that it can be
 ** freed via single procedure call.
 **
 ******************************************************************************
 */

typedef struct _DlModuleListItemRec
{
  char				*modulePath;
  char 				*moduleName;
  DlBindState			bindState;
  struct _DlModuleListItemRec	*next;
} DlModuleListItemRec, *DlModuleListItemPtr;



/*
 ******************************************************************************
 **
 ** Extension dependency clases
 **
 ******************************************************************************
 */

typedef unsigned int DlExtensionDdxDepend;

/* The following are the legal values of the DlExtensionDdxDepend type: */
#define DL_EXTN_DDX_DEPEND_NONE	      1 /* Extension will work with no DDX
				 	 * involvement and no DDX involvement
				         * is possible.
     					 */
#define DL_EXTN_DDX_DEPEND_BENEFICIAL 2 /* Extension will work without DDX
					 * involvement, however performance
					 * and/or functionality will improve
				  	 * with DDX support.
					 */
#define DL_EXTN_DDX_DEPEND_TIGHT      3 /* Extension utilizes DDX support.
					 * However, it will ignore DDX's 
					 * that are not involved.
				 	 */
#define DL_EXTN_DDX_DEPEND_EXTREME    4 /* Extension requires DDX support
					 * to function.
					 */

/*
 ******************************************************************************
 **
 ** DDX extension dependency votes
 **
 ******************************************************************************
 */


typedef Card32 DlDdxExtensionVotes;

/* The following are the legal values of the DlExtensionVotes type: */
#define DL_DDX_EXTN_VOTE_YES	     11 /* DDX: Yes, load extension. */
#define DL_DDX_EXTN_VOTE_VETO	     12 /* DDX: No, do not load extension. */
#define DL_DDX_EXTN_VOTE_ABSTAIN     13 /* DDX: Load extension based upon
					 * other DDX broker votes.
					 */

/*
 ******************************************************************************
 **
 ** Information returned by an Extension Broker's
 ** <broker_basefile_name>GetExtensionInfo() routine:
 **
 ******************************************************************************
 */

typedef struct _DlExtensionInfoRec
{
    char	     		*name;	  	/* Extension's name */
    char	     		*vendor;	/* Vendor's name */
    HpddxVersionRec 		version;	/* Extension's version */
    DlExtensionDdxDepend  	inputDepend;	/* Dependancy between DDX input
					 	 * modules and this extension
					 	 */
    DlExtensionDdxDepend  	screenDepend;	/* Dependancy between DDX screen
					 	 * modules and this extension
					 	 */
} DlExtensionInfoRec, *DlExtensionInfoPtr;

/*
 ******************************************************************************
 **
 ** Information passed to DDX Brokers to describe extensions and collect
 ** their extension votes.
 **
 ******************************************************************************
 */

typedef struct _DlExtensionBallotItemRec
{
  DlExtensionInfoPtr			pExtensionDescription;	
  DlDdxExtensionVotes	  	   	vote[MAXSCREENS];
  struct _DlExtensionBallotItemRec	*next;
} DlExtensionBallotItemRec, *DlExtensionBallotItemPtr;



/*
 ******************************************************************************
 **
 ** Information used to  descibe screen graphics frame buffer
 **
 ******************************************************************************
 */

typedef struct _DlDdxScreenDeviceRec
{
/* SLS
 * default visual
 * ????
 * 
 */
  char			*deviceFileName;
  crt_frame_buffer_t    desc;
} DlDdxScreenDeviceRec, *DlDdxScreenDevicePtr;

/*
 ******************************************************************************
 **
 ** Information passed to DDX screen Broker's 
 ** <broker_basefile_name>GetScreenBid() routine:
 **
 ******************************************************************************
 */

typedef struct _DlDdxScreenBidRec
{
  Int32					nScreensToBidOn;
  DlDdxScreenDeviceRec			deviceList[MAXSCREENS];
  Card32				bid[MAXSCREENS];
  DlExtensionBallotItemPtr		pExtensionBallotList;
} DlDdxScreenBidRec, *DlDdxScreenBidPtr;

#endif
