/*
** File:
**
**   xvdebug.h --- Macros to produce debugging and error messages for xv.
**                 Messages are written to the file /usr/local/xv/xv_log.
**                 Debugging may be turned off at compile time by failing
**                 to define XV_DIAG. Note that changing the value of 
**                 XV_DIAG may be used to provide up to 32 different
**                 debugging message enables.
**
** Author:
**
**   Andy Goodrich (RasterOps)
**
** Revisions:
**
**   08.08.92 Andy Goodrich
**     - Initial Definition.
** $Log:   /plxhp/dave/pvcsrc/xv/include/archives/xvdebug.h_v  $
 * 
 *    Rev 1.5   09 Sep 1996 10:39:26   dave
 * Andy Goodrich: Removed extraneous ; from DEFINE_TRACEF macro.
** Revision 1.5  1995/09/14 12:55:45  dist
** Andy Goodrich: Removed extraneous ; from DEFINE_TRACEF macro.
**
 * Revision 1.4  95/09/14  12:53:21  12:53:21  dist ()
 * Andy Goodrich: Added TRACEF run-time debugging macro and flag to record
 *                RCS changes.
 * 
*
*/

#ifndef xvdebug_INCLUDED
#define xvdebug_INCLUDED
/*
 *  DEBUGGING AND ERROR MACROS
 */
#ifdef XV_DIAG
static int	debug_temp;		/* Temporary for stuff. */
#   define DEBUGF( level, args ) \
    {\
        if ( level & XV_DIAG ) \
        { \
            FILE *fd;  \
            fd = stderr; \
            fprintf args; \
            fd = fopen("/tmp/xv_log", "a+");\
            if ( fd ) fprintf args ;\
            fclose(fd); \
        } \
    }
#   define ERRORF( args ) DEBUGF( 0xFFFFFFFF, args )
#else  /* !XV_DIAG */
#   define DEBUGF( level, args )
#   define ERRORF( args ) \
    { \
		FILE *fd;  \
		fd = stderr; \
		fprintf args; \
		fd = fopen("/tmp/xv_log", "a+");\
		if ( fd ) fprintf args ;\
		fclose(fd); \
    }
#endif /* XV_DIAG */

#define DEFINE_TRACEF( value ) \
	int xv_trace_flags = value

#define SET_TRACEF( value ) \
{ \
	extern int	xv_trace_flags; \
	xv_trace_flags = value; \
}

#define TRACEF( level, args ) \
{\
	extern int	xv_trace_flags; \
	if ( level & xv_trace_flags ) \
	{ \
		FILE *fd;  \
		fd = stderr; \
		fprintf args; \
		fd = fopen("/tmp/xv_log", "a+");\
		if ( fd ) fprintf args ;\
		fclose(fd); \
	} \
}

#endif  /* xvdebug_INCLUDED */
