/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 */
/*
** File:
**
**  xvimage_PGIpv.c: Hardware specific DD layer routines for the Parallax
**                   PowerVideo adaptor image support.
**
** Author:
**
**   Andy Goodrich (Parallax)
**
** Revisions:
**
**   07.03.94 Goodrich
**     - initial coding.
**
** $Log:   /plxhp/dave/pvcsrc/xv/server/ddx/PGIpv/archives/xvimage_PGIpv.c_v  $
 * 
 *    Rev 1.9   09 Sep 1996 10:44:22   dave
 * fixed hp diferences in c3lib.c
** Revision 1.9  1996/07/22 12:43:30  dave
** fixed hp diferences in c3lib.c
**
 * Revision 1.8  96/06/18  09:47:29  09:47:29  dist (Software Distribution Account)
 * added shared memory
 * 
 * Revision 1.7  1995/12/11  15:02:31  dist
 * Andy Goodrich: Checkin / merging of hedge changes.
 *
 * Revision 1.5  95/07/19  11:38:57  11:38:57  dist ()
 * Andy Goodrich: Added code for XvImage support.
 * 
 * Revision 1.4  95/06/23  15:05:10  15:05:10  dist ()
 * Andy Goodrich: Enabled code for XvPutImage and XvGetImage.
 * 
 * Revision 1.3  95/06/02  11:44:16  11:44:16  dist ()
 * Andy Goodrich: Fixed GetImage call to use drawable and offset specifications.
 * 
 * Revision 1.2  95/05/25  17:03:12  17:03:12  dist ()
 * Andy Goodrich - Combined HP/AIX version for HP 9.1 code freeze.
 * 
** Revision 1.1  95/05/12  13:45:10  13:45:10  dist ()
** Initial revision
*/
#include <sys/types.h>      /* Pick up u_int and friends. */
#include <stdio.h>          /* For diagnostics and error messages. */

#define SHARED_MEMORY

#ifdef SHARED_MEMORY
#include <sys/ipc.h>
#include <sys/shm.h>
#endif

#include "X.h"
#define NEED_REPLIES
#define NEED_EVENTS
#include "Xproto.h"
#include "Xv.h"
#include "Xvproto.h"         /* This must come before Xvplxproto.h */
#include "Xvplxproto.h"
#include "scrnintstr.h"
#include "validate.h"
#include "windowstr.h"
#include "pixmapstr.h"
#include "gcstruct.h"
#include "dixstruct.h"
#include "servermd.h"		 /* Pick up IMAGE_BUFSIZE. */
#include "xvdix.h"           /* Pick up device-independent layer. */
#include "xvddx.h"           /* Pick up device-dependent layer: XvddConfigPtr.*/
#include "xvhw_PGIpv.h"      /* Pick up hardware layer: private object defs. */
#include "Xvddx_PGI.h"       /* Pick up XV_xxx encoding declarations. */
/* #define XV_DIAG 1 */
#include "xvdebug.h"         /* Pick up debugging macros and definitions. */

#ifdef SHARED_MEMORY

extern void PlxShmGetGlobals(unsigned char *,int *, int *, RESTYPE *);
extern void PlxShmExtensionInit();
static unsigned char shmReqCode;
static int shmCompletionCode, shmBadSegCode;
RESTYPE shmSegType;

#define VERIFY_SHMSEG(shmseg,shmdesc,client) \
{ \
    shmdesc = (ShmDescPtr)LookupIDByType(shmseg, shmSegType); \
    if (!shmdesc) \
    { \
	client->errorValue = shmseg; \
	return shmBadSegCode; \
    } \
}

#define VERIFY_SHMPTR(shmseg,offset,needwrite,shmdesc,client) \
{ \
    VERIFY_SHMSEG(shmseg, shmdesc, client); \
    if ((offset & 3) || (offset > shmdesc->size)) \
    { \
	client->errorValue = offset; \
	return BadValue; \
    } \
    if (needwrite && !shmdesc->writable) \
	return BadAccess; \
}

typedef unsigned long ShmSeg;

typedef struct _ShmDesc {
    struct _ShmDesc *next;
    int shmid;
    int refcnt;
    char *addr;
    Bool writable;
    unsigned long size;
} ShmDescRec, *ShmDescPtr;

typedef struct _ShmCompletion {
    BYTE        type;           /* always eventBase + ShmCompletion */
    BYTE        bpad0;
    CARD16      sequenceNumber B16;
    Drawable    drawable B32;
    CARD16      minorEvent B16;
    BYTE        majorEvent;
    BYTE        bpad1;
    ShmSeg      shmseg B32;
    CARD32      offset B32;
    CARD32      pad0 B32;
    CARD32      pad1 B32;
    CARD32      pad2 B32;
} xShmCompletionEvent;

#else

static int shm_flag = 0; /* #### */

#endif	/* SHARED_MEMORY */

static int PGIpvGetJPEG();
static int PGIpvGetXImage();
static int PGIpvPutJPEG();
static int PGIpvPutXImage();
static int SWritePlxGetImageReply();

#define _WritePlxGetImageReply(_c,_d) \
  if ((_c)->swapped) SWritePlxGetImageReply(_c, _d); \
  else WriteToClient(_c, sz_xvPlxGetImageReply, _d)

static Atom atomXvImageJPEG;
static Atom atomXvImageXImage;
static Atom atomXvImageXImage32;

extern int  (* ProcVector[256]) ();
extern int  (* ReplySwapVector[256]) ();


#ifndef _AIX
/*----------------------------------------------------------------------------*
 |"PGIpvGetImage"
 |
 |  Get an image of a specified type from the video frame buffer. (This
 |  request is implemented via XvControl.)
 |
 |    = 0 if success
 |    = error number if failure
 *----------------------------------------------------------------------------*/
int PGIpvGetImage(client, pDraw, pPort, pGC, 
	drw_x, drw_y, drw_w, drw_h,
	image_x, image_y, image_w, image_h, image_type
)
    ClientPtr       client;     /* -> client request information. */
    DrawablePtr     pDraw;      /* -> window to get image from. */
    XvPortPtr       pPort;      /* -> device-independent port information. */
    GCPtr           pGC;        /* -> graphics context for 'pDraw'. */
	int				drw_x;		/* =  drawable x of upper left corner of get. */
	int				drw_y;		/* =  drawable y of upper left corner of get. */
	int				drw_w;		/* =  drawable width of get. */
	int				drw_h;		/* =  drawable height of get. */
	int				image_x;	/* =  x of requested image upper left corner. */
	int				image_y;	/* =  y of requested image upper left corner. */
	int				image_w;	/* =  requested image width. */
	int				image_h;	/* =  requested image height. */
	int				image_type;	/* =  type of image requested. */
{
    int                 nBuf;       /* Size of 'buf'. */
    int                 nData;      /* Length of compressed data. */
    int                 *pBuf;      /* Buffer to hold compressed data. */
    XvddPortPtr         pddPort;    /* Device-dependent port information. */
    int                 present;    /* Indicates if ccube present. */
    ShmDescPtr          shmdesc;    /* Shared memory description info. */
    xvPlxGetImageReply  rep;        /* Reply. */

	/*  PROCESS JPEG IMAGE. */

	if ( image_type == atomXvImageJPEG )
	{
		return PGIpvGetJPEG( client, pDraw, pPort, pGC,
			drw_x, drw_y, drw_w, drw_h, image_w, image_h);
	}


	/*  PROCESS STANDARD X IMAGE. */

	else if ( image_type == atomXvImageXImage )
	{
		return PGIpvGetXImage( client, pDraw, pPort, pGC,
			drw_x, drw_y, drw_w, drw_h, image_w, image_h);
	}

    return (Success);
}


/*----------------------------------------------------------------------------*
 |"PGIpvGetJPEG"
 |
 |  Get a compressed image from the video frame buffer.
 |
 |    = 0 if success
 |    = error number if failure
 *----------------------------------------------------------------------------*/
static int PGIpvGetJPEG(client, pDraw, pPort, pGC, 
	drw_x, drw_y, drw_w, drw_h, image_w, image_h
)
    ClientPtr       client;     /* -> client request information. */
    DrawablePtr     pDraw;      /* -> window to get image from. */
    XvPortPtr       pPort;      /* -> device-independent port information. */
    GCPtr           pGC;        /* -> graphics context for 'pDraw'. */
	int				drw_x;		/* =  drawable x of upper left corner of get. */
	int				drw_y;		/* =  drawable y of upper left corner of get. */
	int				drw_w;		/* =  drawable width of get. */
	int				drw_h;		/* =  drawable height of get. */
	int				image_w;	/* =  requested image width. */
	int				image_h;	/* =  requested image height. */
{
#define CCCBUFSIZE 0x35000

    static int          *c3buf1;    /* Buffer to hold compressed image in. */
    int                 nBuf;       /* Size of 'buf'. */
    int                 nData;      /* Length of compressed data. */
    int                 *pBuf;      /* Buffer to hold compressed data. */
    XvddPortPtr         pddPort;    /* Device-dependent port information. */
    int                 present;    /* Indicates if ccube present. */
    xvPlxGetImageReply  rep;        /* Reply. */


    pddPort = (XvddPortPtr)pPort->devPriv.ptr;
    Xvhw_Control(pddPort->phw, "CCUBE", &present);
    if ( !present ) 
        return BadMatch;


	if (!c3buf1) c3buf1 = (int *) xalloc(CCCBUFSIZE+4);
	if (c3buf1 == NULL) return(BadAlloc);
	pBuf = c3buf1;
	nBuf = CCCBUFSIZE;

    nData = Xvplx_Get_Compressed_Image(pddPort->phw, nBuf, pBuf,
        drw_x + pDraw->x, drw_y + pDraw->y, drw_w, drw_h, 
		0, 0, image_w, image_h);

	SendXvGetImageReply(client, image_w, image_h, nData, pBuf);

    return (Success);
}


/*----------------------------------------------------------------------------*
 |"PGIpvGetXImage"
 |
 |  Get a standard X image from the video frame buffer.
 |
 |    = 0 if success
 |    = error number if failure
 *----------------------------------------------------------------------------*/
static int PGIpvGetXImage(client, pDraw, pPort, pGC, 
	drw_x, drw_y, drw_w, drw_h, image_w, image_h
)
    ClientPtr       client;     /* -> client request information. */
    DrawablePtr     pDraw;      /* -> window to get image from. */
    XvPortPtr       pPort;      /* -> device-independent port information. */
    GCPtr           pGC;        /* -> graphics context for 'pDraw'. */
	int				drw_x;		/* =  drawable x of upper left corner of get. */
	int				drw_y;		/* =  drawable y of upper left corner of get. */
	int				drw_w;		/* =  drawable width of get. */
	int				drw_h;		/* =  drawable height of get. */
	int				image_w;	/* =  requested image width. */
	int				image_h;	/* =  requested image height. */
{
    register int	     height;
	Xvhw_Image			 image_info; /* Image geometry information. */
	long                 length;
	register int         linesDone;
	int                  linesPerBuf;
	int					 nData;		 /* # of bytes returned by image get. */
    char		         *pBuf;
    XvddPortPtr	         pddPort;	 /* Pointer to Xvrop pointer. */
    int                  iLine;      /* Index: line now getting. */
    int	 	 			 nReply;	 /* Length of reply in bytes. */
    long		         widthBytesLine;


    /*  CLIP REQUEST TO LIMITS OF SCREEN AND SET UP BUFFERS AND SIZES. */
	 
    if( (drw_x < 0) || ((drw_x+(int)drw_w) > pDraw->width) ||
        (drw_y < 0) || ((drw_y+(int)drw_h) > pDraw->height)
    )
	{
		return(BadMatch);
	}
	height = drw_h;
	widthBytesLine = ((drw_w * 32 / 8) + 3) & ~3;
    nReply = widthBytesLine * height;

    if (widthBytesLine == 0 || height == 0)
	{
		linesPerBuf = 0;
	}
    else if (widthBytesLine >= IMAGE_BUFSIZE)
	{
		linesPerBuf = 1;
	}
    else
    {
		linesPerBuf = IMAGE_BUFSIZE / widthBytesLine;
		if (linesPerBuf > height)
			linesPerBuf = height;
    }
    length = linesPerBuf * widthBytesLine;
    if (linesPerBuf < height)
    {
		/* we have to make sure intermediate buffers don't need padding */
		while ((linesPerBuf > 1) && (length & 3))
		{
			linesPerBuf--;
			length -= widthBytesLine;
		}
		while (length & 3)
		{
			linesPerBuf++;
			length += widthBytesLine;
		}
    }
    if(!(pBuf = (char *) ALLOCATE_LOCAL(length)))
        return (BadAlloc);


	/*  SEND OFF REPLY HEADER, THEN LOOP GETTING DATA FROM HARDWARE LAYER. */

	SendXvGetImageReply(client, drw_w, drw_h, nReply, 0);

    image_info.drawable_x = pDraw->x;
    image_info.drawable_y = pDraw->y;
    image_info.offset_x = drw_x;
    image_info.image_x = 0;
    image_info.image_y = 0;
    image_info.image_w = drw_w; 
    image_info.image_depth = 32;
    image_info.image_data_depth = 32;
    image_info.image_stride = drw_w;

    pddPort = (XvddPortPtr)pPort->devPriv.ptr;
    if (linesPerBuf != 0 )
    {
        linesDone = 0;
        while (height - linesDone > 0)
        {
			image_info.image_h = min(linesPerBuf, height - linesDone);
    		image_info.offset_y = drw_y+linesDone;
			Xvhw_Get_Image(pddPort->phw, &image_info, pBuf);

			/* Note that this is NOT a call to WriteSwappedDataToClient,
               as we do NOT byte swap */
			(void)WriteToClient(client, (int)(image_info.image_h * 
				widthBytesLine), pBuf);
			linesDone += image_info.image_h;
        }
    }
    DEALLOCATE_LOCAL(pBuf);
    return (Success);
}
#endif /* !_AIX #### */


/*----------------------------------------------------------------------------*
 |"PGIpvImageInit"
 |
 |  Initialize generic image support for the PGI pv.
 |
 *----------------------------------------------------------------------------*/
void
PGIpvImageInit(pAdaptor, phw)
    XvAdaptorPtr  pAdaptor;     /* -> adaptor record for this device. */
{
   atomXvImageXImage   = MakeAtom("XV_IMAGE_XIMAGE",
                                  strlen("XV_IMAGE_XIMAGE"),
                                  xTrue);
   atomXvImageXImage32 = MakeAtom("XV_IMAGE_PLX_XIMAGE32",
                                  strlen("XV_IMAGE_PLX_XIMAGE32"),
                                  xTrue);
   atomXvImageJPEG     = MakeAtom("XV_IMAGE_JPEG",
                                  strlen("XV_IMAGE_JPEG"),
                                  xTrue);
#ifdef SHARED_MEMORY
	PlxShmExtensionInit();
	PlxShmGetGlobals(&shmReqCode, &shmCompletionCode, &shmBadSegCode,
					 &shmSegType);
#endif
}

#ifndef _AIX
/*----------------------------------------------------------------------------*
 |"PGIpvPutImage"
 |
 |  Put an image into a drawable under Xv control.
 |
 |    = 1 if success
 |    = 0 if failure
 *----------------------------------------------------------------------------*/
int PGIpvPutImage(client, pDraw, pPort, pGC, 
	drw_x, drw_y, drw_w, drw_h,
	image_x, image_y, image_w, image_h, image_type, n_image, p_image
)
    ClientPtr       client;     /* -> client request information. */
    DrawablePtr     pDraw;      /* -> window to get image from. */
    XvPortPtr       pPort;      /* -> device-independent port information. */
    GCPtr           pGC;        /* -> graphics context for 'pDraw'. */
	int				drw_x;		/* =  drawable x of upper left corner of get. */
	int				drw_y;		/* =  drawable y of upper left corner of get. */
	int				drw_w;		/* =  drawable width of get. */
	int				drw_h;		/* =  drawable height of get. */
	int				image_x;	/* =  x of requested image upper left corner. */
	int				image_y;	/* =  y of requested image upper left corner. */
	int				image_w;	/* =  requested image width. */
	int				image_h;	/* =  requested image height. */
	int				image_type;	/* =  type of image requested. */
	int				n_image;	/* =  length of image information in bytes. */
	int				p_image;	/* -> image information.  */
{

	/*  PROCESS JPEG IMAGE. */

	if ( image_type == atomXvImageJPEG )
	{
		return PGIpvPutJPEG( client, pPort, pDraw, pGC, n_image, p_image,
			image_x, image_y, image_w, image_h, 
			drw_x, drw_y, drw_w, drw_h
		);

	}


	/*  PROCESS STANDARD X IMAGE. */

	else if ( image_type == atomXvImageXImage )
	{
		return PGIpvPutXImage( client, pPort, pDraw, pGC, 24, 
			n_image, p_image, image_x, image_y, image_w, image_h, 
			drw_x, drw_y, drw_w, drw_h
		);
	}


	/*  PROCESS X IMAGE WHICH INCLUDES WID DATA. */

	else if ( image_type == atomXvImageXImage32 )
	{
		return PGIpvPutXImage( client, pPort, pDraw, pGC, 32, 
			n_image, p_image, image_x, image_y, image_w, image_h, 
			drw_x, drw_y, drw_w, drw_h
		);
	}

    return (BadMatch);
}


/*----------------------------------------------------------------------------*
 |"PGIpvPutJPEG"
 |
 |  Put a JPEG image into a drawable under Xv control.
 |
 |    = 1 if success
 |    = 0 if failure
 *----------------------------------------------------------------------------*/
static int PGIpvPutJPEG(client, pPort, pDraw, pGC, nData, pData,
	src_x, src_y, src_w, src_h, dst_x, dst_y, dst_w, dst_h
	)
    ClientPtr   client;     /* -> client request information. */
    XvPortPtr   pPort;      /* -> device-independent port information. */
    DrawablePtr pDraw;      /* Window to get image from. */
    GCPtr       pGC;        /* Graphics context for 'pDraw'. */
	int			nData;		/* Length of data in bytes. */
	void*		pData;		/* Pointer to data. */
	int			src_x;		/* Frame buffer x for get. */
	int			src_y;		/* Frame buffer y for get. */
	int			src_w;		/* Frame buffer width for get. */
	int			src_h;		/* Frame buffer height for get. */
	int			dst_w;		/* Image requested width for get. */
	int			dst_h;		/* Image requested height for get. */
{
    XvddPortPtr pddPort;        /* Device-dependent port information. */
    int         present;        /* Indicates if ccube present. */
 

    pddPort = (XvddPortPtr)pPort->devPriv.ptr;
    Xvhw_Control(pddPort->phw, "CCUBE", &present);
    if ( !present ) return BadMatch;


    /*  CENTER THE OUTPUT IMAGE AND FAKE THE DESTINATION SIZE WHEN THE
     *  DESTINATION SIZE IS GREATER THAN THE SOURCE SIZE.
     */
    if (dst_w > src_w)
    {
        dst_x = src_x + (dst_w-src_w)/2;
        dst_w = src_w;
    }
    if (dst_h > src_h )
    {
        dst_y = src_y + (dst_h-src_h)/2;
        dst_h = src_h;
    }


    /*  OPTIONALLY PREPARE THE FRAME BUFFER LOCATIONS WHICH WILL RECIEVE 
     *  THE IMAGE. 
     */
    Xvhw_Off(pddPort->phw, 1);
    if ( (pDraw != pPort->pDraw) ||
		 (pPort->serialNumber != pDraw->serialNumber) )
    {
		XvddAttachPort(pDraw, pPort, pGC, XvImageToDrawable, 0, 0, 
			(src_w < dst_w) ? dst_w : src_w, 
			(src_h < dst_h) ? dst_h : src_h,
			dst_x, dst_y, dst_w, dst_h);

		pddPort->vx = 0;
		pddPort->vy = 0;;
    	pddPort->vw = (src_w < dst_w) ? dst_w : src_w, 
    	pddPort->vh = (src_h < dst_h) ? dst_h : src_h,
    	pddPort->dx = dst_x;
    	pddPort->dy = dst_y;
    	pddPort->dw = dst_w;
    	pddPort->dh = dst_h;
    }
 

    /*  OUTPUT THE IMAGE AND SEND REPLY TO OUR REQUESTOR.  */

    Xvplx_Put_Compressed_Image(pddPort->phw, nData, pData,
        src_x, src_y, src_w, src_h,
        dst_x + pDraw->x, dst_y + pDraw->y, dst_w, dst_h);

    return (Success);
}


/*----------------------------------------------------------------------------*
 |"PGIpvPutXImage"
 |
 |  Put an XImage image into a drawable under Xv control.
 |
 |    = 1 if success
 |    = 0 if failure
 *----------------------------------------------------------------------------*/
static int PGIpvPutXImage(client, pPort, pDraw, pGC, depth, nData, pData,
	src_x, src_y, src_w, src_h, dst_x, dst_y, dst_w, dst_h
)
    ClientPtr   client;     /* -> client request information. */
    XvPortPtr   pPort;      /* -> device-independent port information. */
    DrawablePtr pDraw;      /* Window to get image from. */
    GCPtr       pGC;        /* Graphics context for 'pDraw'. */
	int			depth;		/* Depth of put image, either 24 or 32. */
	int			nData;		/* Length of data in bytes. */
	void*		pData;		/* Pointer to data. */
	int			src_x;		/* Frame buffer x for get. */
	int			src_y;		/* Frame buffer y for get. */
	int			src_w;		/* Frame buffer width for get. */
	int			src_h;		/* Frame buffer height for get. */
	int			dst_w;		/* Image requested width for get. */
	int			dst_h;		/* Image requested height for get. */
{
	Xvhw_Image  image_info;		/* Information about image. */
    XvddPortPtr pddPort;        /* Device-dependent port information. */
    int         present;        /* Indicates if ccube present. */
 

    pddPort = (XvddPortPtr)pPort->devPriv.ptr;


    /*  OPTIONALLY PREPARE THE FRAME BUFFER LOCATIONS WHICH WILL RECIEVE 
     *  THE IMAGE. 
     */
    if ( depth == 24 ) Xvhw_Off(pddPort->phw, 1);
    if ( (pDraw != pPort->pDraw) ||
		 (pPort->pGC != pGC ) ||
		 (pPort->serialNumber != pDraw->serialNumber) )
    {
		DEBUGF(1, (fd, "PGIpvPutXImage: attaching at %d, %d, %d, %d\n", 0, 0, pDraw->width, pDraw->height));
		XvddAttachPort(pDraw, pPort, pGC, XvImageToDrawable, 0, 0, 
			pDraw->width, pDraw->height, 0, 0, pDraw->width, pDraw->height);

		pddPort->vx = 0;
		pddPort->vy = 0;;
    	pddPort->vw = pDraw->width;
    	pddPort->vh = pDraw->height;
    	pddPort->dx = 0;
    	pddPort->dy = 0;
    	pddPort->dw = pDraw->width;
    	pddPort->dh = pDraw->height;
    }
 

    /*  OUTPUT THE IMAGE AND SEND REPLY TO OUR REQUESTOR.  */

    image_info.drawable_x = pDraw->x;
    image_info.drawable_y = pDraw->y;
    image_info.offset_x = dst_x;
    image_info.offset_y = dst_y;
    image_info.image_x = 0;
    image_info.image_y = 0;
    image_info.image_w = dst_w;
    image_info.image_h = dst_h;
    image_info.image_depth = 32;
    image_info.image_data_depth = depth;
    image_info.image_stride = dst_h;
	Xvhw_Put_Image(pddPort->phw, &image_info, pData);

    return (Success);
}

#endif /* !_AIX #### */

/*----------------------------------------------------------------------------*
 |"PlxGetCImage"
 |
 |  Get a compressed image from the video frame buffer.
 |
 |    = 0 if success
 |    = error number if failure
 *----------------------------------------------------------------------------*/
#ifdef SHARED_MEMORY
PlxGetCImage(client, pPort, shm_flag)
    ClientPtr       client;     /* -> client request information. */
    XvPortPtr       pPort;      /* -> device-independent port information. */
    int                         shm_flag;   /* shared mem flag */
#else
int
PlxGetCImage(client, pPort)
    ClientPtr       client;     /* -> client request information. */
    XvPortPtr       pPort;      /* -> device-independent port information. */
#endif
{
#define CCCBUFSIZE 0x35000

    static int          *c3buf1;    /* Buffer to hold compressed image in. */
    int                 nBuf;       /* Size of 'buf'. */
    int                 nData;      /* Length of compressed data. */
    int                 *pBuf;      /* Buffer to hold compressed data. */
    XvddPortPtr         pddPort;    /* Device-dependent port information. */
    DrawablePtr         pDraw;      /* Window to get image from. */
    GCPtr               pGC;        /* Graphics context for 'pDraw'. */
    int                 present;    /* Indicates if ccube present. */
    ShmDescPtr          shmdesc;    /* Shared memory description info. */
    xvPlxGetImageReply  rep;        /* Reply. */

    /*
     *  SET UP POINTERS WE WILL USE:
     */
    REQUEST(xvPlxImageReq);
    VALIDATE_DRAWABLE_AND_GC(stuff->drawable, pDraw, pGC, client);
    pddPort = (XvddPortPtr)pPort->devPriv.ptr;

    Xvhw_Control(pddPort->phw, "CCUBE", &present);

    if ( !present ) 
#ifdef GEORGE_FIXED
        return BadMatch;
#else  /* !GEORGE_FIXED */
    {
        rep.type = X_Reply;
        rep.depth = pDraw->depth;
        rep.sequenceNumber = client->sequence;
        rep.visual = None;
        rep.size = 0;
        rep.length = 0;
        _WritePlxGetImageReply(client, &rep);
        return (Success);
    }
#endif /* !GEORGE_FIXED */

    /* 
     *  SET UP BUFFER TO HOLD COMPRESSED IMAGE 
     */
#ifdef SHARED_MEMORY
    if (shm_flag)
    {
        VERIFY_SHMPTR(stuff->shmseg,stuff->offset, TRUE, shmdesc, client);
        pBuf = (int *)(shmdesc->addr + stuff->offset);
        nBuf = shmdesc->size - stuff->offset;
    }
    else
#endif /* SHARED_MEMORY */
    {
        if (!c3buf1) c3buf1 = (int *) xalloc(CCCBUFSIZE+4);
        if (c3buf1 == NULL) return(BadAlloc);
        pBuf = c3buf1;
        nBuf = CCCBUFSIZE;
    }

    nData = Xvplx_Get_Compressed_Image(pddPort->phw, nBuf, pBuf,
        stuff->src_x + pDraw->x, stuff->src_y + pDraw->y, 
        stuff->src_w, stuff->src_h,
        stuff->dst_x, stuff->dst_y, stuff->dst_w, stuff->dst_h);

    /*
     *  SEND OUT THE REPLY
     */
    rep.type = X_Reply;
    rep.depth = pDraw->depth;
    rep.sequenceNumber = client->sequence;

    if (nData > 0)
    {
        rep.visual = wVisual(((WindowPtr)pDraw)); /* #### always 24? */
        rep.size = nData;
        rep.length = (nData + 3) >> 2;
    }
    else
    {
        /* ERRORF((fd, "Bad return from Xvplx_Get_Compressed_Image\n")); */
        rep.visual = None;
        rep.size = 0;
        rep.length = 0;
    }

    _WritePlxGetImageReply(client, &rep); 

    if ((nData > 0) && (shm_flag == 0))
            WriteToClient(client, nData, pBuf);

    return (Success);
}


/*----------------------------------------------------------------------------*
 |"PlxPutCImage"
 |
 |  Transfer vector routine to set new video location on screen.
 |  Calls XvddPutVideoIntegrated or XvddPutVideoSeparate to do
 |  the actual work.
 |
 |    = 1 if success
 |    = 0 if failure
 *----------------------------------------------------------------------------*/
#ifdef SHARED_MEMORY
PlxPutCImage(client, pPort, prepare_surface, shm_flag)
	ClientPtr   client;          /* -> client request information. */
	XvPortPtr   pPort;           /* -> device-independent port information. */
	int         prepare_surface; /* =  1 should initialize surface, 0 don't. */
	int			shm_flag;
#else
int
PlxPutCImage(client, pPort, prepare_surface)
    ClientPtr   client;          /* -> client request information. */
    XvPortPtr   pPort;           /* -> device-independent port information. */
    int         prepare_surface; /* =  1 should initialize surface, 0 don't. */
#endif
{
    int         nData;          /* Length of data being put. */
    XvddPortPtr pddPort;        /* Device-dependent port information. */
    DrawablePtr pDraw;          /* Drawable putting image into. */
    GCPtr       pGC;            /* Graphics context for 'pDraw'. */
    int         present;        /* Indicates if ccube present. */
#ifdef SHARED_MEMORY
	ShmDescPtr  shmdesc;        /* Shared memory escription info. */
	char        *pData;         /* pointer to buffer */
#endif

 
    /*
     *  SET UP POINTERS WE WILL USE:
     */
    REQUEST(xvPlxImageReq);

#ifdef SHARED_MEMORY
    if(shm_flag)
    {
		VERIFY_SHMPTR(stuff->shmseg, stuff->offset, FALSE, shmdesc, client);
        pData = (char *)(shmdesc->addr + stuff->offset);
        nData = stuff->planeMask;
    }
    else
    {
		nData = (stuff->length << 2) - sz_xvPlxImageReq;
		pData = (char *)(stuff+1);
    }

#endif
    VALIDATE_DRAWABLE_AND_GC(stuff->drawable, pDraw, pGC, client);
    pddPort = (XvddPortPtr)pPort->devPriv.ptr;
 
    Xvhw_Control(pddPort->phw, "CCUBE", &present);
#ifdef GEORGE_FIXED
    if ( !present ) return BadMatch;
#else
    if ( !present ) return (Success);
#endif


    /*  OPTIONALLY PREPARE THE FRAME BUFFER LOCATIONS WHICH WILL RECIEVE 
     *  THE IMAGE. 
     */
    Xvhw_Off(pddPort->phw, 1);
    if ( prepare_surface 							  || 
         (pDraw != pPort->pDraw) 					  ||
         (pGC != pPort->pGC )                         ||
         (pPort->serialNumber != pDraw->serialNumber) ||
         (pPort->serialNumber != pGC->serialNumber)   ||
		 (stuff->dst_x != pddPort->dx)                ||
		 (stuff->dst_y != pddPort->dy)                ||
		 (stuff->dst_w != pddPort->dw)                ||
		 (stuff->dst_h != pddPort->dh) 
	)
    {
		pddPort->vx = 0;
		pddPort->vy = 0;;
    	pddPort->vw = (stuff->src_w<stuff->dst_w) ? stuff->dst_w : stuff->src_w;
    	pddPort->vh = (stuff->src_h<stuff->dst_h) ? stuff->dst_h : stuff->src_h;
    	pddPort->dx = stuff->dst_x;
    	pddPort->dy = stuff->dst_y;
    	pddPort->dw = stuff->dst_w;
    	pddPort->dh = stuff->dst_h;

		XvddAttachPort(pDraw, pPort, pGC, XvImageToDrawable,
			pddPort->vx, pddPort->vy, pddPort->vw, pddPort->vh,
			pddPort->dx, pddPort->dy, pddPort->dw, pddPort->dh
		);
    }
 

    /*  OUTPUT THE IMAGE AND SEND REPLY TO OUR REQUESTOR.  */

#ifdef SHARED_MEMORY
    Xvplx_Put_Compressed_Image(pddPort->phw, nData, pData,
#else
    nData = (stuff->length << 2) - sz_xvPlxImageReq;
    Xvplx_Put_Compressed_Image(pddPort->phw, nData, stuff+1,
#endif
    stuff->src_x, stuff->src_y, stuff->src_w, stuff->src_h,
    stuff->dst_x + pDraw->x, stuff->dst_y + pDraw->y, 
    stuff->dst_w, stuff->dst_h);

#ifdef SHARED_MEMORY
    if (shm_flag && stuff->sendEvent)
    {
        xShmCompletionEvent ev;
        ev.type =  shmCompletionCode; /* ShmCompletionCode; */
        ev.drawable = stuff->drawable;
        ev.sequenceNumber = client->sequence;
        ev.minorEvent = 0;
        ev.majorEvent = shmReqCode; /* ShmReqCode */;
        ev.shmseg = stuff->shmseg;
        ev.offset = stuff->offset;
        WriteEventsToClient(client, 1, (xEvent *) &ev);
    }
#endif
    return (Success);
}

static int
SWritePlxGetImageReply(client, rep)
    register ClientPtr client;
    xvPlxGetImageReply *rep;

{
  register char n;

  swaps(&rep->sequenceNumber, n);
  swapl(&rep->depth, n);
  swapl(&rep->length, n);
  swapl(&rep->visual, n);

  (void)WriteToClient(client, sz_xvPlxGetImageReply, (char *)rep);

}
