/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *
 *                        All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software and its 
 * documentation for any purpose and without fee is hereby granted, 
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in 
 * supporting documentation, and that the name of Parallax not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  
 *
 * The information in this software is subject to change without notice.
 * No committment is made as to the usability or reliability of this
 * software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 */
/*
** File:
**
**  xvhw_Dummy.c: Implementation of hardware level xv extension support for
**                Dummy adaptor.
**
** Author:
**
**   Andy Goodrich (Parallax)
**
** Revisions:
**
**   25.01.94 Goodrich
**     - Initial coding.
*/
#include <stdio.h>                  /* Standard io defs, e.g., stderr. */
#include <sys/types.h>              /* General type defs. */
#include <sys/param.h>              /* General system parameters and limits. */
#include <sys/time.h>               /* General timer parameters. */

#include <sys/buf.h>                /* Input / Output buffer defs. */
#include <sys/errno.h>              /* Kernel error defs. */
#include <sys/fcntl.h>              /* O_RDWR and friends. */
#include <sys/mman.h>               /* Virtual memory related macros. */

#include "Xvddx.h"					/* Pick up XVS_Sync and friends. */
#include "xvhw_Dummy.h"             /* Definition for Pip Object. */
#define MSG 0x08
#define XV_DIAG (MSG)
#include "xvdebug.h"                /* Pick up debugging macros. */


/*
 *  FORWARD REFERENCES:
 */
static int  dummy_control();               /* Perform control function. */
static void dummy_dump();                  /* Dump pip object contents. */
static int  dummy_fill_true_color();       /* Fill true color memory. */
static int  dummy_fill_video_enable();     /* Fill video enable memory. */
static void dummy_get_levels();            /* Get the picture levels. */
static void dummy_get_drawable_area();     /* Get the pip output area. */
static int  dummy_get_image();             /* Get image data. */
static void dummy_get_signal_area();       /* Get the pip signal area. */
static void dummy_get_signal_type();       /* Get the pip signal(e.g., rgb). */
static int  dummy_get_true_color();        /* Get true color data. */
static int  dummy_init();                  /* Initialize pip. */
static int  dummy_off();                   /* Turn off pip. */
static int  dummy_on();                    /* Turn on pip. */
static int  dummy_one_shot();              /* Turn on pip for frame. */
static int  dummy_put_image();             /* Put image data. */
static int  dummy_resume();                /* Pop pip "off" stack. */
static void dummy_set_levels();            /* Set the picture levels. */
static void dummy_set_drawable_area();     /* Set pip output origin,size&rect*/
static void dummy_set_signal_area();       /* Set the pip signal area. */
static void dummy_set_signal_type();       /* Set the pip signal(e.g., rgb). */
static int  dummy_status();                /* Return pip operational status. */
static int  dummy_suspend();               /* Push pip "off" stack. */
static void dummy_terminate();             /* Terminate use of a pip object. */
HwDummy*    xv_dummy_open();               /* Open a device. */

/*----------------------------------------------------------------------------*
 |"dummy_control"
 | 
 |  Perform a control function on a pip object. Note the string tests in this
 |  routine are in order of expected occurence for optimization, not in 
 |  alphabetical order.
 |
 |      = 1 if success
 |      = 0 if failure
 *----------------------------------------------------------------------------*/
static dummy_control(p_me, command, arg)
    HwDummy  *p_me;      /* -> control block for this xv instance. */
    char        *command;   /* -> command to perform. */
    caddr_t     arg;        /* =  optional argument. */
{
    Xvhw_Drw_Area*   area;          /* Current output area. */
    short*           halfword;      /* Alias to point to halfwords. */
    int              input_status;  /* Input status register value. */
    int              rc;            /* Return code from calls to routines. */
    int              reg_value;     /* Value of hardware register. */
    struct timeval   to;            /* Time stamp for performing wait. */

	DEBUGF(MSG, (fd, "dummy_control(request=%08x, arg=%08x (%d))\n",
		command, arg, arg));

    return 1;
}

/*----------------------------------------------------------------------------*
 |"dummy_dump"
 | 
 |  Dump the contents of a pip object to a file
 |
 *----------------------------------------------------------------------------*/
static void dummy_dump(p_me, fd)
    HwDummy  *p_me;      /* -> control block for this xv instance. */
    FILE        *fd;        /* -> file descriptor to dump to. */
{

    if ( fd == 0 ) fd = stderr;

    fprintf(fd, "Pip Object at address 0x%08x:\n", p_me);

    fprintf(fd, "  Flags:");
    if ( p_me->public.attr.debug )           fprintf(fd, " DEBUG");
    if ( p_me->public.attr.stop_for_update ) fprintf(fd, " STOP_FOR_UPDATE");
    if ( p_me->public.attr.separate_fb )     fprintf(fd, " SEPARATE_FB");
    fprintf(fd, "\n");

    fprintf(fd, "Transfer vector:\n");
    fprintf(fd, "    Routine:             Expected:  Actual:\n");
    fprintf(fd, "    -------              --------   ------ \n");
    fprintf(fd, "    control              0x%08x 0x%08x\n", 
        dummy_control, p_me->public.control);
    fprintf(fd, "    dump                 0x%08x 0x%08x\n", 
        dummy_dump, p_me->public.dump);
    fprintf(fd, "    fill_true_color      0x%08x 0x%08x\n", 
        dummy_fill_true_color, p_me->public.fill_true_color);
    fprintf(fd, "    fill_video_enable    0x%08x 0x%08x\n",
        dummy_fill_video_enable, p_me->public.fill_video_enable);
    fprintf(fd, "    get_levels           0x%08x 0x%08x\n", 
        dummy_get_levels, p_me->public.get_levels);
    fprintf(fd, "    get_drawable_area 0x%08x 0x%08x\n", 
        dummy_get_drawable_area, p_me->public.get_drawable_area);
    fprintf(fd, "    get_image            0x%08x 0x%08x\n", 
        dummy_get_image, p_me->public.get_image);
    fprintf(fd, "    get_signal_area      0x%08x 0x%08x\n", 
        dummy_get_signal_area, p_me->public.get_signal_area);
    fprintf(fd, "    get_signal_type      0x%08x 0x%08x\n", 
        dummy_get_signal_type, p_me->public.get_signal_type);
    fprintf(fd, "    initialize           0x%08x 0x%08x\n", 
        dummy_init, p_me->public.initialize);
    fprintf(fd, "    off                  0x%08x 0x%08x\n", 
        dummy_off, p_me->public.off);
    fprintf(fd, "    on                   0x%08x 0x%08x\n", 
        dummy_on, p_me->public.on);
    fprintf(fd, "    one_shot             0x%08x 0x%08x\n", 
        dummy_one_shot, p_me->public.one_shot);
    fprintf(fd, "    put_image            0x%08x 0x%08x\n", 
        dummy_put_image, p_me->public.put_image);
    fprintf(fd, "    resume               0x%08x 0x%08x\n", 
        dummy_resume, p_me->public.resume);
    fprintf(fd, "    set_levels           0x%08x 0x%08x\n", 
        dummy_set_levels, p_me->public.set_levels);
    fprintf(fd, "    set_drawable_area 0x%08x 0x%08x\n", 
        dummy_set_drawable_area, p_me->public.set_drawable_area);
    fprintf(fd, "    set_signal_area      0x%08x 0x%08x\n", 
        dummy_set_signal_area, p_me->public.set_signal_area);
    fprintf(fd, "    set_signal_type      0x%08x 0x%08x\n", 
        dummy_set_signal_type, p_me->public.set_signal_type);
    fprintf(fd, "    status               0x%08x 0x%08x\n", 
        dummy_status, p_me->public.status);
    fprintf(fd, "    suspend              0x%08x 0x%08x\n", 
        dummy_suspend, p_me->public.suspend);
    fprintf(fd, "    terminate            0x%08x 0x%08x\n", 
        dummy_terminate, p_me->public.terminate);

}

/*----------------------------------------------------------------------------*
 |"dummy_fill_true_color"
 | 
 |  Fill a rectangular region in the true color memory.
 *----------------------------------------------------------------------------*/
static dummy_fill_true_color(p_me, left, top, width, height, color)
    HwDummy  *p_me;      /* -> control block for this xv instance. */
    int         left;       /* =  left side of region to fill in. */
    int         top;        /* =  top side of region to fill in. */
    int         width;      /* =  width of region to fill in. */
    int         height;     /* =  height of region to fill in. */
    u_int       color;      /* =  color to fill pixels with. */
{
	DEBUGF(MSG, 
		(fd, "dummy_fill_true_color(x=%d, y=%d, w=%d, h=%d, color=%08x\n)",
		left, top, width, height, color));
    return 1;
}

/*----------------------------------------------------------------------------*
 |"dummy_fill_video_enable"
 | 
 |  Fill a rectangular region in the video enable memory. Note we must fill
 |  both the tag memory and true color memory if enable is zero since this 
 |  hardware always sums the output of its frame buffer with the host's 
 |  frame buffer. If enable is one we fill only the tag memory since there
 |  may already be active video where we are filling.
 |
 |  Note that we have conveniently chosen the values of p_me->wid to be
 |  the tag value to use.
 |
 |      = 1 if success
 |      = 0 if rectangle is not within boundaries of video enable memory
 *----------------------------------------------------------------------------*/
static dummy_fill_video_enable(p_me, left, top, width, height, enable)
    HwDummy  *p_me;      /* -> control block for this xv instance. */
    int         left;       /* =  left side of region to fill in. */
    int         top;        /* =  top side of region to fill in. */
    int         width;      /* =  width of region to fill in. */
    int         height;     /* =  height of region to fill in. */
    int         enable;     /* =  enable if non-zero, disable if zero. */  
{
	DEBUGF(MSG, 
		(fd, "dummy_fill_video_enable(x=%d, y=%d, w=%d, h=%d, %s)\n",
		left, top, width, height, (char*)(enable ? "ENABLE" : "DISABLE")));
    return 1;
}

/*----------------------------------------------------------------------------*
 |"dummy_get_drawable_area"
 |
 |  Return the current output area information.
 *----------------------------------------------------------------------------*/
static void dummy_get_drawable_area( p_me, p_area )
    HwDummy      *p_me;   /* -> control block for this xv instance. */
    Xvhw_Drw_Area   *p_area; /* -> where to put current output area. */
{
    Xvhw_Drw_Area*  p_drw;  /* Fast access to p_me->drawable_area. */
    int             zoom;   /* Zoom factor in use. */

    p_drw = &p_me->drawable_area;
    *p_area = p_me->drawable_area;
	DEBUGF(MSG, (fd, "dummy_get_drawable_area: x=%d, y=%d, w=%d, h=%d\n",
		p_area->left, p_area->top, p_area->width, p_area->height));
}
    
/*----------------------------------------------------------------------------*
 |"dummy_get_image"
 |
 |  Return the data from the true color buffer. Note that we must keep track
 |  of the portions of the requested area which are off screen. For those
 |  pixels we return a value of 0x00ff0000. There are 3 strips of data in 
 |  each direction. The ones for the x direction are:
 |
 |    (1) left of screen:  x_bgn_left  ---> x_end_left  = 0x00ff0000
 |    (2) on screen:       x_bgn_on    ---> x_end_on    = pixel value
 |    (3) right of screen: x_bgn_right ---> x_end_right = 0x00ff0000
 |
 |  The strips in the y direction are:
 |
 |    (1) above screen: y_bgn_above ---> y_end_above = 0x00ff0000
 |    (2) on screen:    y_bgn_on    ---> y_end_on    = value wrt x strips
 |    (3) below screen: y_bgn_below ---> y_end_below = 0x00ff0000
 |
 |    = size of image read
 *----------------------------------------------------------------------------*/
static int dummy_get_image( p_me, x, y, w, h, buf )
    HwDummy      *p_me;   /* -> control block for this xv instance. */
    int             x;       /* =  x coordinate of data to be returned. */
    int             y;       /* =  y coordinate of data to be returned. */
    int             w;       /* =  width of data to be returned. */
    int             h;       /* =  height of data to be returned. */
    u_int           *buf;    /* -> buffer to place data into. */
{
	DEBUGF(MSG, (fd, "dummy_get_image(x=%d, y=%d, w=%d, h=%d)\n", x, y, w, h));
	return 4 * w * h;
}
    
/*----------------------------------------------------------------------------*
 |"dummy_get_levels"
 |
 |  Return the picture levels (brightness, contrast, hue, saturation).
 *----------------------------------------------------------------------------*/
static void dummy_get_levels( p_me, levels )
    HwDummy  *p_me;   /* -> control block for this xv instance. */
    Xvhw_Levels *levels; /* -> where to put current levels. */
{
	DEBUGF(MSG, (fd, 
		"dummy_get_levels: brightness=%d, contrast=%d, hue=%d, saturation=%d\n",
		levels->brightness, levels->contrast, levels->hue, levels->saturation));
	*levels = p_me->levels;
}
    
/*----------------------------------------------------------------------------*
 |"dummy_get_signal_area"
 |
 |  Return the current output area information.
 |
 |  (1) We want to return a value which reflects the fact that 0,0 should 
 |      be the origin of the normal video signal for the timing regimen 
 |      being used. So we need to adjust the left and top values by the
 |      offsets specified in the tv private data area.
 |  (2) The area specified by source_start and source_end values includes
 |      those end points, so always add 1 to the width and 2 to the height
 |      before returning them.
 *----------------------------------------------------------------------------*/
static void dummy_get_signal_area( p_me, p_signal )
    HwDummy      *p_me;      /* -> control block for this xv instance. */
    Xvhw_Sig_Area   *p_signal;  /* -> where to put signal area info. */
{
    *p_signal = p_me->signal_area;
	DEBUGF(MSG, (fd, "dummy_get_signal_area: x=%d, y=%d, w=%d, h=%d\n",
		p_signal->left, p_signal->top, p_signal->width, p_signal->height));
}

/*----------------------------------------------------------------------------*
 |"dummy_get_signal_type"
 |
 |  Get the type and timing of signal being used. Update the information
 |  in the object's signal fields from the hardware, and then return that
 |  information to the caller. Note, the timing cannot be determined from
 |  reading the hardware, so use the information already in the record.
 *----------------------------------------------------------------------------*/
static void dummy_get_signal_type(p_me, p_signal)
    HwDummy  *p_me;      /* -> pip object. */
    Xvhw_Signal *p_signal;  /* -> signal specification. */
{
    *p_signal = p_me->signal;
	DEBUGF(MSG, (fd, "dummy_get_signal_type: type=%08x, timing=%08x\n",
		p_signal->type, p_signal->timing));
}
    
/*----------------------------------------------------------------------------*
 |"dummy_init"
 |
 |  Initialize pip hardware to an operable state. After this
 |  call the pip will be off. Enable it with a call to dummy_on.
 *----------------------------------------------------------------------------*/
static int dummy_init(p_me, signal)
    HwDummy  *p_me;      /* -> pip object. */
    Xvhw_Signal *signal;    /* -> signal type information. */
{
	DEBUGF(MSG, (fd, "dummy_init\n"));
    return 1;
}

/*----------------------------------------------------------------------------*
 |"dummy_off"
 |
 |  Turn off video digitization but not mixing. Set the video_frozen flag so 
 |  that operations on the  resume/suspend semaphore will not cause video to 
 |  come on. (This requires an explicit call to dummy_on.) Return its previous 
 |  operational state. Note we use a one shot to turn off the pip, to leave 
 |  a good image.
 |
 |      = 1 if pip was operating at time of call
 |      = 0 if pip was halted at time of call
 *----------------------------------------------------------------------------*/
static int dummy_off(p_me, wait_for_inactive)
    HwDummy  *p_me;              /* -> device instance dependent data. */
    int         wait_for_inactive;  /* =  1 if wait for pip to turn off */
{
    int         prior_state;    /* Previous state of pip operations. */

    if ( p_me->video_frozen ) 
	{
		prior_state = 0;
	}
	else
	{
		prior_state = (p_me->video_off_semaphore <= 0) ? 1 : 0;
	}
    p_me->video_frozen = 1;

	DEBUGF(MSG, (fd, "dummy_off: video OFF, prior_state = %d\n", prior_state));

    return prior_state;
}


/*----------------------------------------------------------------------------*
 |"dummy_on"
 |
 |  Turn on video digitization and mixing. Set the video_frozen flag to zero so
 |  that suspend/resumes will work. Initialize suspend/resume semaphore to its
 |  neutral state. Return the previous operational state.
 |  Note that we set the video cross over point in this code since other
 |  code will change it out from under us.
 |
 |    = 1 if wait_for_active is zero, or pip is now running
 |    = 0 if pip did not turn on (hardware error)
 *----------------------------------------------------------------------------*/static int dummy_on(p_me, wait_for_active)
    HwDummy  *p_me;           /* -> device instance dependent data. */
    int         wait_for_active; /* = 1 if should wait for pip to go active. */
{
    p_me->video_frozen = 0;
    p_me->video_off_semaphore = 0;

	DEBUGF(MSG, (fd, "dummy_on: video ON\n"));

    return 1;
}


/*----------------------------------------------------------------------------*
 |"dummy_one_shot"
 |
 |  Turn pip on for a frame and restart mixing. Upon return the pip will
 |  be off, with the "off" semaphore incremented appropriately.
 |    = 1 if success
 |    = 0 if error detected
 *----------------------------------------------------------------------------*/
static int dummy_one_shot(p_me, wait)
    HwDummy  *p_me;              /* -> device instance dependent data. */
    int         wait;               /* =  1 if should wait till video is off. */
{
    p_me->video_off_semaphore++;

	DEBUGF(MSG, 
		(fd, "dummy_one_shot: semaphore %d\n", p_me->video_off_semaphore));
    return 1;
}


/*----------------------------------------------------------------------------*
 |"dummy_put_image"
 |
 |  Place data into the true color buffer. This routine assumes caller is
 |  handling suspension and resumption of video display.
 *----------------------------------------------------------------------------*/
static int dummy_put_image( p_me, depth, x, y, w, h, buf )
    HwDummy      *p_me;   /* -> control block for this xv instance. */
    int             depth;   /* =  depth of image. */
    int             x;       /* =  x coordinate of data to be written. */
    int             y;       /* =  y coordinate of data to be written. */
    int             w;       /* =  width of data to be written. */
    int             h;       /* =  height of data to be written. */
    u_int           *buf;    /* -> buffer to get data from. */
{
	DEBUGF(MSG, 
		(fd, "dummy_put_image(depth=%d, x=%d, y=%d, w=%d, h=%d buf=%08x)\n",
		depth, x, y, w, h, buf));

	return 1;
}

/*---------------------------------------------------------------------------*
 |"dummy_resume"
 |  Pop the pip "off" semaphore, and if it goes to zero and video_frozen
 |  is not set, start up the pip and start video mixing again. Sending 
 |  a pop_stack parameter of 1 will start video and clear the semaphore.
 |
 |      = 1 if pip now operating
 |      = 0 if pip still halted
 *---------------------------------------------------------------------------*/
static int dummy_resume(p_me, pop_stack)
    HwDummy  *p_me;      /* -> control block for this xv instance. */
    int         pop_stack;  /* =  1 if should pop semaphore stack to active. */
{
    if ( p_me->video_frozen ) 
	{
	    DEBUGF(MSG, (fd, "dummy_resume: video is frozen, call ignored\n"));
		return 0;
	}

    p_me->video_off_semaphore = (pop_stack) ? 0 : --p_me->video_off_semaphore;
	DEBUGF(MSG, (fd, "dummy_resume: semaphore now %d\n", 
		p_me->video_off_semaphore));
    if ( p_me->video_off_semaphore <= 0 )
    {
        p_me->video_off_semaphore = 0;
        (*p_me->public.on)(p_me, 1);
    }
		
    return (p_me->video_off_semaphore) ? 0 : 1;
}

/*----------------------------------------------------------------------------*
 |"dummy_set_drawable_area"
 |
 |  Set the size of the rectangle for the pip output area. 
 *----------------------------------------------------------------------------*/
static void dummy_set_drawable_area(p_me, p_area)
    HwDummy      *p_me;      /* -> control block for this xv instance. */
    Xvhw_Drw_Area   *p_area;    /* -> output area specification. */
{
    p_me->drawable_area = *p_area;

	DEBUGF(MSG, (fd, "dummy_set_drawable_area(x=%d, y=%d, w=%d, h=%d)\n",
		p_area->left, p_area->top, p_area->width, p_area->height));
}

/*----------------------------------------------------------------------------*
 |"dummy_set_levels"
 |
 |  Set the picture levels (brightness, contrast, hue, saturation).
 *----------------------------------------------------------------------------*/
static void dummy_set_levels( p_me, levels )
    HwDummy  *p_me;   /* -> control block for this xv instance. */
    Xvhw_Levels *levels; /* -> where to get new levels. */
{

	DEBUGF(MSG, 
		(fd, "dummy_set_levels: bright=%d contrast=%d hue=%d saturation=%d\n",
		levels->brightness, levels->contrast, levels->hue, levels->saturation));
    p_me->levels = *levels;
}
    
/*----------------------------------------------------------------------------*
 |"dummy_set_signal_area"
 |
 |  Set the size of the rectangle for the signal input area.
 |
 *----------------------------------------------------------------------------*/
static void dummy_set_signal_area(p_me, p_signal)
    HwDummy      *p_me;      /* -> control block for this xv instance. */
    Xvhw_Sig_Area   *p_signal;  /* -> new signal input area. */
{
	DEBUGF(MSG, (fd, "dummy_set_signal_area: x=%d y=%d w=%d h=%d\n",
		p_signal->left, p_signal->top, p_signal->width, p_signal->height));

    p_me->signal_area = *p_signal;
}

/*----------------------------------------------------------------------------*
 |"dummy_set_signal_type"
 |
 |  Set the type and timing of signal being used. Only reprogram if there is 
 |  a change. A value of -1 in any of the fields of "signal" indicates the 
 |  current setting should be used. Note we clear the video memory so the 
 |  user can tell if there is an active video signal or not.
 *----------------------------------------------------------------------------*/
static void dummy_set_signal_type(p_me, signal)
    HwDummy  *p_me;      /* -> pip object. */
    Xvhw_Signal *signal;    /* -> signal specification. */
{
    int     (*config)();            /* Configuration routine to use. */
	int		instance;				/* Input instance to use. */
    int     plxv_input_config();    /* Input configuration routine. */
    int     plxv_output_config();   /* Output configuration routine. */
	char*	p_timing;				/* Timing value to use. */
	char*	p_type;					/* Type of encoding to use. */

	DEBUGF(MSG, (fd, "dummy_set_signal_type(type=%08x, timing=%08x)\n",
		signal->type, signal->timing));
    p_me->signal = *signal;
}

/*----------------------------------------------------------------------------*
 |"dummy_status"
 |
 |  Return current status of pip operation.
 |
 |    = status
 *----------------------------------------------------------------------------*/
static int dummy_status(p_me)
    HwDummy*  p_me;      /* -> pip object. */
{    
    int	status;		/* Status value to return. */

	status = XVS_Input | XVS_Sync;;

	DEBUGF(MSG, (fd, "dummy_status: returning %08x\n", status));
	return status;
}
    
/*----------------------------------------------------------------------------*
 |"dummy_suspend"
 |
 |  Push the pip "off" semaphore by one. If it is running stop it and
 |  optionally wait for the pip to actually stop running.
 |  Note we do a one shot to turn off the pip so we get a good image. 
 |
 |      = 1 if pip was operating at time of call
 |      = 0 if pip was halted at time of call
 *----------------------------------------------------------------------------*/
static int dummy_suspend(p_me, wait)
    HwDummy  *p_me;      /* -> control block for this xv instance. */
    int         wait;       /* =  1 if should wait for pip to stop. */
{
    int             prior_state;    /* State of pip prior to this call. */

    if ( p_me->video_frozen ) 
    {
        prior_state = 0;
    }
    else
    {
        prior_state = (p_me->video_off_semaphore <= 0) ? 1 : 0;
        if ( prior_state )
        {
            (*p_me->public.one_shot)(p_me, wait);
            p_me->video_off_semaphore = 1;
        }
        else
        {
            p_me->video_off_semaphore++;
        }
    }

	DEBUGF(MSG, (fd, "dummy_suspend: frozen: %d semaphore: %d prior: %d\n",
		p_me->video_frozen, p_me->video_off_semaphore, prior_state));
    return prior_state;
}

/*----------------------------------------------------------------------------*
 |"dummy_terminate" 
 | 
 |  Terminate operations of Pip object: turn off video mixing
 |  and free dynamically allocated storage. If this is port 0 un-map the
 |  card by closing the device's file descriptor.
 *----------------------------------------------------------------------------*/
static void dummy_terminate(p_me) 
    HwDummy  *p_me;  /* -> object to terminate. */ 
{    
	DEBUGF(MSG, (fd, "dummy_terminate: object termination\n"));
    free(p_me);
}    
 
/*----------------------------------------------------------------------------*
 |"xv_dummy_map"
 |
 |  Given a device name map in a dummy card.
 |
 |      -> device mapping info on success
 |      =  0 if failure
 *----------------------------------------------------------------------------*/
void* xv_dummy_map(device)
    char *device;   /* -> base name of devices to open. */
{
    int      image_fd; /* Mapping of memory areas of card. */
    void*    p_map;    /* Map region eventually passed to xv_dummy_open*/


    if ( !(p_map = (void*)malloc(sizeof(20))) )
    {
        ERRORF((fd, "Unable to allocate storage device %s\n", device)); 
        return 0;
    }

	DEBUGF(MSG, (fd, "xv_dummy_map: mapping dummy device\n"));
    return p_map;
}


/*----------------------------------------------------------------------------*
 |"xv_dummy_open"
 |
 |  Given a file descriptor, open a pip connected to a tv card as a X video 
 |  extension device. 
 |
 |      -> X video extension control block
 |      =  0 if failure
 *----------------------------------------------------------------------------*/
HwDummy* xv_dummy_open( i_port, p_map )
    int      i_port; /* =  port being opened. */
    void*    p_map;  /* -> mmap object or zero. */
{
    int                pgi;          /* Index: next plane group to look at. */
    int                pip_size;     /* Size of memory map or zero. */
    int                pip_offset;   /* Offset for memory mapping. */
    HwDummy*           p_me;         /* X video extension instance cb. */
    Xvhw_Signal        signal;       /* Video signal type and timing. */

    /*
     *  Allocate pip object and fill in its transfer vector.
     */
    if ( !(p_me = (HwDummy  *)malloc(sizeof(HwDummy))) )
    {
        return 0;
    }
	DEBUGF(MSG, (fd, "xv_dummy_open: hardware control block at %08x\n", p_me));

    p_me->public.attr.debug = 0;
    p_me->public.attr.stop_for_update = 0;
    p_me->public.attr.separate_fb = 1;
    p_me->public.adaptor_port_number = i_port; 

    p_me->public.control = dummy_control;
    p_me->public.dump = dummy_dump;
    p_me->public.fill_true_color = dummy_fill_true_color;
    p_me->public.fill_video_enable = dummy_fill_video_enable;
    p_me->public.get_levels = dummy_get_levels;
    p_me->public.get_drawable_area = dummy_get_drawable_area;
    p_me->public.get_image = dummy_get_image;
    p_me->public.get_signal_area = dummy_get_signal_area;
    p_me->public.get_signal_type = dummy_get_signal_type;
    p_me->public.initialize = dummy_init;
    p_me->public.off = dummy_off;
    p_me->public.on = dummy_on;
    p_me->public.one_shot = dummy_one_shot;
    p_me->public.put_image = dummy_put_image;
    p_me->public.resume = dummy_resume;
    p_me->public.set_levels = dummy_set_levels;
    p_me->public.set_drawable_area = dummy_set_drawable_area;
    p_me->public.set_signal_area = dummy_set_signal_area;
    p_me->public.set_signal_type = dummy_set_signal_type;
    p_me->public.status = dummy_status;
    p_me->public.suspend = dummy_suspend;
    p_me->public.terminate = dummy_terminate;

    p_me->public.video_pitch =  DUMMY_FB_WIDTH;
    p_me->public.video_height = DUMMY_FB_HEIGHT;

    p_me->drawable_area.fields_inverted = 0;
    p_me->drawable_area.horizontal_flip = 0;
    p_me->drawable_area.vertical_flip = 0;
    p_me->fb_pitch =  DUMMY_FB_WIDTH;
    p_me->fb_height = DUMMY_FB_HEIGHT;
    p_me->video_frozen = 0;
    p_me->video_off_semaphore = 0;

    /*  SET THE SOURCE TIMING TO THE DEFAULT VALUES */

    signal.timing = XV_NTSC;
    signal.type = XV_COMPOSITE;
    signal.gray_scale = 0;
    signal.vtr_mode = 0;
    dummy_init(p_me, &signal);

    return p_me;
}
