/****************************************************************************
 *
 *
 *   XV extension broker for dynamic loading of XVideo Extension
 *
 * (c) Copyright Hewlett-Packard Company, 1994.
 * All rights are reserved.  Copying or other
 * reproduction of this program except for archival
 * purposes is prohibited without the prior
 * written consent of Hewlett-Packard Company.
 *
 * RESTRICTED RIGHTS LEGEND
 *
 * Use, duplication, or disclosure by the U.S. Government
 * is subject to restrictions as set forth in
 * subdivision (c) (1) (ii) of the Rights in Technical
 * Data and Computer Software clause at
 * 252.227-7013.
 *
 * HEWLETT-PACKARD COMPANY
 * Corvallis, Oregon
 *
 *  File: xv_broker.c
 *
 *  Created: 07/18/95        --George Sachs
 *
 *  $Header:   /export/home/dist1/source/xv/server/dix/archives/xv_broker.c_v   1.2   17 Sep 1996 09:47:26   dave  $
 *
 ***************************************************************************/
static char xv_broker_what_string[]=
"@(#)HP_XV Extension Broker    $Revision:   1.2  $   $Date:   17 Sep 1996 09:47:26  $";

#define NEED_EVENTS
#include <sys/framebuf.h>
#include "broker.h"
#include "hpddxScreen.h"
#include "Xv.h"

/******************************************************************************
 * Extension Broker Procedure:         XvGetExtensionBrokerInterface()
 *
 *      This routine is invoked to determine which broker interface version
 *      to use as the basis for all further broker communications.
 *
 *      The calling routine will suggest an initial broker interface version
 *      for communication.  This routine, *GetExtensionBrokerInterface()
 *      needs to respond with a matching or supported earlier broker interface
 *      version for further consideration as a dynamically loaded component.
 *
void XvGetExtensionBrokerInterface(HpddxVersionRec *pVersion)
 */
void XvGetExtensionBrokerInterface(pVersion)
HpddxVersionRec *pVersion;
{
#ifdef ANSCII
    HpddxVersionRec version = {BROKER_INTERFACE_MAJOR_NUMBER, BROKER_INTERFACE_MINOR_NUMBER};
#endif
    HpddxVersionRec version;
    version.majorNumber= BROKER_INTERFACE_MAJOR_NUMBER;
    version.minorNumber= BROKER_INTERFACE_MINOR_NUMBER;
    *pVersion = version;
}

/******************************************************************************
 * XV Extension Broker Procedure:         XvGetExtensionInformation()
 *
 *      This routine is invoked to define contending extension.
 *
 *      Extension information provided is used by the DDX screen bidders
 *      to determine their ability to support this extension.
void XvGetExtensionInformation(DlExtensionInfoPtr pExtensionInfoRec)
 */
void XvGetExtensionInformation(pExtensionInfoRec)
DlExtensionInfoPtr pExtensionInfoRec;
{
    pExtensionInfoRec->name         = XvName;
    pExtensionInfoRec->vendor       = "Hewlett-Packard";
    pExtensionInfoRec->version.majorNumber = 1;
    pExtensionInfoRec->version.minorNumber = 0;
    pExtensionInfoRec->inputDepend  = DL_EXTN_DDX_DEPEND_NONE;
    pExtensionInfoRec->screenDepend = DL_EXTN_DDX_DEPEND_NONE;
}

/******************************************************************************
 * XV Extension Broker Procedure:         XvGetExtensionModuleList()
 *
 *      This routine is invoked to define extension support modules.
 *
 *      Module information provided is used load this extension.
 *      The name of the extension initialization routine is required from
 *      all extensions.  The name of the extension screen registration
 *      routine is only needed from extensions requiring DDX support.
 *
 *      The module list must be created from a continuous block of memory
 *      allocated by this routine and later freed by the X server.
 *      The first NULL pointer to the "next" data structure in the module
 *      list is interpreted by the X server as the end of the module list.
 *
 *      Returning a ppModuleList == NULL indicates a failure state.
 */
void XvGetExtensionModuleList(ppModuleList,nameExtnInit,nameExtnRegister)
        DlModuleListItemPtr *ppModuleList;
        char               **nameExtnInit;
        char               **nameExtnRegister;
{
    DlModuleListItemPtr    pList;

    if (!(*ppModuleList = (DlModuleListItemPtr)
                                    Xalloc(2 * sizeof(DlModuleListItemRec))))
        ErrorF("XV broker, unable to alloc required memory.\n");
    else
    {
	pList = *ppModuleList;
	pList->moduleName = "hprop.1";
	pList->modulePath = "hp";
	pList->bindState = DL_BIND_DEFERRED;
	pList->next = pList+1;
 
	pList++;
        pList->moduleName = "xv.1";
        pList->modulePath = "hp";
        pList->bindState = DL_BIND_DEFERRED | DL_BIND_VERBOSE;
        pList->next = NULL;

        *nameExtnInit     = "XvExtensionInit";
#ifdef  ORIGINAL
        *nameExtnRegister = ""; /* don't need one */
#else
        *nameExtnRegister = "XvExtensionInit"; /* don't need one */
#endif
    }
}
