static char sccsid[] = "@(#)77  1.2  src/UMS/usr/lpp/XV/server/dix/xvimage.c, Xv , ums212 2/28/95 15:36:17";
/*
 *   COMPONENT_NAME: xv
 *
 *   FUNCTIONS: none
 *
 *   ORIGINS: 16, 145, 27
 *
 *   This module contains IBM CONFIDENTIAL code.
 *   OBJECT CODE ONLY SOURCE MATERIALS
 *
 *   (C) COPYRIGHT International Business Machines Corp. 1995
 *   All Rights Reserved
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/***********************************************************
Copyright 1991 by Digital Equipment Corporation, Maynard, Massachusetts,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/
/*
** File: 
**
**   xvddx_image.c --- Xv server extension support for get and  put image
**
** Author: 
**
**   Andy Goodrich (RasterOps)
**
** Revisions:
**
**   01.04.91 Goodrich
**     - initial coding
**
** 
** $Log:   /plxhp/dave/pvcsrc/xv/server/dix/archives/xvimage.c_v  $
 * 
 *    Rev 1.6   09 Sep 1996 10:46:28   dave
 * no osstruct.h in X11R6 hp
** Revision 1.6  1996/04/04 15:17:56  dist
** no osstruct.h in X11R6 hp
**
 * Revision 1.5  96/04/04  19:24:40  19:24:40  dist (Software Distribution Account)
 * took out old "wrapped" get/put image
 * 
 * Revision 1.4  95/11/03  12:29:37  12:29:37  dist ()
 * Andy Goodrich: Check in at start of port to Solaris 2.5.
 * 
 * Revision 1.3  95/06/01  10:44:31  10:44:31  dist ()
 * Andy Goodrich: Split x,y position for GetImage hardware call back into
 *                its constituent pieces drawable position and offset.
 *                This is necessary for proper processing of zoomed images.
 * 
 * Revision 1.2  95/05/25  17:01:10  17:01:10  dist ()
 * Andy Goodrich - Combined HP/AIX version for HP 9.1 code freeze.
 * 
** Revision 1.2  95/04/18  11:57:20  11:57:20  dist ()
**   Changed calling sequence to hardware level for put image to accommodate
**   zoom support.
** 
**
*/

#include <stdio.h>
#include <sys/types.h>

#include "X.h"
#define NEED_REPLIES
#define NEED_EVENTS
#include "Xproto.h"
#include "windowstr.h"
#include "gcstruct.h"
#ifndef X11R6
#include "osstruct.h"
#endif
#include "selection.h"
#include "colormapst.h"
#include "cursorstr.h"
#include "scrnintstr.h"
#include "opaque.h"
#include "input.h"
#include "servermd.h"
#include "extnsionst.h"

#include "Xv.h"
#include "Xvproto.h"
#include "xvdix.h"
#include "xvddx.h"
#include "xvhw.h"
/* #define XV_DIAG 1 */
#include "xvdebug.h"

/* GLOBAL */

#ifdef _AIX
static int (* XvRealGetImage)() = 0;
static int (* XvRealPutImage)() = 0;
#endif

/* EXTERNAL */

extern int  (* ProcVector[256]) ();
extern int  (* ReplySwapVector[256]) ();

/* INTERNAL */

static int ProcXvGetImage();
static int ProcXvPutImage();

#define _WriteQueryAdaptorsReply(_c,_d) \
  if ((_c)->swapped) SWriteQueryAdaptorsReply(_c, _d); \
  else WriteToClient(_c, sz_xvQueryAdaptorsReply, _d)

/*----------------------------------------------------------------------------*
 |"ProcXvGetImage"
 |
 |  The Xv video extension version of ProcGetImage. This code will act
 |  normally unless the drawable specified has video attached to it and
 |  the video comes from a TV (mixer) style source. In this case a 24-bit
 |  image will be returned regardless of the type of drawable supplied.
 |    = client->noClientException on success 
 *----------------------------------------------------------------------------*/
static int
ProcXvGetImage(client)
    register ClientPtr	client;
{

    register int	     height;
	Xvhw_Image			 image_info; /* Image geometry info for hw layer. */
	long                 length;
	register int         linesDone;
	int                  linesPerBuf;
	int					 nData;		 /* # of bytes returned by image get. */
    char		         *pBuf;
    register DrawablePtr pDraw;
    XvddPortPtr	         pddPort;	 /* Pointer to Xvrop pointer. */
    Mask		         plane;
    XvPortPtr            pPort;   	 /* Pointer to Xv port for pDraw. */
    int                  iLine;      /* Index: line now getting. */
    long		         widthBytesLine;
    xGetImageReply	     xgi;

    REQUEST(xGetImageReq);

    REQUEST_SIZE_MATCH(xGetImageReq);
    if ((stuff->format != XYPixmap) && (stuff->format != ZPixmap))
    {
		client->errorValue = stuff->format;
        return(BadValue);
    }
    if(!(pDraw = LOOKUP_DRAWABLE(stuff->drawable, client) ))
    {
		client->errorValue = stuff->drawable;
		return (BadDrawable);
    }

#ifdef _AIX
    /* DETERMINE IF THERE IS A DEVICE-DEPENDENT XV GET IMAGE ROUTINE
     * IF NOT USE THE STANDARD GET IMAGE ROUTINE IN THE TRANSFER VECTOR.
     */
    pPort = (XvPortPtr)XvPort( pDraw );
    if ( !pPort ||
	     (stuff->depth != 32 && stuff->depth != 16) ||
	     !(pddPort = (XvddPortPtr)pPort->devPriv.ptr) ||
	     !((XvhwPtr)pddPort->phw)->get_image ||
	     stuff->format != ZPixmap )
    {
        return (*XvRealGetImage)(client);
    }
#endif

    /*  WE WILL USE THE XV DEVICE DEPENDENT GET IMAGE ROUTINE. 
	 *  CLIP REQUEST TO LIMITS OF SCREEN AND WINDOW.
	 */
    if(stuff->x < 0 ||
       stuff->x+(int)stuff->width > pDraw->width ||
       stuff->y < 0 ||
       stuff->y+(int)stuff->height > pDraw->height
    )
	{
		return(BadMatch);
	}
    xgi.visual = None;
    xgi.type = X_Reply;
    xgi.sequenceNumber = client->sequence;
#ifdef sun
    xgi.depth = 16;
#else  /* !sun */
    xgi.depth = 32;
#endif /* sun */
	height = stuff->height;
	widthBytesLine = ((stuff->width * xgi.depth / 8) + 3) & ~3;
    length = widthBytesLine * height;
    xgi.length = (length + 3) >> 2;

    if (widthBytesLine == 0 || height == 0)
		linesPerBuf = 0;
    else if (widthBytesLine >= IMAGE_BUFSIZE)
		linesPerBuf = 1;
    else
    {
		linesPerBuf = IMAGE_BUFSIZE / widthBytesLine;
		if (linesPerBuf > height)
			linesPerBuf = height;
    }
    length = linesPerBuf * widthBytesLine;
    if (linesPerBuf < height)
    {
		/* we have to make sure intermediate buffers don't need padding */
		while ((linesPerBuf > 1) && (length & 3))
		{
			linesPerBuf--;
			length -= widthBytesLine;
		}
		while (length & 3)
		{
			linesPerBuf++;
			length += widthBytesLine;
		}
    }
    if(!(pBuf = (char *) ALLOCATE_LOCAL(length)))
        return (BadAlloc);

    WriteReplyToClient(client, sizeof (xGetImageReply), &xgi);

    image_info.drawable_x = pDraw->x;
    image_info.drawable_y = pDraw->y;
    image_info.offset_x = stuff->x; 
    image_info.image_x = 0; 
    image_info.image_y = 0;
    image_info.image_w = stuff->width; 
    image_info.image_depth = 32;
    image_info.image_data_depth = 24;
    image_info.image_stride = stuff->width;
    if (linesPerBuf != 0 )
    {
		Xvhw_Suspend(pddPort->phw, 1);
        linesDone = 0;
        while (height - linesDone > 0)
        {
    		image_info.image_h = min(linesPerBuf, height - linesDone);
    		image_info.offset_y = stuff->y+linesDone;
			Xvhw_Get_Image(pddPort->phw, &image_info, pBuf);

			/* Note that this is NOT a call to WriteSwappedDataToClient,
               as we do NOT byte swap */
			(void)WriteToClient(client, 
				(int)(image_info.image_h * widthBytesLine), pBuf);
			linesDone += image_info.image_h;
        }
		Xvhw_Resume(pddPort->phw, 0);
    }
    DEALLOCATE_LOCAL(pBuf);
    return (client->noClientException);
}

/*----------------------------------------------------------------------------*
 |"ProcXvPutImage"
 |
 |  The Xv video extension version of ProcPutImage. This code will act
 |  normally unless the image being put is 16 or 32 bits in depth and the
 |  drawable specified has video attached to it and the video comes from 
 |  a TV (mixer) style source. In this case a 32-bit image will be put.
 |    = client->noClientException on success 
 *----------------------------------------------------------------------------*/
static int
ProcXvPutImage(client)
    register ClientPtr client;
{
	Xvhw_Image           image_info;	/* Information about image geometry. */
    long                 length;
    XvddPortPtr	         pddPort;		/* Pointer to Xvrop pointer. */
    register DrawablePtr pDraw;
    register GC          *pGC;
    XvPortPtr            pPort;     	/* Pointer to Xv port for pDraw. */

    REQUEST(xPutImageReq);
    REQUEST_AT_LEAST_SIZE(xPutImageReq);
    VALIDATE_DRAWABLE_AND_GC(stuff->drawable, pDraw, pGC, client);

#ifdef _AIX
    /* IF THE DEPTH OF THE IMAGE BEING PUT IS 32 AND THERE IS A
     * DEVICE-DEPENDENT XV PUT IMAGE ROUTINE USE IT, OTHERWISE USE
     * THE STANDARD PUT IMAGE ROUTINE IN THE TRANSFER VECTOR.
     */
    if ( stuff->format != ZPixmap || 
	     (stuff->depth != 32 && stuff->depth != 16) ||
	     stuff->leftPad != 0 ||
         !(pPort = (XvPortPtr)XvPort( pDraw )) ||
	     !(pddPort = (XvddPortPtr)pPort->devPriv.ptr) ||
	     !((XvhwPtr)pddPort->phw)->put_image )
    {
        return (*XvRealPutImage)(client);
    }
#endif

    length = (stuff->width * stuff->depth / 8 + 3) & ~3;
    length *= stuff->height;
    if ((((length + 3) >> 2) + (sizeof(xPutImageReq) >> 2)) != stuff->length)
		return BadLength;
	image_info.drawable_x = pDraw->x;
    image_info.drawable_y = pDraw->y;  
    image_info.offset_x = stuff->dstX;     
    image_info.offset_y = stuff->dstY;      
    image_info.image_x = 0;        
    image_info.image_y = 0;  
    image_info.image_w = stuff->width;          
    image_info.image_h = stuff->height;           
    image_info.image_depth = (stuff->depth == 24) ? 32 : stuff->depth;        
    image_info.image_data_depth = stuff->depth;    
    image_info.image_stride = stuff->width;
	Xvhw_Suspend(pddPort->phw, 1);
#ifdef OLDSTUFF
    Xvhw_Put_Image(pddPort->phw, stuff->depth, 
					pDraw->x, pDraw->y, stuff->dstX, stuff->dstY,
					stuff->width, stuff->height, (char *)&stuff[1]);
#else
    Xvhw_Put_Image(pddPort->phw, &image_info, (char *)&stuff[1]);
#endif
	Xvhw_Resume(pddPort->phw, 0);
    return (client->noClientException);
}

/*----------------------------------------------------------------------------*
 |"XvImageInit"
 |
 |  Set up to intercept the normal ProcGetImage and ProcPutImage calls by 
 |  changing the entries in the transfer vector.
 *----------------------------------------------------------------------------*/
void
XvImageInit()
{
#ifdef _AIX
#ifdef NOTDEF
    if ( (int)XvRealGetImage != (int)ProcVector[X_GetImage] )
#endif
    if ( (int)ProcXvGetImage != (int)ProcVector[X_GetImage] )
    {
    	XvRealGetImage = ProcVector[X_GetImage];  
    	ProcVector[X_GetImage] = ProcXvGetImage;
    }
#ifdef NOTDEF
    if ( (int)XvRealPutImage != (int)ProcVector[X_PutImage] )
#endif
    if ( (int)ProcXvPutImage != (int)ProcVector[X_PutImage] )
    {
    	XvRealPutImage = ProcVector[X_PutImage];  
    	ProcVector[X_PutImage] = ProcXvPutImage;
    }
#endif
}

/*----------------------------------------------------------------------------*
 |"XvImageTerm"
 |
 |  Undo the intercept we were using for Get and Put Image.
 |  By changing back the entries in the transfer vector.
 *----------------------------------------------------------------------------*/
void
XvImageTerm()
{
#ifdef _AIX
    if ( (int)ProcXvGetImage == (int)ProcVector[X_GetImage] )
    {
    	ProcVector[X_GetImage] = XvRealGetImage;  
    }
    if ( (int)ProcXvPutImage == (int)ProcVector[X_PutImage] )
    {
    	ProcVector[X_PutImage] = XvRealPutImage;
    }
#endif
}
