static char sccsid[] = "%Z%%M%  %I%  %W% %G% %U%";
/*
 *   COMPONENT_NAME: xv
 *
 *   FUNCTIONS: none
 *
 *   ORIGINS: 16, 145, 27
 *
 *   This module contains IBM CONFIDENTIAL code.
 *   OBJECT CODE ONLY SOURCE MATERIALS
 *
 *   (C) COPYRIGHT International Business Machines Corp. 1995
 *   All Rights Reserved
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/***********************************************************
Copyright 1991 by Digital Equipment Corporation, Maynard, Massachusetts,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.
Copyright 1994 by Parallax Corporation, Santa Clara, California.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/
/*
** File: 
**
**   xvmain.c --- Xv server extension main device independent module.
**   
** Author: 
**
**   David Carver (Digital Workstation Engineering/Project Athena)
**
** Revisions:
**
**   12.02.94 Goodrich
**     - moved functions to xvwrapper.c and other changes to better define
**       interface between device-independent and device-dependent layers.
**
**   12.06.92 Goodrich
**     - added screen close code to XvResetProc to allow device-dependent
**       cleanup for HP
**    
**   05.02.92 Goodrich
**     - move invocation of Xv per screen ddx initialization from 
**       screen device drivers into XvExtensionInit.
**
**   04.09.91 Carver
**     - change: stop video always generates an event even when video
**       wasn't active
**
**   29.08.91 Carver
**     - change: unrealizing windows no longer preempts video
**
**   11.06.91 Carver
**     - changed SetPortControl to SetPortAttribute
**     - changed GetPortControl to GetPortAttribute
**     - changed QueryBestSize
**
**   28.05.91 Carver
**     - fixed Put and Get requests to not preempt operations to same drawable
**
**   15.05.91 Carver
**     - version 2.0 upgrade
**
**   19.03.91 Carver
**     - fixed Put and Get requests to honor grabbed ports.
**     - fixed Video requests to update di structure with new drawable, and
**       client after calling ddx.
**
**   24.01.91 Carver
**     - version 1.4 upgrade
**       
** Notes:
**
**   Port structures reference client structures in a two different
**   ways: when grabs, or video is active.  Each reference is encoded
**   as fake client resources and thus when the client is goes away so
**   does the reference (it is zeroed).  No other action is taken, so
**   video doesn't necessarily stop.  It probably will as a result of
**   other resources going away, but if a client starts video using
**   none of its own resources, then the video will continue to play
**   after the client disappears.
**
**
 * $Log:   /export/home/dist1/source/xv/server/dix/archives/xvmain.c_v  $
 * 
 *    Rev 1.6   17 Sep 1996 09:47:30   dave
 *  
 * Revision 1.3  95/06/01  10:43:55  10:43:55  dist ()
 * Andy Goodrich: Add zeroing of pPort->rid for IBM-style resource tracking.
 * 
 * Revision 1.2  95/05/25  17:01:17  17:01:17  dist ()
 * Andy Goodrich - Combined HP/AIX version for HP 9.1 code freeze.
 * 
*/

#include <stdio.h>

#include "X.h"
#include "Xproto.h"
#include "misc.h"
#include "os.h"
#include "scrnintstr.h"
#include "windowstr.h"
#include "pixmapstr.h"
#include "gc.h"
#include "extnsionst.h"
#include "dixstruct.h"
#include "gcstruct.h"
#include "resource.h"
#include "opaque.h"
#include "input.h"

#ifdef __hpux
#define XV_MODULE_PATH  "/usr/lib/X11/Xserver/modules/extensions/hp/hprop.1"
#define XV_INIT_PROC    "XvDdxInit"
#include <dl.h>
#endif /* __hpux */

#include "Xv.h"
#include "Xvproto.h"
#define GLOBAL
#include "xvdix.h"
#undef GLOBAL

/*  XV_DIAG BIT MEANINGS:
 *
 *    0x01 - normal one-shot debugging
 *    0x20 - notification debugging
 *    0x40 - wrapper debugging
 *    0x80 - initialization debugging
 */
/* #define XV_DIAG 0xC1 */
#include "xvdebug.h"



/* FORWARD REFERENCES: */

extern XID 			clientErrorValue;
Bool 				CreateResourceTypes();
extern void 		(* EventSwapVector[128])();
extern Atom 		MakeAtom();
extern WindowPtr* 	WindowTable;
static void 		WriteSwappedPortNotifyEvent();
static void 		WriteSwappedVideoNotifyEvent();

XvScreenPtr XvScreens[MAXSCREENS];


/*
** XvExtensionInit
**
**
*/

void 
XvExtensionInit()
{
	ExtensionEntry* extEntry; /* Entry for our extension. */
    int 			i_screen; /* # of screen in screenInfo.screens accessing. */

	DEBUGF(0x80, (fd, "\n\nXvExtensionInit: entry stack @ %08x\n", &extEntry));

	/*
	 *  LOOK TO SEE IF ANY SCREENS WERE INITIALIZED; IF NOT THEN
	 *  INIT GLOBAL VARIABLES SO THE EXTENSION CAN FUNCTION 
	 */
	if (XvScreenGeneration != serverGeneration)
    {
		DEBUGF(0x80, (fd, "XvExtensionInit: before CreateResourceTypes\n"));
	    if (!CreateResourceTypes())
        {
            ErrorF("XvExtensionInit: Unable to allocate resource types\n");
            return;
        }
		DEBUGF(0x80, (fd, "XvExtensionInit: after CreateResourceTypes\n"));
        XvScreenIndex = AllocateScreenPrivateIndex ();
        if (XvScreenIndex < 0)
        {
            ErrorF("XvExtensionInit: Can't allocate screen private index\n");
            return;
        }

#ifdef __hpux /* Dynamic loading code */

#define ALL_SHLIBS_HANDLE &dummy_handle

		/*
		 * If the driver entrypoint is already visible, skip the load.
		 */
		{
			int			    ret_val;
			shl_t			ldr_module_id;
			long			ldr_module_entry=0;
			char			*dummy_handle = NULL;


			ret_val = shl_findsym((shl_t *) ALL_SHLIBS_HANDLE, XV_INIT_PROC,
								  TYPE_PROCEDURE, &ldr_module_entry );
			if (ret_val) {
				ldr_module_id = shl_load(XV_MODULE_PATH,
										 (BIND_DEFERRED | BIND_VERBOSE), 0L);
			if ( ldr_module_id == NULL )
				FatalError ("X server: Failed to load shared library %s\n",
							XV_MODULE_PATH);
			}
		}
#endif /* __hpux End dynamic loading code */

        /*  PROCESS SCREENS' XV DEVICES:
         *
         *  Note we make sure that each screen starts with a null devprivate
         *  pointer. This is necessary since main.c doesn't initialize the 
         *  devprivate pointers for a newly created screen, nor are they 
         *  initialized when a new screen private index is allocated.
         */
		DEBUGF(0x80, (fd, "XvExtensionInit: Setting up devPrivates\n"));
        for (i_screen = 0; i_screen < screenInfo.numScreens; i_screen++)
		{
			DEBUGF(0x80,(fd,"XvScreenIndex %x dp %x &dp %x &dp[s] %x &ptr %x\n",
        		XvScreenIndex, screenInfo.screens[i_screen]->devPrivates,
				&screenInfo.screens[i_screen]->devPrivates, 
				&screenInfo.screens[i_screen]->devPrivates[XvScreenIndex], 
				&screenInfo.screens[i_screen]->devPrivates[XvScreenIndex].ptr)
			);
            screenInfo.screens[i_screen]->devPrivates[XvScreenIndex].ptr = 
				(pointer)NULL;
			XvddScreenInit(screenInfo.screens[i_screen]);
		}
		DEBUGF(0x80, (fd, "XvExtensionInit: After screen initializations\n"));

        XvScreenGeneration = serverGeneration;
    }

    if (XvExtensionGeneration != serverGeneration)
    {
        XvExtensionGeneration = serverGeneration;

		DEBUGF(0x80, (fd, "XvExtensionInit: Before AddExtension\n"));
        extEntry = AddExtension(XvName, XvNumEvents, XvNumErrors, 
                              ProcXvDispatch, SProcXvDispatch,
                              XvResetProc, StandardMinorOpcode);
		DEBUGF(0x80, (fd, "XvExtensionInit: After AddExtension\n"));
        if (!extEntry) 
        {
            FatalError("XvExtensionInit: AddExtensions failed\n");
        }
        XvReqCode = extEntry->base;
        XvEventBase = extEntry->eventBase;
        XvErrorBase = extEntry->errorBase;

        EventSwapVector[XvEventBase+XvVideoNotify] = 
            WriteSwappedVideoNotifyEvent;
        EventSwapVector[XvEventBase+XvPortNotify] = 
            WriteSwappedPortNotifyEvent;

        (void)MakeAtom(XvName, strlen(XvName), xTrue);
		m_Symtab_debug("#### after XvName");
		DEBUGF(0x80, (fd, "XvExtensionInit: After making XvName atom\n"));
    }
	DEBUGF(0x80, (fd, "XvExtensionInit: exiting stack @ %08x\n", &extEntry));
}

Bool CreateResourceTypes()

{
  
  if (XvResourceGeneration == serverGeneration) return TRUE;

  XvResourceGeneration = serverGeneration;

  if (!(XvRTPort = CreateNewResourceType(XvdiDestroyPort)))
    {
      ErrorF("CreateResourceTypes: failed to allocate port resource.\n");
      return FALSE;
    }
  
  if (!(XvRTGrab = CreateNewResourceType(XvdiDestroyGrab)))
    {
      ErrorF("CreateResourceTypes: failed to allocate grab resource.\n");
      return FALSE;
    }
  
  if (!(XvRTEncoding = CreateNewResourceType(XvdiDestroyEncoding)))
    {
      ErrorF("CreateResourceTypes: failed to allocate encoding resource.\n");
      return FALSE;
    }
  
  if (!(XvRTVideoNotify = CreateNewResourceType(XvdiDestroyVideoNotify)))
    {
      ErrorF("CreateResourceTypes: failed to allocate video notify resource.\n");
      return FALSE;
    }
  
  if (!(XvRTVideoNotifyList = CreateNewResourceType(XvdiDestroyVideoNotifyList)))
    {
      ErrorF("CreateResourceTypes: failed to allocate video notify list resource.\n");
      return FALSE;
    }

  if (!(XvRTPortNotify = CreateNewResourceType(XvdiDestroyPortNotify)))
    {
      ErrorF("CreateResourceTypes: failed to allocate port notify resource.\n");
      return FALSE;
    }

  return TRUE;

}

void
XvResetProc()
{
    int         i_screen;    /* Screen in screenInfo->screens to access. */

	DEBUGF(0x80, (fd, "Entering reset proc stack =%08x\n", &i_screen));
    for ( i_screen = 0; i_screen < screenInfo.numScreens; i_screen++ )
    {
        if ( XvScreens[i_screen] != 0 )
        {
			DEBUGF(0x80, (fd, "Closing screen %d\n", i_screen));
            XvCloseScreen(i_screen, screenInfo.screens[i_screen]);
            XvScreens[i_screen] = 0; 
        }
    }
	DEBUGF(0x80, (fd, "Exiting reset proc stack =%08x\n", &i_screen));
}


/* The XvdiVideoStopped procedure is a hook for the device dependent layer.
   It provides a way for the dd layer to inform the di layer that video has
   stopped in a port for reasons that the di layer had no control over; note
   that it doesn't call back into the dd layer */

int
XvdiVideoStopped(pPort, reason)
  XvPortPtr pPort;
  int       reason;
{
  
    /* IF PORT ISN'T ACTIVE THEN WE'RE DONE */

    if (!pPort->pDraw) return Success;

    XvdiSendVideoNotify(pPort, pPort->pDraw, reason);

    pPort->pDraw = (DrawablePtr)NULL;
    pPort->client = (ClientPtr)NULL;
    pPort->time = currentTime;

    return Success;

}

int 
XvdiDestroyPort(pPort)
  XvPortPtr pPort;
{
    return (* pPort->pAdaptor->ddFreePort)(pPort);
}

int
XvdiDestroyGrab(pGrab)
  XvGrabPtr pGrab;
{
    pGrab->client = (ClientPtr)NULL;
}

int
XvdiDestroyVideoNotify(pn)
  XvVideoNotifyPtr pn;

{
    /* JUST CLEAR OUT THE client POINTER FIELD */

    pn->client = (ClientPtr)NULL;
}

int
XvdiDestroyPortNotify(pn)
  XvPortNotifyPtr pn;

{
    /* JUST CLEAR OUT THE client POINTER FIELD */

    pn->client = (ClientPtr)NULL;
}

int
XvdiDestroyVideoNotifyList(pn)
  XvVideoNotifyPtr pn;

{
    XvVideoNotifyPtr npn,cpn;

    /* ACTUALLY DESTROY THE NOTITY LIST */

    cpn = pn;

    while (cpn)
    {
        npn = cpn->next;
        if (cpn->client) FreeResource(cpn->id, XvRTVideoNotify);
        xfree(cpn);
        cpn = npn;
    }
}

int
XvdiDestroyEncoding()
{
}

int
XvdiSendVideoNotify(pPort, pDraw, reason)
  XvPortPtr   pPort;
  DrawablePtr pDraw;
  int         reason;
{
    xvEvent          event;
    XvVideoNotifyPtr pn;

    pn = (XvVideoNotifyPtr)LookupIDByType(pDraw->id, XvRTVideoNotifyList);
  
    while (pn) 
    {
        if (pn->client)
        {
            event.u.u.type = XvEventBase + XvVideoNotify;
            event.u.u.sequenceNumber = pn->client->sequence;
            event.u.videoNotify.time = currentTime.milliseconds;
            event.u.videoNotify.drawable = pDraw->id;
            event.u.videoNotify.port = pPort->id;
            event.u.videoNotify.reason = reason;
            (void) TryClientEvents(pn->client, &event, 1, NoEventMask,
                                   NoEventMask, NullGrab);
        }
        pn = pn->next;
    }

    return Success;

}


int
XvdiSendPortNotify(pPort, attribute, value)
  XvPortPtr pPort;
  Atom      attribute;
  INT32     value;
{
    xvEvent         event;
    XvPortNotifyPtr pn;

    pn = pPort->pNotify;

	DEBUGF(0x20, (fd, "XvSendPortNotify: pn=%08x pn->client %08x\n",
                    pn, pn->client));

    while (pn) 
    {
        if (pn->client)
        {
            event.u.u.type = XvEventBase + XvPortNotify;
            event.u.u.sequenceNumber = pn->client->sequence;
            event.u.portNotify.time = currentTime.milliseconds;
            event.u.portNotify.port = pPort->id;
            event.u.portNotify.attribute = attribute;
            event.u.portNotify.value = value;
            (void) TryClientEvents(pn->client, &event, 1, NoEventMask,
                                   NoEventMask, NullGrab);
        }
        pn = pn->next;
    }

    return Success;

}

int
XvdiPutVideo(client, pDraw, pPort, pGC, vid_x, vid_y, vid_w, vid_h,
             drw_x, drw_y, drw_w, drw_h)

  ClientPtr client;
  DrawablePtr pDraw;
  XvPortPtr pPort;
  GCPtr pGC;
  INT16 vid_x, vid_y, drw_x, drw_y;
  CARD16 vid_w, vid_h, drw_w, drw_h;
  
{
    int status, id;
    DrawablePtr pOldDraw;

    /* UPDATE TIME VARIABLES FOR USE IN EVENTS */

    UpdateCurrentTime();

    /* CHECK FOR GRAB; IF THIS CLIENT DOESN'T HAVE THE PORT GRABBED THEN
       INFORM CLIENT OF ITS FAILURE */

    if (pPort->grab.client && (pPort->grab.client != client))
    {
        XvdiSendVideoNotify(pPort, pDraw, XvBusy);
        return Success;
    }

    /* CHECK TO SEE IF PORT IS IN USE; IF SO THEN WE MUST DELIVER INTERRUPTED
       EVENTS TO ANY CLIENTS WHO WANT THEM */

    pOldDraw = pPort->pDraw;
    if ((pOldDraw) && (pOldDraw != pDraw))
    {
        XvdiSendVideoNotify(pPort, pPort->pDraw, XvPreempted);
    }

    status = (* pPort->pAdaptor->ddPutVideo)(client, pDraw, pPort, pGC, 
                                             vid_x, vid_y, vid_w, vid_h, 
                                             drw_x, drw_y, drw_w, drw_h);

    if ( (pPort->pDraw) && (pOldDraw != pDraw) )
    {
        pPort->client = client;
        XvdiSendVideoNotify(pPort, pPort->pDraw, XvStarted);
    }

#ifdef _AIX
    pPort->rid = FakeClientID(client->index);
    if(!AddResource(pPort->rid, XvRTPort, pPort)) 
	{
        DEBUGF(0x80, (fd, "AddResource failure on PutVideo\n"));
    }
#endif 

    pPort->time = currentTime;

    return (Success);

}

int
XvdiPutStill(client, pDraw, pPort, pGC, vid_x, vid_y, vid_w, vid_h,
             drw_x, drw_y, drw_w, drw_h)

    ClientPtr client;
    DrawablePtr pDraw;
    XvPortPtr pPort;
    GCPtr pGC;
    INT16 vid_x, vid_y, drw_x, drw_y;
    CARD16 vid_w, vid_h, drw_w, drw_h;

{
    int status, id;

    /* UPDATE TIME VARIABLES FOR USE IN EVENTS */

    UpdateCurrentTime();

    /* CHECK FOR GRAB; IF THIS CLIENT DOESN'T HAVE THE PORT GRABBED THEN
       INFORM CLIENT OF ITS FAILURE */

    if (pPort->grab.client && (pPort->grab.client != client))
    {
        XvdiSendVideoNotify(pPort, pDraw, XvBusy);
        return Success;
    }

	pPort->rid = NULL;
    pPort->time = currentTime;

    status = (* pPort->pAdaptor->ddPutStill)(client, pDraw, pPort, pGC, 
                                             vid_x, vid_y, vid_w, vid_h, 
                                             drw_x, drw_y, drw_w, drw_h);

    return status;

}
int
XvdiGetVideo(client, pDraw, pPort, pGC, vid_x, vid_y, vid_w, vid_h,
             drw_x, drw_y, drw_w, drw_h)

    ClientPtr client;
    DrawablePtr pDraw;
    XvPortPtr pPort;
    GCPtr pGC;
    INT16 vid_x, vid_y, drw_x, drw_y;
    CARD16 vid_w, vid_h, drw_w, drw_h;

{
    int status, id;
    DrawablePtr pOldDraw;

    /* UPDATE TIME VARIABLES FOR USE IN EVENTS */

    UpdateCurrentTime();

    /* CHECK FOR GRAB; IF THIS CLIENT DOESN'T HAVE THE PORT GRABBED THEN
       INFORM CLIENT OF ITS FAILURE */

    if (pPort->grab.client && (pPort->grab.client != client))
    {
        XvdiSendVideoNotify(pPort, pDraw, XvBusy);
        return Success;
    }

    /* CHECK TO SEE IF PORT IS IN USE; IF SO THEN WE MUST DELIVER INTERRUPTED
       EVENTS TO ANY CLIENTS WHO WANT THEM */

    pOldDraw = pPort->pDraw;
    if ((pOldDraw) && (pOldDraw != pDraw))
    {
        XvdiSendVideoNotify(pPort, pPort->pDraw, XvPreempted);
    }

    status = (* pPort->pAdaptor->ddGetVideo)(client, pDraw, pPort, pGC,
                                             vid_x, vid_y, vid_w, vid_h, 
                                             drw_x, drw_y, drw_w, drw_h);

    if ((pPort->pDraw) /* && (pOldDraw != pDraw)*/ )
    {
        pPort->client = client;
        XvdiSendVideoNotify(pPort, pPort->pDraw, XvStarted);
    }

    pPort->time = currentTime;

#ifdef _AIX
    pPort->rid = FakeClientID(client->index);
    if(!AddResource(pPort->rid, XvRTPort, pPort)) {
        DEBUGF(0x80, (fd, "AddResource failure on GetVideo\n"));
    }
#endif

    return (Success);

}

int
XvdiGetStill(client, pDraw, pPort, pGC, vid_x, vid_y, vid_w, vid_h,
             drw_x, drw_y, drw_w, drw_h)

    ClientPtr client;
    DrawablePtr pDraw;
    XvPortPtr pPort;
    GCPtr pGC;
    INT16 vid_x, vid_y, drw_x, drw_y;
    CARD16 vid_w, vid_h, drw_w, drw_h;

{
    int status, id;

    /* UPDATE TIME VARIABLES FOR USE IN EVENTS */
  
    UpdateCurrentTime();

    /* CHECK FOR GRAB; IF THIS CLIENT DOESN'T HAVE THE PORT GRABBED THEN
       INFORM CLIENT OF ITS FAILURE */

    if (pPort->grab.client && (pPort->grab.client != client))
    {
        XvdiSendVideoNotify(pPort, pDraw, XvBusy);
        return Success;
    }

    status = (* pPort->pAdaptor->ddGetStill)(client, pDraw, pPort, pGC, 
                                             vid_x, vid_y, vid_w, vid_h, 
                                             drw_x, drw_y, drw_w, drw_h);

	pPort->rid = NULL;
    pPort->time = currentTime;

    return status;

}

int
XvdiGrabPort(client, pPort, ctime, p_result)

    ClientPtr client;
    XvPortPtr pPort;
    Time ctime;
    int *p_result;

{
    unsigned long id;
    TimeStamp time;

    UpdateCurrentTime();
    time = ClientTimeToServerTime(ctime);

    if (pPort->grab.client && (client != pPort->grab.client))
    {
        *p_result = XvAlreadyGrabbed;
        return Success;
    }

    if ((CompareTimeStamps(time, currentTime) == LATER) ||
        (CompareTimeStamps(time, pPort->time) == EARLIER))
    {
        *p_result = XvInvalidTime;
        return Success;
    }

    if (client == pPort->grab.client)
    {
        *p_result = Success;
        return Success;
    }

    id = FakeClientID(client->index);

    if (!AddResource(id, XvRTGrab, &pPort->grab))
    {
        return BadAlloc;
    }

    /* IF THERE IS ACTIVE VIDEO THEN STOP IT */

    if ((pPort->pDraw) && (client != pPort->client))
    {
        XVCALL(diStopVideo)((ClientPtr)NULL, pPort, pPort->pDraw);
    }

    pPort->grab.client = client;
    pPort->grab.id = id;

    pPort->time = currentTime;

    *p_result = Success;

    return Success;

}

int
XvdiUngrabPort(client, pPort, ctime)

    ClientPtr client;
    XvPortPtr pPort;
    Time ctime;

{
    TimeStamp time;

    UpdateCurrentTime();
    time = ClientTimeToServerTime(ctime);

    if ((!pPort->grab.client) || (client != pPort->grab.client))
    {
        return Success;
    }

    if ((CompareTimeStamps(time, currentTime) == LATER) ||
        (CompareTimeStamps(time, pPort->time) == EARLIER))
    {
        return Success;
    }

    /* FREE THE GRAB RESOURCE; AND SET THE GRAB CLIENT TO NULL */

    FreeResource(pPort->grab.id, XvRTGrab);
    pPort->grab.client = (ClientPtr)NULL;

    pPort->time = currentTime;

    return Success;

}


int
XvdiSelectVideoNotify(client, pDraw, onoff)

    ClientPtr client;
    DrawablePtr pDraw;
    BOOL onoff;

{
    register int ii;
    int id;
    XvVideoNotifyPtr pn,tpn,fpn;
  
    /* FIND VideoNotify LIST */

    pn = (XvVideoNotifyPtr)LookupIDByType(pDraw->id, XvRTVideoNotifyList);

    /* IF ONE DONES'T EXIST AND NO MASK, THEN JUST RETURN */

    if (!onoff && !pn) return Success;

    /* IF ONE DOESN'T EXIST CREATE IT AND ADD A RESOURCE SO THAT THE LIST
       WILL BE DELETED WHEN THE DRAWABLE IS DESTROYED */

    if (!pn) 
    {
        if (!(tpn = (XvVideoNotifyPtr)xalloc(sizeof(XvVideoNotifyRec))))
            return BadAlloc;
        tpn->next = (XvVideoNotifyPtr)NULL;
        if (!AddResource(pDraw->id, XvRTVideoNotifyList, tpn))
        {
            xfree(tpn);
            return BadAlloc;
        }
    }
    else
    {
        /* LOOK TO SEE IF ENTRY ALREADY EXISTS */

        fpn = (XvVideoNotifyPtr)NULL;
        tpn = pn;
        while (tpn)
        {
            if (tpn->client == client) 
            {
                if (!onoff) tpn->client = (ClientPtr)NULL;
                return Success;
            }
            if (!tpn->client) fpn = tpn; /* TAKE NOTE OF FREE ENTRY */
            tpn = tpn->next;
        }

        /* IF TUNNING OFF, THEN JUST RETURN */

        if (!onoff) return Success;

        /* IF ONE ISN'T FOUND THEN ALLOCATE ONE AND LINK IT INTO THE LIST */

        if (fpn)
        {
            tpn = fpn;
        }
        else
        {
            if (!(tpn = (XvVideoNotifyPtr)xalloc(sizeof(XvVideoNotifyRec))))
                return BadAlloc;
            tpn->next = pn->next;
            pn->next = tpn;
        }
    }

    /* INIT CLIENT PTR IN CASE WE CAN'T ADD RESOURCE */
    /* ADD RESOURCE SO THAT IF CLIENT EXITS THE CLIENT PTR WILL BE CLEARED */

    tpn->client = (ClientPtr)NULL;
    tpn->id = FakeClientID(client->index);
    AddResource(tpn->id, XvRTVideoNotify, tpn);

    tpn->client = client;
    return Success;

}

int
XvdiSelectPortNotify(client, pPort, onoff)

    ClientPtr client;
    XvPortPtr pPort;
    BOOL onoff;

{
    register int ii;
    int id;
    XvPortNotifyPtr pn,tpn;

    /* SEE IF CLIENT IS ALREADY IN LIST */

    tpn = (XvPortNotifyPtr)NULL;
    pn = pPort->pNotify;
    while (pn)
    {
        if (!pn->client) tpn = pn; /* TAKE NOTE OF FREE ENTRY */
        if (pn->client == client) break;
        pn = pn->next;
    }

    /* IS THE CLIENT ALREADY ON THE LIST? */

    if (pn)
    {
        /* REMOVE IT? */

        if (!onoff)
        {
            pn->client = (ClientPtr)NULL;
            FreeResource(pn->id, XvRTPortNotify);
        }

        return Success;
    }

    /* DIDN'T FIND IT; SO REUSE LIST ELEMENT IF ONE IS FREE OTHERWISE 
       CREATE A NEW ONE AND ADD IT TO THE BEGINNING OF THE LIST */

    if (!tpn)
    {
        if (!(tpn = (XvPortNotifyPtr)xalloc(sizeof(XvPortNotifyRec))))
            return BadAlloc;
        tpn->next = pPort->pNotify;
        pPort->pNotify = tpn;
    }

    tpn->client = client;
    tpn->id = FakeClientID(client->index);
    AddResource(tpn->id, XvRTPortNotify, tpn);

    return Success;

}

int
XvdiStopVideo(client, pPort, pDraw)

    ClientPtr client;
    XvPortPtr pPort;
    DrawablePtr pDraw;

{
    /* IF PORT ISN'T ACTIVE THEN WE'RE DONE */

    if (!pPort->pDraw || (pPort->pDraw != pDraw)) 
    {
        XvdiSendVideoNotify(pPort, pDraw, XvStopped);
        return Success;
    }

    /* CHECK FOR GRAB; IF THIS CLIENT DOESN'T HAVE THE PORT GRABBED THEN
       INFORM CLIENT OF ITS FAILURE */

    if ((client) && (pPort->grab.client) && (pPort->grab.client != client))
    {
        XvdiSendVideoNotify(pPort, pDraw, XvBusy);
        return Success;
    }

    XvdiSendVideoNotify(pPort, pDraw, XvStopped);

#ifdef _AIXNEW
	FreeResource(pPort->rid, XvRTPort);
	pPort->rid = NULL;
#endif

    (* pPort->pAdaptor->ddStopVideo)(client, pPort, pDraw);

	return Success;
}

int
XvdiPreemptVideo(client, pPort, pDraw)

    XvPortPtr pPort;
    DrawablePtr pDraw;
  
{
    /* IF PORT ISN'T ACTIVE THEN WE'RE DONE */

    if (!pPort->pDraw || (pPort->pDraw != pDraw)) return Success;

    XvdiSendVideoNotify(pPort, pPort->pDraw, XvPreempted);

    (* pPort->pAdaptor->ddStopVideo)(client, pPort, pPort->pDraw);

    pPort->pDraw = (DrawablePtr)NULL;
    pPort->client = (ClientPtr)client;
    pPort->time = currentTime;

    return Success;

}

XvdiMatchPort(pPort, pDraw)
    XvPortPtr pPort;
    DrawablePtr pDraw;

{

    XvAdaptorPtr pa;
    XvFormatPtr pf;
    int nf;

    pa = pPort->pAdaptor;

    if (pa->pScreen != pDraw->pScreen) return BadMatch;

    nf = pa->nFormats;
    pf = pa->pFormats;

    while (nf--)
    {
        if ((pf->depth == pDraw->depth) &&
            ((pDraw->type == DRAWABLE_PIXMAP) || 
             (wVisual(((WindowPtr)pDraw)) == pf->visual)))
            return Success;
        pf++;
    }

    return BadMatch;

}

int
XvdiSetPortAttribute(client, pPort, attribute, value)
    ClientPtr client;
    XvPortPtr pPort;
    Atom attribute;
    INT32 value;

{

    return 
      (* pPort->pAdaptor->ddSetPortAttribute)(client, pPort, attribute, value);

}

int
XvdiGetPortAttribute(client, pPort, attribute, p_value)
    ClientPtr client;
    XvPortPtr pPort;
    Atom attribute;
    INT32 *p_value;

{

    return 
      (*pPort->pAdaptor->ddGetPortAttribute)(client, pPort, attribute, p_value);

}

static void
WriteSwappedVideoNotifyEvent(from, to)

    xvEvent *from, *to;

{

    to->u.u.type = from->u.u.type;
    to->u.u.detail = from->u.u.detail;
    cpswaps(from->u.videoNotify.sequenceNumber, 
            to->u.videoNotify.sequenceNumber);
    cpswapl(from->u.videoNotify.time, to->u.videoNotify.time);
    cpswapl(from->u.videoNotify.drawable, to->u.videoNotify.drawable);
    cpswapl(from->u.videoNotify.port, to->u.videoNotify.port);

}

static void
WriteSwappedPortNotifyEvent(from, to)

    xvEvent *from, *to;

{

    to->u.u.type = from->u.u.type;
    to->u.u.detail = from->u.u.detail;
    cpswaps(from->u.portNotify.sequenceNumber, to->u.portNotify.sequenceNumber);
    cpswapl(from->u.portNotify.time, to->u.portNotify.time);
    cpswapl(from->u.portNotify.port, to->u.portNotify.port);
    cpswapl(from->u.portNotify.attribute, to->u.portNotify.attribute);
    cpswapl(from->u.portNotify.value, to->u.portNotify.value);

}

/*----------------------------------------------------------------------------*
 |"XvPort"
 |
 |  Externally called routine to check if a drawable has an XV port
 |  connected to it. We need to find a port whose drawable matches the one 
 |  passed in to us. For each adaptor present in the system we search its 
 |  list of ports. 
 |
 |    -> port on success
 |    =  0 on failure
 *----------------------------------------------------------------------------*/
XvPortPtr XvPort( pDraw )
    DrawablePtr  pDraw;   /* -> drawable checking for Tv port. */
{
    int          iAdaptor;     /* Index: # of adaptor checking. */
    int          nAdaptor;     /* Number of adaptors to check. */
    int          nPort;        /* Index: number of ports left to check. */
    XvAdaptorPtr pAdaptor;     /* Pointer to adaptor now checking. */
    XvPortPtr    pPort;        /* Pointer to port now checking. */
    ScreenPtr    pScreen;      /* Pointer to screen for pDraw. */
    XvScreenPtr  pXvScreen;    /* Pointer to Xv screen structure for pScreen. */


    if ( pDraw->type != DRAWABLE_WINDOW ) return (XvPortPtr) 0;

    pScreen = pDraw->pScreen;
    pXvScreen = (XvScreenPtr)pScreen->devPrivates[XvScreenIndex].ptr;
    if ( !pXvScreen ) return (XvPortPtr) 0;

    nAdaptor = pXvScreen->nAdaptors;
    for ( iAdaptor = 0; iAdaptor < nAdaptor; iAdaptor++ )
    {
		pAdaptor = pXvScreen->pPadaptors[iAdaptor];
        nPort = pAdaptor->nPorts;
        pPort = pAdaptor->pPorts;
        while ( nPort-- )
        {
            if ( pPort->pDraw == pDraw )return pPort;
            pPort++;
        }
    }
    return 0;
}

