static char sccsid[] = "%Z%%M%  %I%  %W% %G% %U%";
/*
 *   COMPONENT_NAME: xv
 *
 *   FUNCTIONS: none
 *
 *   ORIGINS: 16, 145, 27
 *
 *   This module contains IBM CONFIDENTIAL code.
 *   OBJECT CODE ONLY SOURCE MATERIALS
 *
 *   (C) COPYRIGHT International Business Machines Corp. 1995
 *   All Rights Reserved
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/***********************************************************
Copyright 1991 by Digital Equipment Corporation, Maynard, Massachusetts,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/
/*
** File: 
**
**   xvddx_hw.c --- Generic Xv DD layer routines which call a hardware
**                  specific layer for vendor-specific services.
**
** Authors: 
**
**   David Carver (Digital Workstation Engineering/Project Athena)
**   Andy Goodrich (Parallax and RasterOps)
**   
** Revisions:
**
**   11.11.94 Goodrich
**     - rewrite from original xvddx.c done by Dave Carver. This new version
**       utilizes the Xvhw interface and does not contain the routines which
**       were moved to server/xvddx.c.
*
 * $Log:   /plxhp/dave/pvcsrc/xv/server/dix/archives/xvddx_hw.c_v  $
 * 
 *    Rev 1.7   09 Sep 1996 10:45:34   dave
 * fixed output detach after client leaves without shutting off
 * Revision 1.7  1996/04/19 22:01:20  dist
 * fixed output detach after client leaves without shutting off
 *
 * Revision 1.6  1996/04/04  19:23:54  dist
 * fixed some output probs.
 *
 * Revision 1.5  1996/04/04  01:49:28  dist
 * needed to detach port on output off
 *
 * Revision 1.4  95/12/11  15:26:43  15:26:43  dist (Software Distribution Account)
 * Andy Goodrich: Merge of hedge changes into existing HP/AIX source.
 * 
 * Revision 1.3  95/11/03  12:29:22  12:29:22  dist ()
 * Andy Goodrich: Check in at start of port to Solaris 2.5.
 * 
 * Revision 1.2  95/05/25  17:00:36  17:00:36  dist ()
 * Andy Goodrich - Combined HP/AIX version for HP 9.1 code freeze.
 * 
 * Revision 1.1  95/04/18  11:57:12  11:57:12  dist ()
 * Initial revision
 * 
 * Revision 1.1  1995/04/17  20:06:19  dist
 * Initial revision
 *
*/
#include <stdio.h>
#include <sys/types.h>

/*  XV_DIAG DEBUGGING BITS:
 *   0x01  - normal one-shot debugging.
 *   0x10  - port allocation.
 *   0x20  - gc wrapper debugging.
 *   0x40  - window wrapper debugging.
 *   0x80  - initialization debugging.
 */
/* #define XV_DIAG 0x11  */
#include "xvdebug.h"

#include "X.h"
#include "Xproto.h"
#include "scrnintstr.h"
#include "validate.h"
#include "windowstr.h"
#include "pixmapstr.h"
#include "gcstruct.h"
#include "dixstruct.h"
#ifndef __hpux
#ifndef PSZ
#define PSZ 8
#endif
#include "cfb.h"                    /* 8 PLANE DDX HEADER */
#endif /* !__hpux */

#include "Xv.h"
#include "Xvproto.h"
#include "Xvddx.h"
#include "xvdix.h"

#include "xvddx.h"
#include "xvhw.h"

#ifndef __hpux
#ifdef DECWINDOWS
#include "tfb.h"                    /* 24 PLANE DDX HEADER */
#else
#define tfbPrivGCPtr cfbPrivGCPtr
#define tfbGCPrivateIndex cfbGCPrivateIndex
#endif
#endif

#include <sys/time.h>


#define HALFWAY         ((unsigned long)1<<31)
#define AFTER(ta, tb)   ((tb) - (ta) >=  HALFWAY)
#define BEFORE(ta, tb)  ((ta) - (tb) >=  HALFWAY)

/* FORWARD REFERENCES: */

static void    XvddAdaptorClipNotify();
static void    XvddAdaptorCopyWindow();
static Bool    XvddAdaptorTerminate();
static void    XvddAdaptorUnrealizeWindow();
static void    XvddAdaptorWindowExposures();
static int     XvddAllocatePort();
static void    XvddConstrainVideo();
static void    XvddDetachPort();
static int     XvddFreePort();
static int     XvddGetStill();
static int     XvddGetVideo();
static int     XvddQueryAdaptors();
static int     XvddQueryBestSize();
static int     XvddMissingMethod();
static int     XvddNoopMethod();
static int     XvddPrepareSurface();
static int     XvddPutStill();
static int     XvddPutVideo();
static void    XvddResumeVideo();
static void    XvddStopOverlay();
static void    XvddSuspendVideo();


/*  GRAPHICS CONTEXT WRAPPER FUNCTIONS:
 *
 *  Note that we only need to wrap one GC at a time. For this reason we 
 *  don't use a GC private structure to save the wrap functions. When we 
 *  are finished using a GC for video we want to unwrap it, but we need 
 *  to be in a function in order to unwrap it. So we use ChangeGC with a 
 *  null mask to do the unwrapping.
 */
static void XvddChangeClip();
static void XvddChangeGC();
static void XvddCopyClip();
static void XvddCopyGC();
static void XvddDestroyGC();
static void XvddDestroyClip(); 
static void XvddValidateGC();

GCFuncs  XvddGCFuncs = 
{
  XvddValidateGC,
  XvddChangeGC,
  XvddCopyGC,
  XvddDestroyGC,
  XvddChangeClip,
  XvddDestroyClip,
  XvddCopyClip
};


/*
 *  STORAGE REFERENCES ACROSS MODULES:
 */
static int     n_mono_vids = 0;   /* Number of monochrome visual ids. */
static u_long* mono_vids;        /* Vector of monochrome visual ids. */


#ifdef NOTDEF


/*----------------------------------------------------------------------------*
 |"dump_GC"
 |
 |  Dump a graphics context.
 |
 *----------------------------------------------------------------------------*/
dump_GC( pGC )
        GCPtr pGC;              /* -> graphics context to dump. */
{
	DEBUGF(0x1, (fd, "Graphics context at address %08x\n", pGC));
	DEBUGF(0x1, (fd, "  pScreen           %08x\n", pGC->pScreen));
	DEBUGF(0x1, (fd, "  depth             %d\n",   pGC->depth  ));
	DEBUGF(0x1, (fd, "  alu               %02x\n", pGC->alu    ));
	DEBUGF(0x1, (fd, "  lineWidth         %d\n",   pGC->lineWidth));
	DEBUGF(0x1, (fd, "  dashOffset        %d\n",   pGC->dashOffset));
	DEBUGF(0x1, (fd, "  numInDashList     %d\n",   pGC->numInDashList));
	DEBUGF(0x1, (fd, "  dash              %08x\n", pGC->dash));
	/*.... */
	DEBUGF(0x1, (fd, "  planemask         %08x\n", pGC->planemask));
	DEBUGF(0x1, (fd, "  fgPixel           %08x\n", pGC->fgPixel  ));
	DEBUGF(0x1, (fd, "  bgPixel           %08x\n", pGC->bgPixel  ));
	DEBUGF(0x1, (fd, "  clipOrg           x:%d y:%d\n", 
			pGC->clipOrg.x, pGC->clipOrg.y));
	DEBUGF(0x1, (fd, "  lastWinOrg        x:%d y:%d\n", 
			pGC->lastWinOrg.x, pGC->lastWinOrg.y));
	DEBUGF(0x1, (fd, "  serialNumber      %08x\n", pGC->serialNumber  ));
	DEBUGF(0x1, (fd, "  devPrivates[0]    %08x\n", pGC->devPrivates[0].ptr));
	DEBUGF(0x1, (fd, "  devPrivates[1]    %08x\n", pGC->devPrivates[1].ptr));
}

/*----------------------------------------------------------------------------*
 |"dump_cfbPrivGC"
 |
 |  Dump the contents of a cfb private GC structure given a pointer to it.
 *----------------------------------------------------------------------------*/
dump_CFBGC( pCFBGC )
  cfbPrivGCPtr pCFBGC;
{
	int         i_rect;             /* Index: next rect entry to access. */
	RegionPtr   pCC;                /* Composite clip information. */
	BoxRec      *rect;              /* Pointer to rectangle to dump. */

	DEBUGF(0x1, (fd, "  cfb Private graphics context at address %08x\n", pCFBGC));
	DEBUGF(0x1, (fd, "    rop                   %02x\n",  pCFBGC->rop));
	DEBUGF(0x1, (fd, "    fExpose               %d\n",    pCFBGC->fExpose));
	DEBUGF(0x1, (fd, "    freeCompClip          %d\n",    pCFBGC->freeCompClip));
	DEBUGF(0x1, (fd, "    pRotatedPixmap        %08x\n",  pCFBGC->pRotatedPixmap));
	DEBUGF(0x1, (fd, "    pCompositeClip        %08x\n",  pCFBGC->pCompositeClip));
#ifndef ibm
	DEBUGF(0x1, (fd, "    xor                   %08x\n",  pCFBGC->xor           ));
	DEBUGF(0x1, (fd, "    and                   %08x\n",  pCFBGC->and           ));
#endif  /* !ibm */
	dump_region(pCFBGC->pCompositeClip);
}
        
/*----------------------------------------------------------------------------*
 |"dump_region"
 |
 |  Dump the contents of a cfb private GC structure given a pointer to it.
 *----------------------------------------------------------------------------*/
dump_region( pCC )
        RegionPtr       pCC;            /* Composite clip information. */
{
	int         i_rect;             /* Index: next rect entry to access. */
	BoxRec      *rect;              /* Pointer to rectangle to dump. */

	if ( pCC )
	{
		DEBUGF(0x1, (fd, "    extents.x1          %d\n",    pCC->extents.x1)); 
		DEBUGF(0x1, (fd, "    extents.y1          %d\n",    pCC->extents.y1)); 
		DEBUGF(0x1, (fd, "    extents.x2          %d\n",    pCC->extents.x2)); 
		DEBUGF(0x1, (fd, "    extents.y2          %d\n",    pCC->extents.y2)); 
		DEBUGF(0x1, (fd, "    data->size          %d\n",    REGION_SIZE(pCC)); 
		DEBUGF(0x1, (fd, "    data->numRects      %d\n",    REGION_NUM_RECTS(pCC));
		for ( i_rect=0, rect=REGION_RECTS(pCC); 
				  i_rect<(int)REGION_NUM_RECTS(pCC); i_rect++,rect++ )
		{
			DEBUGF(0x1, (fd, "    data->rects[%2d].x1 %d\n",    i_rect, rect->x1));
			DEBUGF(0x1, (fd, "    data->rects[%2d].y1 %d\n",    i_rect, rect->y1));
			DEBUGF(0x1, (fd, "    data->rects[%2d].x2 %d\n",    i_rect, rect->x2));
			DEBUGF(0x1, (fd, "    data->rects[%2d].y2 %d\n",    i_rect, rect->y2));
		}       
	}
}
        
#endif /* NOTDEF */


/*----------------------------------------------------------------------------* 
 |"XvddAdaptorClipNotify"
 |
 |  Screen transfer vector wrapper routine to indicate a clipping change.
 *----------------------------------------------------------------------------*/
static void
XvddAdaptorClipNotify(pxvAdapt, pWin, dx, dy)
	XvAdaptorPtr	pxvAdapt;	/* -> adaptor accessing. */
	WindowPtr 		pWin;
	int 			dx;
	int				dy;
{

	int			    iPort;		/* Entry in pxvAdapt->pPorts  processing. */
	XvPortPtr       pxvPort;	/* Port currently checking for video restart. */

	DEBUGF(0x40, (fd, "XvddAdaptorClipNotify: entry\n"));

	/*  LOOP THROUGH PORTS ASSOCIATED WITH THIS ADAPTOR SUSPENDING VIDEO
	 *  IF THE WINDOW BEING CLIPPED CONTAINS VIDEO.
	 */
	for ( iPort = 0; iPort < pxvAdapt->nPorts; iPort++ )
	{
		pxvPort = &pxvAdapt->pPorts[iPort];
		if (pxvPort->pDraw == (DrawablePtr)pWin)
		{
			(* pxvAdapt->ddSuspendVideo)(pxvPort);
		}
	}

	DEBUGF(0x40, (fd, "XvddAdaptorClipNotify: exit\n"));
}


/*----------------------------------------------------------------------------*
 |"XvddAdaptorCopyWindow"
 |
 |  Adaptor transfer vector wrapper routine to copy a window.
 *----------------------------------------------------------------------------*/
static void
XvddAdaptorCopyWindow(pxvAdapt, pWin, lastposition, pRegionSrc)
	XvAdaptorPtr	pxvAdapt;	/* -> adaptor checking ports of. */
	WindowPtr 		pWin;
	DDXPointRec     lastposition;
	RegionPtr       pRegionSrc;
{
	int			    iPort;		/* Entry in pxvAdapt->pPorts processing. */
	WindowPtr       pChild;		/* Child of window now examining. */
	XvPortPtr       pxvPort;	/* Port currently checking for video restart. */

	DEBUGF(0x40, (fd, "XvddScreenCopyWindow: entry\n"));


	/*  LOOP THROUGH PORTS ASSOCIATED WITH THIS ADAPTOR SUSPENDING VIDEO
	 *  WHERE NEEDED.
	 */
	for ( pChild = pWin;; )
    {
		if (pChild->viewable)
        {
			for ( iPort = 0; iPort < pxvAdapt->nPorts; iPort++ )
			{
				pxvPort = &pxvAdapt->pPorts[iPort];
				if (pxvPort->pDraw == (DrawablePtr)pChild)
					(* pxvAdapt->ddSuspendVideo)(pxvPort);
			}

			if (pChild->firstChild)
			{
				pChild = pChild->firstChild;
				continue;
			}
        }
		while (!pChild->nextSib && (pChild != pWin))
		{
			pChild = pChild->parent;
		}
		if (pChild == pWin)
			break;
		pChild = pChild->nextSib;
    }

	DEBUGF(0x40, (fd, "XvddAdaptorCopyWindow: exit\n"));
}


/*---------------------------------------------------------------------------*
 |"XvddAdaptorInit"
 |   
 |  Allocate and initialize an Xv adaptor record with generic values.
 |   
 |  Notes:
 |    (1) This routine assumes that the caller is responsible for filling in 
 |        the following adaptor record fields:
 |            a) name
 |    (2) The adaptor record and any port records will be fille in with
 |        default values, so any device-specific substitutions should be
 |        made after this routine is called.
 *---------------------------------------------------------------------------*/
XvAdaptorPtr XvddAdaptorInit(pScreen, nPorts, overlay)
	ScreenPtr   	pScreen;    /* -> screen this adaptor connected to. */
	int				nPorts;		/* =  # of ports on this adpator. */
	int				overlay;	/* =  1 if this is an overlay type card. */
{
	int			   id;			/* Port identification number. */
    int            iDepth;      /* # of depths processed so far. */
    int            iFormat;     /* # of format records processed so far. */
	int			   iPort;		/* # of port record now accessing. */
    int            iVids;       /* # of visual ids processed so far. */
    int            iVisual;     /* # of pScreen visual structure accessing. */
    int            nFormats;    /* # of format records storage allocated for. */
	XvAdaptorPtr   pAdaptor;    /* -> adpator record to be filled in. */
	XvddPortPtr	   pddPort;		/* -> device-dependent port info for pPort. */
    DepthPtr       pDepth;      /* -> depth info in pScreen now accessing. */
    XvEncodingPtr  pEncode;     /* -> encoding record accessing. */
    XvFormatPtr    pFormat;     /* -> format record accessing. */
    XvFormatPtr    pFormats;    /* -> storage allocated for format records. */
	XvPortPtr	   pPort;		/* -> port record now accessing. */
	XvPortPtr	   pPorts;		/* -> port records for this adaptor. */
    unsigned long* pVid;        /* -> visual id accessing (in depth group). */
    VisualPtr      pVisual;     /* -> visual in pScreen currently accessing. */

    DEBUGF(0x80, (fd, "XvAdaptorInit: entry\n"));


	/* (1) ALLOCATE AN ADAPTOR RECORD AND STORAGE FOR ITS PORTS. */

    DEBUGF(0x80,(fd, "Xvdd0AdaptorInit: allocating storage for adaptor\n"));
    if ( !(pAdaptor = (XvAdaptorPtr)xalloc(sizeof(XvAdaptorRec))) )
    {
        FatalError("XvdddaptorInit: couldn't alloc Adaptor struct\n");
    }

    DEBUGF(0x80,(fd, "Xvdd0AdaptorInit: allocating %d ports\n", nPorts));
    pPorts = (XvPortPtr)xalloc(sizeof(XvPortRec)*nPorts);
    if (!pPorts)
    {
        FatalError("Xvdd0AdaptorInit: couldn't alloc Port struct\n");
    }

	
	/*  (2) INITIALIZE THE GENERIC PORT INFORMATION:
     *
     *  We allocate the di and dd port records here and initialize them
     *  with default values. These may be changed by our caller.
	 */
    pPort = pPorts;
    for ( iPort = 0; iPort < nPorts; iPort++, pPort++ )
    {
        DEBUGF(0x80,(fd, "XvddAdaptorInit: intializing port %d\n", iPort));
        id = FakeClientID(0);
        if (!AddResource(id, XvRTPort, pPort))
        {
            FatalError("XvddInitScreen: couldn't add Port resource\n");
        }
        

        /* FILL IN STANDARD PORT INFORMATION: */
        
        pddPort = (XvddPortPtr)xalloc(sizeof(XvddPortRec));
        if (!pddPort)
        {
            FatalError("XvddAdaptorInit: couldn't alloc ddPort struct\n");
        }
        pPort->devPriv.ptr = (pointer)pddPort;
        pPort->id = id;
        pPort->myFuncs = XvddGCFuncs;
        pPort->myPort = iPort;
        pPort->pAdaptor = pAdaptor;
        pPort->pDraw = (DrawablePtr)NULL;
        pPort->pGC = (GCPtr)NULL;
        pPort->pNotify = (XvPortNotifyPtr)NULL;
        pPort->grab.client = (ClientPtr)NULL;
        pPort->serialNumber = 0;
        pPort->time = currentTime;
 
        pddPort->brightness = 0;
        pddPort->contrast = 0;
        pddPort->flip_horizontal = 0;
        pddPort->flip_vertical = 0;
        pddPort->gray = FALSE;
        pddPort->hue = 0;
        pddPort->invert_fields = 0;
        pddPort->saturation = 0;
		pddPort->visible_area.extents.x2 = 0;
	}
	

	/* (3) SET FORMATS SUPPORTED BY THIS ADAPTOR.
     *
	 *     This is based on the depths available for this screen, and
	 *     the depths supported by this adaptor. There are 2 cases:
	 *       a) Frame buffer resident PIP: 24-bit and optionally StaticGray
	 *       b) Overlay (mixer) adaptor: all bits depths
	 */
	iFormat = 0;
	nFormats = 1; 
	pFormats = (XvFormatPtr)xalloc(sizeof(XvFormatRec)*nFormats);
	if (!pFormats)
	{
		FatalError("XvAdaptorInit: couldn't alloc Format struct\n");
	}
	pFormat = pFormats;

	pDepth = pScreen->allowedDepths;
	for (iDepth=0; iDepth<pScreen->numDepths; iDepth++)
	{
		if ( (pDepth->depth == 24) || overlay )
		{
			pVid = pDepth->vids;
			for (iVids=0; iVids<pDepth->numVids; iVids++)
			{
				if (iFormat >= nFormats) 
				{
					nFormats *= 2;
					pFormats = (XvFormatPtr)xrealloc(pFormats, 
						sizeof(XvFormatRec)*nFormats);
					pFormat = &pFormats[iFormat];
				}
			  
				pFormat->depth = pDepth->depth;
				pFormat->visual = *pVid;
				pFormat++;
				iFormat++;
				pVid++;
			}
		}
#ifdef sun
		else if ( (pDepth->depth == 1) )
		{
			mono_vids = pDepth->vids;
			n_mono_vids = pDepth->numVids;
		}
#endif /* sun */
		pDepth++;
	}


	/* FIND STATIC GRAY VISUAL */

	pVisual = pScreen->visuals;
	for (iVisual=0; iVisual<pScreen->numVisuals; iVisual++)
	{
		if (pVisual->class == StaticGray) break;
		pVisual++;
	}

	/* IF WE FOUND ONE THEN FIND MATCHING DEPTH VISUALID */

	if (iVisual < pScreen->numVisuals)
	{
		pDepth = pScreen->allowedDepths;
		for (iDepth=0; iDepth < pScreen->numDepths; iDepth++)
		{
			if (pDepth->depth == 8)
			{
				pVid = pDepth->vids;
				for (iVids=0; iVids<pDepth->numVids; iVids++)
				{
					if (pVisual->vid == *pVid)
					{
						if (iFormat >= nFormats) 
						{
							nFormats *= 2;
							pFormats = (XvFormatPtr) xrealloc(pFormats,
								sizeof(XvFormatRec)*nFormats);
							pFormat = &pFormats[iFormat];
						}
						pFormat->depth = pDepth->depth;
						pFormat->visual = *pVid;
						pFormat++;
						iFormat++;
						break;
					}
					pVid++;
				}
			}
			pDepth++;
		}
	}


	/*  (4) FILL IN ADAPTOR RECORD WITH INFORMATION WE HAVE GENERATED: */
	 
	pAdaptor->base_id                = pPorts->id;
	pAdaptor->ddAllocatePort         = XvddAllocatePort;
	pAdaptor->ddClipNotify           = XvddAdaptorClipNotify;
	pAdaptor->ddControl              = XvddMissingMethod;
	pAdaptor->ddCopyWindow           = XvddAdaptorCopyWindow;
#ifndef _AIX
	pAdaptor->ddDetachPort           = XvddDetachPort;
#endif /* !_AIX */
	pAdaptor->ddFreePort             = XvddFreePort;
#ifndef _AIX
	pAdaptor->ddGetImage 			 = XvddMissingMethod;
#endif /* !_AIX */
	pAdaptor->ddGetLongPortAttribute = XvddMissingMethod;
	pAdaptor->ddGetPortAttribute     = XvddMissingMethod;
	pAdaptor->ddGetStill             = XvddGetStill;
	pAdaptor->ddGetVideo             = XvddGetVideo;
	pAdaptor->ddQueryBestSize        = XvddQueryBestSize;
	pAdaptor->ddPrepareSurface       = XvddPrepareSurface;
#ifndef _AIX
	pAdaptor->ddPutImage 			 = XvddMissingMethod;
#endif /* !_AIX */
	pAdaptor->ddPutStill             = XvddPutStill;
	pAdaptor->ddPutVideo             = XvddPutVideo;
	pAdaptor->ddResumeVideo          = XvddResumeVideo;
#ifndef _AIX
	pAdaptor->ddSaveScreen           = XvddNoopMethod;
#endif /* !_AIX */
	pAdaptor->ddSetLongPortAttribute = XvddMissingMethod;
	pAdaptor->ddSetPortAttribute     = XvddMissingMethod;
	pAdaptor->ddStopVideo            = overlay ? XvddStopOverlay:XvddDetachPort;
	pAdaptor->ddSuspendVideo         = XvddSuspendVideo;
	pAdaptor->ddTerminate            = XvddAdaptorTerminate;
	pAdaptor->ddUnrealizeWindow      = XvddAdaptorUnrealizeWindow;
	pAdaptor->ddWindowExposures      = XvddAdaptorWindowExposures;

	pAdaptor->nFormats               = iFormat;
	pAdaptor->nPorts                 = nPorts;
	pAdaptor->pFormats               = pFormats;
	pAdaptor->pPorts				 = pPorts;
	pAdaptor->pScreen                = pScreen;


	DEBUGF(0x80, (fd, "XvAdaptorInit: exit\n"));

	return pAdaptor;
}


/*----------------------------------------------------------------------------*
 |"XvddAdaptorTerminate"
 |
 |  Transfer vector routine to close down operations and release an adaptor.
 |
 |    = TRUE always
 *----------------------------------------------------------------------------*/
static Bool XvddAdaptorTerminate(pxvAdapt)
	XvAdaptorPtr  pxvAdapt;
{
	int           iPort;
	XvddPortPtr   pddPort;
	XvPortPtr     pxvPort;

	DEBUGF(0x40, (fd, "XvddAdaptorTerminate: entry\n"));

	pxvPort = pxvAdapt->pPorts;
	for (iPort = 0; iPort < pxvAdapt->nPorts; iPort++, pxvPort++)
	{
		if ( pddPort = (XvddPortPtr)pxvPort->devPriv.ptr )
		{
			Xvhw_Terminate(pddPort->phw);
			xfree(pddPort);
		}
	}
	if (pxvAdapt->pPorts) xfree(pxvAdapt->pPorts);
	xfree(pxvAdapt);

	DEBUGF(0x40, (fd, "XvddAdaptorTerminate: exit\n"));

	return TRUE;
}


/*----------------------------------------------------------------------------* 
 |"XvddAdaptorUnrealizeWindow"
 |
 |  Adaptor transfer vector wrapper routine to handle removal of a window
 |  from the screen.
 *----------------------------------------------------------------------------*/
static void XvddAdaptorUnrealizeWindow(pxvAdapt, pWin)
	XvAdaptorPtr	pxvAdapt;	/* -> adaptor checking ports of. */
	WindowPtr 		pWin;		/* -> window being removed from screen. */
{
	int			    iPort;		/* Entry in pxvAdapt->pPorts  processing. */
	XvPortPtr       pxvPort;	/* Port currently checking for video restart. */

	DEBUGF(0x40, (fd, "XvddAdaptorUnrealizeWindow: entry\n"));

	for ( iPort = 0; iPort < pxvAdapt->nPorts; iPort++ )
	{
		pxvPort = &pxvAdapt->pPorts[iPort];
		if (pxvPort->pDraw == (DrawablePtr)pWin)
		{
			XvddDetachPort(0, pxvPort, pxvPort->pDraw);
		}
	}

	DEBUGF(0x40, (fd, "XvddAdaptorUnrealizeWindow: exit\n"));
}


/*----------------------------------------------------------------------------*
 |"XvddAdaptorWindowExposures"
 |
 |  Adaptor transfer vector wrapper routine to handle window exposures.
 |  If the window being exposed is the one in which video is playing restart
 |  video after setting its new geometry. 
 *----------------------------------------------------------------------------*/
static void
XvddAdaptorWindowExposures(pxvAdapt, pWin, pReg, pOtherReg)
	XvAdaptorPtr	pxvAdapt;	/* Adaptor currently checking ports of. */
	WindowPtr 		pWin;
	RegionPtr 		pReg;
	RegionPtr 		pOtherReg;

{
	int			    iPort;		/* Entry in pxvAdapt->pPorts  processing. */
	XvPortPtr       pxvPort;	/* Port currently checking for video restart. */

	DEBUGF(0x40, (fd, "XvddAdaptorWindowExposures: entry\n"));

	for ( iPort = 0; iPort < pxvAdapt->nPorts; iPort++ )
	{
		pxvPort = &pxvAdapt->pPorts[iPort];

		if ((pxvPort->pDraw) && (((WindowPtr)pxvPort->pDraw)->realized))
		{
			if (pxvPort->pDraw->id == pWin->drawable.id
            &&  pxvPort->pDraw->serialNumber != pxvPort->serialNumber)
			{
				(* pxvPort->pAdaptor->ddResumeVideo)(pxvPort);
			}
		}
	}

	DEBUGF(0x40, (fd, "XvddAdaptorWindowExposures: exit\n"));
}


/*----------------------------------------------------------------------------*
 |"XvddAllocatePort"
 |
 |  Transfer vector routine to allocate a port.
 *----------------------------------------------------------------------------*/
static int
XvddAllocatePort(port, pPort, p_pPort)
	unsigned long port;
	XvPortPtr     pPort;
	XvPortPtr     *p_pPort;
{
	*p_pPort = pPort;
	return Success;
}

/*----------------------------------------------------------------------------* 
 |"XvddAttachPort"
 |
 |  Internal routine to actually put video at a specific location on the
 |  screen.
 |  mixing of video). 
 |
 |  This routine is called from XvddGetStill, XvddGetVideo, XvddPutStill,
 |  XvddPutVideo, and XvddResume.
 |
 |  The procedure is to stop the current video, take down the current
 |  video location (if any), set up the new video location, wrap the 
 |  drawable's graphics context, and resume the video.
 |
 |    = 1 on success
 |    = 0 if error detected
 *----------------------------------------------------------------------------*/
int
XvddAttachPort(pDraw, pPort, pGC, direction,
                      vid_x, vid_y, vid_w, vid_h, drw_x, drw_y, drw_w, drw_h)
	DrawablePtr pDraw;
	XvPortPtr   pPort;
	GCPtr       pGC;
	int			direction;		/* = direction video will go. */
	CARD16      vid_h;
	CARD16      vid_w;
	INT16       vid_x;
	INT16       vid_y;
	CARD16      drw_h;
	CARD16      drw_w;
	INT16       drw_x;
	INT16       drw_y;
{
	Xvhw_Drw_Area      drw_area;    /* Destination video area. */
	XvddPortPtr        pddPort;		/* Device-dependent port information. */
	GCPtr              pGCTemp;		/* Save for GC pointer used in GC unwrap. */
	Xvhw_Sig_Area      sig_area;    /* Source signal area. */


	/*  VALIDATE OUR DRAWABLE AND REMEMBER INFORMATION ABOUT THIS ATTACH. */

	if (pDraw->type != DRAWABLE_WINDOW)
	{
		pPort->pDraw = (DrawablePtr)NULL;
		return BadAlloc;
	}
	DEBUGF(0x10, (fd, "XvddAttachPort: entry %x\n", pDraw));
	pPort->direction = direction;
	pPort->pDraw = pDraw;
	pPort->serialNumber = pDraw->serialNumber;	


	/* CONSTRAIN THE REGIONS TO THOSE THAT ARE SUPPORTED; MAINTAIN ASPECT
     * RATIO WHEN POSSIBLE 
	 */
	XvddConstrainVideo(pDraw, 
					  (direction==XvImageToDrawable) ? 0 : pPort->pEncoding, 
                      vid_x, vid_y, vid_w, vid_h,
                      drw_x, drw_y, drw_w, drw_h,
                      &vid_x, &vid_y, &vid_w, &vid_h,
                      &drw_x, &drw_y, &drw_w, &drw_h);


	/* UNWRAP THE OLD GC PROCEDURES IF NECESSARY */

	if ((pPort->pGC) && (pGC != pPort->pGC))
	{
		pGCTemp = pPort->pGC;
		pPort->pGC = (GCPtr)NULL;
		ChangeGC(pGCTemp, 0, 0); /* USE CHANGE GC TO UNWRAP GC */
	}


	/*  SET NEW DIGITIZER GEOMETRY AND PREPARE SURFACE FOR VIDEO.
	 *
	 *  Note that we must set the signal and drawable areas up
	 *  before preparing the surface.
	 */
	pddPort = (XvddPortPtr)pPort->devPriv.ptr;
	Xvhw_Suspend(pddPort->phw, 1);	

	sig_area.left = vid_x;           
	sig_area.top = vid_y;
	sig_area.width = vid_w;
	sig_area.height = vid_h;
	Xvhw_Set_Signal_Area(pddPort->phw, &sig_area);

	Xvhw_Get_Drawable_Area(pddPort->phw, &drw_area); 
	drw_area.direction = direction;
	drw_area.left = pDraw->x+drw_x;
	drw_area.top = pDraw->y+drw_y;
	drw_area.width = drw_w;
	drw_area.height = drw_h;
	Xvhw_Set_Drawable_Area(pddPort->phw, &drw_area);

	/*  WRAP THE NEW GC PROCEDURES IF NECESSARY.  */

	if (pGC != pPort->pGC)
	{
		pPort->pGC = pGC;
		pPort->wrapFuncs = pGC->funcs;
		pGC->funcs = (GCFuncs *)pPort;
	}

	(* pPort->pAdaptor->ddPrepareSurface)(pPort, XvPSenable);

	Xvhw_Resume(pddPort->phw, 0);

	DEBUGF(0x10, (fd, "XvddAttachPort: exit %x\n", pDraw));
	return Success;
}


/*----------------------------------------------------------------------------*
 |"XvddChangeClip"
 |
 *----------------------------------------------------------------------------*/
static void
XvddChangeClip(pGC, type, pvalue, nrects)
    GCPtr       pGC;
    int         type;
    pointer     pvalue;
    int         nrects;
{
	XvPortPtr pPort;

	DEBUGF(0x20, (fd, "XvddChangeClip: entry\n"));

	pPort = (XvPortPtr)pGC->funcs;

	pGC->funcs = pPort->wrapFuncs;

	(* pGC->funcs->ChangeClip)(pGC, type, pvalue, nrects);

	pGC->funcs = (GCFuncs *)pPort;

	DEBUGF(0x20, (fd, "XvddChangeClip: exit\n"));
}

/*----------------------------------------------------------------------------*
 |"XvddChangeGC"
 | 
 |  GC transfer vector wrapper routine to change a graphics context.
 *----------------------------------------------------------------------------*/
static void
XvddChangeGC(pGC, mask)
    GCPtr   pGC;
    unsigned long   mask;
{
	XvPortPtr pPort;

	DEBUGF(0x20, (fd, "XvddChangeGC: entry\n"));

	pPort = (XvPortPtr)pGC->funcs;

	pGC->funcs = pPort->wrapFuncs;

	(*pGC->funcs->ChangeGC) (pGC, mask);

	/*  CHECK TO SEE IF GC IS STILL BEING USED FOR VIDEO; IF NOT THEN
     *  LEAVE IT UNWRAPPED 
	 */
	if (pPort->pGC == pGC)
    {
		pGC->funcs = (GCFuncs *)pPort;
    }

	DEBUGF(0x20, (fd, "XvddChangeGC: exit\n"));
}


/*----------------------------------------------------------------------------*  |
 *----------------------------------------------------------------------------*/
static void
XvddCopyClip(pgcDst, pgcSrc)
    GCPtr pgcDst, pgcSrc;
{
	XvPortPtr pPort;

	DEBUGF(0x20, (fd, "XvddCopyClip: entry\n"));

	pPort = (XvPortPtr)pgcDst->funcs;

	pgcDst->funcs = pPort->wrapFuncs;

	(* pgcDst->funcs->CopyClip)(pgcDst, pgcSrc);

	pgcDst->funcs = (GCFuncs *)pPort;

	DEBUGF(0x20, (fd, "XvddCopyClip: exit\n"));
}

/*----------------------------------------------------------------------------*
 | "XvddCopyGC"
 | 
 |  GC transfer vector wrapper routine to copy a graphics context.
 *----------------------------------------------------------------------------*/
static void
XvddCopyGC (pgcSrc, mask, pgcDst)
    GCPtr   pgcSrc, pgcDst;
    unsigned long   mask;
{
	XvPortPtr pPort;

	DEBUGF(0x20, (fd, "XvddCopyGC: entry\n"));

	pPort = (XvPortPtr)pgcDst->funcs;

	pgcDst->funcs = pPort->wrapFuncs;

	(*pgcDst->funcs->CopyGC) (pgcSrc, mask, pgcDst);

	pgcDst->funcs = (GCFuncs *)pPort;

	DEBUGF(0x20, (fd, "XvddCopyGC: exit\n"));
}


/*----------------------------------------------------------------------------*
 |"XvddConstrainVideo"
 |
 |  Internal routine to constrain a video specification to the limits of the
 |  device being used.
 *----------------------------------------------------------------------------*/
static void
XvddConstrainVideo(pWin, pEncode, 
                   vid_x, vid_y, vid_w, vid_h,
                   drw_x, drw_y, drw_w, drw_h,
                   p_vid_x, p_vid_y, p_vid_w, p_vid_h,
                   p_drw_x, p_drw_y, p_drw_w, p_drw_h)
 
    WindowPtr       pWin;        /* -> window video is to appear in. */
    XvEncodingPtr   pEncode;     /* -> current encoding information. */
    INT16           vid_x;       /* =  video signal x left. */
    INT16           vid_y;       /* =  video signal y top. */
    INT16           vid_w;       /* =  video signal width. */
    INT16           vid_h;       /* =  video signal height. */
    INT16           drw_x;       /* =  window x left. */
    INT16           drw_y;       /* =  window y top. */
    INT16           drw_w;       /* =  window height. */
    INT16           drw_h;       /* =  window width. */
    INT16           *p_vid_x;    /* -> where to put constrained signal left. */
    INT16           *p_vid_y;    /* -> where to put constrained signal top. */
    INT16           *p_vid_w;    /* -> where to put constrained signal width. */
    INT16           *p_vid_h;    /* -> where to put constrained signal height.*/
    INT16           *p_drw_x;    /* -> where to put constrained window left. */
    INT16           *p_drw_y;    /* -> where to put constrained window top. */
    INT16           *p_drw_w;    /* -> where to put constrained window width. */
    INT16           *p_drw_h;    /* -> where to put constrained window height.*/
{ 
    INT16       delta_x;        /* Amount x is out of bounds on left. */
    INT16       delta_y;        /* Amount y is out of bounds on top. */
    INT16       new_drw_width;  /* New width for window when out of bounds. */
    INT16       new_drw_height; /* New height for window when out of bounds. */
    INT16       new_sig_height; /* New height for signal when out of bounds. */
    INT16       new_sig_width;  /* New width for signal when out of bounds. */
	int			scr_h;			/* Screen width. */
	int			scr_w;			/* Screen height. */
	int			win_x;			/* X offset on screen of window. */
	int			win_y;			/* X offset on screen of window. */

	win_x = pWin->drawable.x;
	win_y = pWin->drawable.y;
	scr_w = pWin->drawable.pScreen->width;
	scr_h = pWin->drawable.pScreen->height;

    /*
     *  USE THE ARTIFACE OF A LOOP TO ALLOW EXIT TO A NULL REGION FOR VIDEO.
     */
    for (;;)
    {
        /*
         *  IF SIGNAL X,Y ARE TOO LARGE GENERATE A NULL REGION FOR VIDEO.
         */
        if( vid_w <= 0 || vid_h <= 0 )
        {
            break;
        }

		if ( pEncode )
		{
			if( vid_x > pEncode->x_max || vid_y > pEncode->y_max )
			{
				break;
			}

			/*
			 *  CLIP LOWER BOUNDS OF SIGNAL (SOURCE) REGION, ADJUST WINDOW ALSO.
			 */
			if ( vid_x < pEncode->x_min )
			{
				delta_x = pEncode->x_min - vid_x;
				new_drw_width = ( drw_w * (vid_w + delta_x) ) / vid_w;
				drw_x += drw_w - new_drw_width;
				drw_w = new_drw_width;
				vid_x = pEncode->x_min;
				vid_w -= delta_x;
				if ( drw_w <= 0 || vid_w <= 0 ) break;
			}

			if ( vid_y < pEncode->y_min )
			{
				delta_y = pEncode->y_min - vid_y;
				new_drw_height = ( drw_h * (vid_h + delta_y) ) / vid_h;
				drw_y += drw_h - new_drw_height;
				drw_h = new_drw_height;
				vid_y = pEncode->y_min;
				vid_h -= delta_y;
				if ( drw_h <= 0 || vid_h <= 0 ) break;
			}

			/* 
			 *  CLIP UPPER BOUNDS OF SIGNAL (SOURCE) REGION, ADJUST WINDOW ALSO.
			 */
			if ( vid_x + vid_w > pEncode->x_max )
			{
				drw_w = (drw_w*(pEncode->x_max - vid_x)) / vid_w;
				vid_w = pEncode->x_max - vid_x;
				if ( drw_w <= 0 || vid_w <= 0 ) break;
			}

			if ( vid_y + vid_h > pEncode->y_max )
			{
				drw_h = (drw_h*(pEncode->y_max - vid_y)) / vid_h;
				vid_h = pEncode->y_max - vid_y;
				if ( drw_h <= 0 || vid_h <= 0 ) break;
			}
		}

        /*
         *  CLIP WINDOW (DESTINATION) REGION TO LOWER BOUNDS OF THE SCREEN.
         */
        if ( win_x + drw_x < 0 )
        {
            new_drw_width = drw_w + (win_x + drw_x);
            new_sig_width = ( new_drw_width * vid_w ) / drw_w;
            vid_x += vid_w- new_sig_width;
            vid_w = new_sig_width;
            drw_x = -win_x;
            drw_w = new_drw_width;
            if ( drw_w <=0 || vid_w <= 0 ) break;
        }

        if ( win_y + drw_y < 0 )
        {
            if ( drw_h <= 0 ) break;
            new_drw_height = drw_h + (win_y + drw_y);
            new_sig_height = (new_drw_height * vid_h) / drw_h;
            vid_y += vid_h - new_sig_height;
            vid_h = new_sig_height;
            drw_y = - win_y;
            drw_h = new_drw_height;
            if ( drw_h <= 0 || vid_h <= 0 ) break;
        }

        /*
         *  CLIP WINDOW (DESTINATION) REGION TO UPPER BOUNDS OF THE SCREEN.
         */
        if ( win_x + drw_x + drw_w > scr_w )
        {
            new_drw_width = scr_w - win_x - drw_x;
            vid_w = ( vid_w * new_drw_width ) / drw_w;
            drw_w = new_drw_width;
            if ( drw_w <= 0 || vid_w <= 0 ) break;
        }

        if ( win_y + drw_y + drw_h > scr_h )
        {
            new_drw_height = scr_h - win_y - drw_y;
            vid_h = ( vid_h * new_drw_height ) / drw_h;
            drw_h = new_drw_height;
            if ( drw_h <= 0 || vid_h <= 0 ) break;
        }

        /*
         *  CHECK DESTINATION SIZE AGAINST SOURCE VIDEO SIZE.
         */
        if ( drw_w > vid_w )
        {
            drw_h = ( drw_h * vid_w ) / drw_w;
            drw_w = vid_w;
            if ( drw_w <= 0 || vid_w <= 0 ) break;
        }

        if ( drw_h > vid_h )
        {
            drw_w = ( drw_w * vid_h ) / drw_h;
            drw_h = vid_h;
            if ( drw_h <= 0 || vid_h <= 0 ) break;
        }

		*p_vid_x = vid_x;
		*p_vid_y = vid_y;
		*p_vid_w = vid_w;
		*p_vid_h = vid_h;
		*p_drw_x = drw_x;
		*p_drw_y = drw_y;
		*p_drw_w = drw_w;
		*p_drw_h = drw_h;
        
        return;
    }

    /*
     *  IF WE GET HERE WE FELL OUT OF THE LOOP ABOVE, GENERATE A NULL REGION.
     */
    *p_vid_x = 0;
    *p_vid_y = 0;
    *p_vid_w = 0;
    *p_vid_h = 0;
    *p_drw_x = 0;
    *p_drw_y = 0;
    *p_drw_w = 0;
    *p_drw_h = 0;
}

/*----------------------------------------------------------------------------*
 |"XvddDestroyClip"
 |
 |  GC transfer vector wrapper routine to destroy a clipping region.
 *----------------------------------------------------------------------------*/
static void
XvddDestroyClip(pGC)
    GCPtr       pGC;
{
	XvPortPtr pPort;

	DEBUGF(0x20, (fd, "XvddDestroyClip: entry\n"));

	pPort = (XvPortPtr)pGC->funcs;

	pGC->funcs = pPort->wrapFuncs;

	(* pGC->funcs->DestroyClip)(pGC);

	pGC->funcs = (GCFuncs *)pPort;

	DEBUGF(0x20, (fd, "XvddDestroyClip: exit\n"));
}

/*----------------------------------------------------------------------------*
 |"XvddDestroyGC"
 |
 |  GC transfer vector wrapper routine to destroy a graphics context.
 *----------------------------------------------------------------------------*/
static void
XvddDestroyGC(pGC)
	GCPtr pGC;
{

	Bool status;
	XvPortPtr pPort;
  
	DEBUGF(0x20, (fd, "XvddDestroyGC: entry\n"));

	pPort = (XvPortPtr)pGC->funcs;

	/* JUST CHECKING */

	if (pPort->pGC == pGC)
    {
		pPort->pGC = (GCPtr)NULL;
    }

	pGC->funcs = pPort->wrapFuncs;
	(* pGC->funcs->DestroyGC)(pGC);

	/* DON'T REWRAP THE FUNCS */

	DEBUGF(0x20, (fd, "XvddDestroyGC: exit\n"));

}


/*----------------------------------------------------------------------------*
 |"XvddDetachPort"
 |
 |  Routine to detach a port from a drawable. Note the calling sequence
 |  matches that for the ddStopVideo transfer vector routine, but neither 
 |  the client nor the pDraw parameters are guaranteed to be valid.
 *----------------------------------------------------------------------------*/
static void XvddDetachPort(client, pPort, pDraw)
    ClientPtr   client;
    XvPortPtr   pPort;
    DrawablePtr pDraw;
{
	XvddPortPtr pddPort;	/* Device-dependent storage for this port. */
	GCPtr       pGCTemp;	/* Temporary GC to unwrap with. */

    DEBUGF(0x10, (fd, "XvddDetachPort: entry, port %d win %x\n",
                        pPort->id, pDraw));
	pddPort = (XvddPortPtr)pPort->devPriv.ptr;

	Xvhw_Off(pddPort->phw, 1);
	if ( pPort->direction != XvDrawableToSignal )
	{
		(* pPort->pAdaptor->ddPrepareSurface)(pPort, XvPSdisable);
	}

	if (pPort->pGC)
	{
		pGCTemp = pPort->pGC;
		pPort->pGC = (GCPtr)NULL;
		ChangeGC(pGCTemp, 0, 0); /* USE CHANGE GC TO UNWRAP GC */
	}

    pPort->pDraw = (DrawablePtr)NULL;
    pPort->client = (ClientPtr)client;
    pPort->time = currentTime;

    DEBUGF(0x10, (fd, "XvddDetachPort: exit, port %d win %x\n",
                        pPort->id, pDraw));
}


/*----------------------------------------------------------------------------*
 |"XvddFreePort"
 |
 |  Transfer vector routine to free a port.
 *----------------------------------------------------------------------------*/
static int
XvddFreePort(pPort)
	XvPortPtr pPort;
{
    DEBUGF(0x10, (fd, "XvddFreePort: entry, port %d\n",
                        pPort->id));
    XvddDetachPort((ClientPtr)NULL, pPort, pPort->pDraw);
    DEBUGF(0x10, (fd, "XvddFreePort: exit, port %d\n",
                        pPort->id));
	return Success;
}


/*----------------------------------------------------------------------------*
 |"XvddGetStill"
 |
 |  Transfer vector routine to generate a still video frame from a region of
 |  a drawable.
 *----------------------------------------------------------------------------*/
static int
XvddGetStill(client, pDraw, pPort, pGC, vid_x, vid_y, vid_w, vid_h,
              drw_x, drw_y, drw_w, drw_h)
	ClientPtr   client;
	DrawablePtr pDraw;
	XvPortPtr   pPort;
	GCPtr       pGC;
	INT16       vid_x;
    INT16       vid_y;
	CARD16      vid_w;
	CARD16      vid_h;
    INT16       drw_x;
    INT16       drw_y;
    CARD16      drw_w;
    CARD16      drw_h;

{
	
	XvddPortPtr pddPort;	/* Device-dependent storage for this port. */
	int			saveDir;	/* Save of pPort->direction so can restore it. */
	DrawablePtr	saveDraw;	/* Save of pPort->pDraw so can restore it. */
	GCPtr	 	saveGC;	  	/* Save of pPort->pGC so can restore it. */


	if (pDraw->type != DRAWABLE_WINDOW)
    {
		return BadAlloc;
    }
	pddPort = (XvddPortPtr)pPort->devPriv.ptr;


	/*  SAVE INFORMATION ABOUT THE PORT'S CURRENT STATE SO WE CAN RESTORE IT. */

	saveDir = pPort->direction;
	saveDraw = pPort->pDraw;
	saveGC = pPort->pGC;


	/* TURN PIP OFF SO WE CAN CHANGE PIP PARAMETERS AND GET A CLEAN PICTURE */

	Xvhw_Suspend(pddPort->phw, 1);


	/* SET LOCATION OF STILL OPERATION AND PERFORM A ONE-SHOT. */

	XvddAttachPort(pDraw, pPort, pGC, XvDrawableToSignal,
	   vid_x, vid_y, vid_w, vid_h, drw_x, drw_y, drw_w, drw_h);

	if ( !Xvhw_One_Shot(pddPort->phw) )
    {
		ERRORF((fd, "GetStill error 2\n"));
		XvdiSendVideoNotify(pPort, pDraw, XvHardError);
    }


	/* RESTART VIDEO IN ORIGINAL AREA IF WE STOPPED IT */


	if (pGC != saveGC )
    {
		pPort->serialNumber = NEXT_SERIAL_NUMBER;
    }

	if ( saveDraw )
    {
		/* IF THE GC HAS BEEN DESTROYED THEN JUST PREEMPT THE VIDEO */

		if ( !saveGC )
        {
			ERRORF((fd, "GetStill error 3\n"));
			XvdiSendVideoNotify(pPort, pDraw, XvPreempted);
			return Success;
        }

		if ( saveGC->serialNumber != saveDraw->serialNumber)
			ValidateGC( saveDraw, saveGC );

		XvddAttachPort(saveDraw, pPort, saveGC, saveDir,
		   pddPort->vx, pddPort->vy, pddPort->vw, pddPort->vh,
		   pddPort->dx, pddPort->dy, pddPort->dw, pddPort->dh);
		Xvhw_Resume(pddPort->phw, 0); /* Because of one-shot. */
    }

	Xvhw_Resume(pddPort->phw, 0); /* To match Xvhw_Suspend. */

	return Success;

}


/*----------------------------------------------------------------------------*
 |"XvddGetVideo"
 |
 |  Transfer vector routine to generate video from a region of a drawable.
 *----------------------------------------------------------------------------*/
static int
XvddGetVideo(client, pDraw, pPort, pGC, vid_x, vid_y, vid_w, vid_h,
              drw_x, drw_y, drw_w, drw_h)
	ClientPtr   client;
   	DrawablePtr pDraw;
    GCPtr       pGC;
    XvPortPtr   pPort;
    CARD16      vid_h;
    CARD16      vid_w;
    INT16       vid_x;
    INT16       vid_y;
    CARD16      drw_h;
    CARD16      drw_w;
    INT16       drw_x;
    INT16       drw_y;
{
	XvddPortPtr        pddPort;     /* Device-dependent port information. */


	/* VERIFY WE HAVE A DRAWABLE AND THAT GET VIDEO IS ALLOWED. */

    if (pDraw->type != DRAWABLE_WINDOW)
    {
        pPort->pDraw = (DrawablePtr)NULL;
        return BadAlloc;
    }
    pddPort = (XvddPortPtr)pPort->devPriv.ptr;


	/* SAVE PARAMETERS IN DD STORAGE, WE'LL NEED IT FOR RESTARTS. */
 
    pddPort->vx = vid_x;
    pddPort->vy = vid_y;
    pddPort->vw = vid_w;
    pddPort->vh = vid_h;
    pddPort->dx = drw_x;
    pddPort->dy = drw_y;
    pddPort->dw = drw_w;
    pddPort->dh = drw_h;
 

	/* POSITION VIDEO TO REQUESTED LOCATION. */

	XvddAttachPort(pDraw, pPort, pGC, XvDrawableToSignal,
		 vid_x, vid_y, vid_w, vid_h,
		 drw_x, drw_y, drw_w, drw_h);


	/* FORCE VIDEO ON. */

	Xvhw_On(pddPort->phw, 1);

    return Success;
}


/*---------------------------------------------------------------------------*
 |"XvddMissingMethod"
 |
 |  This routine should never get called. If it does the device-dependent
 |  level did not assign a routine for some method. Return a BadMatch.
 |
 |    = BadMatch
 *---------------------------------------------------------------------------*/
static int XvddMissingMethod()
{
    ERRORF((fd, "*** Missing method called\n"));
    return BadMatch;
}


/*---------------------------------------------------------------------------*
 |"XvddNoopMethod"
 |
 |  This routine will be called for optional methods which an adaptor does 
 |  not assign a routine for. Always return Success.
 |
 |    = Success
 *---------------------------------------------------------------------------*/
static int XvddNoopMethod()
{
    return Success;
}


/*----------------------------------------------------------------------------* 
 |"XvddPrepareSurface"
 |
 |  Internal routine to prepare the frame buffer to receive video for an
 |  adaptor which is separate from the frame buffer (overlay/mixing of video).
 |
 |  This routine is called from XvddAttachPort.
 |
 |
 |    = 1 on success
 |    = 0 if error detected
 *----------------------------------------------------------------------------*/
static int XvddPrepareSurface(pPort, operation)
    XvPortPtr   pPort;          /* -> port to prepare surface for. */   
    int         operation;      /* =  operation requested (e.g., XvPSenable). */
{
	Xvhw_Drw_Area      drw_area;    /* Destination video area. */
	int                fill_h;		/* Height of enable memory to fill. */
	int                fill_w;		/* Width of enable memory to fill. */
	int                fill_x;		/* Starting x of enable memory to fill. */
	int                fill_y;		/* Starting y of enable memory to fill. */
	int                iBox;		/* # of box processing in region list. */
	int                nBoxes;		/* # of boxes in region list. */
	BoxPtr             pBox;		/* Box now processing in region list. */
	RegionPtr          pClip;		/* pDraw's current clipping region. */
	DrawablePtr		   pDraw;		/* Drawable associated with this port. */
	XvddPortPtr        pddPort;		/* Device-dependent port information. */
	RegionPtr          pDisable;    /* Pointer to regDisable or 0. */          
	RegionPtr   	   pSignal;     /* Pointer to 'regSignal'. */
	RegionPtr   	   pVideo;      /* Region on screen where video is video. */
	RegionRec          regDisable;  /* Area to have video disabled. */
	static RegionRec   regSignal =  /* Current signal area on screen. */
		{{0, 0, 0, 0}, 0};
	RegionRec		   regVideo;	/* Current area of video visibility. */
	Xvhw_Sig_Area      sig_area;    /* Source signal area. */
	short              xy[2];       /* New mix position for Xvhw_Control. */

	pddPort = (XvddPortPtr)pPort->devPriv.ptr;
	pDraw = pPort->pDraw;
    Xvhw_Get_Drawable_Area(pddPort->phw, &drw_area);


    /*  IF THIS IS A DISABLE REQUEST JUST CLEAR THE AREA CURRENTLY ENABLED. */

    if ( (operation==XvPSdisable) || (drw_area.direction==XvDrawableToSignal) )
    {
		pDisable = &pddPort->visible_area;
		if ( pDisable->extents.x2 != 0 )
		{
			pBox = REGION_RECTS(pDisable);
			nBoxes = (int)REGION_NUM_RECTS(pDisable);
			for (iBox = 0; iBox < nBoxes; iBox++, pBox++)
			{
		  
				fill_x = pBox->x1;
				fill_y = pBox->y1;
				fill_w = (pBox->x2 - fill_x) + 1;
				fill_h = (pBox->y2 - fill_y) + 1;
				Xvhw_Fill_Video_Enable(pddPort->phw, 
					fill_x,fill_y,fill_w,fill_h, 0x00);
			  
			}
			(* pDraw->pScreen->RegionUninit)(pDisable);
			pDisable->extents.x2 = 0;
		}
        return 1;
    }


    /*  PICK UP THE COMPOSITE CLIP LIST FOR THE REGION TO RECEIVE VIDEO.
     *   
     *  We do this based on the depth of the visual (i.e., its cfb or tc).
     */
#ifdef __hpux
	pClip = &((WindowPtr)pDraw)->clipList;
#else  /* !__hpux */
	pClip = (pDraw->depth == 8) ? 
		((cfbPrivGCPtr)
		    pPort->pGC->devPrivates[cfbGCPrivateIndex].ptr)->pCompositeClip :
		((tfbPrivGCPtr)
			pPort->pGC->devPrivates[tfbGCPrivateIndex].ptr)->pCompositeClip;
#endif /* __hpux */


	/*  CALCULATE AREA OF SCREEN WHICH WILL RECIEVE THE VIDEO SIGNAL. */
	 
    Xvhw_Get_Signal_Area(pddPort->phw, &sig_area);
	pSignal = &regSignal;
	pVideo = &regVideo;
    pSignal->extents.x1 = drw_area.left;
    pSignal->extents.y1 = drw_area.top;
    pSignal->extents.x2 = pSignal->extents.x1 +
        ((drw_area.width < sig_area.width) ? drw_area.width : sig_area.width);
    pSignal->extents.y2 = pSignal->extents.y1 +
        ((drw_area.height < sig_area.height) ?
        drw_area.height : sig_area.height);
	(* pDraw->pScreen->RegionInit)(pVideo, NullBox, 1);
	(* pDraw->pScreen->Intersect)(pVideo, pSignal, pClip);
	 

	/*  IF THERE IS CURRENTLY VIDEO BEING DISPLAYED ON THIS PORT DISABLE IT.
     *
     *  We do this by calculating the areas of currently enabled video which
     *  lie outside the new region of visible video.
     */
	if ( pddPort->visible_area.extents.x2 != 0 )
	{
		pDisable = &regDisable;
		(* pDraw->pScreen->RegionInit)(pDisable, NullBox, 1);
		(* pDraw->pScreen->Subtract)(pDisable, &pddPort->visible_area, pVideo);

		xy[0] = drw_area.left;
		xy[1] = drw_area.top;
		Xvhw_Control(pddPort->phw, "MIX_POSITION", xy);

		pBox = REGION_RECTS(pDisable);
		nBoxes = (int)REGION_NUM_RECTS(pDisable);
		for (iBox = 0; iBox < nBoxes; iBox++, pBox++)
		{
      
			fill_x = pBox->x1;
			fill_y = pBox->y1;
			fill_w = (pBox->x2 - fill_x) + 1;
			fill_h = (pBox->y2 - fill_y) + 1;
			Xvhw_Fill_Video_Enable(pddPort->phw, fill_x,fill_y,fill_w,fill_h,0);
		}
		(* pDraw->pScreen->RegionUninit)(pDisable);
		(* pDraw->pScreen->RegionUninit)(&pddPort->visible_area);
		pddPort->visible_area.extents.x2 = 0;
	}


	/*  ENABLE THE REGION WHICH WILL HAVE VISIBLE VIDEO. */

	pddPort->visible_area = *pVideo; 
	nBoxes = (int)REGION_NUM_RECTS(pVideo);
	pBox = REGION_RECTS(pVideo);         
	for ( iBox = 0; iBox < nBoxes; iBox++, pBox++ )
	{
  
		fill_x = pBox->x1;
		fill_y = pBox->y1;
		fill_w = (pBox->x2 - fill_x) + 1;
		fill_h = (pBox->y2 - fill_y) + 1;
		Xvhw_Fill_Video_Enable(pddPort->phw, fill_x,fill_y,fill_w,fill_h,0xff);
	}

	return Success;
}


/*----------------------------------------------------------------------------* 
 |"XvddPutStill"
 |
 |  Transfer vector routine to place a single frame of video in a drawable.
 |    = 1 if success 
 |    = 0 if error detected
 *----------------------------------------------------------------------------*/
static int
XvddPutStill(client, pDraw, pPort, pGC, vid_x, vid_y, vid_w, vid_h,
               drw_x, drw_y, drw_w, drw_h)
	ClientPtr   client;
	DrawablePtr pDraw;
	XvPortPtr   pPort;
	GCPtr       pGC;
	INT16       vid_x;
    INT16       vid_y;
	CARD16      vid_w;
	CARD16	    vid_h;
    INT16       drw_x;
    INT16       drw_y;
	CARD16	    drw_w;
	CARD16	    drw_h;
{
	XvddPortPtr pddPort;	/* Device-dependent storage for this port. */
	int			saveDir;	/* Save of pPort->direction so can restore it. */
	DrawablePtr	saveDraw;	/* Save of pPort->pDraw so can restore it. */
	GCPtr	 	saveGC;	  	/* Save of pPort->pGC so can restore it. */


	if (pDraw->type != DRAWABLE_WINDOW)
    {
		return BadAlloc;
    }
	pddPort = (XvddPortPtr)pPort->devPriv.ptr;


	/*  SAVE INFORMATION ABOUT THE PORT'S CURRENT STATE SO WE CAN RESTORE IT. */

	saveDir = pPort->direction;
	saveDraw = pPort->pDraw;
	saveGC = pPort->pGC;


	/* SET LOCATION OF STILL OPERATION AND PERFORM A ONE-SHOT. */

	Xvhw_Suspend(pddPort->phw, 1);
	XvddAttachPort(pDraw, pPort, pGC, XvSignalToDrawable,
	   vid_x, vid_y, vid_w, vid_h, drw_x, drw_y, drw_w, drw_h);

#ifdef DEC
	vis = ((WindowPtr)pDraw)->visibility;
	Xvhw_Control(pddPort->phw, "PRIORITY", (vis==VisibilityUnobscured)? 0:256);
#endif

	if ( !Xvhw_One_Shot(pddPort->phw) )
    {
		ERRORF((fd, "PutStill error 2\n"));
		XvdiSendVideoNotify(pPort, pDraw, XvHardError);
    }


	/* RESTART VIDEO IN ORIGINAL AREA IF WE STOPPED IT */


	if (pGC != saveGC )
    {
		pPort->serialNumber = NEXT_SERIAL_NUMBER;
    }

	if ( saveDraw )
    {
		if ( !saveGC )	/* GC DESTROYED!!!  ACT AS IF PRE-EMPTED */
        {
			ERRORF((fd, "PutStill error 3\n"));
			XvdiSendVideoNotify(pPort, pDraw, XvPreempted);
			return Success;
        }

		if ( saveGC->serialNumber != saveDraw->serialNumber)
			ValidateGC( saveDraw, saveGC );

		XvddAttachPort(saveDraw, pPort, saveGC, saveDir,
		   pddPort->vx, pddPort->vy, pddPort->vw, pddPort->vh,
		   pddPort->dx, pddPort->dy, pddPort->dw, pddPort->dh);
		Xvhw_Resume(pddPort->phw, 0); /* Because of one-shot. */
    }

	Xvhw_Resume(pddPort->phw, 0); /* To match Xvhw_Suspend. */

	return Success;

}


/*----------------------------------------------------------------------------* 
 |"XvddPutVideo"
 |
 |  Transfer vector routine to set new video location on screen.
 |  Calls XvddAttachPort to do the actual work.
 |
 |    = 1 if success
 |    = 0 if failure
 *----------------------------------------------------------------------------*/
static int
XvddPutVideo(client, pDraw, pPort, pGC, vid_x, vid_y, vid_w, vid_h,
               drw_x, drw_y, drw_w, drw_h)
	ClientPtr   client;
	DrawablePtr pDraw;
	XvPortPtr   pPort;
	GCPtr       pGC;
	INT16       vid_x;
	INT16       vid_y;
	CARD16      vid_w;
	CARD16      vid_h;
	INT16       drw_x;
	INT16       drw_y;
	CARD16      drw_w;
	CARD16      drw_h;
{
	XvddPortPtr pddPort;          /* Pointer to Xv instance we are using. */

	DEBUGF(1, (fd, "XvddPutVideo entry\n"));
	if (pDraw->type != DRAWABLE_WINDOW)
	{
		pPort->pDraw = (DrawablePtr)NULL;
		return BadAlloc;
	}
	pddPort = (XvddPortPtr)pPort->devPriv.ptr;


   /* SAVE ORIGINAL WINDOW GEOMETRY REQUESTED. */

    pddPort->vx = vid_x;
    pddPort->vy = vid_y;
    pddPort->vw = vid_w;
    pddPort->vh = vid_h;
    pddPort->dx = drw_x;
    pddPort->dy = drw_y;
    pddPort->dw = drw_w;
    pddPort->dh = drw_h;


	/* POSITION VIDEO TO REQUESTED LOCATION. */

	DEBUGF(1, (fd, "XvddPutVideo before attach\n"));
	XvddAttachPort(pDraw, pPort, pGC, XvSignalToDrawable,
		 vid_x, vid_y, vid_w, vid_h,
		 drw_x, drw_y, drw_w, drw_h);


	/* FORCE VIDEO ON. */

	DEBUGF(1, (fd, "XvddPutVideo before on\n"));
	Xvhw_On(pddPort->phw, 1);

	DEBUGF(1, (fd, "XvddPutVideo exit\n"));
	return Success;
}


/*----------------------------------------------------------------------------*
 |"XvddQueryBestSize"
 |
 |  Transfer vector routine to query the best size for a video window.
 *----------------------------------------------------------------------------*/
static int
XvddQueryBestSize(client, pPort, motion, vid_w, vid_h, drw_w, drw_h, p_w, p_h)
	ClientPtr client;
	XvPortPtr pPort;
	CARD8 motion;
	CARD16 vid_w,vid_h;
	CARD16 drw_w,drw_h;
	CARD16 *p_w, *p_h;
{
	XvddPortPtr pddPort;
	XvEncodingPtr pEncode;
	int vid_x,vid_y,drw_x,drw_y;
	extern WindowPtr *WindowTable;

	vid_x = vid_y = drw_x = drw_y = 0;

	pddPort = (XvddPortPtr)pPort->devPriv.ptr;
	pEncode = pPort->pEncoding;

	/* USE THE SAME GEOMETRY CONSTRAINT PROC THAT WE USE FOR GET AND PUT */

	XvddConstrainVideo(WindowTable[pPort->pAdaptor->pScreen->myNum], pEncode,
                      vid_x, vid_y, vid_w, vid_h,
                      drw_x, drw_y, drw_w, drw_h,
                      &vid_x, &vid_y, &vid_w, &vid_h,
                      &drw_x, &drw_y, p_w, p_h);

	return Success;

}


/*----------------------------------------------------------------------------* 
 |"XvddResumeVideo"
 |
 |  Routine to handle restarting of suspended video.
 *----------------------------------------------------------------------------*/
static void XvddResumeVideo(pPort)
	XvPortPtr pPort;
{
	XvddPortPtr pddPort;
  
	pddPort= (XvddPortPtr)pPort->devPriv.ptr;

	if ( pPort->pGC )
    {

		if (pPort->pGC->serialNumber != pPort->pDraw->serialNumber)
			ValidateGC(pPort->pDraw, pPort->pGC);

		XvddAttachPort(
			pPort->pDraw, pPort, pPort->pGC, pPort->direction, 
			pddPort->vx, pddPort->vy, pddPort->vw, pddPort->vh,
			pddPort->dx, pddPort->dy, pddPort->dw, pddPort->dh);
    }
	else
    {
		ERRORF((fd, "ResumeVideo: error, preempted\n"));
		XvdiSendVideoNotify(pPort, pPort->pDraw, XvPreempted);
		Xvhw_Off( ((XvddPortPtr)pPort->devPriv.ptr)->phw, 1);
		pPort->pDraw = (DrawablePtr)NULL;
    }

	Xvhw_Resume(pddPort->phw, 1); /* Pop all suspends off stack and resume...*/
}


/*----------------------------------------------------------------------------* 
 |"XvddStopOverlay"
 |
 |  Routine to stop video on an overlay type adaptor. It leaves the port 
 |  attached.
 *----------------------------------------------------------------------------*/
static void XvddStopOverlay(client, pPort, pDraw)
	ClientPtr   client;
	XvPortPtr   pPort;
	DrawablePtr	pDraw;
{
	Xvhw_Off( ((XvddPortPtr)pPort->devPriv.ptr)->phw, 1);
	if ( pPort->direction == XvDrawableToSignal )
		XvddDetachPort(client, pPort, pDraw);
}


/*----------------------------------------------------------------------------* 
 |"XvddSuspendVideo"
 |
 |  Routine to suspend video operations during window movements.
 *----------------------------------------------------------------------------*/
static void XvddSuspendVideo(pPort)
	XvPortPtr   pPort;
{
	Xvhw_Suspend( ((XvddPortPtr)pPort->devPriv.ptr)->phw, 1);
}


/*----------------------------------------------------------------------------*
 |"XvddValidateGC"
 |
 |  GC transfer vector wrapper routine to validate a graphics context.
 *----------------------------------------------------------------------------*/
static void
XvddValidateGC(pGC, changes, pDraw)
	GCPtr pGC;
	Mask changes;
	DrawablePtr pDraw;
{
	Bool status;
	ScreenPtr pScreen;
	XvScreenPtr pxvScrn;
	XvPortPtr pPort;

	DEBUGF(0x20, (fd, "XvddValidateGC: entry: pGC %x pDraw %x fp %x\n",
            pGC, pDraw, *(int *)pGC->funcs->ValidateGC));

	pPort = (XvPortPtr)pGC->funcs;

	pGC->funcs = pPort->wrapFuncs;

	(* pGC->funcs->ValidateGC)(pGC, changes, pDraw);

	DEBUGF(0x20, (fd, "XvddValidateGC: return\n"));

	/* save funcs as ValidateGC may have changed them (edg 30oct91) */

	pPort->wrapFuncs = pGC->funcs;

	/* IF THIS GC IS BEING USED FOR VIDEO THEN UPDATE THE VIDEO SERIAL NUMBER */

	if (pPort->pGC == pGC)
    {
		pPort->serialNumber = NEXT_SERIAL_NUMBER;
    }

	pGC->funcs = (GCFuncs *)pPort;

	DEBUGF(0x20, (fd, "XvddValidateGC: exit\n"));
}
